--- 配置相关数据
GameCfg=require("src.app.configs.GameCfg")

--- 全局常量定义
TOP_ORDER = 1000

--- 公用事件派发器(广播器)
CommonBroadcast = LuaEventDispatcher2()

--- 相对定位器
REL=RelativeCoordinate

--- 位操作
BIT=BitOperator

--- cocos相关的一些工具方法
ccUtils=Cocos2dxUtils

--- ccui组件相对定位
CCUIREL=require("src.app.utils.CCUIRel")

--- 本地信息推送
LocalNoti = nil
if cc.PLATFORM_OS_ANDROID == C_PFM then
	LocalNoti=require("src.app.utils.LocalNotification_android")()
else
	LocalNoti=require("src.app.utils.LocalNotification")
end

--- 数据存储
DataStorage=require("src.app.utils.DataStorage")

--- 网络连接管理器
NetManager = require("src.app.logic.NetManager")

--- 声音配置
SoundRes = require("src.app.SoundRes")

--- 事件类型
require("src.app.EventTypes")

--- 服务器列表管理器
ServerListMgr=require("src.app.model.serverList.ServerListMgr")()
--- 账号管理器
AccountMgr=require("src.app.model.serverList.AccountMgr")

--- 战机相关工具
AircraftUtils=require("src.app.utils.AircraftUtils")

local winsize = Director:getWinSize()
local winsizeFlag = winsize.width / winsize.height
if math.abs(winsizeFlag - 640 / 960) < math.abs(winsizeFlag - 640 / 1136) then
	UI_VSIZE = cc.size(640, 960)
else
	UI_VSIZE = cc.size(640, 1136)
end

--- uiJson管理器
UIJsonMgr = require("src.app.UIJsonMgr")

--- 场景列表
ScenesList = require("src.app.uiScript.ScenesList")

--- 重写scence
require("src.app.uiScript.components.common.RewriteScence"):rewrite()

--- 还原对象的缩放比例
-- @param target 目标对象
-- @param scaleX 新的X缩放比例
-- @param scaleY 新的Y绽放比例
function reductionScale(target,scaleX,scaleY)
	scaleX=scaleX or 1
	scaleY=scaleY or 1

	if contentScaleX < contentScaleY then
		target:setScaleX(contentScaleY/contentScaleX*scaleX)
		target:setScaleY(scaleY)
	elseif contentScaleX > contentScaleY then
		target:setScaleX(scaleX)
		target:setScaleY(contentScaleX/contentScaleY*scaleY)
	end

	return target
end

App={}

--- 启动
function App:startup()
	--平台SDK接入
	require("src.app.PlatformEmbeddedIn")

	LocalNoti:app2foreground()
	LocalNoti:includeCategory(LocalNoti.CATEGORY_BUILD)
	LocalNoti:includeCategory(LocalNoti.PVE_POINT_TIP)

	self:preInit()

	-- 预加载公共按钮声音
	AudioManager:preloadEffect(SoundRes.SOUND_COMMON_BUTTON)

	--- 初始化信息提示框,提示框越早初始化越好,因为在很早的时候就可能用到它
	AbstractAlter = require("src.app.uiScript.components.common.AbstractAlter")
	MUIAlter = require("src.app.uiScript.components.common.MUIAlter")
	MUIAlter_singleLine = require("src.app.uiScript.components.common.MUIAlter_singleLine")

	self:enterLoadingScene()

	--- 侦听内存溢出警告
	CppCocos2dxUtils:setMemoryWarningCallback(function()
		local message="触发内存警告!"

		__print__("AppErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"AppErr")

		--不管,还能继续跑!不能怂了
		--		MUIAlter.show({
		--			info=Language:get("ug80")
		--		})
	end)
end

--- 预初始化处理
function App:preInit()
	--app前后台切换时派发事件
	CppCocos2dxUtils:setApp2BackgroundCallback(function()
		print("进入后台")
		LocalNoti:app2background()
		CommonBroadcast:luaDispatchEvent("app_2_background")
	end)
	CppCocos2dxUtils:setApp2ForegroundCallback(function()
		print("进入前台")
		LocalNoti:app2foreground()
		CommonBroadcast:luaDispatchEvent("app_2_foreground")
	end)

	-- 初始化字体样式
	CssStyles = require("src.app.configs.CssStyles")

	-- 默认字体样式
	defaultCssStyle=CssStyles.default

	-- 设置默认的文本样式
	XMLLabel:setCssStyleList(CssStyles)
	XMLTextArea:setCssStyleList(CssStyles)

	--恢复上次的音频状态
	DataStorage:getAudioStatus()

	self:initLanguage()
end

--- 初始化语言
function App:initLanguage()
	--模板
	local t=require("src.app.utils.LanguageUtils")
	t:init()
	t:loadFirstFiles()

	unrequire("res.data.lang.languageList")
end

function App:enterLoadingScene()
	local s=require("src.app.uiScript.scenes.LoadingScene")()

	if Director:getRunningScene() then
		Director:replaceScene(s)
	else
		Director:runWithScene(s)
	end
end

return App
