local T = { }

--- 奖励类型常量
T.ADD_EXP = BitOperator:_ls(1, 0)-- 添加经验
T.ADD_REPUTATION = BitOperator:_ls(1, 1)-- 添加声望
T.ADD_GEMS = BitOperator:_ls(1, 2)-- 添加钻石
T.ADD_MONEY = BitOperator:_ls(1, 3)-- 添加金币
T.ADD_STEEL = BitOperator:_ls(1, 4)-- 添加钢铁
T.ADD_OIL = BitOperator:_ls(1, 5)-- 添加石油
T.ADD_SILICON = BitOperator:_ls(1, 6)-- 添加硅矿
T.ADD_TITANIUM = BitOperator:_ls(1, 7)-- 添加钛矿
T.ADD_PROP = BitOperator:_ls(1, 8)-- 添加道具
T.ADD_AIRCRAFT = BitOperator:_ls(1, 9)-- 添加战机
T.ADD_CONTRIBUTION=BitOperator:_ls(1, 10)-- 联盟贡献点
T.ADD_ALLIANCE_TECH_POINTS=BitOperator:_ls(1, 11)-- 联盟科技点
T.ADD_VIP_POINTS=BitOperator:_ls(1, 12)-- VIP点数
T.ADD_ACHI_POINTS=BitOperator:_ls(1, 13)-- 成就点数
T.ADD_PVE_POINTS = BitOperator:_ls(1, 14) -- pve能量点数
T.ADD_ARENA_POINT=BitOperator:_ls(1, 15) -- 竞技场积分

local AWARD_ITEM_WIDTH=248
local AWARD_ITEM_HEIGHT=80

--- 下面都要派发对应的事件
function T:decode(data)
	local r = { }

	local type = data:readUnsignedInt()
	if BitOperator:_and(type, self.ADD_EXP) ~= 0 then
		r.exp = data:readUnsignedInt()
	end
	if BitOperator:_and(type, self.ADD_REPUTATION) ~= 0 then
		r.reputation = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_PVE_POINTS) ~= 0 then -- 能量
		r.pvePoint = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_GEMS) ~= 0 then
		r.gems = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_MONEY) ~= 0 then
		r.money = data:readUnsignedInt()
	end
	if BitOperator:_and(type, self.ADD_STEEL) ~= 0 then
		r.steel = data:readUnsignedInt()
	end
	if BitOperator:_and(type, self.ADD_OIL) ~= 0 then
		r.oil = data:readUnsignedInt()
	end
	if BitOperator:_and(type, self.ADD_SILICON) ~= 0 then
		r.silicon = data:readUnsignedInt()
	end
	if BitOperator:_and(type, self.ADD_TITANIUM) ~= 0 then
		r.titanium = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_CONTRIBUTION) ~= 0 then
		r.contribution = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_ALLIANCE_TECH_POINTS) ~= 0 then
		r.allianceTechPoint = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_VIP_POINTS) ~= 0 then
		r.vipPoint = data:readUnsignedInt()
	end
    if BitOperator:_and(type, self.ADD_ACHI_POINTS) ~= 0 then
		r.achiPoint = data:readUnsignedInt()
	end

	--竞技场积分
	if BitOperator:_and(type, self.ADD_ARENA_POINT) ~= 0 then
		r.arenaIntegral = data:readUnsignedInt()
	end
	if BitOperator:_and(type, self.ADD_PROP) ~= 0 then
		local l = { }

		local len = data:readUnsignedByte() / 2
		local i = 1
		while i <= len do
			local sid = data:readUnsignedInt()
			local count = data:readUnsignedInt()
			--l[#l] = { sid = sid, count = count }
			l[i] = { sid = sid, count = count } -- add by yuding
			i = i + 1
		end
		r.props = l
	end
	if BitOperator:_and(type, self.ADD_AIRCRAFT) ~= 0 then
		local l = { }

		local len = data:readUnsignedByte() / 2
		local i = 1
		while i <= len do
			local sid = data:readUnsignedInt()
			local count = data:readUnsignedInt()
			--l[#l] = { sid = sid, count = count }
			l[i] = { sid = sid, count = count } -- add by yuding
			i = i + 1
		end

		r.aircrafts = l
	end

	return r
end

--- 添加奖励
function T:addAWard(award)
	if award.exp and award.exp > 0 then
		PlayerInstance:addPlayerExp(award.exp)
	end

	if award.reputation and award.reputation > 0 then
		PlayerInstance:addReputationExp(award.reputation)
	end

	if award.gems and award.gems > 0 then
		PlayerInstance:setDiamond(PlayerInstance:getDiamond()+award.gems)
	end

	if award.money and award.money > 0 then
		PlayerInstance:addGold(award.money)
	end

	if award.steel and award.steel > 0 then
		PlayerInstance:addSteel(award.steel)
	end

	if award.oil and award.oil > 0 then
		PlayerInstance:addOil(award.oil)
	end

	if award.silicon and award.silicon > 0 then
		PlayerInstance:addSilicon(award.silicon)
	end

	if award.titanium and award.titanium > 0 then
		PlayerInstance:addTati(award.titanium)
	end

	if award.pvePoint and award.pvePoint > 0 then  -- 可以超过最大值
		PlayerInstance:addPlayerEnergy(award.pvePoint)
	end

	if award.props then
		for k, v in pairs(award.props) do
			PropsInfoMgr:addPlayerPropByID(v.sid, v.count)
		end
	end

	if award.aircrafts then
		for k, v in pairs(award.aircrafts) do
			PlayerInstance:addAircraftNum(v.sid, v.count)
		end
	end

	if award.contribution and award.contribution > 0 then
		AllianceManager:addTodayContributionPoint(award.contribution)
		AllianceManager:addContributionPoint(award.contribution)
		AllianceManager:addTotalContributionPoint(award.contribution)
	end

	if award.allianceTechPoint and award.allianceTechPoint > 0 then
		--科技点是具体到某一个科技项上页面的,这儿只做显示用,具体刷新由后台推送
	end


	--	if award.pvePoint and award.pvePoint > 0 then      -- 这个没有
	--		PlayerInstance:addPlayerEnergy(award.pvePoint)
	--	end


	if award.vipPoint and award.vipPoint > 0 then
		PlayerInstance:addVipPoint(award.vipPoint)
	end

	if award.achiPoint and award.achiPoint > 0 then

	end

	if award.arenaIntegral and award.arenaIntegral > 0 then
		ArenaInfoMgr:addIntegral(award.arenaIntegral)
	end
end

--- 将奖励数据转换为HTML字符串,以便显示
-- @param 奖励数据
function T:award2HTML(award)
	local list=self:awardResolve(award)

	local sep=Language:get("ug1100")

	local html = (html and html..sep or "")..Language:get("uc314")

	for _,v in ipairs(list) do
		html=(html and html..sep or "")..Language:get("ug1101",v.name,UnitConversion.unitTrans(v.count))
	end

	return html
end

--- 将奖励显示项排列到容器内
function T:arrangeInLayout(awardItemUIList,uiContainer)
	local s=uiContainer:getContentSize()
	local cW,cH=s.width,s.height

	local l=#awardItemUIList
	local w,h=AWARD_ITEM_WIDTH,AWARD_ITEM_HEIGHT
	local wI,hI=10,10

	--列数
	local colsNum=math.floor(cW/(w+wI))
	local tW=(w+wI)*colsNum+wI
	if tW > cW then
		colsNum=colsNum-1
		tW=(w+wI)*colsNum+wI
	end
	local cH=math.ceil(l/colsNum)*(h+hI)+hI
	uiContainer:setContentSize(cc.size(cW,cH))

	local offsetX=tW/2

	--更新项的显示位置
	local top
	for i=0,l-1,1 do
		local item=awardItemUIList[i+1]
		uiContainer:addChild(item)

		local m=i%colsNum
		if m==0 then
			top=(i/colsNum)*(h+hI)+hI
		end
		REL:toOther("center","top",item,uiContainer,(m*(w+wI)+wI+w/2)-offsetX,top)
	end

	return cW,cH
end

--- 将奖励数据转换为layout显示对象列表
function T:award2LayoutList(award)
	local iconScale=IconManager.baseScale

	local createItem=function(data)
		local uiLayout = ccui.Layout:create()
		uiLayout:setContentSize(cc.size(AWARD_ITEM_WIDTH, AWARD_ITEM_HEIGHT))

		--- 物品对应图片
		local uiIcon = ccui.ImageView:create(data.iconUrl,ccui.TextureResType.localType)
		uiLayout:addChild(uiIcon)
		uiIcon:setScale(iconScale[1], iconScale[2])
		REL:relRect(uiIcon,cc.rect(0,0,AWARD_ITEM_HEIGHT,AWARD_ITEM_HEIGHT),0,0,0,0)
		REL:toOther("left", "middle", uiIcon, uiLayout, -10, 0)

		uiIcon.data=data
		uiIcon:setTouchEnabled(true)
		uiIcon:addTouchEventListener(function(t, e)
			if e == ccui.TouchEventType.ended then
				local data=t.data
				if not data then
					return false
				end
				MUIAlter_noTitle.show({
					info = ItemPopInfo.new(data.iconUrl, data.name, data.count, data.des or Language:get("uy130")),
					flag = 1
				})
			end
		end)

		--- 创建名字
		local uiName = XMLLabel(data.name)
		uiName:setCssStyle(CssStyles.default20)
		uiName:setAlign("left","middle")
		uiName:setHeight(36)
		uiLayout:addChild(uiName)
		REL:relOther(uiName,uiLayout,AWARD_ITEM_HEIGHT,0,0,SIZE_NIL)

		--- 数量
		local uiCount = XMLLabel(data.id == "gems" and tostring(data.count) or UnitConversion.unitTrans(data.count))
		uiCount:setCssStyle(CssStyles.default20)
		uiCount:setAlign("left","middle")
		uiCount:setHeight(36)
		uiLayout:addChild(uiCount)
		REL:relOther(uiCount,uiLayout,AWARD_ITEM_HEIGHT,0,SIZE_NIL,0)

		return uiLayout
	end

	local list=self:awardResolve(award)

	local uiItemList={}
	for _,v in ipairs(list) do
		uiItemList[#uiItemList+1]=createItem(v)
	end
	return uiItemList
end

--- 解析奖励数据为奖励列表
function T:awardResolve(award)
	local html=""

	local list={}
	for k,v in pairs(award) do
		local t=AwardTypes[k]
		if k == "props" then
			for _,iv in pairs(v) do
				if iv.count > 0 then
					local data = Prop_temp[iv.sid]
					list[#list+1]={
						id=k,
						name=data and Language:get(data.prop) or "?",
						iconUrl=IconManager.iconPath..(data and data.propIcon or "propIcon/defalutIcon.png"),
						count=iv.count,
						des = PlayerInstance:getPropDescripe(iv.sid),
						orderPriority=t and t.orderPriority or 100
					}
				end
			end
		elseif k == "aircrafts" then
			for _,iv in pairs(v) do
				if iv.count > 0 then
					local data=Aircraft_temp[iv.sid]
					list[#list+1]={
						id=k,
						name=AircraftUtils:getRichName(data) or "?",
						iconUrl=IconManager.iconPath..(data and data.icon or "propIcon/defalutIcon.png"),
						count=iv.count,
						des = Language:get(data.describe,data.capacity),
						orderPriority=t and t.orderPriority or 100 + iv.sid -- add by yuding
					}

				end
			end
		else
			if v > 0 then
				list[#list+1]={
					id=k,
					name=t and Language:get(t.name) or "?",
					iconUrl=IconManager.iconPath..(t and t.iconUrl or "propIcon/defalutIcon.png"),
					count=v,
					des = Language:get(t.des),
					orderPriority=t and t.orderPriority or 100
				}
			end
		end
	end

	table.sort(list,function(a,b)
		return a.orderPriority < b.orderPriority
	end)

	return list
end

--- 在列表中的显示:声望，钻石，道具，vip积分;显示上道具在飞机前面
function T:awardResolve2(award)
	local html=""

	local list={}
	for k,v in pairs(award) do
		local t=AwardTypes[k]
		if k == "props" then
			local i = 0
			for _,iv in pairs(v) do
				if iv.count > 0 then
					local data = Prop_temp[iv.sid]
					list[#list+1]={
						name=data and Language:get(data.prop) or "?",
						iconUrl=IconManager.iconPath..(data and data.propIcon or "propIcon/defalutIcon.png"),
						count=iv.count,
						des = PlayerInstance:getPropDescripe(iv.sid),
						orderPriority = i + 200,
						type = k,
						sid = iv.sid
					}
					i = i + 1
				end
			end
		elseif k == "aircrafts" then
			local i = 0
			for _,iv in pairs(v) do
				if iv.count > 0 then
					local data=Aircraft_temp[iv.sid]
					list[#list+1]={
						name=AircraftUtils:getRichName(data),
						iconUrl=IconManager.iconPath..(data and data.icon or "propIcon/defalutIcon.png"),
						count=iv.count,
						des = Language:get(data.describe,data.capacity),
						orderPriority = i + 500,
						type = k,
						sid = iv.sid
					}
					i = i + 1
				end
			end
		else
			if t and v > 0 then -- 考虑到配置可以配不属于奖励品类型的数据结构0
				list[#list+1]={
					name=t and Language:get(t.name) or "?",
					iconUrl=IconManager.iconPath..(t and t.iconUrl or "propIcon/defalutIcon.png"),
					count=v,
					des = Language:get(t.des),
					orderPriority = self:getListOrderPriority(t.id),
					type = k,
					sid = nil
				}
			end
		end
	end

	table.sort(list,function(a,b)
		return a.orderPriority < b.orderPriority
	end)

	return list
end

--- 在列表中的显示:声望，钻石，道具，vip积分;显示上飞机在道具前面
function T:awardResolve3(award)
	local html=""

	local list={}
	for k,v in pairs(award) do
		local t=AwardTypes[k]
		if k == "props" then
			local i = 0
			for _,iv in pairs(v) do
				if iv.count > 0 then
					local data = Prop_temp[iv.sid]
					list[#list+1]={
						name=data and Language:get(data.prop) or "?",
						iconUrl=IconManager.iconPath..(data and data.propIcon or "propIcon/defalutIcon.png"),
						count=iv.count,
						des = PlayerInstance:getPropDescripe(iv.sid),
						orderPriority = i + 500,
						type = k,
						sid = iv.sid
					}
					i = i + 1
				end
			end
		elseif k == "aircrafts" then
			local i = 0
			for _,iv in pairs(v) do
				if iv.count > 0 then
					local data=Aircraft_temp[iv.sid]
					list[#list+1]={
						name=AircraftUtils:getRichName(data),
						iconUrl=IconManager.iconPath..(data and data.icon or "propIcon/defalutIcon.png"),
						count=iv.count,
						des = Language:get(data.describe,data.capacity),
						orderPriority = i + 200,
						type = k,
						sid = iv.sid
					}
					i = i + 1
				end
			end
		else
			if t and v > 0 then -- 考虑到配置可以配不属于奖励品类型的数据结构
				list[#list+1]={
					name=t and Language:get(t.name) or "?",
					iconUrl=IconManager.iconPath..(t and t.iconUrl or "propIcon/defalutIcon.png"),
					count=v,
					des = Language:get(t.des),
					orderPriority = self:getListOrderPriority(t.id),
					type = k,
					sid = nil
				}
			end
		end
	end

	table.sort(list,function(a,b)
		return a.orderPriority < b.orderPriority
	end)

	return list
end

-- 顺序为：声望，钻石，道具，vip积分   (道具优先级在 200-400  飞机优先级在 500 - 800)
function T:getListOrderPriority(name)
	if name == "exp" then
		return 1
	elseif name == "money" then
        return 4
	elseif name == "steel" then
    	return 5
	elseif name == "oil" then
    	return 6
	elseif name == "silicon" then
    	return 7
	elseif name == "titanium" then
    	return 8
	elseif name == "reputation" then
		return 1
	elseif name == "pvePoint" then
		return 1200
	elseif name == "threeGems" then
		return 2
	elseif name == "gems" then
		return 3
	elseif name == "contribution" then
	elseif name == "allianceTechPoint" then
	elseif name == "vipPoint" then
		return 1000
	elseif name == "achiPoint" then
	end
end


--- 返回一个layout
-- @param rList 奖励表
function T:award2LayoutContainer(rList)
	local distance = 20 -- 两个图片之间的距离

	local func = function(parent, res, num, name, des, width)
		local r = ccui.ImageView:create()
		r:loadTexture(res)
		parent:addChild(r)
		BaseButton:extend(r)
		r:addTouchEventListener(function(t, e)
			if e ~= ccui.TouchEventType.ended then return end
			if not des then
				des = Language:get("uy130") -- NULL
			end

			local layout = ItemPopInfo.new(res, name, num, des)
			MUIAlter_noTitle.show({
				info = layout,
				flag = 1
			})

		end)
		CCUIREL:toOther("left", "bottom", r, parent, width, 0)

		local nText = XMLLabel()           -- 数量
		nText:setContentSize(cc.size(150, 27))
		nText:setHAlign("right")
		nText:setVAlign("bottom")
		nText:setString("x" .. num)

		r:addChild(nText)

		REL:toOther("right", "bottom", nText, r, 0, 0) -- 右下角对齐

		return r
	end

	local n = table.maxn(rList)

	local lay = ccui.Layout:create()
	lay:setContentSize(cc.size(n * 100 + (n - 1) * distance, 100))

	local res = {}
	for i = 1, n do
		local r=rList[i]
		func(lay, r.iconUrl, r.count, r.name, r.des, (i - 1) * (100 + distance))
	end

	return lay
end

--- 显示窗口
function T:showSuccessWindow(tb,titleText,sndURL) -- 标题可以改
	local w,h=580,240
	local top=16
    local playerLvlDiff = PlayerInstance:getPlayerLvlDiff()
	local lay = ccui.Layout:create()
	lay:setContentSize(cc.size(w, h))

	local title = XMLLabel(titleText or Language:get("uy68"))
	title:setAlign("center","middle")
	title:setHeight(36)
	lay:addChild(title)

	local uiAwardList=ccui.Layout:create()
	REL:relOther(uiAwardList,lay,10,10,10,16)
	lay:addChild(uiAwardList)

	local uiList=require("src.app.communication.award.AwardDataDecoder"):award2LayoutList(tb)
	local w,h=require("src.app.communication.award.AwardDataDecoder"):arrangeInLayout(uiList,uiAwardList)

	lay:setContentSize(cc.size(
		w,
		top+ --上边距
		title:getHeight()+ --title
		10+ --间隔
		h+ --奖励列表
		16) --下边距
	)

	--设置位置
	REL:relOther(title,lay,10,10,top,SIZE_NIL)
	top=top+title:getHeight()+10
	REL:toOther("center","top",uiAwardList,lay,0,top)

	--- 显示窗口
	MUIAlter_noTitle.show({
		info = lay,
        popSnd = sndURL or SoundRes.SND_SND_GET_AWARD,
        callback = function(flag)
            local playerLvlDiff = PlayerInstance:getPlayerLvlDiff()
            if playerLvlDiff == true then
                local layout = require("src.app.uiScript.components.share.PlayerLvlUpPopWindow").new()
	            local muialter = MUIAlter.show(
		            {
			            title = Language:get("uc930"),
			            info = layout,
			            flag = 4,
                        isShowCloseButton = true,
			            callback = function(flag)
			            end
		            }
	            )
	            layout:setMuialter(muialter)

            end

		end
	})

end

return T
