--- 进入战斗演播命令
local T=Class.AbstractEnterFightPlayCommand()

--- 战斗场景配置
T.FightSceneCfg=require("src.app.uiScript.components.fight.FightWindow")

--- 战斗场景
T.FightWindow=require("src.app.uiScript.components.fight.FightWindow")

--- 战斗演播器
T.FightPlayer=require("src.app.logic.fight.FightPlayer")

--- 战斗逻辑执行器
T.FightLogicCalcer=require("src.app.logic.fightLogic.FightLogicCalcer")

--- 战斗加载
T.FightLoading=require("src.app.uiScript.components.fight.FightLoading")

function T:def()
--- 战斗数据
-- fightData

--- 战机资源列表
-- aircraftRes

--- 战斗类型
-- fightType

--- 战斗加载视图
-- uiFightLoading
end

--- 进入战斗演播命令
-- @param fightType 战斗类型
function T:ctor(fightType)
	self.fightType=fightType
end

--- 切换到过度窗口
function T:switchExcessiveWindow()
	ResLoader:loadReses(self.FightSceneCfg.preResList)

	--- 处理网络关闭
	local netExceptionHandler
	netExceptionHandler=function(type,code)
		if code == NET_CODE_SOCKET_CLOSE then
			NetManager:luaRemoveEventListener(NET_EXCEPTION_EVENT,nil,netExceptionHandler)

			if self.uiFightLoading then
				self.uiFightLoading:stop()
			end

			--清理缓存的纹理
			ResLoader:unloadReses(self.FightSceneCfg.preResList)
			if self.aircraftRes then
				ResLoader:unloadReses(self.aircraftRes)
			end

			self:_quitFightWindow()
		end
	end
	NetManager:luaAddEventListener(NET_EXCEPTION_EVENT,nil,netExceptionHandler)

	local uiFightLoading=self.FightLoading()
	uiFightLoading:setLocalZOrder(TOP_ORDER+1)
	Director:getRunningScene():addChild(uiFightLoading)
	REL:toScreen("center","middle",uiFightLoading,0,0)
	uiFightLoading:setMaxPercent(40)
	uiFightLoading:start(function()
		self:getFightData(function(fightData)
			NetManager:luaRemoveEventListener(NET_EXCEPTION_EVENT,nil,netExceptionHandler)

			fightData.fightInitData.fightType=self.fightType
			self.fightData=fightData

			-- 分析后续需要加载的资源
			self.aircraftRes=self:getAircraftReses(fightData.fightInitData.camp1.aircrafts,fightData.fightInitData.camp2.aircrafts)

			uiFightLoading:setMaxPercent(100)

			ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,self,self.__switchFightWindow)
			ResLoader:loadReses(self.aircraftRes)
		end)
	end)
	self.uiFightLoading=uiFightLoading
end

--- 切换到战斗窗口
function T:__switchFightWindow()
	ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,self,self.__switchFightWindow)

	self:_switchFightWindow()

	CommonBroadcast:luaAddEventListener(EVENT_FIGHT_SCENE_QUIT,self,self.__quitFightHandler)
end

--- 切换到战斗窗口
function T:_switchFightWindow()
	local fW=self.FightWindow.new()

	--初始化战斗演播器
	local fp=self.FightPlayer()
	fp:initFightScene(fW)

	--初始化战斗逻辑运算器
	local fl=self.FightLogicCalcer()
	fl:init(self.fightData)

	fl:start()
	if self.uiFightLoading then
		self.uiFightLoading:stop(function()
			self.uiFightLoading:removeFromParent()
			self.uiFightLoading=nil
		end)
	end

	local curWindow=WindowManager:getCurrentWindow()
	if curWindow and curWindow.__cname == "ArmyWindow" or curWindow.__cname == "fightWindow" then
		--从阵形布置页面/战斗页面切换
		WindowManager:switch(fW)
	else
		--从战场过度页面切换
		WindowManager:push(fW)
	end
end

--- 退出战斗窗口
function T:__quitFightHandler(endData)
	CommonBroadcast:luaRemoveEventListener(EVENT_FIGHT_SCENE_QUIT,self,self.__quitFightHandler)

	self:_updateData(endData)
	self:_showEndTip(endData)
end

--- 更新数据
function T:_updateData(endData)

end

--- 显示战斗结束提示框
function T:_showEndTip(endData)
	if endData then
		local hasNeedFixAircraft=AircraftManager:judgeAirIsRepair()

		local camps=endData.camps
		if endData.isWinner then
			MUIAlter_custom.show({
				info=require("src.app.uiScript.components.fight.fightEnd.VictoryCvs")(
					self.fightType,
					endData,
					hasNeedFixAircraft,
					function(toWindowStr)
						self:__quitFightWindow(toWindowStr)
					end,
					self.fightData.fightInitData.originalFightType
				),
				triggerCloseMethod="any"
			})
		else
			MUIAlter_custom.show({
				info=require("src.app.uiScript.components.fight.fightEnd.FailCvs")(
					self.fightType,
					endData,
					hasNeedFixAircraft,
					function(toWindowStr)
						self:__quitFightWindow(toWindowStr)
					end,
					self.fightData.fightInitData.originalFightType
				),
				triggerCloseMethod="any"
			})
		end
	else
		self:__quitFightWindow()
	end
end

--- 退出战斗窗口
function T:__quitFightWindow(toWindowStr)
	if toWindowStr == "toReplay" then
		require("src.app.controler.enterFight.EnterReplayFightCommand")(self.fightData,function()
--			WindowManager:pop()
		end)

		return
	end

	CommonBroadcast:luaDispatchEvent(EVENT_FIGHT_COMPLETE,self.fightType)
	WindowManager:pop()

	--清理缓存的纹理
	ResLoader:unloadReses(self.FightSceneCfg.preResList)
	if self.aircraftRes then
		ResLoader:unloadReses(self.aircraftRes)
	end

	self:_quitFightWindow(toWindowStr)
end

function T:_quitFightWindow(toWindowStr)
	if self.uiFightLoading then
		self.uiFightLoading:stop(function()
			self.uiFightLoading:removeFromParent()
			self.uiFightLoading=nil
		end)
	end

	if not toWindowStr then

	else
		if toWindowStr == "toProduceAircraft" then
			self:__toProduceAircraft()
		elseif toWindowStr == "toFixAircraft" then
			self:__toFixAircraft()
		elseif toWindowStr == "toUpgradeSkill" then
			self:__toUpgradeSkill()
		elseif toWindowStr == "toUpgradeCommand" then
			self:__toUpgradeCommand()
		elseif toWindowStr == "toUpgradeScence" then
			self:__toUpgradeScence()
		elseif toWindowStr == "toParts" then
			self:__toParts()
		elseif toWindowStr == "toReplay" then

		end
	end
end

--- 跳转到配置页面
function T:__toParts()
	WindowManager:push(
		require("src.app.uiScript.components.accessory.accessorySystem.AccessorySystemWindow").new()
	)
end

--- 跳转到战机生产界面
function T:__toProduceAircraft()
	local info
	if BuildingsInfoMgr:getBuildingInfos()[2] then
		info = BuildingsInfoMgr:getBuildingInfos()[2]
	elseif BuildingsInfoMgr:getBuildingInfos()[3] then
		info = BuildingsInfoMgr:getBuildingInfos()[3]
	else
		info = {
			sid=2,
			id = nil,
			remainingSec = 0,
			location = 2,
			level = 0,
		}
	end
	WindowManager:push(require("src.app.uiScript.components.aircraftFactory.AircraftFactoryWindow").new(info,2))
end

--- 跳转到战机修理界面
function T:__toFixAircraft()
	WindowManager:push(
		require("src.app.uiScript.components.army.ArmyWindow").new({tag = 1},3)
	)
end

--- 跳转到技能提升界面
function T:__toUpgradeSkill()
	local view = require("src.app.uiScript.scenes.playerInfo.PlayerInfoWindow").new(2)

	WindowManager:push(view)
end

--- 跳转到指挥提升界面
function T:__toUpgradeCommand()
	local view = require("src.app.uiScript.scenes.playerInfo.PlayerInfoWindow").new(1)

	WindowManager:push(view)
end

--- 跳转到科技升级界面
function T:__toUpgradeScence()
	local info = BuildingsInfoMgr:getBuildingInfos()[4]
	if not info then
		info = {
			sid=3,
			id = nil,
			remainingSec = 0,
			location = 4,
			level = 0,
		}
	end
	WindowManager:push(
		require("src.app.uiScript.components.building.BuildingWindow").new(info,2)
	)
end

--- 获取战机所需资源
function T:getAircraftReses(camp1Aircrafts,camp2Aircrafts)
	local r,m={},{}--m用于剔出重复项

	local function _get(sid)
		local t=Aircraft_temp[sid]

		local atkEff=require(GameUtils:getAttackUrl(t.attackEffName))

		local l=atkEff.preResList
		if l then
			for k,v in pairs(l) do
				if not m[v] then
					r[#r+1]=v

					m[v]=true
				end
			end
		end
	end

	for k,v in pairs(camp1Aircrafts) do
		_get(v.sid)
	end

	for k,v in pairs(camp2Aircrafts) do
		_get(v.sid)
	end

	return r
end

--- 获取战斗数据
function T:getFightData(getCompleteHandler)

end

return T
