local T=Class.GetServerListCommand()

--- 参数-是否是获取测试服务器地址
T.PARAM_IS_STAGING="staging={is_staging}"

--- 参数-是否获取用户唯一ID
T.PARAM_IS_NEED_SIGN="needsign={is_need_sign}"

--- 参数-lua代码及资源的版本号
T.PARAM_LUA_VERSION="sversion={lua_version}"

--- 参数-平台代码
T.PARAM_PLATFORM_CODE="channel={platformCode}"

--- 获取服务器列表
-- @cppVersion cpp的当前版本号
-- @luaVersion lua代码及资源的当前版本号
-- @isStaging 是否是获取测试服务器地址
-- @isNeedSign 是否获取用户唯一ID
-- @callback 回调,接受1个参数,表示服务器列表数据
-- @failCallback 请求失败的回调,接受2个参数:code,errInfo
function T:ctor(cppVersion,luaVersion,isStaging,isNeedSign,callback,failCallback)
	local url=GameCfg.GET_SERVER_LIST_URL

	--判断自动更新测试状态是否打开
	local testSwitchData=DataStorage:getTestSwitchData()
	if testSwitchData then
		if testSwitchData.isOpen then
			print("进行自动更新测试")
			url=GameCfg.GET_SERVER_LIST_URL_TEST_UPDATE
		end
	end

	url=url:gsub("{cpp_version}",cppVersion)
	url=url:gsub("{lua_version}",luaVersion)
	if isStaging then
		url=url.."&"..self.PARAM_IS_STAGING:gsub("{is_staging}",isStaging and 1 or 0)
	end
	if isNeedSign then
		url=url.."&"..self.PARAM_IS_NEED_SIGN:gsub("{is_need_sign}",isNeedSign and 1 or 0)
	end

	if SDK_Delegate then
		local pltCode=SDK_Delegate:getPlatformCode()
		if pltCode then
			url=url.."&"..self.PARAM_PLATFORM_CODE:gsub("{platformCode}",pltCode)
		end
	end

	print(url)

	--	do
	--		callback( {
	--			serverList={{
	--				id=1,
	--				name="dfsdf",
	--				ip="192.168.0.238",
	--				port=8501
	--			}},	--服务器列表
	--			notice=nil,						--公告
	--			updateList=nil,					--lua更新列表
	--			cppVersion="",					--cpp代码版本
	--			uniqueId="sdfsaodfsdlfsdfbflsjskdfsbflsd",					--唯一ID
	--			cppUpdateUrl=nil				--cpp代码更新的 url
	--		})
	--		return;
	--	end

	LuaHttp:get(
		url,
		function(jsonStr)
			LuaHttp:pause()

			print(jsonStr)
			if callback then
				callback(self:decode(jsonStr))
			end
		end ,
		function(code, errInfo)
			LuaHttp:pause()

			print("获取最新版本数据失败")

			if failCallback then
				failCallback(code,errInfo)
			end
		end,
		5,--连接超时
		5--读写数据超时
	)
end

--- 封闭服务器列表数据,并注释出各属性功能
-- @param jsonStr 从服务器获取的json字符串
function T:decode(jsonStr)
	local jsonObj
	local b,errStr=pcall(function()
		jsonObj=json:decode(jsonStr)
	end,jsonStr)

	if not b then
		local message=errStr

		__print__("DecodeServerListErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"DecodeServerListErr")

		return nil
	end

	return {
		serverList=jsonObj.servers,					--服务器列表
		notice=jsonObj.ann,							--公告
		updateList=jsonObj.update,					--lua更新列表
		cppVersion=tostring(jsonObj.version),		--cpp代码版本
		uniqueId=jsonObj.sign,						--唯一ID
		cppUpdateUrl=jsonObj.cppUpdateUrl			--cpp代码更新的 url
	}
end

return T
