local T=Class.NetDetector()

--- 最大重连次数
T.MAX_RECONNECT_COUNT=5

--- 重连延迟
T.RECONNECT_DELAY_SEC=1

function T:def()
	--- 重连次数
	self.reconnectCount=0

	--- 延迟重连ID
	-- reconnectDelayId

	--- 心跳发送器
	-- HeartBeater
end

function T:ctor()

end

--- 开始检测
function T:start()
	NetManager:luaAddEventListener(NET_EXCEPTION_EVENT,self,self.netExceptionHandler)

	--开始心跳检测
	if not self.HeartBeater then
		self.HeartBeater=require("src.app.logic.HeartBeater")()
	end
	self.HeartBeater:startHeartBeatCheck()
end

--- 停止检测
function T:stop()
	self:reset()

	if self.HeartBeater then
		self.HeartBeater:stop()
	end
end

--- 处理网络关闭
function T:netExceptionHandler(type,code)
	if code == NET_CODE_SOCKET_CLOSE then
		NetManager:luaRemoveEventListener(NET_EXCEPTION_EVENT,self,self.netExceptionHandler)
		if BGMgr:isGuide() then
			MUIAlter.show({
				info=Language:get("ug10",code),
				zOrder=TOP_ORDER + 200+10,
				callback=function()
					self:reset()

					print("新手引导中网络连接断开")
					--退回到登陆界面
					require("src.app.controler.Back2LoginScenceCommand")()
				end
			})

			return
		end

		UserInfoMgr:stopUpdateCheck()

		self.HeartBeater:stop()

		self:reconnect()
	end
end

--- 重连
function T:reconnect()
	if self.reconnectCount >= 4 then
		self:showTip()
	end

	NetManager:luaAddEventListener(NET_EXCEPTION_EVENT,self,self.re_netExceptionHandler)

	self.reconnectCount=0
	self:addConnectListeners()
	self:_reconnect()
end

--- 重置
function T:reset()
	self:removeConnectListeners()
	NetManager:luaRemoveEventListener(NET_EXCEPTION_EVENT,self,self.netExceptionHandler)
	NetManager:luaRemoveEventListener(NET_EXCEPTION_EVENT,self,self.re_netExceptionHandler)

	self:hideTip()
	self.reconnectCount=0
end

--- 重连阶段的网络异常
function T:re_netExceptionHandler(type,code)
	if code == NET_CODE_SOCKET_CLOSE then
		self.reconnectCount = self.reconnectCount + 1
		if self.reconnectCount >= self.MAX_RECONNECT_COUNT then
			self:reset()

			print("多次重连失败")

			--退回到登陆界面
			require("src.app.controler.Back2LoginScenceCommand")()
			return
		end

		--延迟后重连
		Cocos2dxUtils:timeOut(function()
			self:_reconnect()
		end,self.RECONNECT_DELAY_SEC)
	end
end

--- 连接失败
function T:connectFailHandler()
	--手动派发异常事件,因为连接未成功,所以不存在关闭连接
	self:re_netExceptionHandler(nil,NET_CODE_SOCKET_CLOSE)
end

--- 获取密钥超时
function T:encyptionTimeoutHandler()
	NetManager:close()
end

--- 初始化网络完成
function T:initCommunicationCompleteHandler()
	self:removeConnectListeners()

	self:relogin()
end

--- 添加连接阶段的事件侦听器
function T:addConnectListeners()
	NetManager:luaAddEventListener(NET_CONNECT_FAIL,self,self.connectFailHandler)
	NetManager:luaAddEventListener(NET_ENCYPTION_TIMEOUT,self,self.encyptionTimeoutHandler)

	NetManager:luaAddEventListener(NET_INIT_COMMUNICTION_COMPLETE,self,self.initCommunicationCompleteHandler)
end

--- 移除连接阶段的事件侦听器
function T:removeConnectListeners()
	NetManager:luaRemoveEventListener(NET_CONNECT_FAIL,self,self.connectFailHandler)
	NetManager:luaRemoveEventListener(NET_ENCYPTION_TIMEOUT,self,self.encyptionTimeoutHandler)

	NetManager:luaRemoveEventListener(NET_INIT_COMMUNICTION_COMPLETE,self,self.initCommunicationCompleteHandler)
end

--- 重连
function T:_reconnect()
	print("重连",self.reconnectCount)

	NetManager:connect()
end

--- 重登陆成功
function T:loginSuccHandler()
	self:removeLoginListeners()
	NetManager:luaRemoveEventListener(NET_EXCEPTION_EVENT,self,self.re_netExceptionHandler)

	self:hideTip()

	self:start()
end

--- 重登陆失败(可能是账号出了问题,所以不再重试,确定提示框后直接弹到登陆界面)
function T:loginFailHandler()
	self:removeLoginListeners()

	self.reconnectCount = self.MAX_RECONNECT_COUNT
	NetManager:close()
end

--- 移除登陆阶段的事件侦听器
function T:removeLoginListeners()
	NetManager:luaRemoveEventListener(NET_LOGIN_SUCC,self,self.loginSuccHandler)
	NetManager:luaRemoveEventListener(NET_LOGIN_FAIL,self,self.loginFailHandler)
end

--- 重登陆
function T:relogin()
	NetManager:luaAddEventListener(NET_LOGIN_SUCC,self,self.loginSuccHandler)
	NetManager:luaAddEventListener(NET_LOGIN_FAIL,self,self.loginFailHandler)

	NetManager:loginServer()
end

local waitingTip

--- 显示提示
function T:showTip()
	waitingTip=require("src.app.uiScript.components.common.WaitingMessageText"):show(Language:get("ug15"))
end

--- 隐藏提示
function T:hideTip()
	if waitingTip then
		waitingTip:hide()
		waitingTip = nil
	end
end

return T
