NET_CODE_INIT_ENCRYPTION = 2

NET_CODE_SEND_DATA_FAIL = 3

NET_CODE_RECEIVE_DATA_FAIL = 4

NET_CODE_SOCKET_CLOSE = 5

NET_CODE_DECODE_DATA_ERROR = 6

local _print=print
local function print(...)
	_print("<NetManager>: ",table.concat({...},"\t"))
end

--- 网络连接管理器
local T=Class.NetManager(LuaEventDispatcher2)

function T:def()
	LuaEventDispatcher2.def(self)

	--- 连接相关数据
	-- self.data = nil

	--- 转发器
	-- self.adapter = nil

	--- 是否已初始化完成
	self.isInited = false

	--- 等待密钥的超时ID
	-- self.encyptionTimeoutId = nil
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 初始化
function T:init(connectData)
	self.data = connectData
end

--- 创建连接转发器
function T:_createAdapter()
	local adapter = LuaSocketAdapter:new();
	adapter:luaAddEventListener("net_socket_event",
		function(type, code)
			self:_netEventHandler(type, code)
		end
	)
	self.adapter = adapter
end

--- 连接
function T:connect()
	print( "创建新的连接,并连接服务器")

	self:close()
	self:_createAdapter()

	local adapter = self.adapter

	local serverInfo = ServerListMgr:getServerData(self.data.serverId)
	local ip = serverInfo.ip
	local port = serverInfo.port

	self:luaDispatchEvent(NET_START_CONNECT)
	adapter:connect(ip, port, function(b)
		if b then
			print( "连接服务器成功")
			self:luaDispatchEvent(NET_CONNECT_SUCC)

			adapter:check()

			self:_waitingEncyption()
		else
			print( "连接服务器失败")
			self.adapter=nil

			self:luaDispatchEvent(NET_CONNECT_FAIL)
		end
	end )
end

--- 关闭
function T:close()
	local adapter = self.adapter
	if adapter then
		print( "关闭连接")

		adapter:close()
		self.adapter=nil
	end
end

--- 等待服务器的密钥
function T:_waitingEncyption()
	print( "等待服务器的密钥")

	self:luaDispatchEvent(NET_GET_ENCYPTION)

	self.encyptionTimeoutId = Cocos2dxUtils:timeOut( function()
		print( "等待服务器的密钥超时")
		self:close()

		self.encyptionTimeoutId = nil

		self:luaDispatchEvent(NET_ENCYPTION_TIMEOUT)
	end , 15)
end

--- 转发器事件
function T:_netEventHandler(type, code)
	print("<net_event>:", type, code)

	if self.encyptionTimeoutId then
		Cocos2dxUtils:clearScheduler(self.encyptionTimeoutId)
	end

	if code == NET_CODE_INIT_ENCRYPTION then
		print( "已收到服务器密钥,开始初始化网络环境")

		--延迟一帧的原因中,在初始化网络连接时(InitCommunication)中,会添加对此事件的侦听,
		--所以会出现在回调添加侦听的问题.会造成C++出错
		Cocos2dxUtils:timeOut( function()
			if not self.adapter then
				-- 在延迟的这一帧时间里,转发器已经被关闭
				return
			end
			self.isInited = true
			self:luaDispatchEvent(NET_INIT_ENCYPTION_COMPLETE)

			require("src.app.communication.InitCommunication"):exec(self.adapter)

			self:luaDispatchEvent(NET_INIT_COMMUNICTION_COMPLETE)
		end , 0)
	elseif code == NET_CODE_SOCKET_CLOSE then
		print( "网络连接转发器被关闭!")
		self.adapter = nil

		self:luaDispatchEvent(NET_EXCEPTION_EVENT, type, code)
	elseif code == NET_CODE_DECODE_DATA_ERROR then
		print( "解析服务器返回的数据错误!")
	end
end

--- 登陆服务器
function T:loginServer()
	print( "初始化网络环境完成,开始登陆用户")

	local succHandler=function(state,info,isForceUpdate,updateURL)
		if state then
			print( "登陆用户成功")

			if CUR_SDK then
				if CUR_SDK == SDKType.PLATFORM_ENJOY_GAME then
					local serverInfo = ServerListMgr:getServerData(self.data.serverId)

					SDK_Delegate:traking_gameServerName(serverInfo.name)
					SDK_Delegate:traking_roleLevel(UserInfo.level)
				elseif CUR_SDK == SDKType.PLATFORM_TENGLONG_1 or CUR_SDK == SDKType.PLATFORM_UC or CUR_SDK == SDKType.PLATFORM_VIVO or CUR_SDK == SDKType.PLATFORM_HUA_WEI then
					local serverInfo = ServerListMgr:getServerData(self.data.serverId)

					SDK_Delegate:trakingEnterGame(UserInfo.id,UserInfo.name,UserInfo.level,serverInfo.id,serverInfo.name)
				end
			end

			self:luaDispatchEvent(NET_LOGIN_SUCC)
		else
			if CUR_SDK == SDKType.PLATFORM_ENJOY_GAME then
			elseif CUR_SDK and CUR_SDK == SDKType.PLATFORM_TENGLONG_1 or CUR_SDK == SDKType.PLATFORM_UC or CUR_SDK == SDKType.PLATFORM_VIVO or CUR_SDK == SDKType.PLATFORM_HUA_WEI  then
				--是否需要强制更新
				if isForceUpdate then
					MUIAlter.show({
						info = Language:get("ug_needUpdateCpp"),
						size=30,
						callback=function()
							if SDK_Delegate and SDK_Delegate.doOpenUrl then
								SDK_Delegate:doOpenUrl(updateURL)

								return
							end

							OpenURL:getInstance():open(updateURL)
						end
					})
				else
					MUIAlter.show({
						info=info
					})
				end

				self:luaDispatchEvent(NET_LOGIN_FAIL,isForceUpdate,info)
			else
				self:luaDispatchEvent(NET_LOGIN_FAIL,info)
			end
		end
	end

	if CUR_SDK then
		if CUR_SDK == SDKType.PLATFORM_360 then
			require("src.app.communication.login.LoginWith360Command")():exec(SDK_Delegate:getAccessToken(),succHandler)
		elseif CUR_SDK == SDKType.PLATFORM_ENJOY_GAME then
			require("src.app.communication.login.LoginWithEnjoyGameCommand")():exec(SDK_Delegate:getUid(),SDK_Delegate:getAccessToken(),succHandler)
		elseif CUR_SDK == SDKType.PLATFORM_TENGLONG_1 then
			require("src.app.communication.login.LoginWith1Command")():exec(SDK_Delegate:getChannelId(),SDK_Delegate:getUid(),SDK_Delegate:getAccessToken(),CFG:getVersion(),succHandler)
		elseif CUR_SDK == SDKType.PLATFORM_UC then
			require("src.app.communication.login.LoginWithUCCommand")():exec(SDK_Delegate:getAccessToken(),CFG:getVersion(),succHandler)
		elseif CUR_SDK == SDKType.PLATFORM_HUA_WEI then
			require("src.app.communication.login.LoginWithHuaWeiCommand")():exec(SDK_Delegate:getAccessToken(),CFG:getVersion(),succHandler)
		elseif CUR_SDK == SDKType.PLATFORM_VIVO then
			require("src.app.communication.login.LoginWithVivoCommand")():exec(SDK_Delegate:getAccessToken(),CFG:getVersion(),succHandler)
		end
	else
		local connectData = self.data
		local uid = connectData.uid
		local pwd = connectData.pwd

		if connectData.isTourist then
			require("src.app.communication.login.LoginWithTouristCommand")():exec(uid,succHandler)
		else
			require("src.app.communication.login.LoginCommand")():exec(uid,pwd,succHandler)
		end
	end
end

--- 获取登陆数据
function T:getConnectData()
	return self.data
end

return T()
