--- 战斗演播器
local T=Class.FightPlayer()

function T:def()
	--- 被显示在左下方的阵营
	-- self.leftCamp=nil

	--- 战斗动作数据队列
	self.actionQueue=Queue()

	--- 开始时间
	-- self.startTimeMS=nil

	--- 事件队列检测计时器
	-- self.checkIntervalSid=nil
end

--- 构造
function T:ctor()
	Local_BroadcastFight:luaAddEventListener(
		Local_BroadcastFight.EVENT_TYPE_FIGHT_EVENT,
		self,
		self.push
	)
end

--- 压入事件
function T:push(action)
	Queue.push(self.actionQueue,action)
end

--- 初始化战斗场景
function T:initFightScene(fightScene)
	fightScene:luaAddEventListener(CocosEvent.ENTER,self,self.enterFightSceneHandler)
	fightScene:luaAddEventListener(CocosEvent.EXIT,self,self.exitFightSceneHandler)

	self.fightScene=fightScene
end

--- 进入战斗场景
function T:enterFightSceneHandler()
	self.fightScene:luaRemoveEventListener(CocosEvent.ENTER,self,self.enterFightSceneHandler)

	self:startPlay()
end

--- 退出战斗场景
function T:exitFightSceneHandler()
	self.fightScene:luaRemoveEventListener(CocosEvent.EXIT,self,self.exitFightSceneHandler)

	if self.checkIntervalSid then
		Cocos2dxUtils:clearScheduler(self.checkIntervalSid)
		self.checkIntervalSid=nil
	end
	self.fightScene=nil
end

--- 开始演播
function T:startPlay()
	self.startTimeMS=TimeUtils:getSyncMillis()

	self:checkAndPop()
	self.checkIntervalSid=Cocos2dxUtils:interval(function()
		self:checkAndPop()
	end, 0)
end

--- 检查事件队列,如果有且到了执行的时间,则开始执行
function T:checkAndPop()
	if self.actionQueue:isEmpty() then
		return
	end

	--当前的战斗时间
	local curFightTime=TimeUtils:getSyncMillis()-self.startTimeMS
	while true do
		local action=self.actionQueue:look()
		if action and action.time <= curFightTime then
			xpcall(
				function()
					self:execAction(action)
				end,
				function(errStr)
					local message=tostring(errStr).."\n"..debug.traceback()

					__print__("execFightActionErr",message)
					LogRecorder:push(message,LOG_LEVEL_ERROR,"execFightActionErr")
				end
			)
			self.actionQueue:pop()
		else
			break
		end
	end
end

--- 执行事件
function T:execAction(action)
	local t=action.type
	local data=action.data

	if t == FightActionType.INIT.type then
		self:fightInitHandler(data)
	elseif t == FightActionType.START.type then
		self.fightScene:start(data)
	elseif t == FightActionType.NEXT_ROUND.type then
		self.fightScene:setRound(data)
	elseif t == FightActionType.AIRCRAFT_ATTACK.type then
		self.fightScene:attack(data)
	elseif t == FightActionType.AIRCRAFT_BE_ATTACKED.type then
		self.fightScene:beAttack(data)
	elseif t == FightActionType.FIGHT_END.type then
		CommonBroadcast:luaDispatchEvent(EVENT_FIGHT_SCENE_QUIT,data)
	elseif t == FightActionType.FIGHT_ERR.type then
		MUIAlter.show({
			info = Language:get("ug400"),
			size = 30,
			callback=function(flag)
				CommonBroadcast:luaDispatchEvent(EVENT_FIGHT_SCENE_QUIT)
			end
		})
	end
end

--- 初始化战斗
function T:fightInitHandler(data)
	self.fightScene:initFightData(data)
end

return T
