--- 攻击类型计算器
local AttackTypeCalcer = { }

--- 获取攻击目标列表
function AttackTypeCalcer:getTargets(attackType, srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	if attackType == AttackType.AT_1 then
		return self:findTargetByAT1(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	elseif attackType == AttackType.AT_2 then
		return self:findTargetByAT2(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	elseif attackType == AttackType.AT_3 then
		return self:findTargetByAT3(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	elseif attackType == AttackType.AT_4 then
		return self:findTargetByAT4(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	end
end

--- 获得横排攻击目标
function AttackTypeCalcer:findTargetByAT1(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	local enemyAircrafts =(srcCamp == Camp.CAMP1) and camp2Aircrafts or camp1Aircrafts

	local r = { }
	for i = 0, 2, 1 do
		if enemyAircrafts[i] then
			r[#r + 1] = enemyAircrafts[i]
		elseif enemyAircrafts[i + 3] then
			r[#r + 1] = enemyAircrafts[i + 3]
		end
	end
	return r
end

--- 获得单体攻击目标
function AttackTypeCalcer:findTargetByAT2(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	local enemyAircrafts =(srcCamp == Camp.CAMP1) and camp2Aircrafts or camp1Aircrafts

	local r = { }

	-- 是否正序寻找下一个目标
	local isAsce = srcLoc % 3 ~= 2;

	-- 每次寻找偏移量
	local shiftingNum = 0;
	for i = 0, 2, 1 do
		local targetLoc =(srcLoc + shiftingNum) % 3;
		shiftingNum = isAsce and shiftingNum + 1 or shiftingNum + 2

		if enemyAircrafts[targetLoc] then
			r[#r + 1] = enemyAircrafts[targetLoc]
			break
		elseif enemyAircrafts[targetLoc + 3] then
			r[#r + 1] = enemyAircrafts[targetLoc + 3]
			break
		end
	end
	return r
end

--- 获得纵排攻击目标
function AttackTypeCalcer:findTargetByAT3(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	local enemyAircrafts =(srcCamp == Camp.CAMP1) and camp2Aircrafts or camp1Aircrafts

	local r = { }

	-- 是否正序寻找下一个目标
	local isAsce = srcLoc % 3 ~= 2;

	-- 每次寻找偏移量
	local shiftingNum = 0;

	for i = 0, 2, 1 do
		local targetLoc =(srcLoc + shiftingNum) % 3;
		shiftingNum = isAsce and shiftingNum + 1 or shiftingNum + 2

		if enemyAircrafts[targetLoc] or enemyAircrafts[targetLoc + 3] then
			if enemyAircrafts[targetLoc] then
				r[#r + 1] = enemyAircrafts[targetLoc]
			end
			if enemyAircrafts[targetLoc + 3] then
				r[#r + 1] = enemyAircrafts[targetLoc + 3]
			end
			break
		end
	end
	return r
end

--- 获得全体攻击目标
function AttackTypeCalcer:findTargetByAT4(srcCamp, srcLoc, camp1Aircrafts, camp2Aircrafts)
	local enemyAircrafts =(srcCamp == Camp.CAMP1) and camp2Aircrafts or camp1Aircrafts

	local r = { }
	for i = 0, 5, 1 do
		if enemyAircrafts[i] then
			r[#r + 1] = enemyAircrafts[i]
		end
	end
	return r
end

--- 攻击加成计算
-- @param srcType 攻击战机类型
-- @param desType 被攻击战机类型
-- @param damage 伤害
-- @param restraintAdd 克制伤害加成
-- @param antiRestraintAdd 反克制伤害加成
function AttackTypeCalcer:attackBonusCalc(srcType, desType, damage, restraintAdd, antiRestraintAdd)
    restraintAdd=restraintAdd or 0
    antiRestraintAdd=antiRestraintAdd or 0

	if srcType == AttackType.AT_1 then
		if desType == AttackType.AT_4 then
			-- 横排克群攻125%
			damage = damage *(1.25 + (restraintAdd - antiRestraintAdd)/100)
		end
		-- 横排没有不克的
	elseif srcType == AttackType.AT_2 then
		if desType == AttackType.AT_1 then
			-- 单体克横排125%
			damage = damage *(1.25 + (restraintAdd - antiRestraintAdd)/100)
		elseif desType == AttackType.AT_3 then
			-- 单体不克穿刺80%
			damage = damage * 0.8
		end
	elseif srcType == AttackType.AT_3 then
		if desType == AttackType.AT_2 then
			-- 穿刺克单体125%
			damage = damage *(1.25 + (restraintAdd - antiRestraintAdd)/100)
		elseif desType == AttackType.AT_1 then
			-- 穿刺不克横排80%
			damage = damage * 0.8
		end
	elseif srcType == AttackType.AT_4 then
		if desType == AttackType.AT_3 then
			-- 群体克穿刺125%
			damage = damage *(1.25 + (restraintAdd - antiRestraintAdd)/100)
		elseif desType == AttackType.AT_2 then
			-- 群体不克单体80%
			damage = damage * 0.8
		end
	end

	return damage
end

return AttackTypeCalcer
