local _print = print
local function print(...)
--    _print("<FightLogicCalcer>: ", table.concat( { ...}, "\t"))
end

--- 战斗逻辑执行器
local T = Class.FightLogicCalcer()

--- 战机逻辑执行器
T.AircraftLogicExecor = require("src.app.logic.fightLogic.AircraftLogicCalcer")

--- 攻击类型计算器
T.AttackTypeCalcer = require("src.app.logic.fightLogic.AttackTypeCalcer")

--- 战机buff计算器
T.FightBuffCalcer = require("src.app.logic.fightLogic.FightBuffCalcer")

--- 玩家的buff对战斗的影响计算器
T.PropBuffCalcer = require("src.app.logic.fightLogic.PropBuffCalcer")

--- 战败条件计算器
T.FailureConditionsCalcer = require("src.app.logic.fightLogic.FailureConditionsCalcer")

--- 最大回合数,超过最大回合数,则阵营2(防御方)胜利
T.MAX_ROUND_COUNT = 20

--- 最大战机数
T.MAX_AIRCRAFT_COUNT = 12

--- 战斗编号
T.id = 0

function T:def()
    T.id = T.id + 1

    --- 战斗编号
    self._id = T.id

    --- 开始时间
    -- self.startTimeMS = nil

    --- 战斗数据
    -- self.fightData = nil

    --- 胜利方(1:阵营1,2:阵营2)
    -- self.winner = nil

    --- 先手(0:阵营1先手,1:阵营2先手)
    self.first = 0

    --- 当前开火的战机索引
    -- self.curAircraftIndex = nil

    --- 阵营1战败条件
    -- self.camp1FailureConditions = nil

    --- 阵营2战败条件
    -- self.camp2FailureConditions = nil

    --- 阵营1当前开火的战机索引
    self.camp1CurAircraftIndex = 0

    --- 阵营1当前开火的战机索引
    self.camp2CurAircraftIndex = 0

    --- 当前回合
    -- self.curRound = nil

    --- 阵营1玩家buff标记
    -- self.camp1PropBuffCode=nil

    --- 阵营1各类型战机攻击后再次出手的概率
    -- self.camp1ContinuesRates=nil

    --- 阵营1战机列表
    -- self.camp1Aircrafts = nil

    --- 阵营1buff列表
    -- self.camp1Buffs = nil

    --- 阵营2玩家buff标记
    -- self.camp2PropBuffCode=nil

    --- 阵营2各类型战机攻击后再次出手的概率
    -- self.camp2ContinuesRates=nil

    --- 阵营2战机列表
    -- self.camp2Aircrafts = nil

    --- 阵营2buff列表
    -- self.camp2Buffs = nil

    --- 下一轮的延迟计时器ID
    -- self.nextRoundTimeoutSid=nil

    --- 攻击到击中目标的延迟计时器ID
    -- self.attack2HitTimeoutSid=nil

    --- 击中目标后到下一个战机攻击的延迟计时器ID
    -- self.toNextAircraftTimeoutSid=nil
end

function T:ctor()
    self.random = Random:new()
end

--- 初始化
-- @param fightData 战斗数据
function T:init(fightData)
    self.fightData = fightData
    self:initFightData(fightData.fightInitData)

    -- 收到"跳过战斗"的信息
    Adapter_local:addChannel(ChannelIds.SKIP_FIGHT, function()
        print("id=" .. self._id, "收到跳过战斗的信息")

        if self.nextRoundTimeoutSid then
            Cocos2dxUtils:clearScheduler(self.nextRoundTimeoutSid)
        end

        if self.attack2HitTimeoutSid then
            Cocos2dxUtils:clearScheduler(self.attack2HitTimeoutSid)
        end

        if self.toNextAircraftTimeoutSid then
            Cocos2dxUtils:clearScheduler(self.toNextAircraftTimeoutSid)
        end

        self:fightEnd(self.winner == Camp.CAMP1 and Camp.CAMP2 or Camp.CAMP1)

        return true
    end )
end

--- 初始化战斗数据
function T:initFightData(initData)
    self.startTimeMS = TimeUtils:getSyncMillis()
    self.curRound = 0

    -- 初始化随机数种子
    self.random:setSeed(initData.seed)

    self.first = initData.first
    self.winner = initData.winner

    local camp1 = initData.camp1
    local camp2 = initData.camp2

    self.camp1FailureConditions = camp1.failureConditions
    self.camp2FailureConditions = camp2.failureConditions

    self.camp1PropBuffCode = camp1.playerBuffCode
    self.camp1ContinuesRates = {
        [AttackType.AT_1] = camp1.continuesAT_1,
        [AttackType.AT_2] = camp1.continuesAT_2,
        [AttackType.AT_3] = camp1.continuesAT_3,
        [AttackType.AT_4] = camp1.continuesAT_4
    }
    local camp1Aircrafts = { }
    for k, v in pairs(camp1.aircrafts) do
        local aircraft = self.AircraftLogicExecor(self, Camp.CAMP1, k, v)
        aircraft:luaAddEventListener(self.AircraftLogicExecor.EVENT_ATTACK, self, self.attackHandler)

        camp1Aircrafts[k] = aircraft
    end
    self.camp1Aircrafts = camp1Aircrafts

    self.camp2PropBuffCode = camp2.playerBuffCode
    self.camp2ContinuesRates = {
        [AttackType.AT_1] = camp2.continuesAT_1,
        [AttackType.AT_2] = camp2.continuesAT_2,
        [AttackType.AT_3] = camp2.continuesAT_3,
        [AttackType.AT_4] = camp2.continuesAT_4
    }
    local camp2Aircrafts = { }
    for k, v in pairs(camp2.aircrafts) do
        local aircraft = self.AircraftLogicExecor(self, Camp.CAMP2, k, v)
        aircraft:luaAddEventListener(self.AircraftLogicExecor.EVENT_ATTACK, self, self.attackHandler)

        camp2Aircrafts[k] = aircraft
    end
    self.camp2Aircrafts = camp2Aircrafts

    self:dispatchFightAction( {
        type = FightActionType.INIT.type,
        data = initData
    } )
end

--- 战斗开始
function T:start()
    self:dispatchFightAction( {
        type = FightActionType.START.type,
        data = nil
    } )
    self.nextRoundTimeoutSid = Cocos2dxUtils:timeOut( function()
        self.nextRoundTimeoutSid = nil

        self:nextRound()
    end , FightActionType.START.keepTimeMS / 1000)
end

--- 下一回合
function T:nextRound()
    self.curRound = self.curRound + 1

    -- 超过最大回合数,则阵营2胜利
    if self.curRound > self.MAX_ROUND_COUNT then
        self:fightEnd(Camp.CMAP2)
    end

    print("id=" .. self._id, "第" .. self.curRound .. "回合开始")

    if self.first == 0 or self.first == 2 then
        -- 阵营1先手
        self.curAircraftIndex = -1
        print("id=" .. self._id, "阵营1先手")
    else
        -- 阵营2先手
        self.curAircraftIndex = -2
        print("id=" .. self._id, "阵营2先手")
    end

    -- 重置当前战机索引
    self.camp1CurAircraftIndex = 0
    self.camp2CurAircraftIndex = 0

    self:dispatchFightAction( {
        type = FightActionType.NEXT_ROUND.type,
        data = self.curRound
    } )

    self.nextRoundTimeoutSid = Cocos2dxUtils:timeOut( function()
        self.nextRoundTimeoutSid = nil

        self:nextAircraft()
    end , FightActionType.NEXT_ROUND.keepTimeMS / 1000)
end

--- 下一个战机
function T:nextAircraft()
    -- 更新执行AI的战机的索引,索引大于最大索引则进入下一回合
    local i = self.curAircraftIndex + 1
    if i >= self.MAX_AIRCRAFT_COUNT then
        self:nextRound()
        return
    end
    self.curAircraftIndex = i

    local aircraft
    if i % 2 == 0 then
        local j = self.camp1CurAircraftIndex
        while j <= 5 do
            aircraft = self.camp1Aircrafts[j]
            if aircraft then
                break
            end
            j = j + 1
        end

        self.camp1CurAircraftIndex = j + 1
    else
        local j = self.camp2CurAircraftIndex
        while j <= 5 do
            aircraft = self.camp2Aircrafts[j]
            if aircraft then
                break
            end
            j = j + 1
        end

        self.camp2CurAircraftIndex = j + 1
    end

    if aircraft then
        --- 刷新buff列表
        self.camp1Buffs = self.FightBuffCalcer:getBuffList(self.camp1Aircrafts)
        self.camp2Buffs = self.FightBuffCalcer:getBuffList(self.camp2Aircrafts)

        print("id=" .. self._id, "<战斗逻辑>",
        (aircraft.camp == Camp.CAMP1 and "阵营1" or "阵营2") .. "第" .. aircraft.loc .. "号位执行AI"
        )
        aircraft:execAI()
    else
        self:nextAircraft()
    end
end

-- -
function T:attackHandler(srcAircraft, attackEffName)
    local l = self.AttackTypeCalcer:getTargets(
    srcAircraft.dataTemplate.attackType,
    srcAircraft.camp,
    srcAircraft.loc,
    self.camp1Aircrafts,
    self.camp2Aircrafts
    )

    if #l > 0 then

    else
        print("id=" .. self._id, "未找到攻击目标")
    end

    local baseDamage = srcAircraft.singleAttack * srcAircraft.count

    print("id=" .. self._id, "攻击=>", "类型=" .. srcAircraft.dataTemplate.attackType)

    local desList = { }
    for k, v in pairs(l) do
        local desCamp = v.camp
        print("id=" .. self._id, "攻击目标=>",(v.camp == Camp.CAMP1 and "阵营1" or "阵营2") .. "第" .. v.loc .. "号位")

        -- 目标
        local des = { camp = desCamp, loc = v.loc }
        desList[#desList + 1] = des

        -- 进攻方玩家buff标记
        local attackPropBuffCode
        -- 防御方玩家buff标记
        local defencePropBuffCode
        if srcAircraft.camp == Camp.CAMP1 then
            attackPropBuffCode = self.camp1PropBuffCode
            defencePropBuffCode = self.camp2PropBuffCode
        else
            attackPropBuffCode = self.camp2PropBuffCode
            defencePropBuffCode = self.camp1PropBuffCode
        end

        -- 进攻方部队buff列表
        local attackBuffs
        -- 防御方部队buff列表
        local defenceBuffs
        if srcAircraft.camp == Camp.CAMP1 then
            attackBuffs = self.camp1Buffs
            defenceBuffs = self.camp2Buffs
        else
            attackBuffs = self.camp2Buffs
            defenceBuffs = self.camp1Buffs
        end

        -- 攻击方攻击类型
        local attackAttackType = srcAircraft.dataTemplate.attackType
        -- 防御方攻击类型
        local defenceAttackType = v.dataTemplate.attackType

        -- 命中计算>>------------------------------------------------------------
        -- 命中
        local hit, add, rate = srcAircraft.baseHit, 0, 0
        print("id=" .. self._id, "攻击=>", "基础     [命中] =" .. hit)

        -- 玩家buff对命中的影响
        add = add + self.PropBuffCalcer:calcAttackHitAdd(attackPropBuffCode, defencePropBuffCode)
        print("id=" .. self._id, "攻击=>", "玩家buff [命中增加] =" .. add .. "," .. rate .. "%")

        -- 部队buff对命中的影响
        local t1, t2 = self.FightBuffCalcer:calcAttackHitAdd(attackBuffs, defenceAttackType)
        add, rate = add + t1, rate + t2
        print("id=" .. self._id, "攻击=>", "部队buff [命中增加] =" .. add .. "," .. rate .. "%")

        hit = hit *(1 + rate / 100) + add
        print("id=" .. self._id, "攻击=>", "最终     [命中] =" .. hit)
        -- <<命中计算------------------------------------------------------------

        -- 闪避计算>>------------------------------------------------------------
        -- 闪避
        local dodge, add, rate = v.baseDodge, 0, 0
        print("id=" .. self._id, "攻击=>", "基础     [闪避] =" .. dodge)

        -- 玩家buff对闪避的影响
        add = add + self.PropBuffCalcer:calcDefenceDodgeAdd(attackPropBuffCode, defencePropBuffCode)
        print("id=" .. self._id, "攻击=>", "玩家buff [闪避增加] =" .. add .. "," .. rate .. "%")

        -- 部队buff对闪避的影响
        local t1, t2 = self.FightBuffCalcer:calcDefenceDodgeAdd(defenceBuffs, attackAttackType)
        add, rate = add + t1, rate + t2
        print("id=" .. self._id, "攻击=>", "部队buff [闪避增加] =" .. add .. "," .. rate .. "%")

        dodge = dodge *(1 + rate / 100) + add
        print("id=" .. self._id, "攻击=>", "最终     [闪避] =" .. dodge)
        -- <<闪避计算------------------------------------------------------------

        -- 计算是否命中>>--------------------------------------------------------
        -- 是否命中
        local isHit = true
        local rand = self.random:randomBetweenInt(0, 10000) / 100
        if rand >= hit + 100 - dodge then
            isHit = false
        end
        print("id=" .. self._id, "攻击=>", "随机数=" .. rand .. ", 是否命中=" .. tostring(isHit))
        des.isHit = isHit
        -- <<计算是否命中--------------------------------------------------------

        if isHit then
            -- 暴击计算>>------------------------------------------------------------
            -- 暴击
            local crit, add, rate = srcAircraft.baseCrit, 0, 0
            print("id=" .. self._id, "攻击=>", "基础     [暴击] =" .. crit)

            -- 玩家buff对暴击的影响
            add = add + self.PropBuffCalcer:calcAttackCritAdd(attackPropBuffCode, defencePropBuffCode)
            print("id=" .. self._id, "攻击=>", "玩家buff [暴击增加] =" .. add .. "," .. rate .. "%")

            -- 部队buff对暴击的影响
            local t1, t2 = self.FightBuffCalcer:calcAttackCritAdd(attackBuffs, defenceAttackType)
            add, rate = add + t1, rate + t2
            print("id=" .. self._id, "攻击=>", "部队buff [暴击增加] =" .. add .. "," .. rate .. "%")

            crit = crit *(1 + rate / 100) + add
            print("id=" .. self._id, "攻击=>", "最终     [暴击] =" .. crit)
            -- <<暴击计算------------------------------------------------------------

            -- 暴击抵抗计算>>--------------------------------------------------------
            -- 暴击抵抗
            local critResist, add, rate = v.baseCritResist, 0, 0
            print("id=" .. self._id, "攻击=>", "基础     [暴击抵抗] =" .. critResist)

            -- 玩家buff对暴击的影响
            add = add + self.PropBuffCalcer:calcDefenceCritResistAdd(attackPropBuffCode, defencePropBuffCode)
            print("id=" .. self._id, "攻击=>", "玩家buff [暴击抵抗增加] =" .. add .. "," .. rate .. "%")

            -- 部队buff对暴击的影响
            local t1, t2 = self.FightBuffCalcer:calcDefenceCritResistAdd(defenceBuffs, attackAttackType)
            add, rate = add + t1, rate + t2
            print("id=" .. self._id, "攻击=>", "部队buff [暴击抵抗增加] =" .. add .. "," .. rate .. "%")

            critResist = critResist *(1 + rate / 100) + add
            print("id=" .. self._id, "攻击=>", "最终     [暴击抵抗] =" .. critResist)
            -- <<暴击抵抗计算--------------------------------------------------------

            -- 计算是否暴击>>--------------------------------------------------------
            -- 是否暴击
            local isCrit = false
            local rand = self.random:randomBetweenInt(0, 10000) / 100
            if rand < crit - critResist then
                isCrit = true
            end
            print("id=" .. self._id, "攻击=>", "随机数=" .. rand .. ", [是否暴击] =" .. tostring(isCrit))
            des.isCrit = isCrit
            -- <<计算是否暴击--------------------------------------------------------

            -- 伤害计算>>------------------------------------------------------------
            -- 伤害
            local damage = baseDamage * 100
            print("id=" .. self._id, "攻击=>", "基础     [伤害] =" .. damage)

            if isCrit then
                damage = damage *(( 200 + srcAircraft.critIncr - v.critReduce)/100)
            end
            print("id=" .. self._id, "攻击=>", "计算暴击 [伤害] =" .. damage)

            -- 玩家buff对伤害的影响
            damage =damage *(100 + self.PropBuffCalcer:calcDamageInfluence(attackPropBuffCode, defencePropBuffCode)) / 100
            print("id=" .. self._id, "攻击=>", "玩家buff [伤害] =" .. damage)

            -- 部队buff对伤害的影响
            damage = self.FightBuffCalcer:calcAttackBuffAdd(attackBuffs, damage, defenceAttackType)
            print("id=" .. self._id, "攻击=>", "部队加成 [伤害] =" .. damage)

            -- 计算伤害减免
            damage = self.FightBuffCalcer:calcDefenceBuffMin(defenceBuffs, damage, attackAttackType)
            print("id=" .. self._id, "攻击=>", "部队减免 [伤害] =" .. damage)

            -- 计算克制
            damage = self.AttackTypeCalcer:attackBonusCalc(attackAttackType, defenceAttackType, damage, srcAircraft.restraint, v.antiRestraint)
            print("id=" .. self._id, "攻击=>", "计算克制 [伤害] =" .. damage)

            damage = math.floor(damage / 100)
            if damage <= 0 then
                damage = 1
            end
            print("id=" .. self._id, "攻击=>", "最终 [伤害] =" .. damage)

            des.damage = damage
            -- <<伤害计算------------------------------------------------------------
        end
    end

    local data = {
        src =
        {
            camp = srcAircraft.camp,
            loc = srcAircraft.loc,
            attackEffName = attackEffName
        },
        desList = desList,
        keepTimeMS = FightActionType.AIRCRAFT_ATTACK.keepTimeMS
    }



    self:dispatchFightAction( {
        type = FightActionType.AIRCRAFT_ATTACK.type,
        data = data
    } )

    self.attack2HitTimeoutSid = Cocos2dxUtils:timeOut( function()
        self.attack2HitTimeoutSid = nil

        self:beAttacked(data)
    end , data.keepTimeMS / 1000)
end

--- 导弹飞行完成
function T:beAttacked(hitData)
    local src = hitData.src
    local srcAircraft =(src.camp == Camp.CAMP1) and self.camp1Aircrafts[src.loc] or self.camp2Aircrafts[src.loc]

    -- 失败阵营
    local failureCamp = nil

    -- 遍历被攻击战机,计算伤害
    for k, v in pairs(hitData.desList) do
        local desCamp = v.camp
        local desLoc = v.loc
        local desCampAircrafts =(desCamp == Camp.CAMP1) and self.camp1Aircrafts or self.camp2Aircrafts
        local desAircraft = desCampAircrafts[desLoc]

        v.count = desAircraft:beAttack(srcAircraft, v)
        if v.count == 0 then
            desCampAircrafts[desLoc] = nil
        end

        print("id=" .. self._id, "被攻击=>",
        "目标=" ..(desCamp == Camp.CAMP1 and "阵营1" or "阵营2") .. "第" .. desLoc .. "号位",
        "伤害=" .. tostring(v.damage),
        "剩余战机数量=" .. v.count
        )

        -- 战败条件
        if self.FailureConditionsCalcer:calc(desCampAircrafts, desLoc,(desCamp == Camp.CAMP1) and self.camp1FailureConditions or self.camp2FailureConditions) then
            failureCamp = desCamp

            print("id=" .. self._id, "战斗结束=>", "战败阵营:" ..(failureCamp == Camp.CAMP1 and "阵营1" or "阵营2"))
        end
    end
    hitData.keepTimeMS = FightActionType.AIRCRAFT_BE_ATTACKED.keepTimeMS

    self:dispatchFightAction( {
        type = FightActionType.AIRCRAFT_BE_ATTACKED.type,
        data = hitData
    } )

    self.toNextAircraftTimeoutSid = Cocos2dxUtils:timeOut( function()
        self.toNextAircraftTimeoutSid = nil

        if failureCamp then
            self:fightEnd(failureCamp)
        else
            if not srcAircraft.isSecondAttack then
                -- 再次攻击的概率
                local continuesRate =(src.camp == Camp.CAMP1) and self.camp1ContinuesRates[srcAircraft.dataTemplate.attackType] or self.camp2ContinuesRates[srcAircraft.dataTemplate.attackType]
                if continuesRate > 0 then
                    local rand = self.random:randomBetweenInt(0, 100)
                    if rand < continuesRate then
                        self.curAircraftIndex = self.curAircraftIndex - 1
                        if srcAircraft.camp == Camp.CAMP1 then
                            self.camp1CurAircraftIndex = self.camp1CurAircraftIndex - 1
                        else
                            self.camp2CurAircraftIndex = self.camp2CurAircraftIndex - 1
                        end
                        srcAircraft.isSecondAttack = true
                    end
                end
            else
                srcAircraft.isSecondAttack = nil
            end

            self:nextAircraft()
        end
    end , hitData.keepTimeMS / 1000)
end

--- 战斗结束
-- @param failureCamp 战斗失败阵营
function T:fightEnd(failureCamp)
    Adapter_local:rmChannel(ChannelIds.SKIP_FIGHT)

    if self.winner == failureCamp then
        print("id=" .. self._id, "<战斗逻辑ERR>", "战斗结束", "结果与服务器的不一致")

        self:dispatchFightAction( {
            type = FightActionType.FIGHT_ERR.type,
            data =
            {
                errCode = "result is incorrect"
            }
        } )

        return
    end

    local isWinner, initData = false, self.fightData.fightInitData
    if initData.camp1.id == UserInfo.id and failureCamp == Camp.CAMP2
        or
        initData.camp2.id == UserInfo.id and failureCamp == Camp.CAMP1
    then
        isWinner = true
    end

    local fightData = self.fightData

    -- 战斗结果数据
    local endData = fightData.endData or { isWinner = isWinner }
    if endData.isWinner == nil then
        endData.isWinner = isWinner
    end

    -- 双方阵营
    endData.camps = {
        camp1 = fightData.fightInitData.camp1,
        camp2 = fightData.fightInitData.camp2
    }

    self:dispatchFightAction( {
        type = FightActionType.FIGHT_END.type,
        data = fightData.endData
    } )
end

--- 派发战斗事件
function T:dispatchFightAction(action)
    local curFightTime = TimeUtils:getSyncMillis() - self.startTimeMS

    action.time = curFightTime

    Adapter_local:write(BroadcastRegister.LOCAL_FIGHT, action)
end

return T
