--- 每日状态列表
local T = Class.DailyStates(LuaEventDispatcher2)

--- 事件类型-每日状态被更新
T.EVT_TYPE_UPDATE_DAILY_STATES="evt_type_update_daily_states"

--- 在线奖励登录时间，因为在线奖励每日要做清理，所以登录时间也要清理
T.ONLINE_LOGIN_TIME = 1

--- 钻石购买精力点次数
T.PVE_POINT_BUY_COUNT = 2

--- 联盟邮件发送次数
T.ALLIANCE_MAIL_COUNT = 3

--- 竞技场每日挑战次数
T.ARENA_FIGHT_NUM=4

--- 竞技场每日购买次数
T.ARENA_BUY_NUM=5

--- 补给线每日购买次数
T.SUPPLYLINE_BUY_NUM=6


function T:def()
	LuaEventDispatcher2.def(self)

	--- 状态列表
	T.list = { }

    --- 大转盘次数
    self.bigWheelNumState = require("src.app.model.BigWheelNumState")()

	--- 当前日期
--	T.currentDay=-1
end

--- 解析每日状态
function T:decode(data)
	local cDate=os.date("*t",TimeUtils:getSyncMillis()/1000)
--	self.currentDay=cDate.day

	local len = data:readUnsignedShort()

	local list = { }
	for i = 1, len, 2 do
		local key = data:readUnsignedShort()
		local value = data:readInt()

		list[key] = value
	end

	self.list = list

    self.bigWheelNumState:setList(list)

end

-- 重置每日状态
function T:reset(data)
	self:decode(data)
	
    -- 还有补给线呢？？？？？

	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end

-- 清除每日状态
function T:clear()  
    self.list = {}

    SupplyLineManager.energyCostMark = 0
    SupplyLineManager.supplyLineMark = 0


    self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end

----- 清除每日状态
--function T:clear()
--	local cDate=os.date("*t",TimeUtils:getSyncMillis()/1000)
--	self.currentDay=cDate.day
--
--	self.list = {}
--	
--	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
--end

--- 获取联盟已发送邮件数目
function T:getAllianceMailCount()
	return self.list[self.ALLIANCE_MAIL_COUNT] or 0
end
--- 设置联盟已发送邮件数目
function T:setAllianceMailCount(value)
	self.list[self.ALLIANCE_MAIL_COUNT] = value
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end
--- 增加联盟已发送邮件数目
function T:addAllianceMailCount(value)
	local v = self.list[self.ALLIANCE_MAIL_COUNT] or 0
	self.list[self.ALLIANCE_MAIL_COUNT] = v + value
end


--- 获取钻石购买精力点次数
function T:getPointCount()
	return self.list[self.PVE_POINT_BUY_COUNT] or 0
end
--- 设置钻石购买精力点次数
function T:setPointCount(value)
	self.list[self.PVE_POINT_BUY_COUNT] = value
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end
--- 增加钻石购买精力点次数
function T:addPointCount(value)
	local v = self.list[self.PVE_POINT_BUY_COUNT] or 0
	self.list[self.PVE_POINT_BUY_COUNT] = v + value
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end

--- 购买下一次精力点耗费的钻石数目
function T:buyPointCostGems()
	local point = self:getPointCount()
	return (point + 1) * 5
end

--- 增加今日竞技场挑战次数
function T:addArenaFightNum(value)
	self.list[self.ARENA_FIGHT_NUM] = self:getArenaFightNum() + value
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end
--- 获取今日竞技场挑战次数
function T:getArenaFightNum()
	return self.list[self.ARENA_FIGHT_NUM] or 0
end

--- 增加今日竞技场挑战点购买次数
function T:addArenaBuyNum(value)
	self.list[self.ARENA_BUY_NUM]=self:getArenaBuyNum() + value
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end
--- 获取今日竞技场挑战点购买次数
function T:getArenaBuyNum()
	return self.list[self.ARENA_BUY_NUM] or 0
end



--- 增加今日补给线购买次数
function T:addSupplyLineBuyNum(value)
	self.list[self.SUPPLYLINE_BUY_NUM] = self:getSupplyLineBuyNum() + value

	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_DAILY_STATES)
end

--- 获取今日补给线购买次数
function T:getSupplyLineBuyNum()
	return self.list[self.SUPPLYLINE_BUY_NUM] or 0
end

return T
