--[[
    配件盒子
]]
local T = Class.AccessoryBox(LuaEventDispatcher2)

-- 配件信息
T.AccessoryInfo = require("src.app.model.accessory.AccessoryInfo")

-- 改造耗费的蓝图数目,每一个改造等级到下一个改造等级对应的sid数目(这个也是分解配件后所要累加前面的数量给蓝图)
T.reFormBlueCount = {1,3,5,10,20}

--- 强化成功率数组:配件从1级到59级时升级的成功率
T.successRate = {100,100,100,100,90,90,90,90,90,
                 80,80,80,80,80,70,70,70,70,70,60,60,60,60,60,50,50,50,50,50,40,40,40,40, 
                 40,30,30,30,30,30,20,20,20,20,20,15,15,15,15,15,10,10,10,10,10,5,5,5,5,5}

--- 每种品质改造等级的加成比
T.addPropertyByRrformLevel = {{0.05,0.1,0.15,0.2,0.25},{0.1,0.2,0.3,0.4,0.5},{0.15,0.3,0.45,0.6,0.75},{0.2,0.4,0.6,0.8,1}}

--- 分别代表，紫色闪避加成，橙色闪避加成，紫色命中加成，橙色命中加成(配件sid表)
T.hitOrDodgeSids = {{21,23,25},{31,33,35},{22,24,26},{32,34,36}} 

--- 分别代表，紫色闪避加成，橙色闪避加成，紫色命中加成，橙色命中加成(每个改造等级对应的数值)
T.addHitOrDodgeRelevel = {{0,0,0.2,0.4,0.8,1.5},{0,0,0.5,1,2,5},{0,0,0.2,0.4,0.8,1.5},{0,0,0.5,1,2,5}} 


--- 事件-分解配件
T.EVT_DECOMPOSE_ACCESSORY_REFRESH = "evt_decompose_accessory_refresh"

--- 事件-端口刷新配件
T.EVT_PORT_ACCESSORY_REFRESH = "evt_port_accessory_refresh"

--- 事件-升级成功
T.EVT_LEVEL_UP_SUCCESS_REFRESH = "evt_level_up_success_refresh"

--- 事件-改造成功
T.EVT_REFORM_SUCCESS_REFRESH = "evt_reform_success_refresh"


function T:def()
    LuaEventDispatcher2.def(self)

    -- 配件信息
    self.accessoryInfos = {}

end

function T:decode(data)
	local len = data:readUnsignedByte()

	local infos = {}
	for i = 1,len,1 do
		local info = self.AccessoryInfo()
		info:decode(data)
		infos[info.id] = info
	end

	self.accessoryInfos = infos

end

--- 端口刷新得到一个配件
function T:refreshOneAccessory(data)
    local info = self.AccessoryInfo()
	info:decode(data)

	self.accessoryInfos[info.id] = info

    self:luaDispatchEvent(self.EVT_PORT_ACCESSORY_REFRESH)
    PlayerInstance:setPlayerFightLogic()
end

--- 端口刷新得到一个配件并且得到id
function T:refreshOneAccessoryAndGetId(data)
    local info = self.AccessoryInfo()
	info:decode(data)

	self.accessoryInfos[info.id] = info

    self:luaDispatchEvent(self.EVT_PORT_ACCESSORY_REFRESH)
    PlayerInstance:setPlayerFightLogic()

    return info.id
end

--- 得到某-个槽位的最强的配件id（是没有使用的）（用于显示），这里是一种规则，到时要问翔哥
function T:getStrongestAccessory(slotType)
    local accessoryInfos = self.accessoryInfos
    local getDataTable = {}
    for k,v in pairs(accessoryInfos) do
        if v and v.airType == nil and v.slotType and  v.slotType == slotType then
            getDataTable[#getDataTable + 1] = v
        end
    end

    -- 然后再来从getDataTable选出来一项作为最强的，暂时用第1项,返回的是这一项的数据，不是id
    if #getDataTable > 0 then
        table.sort(getDataTable,function(a,b)
		    return a.sid > b.sid
	    end)
        return getDataTable[1]

    else -- 如果没有的话，返回空值
        return nil
    end

end

--- 分解配件（这里是分解1个，正在使用的是不能分解的）
function T:decomposeAccessory(id)
    self.accessoryInfos[id] = nil
    self:luaDispatchEvent(self.EVT_DECOMPOSE_ACCESSORY_REFRESH)
    PlayerInstance:setPlayerFightLogic()
end


--- 得到批量分解的品质（品质从0到3）
--- 传入参数为一个表：{quality0 = 0,quality1 = 1,quality2 = 2,quality3 = 3},不存在的传nil
function T:getBatchDecomposeAccQuality(quality)
    local value1 = nil
    if quality.quality0 then
        value1 = BitOperator:_ls(1, quality.quality0)
    end

    if quality.quality1 then
        if value1 then
            value1 = BitOperator:_or(value1,BitOperator:_ls(1, quality.quality1))
        else
            value1 = BitOperator:_ls(1, quality.quality1)
        end
    end

    if quality.quality2 then
        if value1 then
            value1 = BitOperator:_or(value1,BitOperator:_ls(1, quality.quality2))
        else
            value1 = BitOperator:_ls(1, quality.quality2)
        end
    end

    if quality.quality3 then
        if value1 then
            value1 = BitOperator:_or(value1,BitOperator:_ls(1, quality.quality3))
        else
            value1 = BitOperator:_ls(1, quality.quality3)
        end
    end


    return value1
end

--- 判断批量分解的是否可以分解,只要有一个，就返回true
function T:judgeIsBatchDecomposeAccessory(quality)
    if quality.quality0 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality0 then
                return true
            end
        end
    end

    if quality.quality1 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality1 then
                return true
            end
        end
    end

    if quality.quality2 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality2 then
                return true
            end
        end
    end

    if quality.quality3 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality3 then
                return true
            end
        end
    end

    return false
end

--- 得到批量分解的对应的所有的蓝图碎片数量总和,这里是必然存在的
function T:getBatchDecomposeAccessoryBlueFragNum(quality)
    local blueSid206Count = 0
    if quality.quality0 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality0 then
                blueSid206Count = blueSid206Count + v:getDecomposeBlueFragNum()
            end
        end
    end

    if quality.quality1 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality1 then
                blueSid206Count = blueSid206Count + v:getDecomposeBlueFragNum()
            end
        end
    end

    local blueSid207Count = 0
    if quality.quality2 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality2 then
                blueSid207Count = blueSid207Count + v:getDecomposeBlueFragNum()
            end
        end
    end

    if quality.quality3 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality3 then
                blueSid207Count = blueSid207Count + v:getDecomposeBlueFragNum()
            end
        end
    end

    local temp = {}
    if blueSid206Count > 0 then
        temp[#temp + 1] = {sid = 206,num = blueSid206Count}
    end

    if blueSid207Count > 0 then
        temp[#temp + 1] = {sid = 207,num = blueSid207Count}
    end

    return temp

end
--- 移除对应品质的所有没使用的配件
--- 传入参数为一个表：{quality0 = 0,quality1 = 1,quality2 = 2,quality3 = 3},不存在的传nil
function T:batchDecomposeAccessoryByQuality(quality)
    if quality.quality0 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality0 then
                self.accessoryInfos[k] = nil
            end
        end
    end

    if quality.quality1 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality1 then
                self.accessoryInfos[k] = nil
            end
        end
    end

    if quality.quality2 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality2 then
                self.accessoryInfos[k] = nil
            end
        end
    end

    if quality.quality3 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality3 then
                self.accessoryInfos[k] = nil
            end
        end
    end
    

    self:luaDispatchEvent(self.EVT_DECOMPOSE_ACCESSORY_REFRESH)

    PlayerInstance:setPlayerFightLogic()

end

--- 得到分解配件所有的金币数目总和
--- 传入参数为一个表：{quality0 = 0,quality1 = 1,quality2 = 2,quality3 = 3},不存在的传nil
function T:getBatchDecomposeAccessoryCoinsByQuality(quality)
    local coin = 0
    if quality.quality0 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality0 then
                coin = coin + v:getDecomposeBackCoin()
            end
        end
    end
    
    if quality.quality1 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality1 then
                coin = coin + v:getDecomposeBackCoin()
            end
        end
    end

    if quality.quality2 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality2 then
                coin = coin + v:getDecomposeBackCoin()
            end
        end
    end

    if quality.quality3 then
        for k,v in pairs(self.accessoryInfos) do
            if v and v.airType == nil and v.quality == quality.quality3 then
                coin = coin + v:getDecomposeBackCoin()
            end
        end
    end

    return coin
end

--- 获取配件的显示规则:先要把那个正在使用的配件排除掉,sid从大到小,如果sid相同呢，？？？？？
function T:getShowAccessoryInfos()
    local accessoryInfos = self.accessoryInfos
    
    local temp = {}
    for k,v in pairs(accessoryInfos) do
        if v and v.airType == nil then
            temp[#temp + 1] = v
        end
    end
    
    if #temp > 0 then
        table.sort(temp,function(a,b)
		    return a.sid > b.sid
	    end)

        return temp
    else
        return nil -- 表示没有配件
    end

end


return T