--[[
    配件信息
]]
local T = Class.AccessoryInfo(LuaEventDispatcher2)

----- 事件-升级成功
--T.EVT_LEVEL_UP_SUCCESS_REFRESH = "evt_level_up_success_refresh"

----- 事件-改造成功
--T.EVT_REFORM_SUCCESS_REFRESH = "evt_reform_success_refresh"

--- 事件-是否正在被使用改变
--T.EVT_IS_USING_REFRESH = "evt_is_using_refresh"


function T:def()
    LuaEventDispatcher2.def(self)

    self.sid = nil -- 这个是道具的sid
    self.id = nil
    self.level = nil -- 等级从1开始的
    self.reLevel = nil -- 改造等级从0开始

    self.slotType = nil  -- 槽位类型

    self.airType = nil -- 所属于的飞机等级类型,也是判断是否正在被使用的标志
    self.quality = nil -- 品质
end

function T:decode(data)
    self.sid = data:readShort()
    self.id = data:readInt()
    self.level = data:readByte()
    self.reLevel = data:readByte()

    -- 根据sid可以得到只能安装到的槽位类型
    self.slotType = Accessory_temp[self.sid].type
    self.quality = Accessory_temp[self.sid].quality

end

--- 判断是否可以解锁
function T:judgeIsLock()
    local slotLevelLock = AccessorySystemManager.slotLevelLock
    local slotLevel = slotLevelLock[self.slotType]
    local playerLevel = UserInfo.level
    if playerLevel < slotLevel then     -- 未解锁
        return false
    else
        return true  -- 解锁了
    end

end

--- 设置飞机所属于的类型
function T:setAirType(airType)
    self.airType = airType
    --self:luaDispatchEvent(self.EVT_IS_USING_REFRESH)
end

-- 取消飞机所属于的类型
function T:cancelAirType()
    self.airType = nil
    --self:luaDispatchEvent(self.EVT_IS_USING_REFRESH)
end


--- 返回这个配件分解所得到的对应蓝图的碎片num,0级的时候算1
function T:getDecomposeBlueFragNum()
    local reLevel = self.reLevel
    local reFormBlueCount = AccessorySystemManager:getAccessoryBox().reFormBlueCount

    local reCount = 1 -- 基础是1
    if reLevel > 0 then
        for i = 1,reLevel,1 do
            reCount = reCount + reFormBlueCount[i]
        end

    end
    

    if self.quality == 0 then
        if reCount then
            return 1 * reCount
        else
            return 1
        end
    elseif self.quality == 1 then
        if reCount then
            return 3 * reCount
        else
            return 3
        end
    elseif self.quality == 2 then
        if reCount then
            return 1 * reCount
        else
            return 1
        end
    else
        if reCount then
            return 5 * reCount
        else
            return 5
        end
    end

end

--- 返回这个配件分解所得到的对应蓝图的碎片sid,num
function T:getDecomposeBlueFragSidAndNum()
    if self.quality == 0 then
        return {sid = 206,num = self:getDecomposeBlueFragNum()}
    elseif self.quality == 1 then
        return {sid = 206,num = self:getDecomposeBlueFragNum()}
    elseif self.quality == 2 then
        return {sid = 207,num = self:getDecomposeBlueFragNum()}
    else
        return {sid = 207,num = self:getDecomposeBlueFragNum()}
    end

end

--- 升级条件（等级从1开始）
function T:levelUpCondition()
    local playerLevel = UserInfo.level
    local level = self.level
    if level < playerLevel then
        return true
    else
        return false
    end

end

--- 升级耗费的金币数
function T:levelUpCost()
    local qualityBase = {5,15,25,40}
    local a = qualityBase[self.quality + 1]

    local b = 0
    local level = self.level
    for i = 1,level - 1,1 do
        b = a/10
        a = a + b
        a = a * 10
        a = math.floor(a + 0.5) -- 四舍五入
        a = a/10
    end

    local money = a * 1000

    return money

end


--- 升级耗费的209道具数目
function T:levelUpCostAccessoryPropNums()
    local num = 1
    local costMoney = self:levelUpCost()

    num = math.floor(costMoney/100000)
    if num >= 0 and num < 1 then
        num = 1
    end

    return num

end


--- 配件分解回收金币(退还升到该等级假设100%成功消耗的金币)
function T:getDecomposeBackCoin()
    local qualityBase = {5,15,25,40}
    local level = self.level
    local money = nil
    if level > 1 then
        money = 0
        for j = 1,level - 1,1 do

            local b = 0
            local a = qualityBase[self.quality + 1]
            for i = 1,j - 1,1 do -- 算单个金币
                b = a/10
                a = a + b
                a = a * 10
                a = math.floor(a + 0.5) -- 四舍五入
                a = a/10
            end
            money = money + a * 1000
        end


    else
        money = 0
    end

    return money
end


--- 升级成功
function T:levelUpSuccess()
    self.level = self.level + 1
    --self:dispachEvent(self.EVT_LEVEL_UP_SUCCESS_REFRESH)

    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    accessoryBox:luaDispatchEvent(accessoryBox.EVT_LEVEL_UP_SUCCESS_REFRESH)

    PlayerInstance:setPlayerFightLogic()
end

--- 得到强化成功的概率
function T:getSuccessRate()
    local successRate = AccessorySystemManager:getAccessoryBox().successRate
    local level = self.level
    if level < 60 then
        return successRate[level]
    else
        return successRate[59] -- 因为60级的时候没概率了,暂时返回到59的，避免出错
    end
    
end

-- 改造条件(等级从0开始到5)
function T:reformCondition()
    local reLevel = self.reLevel
    if reLevel < 5 then
        return true
    else
        return false
    end

end

--- 改造耗费的对应的蓝图数量
function T:reformCost()
    local reFormBlueCount = AccessorySystemManager:getAccessoryBox().reFormBlueCount
    local reLevel = self.reLevel
    return reFormBlueCount[reLevel + 1]
end

--- 改造成功
function T:reformSuccess()
    self.reLevel = self.reLevel + 1
    --self:dispachEvent(self.EVT_REFORM_SUCCESS_REFRESH)

    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    accessoryBox:luaDispatchEvent(accessoryBox.EVT_REFORM_SUCCESS_REFRESH)

    PlayerInstance:setPlayerFightLogic()

end



--- 得到品质的描述
function T:getQualityDes()
    if self.quality == 0 then
        return Language:get("uc585")
    elseif self.quality == 1 then
        return Language:get("uc586")
    elseif self.quality == 2 then
        return Language:get("uc587")
    else
        return Language:get("uc588")
    end

end

----------------------------------------

--- 得到生命百分比加成
function T:getLifeAdd()
    return self:getLifeAddValue(self.level,self.reLevel)
end

--- 得到血量加成
function T:getLifeAddValue(level,reLevel)
    local add = Accessory_temp[self.sid].life
    if add > 0 then
        local addPropertyByRrformLevel = AccessorySystemManager:getAccessoryBox().addPropertyByRrformLevel

        local level = level
        local reLevel = reLevel

        local addProperty = 0
        if reLevel > 0 then
            addProperty = addPropertyByRrformLevel[self.quality + 1][reLevel]
        end

        local trueAdd = level * add * (1 + addProperty) * 100   -- 因为读的配置是百分比，但是那边算的时候除以了100，所以这里先乘以100

        return trueAdd
    else
        return 0
    end
end


--- 得到下一级的血量加成和当前血量加成的差值
function T:getLevelLifeAddDiff()
    local nowLevelLifeAdd = self:getLifeAdd()
    --if nowLevelLifeAdd == 0 then
   --     return 0
    --else
        if self.level < 60 then
            local trueAdd = self:getLifeAddValue(self.level + 1,self.reLevel)
            return trueAdd - nowLevelLifeAdd
        else
            return 0
        end
    --end
end

--- 得到下一改造血量加成和当前血量加成的差值
function T:getReLevelLifeAddDiff()
    local nowLevelLifeAdd = self:getLifeAdd()
    --if nowLevelLifeAdd == 0 then
     --   return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getLifeAddValue(self.level,self.reLevel + 1)
            return trueAdd - nowLevelLifeAdd
        else
            return 0
        end
    --end
end
----------------------------------------

--- 得到攻击百分比加成
function T:getAttackAdd()
    return self:getAttackAddValue(self.level,self.reLevel)
end

--- 得到攻击加成
function T:getAttackAddValue(level,reLevel)
    local add = Accessory_temp[self.sid].attack
    if add > 0 then
        local addPropertyByRrformLevel = AccessorySystemManager:getAccessoryBox().addPropertyByRrformLevel

        local level = level
        local reLevel = reLevel

        local addProperty = 0
        if reLevel > 0 then
            addProperty = addPropertyByRrformLevel[self.quality + 1][reLevel]
        end

        local trueAdd = level * add * (1 + addProperty) * 100   -- 因为读的配置是百分比，但是那边算的时候除以了100，所以这里先乘以100

        return trueAdd
    else
        return 0
    end
end


--- 得到下一级的攻击加成和当前攻击加成的差值
function T:getLevelAttackAddDiff()
    local nowLevelAttackAdd = self:getAttackAdd()
    --if nowLevelAttackAdd == 0 then
       -- return 0
    --else
        if self.level < 60 then
            local trueAdd = self:getAttackAddValue(self.level + 1,self.reLevel)
            return trueAdd - nowLevelAttackAdd
        else
            return 0
        end
   -- end
end

--- 得到下一改造攻击加成和当前攻击加成的差值
function T:getReLevelAttackAddDiff()
    local nowLevelAttackAdd = self:getAttackAdd()
    --if nowLevelAttackAdd == 0 then
     --   return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getAttackAddValue(self.level,self.reLevel + 1)
            return trueAdd - nowLevelAttackAdd
        else
            return 0
        end
    --end
end
----------------------------------------
--- 得到反克制百分比加成
function T:getAntiRestraintAdd()
    return self:getAntiRestraintAddValue(self.level,self.reLevel)
end

--- 得到反克制加成
function T:getAntiRestraintAddValue(level,reLevel)
    local add = Accessory_temp[self.sid].antiRestraint
    if add > 0 then
        local addPropertyByRrformLevel = AccessorySystemManager:getAccessoryBox().addPropertyByRrformLevel

        local level = level
        local reLevel = reLevel

        local addProperty = 0
        if reLevel > 0 then
            addProperty = addPropertyByRrformLevel[self.quality + 1][reLevel]
        end

        local trueAdd = level * add * (1 + addProperty) * 100   -- 因为读的配置是百分比，但是那边算的时候除以了100，所以这里先乘以100

        return trueAdd
    else
        return 0
    end

end

--- 得到下一级的反克制加成和当前反克制加成的差值
function T:getLevelAntiRestraintAddDiff()
    local nowLevelAntiRestraintAdd = self:getAntiRestraintAdd()
    --if nowLevelAntiRestraintAdd == 0 then
    --    return 0
    --else
        if self.level < 60 then
            local trueAdd = self:getAntiRestraintAddValue(self.level + 1,self.reLevel)
            return trueAdd - nowLevelAntiRestraintAdd
        else
            return 0
        end
    --end
end

--- 得到下一改造反克制加成和当前反克制加成的差值
function T:getReLevelAntiRestraintAddDiff()
    local nowLevelAntiRestraintAdd = self:getAntiRestraintAdd()
    --if nowLevelAntiRestraintAdd == 0 then
    --    return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getAntiRestraintAddValue(self.level,self.reLevel + 1)
            return trueAdd - nowLevelAntiRestraintAdd
        else
            return 0
        end
    --end
end


----------------------------------------
--- 得到克制百分比加成
function T:getRestraintAdd()
    return self:getRestraintAddValue(self.level,self.reLevel)
end

--- 得到克制加成
function T:getRestraintAddValue(level,reLevel)
    local add = Accessory_temp[self.sid].restraint
    if add > 0 then
        local addPropertyByRrformLevel = AccessorySystemManager:getAccessoryBox().addPropertyByRrformLevel

        local level = level
        local reLevel = reLevel

        local addProperty = 0
        if reLevel > 0 then
            addProperty = addPropertyByRrformLevel[self.quality + 1][reLevel]
        end

        local trueAdd = level * add * (1 + addProperty) * 100   -- 因为读的配置是百分比，但是那边算的时候除以了100，所以这里先乘以100

        return trueAdd
    else
        return 0
    end

end

--- 得到下一级的克制加成和当前克制加成的差值
function T:getLevelRestraintAddDiff()
    local nowLevelRestraintAdd = self:getRestraintAdd()
    --if nowLevelRestraintAdd == 0 then
       -- return 0
    --else
        if self.level < 60 then
            local trueAdd = self:getRestraintAddValue(self.level + 1,self.reLevel)
            return trueAdd - nowLevelRestraintAdd
        else
            return 0
        end
    --end
end

--- 得到下一改造克制加成和当前克制加成的差值
function T:getReLevelRestraintAddDiff()
    local nowLevelRestraintAdd = self:getRestraintAdd()
    --if nowLevelRestraintAdd == 0 then
       -- return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getRestraintAddValue(self.level,self.reLevel + 1)
            return trueAdd - nowLevelRestraintAdd
        else
            return 0
        end
    --end
end
----------------------------------------
--- 得到暴免百分比加成
function T:getViolenceFreeAdd()
    return self:getViolenceFreeAddValue(self.level,self.reLevel)
end

--- 得到暴免加成
function T:getViolenceFreeAddValue(level,reLevel)
    local add = Accessory_temp[self.sid].violenceFree
    if add > 0 then
        local addPropertyByRrformLevel = AccessorySystemManager:getAccessoryBox().addPropertyByRrformLevel

        local level = level
        local reLevel = reLevel

        local addProperty = 0
        if reLevel > 0 then
            addProperty = addPropertyByRrformLevel[self.quality + 1][reLevel]
        end

        local trueAdd = level * add * (1 + addProperty) * 100   -- 因为读的配置是百分比，但是那边算的时候除以了100，所以这里先乘以100

        return trueAdd
    else
        return 0
    end

end

--- 得到下一级的暴免加成和当前暴免加成的差值
function T:getLevelViolenceFreeAddDiff()
    local nowLevelViolenceFreeAdd = self:getViolenceFreeAdd()
    --if nowLevelViolenceFreeAdd == 0 then
        --return 0
    --else
        if self.level < 60 then
            local trueAdd = self:getViolenceFreeAddValue(self.level + 1,self.reLevel)
            return trueAdd - nowLevelViolenceFreeAdd
        else
            return 0
        end
    --end
end

--- 得到下一改造暴免加成和当前暴免加成的差值
function T:getReLevelViolenceFreeAddDiff()
    local nowLevelViolenceFreeAdd = self:getViolenceFreeAdd()
    --if nowLevelViolenceFreeAdd == 0 then
        --return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getViolenceFreeAddValue(self.level,self.reLevel + 1)
            return trueAdd - nowLevelViolenceFreeAdd
        else
            return 0
        end
    --end
end
----------------------------------------
--- 得到暴伤百分比加成
function T:getViolenceInjuryAdd()
    return self:getViolenceInjuryAddValue(self.level,self.reLevel)
end

--- 得到暴伤加成
function T:getViolenceInjuryAddValue(level,reLevel)
    local add = Accessory_temp[self.sid].violenceInjury
    if add > 0 then
        local addPropertyByRrformLevel = AccessorySystemManager:getAccessoryBox().addPropertyByRrformLevel

        local level = level
        local reLevel = reLevel

        local addProperty = 0
        if reLevel > 0 then
            addProperty = addPropertyByRrformLevel[self.quality + 1][reLevel]
        end

        local trueAdd = level * add * (1 + addProperty) * 100   -- 因为读的配置是百分比，但是那边算的时候除以了100，所以这里先乘以100

        return trueAdd
    else
        return 0
    end

end

--- 得到下一级的暴伤加成和当前暴伤加成的差值
function T:getLevelViolenceInjuryAddDiff()
    local nowLevelViolenceInjuryAdd = self:getViolenceInjuryAdd()
    --if nowLevelViolenceInjuryAdd == 0 then
        --return 0
    --else
        if self.level < 60 then
            local trueAdd = self:getViolenceInjuryAddValue(self.level + 1,self.reLevel)
            return trueAdd - nowLevelViolenceInjuryAdd
        else
            return 0
        end
    --end
end

--- 得到下一改造暴伤加成和当前暴伤加成的差值
function T:getReLevelViolenceInjuryAddDiff()
    local nowLevelViolenceInjuryAdd = self:getViolenceInjuryAdd()
    --if nowLevelViolenceInjuryAdd == 0 then
        --return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getViolenceInjuryAddValue(self.level,self.reLevel + 1)
            return trueAdd - nowLevelViolenceInjuryAdd
        else
            return 0
        end
    --end
end

----------------------------------------  紫色防御类配件增加闪避,攻击类配件增加命中;橙色防御类配件增加闪避,,攻击类配件增加命中(这2种)
-- 得到命中百分比加成
function T:getHitAdd()
    return self:getHitAddValue(self.reLevel)
end


-- 得到命中加成
function T:getHitAddValue(reLevel)
    local data = self:judgeHitOrDodge()
    if data then
        if data.index == 3 or data.index == 4 then
            local addHitOrDodgeRelevel = AccessorySystemManager:getAccessoryBox().addHitOrDodgeRelevel
            local dataTable = addHitOrDodgeRelevel[data.index]

            return dataTable[reLevel + 1]

        else
            return 0
        end

    else
        return 0
    end

end

--- 得到下一改造命中加成和当前命中加成的差值
function T:getReLevelHitAddDiff()
    local nowLevelHitAdd = self:getHitAdd()
    --if nowLevelHitAdd == 0 then
        --return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getHitAddValue(self.reLevel + 1)
            return trueAdd - nowLevelHitAdd
        else
            return 0
        end
    --end
end


----------------------------------------
-- 得到闪避百分比加成
function T:getDodgeAdd()
    return self:getDodgeAddValue(self.reLevel)
end


-- 得到闪避加成
function T:getDodgeAddValue(reLevel)
    local data = self:judgeHitOrDodge()
    if data then
        if data.index == 1 or data.index == 2 then
            local addHitOrDodgeRelevel = AccessorySystemManager:getAccessoryBox().addHitOrDodgeRelevel
            local dataTable = addHitOrDodgeRelevel[data.index]

            return dataTable[reLevel + 1]

        else
            return 0
        end

    else
        return 0
    end
end

--- 得到下一改造闪避加成和当前闪避加成的差值
function T:getReLevelDodgeAddDiff()
    local nowLevelDodgeAdd = self:getDodgeAdd()
    --if nowLevelDodgeAdd == 0 then
        --return 0
    --else
        if self.reLevel < 5 then
            local trueAdd = self:getDodgeAddValue(self.reLevel + 1)
            return trueAdd - nowLevelDodgeAdd
        else
            return 0
        end
    --end
end

--- 判断是命中还是闪避加成
function T:judgeHitOrDodge()
    local hitOrDodgeSids = AccessorySystemManager:getAccessoryBox().hitOrDodgeSids

    local index = nil

    for i = 1,4,1 do
        for k = 1,3,1 do
            if hitOrDodgeSids[i][k] == self.sid then
                index = i
                break
            end

        end
    end
    
    if index then
        local uc1,uc2
        if index == 1 then
            uc1 = "uc791"
            uc2 = "uc795"
        elseif index == 2 then
            uc1 = "uc792"
            uc2 = "uc796"
        elseif index == 3 then
            uc1 = "uc793"
            uc2 = "uc797"
        else
            uc1 = "uc794"
            uc2 = "uc798"
        end

        return {index = index,uc1 = uc1,uc2 = uc2}

    else
        return nil
    end

end



return T