--[[
    配件系统管理
]]
local T = Class.AccessorySystemManager()


--- 飞机类型的sid组合（4种，所影响对应的飞机sid组合）
T.sids = {{1,5,9,13,17},{2,6,10,14,18},{3,7,11,15,19},{4,8,12,16,20}}

--- 飞机槽位限制等级,索引是飞机的槽位类型
T.slotLevelLock = {10,15,20,25,30,35}

--- 配件道具消耗率
T.LEVEL_UP_SUC_RATE = {100,100,100,100,90,90,90,
		90,90,80,80,80,80,80,70,70,70,70,70,60,60,60,60,60,50,50,50,50,50,
		40,40,40,40,40,30,30,30,30,30,20,20,20,20,20,15,15,15,15,15,10,10,
		10,10,10,5,5,5,5,5}



function T:def()
    -- 配件盒子
    self.accessoryBox = require("src.app.model.accessory.AccessoryBox")()

    -- 槽位盒子
    self.slotBox = require("src.app.model.accessory.SlotBox")()


end

--- 得到配件盒子
function T:getAccessoryBox()
    return self.accessoryBox
end

--- 得到槽位盒子
function T:getSlotBox()
    return self.slotBox
end


--- 判断是否有配件安装，或者蓝图可以制造并且安装
function T:judgeIsHasAccessoryOrBlueInstall()
    local accessoryBox = self:getAccessoryBox()
    local accessoryInfos = accessoryBox.accessoryInfos
    local slotBox = AccessorySystemManager:getSlotBox()

    local playerLevel = UserInfo.level
    
    local judge = false

    for d = 1,4,1 do -- d就是飞机类型了   
        for i = 1,6,1 do -- i就是槽位类型了
            local slotLevelLock = AccessorySystemManager.slotLevelLock
            local lockLevel = slotLevelLock[i]
            if playerLevel < lockLevel then     -- 未解锁
            else 
                -- 还要判断这个槽位是否安装了配件
                local slotInfo = slotBox.slotInfos[d][i]
                local id = slotInfo.id


                if id == 0 then -- 没安装配件
                    if accessoryBox:getStrongestAccessory(i) == nil then
                        -- 判断是否有蓝图可以制造
                        local temp = {{1,11,21,31},{2,12,22,32},{3,13,23,33},{4,14,24,34},{5,15,25,35},{6,16,26,36}}

                        local accessorySids = temp[i]
                        local blueSids = {}
                        for n = 1,4,1 do
                            blueSids[n] = Accessory_temp[accessorySids[n]].bluePrintSid
                        end

                
                        local temp = {{},{},{},{}}
                        for j = 1,4,1 do
                            local tables = PropsInfoMgr:getAllBluePrintSidsBySid(blueSids[j])
                            if tables then
                                temp[j] = tables
                            end
                        end

                        local list = {}
                        for p = 1,4,1 do
                            if #temp[p] > 0 then
                                for k,v in pairs(temp[p]) do
                                    if v then
                                        list[#list + 1] = v  -- sid

                                    end
                                end
                            end

                        end

                        if #list > 0 then
                            judge = true
                            return true
                        else             
                    
                        end


                    else -- 有配件可以安装
                        judge = true
                        return true

                    end




                end

            end

        end

    end

    return judge



end



--- 得到某一种飞机类型的所有的生命加成
function T:getAllLifeAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getLifeAdd()
        end

    end

    return add ~= 0 and add / 100 or 0 -- 这个返回的是一个百分比:比如50，不是0.5

end

--- 得到某一种飞机类型的所有的攻击加成
function T:getAllAttackAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getAttackAdd()
        end

    end

    return add ~= 0 and add / 100 or 0 -- 这个返回的是一个百分比:比如0.5，不是50
end

--- 得到某一种飞机类型的所有的反克制加成
function T:getAllAntiRestraintAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getAntiRestraintAdd()
        end

    end

    return add ~= 0 and add / 100 or 0  -- 这个返回的是一个百分比:比如50，不是0.5
end

--- 得到某一种飞机类型的所有的克制加成
function T:getAllRestraintByAirAddType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getRestraintAdd()
        end

    end

    return add ~= 0 and add / 100 or 0  -- 这个返回的是一个百分比:比如50，不是0.5
end

--- 得到某一种飞机类型的所有的暴免加成
function T:getAllViolenceFreeAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getViolenceFreeAdd()
        end

    end

    return add ~= 0 and add / 100 or 0  -- 这个返回的是一个百分比:比如50，不是0.5
end

--- 得到某一种飞机类型的所有的暴伤加成
function T:getAllViolenceInjuryAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getViolenceInjuryAdd()
        end

    end

    return add ~= 0 and add / 100 or 0  -- 这个返回的是一个百分比数值，:比如0.5，不是50
end


--- 得到某一种飞机类型的所有的命中加成
function T:getAllHitAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getHitAdd()
        end

    end

    return add or 0  -- 这个返回的是一个百分比数值，:比如0.5，不是50
end

--- 得到某一种飞机类型的所有的闪避加成
function T:getAllDodgeAddByAirType(airType)
    local slotBox = self:getSlotBox()
    local accessoryBox = self:getAccessoryBox()

    local slotInfos = slotBox.slotInfos
    local accessoryInfos = accessoryBox.accessoryInfos

    local add = 0

    local infos = slotInfos[airType]
    for i = 1,6,1 do
        if infos[i].id > 0 then
            add = add + accessoryInfos[infos[i].id]:getDodgeAdd()
        end

    end

    return add or 0 -- 这个返回的是一个百分比数值，:比如0.5，不是50
end



--- 得到某一个飞机是哪种类型
function T:getAirTypeBySid(sid)
    local sids = self.sids

    local airType = nil
    for i = 1,4,1 do
        if airType then
            break
        end

        local airSids = sids[i]
        local length = #airSids
        for j = 1,length,1 do
            if airSids[j] == sid then
                airType = i
                break
            end
        end
    end

    return airType

end


--- 得到某一种飞机的生命加成
function T:getLifeAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllLifeAddByAirType(airType)
    else
        return 0
    end

end

--- 得到某一种飞机的攻击加成
function T:getAttackAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllAttackAddByAirType(airType)
    else
        return 0
    end
end

--- 得到某一种飞机的反克制加成
function T:getAntiRestraintAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllAntiRestraintAddByAirType(airType)
    else
        return 0
    end
end

--- 得到某一种飞机的克制加成
function T:getRestraintAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllRestraintByAirAddType(airType)
    else
        return 0
    end
end

--- 得到某一种飞机的暴免加成
function T:getViolenceFreeAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllViolenceFreeAddByAirType(airType)
    else
        return 0
    end
end

--- 得到某一种飞机的暴伤加成
function T:getViolenceInjuryAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllViolenceInjuryAddByAirType(airType)
    else
        return 0
    end
end

--- 得到某一种飞机的命中加成
function T:getHitAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllHitAddByAirType(airType)
    else
        return 0
    end
end

--- 得到某一种飞机的闪避加成
function T:getDodgeAddBySid(sid)
    local airType = self:getAirTypeBySid(sid)
    if airType then
        return self:getAllDodgeAddByAirType(airType)
    else
        return 0
    end
end

return T