--[[
    槽位盒子
]]
local T = Class.SlotBox(LuaEventDispatcher2)

-- 槽位信息
T.SlotInfo = require("src.app.model.accessory.SlotInfo")

--- 事件-安装配件
T.EVT_INSTALL_ACCESSORY_REFRESH = "evt_install_accessory_refresh"

--- 事件-卸载配件
T.EVT_UNLOAD_ACCESSORY_REFRESH = "evt_unload_accessory_refresh"


function T:def()
    LuaEventDispatcher2.def(self)


    -- 槽位信息
    self.slotInfos = {}
end


function T:decode(data)
	local infos = {}
	for i = 1,4,1 do
        infos[i] = {}
        for j = 1,6,1 do
            local info = self.SlotInfo()
            info.airType = i
            info.slotType = j
		    info:decode(data)

            infos[i][j] = info
        end

	end

	self.slotInfos = infos
end

-- 判断某一个飞机等级的槽位是否解锁
function T:judgeSlotIsLock(airType,slotType)
    local slotInfos = self.slotInfos
    local needLevel = slotInfos[airType][slotType]:getLockLevel()
    local playerLevel = UserInfo.level

    if needLevel <= playerLevel then
        return true -- 解锁了
    else
        return false -- 未解锁
    end

end

--- 给所有的如果id不等于0的槽位设置飞机类型
function T:setAccessoryAirType()
    local slotInfos = self.slotInfos
	for i = 1,4,1 do
        for j = 1,6,1 do
            local info = slotInfos[i][j]
            if info.id > 0 then
                info:setAccessoryAirType()
            end
        end

	end
end

--- 安装配件
function T:installAccessory(id,airType,slotType)
    local slotInfos = self.slotInfos
    local info = slotInfos[airType][slotType]
    info:installAccessory(id)

    PlayerInstance:setPlayerFightLogic()
end

--- 卸载配件
function T:unloadAccessory(airType,slotType)
    local slotInfos = self.slotInfos
    local info = slotInfos[airType][slotType]
    info:unloadAccessory()

    PlayerInstance:setPlayerFightLogic()

end

--- 判断这个飞机类型的某个槽位是否安装了配件,如果没安装，则返回安装的飞机类型
function T:judgeInstallId(airType,slotType)
    local slotInfos = self.slotInfos
    
    if slotInfos[airType][slotType].id > 0 then
        return nil
    else
        return airType -- 未安装
    end

end

--- 判断所有的飞机类型的某个槽位是否安装了配件，只要有一个没安装就返回那个飞机类型
function T:judgeAllAirTypeInstall(slotType)
    local airType = nil

    for i = 1,4,1 do
        if self:judgeInstallId(i,slotType) then
            airType = i
            break
        end

    end

    return airType

end


return T