--[[
    槽位信息
]]
local T = Class.SlotInfo(LuaEventDispatcher2)

function T:def()
    LuaEventDispatcher2.def(self)

    -- 飞机类型
    self.airType = nil
    -- 槽位类型
    self.slotType = nil
    -- 槽位的配件id,0就是没的槽位id
    self.id = nil

    -- 应该安装的配件的sid类型
    self.accessorySids = {}
    -- 应该安装的蓝图的sid类型
    self.blueSids = {}

end

function T:decode(data)
    self.id = data:readInt()

    self:getAccessoryOrBlueSidType()

end

function T:getAccessoryOrBlueSidType()
    local temp = {{1,11,21,31},{2,12,22,32},{3,13,23,33},{4,14,24,34},{5,15,25,35},{6,16,26,36}}
    self.accessorySids = temp[self.slotType]
    for i = 1,4,1 do
        self.blueSids[i] = Accessory_temp[self.accessorySids[i]].bluePrintSid
    end
    
end

--- 得到解锁的需要的玩家的等级
function T:getLockLevel()
    local slotLevelLock = AccessorySystemManager.slotLevelLock
    return slotLevelLock[self.slotType]
end

--- 给这个配件id设置属于某个飞机类型，其实就是标记这个配件是否正在被使用
function T:setAccessoryAirType()
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    local accessoryInfos = accessoryBox.accessoryInfos
    accessoryInfos[self.id]:setAirType(self.airType)

    local slotBox = AccessorySystemManager:getSlotBox()
    slotBox:luaDispatchEvent(slotBox.EVT_INSTALL_ACCESSORY_REFRESH)
end


-- 安装配件
function T:installAccessory(id)
    -- 有可能是原来的已经有了，只是替换，这个要考虑
    if self.id > 0 then -- 先卸载
        local accessoryBox = AccessorySystemManager:getAccessoryBox()
        local accessoryInfos = accessoryBox.accessoryInfos
        accessoryInfos[self.id]:cancelAirType()
    end
    self.id = id
    self:setAccessoryAirType()
end

-- 卸载配件
function T:unloadAccessory()
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    local accessoryInfos = accessoryBox.accessoryInfos
    accessoryInfos[self.id]:cancelAirType()
    self.id = 0 -- 默认是0

    local slotBox = AccessorySystemManager:getSlotBox()
    slotBox:luaDispatchEvent(slotBox.EVT_UNLOAD_ACCESSORY_REFRESH)

end




return T