local T={}

--- 每个战机的的信息
T.Aircraft = require "src.app.model.aircraft.AircraftInfo"

--- 所有的战机列表
T.infos = nil

--- 维修部队
T.repairAircrafts = nil

--- 防守部队
T.defenceAircrafts = nil


--- 反序列化所有战机(只有登录的时候调用)
function T:decode(data)
	local len = data:readUnsignedByte()

	local infos = {}
	for i=1,len,1 do
		local info=mapping({}, T.Aircraft)
		info.sid=i
		info:decode(data)
		infos[i] = info  -- i就对应sid
	end
	self.infos = infos
end

--- 更新所有战机
function T:refreshAllAircraft(data)
    self:decode(data)
    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH)
    PlayerInstance:setPlayerFightLogic()
end

--- 维修部队（登录的时候）
function T:decodeRepair(data)
    local len = data:readUnsignedByte()
    local infos = {}
	for i = 1,len,1 do
		local info = mapping({}, T.Aircraft)
        info.sid = data:readUnsignedShort()
		info.num = data:readUnsignedInt()
		infos[info.sid] = info  
	end
	self.repairAircrafts = infos
end

--- 刷新所有维修部队(这个主要是端口刷的)：只适用于派出的做任务的战机，不用算所有的战机数量
function T:refreshRepair(data)
    self:decodeRepair(data)
    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_REPAIR_ARMY)
    PlayerInstance:setPlayerFightLogic()
end


--- 设置所有维修部队(这个是通讯请求返回的数据),请求修理所有时候,后台是刷了所有战机的
function T:setRepair(data)
    self:decodeRepair(data)
    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_REPAIR_ARMY)
    PlayerInstance:setPlayerFightLogic()
end

--- 设置某一个维修飞机的数量
function T:setRepairForSid(sid,num)
    local repairAircrafts = self.repairAircrafts
    if repairAircrafts[sid] then
        repairAircrafts[sid].num = num
    end
    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_REPAIR_ARMY)
    PlayerInstance:setPlayerFightLogic()
end

--- 防守部队（登录的时候）
function T:decodeDefend(data)
    local aircraft = {}
    for i = 1,6,1 do  -- 1到6是位置
        local modelAircraft = {}
        modelAircraft.sid = data:readUnsignedShort()
        if modelAircraft.sid <= 0 then
            modelAircraft.count = 0
        else
            modelAircraft.count = data:readUnsignedShort()
        end
        aircraft[i] = modelAircraft
    end
    self.defenceAircrafts = aircraft
end


--- 设置防守部队
function T:setDefend(aircrafts)
    --- 原来的防守战机先给回去,因为那里要算最大战力
    local defenceAircrafts = self.defenceAircrafts
    local infos = self.infos
    for i = 1,6,1 do
        if defenceAircrafts[i] and defenceAircrafts[i].sid > 0 then
            local num1 = infos[defenceAircrafts[i].sid].num
            local num2 = defenceAircrafts[i].count
            infos[defenceAircrafts[i].sid]:addNum(defenceAircrafts[i].count) -- 里面有事件
        end
    end

    local aircraftList = {}
    for i = 1,6,1 do
        local modelAircraft = {}
    	local item = aircrafts[i]
    	if item then
    		modelAircraft.sid = item.sid
            modelAircraft.count = item.count
            -- 减去数量
            infos[item.sid]:minNum(item.count) -- 里面有事件
    	else
    		modelAircraft.sid = 0
            modelAircraft.count = 0
    	end

        aircraftList[i] = modelAircraft
    end

    self.defenceAircrafts = aircraftList


    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_DEFENCE_ARMY)
    PlayerInstance:setPlayerFightLogic()
end

--- 获得防守部队
function T:getDefend()
    return self.defenceAircrafts
end

--- 得到修理部队
function T:getRepair()
    return self.repairAircrafts
end

--- 得到修理部队战机数量
function T:getRepairNum()
    local repairAircrafts = self.repairAircrafts
    local num = 0 
    for k, v in pairs(repairAircrafts) do
        if v then
            if v and v.num > 0 then
                num = num + v.num
            end
        end
    end

    return num

end

--- 得到飞机空闲的部队和防守部队之和,要考虑sid相同的地方
function T:getAllAircrafts()
    local infos = self.infos
    local defenceAircrafts = self:getDefend()

    local team = mapping({},infos)

    for i = 1,6,1 do
        for k,v in pairs(team) do
            if defenceAircrafts[i] and defenceAircrafts[i].sid == v.sid then
                team[k].num = team[k].num + defenceAircrafts[i].count
            end
        end

    end

    return team

end


return T
