local T=Class.AllianceManager(LuaEventDispatcher2)

--- 在联盟中的职位发生改变
T.EVT_TYPE_POST_CHANGE="evt_type_post_change"

--- 创建 联盟
T.EVT_TYPE_CREATED_ALLIANCE="evt_type_created_alliance"

--- 已加入联盟
T.EVT_TYPE_JOINED_ALLIANCE="evt_type_joined_alliance"

--- 申请列表发生改变
T.EVT_TYPE_APPLY_LIST_CHANGE="evt_type_apply_list_change"

--- 被踢出联盟
T.EVT_TYPE_BY_KICK_OUT="evt_type_by_kick_out"

--- 退出联盟
T.EVT_TYPE_EXIT_ALLIANCE="evt_type_exit_alliance"

--- 今日贡献点改变
T.EVT_TYPE_TODAY_CONTRIBUTION_POINT_CHANGE="evt_type_today_contribution_point_change"

--- 联盟捐献公告被改变
T.EVT_TYPE_DONATE_NOTICE="evt_type_donate_notice"

--- 事件-贡献点改变
T.EVT_TYPE_CHANGE_CONTRIBUTION_POINT="evt_type_change_contribution_point"

--- 事件-总贡献点改变
T.EVT_TYPE_CHANGE_TOTAL_CONTRIBUTION_POINT="evt_type_change_total_contribution_point"

--- 事件-科技信息改变
T.EVT_TYPE_CHANGE_TECH_INFO="evt_type_change_tech_info"

--- 事件-联盟等级改变
T.EVT_TYPE_CHANGE_LEVEL="evt_type_change_level"

--- 事件-自己联盟的成员发生改变
T.EVT_TYPE_CHANGE_MEMBER_LIST="evt_type_change_member_list"

--- 命中
T.TYPE_HIT=4

--- 闪避
T.TYPE_DODGE=5

--- 暴击
T.TYPE_CRIT=6

--- 暴击抵抗
T.TYPE_CRIT_RESIST=7

--- 联盟信息
T.MyAllianceInfo=require("src.app.model.alliance.MyAllianceInfo")

function T:def()
	LuaEventDispatcher2.def(self)
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 创建联盟
function T:createAlliance(id,name)
	local mA=UserInfo.allianceBox.myAlliance
	mA.id=id
	mA.level=1
	mA.name=name

	self:luaDispatchEvent(self.EVT_TYPE_CREATED_ALLIANCE)
end

--- 目标玩家在联盟中的职位发生改变
-- @param newPost 新的职位
-- @param newLeaderName 新的会长名
function T:changePost(userId,newPost,newLeaderName)
	local aB=UserInfo.allianceBox
	local mA=aB.myAlliance
	local rB=aB.allianceRankBox

	local nLeadName=newLeaderName
	if userId == UserInfo.id then
		if newPost == AlliancePost.LEADER then
			MUIAlter_singleLine.show({
				info=Language:get("ug692"),
				showSecond=1.5
			})

			nLeadName=UserInfo.name
		elseif newPost == AlliancePost.VICE_CHAIRMAN and mA.post ~= AlliancePost.LEADER then
			MUIAlter_singleLine.show({
				info=Language:get("ug693"),
				showSecond=1.5
			})
		elseif mA.post ~= AlliancePost.LEADER then
			MUIAlter_singleLine.show({
				info=Language:get("ug694"),
				showSecond=1.5
			})
		end

		mA.post=newPost
	else
		local mL=mA.memberList
		if newPost == AlliancePost.LEADER and mL then
			for _,v in pairs(mL) do
				if v.id == userId then
					nLeadName=v.name
					break
				end
			end
		end
	end

	--清除成员列表数据,以便重新获取
	mA.memberList=nil

	--更新联盟排行榜信息
	if nLeadName then
		mA.leaderName=nLeadName

		if rB and rB.rList then
			for _,v in pairs(rB.rList) do
				if v.id == mA.id then
					v.leaderName=nLeadName
					break
				end
			end
		end
	end

	self:luaDispatchEvent(self.EVT_TYPE_POST_CHANGE)
end

--- 解析自己联盟的成功列表
function T:decodeMyAlliaceMemberList(data)
	UserInfo.allianceBox.myAlliance:decodeMemberList(data)
	
	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_MEMBER_LIST)
end

--- 移除自己联盟的成员
function T:rmvMemberFromMyAlliance(id)
	UserInfo.allianceBox.myAlliance:removeMember(id)
	
	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_MEMBER_LIST)
end

--- 已加入联盟
function T:joined()
	--- 已经加入联盟,就没有申请列表了
	UserInfo.allianceBox.myAlliance.applyList=nil
	UserInfo.allianceBox.appliedAllianceIdList=nil

	self:luaDispatchEvent(self.EVT_TYPE_JOINED_ALLIANCE)
end

--- 申请列表发生改变(如果当前在申请列表界面,需要重新获取申请列表数据)
function T:ApplyListChange()
	self:luaDispatchEvent(self.EVT_TYPE_APPLY_LIST_CHANGE)
end

--- 被踢出联盟
function T:byKickOut()
	local aBox=UserInfo.allianceBox
	if aBox.myAlliance.post == AlliancePost.LEADER then
		self:exitAlliance()
	else
		MUIAlter_singleLine.show({
			info=Language:get("ug689",UserInfo.allianceBox.myAlliance.name),
			showSecond=1.5
		})

		aBox.myAlliance=self.MyAllianceInfo()

		self:luaDispatchEvent(self.EVT_TYPE_BY_KICK_OUT)
	end
end

--- 退出联盟
function T:exitAlliance()
	local aBox=UserInfo.allianceBox
	aBox.myAlliance=self.MyAllianceInfo()

	self:luaDispatchEvent(self.EVT_TYPE_EXIT_ALLIANCE)
end

--- 增加今日贡献点
function T:addTodayContributionPoint(value)
	local mA=UserInfo.allianceBox.myAlliance
	mA.todayContributionPoint=mA.todayContributionPoint + value

	self:luaDispatchEvent(self.EVT_TYPE_TODAY_CONTRIBUTION_POINT_CHANGE,mA)
end

--- 设置联盟捐献公告
function T:setDonateNotice(value)
	local mA=UserInfo.allianceBox.myAlliance
	if value ~= mA.donateNotice then
		mA.donateNotice=value

		self:luaDispatchEvent(self.EVT_TYPE_DONATE_NOTICE,mA)
	end
end

--- 增加贡献点
function T:addContributionPoint(value)
	local mA=UserInfo.allianceBox.myAlliance
	mA.allianceContribution=mA.allianceContribution + value

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_CONTRIBUTION_POINT)
end
--- 获取贡献点
function T:getContributionPoint()
	return UserInfo.allianceBox.myAlliance.allianceContribution
end

--- 增加总贡献点
function T:addTotalContributionPoint(value)
	local mA=UserInfo.allianceBox.myAlliance
	mA.allianceTotalContribution=mA.allianceTotalContribution + value

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_TOTAL_CONTRIBUTION_POINT)
end
--- 获取总贡献点
function T:getTotalContributionPoint()
	return UserInfo.allianceBox.myAlliance.allianceTotalContribution
end

--- 解析联盟商店商品列表
function T:decodeAllianceShopGoodsList(byteArray)
	UserInfo.allianceBox:decodeShopGoodsList(byteArray)

	--目前无派发事件需要
end
--- 清除聪明商店商品信息列表
function T:clearAllianceShopGoodsList()
	UserInfo.allianceBox.shopGoodesList=nil
end
--- 获取联盟商店商品信息列表
function T:getAllianceShopGoodsList()
	return UserInfo.allianceBox.shopGoodesList
end

--- 获取联盟等级
function T:getAllianceLv()
	return UserInfo.allianceBox.myAlliance.level
end

--- 获取联盟名称
function T:getAllianceName()
	return UserInfo.allianceBox.myAlliance.name
end

--- 更新联盟科技信息
function T:updateScienceInfo(sid,level,techPoint)
	local sL=UserInfo.allianceBox.myAlliance.scienceList

	if not sL then
		sL={}
		UserInfo.allianceBox.myAlliance.scienceList=sL
	end

	local tech=sL[sid]
	if not tech then
		tech={id=sid,level=0,sciencePoint=0}
		sL[sid]=tech
	end

	if tech.level ~= level or tech.sciencePoint ~= techPoint then
		tech.level=level
		if techPoint then
			tech.sciencePoint=techPoint
		end
		self:luaDispatchEvent(self.EVT_TYPE_CHANGE_TECH_INFO,sid)

		local mA=UserInfo.allianceBox.myAlliance
		if sid == 1 and mA.level ~= level then
			mA.level=level
			self:luaDispatchEvent(self.EVT_TYPE_CHANGE_LEVEL)
		end
	end

	PlayerInstance:setPlayerFightLogic()

end

--- 获取对战机攻击的加成
-- @return 加成比例
function T:getAttckAdd()
	return 0
end


--- 获取对战机血量的加成
-- @return 加成比例
function T:getHpAdd()
	return 0
end

--- 获取对战机命中的加成
-- @return 加成比例
function T:getHitAdd()
	local sl=UserInfo.allianceBox.myAlliance.scienceList
	if sl and sl[4] then
		return sl[4].level or 0
	end

	return 0
end

--- 获取对战机闪避的加成
-- @return 加成比例
function T:getDodgeAdd()
	local sl=UserInfo.allianceBox.myAlliance.scienceList
	if sl and sl[5] then
		return sl[5].level or 0
	end

	return 0
end

--- 获取对暴击的加成
-- @return 加成比例
function T:getCritAdd()
	local sl=UserInfo.allianceBox.myAlliance.scienceList
	if sl and sl[6] then
		return sl[6].level or 0
	end

	return 0
end

--- 获取对暴抗的加成
-- @return 加成比例
function T:getArmorAdd()
	local sl=UserInfo.allianceBox.myAlliance.scienceList
	if sl and sl[7] then
		return sl[7].level or 0
	end

	return 0
end

return T
