--- 联盟信息
local T=Class.MyAllianceInfo()

--- 成员列表更新时间间隔
T.MEMBER_LIST_UPDATE_INTERVAL_MS=30 * 1000

function T:def()
	---编号
	-- self.id=nil

	---职位
	-- self.post=nil

	---排名
	-- self.rank=nil

	---等级
	-- self.level=nil

	---名称
	-- self.name=nil

	--- 联盟贡献
	self.allianceContribution=0

	--- 联盟总贡献(累加,不会被扣除)
	self.allianceTotalContribution=0

	---成员数
	-- self.memberCount=nil

	---最大成员数
	-- self.memberLimit=nil

	---会长名
	-- self.leaderName=nil

	---总战力
	-- self.fight=nil

	---公告
	-- self.notice=nil

	---捐献公告
	-- self.donateNotice=nil

	---宣言
	-- self.dec=nil

	---成员列表
	-- self.memberList=nil

	---成员列表上次的刷新时间
	self.memberListLastUpdataTimeMs=0

	---申请列表
	-- self.applyList=nil

	---联盟科技列表
	-- self.scienceList=nil

	--- 联盟社交
	self.allianceSocial = require("src.app.model.alliance.AllianceSocial")() -- 这里必须建立一个联盟社交的对象
end

function T:ctor()

end

--- 解析自己的联盟信息
function T:decodeMy(data)
	self.post=data:readUnsignedByte()
	self.rank=data:readUnsignedInt()
	self.level=data:readUnsignedByte()
	self.name=data:readString()
	self.memberCount=data:readUnsignedByte()
	self.memberLimit=data:readUnsignedByte()
	self.leaderName=data:readString()
	self.fight=data:readUnsignedInt()
	self.notice=data:readString()
	self.donateNotice=data:readString()
	self.allianceContribution=data:readUnsignedInt()
	self.allianceTotalContribution=data:readUnsignedInt()
end

--- 解析成员列表信息
function T:decodeMemberList(data)
	self.memberListLastUpdataTimeMs=TimeUtils:getSyncMillis()

	local len=data:readUnsignedByte()
	self.memberCount=len
	if self.memberLimit and len > self.memberLimit then
		self.memberLimit=len
	end

	local mList={}
	if len > 0 then
		for i=1,len,1 do
			local mInfo={}
			mInfo.id=data:readUnsignedInt()
			mInfo.level=data:readUnsignedByte()
			mInfo.name=data:readString()
			mInfo.fight=data:readUnsignedInt()
			mInfo.lastLoginSec=GameUtils:modifyTimeZone(data:readUnsignedInt())
			mInfo.post=data:readUnsignedByte()
			mInfo.contribution=data:readUnsignedInt()
			-- 周贡献
			mInfo.weekContribution=data:readInt()

			mList[i]=mInfo
		end
	end
	self.memberList=mList
end

--- 移除成员
function T:removeMember(id)
	local mList=self.memberList
	
	for k,v in pairs(mList) do
		if v.id == id then
			table.remove(mList,k)
			break
		end
	end
	self.memberCount=self.memberCount-1
	
	self.memberListLastUpdataTimeMs=TimeUtils:getSyncMillis()
end

--- 解析科技列表
function T:decodeScienceList(data)
	local len=data:readUnsignedByte()

	local sList={}
	if len > 0 then
		for i=1,len,1 do
			local sInfo={}
			sInfo.id=data:readUnsignedShort()
			sInfo.level=data:readUnsignedByte()

			sList[sInfo.id]=sInfo
		end
	end
	self.scienceList=sList
end

--- 解析联盟社交
function T:decodeSocial(data)
	self.allianceSocial:decode(data)
end

--- 解析科技信息
function T:decodeDonateInfo(data)
	--今日捐献次数
	self.moneyDonateCount=data:readUnsignedByte()
	self.steelDonateCount=data:readUnsignedByte()
	self.oilDonateCount=data:readUnsignedByte()
	self.siliconDonateCount=data:readUnsignedByte()
	self.titaniumDonateCount=data:readUnsignedByte()
	--今日获取的贡献
	self.todayContributionPoint=data:readUnsignedShort()

	--科技列表
	local len=data:readUnsignedByte()
	local sList={}
	if len > 0 then
		for i=1,len,1 do
			local sInfo={}
			sInfo.id=data:readUnsignedShort()
			sInfo.level=data:readUnsignedByte()
			sInfo.sciencePoint=data:readUnsignedShort() --科技点

			sList[sInfo.id]=sInfo
		end
	end
	self.scienceList=sList
end


--- 得到联盟中所有捐献的次数总和，这里有vip限制总的捐献次数
function T:getDonateCount()
    return self.moneyDonateCount + self.steelDonateCount + self.oilDonateCount + self.siliconDonateCount + self.titaniumDonateCount
end

--- 判断vip联盟捐献次数是否捐完了,true:表示已经超过了次数
function T:judgeDonateCountIsFinish(data)
    local allCount = self:getDonateCount()
    local vipCount = VipManager:getVipDonateNum()
    if allCount >= vipCount then
        return true
    else
        return false
    end

end


--- 解析成员申请列表
function T:decodeApplyList(data)
	local len=data:readUnsignedByte()

	local pList=nil
	if len > 0 then
		pList={}
		for i=1,len,1 do
			local pInfo={}
			pInfo.id=data:readUnsignedInt()
			pInfo.name=data:readString()
			pInfo.level=data:readUnsignedByte()
			pInfo.fight=data:readUnsignedInt()

			pList[#pList+1]=pInfo
		end
	end
	self.applyList=pList
end

--- 移除申请列表项
function T:removeFromApplyList(userId)
	local pList=self.applyList

	if pList then
		for k,v in ipairs(pList) do
			if v.id == userId then
				table.remove(pList,k)
				break
			end
		end
	end
end

return T
