--- 竞技场信息
local T=Class.ArenaInfo()

--- 战斗邮件信息
T.AreanFightMailInfo=require("src.app.model.arena.AreanFightMailInfo")

--- 积分兑换项信息
T.IntegralExchangeInfo=require("src.app.model.arena.IntegralExchangeInfo")

function T:def()
	--- 第一名的名字
	-- firstName

	--- 积分
	self.integral=0

	--- 昨日排名
	self.yesterdayRank=0

	--- 基本信息
	-- bInfo

	--- 自己的排行榜数据
	-- _myRankInfo

	--- 参战部队
	-- _team

	--- 排名信息
	-- _rankInfo

	--- 战报简要信息列表
	-- _fightRecordList

	--- 战报信息列表
	self._fightReportList={}

	--- 挑战失败冷却倒计时
	self.failCooldownSec=0
	
	--- 积分兑换列表
	-- _integralExchangeList
end

function T:ctor()

end

--- 解析基础信息
-- @param byteArray 字节数组
function T:decodeBaseInfo(byteArray)
	local bInfo,c={}

	--排行榜
	local rList={}
	c=byteArray:readUnsignedByte()
	for i=1,c,1 do
		local info={}
		info.rank=byteArray:readInt()
		info.playerId=byteArray:readUnsignedInt()
		info.playerLv=byteArray:readUnsignedByte()
		info.playerName=byteArray:readString()
		
		if info.playerId == UserInfo.id then
			self._myRankInfo=info
		end

		rList[#rList+1]=info
	end
	bInfo.rankList=rList
	
	self.yesterdayRank=byteArray:readInt()

	self.bInof=bInfo
end

--- 解码参战部队
-- -- @param byteArray 字节数组
function T:decodeTeam(byteArray)
	local team = {}
	for i = 1,6,1 do  -- 1到6是位置
		local aInfo = {}
		aInfo.sid = byteArray:readUnsignedShort()
        if aInfo.sid ~= 0 then
            aInfo.count = byteArray:readUnsignedShort()

            team[i] = aInfo
        end
	end
	self.team = team
end

--- 解码排行榜数据
function T:decodeRankInfo(byteArray,startIndex)
	startIndex=startIndex or 1

	local rInfo={}

	local rList
	if startIndex == 1 then
		rList={}
	else
		rList=self._rankInfo.rankList
	end
	local len=byteArray:readUnsignedByte()
	for i=1,len,1 do
		local info={}
		info.rank=byteArray:readInt()
		info.playerId=byteArray:readUnsignedInt()
		info.playerLv=byteArray:readUnsignedByte()
		info.playerName=byteArray:readString()

		rList[startIndex]=info
		startIndex=startIndex+1
	end
	rInfo.rankList=rList
	rInfo.isReadEnd=not byteArray:readBoolean()

	self._rankInfo=rInfo
end

--- 解析战报简要信息列表
function T:decodeFightRecordList(byteArray)
	local rList={}

	local len=byteArray:readUnsignedByte()
	for i=1,len,1 do
		local fMailInfo = self.AreanFightMailInfo()
		fMailInfo.fightType=FightType.FIGHT_TYPE_ARENA
		fMailInfo:decode(byteArray)
		rList[fMailInfo.id] = fMailInfo
	end

	self._fightRecordList=rList
end

--- 解析战报详细信息
function T:decodeFightReport(recordId,byteArray)
	local info=require("src.app.model.mail.mailContentBox.MailFightReportContent")()
	info:decode(byteArray)
    self._fightReportList[recordId]=info
    
    local sMailInfo=self._fightRecordList and self._fightRecordList[recordId] or nil
    if sMailInfo then
    	sMailInfo.isRead=true
    end

	return info
end

--- 解析积分兑换列表信息
function T:decodeIntegralExchangeList(byteArray)
	local gList={}
	
	local len=byteArray:readUnsignedByte()
	for i=1,len,1 do
		local gInfo=self.IntegralExchangeInfo(byteArray)
		gList[#gList+1]=gInfo
	end
	
	self._integralExchangeList=gList
end

return T
