--- 竞技场信息按理器
local T=Class.ArenaInfoMgr(LuaEventDispatcher2)

--- 竞技场信息
T.ArenaInfo=require("src.app.model.arena.ArenaInfo")

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件-第一名字被改变
T.EVT_TYPE_CHANGE_FIRST_NAME="evt_type_change_first_name"

--- 事件-积分被改变
T.EVT_TYPE_CHANGE_INTEGRAL="evt_type_change_integral"

--- 事件-基础数据被重置
T.EVT_TYPE_RESET_BASE_INFO="evt_type_reset_base_info"

--- 事件-参战部队信息被重置
T.EVT_TYPE_RESET_TEAM="evt_type_reset_team"

--- 事件-挑战失败冷却倒计时更新
T.EVT_TYPE_CHANGE_FAIL_COOLDOWN="evt_type_change_fail_cooldown"

--- 事件-排行榜数据更新
T.EVT_TYPE_CHANGE_RANK_DATA="evt_type_change_rank_data"

--- 事件-挑战记录列表更新
T.EVT_TYPE_CHANGE_FIGHT_RECORD_LIST="evt_type_change_fight_record_list"

--- 事件-挑战战报详细信息被获取
T.EVT_TYPE_GETED_FIGHT_REPORT="evt_type_geted_fight_report"

function T:def()
	LuaEventDispatcher2.def(self)

	--- 竞技场信息
	-- _info
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 预初始化
function T:preInit()
	self._info=self.ArenaInfo()
end

--- 初始化
function T:init()
	self:luaDispatchEvent(self.EVT_TYPE_RESET)
end

--- 更新检查
function T:updateCheck()
	local remainSec=SetStates:getArenaFailTime() + GameCfg.ARENA_CHALLENGE_FAIL_COOLDOWN_SEC - TimeUtils:getSyncMillis()/1000
	if remainSec then
		self:setFailCooldownSec(remainSec)
	else
		self:stopUpdateCheck()
	end
end

--- 开始更新检查
function T:satarUpdateCheck()
	if self.updateId then
		return
	end

	self.updateId=Cocos2dxUtils:interval(function()
		self:updateCheck()
	end,1)
end

--- 结束更新检查
function T:stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

--- 尝试开始更新检查
function T:tryStartUpdateCheck()
	local remainSec=SetStates:getArenaFailTime() + GameCfg.ARENA_CHALLENGE_FAIL_COOLDOWN_SEC - TimeUtils:getSyncMillis()/1000
	if remainSec > 0 then
		self:updateCheck()
		self:satarUpdateCheck()
	end
end

--- 尝试结束更新检查
function T:tryStopUpdateCheck()
	local remainSec=SetStates:getArenaFailTime() + GameCfg.ARENA_CHALLENGE_FAIL_COOLDOWN_SEC - TimeUtils:getSyncMillis()/1000
	if remainSec <= 0 then
		self:stopUpdateCheck()
	end
end

--- 设置第一名的名字
function T:setFirstName(name)
	if name ~= self._info.firstName then
		self._info.firstName=name

		self:luaDispatchEvent(self.EVT_TYPE_CHANGE_FIRST_NAME)
	end
end
--- 获取第一名的名字
function T:getFirstName()
	return self._info.firstName
end

--- 增加积分
function T:addIntegral(value)
	self._info.integral=self._info.integral + value

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_INTEGRAL)
end
--- 获取当前积分
function T:getIntegral()
	return self._info.integral
end

--- 获取昨日排名
function T:getYesterdayRank()
	return self._info.yesterdayRank
end

--- 解析基础信息
function T:decodeBaseInfo(byteArray)
	self._info:decodeBaseInfo(byteArray)

	self:luaDispatchEvent(self.EVT_TYPE_RESET_BASE_INFO)
end
--- 清除基础数据
function T:clearBaseInfo()
	self._info.bInof=nil

	self:luaDispatchEvent(self.EVT_TYPE_RESET_BASE_INFO)
end
--- 获取基础数据
function T:getBaseInfo()
	return self._info.bInof
end

--- 设置玩家排名
function T:setRank(value)
	local myRankInfo=self._info._myRankInfo
	if myRankInfo then
		if value ~= myRankInfo.rank then
			myRankInfo.rank=value

			require("src.app.communication.arena.GetArenaInfoCommand")():exec()
		end
	end
end
--- 获取玩家排名
function T:getRank()
	local myRankInfo=self._info._myRankInfo
	if myRankInfo then
		return myRankInfo.rank
	end
	return 0
end

--- 解析参战部队信息
function T:decodeTeam(byteArray)
	self._info:decodeTeam(byteArray)

	self:luaDispatchEvent(self.EVT_TYPE_RESET_TEAM)
end
--- 设置参战部队信息
function T:setTeam(team)
	if team ~= self._info.team then
		self._info.team=team

		self:luaDispatchEvent(self.EVT_TYPE_RESET_TEAM)
	end
end
--- 获取参战部队信息
function T:getTeam()
	return self._info.team
end

--- 设置挑战失败冷却倒计时
function T:setFailCooldownSec(value)
	if value ~= self._info.failCooldownSec then
		self._info.failCooldownSec=value

		self:luaDispatchEvent(self.EVT_TYPE_CHANGE_FAIL_COOLDOWN)
	end
end
--- 获取挑战失败冷却倒计时
function T:getFailCooldownSec()
	return self._info.failCooldownSec
end

--- 解析排行榜数据
function T:decodeRankInfo(byteArray,startIndex)
	self._info:decodeRankInfo(byteArray,startIndex)

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_RANK_DATA,startIndex)
end
--- 清除排行榜数据
function T:clearRankInfo()
	self._info._rankInfo=nil

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_RANK_DATA)
end
--- 获取排行榜数据
function T:getRankInfo()
	return self._info._rankInfo
end

--- 解析战报信息列表
function T:decodeFightRecordList(byteArray)
	self._info:decodeFightRecordList(byteArray)

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_FIGHT_RECORD_LIST)
end
--- 清除战报信息列表
function T:clearFightRecordList()
	self._info._fightRecordList=nil
end
--- 清除战报信息
function T:clearFightRecord(recordId)
	self._info._fightRecordList[recordId]=nil
	self:clearFightReport(recordId)

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_FIGHT_RECORD_LIST)
end
--- 获取战报信息列表
function T:getFightRecordList()
	return self._info._fightRecordList
end

--- 解析战报详细信息
function T:decodeFightReport(recordId,byteArray)
	self._info:decodeFightReport(recordId,byteArray)

	self:luaDispatchEvent(self.EVT_TYPE_GETED_FIGHT_REPORT,recordId)
end
--- 清除指定战报详细信息
function T:clearFightReport(recordId)
	self._info._fightReportList[recordId]=nil
end
--- 获取指定战报详细信息
function T:getFightReport(recordId)
	return self._info._fightReportList[recordId]
end

--- 解析积分兑换信息列表
function T:decodeIntegralExchangeList(byteArray)
	self._info:decodeIntegralExchangeList(byteArray)

	--目前无派发事件需要
end
--- 清除积分兑换信息列表
function T:clearIntegralExchangeList()
	self._info._integralExchangeList=nil
end
--- 获取积分兑换信息列表
function T:getIntegralExchangeList()
	return self._info._integralExchangeList
end

--- 购买挑战点
function T:buyChallengePoint()
	if DailyStates:getArenaFightNum() <= 0 then
		MUIAlter_noTitle.show({
			info=Language:get("ug1316")
		})
		return
	end

	--是否可购买挑战点
	if DailyStates:getArenaBuyNum() < VipManager:getArenaChallengePointBuyLimit() then
		MUIAlter.show({
			info=Language:get("ug1307",GameCfg.ARENA_CHALLENGE_POINT_COST_DIAMOND,1),
			flag=3,
			callback=function(flag)
				if flag == FLAG_OK then
					require("src.app.communication.arena.BuyChallengePointCommand")():exec(function(state)
						if state == ReturnCode.SUCCESS then
							MUIAlter_singleLine.show({
								info=Language:get("ug1321"),
								showSecond=1
							})
						else
							MUIAlter_singleLine.show({
								info=ErrorCode:getErrInfo(state)
							},WindowManager:getCurrentWindow())
						end
					end)
				end
			end
		})
	else
		MUIAlter.show({
			info = Language:get("ug1309"),
			size = 30,
			flag = 1,
			confirmText = Language:get("uc332"),
			triggerCloseMethod = "buttons",
			isShowCloseButton = true,
			callback = function(flag)
				-- 转到vip界面
				if flag == 0 then
					WindowManager:push(
						require("src.app.uiScript.components.vip.VIPWindow").new()
					)
				end
			end
		})
	end
end

--- 清除挑战失败后的冷却时间
function T:clearFailCooldown()
	local rSec=self:getFailCooldownSec()

	local min=math.floor(rSec / 60)
	local costDiamond=(min == 0 and 1 or min) * GameCfg.ARENA_CHALLENGE_FAIL_COOLDOWN_PER_MIN_COST

	MUIAlter.show({
		info=Language:get("ug1319",costDiamond),
		flag=3,
		callback=function(flag)
			if flag == FLAG_OK then
				require("src.app.communication.arena.ClearArenaFailCooldownCommand")():exec(function(state)
					if state == ReturnCode.SUCCESS then
						MUIAlter_singleLine.show({
							info=Language:get("ug1320"),
							showSecond=1
						})
					else
						MUIAlter_singleLine.show({
							info=ErrorCode:getErrInfo(state)
						},WindowManager:getCurrentWindow())
					end
				end)
			end
		end
	})
end

--- 获取竞技场战斗记录列表
function T:getFightRecordListFromServer()
	require("src.app.communication.arena.GetChallengeRecordListCommand")():exec(function(state)
		if state == ReturnCode.SUCCESS then
		else
			MUIAlter_singleLine.show({
				info=ErrorCode:getErrInfo(state)
			},WindowManager:getCurrentWindow())
		end
	end)
end

return T()
