local T=Class.ArmyTaskQueueManager(LuaEventDispatcher2)

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件--开始进攻
T.EVT_START_BATTLE_REFRESH = "evt_start_battle_refresh"

--- 事件--进攻（出征）剩余时间刷新
T.EVT_BATTLE_REMAIN_TIME = "evt_battle_remain_time"

--- 事件--进攻（出征）结束
T.EVT_BATTLE_OVER = "evt_battle_over"


--- 驻扎剩余时间刷新
T.EVT_STATION_REMAIN_TIME = "evt_station_remain_time"

--- 驻扎结束
T.EVT_STATION_OVER = "evt_station_over"



--- 事件--开始返回
T.EVT_START_BACK_REFRESH = "evt_start_back_refresh"

--- 部队返回剩余时间刷新
T.EVT_BACK_REMAIN_TIME = "evt_back_remain_time"

--- 部队返回结束:如果部队进攻失败，也会结束
T.EVT_BACK_OVER_TIME = "evt_back_over_time"


--- 主界面快捷按钮事件
T.EVT_HOMEPAGE_RIGHT_BAR = "evt_homepage_right_bar"


function T:def()
	LuaEventDispatcher2.def(self)

	--- 更新的id
	self.updateId = nil
end

function T:init(queueInfos)
	self.queueInfos=queueInfos

	self:startUpdateCheck()

	self:luaDispatchEvent(self.EVT_TYPE_RESET)
end

--- 增加项
function T:addItem(data)
	local queueItem = mapping({}, require("src.app.model.army.armyTask.ArmyTaskQueueItem"))
	queueItem:decode(data)
	queueItem.remainingSec=queueItem.movingCost
	self.queueInfos[queueItem.id] = queueItem

	self:startUpdateCheck()
	self:luaDispatchEvent(self.EVT_START_BATTLE_REFRESH, queueItem)

	--PlayerInstance:setPlayerFightLogic()
end

--- 添加项
function T:addItem2(armyTaskQueueItem)
	self.queueInfos[armyTaskQueueItem.id] = armyTaskQueueItem

	self:startUpdateCheck()
	self:luaDispatchEvent(self.EVT_START_BATTLE_REFRESH, armyTaskQueueItem)
end

--- 根据id移除队列中的项
function T:removeItem(id)
	local list = self.queueInfos
	for k,v in pairs(list) do
		if k == id then
			self:luaDispatchEvent(self.EVT_BACK_OVER_TIME .. k,v)
			list[k] = nil
			self:luaDispatchEvent(self.EVT_BACK_OVER_TIME, v)  -- 这个必须是数据刷新了后派发
			self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR)
		end
	end
	self:startUpdateCheck()
	--PlayerInstance:setPlayerFightLogic()
end

--- 撤军：部队开始返回
function T:withDrawAircraft(data,id)
	self.queueInfos[id] = nil

	local queueItem = mapping({}, require("src.app.model.army.armyTask.ArmyTaskQueueItem"))
	queueItem:decode(data)

	self.queueInfos[id] = queueItem


	local syncSec = TimeUtils:getSyncMillis()/1000
	queueItem.remainingSec = queueItem.endTimeSec - syncSec
	self:luaDispatchEvent(self.EVT_BACK_REMAIN_TIME .. id,queueItem)
	self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR)

	self:startUpdateCheck()

	self:luaDispatchEvent(self.EVT_START_BACK_REFRESH, queueItem)
	--PlayerInstance:setPlayerFightLogic()
end

--- 检测要分状态
function T:startUpdateCheck()
	if self.updateId then
		return
	end

	local function update()
		local list = self.queueInfos

		local syncSec = TimeUtils:getSyncMillis()/1000

		for k,v in pairs(list) do  -- k是id

			-- 这里要分3种情况

			if v.status == 1 then  -- 出征
				v.remainingSec = v.endTimeSec - syncSec
				if v.remainingSec <= 0 then -- 出征结束
					self:luaDispatchEvent(self.EVT_BATTLE_OVER .. v.id,v)
					self:luaDispatchEvent(self.EVT_BATTLE_OVER,v)
					self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR)
					--PlayerInstance:setPlayerFightLogic()
				else -- 刷新出征剩余时间
					self:luaDispatchEvent(self.EVT_BATTLE_REMAIN_TIME .. v.id,v)
					self:luaDispatchEvent(self.EVT_BATTLE_REMAIN_TIME, v)
					self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR)
				end
		elseif v.status == 2 then -- 驻扎
			-- 这里的v.endTimeSec是驻扎的开始时间，要算出结束时间
			-- 算出真正的结束时间
			local realEndTime = self:getStationAllTime(v) + v.endTimeSec
			v.remainingSec = realEndTime - syncSec
			if v.remainingSec <= 0 then -- 驻扎结束
				self:luaDispatchEvent(self.EVT_STATION_OVER .. v.id,v)
				self:luaDispatchEvent(self.EVT_STATION_OVER,v)
				self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR, v)
				--PlayerInstance:setPlayerFightLogic()
			else
				self:luaDispatchEvent(self.EVT_STATION_REMAIN_TIME .. v.id,v)
				self:luaDispatchEvent(self.EVT_STATION_REMAIN_TIME, v)
				self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR, v)
			end


		else -- 返回
			v.remainingSec = v.endTimeSec - syncSec
			if v.remainingSec <= 0 then -- 返回结束

				--				if table.maxn(list) == 0 then
				--					Cocos2dxUtils:clearScheduler(self.updateId)
				--					self.updateId = nil
				--			    end

				self:luaDispatchEvent(self.EVT_BACK_OVER_TIME ..v.id,v)
				list[k] = nil
				self:luaDispatchEvent(self.EVT_BACK_OVER_TIME, v)
				self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR)
				self:tryStopUpdate()
				--PlayerInstance:setPlayerFightLogic()

			else
				self:luaDispatchEvent(self.EVT_BACK_REMAIN_TIME .. v.id,v)
				self:luaDispatchEvent(self.EVT_BACK_REMAIN_TIME, v)
				self:luaDispatchEvent(self.EVT_HOMEPAGE_RIGHT_BAR)
			end

		end

		end
	end

	--	if list and table.maxn(list) > 0 then  -- maxn的索引从1开始，才能这样用的
	--		self.updateId = Cocos2dxUtils:interval(update,1)
	--	end
	if self.updateId then
		return
	end

	local list = self.queueInfos
	if list then
		for _ in pairs(list) do
			self.updateId=Cocos2dxUtils:interval(update,1)

			break
		end
	end
end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
		local list = self.queueInfos

		--如果队列为空,结束时间更新检查
		local isEmpty=true
		for _ in pairs(list) do
			isEmpty=false
			break
		end

		if isEmpty then
			Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId=nil
		end
	end
end

function T:stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

--- 根据飞机队列得到驻扎的总时间:单位为s
function T:getStationAllTime(armyTaskInfo)
	local allLoad = self:getStationLoad(armyTaskInfo.armyExpeditionAircrafts)
	if allLoad == 0 then
		return 0
	end
	return (allLoad/armyTaskInfo.resourceEfficiency) * 60
end

--- 得到驻扎的总载重量
function T:getStationLoad(armyExpeditionAircrafts)
	local capacity = 0
	for i = 1,6,1 do
		if armyExpeditionAircrafts[i] and armyExpeditionAircrafts[i].sid ~= 0 then
			capacity = capacity + Aircraft_temp[armyExpeditionAircrafts[i].sid].capacity * armyExpeditionAircrafts[i].count
		end

	end
	return capacity
end

--- 得到驻扎的已经采集的载重量,time为采集的时间
function T:getHaveStationLoad(armyTaskInfo,time)
	local alltime = self:getStationAllTime(armyTaskInfo)
	local capacity = self:getStationLoad(armyTaskInfo.armyExpeditionAircrafts)

	return (time/alltime) * capacity

end

--- 选择战机前，在资源地图点击攻击的时候加的判断逻辑，如果为true，则可以攻击，否则不能攻击
function T:judgeIsAttackInWorld(data,finsh)
	-- 条件：按照优先级

	----- 在资源地图界面
	-- 首先防护罩：npc没有防护罩，如果是玩家：从data里面判断玩家是否有防护罩，如果有则提示“该玩家处于防护罩状态，无法对其进攻”
	-- 限制攻打的队列数量，提示"部队出征个数已达上限，是否查看部队出征情况"，点是跳转到战斗队列界面
	-- 如果点的资源点或者玩家，并且自己已有部队驻扎在此，提示“该资源点已有己方部队驻扎，不能再次派遣部队”

	local npcType = data.id
	if npcType > 0 then
		-- data里面有一个防护罩时间，如果存在并且为true，则提示
		if data.isProtection and data.isProtection == true then
			MUIPop.PlayerIsProtect()
			return false
		end

		--- 目标有一个标识来判断
		if self:hasPlayerBattle(data) then
			MUIPop.NotSendArmyAgain()
			return false
		end

	else
		local num = PlayerInstance:getVipbattleQueueCount()
		local queueNum = 0
		for k,v in pairs(self.queueInfos) do
			if v then
				queueNum = queueNum + 1
			end
		end

		if queueNum >= num then
			--			local function confirm()
			--			-- 转到部队出征页面,这种方式又是什么方式进入呢(防守部队方式？)

			--			end
			--MUIPop.CheckArmyBattle(confirm)

			local describe1 = Language:get("uc320",UserInfo.vipLevel)
			local describe2 = Language:get("uc321",UserInfo.vipLevel,num)
			local describe3 = Language:get("uc322")

			local layout = VipTipsInfo.new(describe1,describe2,describe3)

			MUIAlter.show({
				info = layout,
				size = 30,
				flag = 1,
				callback = function() -- 转到vip界面
					local dd = 3
				end
			})


			return false
		end

		--- 目标有一个标识来判断
		if self:hasNpcBattle(data) then
			MUIPop.NotSendArmyAgain()
			return false
		end

	end

	--- 如果上面的验证成功了
	if npcType > 0 then -- 攻打玩家
		local myId = UserInfo.allianceBox.myAlliance.id
		playerId = data.allianceId
		if myId and myId > 0 and playerId and myId == playerId then -- 同一个联盟
			local function confirm()
				finsh()
			end
			MUIPop.AttackAlliance(confirm)
		else
			return true
		end
	else
		return true
	end


	----- 在派兵界面
	-- 精力点:安琳哥那边写的
	-- 如果攻打的是玩家并且自己有防护罩，提示“进攻玩家会使自己当前防护罩状态失效，确定进攻吗?”,去除防护罩是通讯成功后做的，然后队列中的防护罩直接消失，
	-- 添加防护罩只有使用buff这一种情况添加

end

-- 玩家--进攻的资源点有自己部队驻扎，不允许进去派兵界面 ,该资源点已有己方部队驻扎，不能再次派遣部队
function T:hasPlayerBattle(data)
	local list = self.queueInfos
	for k,v in pairs(list) do
		if v then
			if v.status == 2 and v.sourceId == data.id  then
				return true
			end
		end
	end

	return false
end

-- NPC--进攻的资源点有自己部队驻扎，不允许进去派兵界面 ,该资源点已有己方部队驻扎，不能再次派遣部队
function T:hasNpcBattle(data)
	local list = self.queueInfos
	for k,v in pairs(list) do
		if v then
			if v.status == 2 and v.posX == data.x and v.posY == data.y then
				return true
			end
		end
	end

	return false
end


--- 计算两点的行军时间
function T:computeBattleTime(x1,y1,x2,y2)
	local x = math.abs(x1 - x2)
	local y = math.abs(y1 - y2)
	local ds = math.floor(math.sqrt(x*x + y*y))

	local techLevel = TechnologiesInfoMgr:getTechLevel(9)  -- 行军科研项的等级
	local time = math.floor((ds * 10 + 120)/(1 + 0.05 * techLevel))
	local marchBuff = BuffQueueInfoMgr:checkActive(12)  -- 行军的buffSid
	if marchBuff then
		time = time/2
	else
		time = time
	end

	-- 还要vip特权
	local marchSpeed = VipManager.getVipMarchSpeed()
	if marchSpeed >= 0 then
		return (100 - marchSpeed)/100 * time  -- 百分比是反起的
	else
		return time
	end
end


--- 判断是否有任务在进行
function T:judgeIsTask()
	local queueInfos = self.queueInfos

	for k,v in pairs(queueInfos) do
		if v then
			return true
		end
	end

	return false

end

--- 释放
function T:release()
	self:stopUpdateCheck()
	self:releaseEventListeners()
end


--- 主界面快捷栏显示,
function T:getData()
	local queueInfos = mapping({},self.queueInfos)

	local timeTb = nil
	for k,v in pairs(queueInfos) do
		if v and v.remainingSec >= 0 and v.status == 1 or v.status == 3 then
			if timeTb then
				if timeTb.remainingSec > v.remainingSec then
					timeTb = v
				end
			else
				timeTb = v
			end
		end
	end

	if timeTb then
		return timeTb
	end

	local percentTb = nil
	for k,v in pairs(queueInfos) do
		if v and v.status == 2 then
			if v.remainingSec <= 0 then
				percentTb = v
				break
			else
				if percentTb then
					if percentTb.remainingSec < v.remainingSec then
						percentTb = v
					end
				else
					percentTb = v
				end
			end
		end
	end

	return percentTb  -- 如果为nil值，则不主界面不显示按钮

end


--- 判断自己是否有战斗队列存在
function T:selfIsQueue()
	local queueInfos = self.queueInfos

	for k,v in pairs(queueInfos) do
		if v then
			if v.sourceId == UserInfo.id then
				return true
			end
		end
	end

	return false
end

--- 获取正在执行的所有任务信息
function T:getTaskInfos()
	return self.queueInfos
end

--- 获取攻击的 信息
function T:getBattleInfos()
	local list = self.queueInfos

	local syncSec = TimeUtils:getSyncMillis()/1000

	local tb = {}
	for id,v in pairs(list) do
		if v.status == 1 then
			v.remainingSec = v.endTimeSec - syncSec
			table.insert(tb, v)
		end
	end

	return tb
end

--- 获取占领队列信息
function T:getOccupationInfos()
	local list = self.queueInfos

	local syncSec = TimeUtils:getSyncMillis()/1000

	local tb = {}
	for id,v in pairs(list) do
		if v.status == 2 then
			v.remainingSec = v.endTimeSec - syncSec
			table.insert(tb, v)
		end
	end

	return tb
end

--- 获取返回的信息
function T:getBackInfos()
	local list = self.queueInfos

	local syncSec = TimeUtils:getSyncMillis()/1000

	local tb = {}
	for id,v in pairs(list) do
		if v.status == 3 then
			v.remainingSec = v.endTimeSec - syncSec
			table.insert(tb, v)
		end
	end

	return tb
end

function T:getRemainTimeById(id)
	local list = self.queueInfos
	local syncSec = TimeUtils:getSyncMillis()/1000

	if list[id] then
		return list[id].endTimeSec - syncSec
	else
		return 0
	end
end


--- 得到队列中执行任务的所有部队总和
function T:getAllAircraftTotal()
	local list = mapping({},self.queueInfos)
	local team = {}
	if list then
		for k,v in pairs(list) do
			if v.armyExpeditionAircrafts then
				if #team == 0 then
					for i=1,16,1 do
						local info = {}
						info.sid = i
						info.count = 0
						team[i] = info
					end
					team = self:getTeamAdd(team,mapping({},v.armyExpeditionAircrafts))
				else
					team = self:getTeamAdd(team,mapping({},v.armyExpeditionAircrafts))
				end
			end

		end
	end


	return team

end

function T:getTeamAdd(team1,team2)
	local team = team1
	for k,v in pairs(team) do
		for i = 1,6,1 do -- team2是1到6
			if team2[i] and team2[i].sid == v.sid then
				team[k].count = team[k].count + team2[i].count
		end
		end

	end

	return team
end


return T
