--[[
    聊天信息
]]

local T = extend()

--- 聊天的唯一id
T.id = nil

--- 用户名的id:对应玩家的id
T.sourceId = nil

--- vip等级
T.vipLevel = nil

--- 头像:这里暂时不加头像，发过来的是1,2（暂时对应于头像1或2），如果为其它则默认一个头像
T.head = nil

--- 聊天信息的类型:0:代表普通信息，1:代表系统信息，2:代表战报信息
T.contentType = nil

--- 用户名:对应于玩家的userName
T.sourceName = nil

--- 内容,如果是战报 oil_mine.17
T.content = nil

--- 战报类型,2表示普通战报,4表示竞技场战报
T.fightReportType = nil

--- 如果聊天信息的类型是战报，则会读一个战报的id
T.battleReportId = nil

--- 战报结果
T.battleResult = nil


--- 战斗结果类型:1:进攻胜利，2：进攻失败，3：防守胜利，4：防守失败，5：攻击无效
T.resultType = nil

--- 真正的标题内容
T.realTitle1 = nil
T.realTitle2 = nil

--- 战斗信息数据
T.fightInfo = nil

--- 战斗回放数据
T.fightData = nil

function T:decode(data)
    self.id = data:readUnsignedInt()
    self.sourceId = data:readUnsignedInt()
    self.vipLevel = data:readUnsignedByte()
    self.head = data:readUnsignedByte()
    self.contentType = data:readUnsignedByte()
    self.sourceName = data:readString()
    self.militaryRank = data:readByte() -- 军衔
    self.content = data:readString()

    if self.contentType == 2 then
        self.battleResult = data:readUnsignedByte() -- 1,2,4,8,16,32
        self.battleReportId = data:readUnsignedInt()
        --TODO
        self.fightReportType = data:readUnsignedByte() -- 2表示普通战报,4表示竞技场战报

        self:setResultType()

        if self.content ~= "" then
            self:setRealTitle()
        end
        
        
    end

    self.languageConfig = data:readBoolean()
    if self.languageConfig == true then -- 读取哈希翻译表
        local length = data:readByte()
        self.languageList = {}
        for i = 1,length,1 do
            local temp = {}
            temp.key = data:readByte()
            temp.content = data:readString()
            self.languageList[temp.key] = temp
        end
        

    end

end

--- 设置战斗结果类型
function T:setResultType()
    local result = self.battleResult
    if BitOperator:_and(result, 1) ~= 0 then
        self.resultType = 1
	end

    if BitOperator:_and(result, 2) ~= 0 then
        self.resultType = 2
	end

    if BitOperator:_and(result, 4) ~= 0 then
        self.resultType = 3
	end

    if BitOperator:_and(result, 8) ~= 0 then
        self.resultType = 4
	end

    if BitOperator:_and(result, 32) ~= 0 then
        self.resultType = 5
	end

end


--- 真正的标题内容
function T:setRealTitle()
    if BitOperator:_and(self.battleResult, 16) ~= 0 then -- npc了
        local first,second = CommonMethod:getCharFromStr(self.content)

        self.realTitle1 = CommonMethod:getRealText(first)
        self.realTitle2 = self.realTitle1 .. "Lv." .. second

	end

end


--- 得到聊天内容
function T:getContent()
    if self.contentType == 1 then -- 系统消息
        if self.languageConfig == true then
            if self.languageList then
                -- 语言
                local LanguageUtils = require("src.app.utils.LanguageUtils")
                local id = LanguageUtils:getId()
                local judge = false
                for k,v in pairs(self.languageList) do
                    if v then
                        if id == v.key then
                            judge = true
                            break
                        end
                    end

                end

                if judge == true then
                    return self.languageList[id].content
                else
                    if self.languageList[0] then -- 英文
                        return self.languageList[0].content
                    elseif self.languageList[19] then -- 繁体
                        return self.languageList[19].content
                    elseif self.languageList[1] then -- 简体
                        return self.languageList[1].content
                    else
                        return self.content
                    end

                end



            else
                return self.content
            end

        else
            return self.content
        end


    else
        return self.content
    end

end


return T