--[[
    阵型设置盒子
]]
local T = Class.FormationBox(LuaEventDispatcher2)

-- 阵型设置信息
T.FormationInfo = require("src.app.model.formation.FormationInfo")

--- 事件--设置阵型(需要与后端通讯的)
T.EVT_SET_FORMATION_REFRESH = "evt_set_formation_refresh"

--- 事件--改变默认阵型（需要与后端通讯的）
T.EVT_CHANGE_DEFAULT__FORMATION_REFRESH = "evt_change_default_formation_refresh"

--- 事件--使用阵型(不需要与后端通讯的，前端只是把阵型放到页面的事件)
T.EVT_USE_FORMATION_REFRESH = "evt_use_formation_refresh"


function T:def()
    LuaEventDispatcher2.def(self)

    self.formationBox = {} -- 阵型盒子
    self.defaultFormationId = 0
end

--- （0，1，2，3，这4个阵型的id）
function T:decode(data)
    local defaultFormationId = data:readByte()
    self.defaultFormationId = defaultFormationId
    local length = data:readByte() -- 必须是4

    local infos = {}
    for i = 1,length,1 do
        local info = self.FormationInfo()
		info:decode(data)
        info.id = i - 1
		infos[info.id] = info
    end

    self.formationBox = infos

    self:setDefultFormation(defaultFormationId)

end

--- 设置默认阵型
function T:setDefultFormation(id)
    self.defaultFormationId = id
    for i = 1,4,1 do
        if self.formationBox[i - 1] then
            if (i - 1) == id then
                self.formationBox[i - 1].isDefault = true
            else
                self.formationBox[i - 1].isDefault = false
            end

        end

    end

    self:luaDispatchEvent(self.EVT_CHANGE_DEFAULT__FORMATION_REFRESH)

end


--- 使用阵型
function T:useFormation(id)
    self:luaDispatchEvent(self.EVT_USE_FORMATION_REFRESH,self.formationBox[id].aircrafts)
end



--- 判断阵型是否超过部队的数量
function T:judgeAircraftNumIsEngough(id)
    local aircrafts = self.formationBox[id].aircrafts

    local allAircrafts = UserInfo.city.aircraftBox.infos

    local judge = false
    if aircrafts then
        for i = 1,6,1 do
            if aircrafts[i] and aircrafts[i].sid and aircrafts[i].sid > 0 then
                if aircrafts[i].count > allAircrafts[aircrafts[i].sid].num then
                    judge = true
                    break
                end
            end

        end

    end


    return judge


end

--- 得到vip是否开放此阵型id,
function T:getFormationIsOpen(id)
    local dataTable = {}
    local vipLevel = UserInfo.vipLevel

    if id == 0 then
        dataTable.isOpen = true
    elseif id == 1 then
        if vipLevel < 3 then
            dataTable.isOpen = false
            dataTable.limitVipLvl = 3
        else
            dataTable.isOpen = true
        end
    elseif id == 2 then
        if vipLevel < 5 then
            dataTable.isOpen = false
            dataTable.limitVipLvl = 5
        else
            dataTable.isOpen = true
        end
    elseif id == 3 then
        if vipLevel < 8 then
            dataTable.isOpen = false
            dataTable.limitVipLvl = 8
        else
            dataTable.isOpen = true
        end
    else
        dataTable.isOpen = false
        dataTable.limitVipLvl = 12
    end

    return dataTable

end




return T