--[[

    邮件管理:这个和聊天是一样的，也要单独来管理，不放到UserInfo下面

]]
local T = Class.MailManager(LuaEventDispatcher2)

--- 事件-第一次获得登陆的邮箱列表信息的的时候派发事件，然后给主界面显示
T.EVT_TYPE_FIRST_GET_MAIL_LIST = "evt_type_first_get_mail_list"

--- 事件-删除所有邮件
T.EVT_TYPE_DELETE_ALL_MAIL = "evt_type_delete_all_mail"

--- 事件-删除某一封邮件
T.EVT_TYPE_DELETE_ONE_MAIL = "evt_type_delete_one_mail"

--- 事件-查看邮件（是要通讯的发消息的查看）
T.EVT_TYPE_READ_MAIL = "evt_type_read_mail"

--- 事件-接收邮件(那3种类型都是刷的，如果你发了的话，其中发件箱也会刷过来)
T.EVT_TYPE_RECEIVE_MAIL = "evt_type_receive_mail"

--- 事件-获取接下来的10条数据
T.EVT_TYPE_GET_TEN_MAIL = "evt_type_get_ten_mail"

--- 邮箱列表信息(3个都有)
T.MailBox = require "src.app.model.mail.mailInfoBox.MailBox"
T.mailBox = mapping({}, T.MailBox)

--- 邮箱的具体内容盒子（3个都有）
T.MailContentBox = require "src.app.model.mail.mailContentBox.MailContentBox"
T.mailContentBox = mapping({}, T.MailContentBox)

function T:def()
	LuaEventDispatcher2.def(self)
end

--- 根据tag值得到某一个邮箱的列表信息1:普通邮件，2：报告，3：发件箱
function T:getMailBoxByIndex(index)
	local mailInfos = nil
	local mailBool = nil
	local mailNotReadCount = nil
	local mailAllCount = nil
	local mailBox = self.mailBox
	if index == 1 then
		mailInfos = mailBox.commonMailInfos
		mailBool = mailBox.commonBool
		mailNotReadCount = mailBox.commonNotReadCount
		mailAllCount = mailBox.commonAllCount
	elseif index == 2 then
		mailInfos = mailBox.fightMailInfos
		mailBool = mailBox.fightBool
		mailNotReadCount = mailBox.fightNotReadCount
		mailAllCount = mailBox.fightAllCount
	elseif index == 3 then
		mailInfos = mailBox.sendMailInfos
		mailBool = mailBox.sendBool
		mailNotReadCount = mailBox.sendNotReadCount
		mailAllCount = mailBox.sendAllCount
	end

	return {mailInfos,mailBool,mailNotReadCount,mailAllCount}  -- 返回一个表
end

--- 接收邮件（刷过来的）:这里面包括
function T:receiveMailInfo(data)
	--- 刷新邮件的3种情况
	local msgType = data:readUnsignedByte()
	if msgType == 1 then
		local commonMailInfo = require("src.app.model.mail.mailInfoBox.CommonAndSendMailInfo")()
		commonMailInfo:decode(data,1)
		if self.mailBox.commonMailInfos == nil then
			self.mailBox.commonMailInfos = {}
		end
		self.mailBox.commonMailInfos[commonMailInfo.id] = commonMailInfo

		if self.mailBox.commonNotReadCount then
			self.mailBox.commonNotReadCount = self.mailBox.commonNotReadCount + 1
		else
			self.mailBox.commonNotReadCount = 1
		end
		if self.mailBox.commonAllCount then
			self.mailBox.commonAllCount = self.mailBox.commonAllCount + 1
		else
			self.mailBox.commonAllCount = 1
		end

		self:luaDispatchEvent(self.EVT_TYPE_RECEIVE_MAIL .. msgType,commonMailInfo) -- 把数据commonMailInfo传过去
	elseif msgType == 2 then
		local fightMailInfo = require("src.app.model.mail.mailInfoBox.FightMailInfo")()
		fightMailInfo:decode(data)
		if self.mailBox.fightMailInfos == nil then
			self.mailBox.fightMailInfos = {}
		end
		self.mailBox.fightMailInfos[fightMailInfo.id] = fightMailInfo
		if self.mailBox.fightNotReadCount then
			self.mailBox.fightNotReadCount = self.mailBox.fightNotReadCount + 1
		else
			self.mailBox.fightNotReadCount = 1
		end
		if self.mailBox.fightAllCount then
			self.mailBox.fightAllCount = self.mailBox.fightAllCount + 1
		else
			self.mailBox.fightAllCount = 1
		end
		self:luaDispatchEvent(self.EVT_TYPE_RECEIVE_MAIL .. msgType,fightMailInfo) -- 把数据fightMailInfo传过去
	elseif msgType == 3 then
		local sendMailInfo = require("src.app.model.mail.mailInfoBox.CommonAndSendMailInfo")()
		sendMailInfo:decode(data)
		if self.mailBox.sendMailInfos == nil then
			self.mailBox.sendMailInfos = {}
		end
		self.mailBox.sendMailInfos[sendMailInfo.id] = sendMailInfo
		if self.mailBox.sendAllCount then
			self.mailBox.sendAllCount = self.mailBox.sendAllCount + 1
		else
			self.mailBox.sendAllCount = 1
		end
		self:luaDispatchEvent(self.EVT_TYPE_RECEIVE_MAIL .. msgType,sendMailInfo) -- 把数据sendMailInfo传过去
	end

	--- 派发事件
	self:luaDispatchEvent(self.EVT_TYPE_RECEIVE_MAIL)

end

--- 手动添加侦查报告信息
function T:addScoutInfo(scoutData)
    local fightMailInfo = scoutData

    if self.mailBox.fightMailInfos == nil then
	    self.mailBox.fightMailInfos = {}
    end
    self.mailBox.fightMailInfos[fightMailInfo.id] = fightMailInfo -- 是已经读了的
    if self.mailBox.fightAllCount then
	    self.mailBox.fightAllCount = self.mailBox.fightAllCount + 1
    else
	    self.mailBox.fightAllCount = 1
    end


    self:luaDispatchEvent(self.EVT_TYPE_RECEIVE_MAIL .. 2,fightMailInfo)
    
end


--- 删除所有邮件：根据index,1,2,3
function T:deleteAllMailByIndex(index)
	local mailBox = self.mailBox
	local mailContentBox = self.mailContentBox
	if index == 1 then
		mailBox.commonMailInfos = nil
		mailContentBox.commonMailContentBox.commonMailContents = {}
		mailBox.commonBool = nil
		mailBox.commonNotReadCount = nil
		mailBox.commonAllCount = nil
	elseif index == 2 then
		mailBox.fightMailInfos = nil
		mailContentBox.mailFightReportContentBox.mailFightReportContents = {}
		mailBox.fightBool = nil
		mailBox.fightNotReadCount = nil
		mailBox.fightAllCount = nil
	elseif index == 3 then
		mailBox.sendMailInfos = nil
		mailContentBox.sendMailContentBox.sendMailContents = {}
		mailBox.sendBool = nil
		mailBox.sendNotReadCount = nil
		mailBox.sendAllCount = nil
	end

	--- 派发事件
	self:luaDispatchEvent(self.EVT_TYPE_DELETE_ALL_MAIL .. index)
	self:luaDispatchEvent(self.EVT_TYPE_DELETE_ALL_MAIL)
end

--- 清除所有邮箱数据
function T:clearAllMailData()
	local mailBox = self.mailBox
	local mailContentBox = self.mailContentBox

	mailBox.commonMailInfos = nil
	mailContentBox.commonMailContentBox.commonMailContents = {}
	mailBox.commonBool = nil
	mailBox.commonNotReadCount = nil
	mailBox.commonAllCount = nil

	mailBox.fightMailInfos = nil
	mailContentBox.mailFightReportContentBox.mailFightReportContents = {}
	mailBox.fightBool = nil
	mailBox.fightNotReadCount = nil
	mailBox.fightAllCount = nil

	mailBox.sendMailInfos = nil
	mailContentBox.sendMailContentBox.sendMailContents = {}
	mailBox.sendBool = nil
	mailBox.sendNotReadCount = nil
	mailBox.sendAllCount = nil

end




--- 批量删除邮件,这个不包括侦查邮件
function T:batchDeleteMail(index,ids)
    for i = 1,#ids,1 do
        self:deleteOneMailByIndexAndId(index,ids[i])
    end
end

--- 删除侦查邮件
function T:batchDeleteScoutMail(ids)
    for i = 1,#ids,1 do
        self:deleteOneMailByIndexAndId(2,ids[i])
    end

end


--- 删除某一封邮件
function T:deleteOneMailByIndexAndId(index,id)
	local mailBox = self.mailBox
	local mailContentBox = self.mailContentBox
	if index == 1 then
		if mailBox.commonMailInfos[id].isRead == false and mailBox.commonNotReadCount then
			mailBox.commonNotReadCount = mailBox.commonNotReadCount - 1
			if mailBox.commonNotReadCount <= 0 then
				mailBox.commonNotReadCount = nil
			end
		end
		if mailBox.commonAllCount then
			mailBox.commonAllCount = mailBox.commonAllCount - 1
			if mailBox.commonAllCount <= 0 then
				mailBox.commonAllCount = nil
			end
		end
		mailBox.commonMailInfos[id] = nil
		mailContentBox.commonMailContentBox.commonMailContents[id] = nil
	elseif index == 2 then
		if mailBox.fightMailInfos[id].isRead == false and mailBox.fightNotReadCount then
			mailBox.fightNotReadCount = mailBox.fightNotReadCount - 1
			if mailBox.fightNotReadCount <= 0 then
				mailBox.fightNotReadCount = nil
			end
		end
		if mailBox.fightAllCount then
			mailBox.fightAllCount = mailBox.fightAllCount - 1
			if mailBox.fightAllCount <= 0 then
				mailBox.fightAllCount = nil
			end
		end

		mailBox.fightMailInfos[id] = nil
		mailContentBox.mailFightReportContentBox.mailFightReportContents[id] = nil
	elseif index == 3 then
		if mailBox.sendMailInfos[id].isRead == false and mailBox.sendNotReadCount then
			mailBox.sendNotReadCount = mailBox.sendNotReadCount - 1
			if mailBox.sendNotReadCount <= 0 then
				mailBox.sendNotReadCount = nil
			end
		end
		if mailBox.sendAllCount then
			mailBox.sendAllCount = mailBox.sendAllCount - 1
			if mailBox.sendAllCount <= 0 then
				mailBox.sendAllCount = nil
			end
		end

		mailBox.sendMailInfos[id] = nil
		mailContentBox.sendMailContentBox.sendMailContents[id] = nil
	end

	--- 派发事件
	self:luaDispatchEvent(self.EVT_TYPE_DELETE_ONE_MAIL .. index .. id)
	self:luaDispatchEvent(self.EVT_TYPE_DELETE_ONE_MAIL)
end

--- 查看邮件
function T:readMail(index,id,data)
	local mailBox = self.mailBox
	local mailContentBox = self.mailContentBox
	if index == 1 then
		if mailBox.commonMailInfos[id].isRead == false then
			mailBox.commonNotReadCount = mailBox.commonNotReadCount - 1
		end
		mailContentBox.commonMailContentBox:decode(id,data)
		mailBox.commonMailInfos[id].isRead = true

	elseif index == 2 then
		if mailBox.fightMailInfos[id].isRead == false then
			mailBox.fightNotReadCount = mailBox.fightNotReadCount - 1
		end
		mailContentBox.mailFightReportContentBox:decode(id,data)
		mailBox.fightMailInfos[id].isRead = true

	elseif index == 3 then
		if mailBox.sendMailInfos[id].isRead == false then
			mailBox.sendNotReadCount = mailBox.sendNotReadCount - 1
		end
		mailContentBox.sendMailContentBox:decode(id,data)
		mailBox.sendMailInfos[id].isRead = true

	end

	--- 派发事件
	self:luaDispatchEvent(self.EVT_TYPE_READ_MAIL .. index .. id)
	self:luaDispatchEvent(self.EVT_TYPE_READ_MAIL)
end

--- 得到未读邮件的数量
function T:getNotReadCount()
	local mailBox = self.mailBox
	local count = 0
	if mailBox.commonNotReadCount then
		count = mailBox.commonNotReadCount
	end

	if mailBox.fightNotReadCount then
		count = count + mailBox.fightNotReadCount
	end

	if mailBox.sendNotReadCount then
		count = count + mailBox.sendNotReadCount
	end

	-- 如果返回0则表示没有邮件可读
	return count
end

--- 判断是否有读的邮件，true:有未读的，false：没有未读的
function T:judgeMailIsRead()
	local commonNotReadCount = self.mailBox.commonNotReadCount
	local fightNotReadCount = self.mailBox.fightNotReadCount

	if commonNotReadCount and commonNotReadCount > 0 then
		return true
	end

	if fightNotReadCount and fightNotReadCount > 0 then
		return true
	end


	return false
end

--- 判断普通邮件是否有读的
function T:judgeNormalIsRead()
    local commonNotReadCount = self.mailBox.commonNotReadCount
	if commonNotReadCount and commonNotReadCount > 0 then
		return true
	end

	return false
end

--- 判断报告邮件是否有读的
function T:judgeFightMailIsRead()
	local fightNotReadCount = self.mailBox.fightNotReadCount

	if fightNotReadCount and fightNotReadCount > 0 then
		return true
	end
    
	return false
end


return T
