--[[
    邮箱盒子:主要是邮箱的简略信息
]]
local T={}

---普通邮件和发送邮件的信息
T.CommonAndSendMailInfo = require "src.app.model.mail.mailInfoBox.CommonAndSendMailInfo"

--- 战报邮件的信息
T.FightMailInfo = require "src.app.model.mail.mailInfoBox.FightMailInfo"

--- 普通邮件
T.commonMailInfos = nil
--- 普通邮件是否有后10条的bool值
T.commonBool = nil
--- 普通邮件未读数量
T.commonNotReadCount = nil
--- 总数量
T.commonAllCount = nil

--- 战报邮件
T.fightMailInfos = nil
--- 战报邮件是否有后10条的bool值,false表示没有战报可以获取了，如果为true的话，表示下面还有邮件可以获取
T.fightBool = nil
--- 战报邮件未读数量，如果为0的话，表示未读邮件为0条，否则有未读的邮件
T.fightNotReadCount = nil
--- 总数量
T.fightAllCount = nil

--- 发送邮件
T.sendMailInfos = nil    
--- 发送邮件是否有后10条的bool值
T.sendBool = nil
--- 发送邮件未读数量
T.sendNotReadCount = nil
--- 总数量
T.sendAllCount = nil

--- tag:请求标记：如果是1：普通邮件，2：战报邮件，3：发送邮件，4：所有邮件（登陆时候请求的）
T.tag = nil

function T:decode(data,tag)
    self.tag = tag
    if tag == 1 then
        self:decodeCommonData(data)
    elseif tag == 2 then
        self:decodeFightData(data)
    elseif tag == 3 then
        self:decodeSendData(data)
    elseif tag == 4 then
        self:decodeLoginData(data)
        MailManager:luaDispatchEvent(MailManager.EVT_TYPE_FIRST_GET_MAIL_LIST)
    end
end

--- 解码登陆数据
function T:decodeLoginData(data)
    -- 普通邮件
    local commonLen = data:readUnsignedByte()
    local commonMailInfos = {}
    if self.commonMailInfos then -- 防止接下来的10条数据覆盖前面的数据，而是必须拼接到后面
        commonMailInfos = self.commonMailInfos
    end
	
	for i=1,commonLen,1 do
		local commonMailInfo = self.CommonAndSendMailInfo()
		commonMailInfo:decode(data,1)
		commonMailInfos[commonMailInfo.id] = commonMailInfo
	end
	self.commonMailInfos = commonMailInfos
    local commonBool = data:readBoolean()
    if self.commonBool == nil then -- 表明是第一次登陆的时候获取的数据
        self.commonBool = commonBool
    else -- 表明是断线重连，这个时候不刷新这个bool值，还是用原来那个值
        
    end

    -- 如果是获取接下来的10条数据，则不会解析这里的
    if self.tag == 4 then
        self.commonNotReadCount = data:readUnsignedByte()   -- 登陆的时候就把所有的发过来
        self.commonAllCount = data:readUnsignedByte()
    end

    -- 战报邮件
    local len = data:readUnsignedByte()
	local fightMailInfos = {}
    if self.fightMailInfos then
        fightMailInfos = self.fightMailInfos
    end
	for i=1,len,1 do
		local fightMailInfo = self.FightMailInfo()
		fightMailInfo:decode(data)
		fightMailInfos[fightMailInfo.id] = fightMailInfo
	end
	self.fightMailInfos = fightMailInfos
    local fightBool = data:readBoolean()
    if self.fightBool == nil then
        self.fightBool = fightBool
    end

    if self.tag == 4 then
        self.fightNotReadCount = data:readUnsignedByte()
        self.fightAllCount = data:readUnsignedByte()
    end

    -- 发件箱信息
    local Len3 = data:readUnsignedByte()
	local sendMailInfos = {}
    if self.sendMailInfos then
        sendMailInfos = self.sendMailInfos
    end
	for i=1,Len3,1 do
		local sendMailInfo = self.CommonAndSendMailInfo()
		sendMailInfo:decode(data)
		sendMailInfos[sendMailInfo.id] = sendMailInfo
	end
	self.sendMailInfos = sendMailInfos
    local sendBool = data:readBoolean()
    if self.sendBool == nil then
        self.sendBool = sendBool
    end

    if self.tag == 4 then
        self.sendNotReadCount = data:readUnsignedByte()
        self.sendAllCount = data:readUnsignedByte()
    end


end


function T:decodeCommonData(data)
    -- 普通邮件
    local commonLen = data:readUnsignedByte()
    local commonMailInfos = {}
    if self.commonMailInfos then -- 防止接下来的10条数据覆盖前面的数据，而是必须拼接到后面
        commonMailInfos = self.commonMailInfos
    end
	
	for i=1,commonLen,1 do
		local commonMailInfo = self.CommonAndSendMailInfo()
		commonMailInfo:decode(data,1)
		commonMailInfos[commonMailInfo.id] = commonMailInfo
	end
	self.commonMailInfos = commonMailInfos
    self.commonBool = data:readBoolean()
    -- 如果是获取接下来的10条数据，则不会解析这里的
    if self.tag == 4 then
        self.commonNotReadCount = data:readUnsignedByte()   -- 登陆的时候就把所有的发过来
        self.commonAllCount = data:readUnsignedByte()
    end
    MailManager:luaDispatchEvent(MailManager.EVT_TYPE_GET_TEN_MAIL .. 1)
    MailManager:luaDispatchEvent(MailManager.EVT_TYPE_GET_TEN_MAIL)
end

function T:decodeFightData(data)
    -- 战报邮件
    local len = data:readUnsignedByte()
	local fightMailInfos = {}
    if self.fightMailInfos then
        fightMailInfos = self.fightMailInfos
    end
	for i=1,len,1 do
		local fightMailInfo = self.FightMailInfo()
		fightMailInfo:decode(data)
		fightMailInfos[fightMailInfo.id] = fightMailInfo
	end
	self.fightMailInfos = fightMailInfos
    self.fightBool = data:readBoolean()

    if self.tag == 4 then
        self.fightNotReadCount = data:readUnsignedByte()
        self.fightAllCount = data:readUnsignedByte()
    end
    
    MailManager:luaDispatchEvent(MailManager.EVT_TYPE_GET_TEN_MAIL .. 2)
    MailManager:luaDispatchEvent(MailManager.EVT_TYPE_GET_TEN_MAIL)
end

function T:decodeSendData(data)
    -- 发件箱信息
    local Len3 = data:readUnsignedByte()
	local sendMailInfos = {}
    if self.sendMailInfos then
        sendMailInfos = self.sendMailInfos
    end
	for i=1,Len3,1 do
		local sendMailInfo = self.CommonAndSendMailInfo()
		sendMailInfo:decode(data)
		sendMailInfos[sendMailInfo.id] = sendMailInfo
	end
	self.sendMailInfos = sendMailInfos
    self.sendBool = data:readBoolean()

    if self.tag == 4 then
        self.sendNotReadCount = data:readUnsignedByte()
        self.sendAllCount = data:readUnsignedByte()
    end

    MailManager:luaDispatchEvent(MailManager.EVT_TYPE_GET_TEN_MAIL .. 3)
    MailManager:luaDispatchEvent(MailManager.EVT_TYPE_GET_TEN_MAIL)
end


return T
