--- 服务器列表管理器
local T=Class.ServerListMgr(LuaEventDispatcher2)

--- 服务器状态-推荐
T.SERVER_STATE_RECOM=1

--- 服务器状态-繁忙
T.SERVER_STATE_BUSY=2

--- 事件-获取服务器列表完成
T.EVT_TYPE_GET_SERVER_LIST_COMPLETE="evt_type_get_server_list_complete"

--- 事件-开始lua代码及资源更新
T.EVT_TYPE_UPDATE_START="evt_type_update_start"

function T:def()
	LuaEventDispatcher2.def(self)

	--- 服务器列表
	-- _serverList

	--- 唯一ID
	-- _uniqueId
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 从服务器获取服务器列表数据
function T:getServerListData()
	local isNeedSign=true

	if CUR_SDK == nil then
		local uniqueId=DataStorage:getUniqueId()
		if uniqueId and string.trim(uniqueId) ~= "" then
			isNeedSign=false
		end
	elseif CUR_SDK == SDKType.PLATFORM_360 then
		isNeedSign=false
	end

	require("src.app.controler.loading.GetServerListCommand")(
		CFG.CPP_VERSION,--cpp的版本号
		CFG:getVersion(),--lua代码及资源的版本号
		GameCfg.IS_STAGING,
		isNeedSign,
		function(serverListData)
			if serverListData then
				self:serverListHandler(serverListData)
			else
				local message=string.format("get server list is empty")

				__print__("GetServerListErr",message)
				LogRecorder:push(message,LOG_LEVEL_ERROR,"GetServerListErr")

				MUIAlter.show({
					info = Language:get("ug_getServerListErr1"),
					size = 30,
					callback = function(flag)
						Cocos2dxUtils:timeOut(function()
							self:getServerListData()
						end,0)
					end
				})
			end
		end,
		function(code,errInfo)
			local message=string.format("get server list failure, code=%s , errInfo=%s",code,errInfo)

			__print__("GetServerListErr",message)
			LogRecorder:push(message,LOG_LEVEL_ERROR,"GetServerListErr")

			MUIAlter.show({
				info = Language:get("ug3"),
				size = 30,
				callback = function(flag)
					Cocos2dxUtils:timeOut(function()
						self:getServerListData()
					end,0)
				end
			})
		end
	)
end

--- 处理服务器列表请求返回的数据
function T:serverListHandler(serverListData)
	if serverListData.uniqueId and string.trim(serverListData.uniqueId) ~= "" then
		DataStorage:putUniqueId(serverListData.uniqueId)
	end
	local notice=serverListData.notice

	local isNotice=notice and string.trim(notice) ~= ""
	local isUpdateCpp=ResUpdate:compareVersion(serverListData.cppVersion, CFG.CPP_VERSION) > 0

	--如果有公告,显示公告,如果需要更新cpp代码,则显示完公告后,进入cpp代码更新
	if isNotice then
		MUIAlter.show({
			info = notice,
			size = 30,
			callback=function()
				if isUpdateCpp then
					self.updateCpp(serverListData.cppUpdateUrl)
				end
			end
		})
	end

	--如果需要更新cpp代码,且没有显示公告,则进入cpp代码更新
	if isUpdateCpp then
		if not isNotice then
			self.updateCpp(serverListData.cppUpdateUrl)
		end
	else
		local sl=serverListData.serverList
		if not sl or # sl == 0 then
			MUIAlter.show({
				info = Language:get("ug_getServerListErr1"),
				size=30,
				callback=function()
					Cocos2dxUtils:timeOut(function()
						self:getServerListData()
					end,0)
				end
			})

			return
		end
		self._serverList=sl

		local uL=serverListData.updateList
		if uL and #uL > 0 then
			print("更新lua代码及资源")

			self:updateLua(uL)
		else
			self:luaDispatchEvent(self.EVT_TYPE_GET_SERVER_LIST_COMPLETE)
		end
	end
end

---  更新cpp代码
function T.updateCpp(url)
	if not url or string.trim(url) == "" then
		url=GameCfg.CPP_UPDATE_URL
	end

	MUIAlter.show({
		info = Language:get("ug_needUpdateCpp"),
		size=30,
		callback=function()
			if SDK_Delegate and SDK_Delegate.doOpenUrl then
				SDK_Delegate:doOpenUrl(url)

				return
			end

			OpenURL:getInstance():open(url)
		end
	})
end

--- 更新lua代码及资源
function T:updateLua(updateList)
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_START)

	self:rgUpdateListeners()

	ResUpdate:execUpdate(updateList)
end

--- 注册更新的相关侦听器
function T:rgUpdateListeners()
	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_ERROR,self,self.updateErrorHandler)
	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_COMPLETE,self,self.updateCompleteHandler)
end

--- 移除更新的相关侦听器
function T:rmUpateListeners()
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_ERROR,self,self.updateErrorHandler)
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_COMPLETE,self,self.updateCompleteHandler)
end

--- 更新错误
function T:updateErrorHandler(errCode)
	local errTip
	if errCode == ResUpdate.ERR_CODE_CREATE_FILE then
		errTip=Language:get("ug_updatingCreateFileErr")
	elseif errCode == ResUpdate.ERR_CODE_NETWORK then
		errTip=Language:get("ug_updatingNetworkErr")
	elseif errCode == ResUpdate.ERR_CODE_NO_NEW_VERSION then
		errTip=Language:get("ug_updatingVersionErr")
	elseif errCode ==ResUpdate.ERR_CODE_UNCOMPRESS then
		errTip=Language:get("ug_updatingUncompressErr")
	else
		errTip=Language:get("ug_updatingUnknowErr")
	end

	MUIAlter.show({
		info=errTip,
		callback=function()
			ResUpdate:update()
		end
	})
end

--- 更新完成
function T:updateCompleteHandler()
	self:rmUpateListeners()

	self:luaDispatchEvent(self.EVT_TYPE_GET_SERVER_LIST_COMPLETE)
end

--- 获取唯一ID
function T:getUniqueId()
	return DataStorage:getUniqueId()
end

--- 获取服务器列表
function T:getServerList()
	return self._serverList
end

--- 获取默认(推荐)服务器数据
function T:getDefaultServerData()
	--id最大的服务器
	local maxIdV
	for k,v in pairs(self._serverList) do
		if v.state == self.SERVER_STATE_RECOM then
			return v
		end

		if not maxIdV or maxIdV.id < v.id then
			maxIdV=v
		end
	end

	return maxIdV
end

--- 获取指定服务器数据
function T:getServerData(id)
	for k,v in pairs(self._serverList) do
		if v.id == id then
			return v
		end
	end

	return self:getDefaultServerData()
end

return T
