--[[
    社交管理
]]
local T = extend(LuaEventDispatcher)

--- 事件-在好友列表框中选择了某一个好友的事件
T.EVT_ONE_FRIEND_REFRESH = "evt_one_friend_refresh"

--- 社交盒子
T.SocialBox = require "src.app.model.socialContact.SocialBox"

--- 社交信息项
T.SocialInfo = require "src.app.model.socialContact.SocialInfo"


--- 好友盒子
T.friendBox = mapping({}, T.SocialBox)

--- 好友最大数量
T.MAX_FRIEND_COUNT = 30

--- 事件，添加好友
T.EVT_ADD_FRIEND_REFRESH = "evt_add_friend_refresh"

--- 事件，删除好友
T.EVT_DELETE_FRIEND_REFRESH = "evt_delete_friend_refresh"



--- 黑名单盒子
T.blackBox = mapping({}, T.SocialBox)

--- 黑名单最大数量
T.MAX_BLACK_COUNT = 30

--- 事件，添加黑名单
T.EVT_ADD_BLACK_REFRESH = "evt_add_black_refresh"

--- 事件，删除黑名单
T.EVT_DELETE_BLACK_REFRESH = "evt_delete_black_refresh"



--- 得到好友列表
function T:getFriendList()
    return self.friendBox.socialList
end

--- 得到黑名单列表
function T:getBlackList()
    return self.blackBox.socialList
end



--- 判断好友数量是否超过最大数量:true:表示超过了最大数量
function T:judgeFriendReachMax()
    local friendList = self:getFriendList()
    local i = 0 
    for k,v in pairs(friendList) do
        if v then
            i = i + 1
        end
    end

    if i < self.MAX_FRIEND_COUNT then
        return false
    else
        return true  -- 超过了最大数量，不能添加好友了
    end
end

-- 判断好友是否好友列表中:true:表示在好友列表中
function T:judgeIsAtFrendList(id)
    local friendList = self:getFriendList()
    for k,v in pairs(friendList) do
        if  v and v.id == id then
            return true
        end
    end
    return false
end

--- 添加好友,如果在黑名单，从黑名单中删除
function T:addFriend(id,name)
    local friendList = self:getFriendList()

    local info = mapping({}, self.SocialInfo)
    info.id = id
    info.name = name
    friendList[info.id] = info

    if self:judgeIsAtBlackList(id) then
        self:deleteBlack(id)
    end

    self:luaDispatchEvent(self.EVT_ADD_FRIEND_REFRESH,info)
end


--- 删除好友
function T:deleteFriend(id)
    local friendList = self:getFriendList()
    if friendList[id] then
        friendList[id] = nil
        self:luaDispatchEvent(self.EVT_DELETE_FRIEND_REFRESH)
    end
    
end

--- 得到好友的数量
function T:getFriendCount()
    local friendList = self:getFriendList()
    local i = 0 
    for k,v in pairs(friendList) do
        if v then
            i = i + 1
        end
    end

    return i
end

-------------------------------------------------------------------------------------

--- 判断黑名单数量是否超过最大数量:true:表示超过了最大数量
function T:judgeBlackReachMax()
    local blackList = self:getBlackList()
    local i = 0 
    for k,v in pairs(blackList) do
        if v then
            i = i + 1
        end
    end

    if i < self.MAX_BLACK_COUNT then
        return false
    else
        return true 
    end
end

-- 判断黑名单是否在黑名单列表中:true:表示在黑名单列表中
function T:judgeIsAtBlackList(id)
    local blackList = self:getBlackList()
    for k,v in pairs(blackList) do
        if  v and v.id == id then
            return true
        end
    end
    return false
end
--- 添加黑名单:如果是好友则删除
function T:addBlack(id,name)
    local blackList = self:getBlackList()

    local info = mapping({}, self.SocialInfo)
    info.id = id
    info.name = name
    blackList[info.id] = info

    if self:judgeIsAtFrendList(id) then
        self:deleteFriend(id)
    end
    self:luaDispatchEvent(self.EVT_ADD_BLACK_REFRESH,info)
end


--- 删除黑名单
function T:deleteBlack(id)
    local blackList = self:getBlackList()
    if blackList[id] then
        blackList[id] = nil
        self:luaDispatchEvent(self.EVT_DELETE_BLACK_REFRESH)
    end
    
end

--- 得到黑名单的数量
function T:getBlackCount()
    local blackList = self:getBlackList()
    local i = 0 
    for k,v in pairs(blackList) do
        if v then
            i = i + 1
        end
    end

    return i
end

return T