--- 补给线管理
local T = Class.SupplyLineManager(LuaEventDispatcher2)

T.SupplyLineInfo = require("src.app.model.supplyLine.SupplyLineInfo")

-- 事件--关卡重置
T.EVT_RESET_ALL_POINT = "evt_reset_all_point"

-- 事件--补给线完成
T.EVT_SUPPLY_LINE_FINISH = "evt_supply_line_finish"

-- 事件--星级数量改变
T.EVT_STARNUN_CHANGE = "evt_starnum_change"


function T:def()
    LuaEventDispatcher2.def(self)

    self.supplyLineInfos = nil -- 补给线信息

end

function T:decode(data)
    self.energyCostMark = data:readInt() --  能量消耗标记,按照sid移位操作
    self.supplyLineMark = data:readInt() --  补给线完成标记,按照sid移位操作

    local infos = {}
    local length = data:readShort()
    for i = 1,length,1 do
        local supplyLineInfo = self.SupplyLineInfo()
        supplyLineInfo:decode(data)
        infos[supplyLineInfo.sid] = supplyLineInfo
    end

    self.supplyLineInfos = infos

end

--- 得到所有补给线信息
function T:getSupplyLineInfos()
    return self.supplyLineInfos
end


--- 当前选择的战役
T._selCampaign = nil
function T:setSelCampaign(value)
	self._selCampaign = value
end
function T:getSelCampaign()
	return self._selCampaign
end


--- 根据sid得到某一项的信息
function T:getSupplyInfoBySid(sid)
    local supplyLineInfos = self:getSupplyLineInfos()
    if supplyLineInfos[sid] then
        return supplyLineInfos[sid]
    else
        local infos = {}
        infos.sid = sid
        infos.starNum = 0
        
        self.supplyLineInfos[infos.sid] = infos

        return infos
    end

end

--- 根据sid设置某一项的星级信息
function T:setSupplyInfoBySid(sid,starNum)
    local supplyLineInfos = self:getSupplyLineInfos()
    if supplyLineInfos[sid] then
        supplyLineInfos[sid].starNum = starNum
    else
        local infos = {}
        infos.sid = sid
        infos.starNum = starNum
        
        self.supplyLineInfos[infos.sid] = infos

    end
    self:luaDispatchEvent(self.EVT_STARNUN_CHANGE)
    self:luaDispatchEvent(self.EVT_STARNUN_CHANGE .. sid)

end

--- 根据sid判断是否有能量消耗
function T:judgeEnergyIsCostBySid(sid)
    local energyCostMark = self.energyCostMark
    return BitOperator:_and(energyCostMark, BitOperator:_ls(1, sid)) ~= 0  -- true表示能量消耗了，false表示未消耗
end

--- 根据sid设置能量已经消耗完成
function T:setEnergyCostFinishBySid(sid)
	self.energyCostMark = BitOperator:_or(self.energyCostMark,BitOperator:_ls(1, sid))
end


--- 根据sid判断补给线是否完成
function T:judgeSupplyIsFinishBySid(sid)
    local supplyLineMark = self.supplyLineMark
    return BitOperator:_and(supplyLineMark, BitOperator:_ls(1, sid)) ~= 0  -- true表示完成了，false表示未完成
end

--- 根据sid设置补给线完成
function T:setSupplyFinishBySid(sid)
    self.supplyLineMark = BitOperator:_or(self.supplyLineMark,BitOperator:_ls(1, sid))
    self:luaDispatchEvent(self.EVT_SUPPLY_LINE_FINISH .. sid)
    self:luaDispatchEvent(self.EVT_SUPPLY_LINE_FINISH)
end

--- 得到可以攻打的所有关卡数,这里要判断是否解锁
function T:getAllPointCount()
    local allPoint = 0
    for i = 1,11,1 do
        local playerLevel = UserInfo.level
        if playerLevel >= SupplyLine_temp[i].openLevel then
            if self:judgeSupplyIsFinishBySid(i) == false then
                allPoint = allPoint + 1
            end
        end
    end

    return allPoint
end

-- 关卡重置
function T:resetAllPoint()
    self.supplyLineMark = 0
    DailyStates:addSupplyLineBuyNum(1)
    self:luaDispatchEvent(self.EVT_RESET_ALL_POINT)
end


--- 得到重置的钻石价格
function T:getResetAllPointGems()
    return 200
end

--- 得到剩余的次数
function T:getRemainCount()
    local num = DailyStates:getSupplyLineBuyNum()
    local vipNum = VipManager:getSupplyLineBuyNumLimit()
    return vipNum - num
end

--- 判断是否有关卡可以攻打，true:可以，false:不可以
function T:judgeIsAttack()
    local num = self:getAllPointCount()
    if num > 0 then
        return true
    else
        return false
    end

end


return T