--- 配件抽卡
local T = Class.AccessoryDrawCardState(LuaEventDispatcher2)

--- 事件--免费结束时间改变
T.EVT_FREETIME_ENDTIME_REFRESH = "evt_freetime_endtime_refresh"

--- 事件--每秒时间改变刷新
T.EVT_TIME_CHANGE_REFRESH = "evt_time_change_refresh"

--- 事件--免费时间结束
T.EVT_FREETIME_OVER = "evt_freetime_over"


--- 这个免费结束时间意思是在结束时间之内不能免费抽取，在时间之内只能用钻石抽取
function T:def()
    LuaEventDispatcher2.def(self)

    self.MARKLIST = { 132,133,134 }  -- 配件抽卡的sid为1，2，3

    self.freeDrawDiamond = { 20,80,360 }


    --- 状态列表
	self.list = {}
end


function T:setList(list)
    self.list = list

    self:startUpdateCheck()

end

--- 根据sid得到配件抽奖的免费时间
function T:getFreeTimeBySid(sid)
    local freeTime = self.list[self.MARKLIST[sid]] or 0

    return GameUtils:modifyTimeZone(freeTime)
end

--- 设置免费结束时间根据sid;也是抽取一次刷新一次数据，也是这里
function T:setFreeTimeBySid(sid,freeTime)
    self.list[self.MARKLIST[sid]] = freeTime

    self:startUpdateCheck()

    self:luaDispatchEvent(self.EVT_FREETIME_ENDTIME_REFRESH)
    self:luaDispatchEvent(self.EVT_FREETIME_ENDTIME_REFRESH .. sid)

end

--- 得到剩余时间
function T:getRemainTime(sid)
    local endTimeSec = self:getFreeTimeBySid(sid)
    local syncSec = TimeUtils:getSyncMillis()/1000
    return endTimeSec - syncSec
end


function T:startUpdateCheck()
	local function update()
        for i = 1,3,1 do
            local endTimeSec = self:getFreeTimeBySid(i)
            local syncSec = TimeUtils:getSyncMillis()/1000
            if endTimeSec <= syncSec then  --结束
                self:luaDispatchEvent(self.EVT_FREETIME_OVER)
                self:luaDispatchEvent(self.EVT_FREETIME_OVER .. i)
                self:tryStopUpdate()
            else
                self:luaDispatchEvent(self.EVT_TIME_CHANGE_REFRESH)
                self:luaDispatchEvent(self.EVT_TIME_CHANGE_REFRESH .. i)
            end

        end
	end

	if self.updateId then
		return
	end

    if self:judgeIsHasRemain() then
        self.updateId = Cocos2dxUtils:interval(update,1)
    end
end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
        if self:judgeIsHasRemain() == false then
            Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId = nil
        end
	end
end

--- 判断所有的结束时间是否有免费剩余时间，如果有至少1个则返回true
function T:judgeIsHasRemain()
    local judge = false
    for i = 1,3,1 do
        if self:judgeIsFree(i) == false then
            judge = true
            break
        end

    end    
    return judge
end



--- 判断是否免费抽取,true:可以免费抽取，否则不能
function T:judgeIsFree(sid)
    local syncSec = TimeUtils:getSyncMillis()/1000 -- 这个是纠正了的
    if syncSec >= self:getFreeTimeBySid(sid) then -- 结束后就可以免费抽取了
        return true
    else
        return false -- 有时间在走
    end
end

--- 得到抽取消耗的钻石数目
function T:getCostDiamond(sid)
    return self.freeDrawDiamond[sid]
end


--- 判断是否可以10连抽:只有第3档次可以10连抽取奖品,并且免费不能10连抽
function T:judgeIsTenDraw(sid)
    if sid == 3 and self:judgeIsFree(3) == false then
        return true
    else
        return false
    end
end

--- 得到10连抽的钻石
function T:getTenDrawDiamond()
    return 3240
end


--- 得到品质描述
function T:getQualityDes(sid)   
    if sid == 1 then
        return Language:get("uc726")
    elseif sid == 2 then
        return Language:get("uc727")
    elseif sid == 3 then
        return Language:get("uc728")
    end

end

--- 主界面感叹号显示,true:有感叹号，false则没有
function T:judgeHomePage()
    local judge = false
    for i = 1,3,1 do
        if self:judgeIsFree(i) == true then
            judge = true
            break
        end

    end    
    return judge
end


return T
