--- 状态列表(永久状态)
local T = Class.SetStates(LuaEventDispatcher2)

--- 事件类型-每日状态被更新
T.EVT_TYPE_UPDATE_STATES="evt_type_update_states"

--- 事件类型-竞技场奖励领取的时间被更新
T.EVT_TYPE_UPDATE_ARENA_AWARD_GET_Y_DAY="evt_type_update_arena_award_get_y_day"

--- 自定义名字标记
T.CUSTOM_NAME=126

--- 竞技场中上一次挑战失败的时间
T.ARENA_FAIL_TIME=122

--- 竞技场奖励领取的时间相对于今年的第n天
T.ARENA_AWARD_GET_Y_DAY=123

function T:def()
	LuaEventDispatcher2.def(self)

    self.list = {} -- 状态值表

    --- 付费提示设置
    self.setPayState = require("src.app.model.systemSet.SetPayState")()
    --- 通知提示
    self.setNoticeState = require("src.app.model.systemSet.SetNoticeState")()
    --- 普通设置
    self.setCommonState = require("src.app.model.systemSet.SetCommonState")()
    --- 礼物状态
    self.giftState = require("src.app.model.systemSet.GiftState")()
    --- 签到
    self.signManager = require("src.app.model.allActivity.sign.SignManager")()
    --- 援军状态
    self.reinforceState = require("src.app.model.systemSet.ReinforceState")()
    --- 统率（指挥）
    self.dominanceLuckyState = require("src.app.model.systemSet.DominanceLuckyState")()
    --- 额外的建造队列数量
    self.buildQueueNumState = require("src.app.model.systemSet.BuildQueueNumState")()
    --- 免费幸运抽奖
    self.luckyLotteryState = require("src.app.model.systemSet.LuckyLotteryState")()
    --- 商品的3倍充值标记
    self.threePurchaseState = require("src.app.model.systemSet.ThreePurchaseState")()
    --- 加入军团奖励
    self.joinAllianceAward = require("src.app.model.systemSet.JoinAllianceAward")()
    --- 声望领取
    self.reputationState = require("src.app.model.systemSet.ReputationState")()
    --- 配件抽卡
    self.accessoryDrawCardState = require("src.app.model.systemSet.AccessoryDrawCardState")()
    --- facebookshare
    self.faceBookShareAward = require("src.app.model.systemSet.FaceBookShareAward")()
    --- 荣誉勋章领取
    self.getMedalAward = require("src.app.model.systemSet.GetMedalAward")()
    --- 充值档位奖励领取标记
    self.reChargeState = require("src.app.model.systemSet.ReChargeState")()

end

--- 解析状态
function T:decode(data)
    local len = data:readUnsignedShort()

    local list = { }
    for i = 1, len, 2 do
        local key = data:readUnsignedShort()
        local value = data:readInt()

        list[key] = value
    end

    self.list = list


    self.setPayState:setList(list)
    self.setNoticeState:setList(list)
    self.setCommonState:setList(list)
    self.giftState:setList(list)
    self.signManager:setList(list)
    self.reinforceState:setList(list)
    self.dominanceLuckyState:setList(list)
    self.buildQueueNumState:setList(list)
    self.luckyLotteryState:setList(list)
    self.threePurchaseState:setList(list)
    self.joinAllianceAward:setList(list)
    self.reputationState:setList(list)
    self.accessoryDrawCardState:setList(list)
    self.faceBookShareAward:setList(list)
    self.getMedalAward:setList(list)
    self.reChargeState:setList(list)

end

function T:getList()
    return self.list
end

--- 设置是否自定义名字
function T:setIsCustomName(value)
	self.list[self.CUSTOM_NAME] = value and 1 or nil
end
--- 获取是否自定义名称
function T:getIsCustomName()
	return self.list[self.CUSTOM_NAME] == 1
end

--- 设置竞技场中上一次挑战失败的时间(必须为服务器传过来的时间,)
function T:setArenaFailTime(value)
	self.list[self.ARENA_FAIL_TIME] = value
	ArenaInfoMgr:tryStartUpdateCheck()
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_STATES)
end
--- 获取竞技场中上一次挑战失败的时间
function T:getArenaFailTime()
	local timeSec=self.list[self.ARENA_FAIL_TIME] or 0
	return GameUtils:modifyTimeZone(timeSec)
end

--- 设置竞技场奖励领取的时间相对于今年的第n天
function T:setArenaAwardGetYDay(value)
	self.list[self.ARENA_AWARD_GET_Y_DAY] = value or 0
	
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_ARENA_AWARD_GET_Y_DAY)
end
--- 获取竞技场奖励领取的时间相对于今年的第n天
function T:getArenaAwardGetYDay()
	return self.list[self.ARENA_AWARD_GET_Y_DAY] or 0
end

return T
