--region TaskMafnager.lua
--Author    : YuDing
--Date      : 2015/1/8
--Introduce : 任务管理器

local T = Class.TaskManager(LuaEventDispatcher2)


--任务条件常量,SECTION=1
--关卡,ATTACK_NPC=2
--攻击NPC,BUILDING=3
--建筑等级,AIRCRAFT=4
--飞机数量* YEILD=5
--玩家等级,PLAYER_LEVEL=6
--声望,REPUTATION=7

--任务分类：
T.noticeMessage = {
	["finishMainTask"] = "finishMainTask", -- 主任务完成情况
	["finishSuccess"] = "finishSuccess",   -- 成功完成任务

	["taskFinishNum"] = "taskFinishNum", -- 任务完成的数量     主要用于界面左边标题栏的提示


	["aircraftNumTask"] = "aircraftFinish", -- 飞机数量类 任务
	["resourceTask"] = "resourceTask",    -- 资源任务
	["pointTask"] = "pointTask",            -- 关卡相关任务 副本
	["maxAttackNPCTask"] = "maxAttackNPCTask",     -- 攻击NPC 最大等级
	["buildingLvlTask"] = "buildingLvlTask",       -- 建筑物等级

	["dailyTaskFresh"] = "dailyTaskFresh", -- 日常任务刷新

}

--- 各个任务的起始 和 结束ID
T.POINT = {1, 99}
T.MAXNPC = {101, 125}
T.CENTER = {150, 208}
T.GOLD = {250, 283}
T.STEEL = {300, 334}
T.OIL = {350, 384}
T.SILICON = {400, 433}
T.TATI = {450, 482}
T.STUDY = {500, 533}
T.STORE = {550, 581}
T.PLANEFACTORY = {600, 615}
T.PLANENUM = {650, 713}
T.RESOURCENUM = {750, 809}



function T:def()
	LuaEventDispatcher2.def(self)

	self.taskLock = false  -- 任务锁
end

function T:ctor()
	--- 增加进入主场景的事件
	CommonBroadcast:luaAddEventListener(MAINSCENE_ENTER, self, self.init)
end

--- 是否完成任务
function T:isTaskFinished(sid)
	if not sid or sid == 0 then
		return false
	end

	local key=math.floor(sid/32)
	local ls=BitOperator:_ls(1,sid%32)

	local t = UserInfo.task.taskInfos[key]
	return t and (BitOperator:_and(t,ls) ~= 0) or false
end

--- 设置任务已经完成
-- @param sid 任务sid
function T:setTaskFinished(sid)
	if not sid or sid == 0 then
		return
	end

	local key=math.floor(sid/32)
	local value=BitOperator:_ls(1,sid%32)
	local t=UserInfo.task.taskInfos[key]
	if not t then
		UserInfo.task.taskInfos[key]=value
	else
		if BitOperator:_and(t,value) ~= 0 then
			return
		end
		UserInfo.task.taskInfos[key]=BitOperator:_or(t, value)
	end
end

--- 是否是身上的任务
function T:isTaskHave(sid)
	if UserInfo.task.onTaskInfos[sid] then
		return true
	else
		return false
	end


end

--- 检查是否有新的任务
function T:checkNewTask()
	local tb = {}

	for k, v in pairs(TaskData) do
		local finish = true
		for i = 1, #v.limitSids do -- 前置任务sid
			if not self:isTaskFinished(v.limitSids[i]) then  -- 一旦其中有没完成的，跳出当前循环
				finish = false
				break
		end
		end

		if finish then -- 如果是完成的
			if not self:isTaskFinished(k) and not self:isTaskHave(k) then  -- 任务sid 不是身上和已完成的任务
				table.insert(tb, k)
		end
		end

	end

	-- 派发新添加的任务

	return tb
end

--- 检查当前sid的任务条件是否完成(主要用在挂在自身上的任务)
-- @param sid 任务sid
function T:checkTaskConditionFinished(sid)
	local d = TaskData[sid]
	local c = 0
	local tb = d.initCondition -- 任务需要开启的条件（如自身等级 或者其他）
	for i = 1, #tb do
		if 1 == tb[1] then -- 关卡sid 是否通过
			if BattlesInfoMgr:isBattleFinished(tb[2]) then c = 1 end
		elseif 2 == tb[1] then -- 是否打过npc资源点 等级
			if self:checkAttackNPCLevel(tb[2]) then c = 2 end
		elseif 3 == tb[1] then -- 不同建筑的等级
			if self:checkBldLevel(tb[2], tb[3]) then c = 3 end
		elseif 4 == tb[1] then -- 不同飞机的数量
			if self:checkAircraftNum(tb[2], tb[3]) then c = 4 end
		elseif 5 == tb[1] then -- 资源的产量(包括增益等)
			if tb[2] == 1 then -- 水晶
				if self:checkResourceOutput(6, tb[3]) then c = 5 end
		elseif tb[2] == 2 then -- 铁
			if self:checkResourceOutput(7, tb[3]) then c = 5 end
		elseif tb[2] == 3 then -- 石油
			if self:checkResourceOutput(8, tb[3]) then c = 5 end
		elseif tb[2] == 4 then -- 硅
			if self:checkResourceOutput(9, tb[3]) then c = 5 end
		end
		end
	end

	return c
end

--- 获得任务类型
-- return "role" 角色任务 "base" 基地建设 "resource" 资源任务
function T:getTaskKind(sid)
	local d = TaskData[sid]
	local tb = d.initCondition
	for i = 1, #tb do
		if 1 == tb[1] or 2 == tb[1] then -- 关卡sid 是否通过
			return "role"
		elseif 3 == tb[1] then -- 不同建筑的等级
			return "base"
		elseif 5 == tb[1] or 4 == tb[1] then -- 资源的产量(包括增益等) 飞机产量
			return "resource"
		end
	end
end
--- 获取任务头像
-- @param sid 任务id
function T:getTaskHeadRes(sid)
	local d = TaskData[sid]
	local res = "res/images/"
	res = res .. d.res

	local scale = {1.0, 1.0}
	if self:isBuildingTaskSid(sid) then
		scale = IconManager.buildScale
	end

	return res, scale
end

--- 是否攻击过NPC等级的资源点
function T:checkAttackNPCLevel(lvl)
	if not lvl then return false end  -- 改变量存着的是 已打过的最大NPC等级

	if PlayerInstance:getMaxAttackNPCLvl() >= lvl then
		return true
	else
		return false
	end

end

--- 检测建筑是否到等级
function T:checkBldLevel(sid, level)
	if not sid or not level then return false end
	local tb = {}
	local c = false
	for k, v in pairs (BuildingsInfoMgr:getBuildingInfos()) do  -- 选出对应sid到一个tb
		if v.sid == sid then
			table.insert(tb, k)
	end
	end

	for k, v in pairs(tb) do        -- 遍历
		if BuildingsInfoMgr:getBuildingInfos()[v].level >= level then -- 大于
			c = true
			break
	end
	end

	return c
end

--- 检查飞机数量
function T:checkAircraftNum(sid, num)
	if not sid or not num then return false end
	if not UserInfo.city.aircraftBox.infos[sid] then return false end

	if UserInfo.city.aircraftBox.infos[sid].num >= num then
		return true
	else
		return false
	end
end

--- 检查资源的产量
function T:checkResourceOutput(sid, num)
	if not sid or not num then return false end

	local m = PlayerInstance:getAllTrueOutput(sid)
	m = m * 60
	if m >= num then
		return true
	else
		return false
	end
end

-- 任务
-- @param tb sid组
function T:getTask(tb)
	-- 发送消息
	-- 成功 add tb to UserInfo.task.onTaskInfos[sid] = 1
	-- self:checkOwnTask()
	GetTaskCommand.new():exec(tb,
		function(state, newTask)
			if state == ReturnCode.SUCCESS then
				for i = 1, #newTask do -- 把新的任务添加到任务身上
					UserInfo.task.onTaskInfos[newTask[i]] = {finish = false}
				end
				self:checkOwnTask()  -- 检测身上任务是否有完成的
			else -- 没添加成功， 也判断身上任务
				self:checkOwnTask()
			end
		end)
end

--- 完成任务
function T:finishTask(sid)
	-- 成功
	-- 成功了之后， 再刷新一遍 self:flushData()
	-- UserInfo.task.onTaskInfos[sid] = nil
	FinishTaskCommand.new():exec(sid, function(state, tb, sId)
		if state == ReturnCode.SUCCESS then
			--local tb = self:getTaskReward(sid)

			self:setTaskFinished(sId)
			UserInfo.task.onTaskInfos[sId] = nil   -- 删除完成的任务

			--- 判断是否是攻打世界地图的资源点 标记置为0
			if self:isMaxAttackNPCTaskSid(sid) == true then
				PlayerInstance:setMaxAttackNPCLvl(0) -- 重置
			end

			self:flushData()                       -- 刷新新的任务

			self:showSuccessWindow(tb) -- 显示成功窗口
			require("src.app.communication.award.AwardDataDecoder"):addAWard(tb)  -- 派发物品
		else
			-- 完成任务失败 也刷新一次身上的任务
			MUIAlter_singleLine.show({
				info = Language:get("uy81"),
				size = 30,
				showSecond = 1,
			})

			self:flushData()
		end

		if BGMgr:isGuide() then
			BGMgr:fire(BGMgr.EVT_TASK_COMPLETED)
		end
	end)
end

--- 检查自身的任务
function T:checkOwnTask()
	--UserInfo.task.onTaskInfos

	-- self.taskLock = true

	local tb = {}
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if UserInfo.task.onTaskInfos[k] then
			if self:checkTaskConditionFinished(k) ~= 0 then
				UserInfo.task.onTaskInfos[k].finish = true
				table.insert(tb, k)
			end
		end
	end

	--for k, v in pairs(tb) do
	--print("已完成的任务id--->" .. v)
	--end


	self:updateKind() -- 分类
	--    if #tb >= 1 then -- 派发事件 （代表有完成的事件）
	--        self:luaDispatchEvent(self.noticeMessage.canFinishTask, tb)
	--    else
	--        self:luaDispatchEvent(self.noticeMessage.notCanFinishTask)
	--    end

	self:updateTaskFinishNum() -- 更新完成任务的数量

	self:luaDispatchEvent(self.noticeMessage.finishMainTask, tb)

	-- self.taskLock = false
end

--- 分类
function T:updateKind()
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if not v.kind then
			v.kind = self:getTaskKind(k)
		end
	end
end

-- 刷新数据
function T:flushData()
	--	local lt = TimeUtils:getSyncMillis()

	local tb = self:checkNewTask()
	if #tb == 0 then    -- 没有新的任务，就检测自身完成情况
		self:checkOwnTask()
	else    -- 获得任务
		self:getTask(tb)
	end

	--	print("刷新任务时间为:",TimeUtils:getSyncMillis()-lt)
end

--- 更新飞机数量任务（飞机数量变化 不会引起其他任务的连锁反应， 所以可以单独提取出来， 增加效率）
function T:updateAircraftNumTask()
	local tb
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if self:isAircraftNumTaskSid(k) then -- 飞机数量
			tb = TaskData[k].initCondition
			if self:checkAircraftNum(tb[2], tb[3]) then
				v.finish = true
			else
				v.finish = false
			end
		end
	end

	self:updateTaskFinishNum()   -- 更新完成任务数量

	self:luaDispatchEvent(self.noticeMessage.aircraftNumTask)
end

--- 更新资源 根据buff 影响现有任务 不会影响其他任务 而引起连锁反应
function T:updateResourceTask()
	local tb, finish

	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if self:isResourceTaskSid(k) then -- 资源数量
			tb = TaskData[k].initCondition

			finish = false
			if tb[2] == 1 then -- 水晶
				if self:checkResourceOutput(6, tb[3]) then finish = true end
			elseif tb[2] == 2 then -- 铁
				if self:checkResourceOutput(7, tb[3]) then finish = true end
			elseif tb[2] == 3 then -- 石油
				if self:checkResourceOutput(8, tb[3]) then finish = true end
			elseif tb[2] == 4 then -- 硅
				if self:checkResourceOutput(9, tb[3]) then finish = true end
			end

			if finish then
				v.finish = true
			else
				v.finish = false
			end
		end
	end

	self:updateTaskFinishNum() -- 更新完成任务数量

	self:luaDispatchEvent(self.noticeMessage.resourceTask)
end

--- 更新关卡任务 就是战斗结束的事件
function T:updatePointTask(pointType)
	if pointType ~= 1 then return end  -- 1 为副本

	local tb
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if self:isPointTaskSid(k) then
			tb = TaskData[k].initCondition
			if BattlesInfoMgr:isBattleFinished(tb[2]) then
				v.finish = true
			else
				v.finish = false
			end
		end

	end

	self:updateTaskFinishNum()   -- 更新完成任务数量

	self:luaDispatchEvent(self.noticeMessage.pointTask)

end

--- 更新攻击过的最大等级NPC
function T:updateMaxAttackNPCTask()
	local tb
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if self:isMaxAttackNPCTaskSid(k) then
			tb = TaskData[k].initCondition
			if self:checkAttackNPCLevel(tb[2]) then
				v.finish = true
			else
				v.finish = false
			end
		end
	end

	self:updateTaskFinishNum()   -- 更新完成任务数量

	self:luaDispatchEvent(self.noticeMessage.maxAttackNPCTask)
end

--- 更新建筑升级任务
function T:updateBuildingLvlTask()
	local tb
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if self:isBuildingTaskSid(k) then
			tb = TaskData[k].initCondition
			if self:checkBldLevel(tb[2], tb[3]) then
				v.finish = true
			else
				v.finish = false
			end
		end
	end

	self:updateTaskFinishNum()   -- 更新完成任务数量

	self:luaDispatchEvent(self.noticeMessage.buildingLvlTask)
end

--- 更新任务的数量
function T:updateTaskFinishNum()
	local num = 0
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if v.finish then
			num = num + 1
			break -- 一旦进入了 就跳出循环 节约时间
		end
	end

	self:luaDispatchEvent(self.noticeMessage.taskFinishNum, num)
end


--建筑升级 （涉及产量和建筑）
--战斗完成 （）
--buff完毕
--人物升级
--飞机数量变化
--资源产量 可以 归并到建筑升级这一项里面
-- 注册外部事件(先强制都刷新一遍)
function T:registerPublicEvent()
	-- PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.flushData)

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl, self, self.updateBuildingLvlTask) -- 建筑升级 涉及产量 等级 其他任务

	CommonBroadcast:luaAddEventListener(EVENT_FIGHT_COMPLETE, self, self.updatePointTask) -- 战斗结束 (影响现有任务)

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.maxAttackNPC, self, self.updateMaxAttackNPCTask) -- 最大npc

	BuffQueueInfoMgr:luaAddEventListener(BuffQueueInfoMgr.EVT_TYPE_BUFF_FINISH, self, self.updateResourceTask) -- buff结束 (影响资源完成任务， 现有任务)

	AircraftManager:luaAddEventListener(AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH, self, self.updateAircraftNumTask) -- 飞机数量改变（影响现有任务）

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl, self, self.updateResourceTask) -- 建筑升级 会影响资源产量任务
end
--- 获得玩家主线任务
function T:getPlayerMainTask()
	return UserInfo.task.onTaskInfos
end

--- 获得
function T:getFinishedTask()
	local tb = {}
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		if v.finish == true then
			table.insert(tb, k)
		end
	end

	table.sort(tb, function(x, y) return x < y end)
	return tb
end

--- 获得最小的sid任务
function T:getMinSidTask()
	local tb = {}
	for k, v in pairs(UserInfo.task.onTaskInfos) do
		table.insert(tb, k)
	end
	if #tb == 0 then  -- 任务做完了
		return 0
	end

	table.sort(tb, function(x, y) return x < y end) -- 排序

	return tb[1]
end

--- 根据sid 打开窗口
function T:openWindowBySid(sid, taskType)
	if taskType == "main" then  -- 主任务
		WindowManager:pop()
		if sid >= self.POINT[1] and sid <= self.POINT[2] then  -- 战场
			WindowManager:push(
				require("src.app.uiScript.components.battle.AllCampaignWindow").new()
			)

		elseif sid >= self.MAXNPC[1] and sid <= self.MAXNPC[2] then -- 世界地图
			HomePageManager:switchViewToWorld()

		elseif sid >= self.CENTER[1] and sid <= self.CENTER[2] then -- 指挥中心
			HomeOpenWindowManager:openCenterEvent()

		elseif sid >= self.GOLD[1] and sid <= self.GOLD[2] then -- 造币厂
			HomeOpenWindowManager:openMoneyFactoryEvent()

		elseif sid >= self.STEEL[1] and sid <= self.STEEL[2] then -- 资源建筑
			HomePageManager:switchViewToOutSide()

		elseif sid >= self.OIL[1] and sid <= self.OIL[2] then
			HomePageManager:switchViewToOutSide()

		elseif sid >= self.SILICON[1] and sid <= self.SILICON[2] then
			HomePageManager:switchViewToOutSide()

		elseif sid >= self.TATI[1] and sid <= self.TATI[2] then
			HomePageManager:switchViewToOutSide()

		elseif sid >= self.STUDY[1] and sid <= self.STUDY[2] then -- 科研中心
			HomeOpenWindowManager:openStudyCenterEvent()

		elseif sid >= self.STORE[1] and sid <= self.STORE[2] then -- 仓库
			if PlayerInstance:getBuildingLvl(6) >= PlayerInstance:getBuildingLvl(7) then
				HomeOpenWindowManager:openStoreHouseOneEvent()
		else
			HomeOpenWindowManager:openStoreHouseTwoEvent()
		end

		elseif sid >= self.PLANEFACTORY[1] and sid <= self.PLANEFACTORY[2] then -- 机库
			if PlayerInstance:getBuildingLvl(2) >= PlayerInstance:getBuildingLvl(3) then
				HomeOpenWindowManager:openPlaneFactoryOneEvent()
		else
			HomeOpenWindowManager:openPlaneFacrtoryTwoEvent()
		end

		elseif sid >= self.PLANENUM[1] and sid <= self.PLANENUM[2] then -- 各种飞机数量
			if PlayerInstance:getBuildingLvl(2) >= PlayerInstance:getBuildingLvl(3) then
				HomeOpenWindowManager:openPlaneFactoryOneEvent()
		else
			HomeOpenWindowManager:openPlaneFacrtoryTwoEvent()
		end

		elseif sid >= self.RESOURCENUM[1] and sid <= self.RESOURCENUM[2] then -- 资源产量
			HomePageManager:switchViewToOutSide()
		end
	end
end

--- 得到当前的任务信息
function T:getCurrentTaskInfo(sid, finish)
	if finish then -- 已完成
		return Language:get("uy64")
	end

	local d = TaskData[sid]
	local tb = d.initCondition -- 任务需要开启的条件（如自身等级 或者其他）

	if sid >= self.POINT[1] and sid <= self.POINT[2] then  -- 战场
		return Language:get("uy62")

	elseif sid >= self.MAXNPC[1] and sid <= self.MAXNPC[2] then -- 世界地图
		return Language:get("uy62")

	elseif sid >= self.CENTER[1] and sid <= self.CENTER[2] then -- 指挥中心
		return Language:get("uy61", PlayerInstance:getBuildingLvl(PlayerInstance.fixedBldLocation.center))

	elseif sid >= self.GOLD[1] and sid <= self.GOLD[2] then -- 造币厂
		return Language:get("uy61", PlayerInstance:getBuildingLvl(PlayerInstance.fixedBldLocation.gold))

	elseif sid >= self.STEEL[1] and sid <= self.STEEL[2] then -- 铁矿
		return Language:get("uy61", PlayerInstance:getBldLevelBySid(7))

	elseif sid >= self.OIL[1] and sid <= self.OIL[2] then -- 石油
		return Language:get("uy61", PlayerInstance:getBldLevelBySid(8))

	elseif sid >= self.SILICON[1] and sid <= self.SILICON[2] then -- 硅矿
		return Language:get("uy61", PlayerInstance:getBldLevelBySid(9))

	elseif sid >= self.TATI[1] and sid <= self.TATI[2] then -- 钛矿
		return Language:get("uy61", PlayerInstance:getBldLevelBySid(10))

	elseif sid >= self.STUDY[1] and sid <= self.STUDY[2] then -- 科研中心
		return Language:get("uy61", PlayerInstance:getBuildingLvl(PlayerInstance.fixedBldLocation.study))

	elseif sid >= self.STORE[1] and sid <= self.STORE[2] then -- 仓库
		return Language:get("uy61", PlayerInstance:getBldLevelBySid(5))

	elseif sid >= self.PLANEFACTORY[1] and sid <= self.PLANEFACTORY[2] then -- 机库
		return Language:get("uy61", PlayerInstance:getBldLevelBySid(2))

	elseif sid >= self.PLANENUM[1] and sid <= self.PLANENUM[2] then -- 各种飞机数量
		return Language:get("uy66", PlayerInstance:getAircraftNumBySid(tb[2]))

	elseif sid >= self.RESOURCENUM[1] and sid <= self.RESOURCENUM[2] then -- 资源产量
		if tb[2] == 1 then -- 水晶
			local output = PlayerInstance:getAllTrueOutput(6)
			return Language:get("uy63", output * 60)

	elseif tb[2] == 2 then -- 铁
		local output = PlayerInstance:getAllTrueOutput(7)
		return Language:get("uy63", output * 60)

	elseif tb[2] == 3 then -- 石油
		local output = PlayerInstance:getAllTrueOutput(8)
		return Language:get("uy63", output * 60)

	elseif tb[2] == 4 then -- 硅
		local output = PlayerInstance:getAllTrueOutput(9)
		return Language:get("uy63", output * 60)

	end
	end
end

--- 是否是飞机数量任务id
function T:isAircraftNumTaskSid(sid)
	if sid >= self.PLANENUM[1] and sid <= self.PLANENUM[2] then
		return true
	else
		return false
	end
end

--- 是否是资源任务sid
function T:isResourceTaskSid(sid)
	if sid >= self.RESOURCENUM[1] and sid <= self.RESOURCENUM[2] then
		return true
	else
		return false
	end
end
--- 是否是关卡任务id
function T:isPointTaskSid(sid)
	if sid >= self.POINT[1] and sid <= self.POINT[2] then
		return true
	else
		return false
	end
end

--- 是否是 攻击最大npc任务id
function T:isMaxAttackNPCTaskSid(sid)
	if sid >= self.MAXNPC[1] and sid <= self.MAXNPC[2] then
		return true
	else
		return false
	end
end

--- 建筑sid (一共5类任务)
function T:isBuildingTaskSid(sid)
	local t = false

	if sid >= self.CENTER[1] and sid <= self.CENTER[2] then
		t = true
	elseif sid >= self.GOLD[1] and sid <= self.GOLD[2] then
		t = true
	elseif sid >= self.STEEL[1] and sid <= self.STEEL[2] then
		t = true
	elseif sid >= self.OIL[1] and sid <= self.OIL[2] then
		t = true
	elseif sid >= self.SILICON[1] and sid <= self.SILICON[2] then
		t = true
	elseif sid >= self.TATI[1] and sid <= self.TATI[2] then
		t = true
	elseif sid >= self.STUDY[1] and sid <= self.STUDY[2] then
		t = true
	elseif sid >= self.STORE[1] and sid <= self.STORE[2] then
		t = true
	elseif sid >= self.PLANEFACTORY[1] and sid <= self.PLANEFACTORY[2] then
		t = true
	end

	return t
end

--- 得到奖励
function T:getTaskReward(sid, kind)
	local tb
	if kind == "daily" then -- 日常
		tb = DailyTaskData[sid]
	else
		tb = TaskData[sid]
	end

	local reward = {}

	reward.exp=tb.exp or 0
	reward.money=tb.money or 0
	reward.steel=tb.steel or 0
	reward.oil=tb.oil or 0
	reward.silicon=tb.silicon or 0
	reward.titanium=tb.titanium or 0
	reward.reputation=tb.reputation or 0
	reward.pvePoint=tb.pvePoint or 0
	reward.gems=tb.gems or 0
	reward.titanium=tb.titanium or 0

	reward.props = {}
	if #tb.propSids > 0 then
		-- 物品id
		for i = 1, #tb.propSids, 2 do
			table.insert(reward.props, {sid = tb.propSids[i], count = tb.propSids[i + 1]})
		end
	end

	reward.aircrafts = {}
	if #tb.aircraftSids > 0 then
		-- 飞机id
		for i = 1, #tb.aircraftSids, 2 do
			table.insert(reward.aircrafts, {sid = tb.aircraftSids[i], count = tb.aircraftSids[i + 1]})
		end
	end

	return reward
end


-- 发奖励
-- tb 奖励的表
-- pop 奖励是否弹出窗口
function T:distributeReward(tb)
	require("src.app.communication.award.AwardDataDecoder"):addAWard(tb)
end

--- 显示奖品信息
function T:showRewardInfoWindow(sid, finish, tb, kind)
	local t

	local name
	if kind == "daily" then
		t = DailyTaskData[sid]
		name = self:getDailyNameBySid(sid)
	else
		t = TaskData[sid]
		name = self:getTaskNameBySid(sid)
	end

	local w,h=580,240
	local top=16

	local lay=ccui.Layout:create()
	lay:setContentSize(cc.size(w, h))

	--奖励显示项列表
	local uiAwardList=ccui.Layout:create()
	REL:relOther(uiAwardList,lay,10,10,10,16)
	lay:addChild(uiAwardList)

	local uiList=require("src.app.communication.award.AwardDataDecoder"):award2LayoutList(tb)
	local w,h=require("src.app.communication.award.AwardDataDecoder"):arrangeInLayout(uiList,uiAwardList)

	lay:setContentSize(cc.size(
		w,
		top+ --上边框
		36+20+ --名称
		36+20+ --进度
		36+4+ --奖励标题
		h+ --奖励列表
		top --下边框
	))

	--任务名称
	local uiName=XMLLabel(name or "")
	uiName:setAlign("left","middle")
	uiName:setHeight(36)
	lay:addChild(uiName)
	REL:relOther(uiName,lay,10,10,top,SIZE_NIL)

	top=top+uiName:getHeight()+20

	--分隔线1
	local uiSep1=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("seqLine1.png"))
	lay:addChild(uiSep1)
	REL:relOther(uiSep1,uiName,0,0,SIZE_NIL,-10)

	--进度
	local content
	if kind == "daily" then
		if finish == true then -- 已完成 未完成
			content = Language:get("uy64")
		else
			content = Language:get("uy62")
		end
	else
		content = TaskManager:getCurrentTaskInfo(sid, finish)
	end

	local uiProgress=XMLLabel(content)
	uiProgress:setAlign("left","middle")
	uiProgress:setHeight(36)
	lay:addChild(uiProgress)
	REL:relOther(uiProgress,lay,10,10,top,SIZE_NIL)

	top=top+uiProgress:getHeight()+20

	--分隔线1
	local uiSep2=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("seqLine1.png"))
	lay:addChild(uiSep2)
	REL:relOther(uiSep2,uiProgress,0,0,SIZE_NIL,-10)

	--奖励标题
	local uiAwardCap=XMLLabel(Language:get("uy67"))
	uiAwardCap:setAlign("left","middle")
	uiAwardCap:setHeight(36)
	lay:addChild(uiAwardCap)
	REL:relOther(uiAwardCap,lay,10,10,top,SIZE_NIL)

	top=top+uiAwardCap:getHeight()+4

	REL:toOther("center","top",uiAwardList,lay,0,top)

	--- 显示窗口
	MUIAlter_noTitle.show({
		info = lay
	})

end

function T:showSuccessWindow(tb)
	local w,h=580,240
	local top=16

	local lay = ccui.Layout:create()
	lay:setContentSize(cc.size(w, h))

	local title = XMLLabel(Language:get("uy68"))
	title:setAlign("center","middle")
	title:setHeight(36)
	lay:addChild(title)

	local uiAwardList=ccui.Layout:create()
	REL:relOther(uiAwardList,lay,10,10,10,16)
	lay:addChild(uiAwardList)

	local uiList=require("src.app.communication.award.AwardDataDecoder"):award2LayoutList(tb)
	local w,h=require("src.app.communication.award.AwardDataDecoder"):arrangeInLayout(uiList,uiAwardList)

	lay:setContentSize(cc.size(
		w,
		top+ --上边距
		title:getHeight()+ --title
		10+ --间隔
		h+ --奖励列表
		16) --下边距
	)

	--设置位置
	REL:relOther(title,lay,10,10,top,SIZE_NIL)
	top=top+title:getHeight()+10
	REL:toOther("center","top",uiAwardList,lay,0,top)

	--- 显示窗口
	MUIAlter_noTitle.show({
		info = lay,
		callback=function()
			if BGMgr:isGuide() then
				BGMgr:fire(BGMgr.EVT_TASK_AWARD_SHOW_COMPLETED)
			else
				local playerLvlDiff = PlayerInstance:getPlayerLvlDiff()
				if playerLvlDiff == true then
					local layout = require("src.app.uiScript.components.share.PlayerLvlUpPopWindow").new()
					local muialter = MUIAlter.show(
						{
							title = Language:get("uc930"),
							info = layout,
							flag = 4,
							isShowCloseButton = true,
							callback = function(flag)
							end
						}
					)
					layout:setMuialter(muialter)

				end
			end
		end,
		popSnd=SoundRes.SND_SND_GET_AWARD
	})
end

--- 检查 普通任务 哪个任务在运行
function T:checkNormalDailyTask()

	local tb = UserInfo.dailyTask:getNormalTask()
	local id = 0
	local finish = false
	local sec = TimeUtils:getSyncMillis() / 1000

	for k, v in pairs(tb) do
		if v[2] >= 1 then  -- 代表有任务在执行
			id = v[1]
			if sec >= v[2] then -- 已完成
				finish = true
			end
			break
		end
	end
	return id, finish
end

--- 检查 军团任务 哪个任务在运行
function T:checkAllianceDailyTask()
	local tb = UserInfo.dailyTask:getAllianceTask()
	local id = 0
	local finish = false
	local sec = TimeUtils:getSyncMillis() / 1000

	for k, v in pairs(tb) do
		if v[2] >= 1 then  -- 代表有任务在执行
			id = v[1]
			if sec >= v[2] then -- 已完成
				finish = true
			end
			break
		end
	end
	return id, finish
end

--- 获得日常 普通任务
function T:getDailyNormalTask()
	return UserInfo.dailyTask:getNormalTask()
end

--- 获得日常 军团任务
function T:getDailyAllianceTask()
	return UserInfo.dailyTask:getAllianceTask()
end

--- 获得 刷新事件
function T:getDailyNormalFreshTime()
	return UserInfo.dailyTask:getNormalTime()
end

function T:getDailyAllianceFreshTime()
	return UserInfo.dailyTask:getAllianceTime()
end

function T:setDailyTaskTime(sid, time)
	UserInfo.dailyTask:setTaskTime(sid, time)
end

--- 是否能 刷新 普通任务
function T:isCanFreshDailyNormal()
	local time = self:getDailyNormalFreshTime()
	local sec = TimeUtils:getSyncMillis() / 1000
	if sec < time then -- 还没有到时间
		return false
	else
		return true
	end
end

--- 是否能 刷新 军团任务
function T:isCanFreshDailyAlliance()
	local time = self:getDailyAllianceFreshTime()
	local sec = TimeUtils:getSyncMillis() / 1000
	if sec < time then -- 还没有到时间
		return false
	else
		return true
	end
end

--- 得到日常 普通任务 提示
function T:getDailyNormalTip()
	--- 检查刷新时间到没有
	local tb = self:getDailyNormalTask()
	if self:isCanFreshDailyNormal() and #tb == 0 then  -- 刷新时间到了 提示
		return true
	end

	--- 刷新时间没到 继续检查 任务的个数
	if #tb == 0 then -- 任务已经全部完成
		return false
	else
		local id, finish = self:checkNormalDailyTask()
		if id > 0 then  -- 有进行中的任务
			if finish == true then  -- 有任务完成 提示关闭
				return true
		else -- 有任务在进行 这时候 不需要提示
			return false
		end
		else -- 没进行中的任务
			return true
		end
	end
end

--- 得到日常 军团任务 提示
function T:getDailyAllianceTip()
	local tb = self:getDailyAllianceTask()
	if self:isCanFreshDailyAlliance() and #tb == 0 then  -- 刷新时间到了 提示
		return true
	end

	--- 刷新时间没到 继续检查 任务的个数
	if #tb == 0 then -- 任务已经全部完成
		return false
	else
		local id, finish = self:checkAllianceDailyTask()
		if id > 0 then  -- 有进行中的任务
			if finish == true then  -- 有任务完成 提示关闭
				return true
		else -- 有任务在进行 这时候 不需要提示
			return false
		end
		else -- 没进行中的任务
			return true
		end
	end
end
----- 设置主线任务对象
--function T:setMainNodeObject(object)
--    self.mainNodeObject = object
--end

----- 释放主线任务对象
--function T:releaseMainNodeObject()
--    self.mainNodeObject = nil
--end

--- 初始化
function T:init()
	self:registerPublicEvent()

	local func = function()
		self:flushData()
	end

	--- 延迟一帧刷新
	Cocos2dxUtils:timeOut(func, 0)
end


--- 得到每日任务的名字
function T:getDailyNameBySid(sid)
	local rome = self:getRome(sid)
	if sid <= 5 and sid >= 1 then
		return Language:get(rome,Language:get("uc878"))
	elseif sid <= 105 and sid >= 101 then
		return Language:get(rome,Language:get("uc879"))
	elseif sid <= 205 and sid >= 201 then
		return Language:get(rome,Language:get("uc880"))
	elseif sid <= 305 and sid >= 301 then
		return Language:get(rome,Language:get("uc881"))
	elseif sid <= 405 and sid >= 401 then
		return Language:get(rome,Language:get("uc882"))
	elseif sid <= 1005 and sid >= 1001 then
		return Language:get(rome,Language:get("uc883"))
	elseif sid <= 1105 and sid >= 1101 then
		return Language:get(rome,Language:get("uc884"))
	elseif sid <= 1205 and sid >= 1201 then
		return Language:get(rome,Language:get("uc885"))
	else
		return Language:get(rome,Language:get("uc878")) --防止蹦
	end

end

--- 得到1到5对应的数字
function T:getRome(sid)
	if sid % 5 == 1 then
		return "uc873"
	elseif sid % 5 == 2 then
		return "uc874"
	elseif sid % 5 == 3 then
		return "uc875"
	elseif sid % 5 == 4 then
		return "uc876"
	elseif sid % 5 == 0 then
		return "uc877"
	else
		return "uc873"

	end


end


--- 得到任务的名字
function T:getTaskNameBySid(sid)
	local nameTable = TaskData[sid].nameTable -- 是一个数字表
	if sid >= 1 and sid <= 99 then
		return Language:get("uc886",nameTable[1],nameTable[2])
	elseif sid >= 101 and sid <= 125 then
		return Language:get("uc887",nameTable[1])
	elseif sid >= 150 and sid <= 208 then
		return Language:get("uc888",nameTable[1])
	elseif sid == 250 then
		return Language:get("uc889")
	elseif sid >= 251 and sid <= 283 then
		return Language:get("uc890",nameTable[1])
	elseif sid == 300 then
		return Language:get("uc891")
	elseif sid >= 301 and sid <= 334 then
		return Language:get("uc892",nameTable[1])
	elseif sid >= 350 and sid <= 384 then
		return Language:get("uc893",nameTable[1])
	elseif sid >= 400 and sid <= 433 then
		return Language:get("uc894",nameTable[1])
	elseif sid == 450 then
		return Language:get("uc895")
	elseif sid >= 451 and sid <= 482 then
		return Language:get("uc896",nameTable[1])
	elseif sid == 500 then
		return Language:get("uc897")
	elseif sid >= 501 and sid <= 533 then
		return Language:get("uc898",nameTable[1])
	elseif sid == 550 then
		return Language:get("uc899")
	elseif sid >= 551 and sid <= 581 then
		return Language:get("uc900",nameTable[1])
	elseif sid == 600 then
		return Language:get("uc901")
	elseif sid >= 601 and sid <= 615 then
		return Language:get("uc902",nameTable[1])


	elseif sid >= 650 and sid <= 713 then
		if sid >= 650 and sid <= 653 then
			return Language:get("uc903",nameTable[1])
		elseif sid >= 654 and sid <= 657 then
			return Language:get("uc904",nameTable[1])
		elseif sid >= 658 and sid <= 661 then
			return Language:get("uc905",nameTable[1])
		elseif sid >= 662 and sid <= 665 then
			return Language:get("uc906",nameTable[1])
		elseif sid >= 666 and sid <= 669 then
			return Language:get("uc907",nameTable[1])
		elseif sid >= 670 and sid <= 673 then
			return Language:get("uc908",nameTable[1])
		elseif sid >= 674 and sid <= 677 then
			return Language:get("uc909",nameTable[1])
		elseif sid >= 678 and sid <= 681 then
			return Language:get("uc910",nameTable[1])
		elseif sid >= 682 and sid <= 685 then
			return Language:get("uc911",nameTable[1])
		elseif sid >= 686 and sid <= 689 then
			return Language:get("uc912",nameTable[1])
		elseif sid >= 690 and sid <= 693 then
			return Language:get("uc913",nameTable[1])
		elseif sid >= 694 and sid <= 697 then
			return Language:get("uc914",nameTable[1])
		elseif sid >= 698 and sid <= 701 then
			return Language:get("uc915",nameTable[1])
		elseif sid >= 702 and sid <= 705 then
			return Language:get("uc916",nameTable[1])
		elseif sid >= 706 and sid <= 709 then
			return Language:get("uc917",nameTable[1])
		elseif sid >= 710 and sid <= 713 then
			return Language:get("uc918",nameTable[1])
		end

	elseif sid >= 750 and sid <= 769 then
		return Language:get("uc919",nameTable[1])
	elseif sid >= 770 and sid <= 789 then
		return Language:get("uc920",nameTable[1])
	elseif sid >= 790 and sid <= 809 then
		return Language:get("uc921",nameTable[1])
	else
		return ""  -- 防止程序蹦
	end


end





return T




















--endregion
