--[[
    警告队列盒子
]]
local T = Class.WarnBox(LuaEventDispatcher2)

-- 警告队列信息
T.WarnInfo = require("src.app.model.warn.WarnInfo")

--- 事件--添加单个队列
T.EVT_ADD_SINGLE_QUEUE_REFRESH = "evt_add_single_queue_refresh"

--- 事件--移除单个队列
T.EVT_REMOVE_SINGLE_QUEUE_REFRESH = "evt_remove_single_queue_refresh"

--- 事件--时间刷新
T.EVT_TIME_REFRESH = "evt_time_refresh"


function T:def()
    LuaEventDispatcher2.def(self)

    self.warnBox = {}
end

function T:decode(data)
    local length = data:readByte()

    local infos = {}
    for i = 1,length,1 do
        local info = self.WarnInfo()
		info:decode(data)
		infos[info.sourceId .. '_' .. info.qid] = info
    end

    self.warnBox = infos
end

--- 判断队列里面有数据没有,true:有队列，false：没有数据
function T:judgeQueue()
    local judge = false
    for k,v in pairs(self.warnBox) do
        if v then
            judge = true
            break
        end
    end

    return judge
end


--- 添加的单个进攻队列
function T:addMarch(data)
    local info = self.WarnInfo()
	info:decode(data)
	self.warnBox[info.sourceId .. '_' .. info.qid] = info

    self:startUpdateCheck()
    self:luaDispatchEvent(self.EVT_ADD_SINGLE_QUEUE_REFRESH,info) -- 并且把数据派发过来

    -- 添加警告信息
    local descripe = Language:get("uc943") .. Language:get("uc866")
	MUIAlter_singleLine.show( {info = descripe,showSecond = 1,popSnd = SoundRes.SOUND_WARNING } )

end

--- 移除单个进攻对列:有可能1个玩家派出几个队列来攻打你，或者有可能那个队列id是一样的（不同玩家拥有不同的队列）
function T:removeMarch(data)
    local sourceId = data:readInt() -- 源id(攻击者id)
    local qid = data:readInt() -- 战斗队列id

    for k,v in pairs(self.warnBox) do
        if v.sourceId == sourceId and v.qid == qid then
            self.warnBox[k] = nil
            self:luaDispatchEvent(self.EVT_REMOVE_SINGLE_QUEUE_REFRESH .. sourceId .. qid)
            self:luaDispatchEvent(self.EVT_REMOVE_SINGLE_QUEUE_REFRESH)
            return
        end

    end
    

end

function T:startUpdateCheck()
	local function update()
		local list = self.warnBox
	    
		local syncSec = TimeUtils:getSyncMillis()/1000
		for k,v in pairs(list) do 
			v.remainingSec = v.endTimeSec - syncSec
            if v.endTimeSec <= syncSec then
                self.warnBox[k] = nil
                self:luaDispatchEvent(self.EVT_REMOVE_SINGLE_QUEUE_REFRESH .. v.sourceId .. v.qid)
                self:luaDispatchEvent(self.EVT_REMOVE_SINGLE_QUEUE_REFRESH)
                self:tryStopUpdate()
            else
                self:luaDispatchEvent(self.EVT_TIME_REFRESH .. v.sourceId .. v.qid)
			end

		end
	end

	if self.updateId then
		return
	end

	local list = self.warnBox
	if list then
		for _ in pairs(list) do
			self.updateId=Cocos2dxUtils:interval(update,1)
			break
		end
	end
end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
		local list = self.warnBox

		--如果队列为空,结束时间更新检查
		local isEmpty=true
		for _ in pairs(list) do
			isEmpty=false
			break
		end

		if isEmpty then
			Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId=nil
		end
	end
end




return T