--- 用户道具信息管理器
PropsInfoMgr=require("src.app.model2.prop.PropsInfoMgr")

--- 战斗信息管理器
BattlesInfoMgr=require("src.app.model2.battle.BattlesInfoMgr")

--- 世界地图信息管理器
WorldMapMgr=require("src.app.model.WorldMap.WorldMapMgr")

--- 新手引导管理器
BeginnersGuideMgr=require("src.app.model.beginnersGuide.BeginnersGuideMgr")
--- 新手引导管理器(简写)
BGMgr=BeginnersGuideMgr

--- 用户信息管理器(单例)
local T=Class.UserInfoMgr(LuaEventDispatcher2)

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件派发pve点数更新事件
T.EVT_TYPE_UPDATE_PVE_POINT="evt_type_update_pve_point"

--- 玩家名被改变
T.EVT_TYPE_CHANGE_PLAYER_NAME="evt_type_change_player_name"

--- 玩家头像ID被改变
T.EVT_TYPE_CHANGE_PHOTO_ID="evt_type_change_photo_id"

--- 精力点的本地推送ID
T.LOCAL_NOTI_ID_PVE_POINT="local_noti_id_pve_point"

function T:def()
	LuaEventDispatcher2.def(self)

	--- 玩家数据
	-- self.info=nil

	--- 更新计时器ID
	-- self.updateId = nil

	--- 距离下次回复的剩余时间
	self.pveRecoverRemaing=0
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 初始化
-- @param userInfo 玩家数据
function T:init(userInfo)
	self.info=userInfo

	PropsInfoMgr:init(userInfo.propsInfo)
	BattlesInfoMgr:init(userInfo.battlesInfo)
	WorldMapMgr:init()
	CityInfoMgr:init(userInfo.city)
	BGMgr:init(userInfo.guideIndex)
	ArenaInfoMgr:init()

	self:localNotiCheck_pvePoint()
	self:audioStateSet()
	self:setCashier()

	-- 聊天信息数据
	self:getChatData()
	-- 邮箱数据
	self:getMailData()

	-- 战斗力要前台自己算，后面刷新战力(要等所有的数据得到后来设置)
	PlayerInstance:setPlayerFightLogic(true)

	PlayerInstance:setPlayerLvlDiff(false)

	AnnounceManager:setAnnounceCrcFront(0)

	self:luaDispatchEvent(self.EVT_TYPE_RESET)
	ResourceManager:luaDispatchEvent(ResourceManager.EVT_TYPE_RESOURCE_REFRESH)
end

--- 开始更新检查
function T:_startUpdateCheck()
	local update=function()
		local info=self.info

		local syncSec=TimeUtils:getSyncMillis()/1000

		--判断是否应该清除每日状态>>---------------------------------------------------------
		--		local cDate=os.date("*t",syncSec)
		--		if cDate.day ~= DailyStates.currentDay then
		--			DailyStates:clear()
		--		end
		--<<判断是否应该清除每日状态---------------------------------------------------------

		--回复pve点数>>----------------------------------------------------------------------
		local intervalSec=GameCfg.PVE_POINT_RECOVER_INTERVAL_SEC
		local addPoint=math.floor((syncSec-info.pveRecoverSec)/intervalSec)
		if addPoint > 0 then
			PlayerInstance:addPlayerEnergy(addPoint,true)
			info.pveRecoverSec=info.pveRecoverSec+(addPoint*intervalSec)
		end
		if info.pvePoint < GameCfg.PVE_POINT_LIMIT then
			self.pveRecoverRemaing=(info.pveRecoverSec+intervalSec)-syncSec
		else
			self.pveRecoverRemaing=0
		end

		self:luaDispatchEvent(self.EVT_TYPE_UPDATE_PVE_POINT)
		--<<回复pve点数----------------------------------------------------------------------
	end

	if self.updateId then
		return
	end

	self.updateId=Cocos2dxUtils:interval(update,1)
end

--- 结束更新检查
function T:_stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

--- 开始更新检查
function T:startUpdateCheck()
	xpcall(
		function()
			CityInfoMgr:startUpdateCheck()

			ArenaInfoMgr:tryStartUpdateCheck()
		end,
		function(errStr)
			local message=tostring(errStr).."\n"..debug.traceback()

			__print__("startMgrErr",message)
			LogRecorder:push(message,LOG_LEVEL_ERROR,"EvtDispatchErr")
		end
	)

	self:_startUpdateCheck()
end

--- 结束更新检查
function T:stopUpdateCheck()
	CityInfoMgr:stopUpdateCheck()

	self:_stopUpdateCheck()
end

--- 本地推送检查-pve点
function T:localNotiCheck_pvePoint()
	local info=self.info

	local c = PlayerInstance:getPlayerMaxEnergy() - PlayerInstance:getPlayerEnergy()
	if c > 0 then
		LocalNoti:push(
			Language:get("ug1000"),
			info.pveRecoverSec+(c*GameCfg.PVE_POINT_RECOVER_INTERVAL_SEC)-TimeUtils:getSyncMillis()/1000,
			LocalNoti.PVE_POINT_TIP,
			info.id..self.LOCAL_NOTI_ID_PVE_POINT
		)
	else
		LocalNoti:cancel(info.id..self.LOCAL_NOTI_ID_PVE_POINT)
	end
end

--- 音频状态设置
function T:audioStateSet()
	local setCommonState = SetStates.setCommonState

	AudioManager:setEnabledMusic(not setCommonState:getClosedValue(setCommonState.CLOSE_BACKGROUND_MUSIC))
	AudioManager:setEnabledPlayEffect(not setCommonState:getClosedValue(setCommonState.CLOSE_MUSIC_EFFECT))

	DataStorage:putAudioStatus()
end

--- 内购状态设置
function T:setCashier()
	--- 判断是否支付内购
	if cc.PLATFORM_OS_WINDOWS == C_PFM
		or
		cc.PLATFORM_OS_ANDROID == C_PFM
		or
		CUR_SDK == SDKType.PLATFORM_ENJOY_GAME
	then
		return
	end

	local iapReceipts = DataStorage:getIapReceipts(UserInfo.id)
	if iapReceipts then
		for k,v in pairs(iapReceipts) do
			if v then
				Command:payGoods(v)
			end
		end
	end

	-- 要考虑特殊断线,回调只能加一次
	local function confirm(data)
		local ipaData = str2obj(data)[1]
		-- ipaData.id -- 产品id
		print(obj2str(ipaData))

		if ipaData.transactionState == 0 then
			print("购买成功,向游戏服务器发送验证数据")
			DataStorage:addReceipt(UserInfo.id,{verifyInfo = ipaData.transactionReceipt,id = ipaData.id})

			Command:payGoods({verifyInfo = ipaData.transactionReceipt,id = ipaData.id})
		elseif ipaData.transactionState == 1 then -- 购买失败
			print("购买失败")

			local errCode=ipaData.errCode
			--如果错误码不为2(用户取消支付)
			if errCode ~= 2 then
				MUIAlter.show({
					info=Language:get("ug_payFail",errCode)
				})
			end

			if GoodsManager and GoodsManager:judgeIsSelf(ipaData.id) then
				CommonBroadcast:luaDispatchEvent(EVENT_IAP_BUY_GOODS_COMPLETE)
			end

			if NewPackageManager and NewPackageManager:judgeIsSelf(ipaData.id) then
				CommonBroadcast:luaDispatchEvent(EVENT_IAP_BUY_NEW_RECHARGE_COMPLETE)
			end
		elseif ipaData.transactionState == 2 then -- 已购买过此商品
			print("已购买过此商品")

			if GoodsManager and GoodsManager:judgeIsSelf(ipaData.id) then
				CommonBroadcast:luaDispatchEvent(EVENT_IAP_BUY_GOODS_COMPLETE)
			end

			if NewPackageManager and NewPackageManager:judgeIsSelf(ipaData.id) then
				CommonBroadcast:luaDispatchEvent(EVENT_IAP_BUY_NEW_RECHARGE_COMPLETE)
			end
		elseif ipaData.transactionState == 3 then -- 正在购买
			print("正在购买")

			if GoodsManager and GoodsManager:judgeIsSelf(ipaData.id) then
			--CommonBroadcast:luaDispatchEvent(EVENT_IAP_BUY_GOODS_COMPLETE)
			end

			if NewPackageManager and NewPackageManager:judgeIsSelf(ipaData.id) then
			--CommonBroadcast:luaDispatchEvent(EVENT_IAP_BUY_NEW_RECHARGE_COMPLETE)
			end
		else -- 有很多其它内购的错误要写上
			print("未知状态")

		end

	end
	Cashier:setPaymentWithProductCompleteCall(confirm)

	-- 初始化应用内购买
	Cashier:init()
end

--- 设置是否已修改玩家名称
function T:setIsChangeName(value)
	if self.info and self.info.isChangeName ~= value then
		self.info.isChangeName=value
	end
end
--- 获取是否已修改玩家名称
function T:getIsChangeName()
	return self.info and self.info.isChangeName
end

--- 设置玩家名
function T:setPlayerName(value)
	if self.info and self.info.name ~= value then
		self.info.name=value

		--		self:luaDispatchEvent(self.EVT_TYPE_CHANGE_PLAYER_NAME,self.info.name)
		PlayerInstance:luaDispatchEvent(PlayerInstance.noticeMessage.name)
	end
end
--- 获取玩家名
function T:getPlayerName()
	return self.info and self.info.name or ""
end

--- 设置玩家头像ID
function T:setPhotoId(value)
	if self.info and self.info.photoId ~= value then
		self.info.photoId=value

		--		self:luaDispatchEvent(self.EVT_TYPE_CHANGE_PHOTO_ID,self.info.photoId)
		PlayerInstance:luaDispatchEvent(PlayerInstance.noticeMessage.headId)
	end
end
--- 获取玩家头像ID
function T:getPhotoId()
	return self.info and self.info.photoId or 0
end

function T:getChatData()
	-- 发送获取所有世界聊天信息
	GetAllWorldChatCommand = require("src.app.communication.chat.GetAllWorldChatCommand")
	GetAllWorldChatCommand:new():exec( function(state)
		if state == ReturnCode.SUCCESS then
			print("获取聊天信息成功")
		else

		end
	end )

end

function T:getMailData()
	if not MailManager then
		MailManager = require("src.app.model.mail.MailManager")()      -- 必须在这里初始化
	end

	GetMailListCommand = require("src.app.communication.mail.GetMailListCommand")

	GetMailListCommand:new():exec( function(state)
		if state == ReturnCode.SUCCESS then
			print("获取邮箱成功")
			-- 邮箱检测
			-- MailManager = require "src.app.model.mail.MailManager"
			-- MailManager:startUpdateCheck()
		else

		end
	end )

end

--- 清除聊天数据和邮箱数据
function T:clearChatAndMailData()
	if MailManager then
		MailManager:clearAllMailData()
	end

    if ChatManager then
		ChatManager:clearAllChatData()
	end

end


return T()
