--- 战机生产队列管理器(单例)
local T=Class.AircraftProduceQueueMgr(LuaEventDispatcher2)

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件
T.EVT_TYPE_AIRCRAFTFACTORY_REFRESH = "evt_type_aircraftFactory_refresh"

--- 事件--生产完成
T.EVT_TYPE_AIRCRAFTFACTORY_PRODUCE_REFRESH = "evt_type_aircraftFactory_produce_refresh"


--- 推送常量
T.AIRCRAFT_FACTORY_QUEUE_MANAGER = "Aircraft_Factory_Queue_Manager"

function T:def()
	LuaEventDispatcher2.def(self)
	
	--- 队列
	-- self.queue=nil

	--- 更新计时器ID
	-- self.updateId = nil
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 初始化
-- @param aircraftProduceQueue
function T:init(aircraftProduceQueue)
	self.queue=aircraftProduceQueue
	
	self:pushBuilding()
	
	self:luaDispatchEvent(self.EVT_TYPE_RESET)
end

--- 生产完成
function T:produceComplete(id)
    local list = self:getQueueItems()
    local queueItem = list[id]

    if queueItem then
        -- 数量更新到内存了里面去，显示上这个项要移除
        local num = UserInfo.city.aircraftBox.infos[queueItem.unitType].num  --  sid
        UserInfo.city.aircraftBox.infos[queueItem.unitType].num = num + queueItem.unitNum

        AircraftManager:setAddAircraftNumBySid(queueItem.unitType,queueItem.unitNum)

        AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH) -- add by yuding
        PlayerInstance:setPlayerFightLogic()

        -- 显示
        local unitNum = queueItem.unitNum
        local sid = queueItem.unitType
        local descripe = Language:get("uc183",AircraftUtils:getRichName(Aircraft_temp[sid]),unitNum)
        MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

        Command:aircraftQueueTimeFinish()

    end

    self:removeItem(id)

    self:luaDispatchEvent(self.EVT_TYPE_AIRCRAFTFACTORY_REFRESH) -- add by yuding
end


--- 检查最小的时间
function T:checkMinTimeId()
    local syncSec=TimeUtils:getSyncMillis()/1000
    local list = self:getQueueItems()
    
    local tb = {}
    for k, v in pairs(list) do 
        if v.endTimeSec > syncSec then -- 有时间剩余
            table.insert(tb, v)
        end
    end

    if #tb > 1 then 
        table.sort(tb, function(x, y) return x.endTimeSec < y.endTimeSec end) -- 排序
    end

    if #tb >= 1 then 
        return tb[1].id
    else    
        return 0
    end
end

function T:startUpdateCheck()
	local function update()
		local list = self:getQueueItems()
	    
		local syncSec=TimeUtils:getSyncMillis()/1000
		for k,v in pairs(list) do  -- k是id
			v.remainingSec=v.endTimeSec-syncSec
            if v.endTimeSec <= syncSec then
                self:produceComplete(v.id)
			end

			self:luaDispatchEvent(self.EVT_TYPE_AIRCRAFTFACTORY_REFRESH, v) -- add by yuding
			self:luaDispatchEvent(self.EVT_TYPE_AIRCRAFTFACTORY_REFRESH..k,v)


		end
	end

	if self.updateId then
		return
	end

	local list = self:getQueueItems()
	if list then
		for _ in pairs(list) do
			self.updateId=Cocos2dxUtils:interval(update,1)

			break
		end
	end
end

--- 添加项
function T:addItem(queueItem)
	local syncSec=TimeUtils:getSyncMillis()/1000
	queueItem.remainingSec = queueItem.endTimeSec - syncSec  -- 立即刷新时间

	-- 推送
    local unitNum = queueItem.unitNum
    local sid = queueItem.unitType
    local name = Aircraft_temp[sid].name

    

    queueItem.pushId = LocalNoti:push(
		Language:get("uc284",unitNum,Language:get(name)),
		queueItem.remainingSec,
		LocalNoti.CATEGORY_BUILD,
		UserInfo.id .. self.AIRCRAFT_FACTORY_QUEUE_MANAGER .. queueItem.id
	)

	self:getQueueItems()[queueItem.id] = queueItem
	self:startUpdateCheck()
end

--- 根据id移除队列中的项
function T:removeItem(id)
	local list = self:getQueueItems()
	local info=list[id]
	if info then
		-- 取消推送
		if info.pushId then
			LocalNoti:cancel(info.pushId)
			info.pushId = nil
		end

		list[id]=nil
        self:luaDispatchEvent(self.EVT_TYPE_AIRCRAFTFACTORY_PRODUCE_REFRESH)
	end

	self:tryStopUpdate()
end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
		local list= self:getQueueItems()

		--如果队列为空,结束时间更新检查
		local isEmpty=true
		for _ in pairs(list) do
			isEmpty=false
			break
		end

		if isEmpty then
			Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId=nil
		end
	end
end

function T:stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

--- 释放
function T:release()
	self:stopUpdateCheck()
	self:releaseEventListeners()
end

--- 推送消息:(取消的话，在playerins里面，还有点击取消按钮，那里改一下)
function T:pushBuilding()
    local list = self:getQueueItems()
	local syncSec = TimeUtils:getSyncMillis()/1000
	for k,v in pairs(list) do
		if v and v.endTimeSec > syncSec then -- 升级过程中
            v.remainingSec = v.endTimeSec - syncSec

            local unitNum = v.unitNum
            local sid = v.unitType
            local name = Language:get(Aircraft_temp[sid].name)

			v.pushId = LocalNoti:push(Language:get("uc284",unitNum,name),v.remainingSec,LocalNoti.CATEGORY_BUILD, UserInfo.id .. self.AIRCRAFT_FACTORY_QUEUE_MANAGER .. v.id)
		end
	end
end

function T:getQueueItems()
	return self.queue.list
end

--- 得到对应id机库的所有队列项
function T:getAircraftQueue(id)
    local queue = self:getQueueItems()
    local queueTabel = {}
    for k,v in pairs(queue) do
		if v then
            local bid = v.bid

            if id == bid then
                queueTabel[k] = v
            end
		end
	end

    return queueTabel
end

--- 检车飞机是否生产
-- @param kind 机库的种类 exp: "one" "two"
-- @return false 没飞机生产 true 有飞机生产
function T:checkPlaneFactoryProduce(kind)
    local location = nil

    if kind == "one" then 
        location = PlayerInstance.fixedBldLocation.planeOne
    elseif kind == "two" then 
        location = PlayerInstance.fixedBldLocation.planeTwo
    end

    local lvl = PlayerInstance:getBuildingLvl(location)
    if lvl == 0 then  -- 0J
        return false
    end


    local list = AircraftProduceQueueMgr:getQueueItems()
    local infos = BuildingsInfoMgr:getBuildingInfos()

    local tmp = {}
    for k, v in pairs(infos) do 
        for m, n in pairs(list) do 
            if v.id == n.bid and location == v.location then  -- 对应的位置
                table.insert(tmp, n.unitType)
            end
        end 
    end

    if #tmp == 0 then
        return false
    else
        return true 
    end

end


return T()