--- 战斗信息管理器
local T=Class.BattlesInfoMgr(LuaEventDispatcher2)

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件-章节buff信息改变
T.EVT_TYPE_CHANGE_CHAPTER_BUFF="evt_type_change_chapter_buff"

--- 战役数据
T.Campaign_temp=require("res.data.battle.campaign")

--- 关卡buff
T.ChapterBuff_temp=require("res.data.battle.chapterBuff")

function T:def()
	LuaEventDispatcher2.def(self)

	--- 战斗信息
	-- self.info=nil
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

function T:init(battlesInfo)
	self.info=battlesInfo

	self:luaDispatchEvent(self.EVT_TYPE_RESET)
end

--- 设置章节buff信息
function T:initChapterBuffsInfo(chapterBuffsInfo)
	self.chapterBuffsInfo=chapterBuffsInfo

	self:luaDispatchEvent(self.EVT_TYPE_CHANGE_CHAPTER_BUFF)
	PlayerInstance:setPlayerFightLogic()
end

--- 战斗是否完成
function T:isBattleFinished(battleSid)
	if not battleSid or battleSid == 0 then
		return false
	end

	local key=math.floor(battleSid/32)
	local ls=BitOperator:_ls(1,battleSid%32)

	local t=self.info.battleInfos[key]
	return t and (BitOperator:_and(t,ls) ~= 0) or false
end

--- 设置战斗已经完成
-- @param battleSid 战役编号
-- @param starNum 获得的星级
function T:setBattleFinished(battleSid,starNum)
	if not battleSid or battleSid == 0 then
		return
	end

	--设置星级
	local starInfos=self.info.startInfos
	if starNum == 3 then
		starInfos[battleSid]=nil
	elseif not self:isBattleFinished(battleSid) or (starInfos[battleSid] or 3) < starNum then
		starInfos[battleSid]=starNum
	end

	--加完成标志
	local battleInfos=self.info.battleInfos

	local key=math.floor(battleSid/32)
	local value=BitOperator:_ls(1,battleSid%32)
	local t=battleInfos[key]
	if not t then
		battleInfos[key]=value
	else
		if BitOperator:_and(t,value) ~= 0 then
			return
		end
		battleInfos[key]=BitOperator:_or(t,value)
	end
end

--- 获取指定战斗(s)的星级
-- @param battleSids 要获取星级的战役或战役列表
function T:getStar(battleSids)
	local startInfos=self.info.startInfos

	local function _g(sid)
		if self:isBattleFinished(sid) then
			local starNum=startInfos[sid]
			if starNum then
				return starNum
			else
				return 3
			end
		end

		return 0
	end

	if type(battleSids) == "table" then
		local t=0
		for _,v in pairs(battleSids) do
			t=t+_g(v)
		end

		return t
	end

	return _g(battleSids)
end

--- 获取总星级
function T:getTotalStar()
	local c=0
	for _,v in pairs(self.Campaign_temp) do
		c=c+self:getStar(v.battles)
	end

	return c
end

--- 获取战役的星级奖励是否已领取,index是移动的位数0,1,2
function T:getStarAwardIsGet(campaignSid,index)
	local status=self.info.campaignAwardStatus[campaignSid]
	if status then
		return BitOperator:_and(status,BitOperator:_ls(1,index)) ~= 0  -- 未取了是false，已经领取是true
	end

	return false
end

--- 设置指定的星级是否已领取,isGet是获取的奖励品是否领取
function T:setStarAwardIsGet(campaignSid,index,isGet)
	local status=self.info.campaignAwardStatus[campaignSid]
	if not status then
		status = 0
	end

	status=BitOperator:_or(status,BitOperator:_ls(isGet and 0 or 1,index))
	self.info.campaignAwardStatus[campaignSid]=status
end

--- 获取章节buff信息
function T:getChapterBuffsInfo()
	return self.chapterBuffsInfo
end

--- 当前选择的战役
T._selCampaign=nil
function T:setSelCampaign(value)
	self._selCampaign=value
end
function T:getSelCampaign()
	return self._selCampaign
end

--- 当前选择的战斗
T._selBattleSid = nil
function T:setSelBattle(value)
	self._selBattleSid = value
end
function T:getSelBattle()
	return self._selBattleSid
end

--- 存储当前选择的战斗的星级数量
T._selBattleStar = nil
function T:setSelBattleStar(value)
	self._selBattleStar = value
end

function T:getSelBattleStar()
	if self._selBattleStar then
		return self._selBattleStar
	else
		return 0
	end
end

--- 存储当前选择的部队
T._team = nil
function T:setTeam(team)
	self._team = team
end

function T:getTeam()
	return self._team
end

--- 获取对战机攻击的加成
-- @return 加成比例
function T:getAttckAdd()
	local lv=self:getChapterBuffsInfo().infos[7]
	if lv and lv > 0 then
		local dt=self.ChapterBuff_temp[6]
		if dt then
			return dt.levelEff[lv] or 0
		end
	end

	return 0
end


--- 获取对战机血量的加成
-- @return 加成比例
function T:getHpAdd()
	local lv=self:getChapterBuffsInfo().infos[6]
	if lv and lv > 0 then
		local dt=self.ChapterBuff_temp[6]
		if dt then
			return dt.levelEff[lv] or 0
		end
	end

	return 0
end

--- 获取对战机命中的加成
-- @return 加成比例
function T:getHitAdd()
	return 0
end

--- 获取对战机闪避的加成
-- @return 加成比例
function T:getDodgeAdd()
	return 0
end

--- 获取对暴击的加成
-- @return 加成比例
function T:getCritAdd()
	return 0
end

--- 获取对暴抗的加成
-- @return 加成比例
function T:getArmorAdd()
	return 0
end

return T()
