local T=Class.BuffQueueInfoMgr(LuaEventDispatcher2)

T.BuffQueueItem=require("src.app.model2.buff.BuffQueueItem")

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件-buff刷新
T.EVT_TYPE_BUFF_REFRESH = "evt_type_buff_refresh"

--- 事件--buff结束
T.EVT_TYPE_BUFF_FINISH = "evt_type_buff_finish"


--- 事件--buff开始
T.EVT_TYPE_BUFF_START = "evt_type_buff_start"


--- 防护罩推送
T.EVT_PROTECT_COVER = "evt_protect_cover"

function T:def()
	LuaEventDispatcher2.def(self)

	--- 信息
	-- self.info=nil

	self.updateId = nil
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

function T:init(buffQueueInfo)
	self.info=buffQueueInfo
	self:startPush()

	self:luaDispatchEvent(self.EVT_TYPE_RESET)

	self:startUpdateCheck()
end

function T:startUpdateCheck()
	local function update()
		local list=self.info.list
		local syncSec=TimeUtils:getSyncMillis()/1000

		for k,v in pairs(list) do
			v.remainingSec=v.endTimeSec-syncSec

			if v.endTimeSec <= syncSec then
				-- buff用完了派发事件
				self:removeItem(k)
			else
				self:luaDispatchEvent(self.EVT_TYPE_BUFF_REFRESH, v.sid)
				self:luaDispatchEvent(self.EVT_TYPE_BUFF_REFRESH..v.sid,v)
			end

		end
	end

	if self.updateId then
		return
	end

	local list=self.info.list
	if list then
		for _ in pairs(list) do
			self.updateId=Cocos2dxUtils:interval(update,1)

			break
		end
	end
end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
		local list=self.info.list

		--如果队列为空,结束时间更新检查
		local isEmpty=true
		for _ in pairs(list) do
			isEmpty=false
			break
		end

		if isEmpty then
			Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId=nil
		end
	end
end

function T:stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

function T:getItems()
	return self.info.list
end

function T:getItem(sid)
	return self.info.list[sid]
end


function T:startPush()
	local info = self.info.list[13]

	if  info and self:checkActive(13) then -- 取消推送
		info.pushId = LocalNoti:push(
			Language:get("uc366"),
			info.remainingSec,
			LocalNoti.PROTECT_COVER,
			UserInfo.id .. self.EVT_PROTECT_COVER .. info.sid
		)
	end
end

function T:removeItem(sid)
	local list=self.info.list

    self:luaDispatchEvent(self.EVT_TYPE_BUFF_FINISH..sid,list[sid])
	self:luaDispatchEvent(self.EVT_TYPE_BUFF_FINISH, sid)


	if sid == 13 then -- 取消推送
		if list[sid] and list[sid].pushId then
			LocalNoti:cancel(list[sid].pushId)
			list[sid].pushId = nil
	end

	end

	list[sid]=nil


	self:tryStopUpdate()

end

--- 加入buff队列项,sid为buff的sid
function T:addItem(sid,endTimeSec)
	local list=self.info.list

	local queueItem = self.BuffQueueItem()
	queueItem.sid = sid
	queueItem.endTimeSec = endTimeSec
	list[sid] = queueItem

	self:startUpdateCheck()

    self:luaDispatchEvent(self.EVT_TYPE_BUFF_START..queueItem.sid,queueItem)
	self:luaDispatchEvent(self.EVT_TYPE_BUFF_START, queueItem.sid)

end

--- 检查buff是否存在,传入的是配置表里面的1到5,dao后面
function T:checkActive(sid)
	local list=self.info.list

	local syncSec=TimeUtils:getSyncMillis()/1000
	local item = list[sid]
	if item then
		local endTimeSec = item.endTimeSec
		if endTimeSec >= syncSec then
			return true
		else
			return false
		end
	else
		return false
	end
end

--- 得到主城皮肤的表，如果存在的话为一个表，如果不存在则为nil
function T:getAllShowTypeSids()
    local temp = {}
    for i = 101,4,1 do
        if self:checkActive(i) == true then
            temp[#temp + 1] = i  -- 这个是存的是buffSid

        end

    end

    if #temp > 0 then
        return temp
    else
        return nil
    end


end

--- 根据buffSid得到道具的sid
function T:getPropSidByBuffSid(buffSid)
    if buffSid == 101 then
        return 114 
    elseif buffSid == 102 then
        return 115
    elseif buffSid == 103 then
        return 116
    elseif buffSid == 104 then
        return 117
    else
        return nil
    end

end

--- 获取对战机命中的加成
-- @return 加成比例
function T:getHitAdd()
	local list,v=self.info.list,0
	if list then
		local t=Prop_temp[108]
		if t and t.buffSid and list[t.buffSid] then
			return t.percent or 0
		end
	end

	return 0
end

--- 获取对战机闪避的加成
-- @return 加成比例
function T:getDodgeAdd()
	local list,v=self.info.list,0
	if list then
		local t=Prop_temp[109]
		if t and t.buffSid and list[t.buffSid] then
			return t.percent or 0
		end
	end

	return 0
end

--- 获取对暴击的加成
-- @return 加成比例
function T:getCritAdd()
	local list,v=self.info.list,0
	if list then
		local t=Prop_temp[110]
		if t and t.buffSid and list[t.buffSid] then
			return t.percent or 0
		end
	end

	return 0
end

--- 获取对暴抗的加成
-- @return 加成比例
function T:getArmorAdd()
	local list,v=self.info.list,0
	if list then
		local t=Prop_temp[111]
		if t and t.buffSid and list[t.buffSid] then
			return t.percent or 0
		end
	end

	return 0
end

--- 获取对战机攻击的加成
-- @return 加成比例
function T:getAttckAdd()
	local list,v=self.info.list,0
	if list then
		local t=Prop_temp[106]
		if t and t.buffSid and list[t.buffSid] then
			return t.percent or 0
		end
	end

	return 0
end

--- 获取对战机生命值的加成
-- @return 加成比例
function T:getHpAdd(hp)
	return 0
end

return T()
