--- 建筑信息管理器
local T=Class.BuildingsInfoMgr(LuaEventDispatcher2)

--- 建筑物信息
T.BuildingInfo=require("src.app.model2.build.BuildingInfo")

--- 建造队列项
T.BuildQueueItem=require("src.app.model2.build.BuildQueueItem")

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

--- 事件-开始升级
T.EVT_TYPE_START_LEVEL_UP="evt_type_start_Level_up"

--- 事件-取消升级
T.EVT_TYPE_CANCEL_LEVEL_UP="evt_type_cancel_level_up"

--- 事件-升级完成
T.EVT_TYPE_LEVEL_UP_COMPLETE="evt_type_level_up_complete"

--- 事件-更新升级剩余时间:建造物唯一id
T.EVT_TYPE_UPDATE_REMAINING_TIME="evt_type_update_remaining_time"

--- 事件-更新升级剩余时间：位置
T.EVT_TYPE_UPDATE_REMAINING_TIME_2="evt_type_update_remaining_time_2"

--- 推送常量
T.BUILDIGN_QUEUE_MANAGER="Building_Queue_Manager"

function T:def()
	LuaEventDispatcher2.def(self)
	
	self.info={}
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

function T:init(info)
	self.info=info

	self:luaDispatchEvent(self.EVT_TYPE_RESET)

	--开始更新
	self:startUpdateCheck()
end

--- 添加建造队列项
-- @param id 建造队列项编号
-- @param endTimeSec 建造完成时间(秒)
-- @param location
-- @param buildSid
function T:addQueueItem(id,endTimeSec,buildingId,location,buildSid)
	local syncSec = TimeUtils:getSyncMillis()/1000
	--如果列表中没有此项信息,则添加一项
	if not self:getBuildingInfos()[location] then
		local remainingSec = endTimeSec - syncSec
		self:addInfoItem(buildSid,remainingSec,location,0,buildingId)
	end

	--添加到倒计时队列
	local item=self.BuildQueueItem()
	item.id=id
	item.endTimeSec=endTimeSec
	item.buildingId=buildingId
    item.remainingSec=item.endTimeSec-syncSec

	--更新倒计时
    local name = self:_getBuildName(item.buildingId)
	item.pushId = LocalNoti:push(Language:get("uc283",name),item.remainingSec,LocalNoti.CATEGORY_BUILD,UserInfo.id .. self.BUILDIGN_QUEUE_MANAGER .. item.id)


	self:getBuildQueue()[item.buildingId]=item
	self:startUpdateCheck()

	self:luaDispatchEvent(self.EVT_TYPE_START_LEVEL_UP,item)
    self:luaDispatchEvent(self.EVT_TYPE_START_LEVEL_UP..location) -- add by yuding
	self:luaDispatchEvent(self.EVT_TYPE_START_LEVEL_UP..item.buildingId,item)    -- 监听的是建筑物的id
end

--- 移除建造队列项
-- @param buildId 建筑物ID
function T:remvoeQueueItem(buildId)
	local list=self.info.buildQueue

    local location = nil
	for k,v in pairs(self:getBuildingInfos()) do
		if buildId and v.id == buildId then
			location = v.location
			break
		end
	end


    for k,v in pairs(list) do
		if buildId and k == buildId then
			if v.pushId then
				LocalNoti:cancel(v.pushId)
				v.pushId = nil
			end
			list[k]=nil
		end
	end

    self:luaDispatchEvent(self.EVT_TYPE_LEVEL_UP_COMPLETE)
    if location then
        self:luaDispatchEvent(self.EVT_TYPE_LEVEL_UP_COMPLETE,location)
	    self:luaDispatchEvent(self.EVT_TYPE_LEVEL_UP_COMPLETE..location,location)
    end


	self:tryStopUpdate()
end

--- 添加建筑物信息项
function T:addInfoItem(sid,remainingSec,location,level,id)
	local info=self.BuildingInfo()
	info.sid = sid
	info.remainingSec = remainingSec
	info.location = location
	info.level = level
	info.id = id

	self:getBuildingInfos()[info.location] = info
end

--- 取消建筑物升级:由于要派发事件，所以发的是位置
function T:cancelLevelUpItem(location)
	-- TODO 为什么不从队列中移除队列项呢?

	self:luaDispatchEvent(self.EVT_TYPE_CANCEL_LEVEL_UP,location)
	self:luaDispatchEvent(self.EVT_TYPE_CANCEL_LEVEL_UP..location,location)

	self:canelLocalNoti(location)
end

--- 升级完成
function T:levelUpCompleteItem(buildingId)
	local location = nil
	for k,v in pairs(self:getBuildingInfos()) do
		if v.id == buildingId then
			location = v.location
			break
		end
	end

	local info = self:getBuildingInfos()[location]
	if info then
		PlayerInstance:buildingLvlUp(location) -- 派发事件

        -- 提示
        local name = self:_getBuildName(buildingId)
        local descripe = Language:get("uc12",name,info.level)
		MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

        Command:buildQueueTimeFinish()

	end

	--移除队列项
	self:getBuildQueue()[info.id] = nil

	self:tryStopUpdate()

	self:luaDispatchEvent(self.EVT_TYPE_LEVEL_UP_COMPLETE,location)
	self:luaDispatchEvent(self.EVT_TYPE_LEVEL_UP_COMPLETE..location,location)

    --print("建筑建造完成！！ --->" .. location)

	self:canelLocalNoti(location)
end


--- 检查最小的时间
function T:checkMinTimeId()
    local syncSec=TimeUtils:getSyncMillis()/1000
    local list = self:getBuildQueue()
    
    local tb = {}
    for k, v in pairs(list) do 
        if v.endTimeSec > syncSec then -- 有时间剩余
            table.insert(tb, v)
        end
    end

    if #tb > 1 then 
        table.sort(tb, function(x, y) return x.endTimeSec < y.endTimeSec end) -- 排序
    end

    if #tb >= 1 then 
        return tb[1].buildingId
    else    
        return 0
    end
end

--- 开始更新倒计时
function T:startUpdateCheck()
	if self.updateId then
		return
	end

	local function update()
		local list = self:getBuildQueue()
	
		local syncSec=TimeUtils:getSyncMillis()/1000

		for k,v in pairs(list) do
			v.remainingSec=v.endTimeSec-syncSec

			if v.endTimeSec <= syncSec then -- 升级完成
				self:levelUpCompleteItem(v.buildingId)
			else
				self:luaDispatchEvent(self.EVT_TYPE_UPDATE_REMAINING_TIME,v)
				self:luaDispatchEvent(self.EVT_TYPE_UPDATE_REMAINING_TIME..v.buildingId,v)   -- buildingId是建筑物的id
				local location = self:getLocation(v.buildingId)
				self:luaDispatchEvent(self.EVT_TYPE_UPDATE_REMAINING_TIME_2..location,v)
			end

		end
	end

	local list = self:getBuildQueue()
	if list then
		for _ in pairs(list) do
			self.updateId=Cocos2dxUtils:interval(update,1)

			break
		end
	end

end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
		local list=self.info.buildQueue

		--如果队列为空,结束时间更新检查
		local isEmpty=true
		for _ in pairs(list) do
			isEmpty=false
			break
		end

		if isEmpty then
			Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId=nil
		end
	end
end

function T:stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

--- 推送消息:(取消的话，在playerins里面，还有点击取消按钮，那里改一下)
function T:pushBuilding()
    local list = self:getBuildQueue()
	local syncSec=TimeUtils:getSyncMillis()/1000
	for k,v in pairs(list) do
		if v and v.endTimeSec > syncSec then -- 升级过程中
            v.remainingSec = v.endTimeSec - syncSec
            local name = self:_getBuildName(v.buildingId)
			v.pushId = LocalNoti:push(Language:get("uc283",name),v.remainingSec,LocalNoti.CATEGORY_BUILD,UserInfo.id .. self.BUILDIGN_QUEUE_MANAGER .. v.id)
		end
	end
end

--- 得到建筑物的名字
function T:_getBuildName(buildingId)
    local location = nil
    local sid = nil
	for k,v in pairs(self:getBuildingInfos()) do
		if v.id == buildingId then
			location = v.location
            sid = v.sid
		end
	end
    local data = Build_temp[sid]

    local name = nil
    if sid == 5 then
		if location == 6 then
            name = Language:get("uc148")
		else
            name = Language:get("uc149")
		end
	elseif sid == 2 then
		if location == 2 then
            name = Language:get("uc150")
		else
            name = Language:get("uc151")
		end
	else
		name = Language:get(data.name)
	end

    return name
end

--- 取消本地推送
function T:canelLocalNoti(loc)
	local list = self:getBuildQueue()
	local id = self:getBuildingInfos()[loc].id
	if list[id] and list[id].pushId then
		LocalNoti:cancel(list[id].pushId)
		list[id].pushId = nil
	end
end

--- 根据id得到位置
function T:getLocation(buildingId)
	local location = nil
	for k,v in pairs(self:getBuildingInfos()) do
		if v.id == buildingId then
			location = v.location
			return location
		end
	end
end

--- 获取建筑集合信息
function T:getBuildingInfos()
	return self.info.infos
end

--- 获取建造队列信息
function T:getBuildQueue()
	return self.info.buildQueue
end

--- 是否有建筑物升级
-- @param l  建筑物位置
function T:checkBuildingUpgrade(l)
    local list = self:getBuildQueue()
	
	local syncSec = TimeUtils:getSyncMillis() / 1000

	for k, v in pairs(list) do
        v.remainingSec = v.endTimeSec - syncSec

		if v.endTimeSec > syncSec then -- 未完成
            if self:getLocation(v.buildingId) == l then 
                return v
            end
		end
	end

    return nil

end

--- 获取建造队列中最小时间的队列项
function T:getBuildQueueMostMinTime()
    local queueItem = nil
    local list = self:getBuildQueue()
    for k,v in pairs(list) do
        if v then
            if queueItem then
                if v.remainingSec and v.remainingSec < queueItem.remainingSec then
                    queueItem = mapping({}, v)
                end
            else
                queueItem = mapping({}, v)
            end

        end
        
    end

    return queueItem

end




return T()
