--- 用户道具信息管理器
local T=Class.PropsInfoMgr(LuaEventDispatcher2)

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

function T:def()
	LuaEventDispatcher2.def(self)

	--- 战斗信息
	-- self.info=nil
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

function T:init(propsInfo)
	self.info=propsInfo

	self:luaDispatchEvent(self.EVT_TYPE_RESET)
end

--- 根据物品sid 获得物品的数量
function T:getPlayerPropByID(sid)
	if sid == nil then
		return 0
	end

	local info=self.info

	if info.infos[sid] == nil then
		info.infos[sid] = {sid = sid, num = 0}
	end
	if not info.infos[sid].num then return 0 end

	return info.infos[sid].num
end

--- 根据物品id 增加物品数量
function T:addPlayerPropByID(sid, num)
	local infos=self.info.infos

	local info=infos[sid]
	if not info then
		info = {sid = sid, num = 0}
		infos[sid] = info
	end
	
	info.num = info.num + num
	if info.num == 1 then
		self:luaDispatchEvent(PlayerInstance.noticeMessage.produceProp,info)  -- 派发事件，并且传输数据
	end

	self:luaDispatchEvent(PlayerInstance.noticeMessage.prop)
	self:luaDispatchEvent(PlayerInstance.noticeMessage.prop .. sid) --派发改变物品信息
end

--- 设置物品数量
function T:setPlayerPropByID(sid, num)
	local infos=self.info.infos

	if infos[sid] == nil then
		infos[sid] = {sid = sid, num = 0}
	end
	infos[sid].num = num  -- 改变

	self:luaDispatchEvent(PlayerInstance.noticeMessage.prop)
	self:luaDispatchEvent(PlayerInstance.noticeMessage.prop .. sid) -- 派发
end

--- 根据物品sid 减少物品数量
function T:reducePlayerPropByID(sid, num)
	local infos=self.info.infos

	local n = infos[sid].num
	if n < num then
		infos[sid].num = 0
	else
		infos[sid].num = n - num
	end

	self:luaDispatchEvent(PlayerInstance.noticeMessage.prop)
	self:luaDispatchEvent(PlayerInstance.noticeMessage.prop .. sid) --派发改变物品信息
end

function T:getPropInfos()
	return self.info.infos
end

--- 兑换对应蓝图所需要的蓝图碎片sid,num
function T:exchangeBluePrint(blueSid)
    if blueSid >= 1021 and blueSid <= 1026 then
        return { sid = 206,num = 80}
    elseif blueSid >= 1031 and blueSid <= 1036 then
        return { sid = 207,num = 20}
    else
        return nil
    end
end

--- 分解的蓝图sid得到的蓝图碎片基本数量
function T:getDecomposeBluePrint(blueSid)
    local propData = Prop_temp[blueSid]
    local accessoryData = Accessory_temp[propData.makeAccessSid]
    local quality = accessoryData.quality

    if quality == 0 then
        return {sid = 206,num = 1}
    elseif quality == 1 then
        return {sid = 206,num = 3}
    elseif quality == 2 then
        return {sid = 207,num = 1}
    else
        return {sid = 207,num = 5}
    end

end


--- 获取商城的：增益，宝箱，其它的sid集合
function T:getShopSid()
	-- 数组索引：1：增益，2：宝箱，3:：其它
	local shopSid = { {106,107,108,109,110,111,105,100,101,102,103,104},
		{210,213,302,303,304}, -- 300,301没加进来
		{200,201,202,203,204,209,112,113,212}  }  -- 205,206,207,还有以后的所有的蓝图都没有放到这里的，这个不能购买，暂时没放到里面
	return shopSid
end

--- 获取物品(道具，背包)的sid集合
function T:getPropSid()
	-- 数组索引：1：增益，2：宝箱，3:：其它
	local temp = {}
	local tempPlus = {}
	local tempChest = {}
	local tempOther = {}

    local tempPropList = {}
	local propInfos = self:getPropInfos()

    -- 排除的sid集合，不在背包里面显示的
    local includeTable = {1001,1002,1003,1004,1005,1006,1011,1012,1013,1014,1015,1016,
                          1021,1022,1023,1024,1025,1026,1031,1032,1033,1034,1035,1036}

    for k,v in pairs(propInfos) do  -- 并且还要排除所有的蓝图
        if v and v.sid < 1001 then  -- 因为小于的都显示
            tempPropList[#tempPropList + 1] = v
        end
    end

    table.sort(tempPropList,function(a,b)
		return a.sid < b.sid
	end)

	for i,j in pairs(tempPropList) do
		for k,v in pairs(Prop_temp) do
			if k == j.sid and tempPropList[i].num and tempPropList[i].num > 0 then
				if v.propType == 1 then -- 增益
					table.insert(tempPlus,tempPropList[i].sid)
				elseif v.propType == 2 then -- 宝箱
					table.insert(tempChest,tempPropList[i].sid)
				else -- 0或者没有则为其它
					table.insert(tempOther,tempPropList[i].sid)
				end

			end
		end
	end


	temp[1] = tempPlus
	temp[2] = tempChest
	temp[3] = tempOther

	return temp
end

--- 得到道具或者商城表
function T:getPropOrShopList()
	local tempList = {}
	tempList[1] = {}
	tempList[2] = {}
	local propList = self:getPropSid()
	local shopList = self:getShopSid()

	-- 背包
	local temp = {}
	local num = #propList
	for i = 1,num,1 do
		local num1 = #propList[i]
		for j = 1,num1,1 do
			table.insert(temp,propList[i][j])
		end
	end
	tempList[1][1] = temp
	tempList[1][2] = propList[1]
	tempList[1][3] = propList[2]
	tempList[1][4] = propList[3]

	-- 商城
	local temp2 = {}
	local numShop = #shopList
	for i = 1,numShop,1 do
		local num1 = #shopList[i]
		for j = 1,num1,1 do
			table.insert(temp2,shopList[i][j])
		end
	end
	tempList[2][1] = temp2
	tempList[2][2] = shopList[1]
	tempList[2][3] = shopList[2]
	tempList[2][4] = shopList[3]

	return tempList

end

--- 得到所有的蓝图，sid从大到小
function T:getAllBluePrint()
    local propList = self:getPropInfos()
    local tempList = {}

    for k,v in pairs(propList) do
        if v and v.sid >= 1001 and v.sid <= 1036 and v.num > 0 then
            tempList[#tempList + 1] = v
        end
    end

    if #tempList > 0 then
        table.sort(tempList,function(a,b)
		    return a.sid > b.sid
	    end)

        return tempList
    else
        return nil  -- 表示不存在
    end


end

--- 根据蓝图sid得到所有的玩家所有的蓝图sid,sid从大到小排序
function T:getAllBluePrintSidsBySid(blueSid)
    local tempList = self:getAllBluePrint()
    if tempList then
        local list = {}
        for i = 1,#tempList,1 do
            if blueSid == tempList[i].sid then
                list[#list + 1] = tempList[i].sid
            end
        end

        if #list > 0 then
            return list
        else
            return nil
        end

    else
        return nil
    end


end

return T()
