local T=Class.TechnologiesInfoMgr(LuaEventDispatcher2)

--- 科技信息项
T.TechnologyInfo=require("src.app.model2.technology.TechnologyInfo")

--- 科技队列项
T.TechnologyQueueItem=require("src.app.model2.technology.TechnologyQueueItem")

--- 事件-开始研究
T.EVT_TYPE_START_STUDY="evt_type_start_study"

--- 事件-取消研究
T.EVT_TYPE_CANCEL_STUDY="evt_type_cancel_study"

--- 事件-研究完成
T.EVT_TYPE_STUDY_COMPLETE="evt_type_study_complete"

--- 事件-更新研究剩余时间
T.EVT_TYPE_UPDATE_REMAINING_TIME="evt_type_update_remaining_time"

--- 事件-数据被重置
T.EVT_TYPE_RESET="evt_type_reset"

function T:def()
	LuaEventDispatcher2.def(self)

	--- 信息
	-- self.info=nil

	self.updateId = nil
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

function T:init(TechnologiesInfo)
	self.info=TechnologiesInfo

	self:luaDispatchEvent(self.EVT_TYPE_RESET)

	self:startUpdateCheck()
end

--- 添加信息项
function T:addInfoItem(technoloySid,level)
	local info=self.TechnologyInfo()
	info.sid=technoloySid
	info.level=level

	self:getInfos()[info.sid]=info
end

--- 添加队列项
function T:addQueueItem(id,endTimeSec,technologySid)
	--如果列表中没有此项信息,则添加一项
	if not self:getInfos()[technologySid] then
		self:addInfoItem(technologySid,0)
	end

	--添加到倒计时队列
	local item=self.TechnologyQueueItem()
	item.id=id
	item.endTimeSec=endTimeSec
	item.technologySid=technologySid

	--更新倒计时
	local syncSec=TimeUtils:getSyncMillis()/1000
	item.remainingSec=item.endTimeSec-syncSec

	self:getQueue()[item.technologySid]=item
	self:startUpdateCheck()

	self:luaDispatchEvent(self.EVT_TYPE_START_STUDY,item)
	self:luaDispatchEvent(self.EVT_TYPE_START_STUDY..item.technologySid,item)

end

--- 取消研究
function T:cancelStudyItem(technologySid)
	self:getQueue()[technologySid]=nil

	self:tryStopUpdate()

	self:luaDispatchEvent(self.EVT_TYPE_CANCEL_STUDY,technologySid)
	self:luaDispatchEvent(self.EVT_TYPE_CANCEL_STUDY..technologySid,technologySid)

end

--- 研究完成
function T:studyCompleteItem(technologySid)
	local info=self:getInfos()[technologySid]

	if info then
		info.level = info.level + 1

        local name = Science_research_temp[technologySid].name
        local technologyLevel = info.level
        local descripe = Language:get("uc184",Language:get(name),technologyLevel)
        MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

        Command:scienceQueueTimeFinish()

	end

	local list= self:getQueue()
	list[technologySid]=nil

	self:tryStopUpdate()

	self:luaDispatchEvent(self.EVT_TYPE_STUDY_COMPLETE,technologySid)
	self:luaDispatchEvent(self.EVT_TYPE_STUDY_COMPLETE..technologySid,technologySid)

    PlayerInstance:setPlayerFightLogic()
end

--- 检查最小的时间
function T:checkMinTimeId()
    local syncSec=TimeUtils:getSyncMillis()/1000
    local list = self:getQueue()
    
    local tb = {}
    for k, v in pairs(list) do 
        if v.endTimeSec > syncSec then -- 有时间剩余
            table.insert(tb, v)
        end
    end

    if #tb > 1 then 
        table.sort(tb, function(x, y) return x.endTimeSec < y.endTimeSec end) -- 排序
    end

    if #tb >= 1 then 
        return tb[1].technologySid
    else    
        return 0
    end
end

--- 开始更新倒计时
function T:startUpdateCheck()
	if self.updateId then
		return
	end

	local function update()
		local list= self:getQueue()

		local syncSec=TimeUtils:getSyncMillis()/1000

		for k,v in pairs(list) do
			v.remainingSec=v.endTimeSec-syncSec

			if v.endTimeSec <= syncSec then
				self:studyCompleteItem(v.technologySid)
			end

			self:luaDispatchEvent(self.EVT_TYPE_UPDATE_REMAINING_TIME, v)
			self:luaDispatchEvent(self.EVT_TYPE_UPDATE_REMAINING_TIME..v.technologySid,v)
		end
	end

	if self.updateId then
		return
	end

	local list= self:getQueue()
	if list then
		for _ in pairs(list) do
			self.updateId=Cocos2dxUtils:interval(update,1)

			break
		end
	end
end

--- 尝试结束更新
function T:tryStopUpdate()
	if self.updateId then
		local list= self:getQueue()

		--如果队列为空,结束时间更新检查
		local isEmpty=true
		for _ in pairs(list) do
			isEmpty=false
			break
		end

		if isEmpty then
			Cocos2dxUtils:clearScheduler(self.updateId)
			self.updateId=nil
		end
	end
end

--- 结束更新
function T:stopUpdateCheck()
	if self.updateId then
		Cocos2dxUtils:clearScheduler(self.updateId)
		self.updateId=nil
	end
end

--- 释放
function T:release()
	self:stopUpdateCheck()
	self:releaseEventListeners()
end

function T:getInfos()
	return self.info.infos
end

function T:getQueue()
	return self.info.queue
end

--- 获取对资源产量的加成
-- @param resSid 资源ID
-- @return 加成比例
function T:getResOutputAdd(resSid)
	local info
	if resSid == 1 then
		info=self:getInfos()[1]
	elseif resSid == 2 then
		info=self:getInfos()[2]
	elseif resSid == 3 then
		info=self:getInfos()[3]
	elseif resSid == 4 then
		info=self:getInfos()[4]
	elseif resSid == 5 then
		info=self:getInfos()[5]
	end
	if info then
		return info.level * 5
	end

	return 0
end

--- 获取对建筑的建造速度的加成
-- @return 加成比例
function T:getBuildingSpeedAdd()
	local info=self:getInfos()[6]
	if info then
		return info.level * 5
	end

	return 0
end

--- 获取对仓库容量的加成
-- @return 加成比例
function T:getCapacityAdd()
	local info=self:getInfos()[7]
	if info then
		return info.level * 5
	end

	return 0
end

--- 获取对行军速度的加成
-- @return 加成比例
function T:getTeamSpeedAdd()
	local info=self:getInfos()[9]
	if info then
		return info.level * 5
	end

	return 0
end

--- 获取科技对战机攻击的加成
-- @param attackType 战机的攻击类型
-- @return 加成比例
function T:getAttckAdd(attackType)
	local info
	if attackType == AttackType.AT_1 then
		info=self:getInfos()[101]
	elseif attackType == AttackType.AT_2 then
		info=self:getInfos()[103]
	elseif attackType == AttackType.AT_3 then
		info=self:getInfos()[105]
	elseif attackType == AttackType.AT_4 then
		info=self:getInfos()[107]
	end
	if info then
		return info.level * 5
	end

	return 0
end

--- 获取对战机血量的加成
-- @param attackType 战机的攻击类型
-- @return 加成比例
function T:getHpAdd(attackType)
	local info
	if attackType == AttackType.AT_1 then
		info=self:getInfos()[102]
	elseif attackType == AttackType.AT_2 then
		info=self:getInfos()[104]
	elseif attackType == AttackType.AT_3 then
		info=self:getInfos()[106]
	elseif attackType == AttackType.AT_4 then
		info=self:getInfos()[108]
	end
	if info then
		return info.level * 5
	end

	return 0
end

--- 获取对战机命中的加成
-- @param attackType 战机的攻击类型
-- @return 加成比例
function T:getHitAdd(attackType)
	return 0
end

--- 获取对战机闪避的加成
-- @param attackType 战机的攻击类型
-- @return 加成比例
function T:getDodgeAdd(attackType)
	return 0
end

--- 获取对战机命中的加成
-- @param attackType 战机的攻击类型
-- @return 加成比例
function T:getCritAdd(attackType)
	return 0
end

--- 获取对战机命中的加成
-- @param attackType 战机的攻击类型
-- @return 加成比例
function T:getArmorAdd(attackType)
	return 0
end

--- 得到某一个科研项的等级
function T:getTechLevel(sid)
	local info = self:getInfos()[sid]
	if info then
		local level = info.level
		return level
	else
		return 0
	end
end

return T()
