--[[
 *	配件弹出窗口项：有2种显示方式
]]
local T = class("T",function() return ccui.Layout:create() end)

--- 标记：1：配件信息点进来的，2：仓库点进来的
T.tag = nil

--- 配件id
T.id = nil

T.airType = nil -- 要安装的飞机类型
T.slotType = nil -- 要安装的槽位

--- 判断是卸下还是安装,1:装配，2：卸下;装配的时候有可能有弹出窗口选择给哪个装配
T.installOrUnload = nil


--- 构造函数
function T:ctor(tag,id,installOrUnload,airType,slotType) -- 后面2个数据只有配件信息才能传进来的,因为airType,slotType是要装配的位置
    self.tag = tag
    self.id = id
    self.airType = airType
    self.slotType = slotType
    self.installOrUnload = installOrUnload

    self:init()
end

function T:init()
    self:registerScriptHandler(
        function(eventType)
            if eventType == CocosEvent.ENTER then
                self:onEnter()
            elseif eventType == CocosEvent.EXIT then
                self:onExit()
            elseif eventType == CocosEvent.CLEAN_UP then
            end
        end
    )


    self:setContentSize(cc.size(400,400))
    
    self:getWidget()

end

--- 进入
function T:onEnter()
    local slotBox = AccessorySystemManager:getSlotBox()
    local function hide()
        self.muialter2:hide()
    end
    self.hide = hide
    slotBox:luaAddEventListener(slotBox.EVT_INSTALL_ACCESSORY_REFRESH,nil,self.hide) -- 为了解决最后一种情况，弹出窗口选择飞机类型安装的配件

    -- 分解成功也要
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    accessoryBox:luaAddEventListener(accessoryBox.EVT_DECOMPOSE_ACCESSORY_REFRESH,nil,self.hide)

end

--- 退出
function T:onExit()
    local slotBox = AccessorySystemManager:getSlotBox()
    slotBox:luaRemoveEventListener(slotBox.EVT_INSTALL_ACCESSORY_REFRESH,nil,self.hide)

    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    accessoryBox:luaRemoveEventListener(accessoryBox.EVT_DECOMPOSE_ACCESSORY_REFRESH,nil,self.hide)

end


--- 得到控件
function T:getWidget()
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    local accessoryInfos = accessoryBox.accessoryInfos

    local info = accessoryInfos[self.id]
    local tempData = Accessory_temp[info.sid]

    local judgeIsLock = info:judgeIsLock() -- true解锁了，false未解锁


    -- icon
    local icon = ccui.ImageView:create()
    icon:loadTexture(IconManager.iconPath .. tempData.icon)
    --icon:setScale(IconManager.baseScale[1],IconManager.baseScale[2])
    self:addChild(icon)
    REL:toOther("left","top",icon,self,0,20)

    -- name
    local name = XMLLabel()
    name:setAlign("center","top")
    name:setContentSize(cc.size(150,36))
    self:addChild(name)
    REL:toOther("center","top",name,icon,0,105)
    name:setString(Language:get(tempData.name))

    -- 改造等级
    local reLevel = XMLLabel()
    reLevel:setAlign("left","middle")
    reLevel:setContentSize(cc.size(400,36))
    self:addChild(reLevel)
    REL:toOther("left","middle",reLevel,icon,130,30)
    reLevel:setString(Language:get("uc589",info.reLevel))

    -- 强化等级
    local level = XMLLabel()
    level:setAlign("left","middle")
    level:setContentSize(cc.size(400,36))
    self:addChild(level)
    REL:toOther("left","middle",level,icon,130,0)
    level:setString(Language:get("uc590",info.level))

    -- 血量还是攻击加成
    local add = XMLLabel()
    add:setAlign("left","middle")
    add:setContentSize(cc.size(400,36))
    self:addChild(add)
    REL:toOther("left","middle",add,icon,130,-30)

    if info:getLifeAdd() > 0 then
        add:setString(Language:get("uc591",UnitConversion.handleDecimal(info:getLifeAdd()/100)))
    else
        add:setString(Language:get("uc592",UnitConversion.handleDecimal(info:getAttackAdd()/100)))
    end

    local stepHeight = -30
    -- 是反克制，克制，暴免，还是暴伤加成
    if info:getAntiRestraintAdd() > 0 or info:getRestraintAdd() > 0 or info:getViolenceFreeAdd() > 0 or info:getViolenceInjuryAdd() > 0 then
        stepHeight = stepHeight - 30
        local addOther = XMLLabel()
        addOther:setAlign("left","middle")
        addOther:setContentSize(cc.size(400,36))
        self:addChild(addOther)
        REL:toOther("left","middle",addOther,icon,130,stepHeight)

        if info:getAntiRestraintAdd() > 0 then
            addOther:setString(Language:get("uc771",UnitConversion.handleDecimal(info:getAntiRestraintAdd()/100)))
        elseif info:getRestraintAdd() > 0 then
            addOther:setString(Language:get("uc772",UnitConversion.handleDecimal(info:getRestraintAdd()/100)))
        elseif info:getViolenceFreeAdd() > 0 then
            addOther:setString(Language:get("uc773",UnitConversion.handleDecimal(info:getViolenceFreeAdd()/100)))
        elseif info:getViolenceInjuryAdd() > 0 then
            addOther:setString(Language:get("uc774",UnitConversion.handleDecimal(info:getViolenceInjuryAdd()/100)))
        end
    end


    -- 有可能有闪避或者命中加成
    local hitOrDodgeData = info:judgeHitOrDodge()
    if hitOrDodgeData then
        if info:getHitAdd() > 0 or info:getDodgeAdd() > 0 then
            stepHeight = stepHeight - 30
            local addValue = 0
            if info:getHitAdd() > 0 then
                addValue = info:getHitAdd()
            else
                addValue = info:getDodgeAdd()
            end


            local hitOrDodge = XMLLabel()
            hitOrDodge:setAlign("left","middle")
            hitOrDodge:setContentSize(cc.size(400,36))
            self:addChild(hitOrDodge)
            REL:toOther("left","middle",hitOrDodge,icon,130,stepHeight)
            hitOrDodge:setString(Language:get(hitOrDodgeData.uc1,UnitConversion.handleDecimal(addValue)))

        end

    end




    -- 装配还是卸载
    local btn = Button.new("false","true")
	btn:setButtonState("longButton_green_normal.png","longButton_general_down.png")	
	self:addChild(btn)
	REL:toOther("left","bottom",btn,self,-20,120)
    btn:setTextShadow()
    btn:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            if self.installOrUnload == 2 then -- 卸载
                local function finish()
                    self.muialter2:hide()
                    local descripe = Language:get("uc599")
		            MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                end
                Command:unloadAccessory(self.airType,self.slotType,finish)
            else -- 装配
                if judgeIsLock == false then
                    local slotLevelLock = AccessorySystemManager.slotLevelLock
                    local slotLevel = slotLevelLock[info.slotType]
                    local descripe = Language:get("uc701",slotLevel)
		            MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    return
                end


                if self.tag == 1 then
                    local function finish()
                        self.muialter2:hide()
                        local descripe = Language:get("uc598")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                    Command:installAccessory(self.id,self.airType,self.slotType,finish)
                else
                    local slotBox = AccessorySystemManager:getSlotBox()
                    if slotBox:judgeAllAirTypeInstall(info.slotType) then -- 可以装配的飞机
                        self.airType = slotBox:judgeAllAirTypeInstall(info.slotType)

                        local function finish()
                            self.muialter2:hide()
                            local descripe = Language:get("uc598")
		                    MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                        end
                        Command:installAccessory(self.id,self.airType,info.slotType,finish)

                    else -- 当所有的飞机类型都装配完了以后，弹出窗口选择
                        local layout = require("src.app.uiScript.components.accessory.accessorySystem.InstallAccessoryPopWindow").new(self.id)
	                    local muialter2 = MUIAlter.show(
		                    {
			                    title = Language:get("uc609"),
			                    info = layout,
                                isShowCloseButton = true,
			                    flag = 4,
			                    callback = function(flag)
			                    end
		                    }
	                    )
	                    layout:setMUIAlter2(muialter2)

                    end
                end


            end
        end
    end)
    if self.installOrUnload == 2 then
        btn:setString(Language:get("uc594"))
    else -- 装配，右上角红点点，只要这种飞机类型有空位，就加红点点
        if judgeIsLock == true then
            local slotBox = AccessorySystemManager:getSlotBox()
            if slotBox:judgeAllAirTypeInstall(info.slotType) then
                local rightRed = ccui.ImageView:create()
                rightRed:loadTexture("redTip.png",ccui.TextureResType.plistType)
                self:addChild(rightRed)
                REL:toOther("right","top",rightRed,btn,-10,-10)
                self.rightRed = rightRed
            end
        else
            btn:setIsGray(true)
        end

        btn:setString(Language:get("uc593"))
    end

    -- 分解
    local decompose = Button.new("false","true")
	decompose:setButtonState("longButton_gray_normal.png","longButton_general_down.png")
    decompose:setString(Language:get("uc595"))
	self:addChild(decompose)
	REL:toOther("right","bottom",decompose,self,-20,120)
    decompose:setTextShadow()
    decompose:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            if self.tag == 1 then
                local descripe = Language:get("uc601")
		        MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
            else -- 发送分解命令：弹出分解窗口
                -- 弹出窗口
                local blueFrags = info:getDecomposeBlueFragSidAndNum()
                local decomposeCoin = info:getDecomposeBackCoin()
                local layout = require("src.app.uiScript.components.accessory.accessorySystem.DecomposeShowWindow").new({blueFrags},{type = 2,typeValue = self.id},decomposeCoin)
	            local muialter2 = MUIAlter.show(
		            {
			            title = Language:get("uc621"),
			            info = layout,
                        isShowCloseButton = true,
			            flag = 4,
			            callback = function(flag)
			            end
		            }
	            )
	            layout:setMUIAlter2(muialter2)
                
            end
        end
    end)

    if self.tag == 1 then
        decompose:setIsGray(true)
    end

    -- 强化
    local levelUp = Button.new("false","true")
	levelUp:setButtonState("longButton_green_normal.png","longButton_general_down.png")
    levelUp:setString(Language:get("uc596"))
	self:addChild(levelUp)
	REL:toOther("left","bottom",levelUp,self,-20,20)
    levelUp:setTextShadow()
    levelUp:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            levelUp:setTouchEnabled(false)
            WindowManager:push(
                require("src.app.uiScript.components.accessory.accessoryRefromAndLevelUp.AccessoryRefromAndLevelUpWindow").new(1,self.id)
            )
            
            self.muialter2:hide()

        end
    end)

    -- 改造
    local reform = Button.new("false","true")
	reform:setButtonState("longButton_green_normal.png","longButton_general_down.png")
    reform:setString(Language:get("uc597"))
	self:addChild(reform)
	REL:toOther("right","bottom",reform,self,-20,20)
    reform:setTextShadow()
    reform:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            reform:setTouchEnabled(false)
            WindowManager:push(
                require("src.app.uiScript.components.accessory.accessoryRefromAndLevelUp.AccessoryRefromAndLevelUpWindow").new(2,self.id)
            )
            self.muialter2:hide()

        end
    end)

end


--- 得到标题
function T:getTitle()
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    local accessoryInfos = accessoryBox.accessoryInfos

    local info = accessoryInfos[self.id]
    local title = info:getQualityDes()

    return title
end


function T:setMUIAlter2(muialter2)
    self.muialter2 = muialter2
end



return T