--[[
 *	蓝图项
]]
local T = class("T",function() return ccui.Layout:create() end)

--- 构造函数
function T:ctor(sid,listView)
    self.sid = sid   -- 蓝图道具的sid
    self.listView = listView


    self:init()

end

function T:init()
    self:registerScriptHandler(
        function(eventType)
            if eventType == CocosEvent.ENTER then
                self:onEnter()
            elseif eventType == CocosEvent.EXIT then
                self:onExit()
            elseif eventType == CocosEvent.CLEAN_UP then
            end
        end
    )

    local listItem = ListItem3.new()
    listItem:init()
    self:addChild(listItem)

    self.size = listItem:getContentSize()
    self:setContentSize(self.size)

    REL:toOther("left","top",listItem,self,0,0)
    self.listItem = listItem


    -- 得到json中的控件
    self:GetWidget()
    -- 设置数据
    self:SetData()
    -- 设置按钮
    self:SetButton()


end

--- 进入
function T:onEnter()
    PropsInfoMgr:luaAddEventListener(PlayerInstance.noticeMessage.prop .. self.sid,self,self.refreshData)
end

--- 退出
function T:onExit()
    PropsInfoMgr:luaRemoveEventListener(PlayerInstance.noticeMessage.prop .. self.sid,self,self.refreshData)
end

--- 得到json中的控件
function T:GetWidget()
    -- 头像
    self._imgHead = self.listItem.icon
    
    -- 分解按钮
    self._btn = self.listItem.btnUse

    -- 名字
    self._labelName = self.listItem.title

    -- 描述
    self._labelDescribe = self.listItem.describe

    -- 拥有
    self._hasUse = self.listItem.hasUse


end

function T:SetData()
    local data = Prop_temp[self.sid]
    -- 头像
    local iconPath = IconManager.iconPath
    self._imgHead:loadTexture(iconPath .. data.propIcon)

    self._labelName:setString(Language:get("uc769",Language:get(data.prop)))

    local num = PropsInfoMgr:getPlayerPropByID(self.sid)
    self._hasUse:setString(Language:get("uc277",num))

    local description = PlayerInstance:getPropDescripe2(self.sid)
    self._labelDescribe:setString(description)

    -- 分解按钮
    self._btn:loadTextures("button_blue_normal.png","button_general_down.png","",ccui.TextureResType.plistType)
    self._btn:setString(Language:get("uc645"))
    REL:toOther("right", "bottom", self._btn, self.listItem.back, 20, 0)
    self._btn:setTextShadow()

    -- 制造按钮
    local makeBtn = Button.new("false","true")
    makeBtn:loadTextures("button_blue_normal.png","button_general_down.png","",ccui.TextureResType.plistType)
    makeBtn:setString(Language:get("uc646"))
    REL:toOther("right", "bottom", makeBtn, self.listItem.back, 20, 80)
    makeBtn:setTextShadow()
    self:addChild(makeBtn)
    self.makeBtn = makeBtn

end

function T:refreshData()
    local num = PropsInfoMgr:getPlayerPropByID(self.sid)
    if num == 0 then
        local index = self.listView:getIndex(self)
        local listView = self.listView
        self.listView:removeItem(index)
        listView:refreshView()
    else
        self._hasUse:setString(Language:get("uc277",num))
    end

end


--- 按钮回调设置
function T:SetButton()
    --进入游戏回调
    local function callback(sender, eventType)
        if eventType == ccui.TouchEventType.ended then 
            if self._btn == sender then
                self:decomposeLogic()
            elseif self.makeBtn == sender then
                self:makeLogic()
            else
            end
        end
    end
    self._btn:addTouchEventListener(callback)
    self.makeBtn:addTouchEventListener(callback)

end

-- 制造逻辑
function T:makeLogic()
    local sid = self.sid
    local propData = Prop_temp[sid]

    local accessoryData = Accessory_temp[propData.makeAccessSid]

    local info = Language:get("uc647",Language:get(propData.prop),Language:get(accessoryData.name))
    local function confirm()
        local function finish()
            local descripe = Language:get("uc648")
		    MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
        end
        Command:makeAccessory(sid,finish)

    end
    MUIPop.MUIImmediate(info,confirm)


end

-- 分解逻辑
function T:decomposeLogic()
    local layout = require("src.app.uiScript.components.accessory.accessorySystem.warhouse.DecomposePopWindow").new(self.sid)
    local muialter2 = MUIAlter.show(
	    {
            title = Language:get("uc651"),
		    info = layout,
            isShowCloseButton = true,
		    flag = 4,
		    callback = function(flag)
		    end
	    }
    )
    layout:setMUIAlter2(muialter2)

end



return T