--[[
 * 抽钻石窗口
]]
local T = class(
    "T",
    function() 
        local o = AbstractWindow.new("windowBack4.jpg")
        o.super = AbstractWindow
        return o
    end
)

--- 奖励品数据解析器
T.AwardDataDecoder = require("src.app.communication.award.AwardDataDecoder")

--- 钻石信息
T.drawDiamondInfo = nil

--- 所有的抽奖数据
T.lotteryData = nil 

--- 数字项
T.DigitalItem = require("src.app.uiScript.components.allActivity.fixAndDynamic.activity.drawDiamond.DigitalItem")

--- 当前的需要的钻石数量
T.needGems = nil

--- 构造函数 
function T:ctor(drawDiamondInfo,lotteryData)
    self.drawDiamondInfo = drawDiamondInfo
    self.lotteryData = lotteryData
    if lotteryData.isLottery then
        self.needGems = lotteryData.needGems
    end
    

    self:setWindowTitle(Language:get("uc506"))

    self:initBack()
    self:initDigital()
    self:initBottom()

    -- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

function T:onEnter()
    self.super.onEnter(self)
    self.drawDiamondInfo:luaAddEventListener(self.drawDiamondInfo.EVT_ACTIVTIY_OPENVALUE_REFRESH .. self.drawDiamondInfo.sid,self,self.closeSelf)
    CommonBroadcast:luaAddEventListener(EVENT_DRAW_DIAMOND_COMPLETE,self,self.refresh)
end

function T:onExit()
    self.super.onExit(self)
    self.drawDiamondInfo:luaRemoveEventListener(self.drawDiamondInfo.EVT_ACTIVTIY_OPENVALUE_REFRESH .. self.drawDiamondInfo.sid,self,self.closeSelf)
    CommonBroadcast:luaRemoveEventListener(EVENT_DRAW_DIAMOND_COMPLETE,self,self.refresh)
end

--- 关闭当前窗口
function T:closeSelf()
    if self.drawDiamondInfo:getOpenValue() == false then
        WindowManager:pop()
    end
    
end

--- 初始化背景
function T:initBack()
    -- 背景图片
   	local f = ResLoader:getSpriteFrame("viewBack.png")
	local s = f:getOriginalSize()
	local viewBack = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,s.height-1,1,1))
	self:addChild(viewBack)
    REL:relOther(viewBack,self.uiContainer,0,0,0,0)
	self.viewBack = viewBack

    -- 背景图片2
    local diamondBack = ccui.ImageView:create()
    diamondBack:loadTexture("diamondBack.png",ccui.TextureResType.plistType)
    self:addChild(diamondBack)
    REL:toOther("center","top",diamondBack,self.viewBack,0,10)
    self.diamondBack = diamondBack

    -- 小秘书
    local sprite = SecretaryGenerator:getBust("lolita",{x=0,y=0,width=480,height=426})
    sprite:setFlippedX(true)
	self:addChild(sprite)
	REL:toOther("center","bottom",sprite,self.viewBack,30,450)
    self.sprite = sprite

end

--- 初始化数字
function T:initDigital()
    -- 背景图片
    local wheelBack = ccui.ImageView:create()
    wheelBack:loadTexture("wheelBack.png",ccui.TextureResType.plistType)
    self:addChild(wheelBack)
    wheelBack:setScaleY(300/616)
    REL:toOther("center","top",wheelBack,self.sprite,-30,420)
    self.wheelBack = wheelBack

    -- 底部图片
    local f = ResLoader:getSpriteFrame("digitalBackBotom.png")
	local s = f:getOriginalSize()
	local digitalBackBotom = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,s.height/2,1,1))
	self:addChild(digitalBackBotom)
    REL:relOther(digitalBackBotom,wheelBack,25,25,30,80)
    
    -- 添加数字项
    -- 左边和顶部
    local baseLeft = 15
    local w = 112
    local step = 0

    self.posLeft = {baseLeft ,baseLeft + (w + step)*1,
                baseLeft + (w + step)*2,baseLeft + (w + step)*3,baseLeft + (w + step)*4}
    
    self.res = {}

    for i = 1,5,1 do
        local res = self.DigitalItem.new(i)
        self:addChild(res)
        self.res[#self.res + 1] = res
        REL:toOther("left","middle",res,digitalBackBotom,self.posLeft[i],0)
    end

end

function T:initBottom()
    local lotteryData = self.lotteryData

    local diamondBottom = ccui.ImageView:create()
    diamondBottom:loadTexture("diamondBottom.png",ccui.TextureResType.plistType)
    self:addChild(diamondBottom)
    REL:toOther("center","bottom",diamondBottom,self.viewBack,0,0)
    self.diamondBottom = diamondBottom

    --- 活动按钮
    local btn = Button.new("false","false")
	btn:setButtonState("bigButton.png","bigButton.png")
	self:addChild(btn)
	REL:toOther("center","top",btn,diamondBottom,-8,50)
    btn:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local gems = UserInfo.gems
            if gems >= self.needGems then
                local function finish(data)
                    self.data = data
                    self:startScroll()            
                end
                Command:joinDynamicAct(self.drawDiamondInfo.sid,finish)
            else
                MUIPop.MUIToReCharge()
            end
        end
    end)
    self.btn = btn

    if lotteryData.isLottery then
        btn:setIsGray(false)
        btn:setTouchEnabled(true)
    else
        btn:setVisible(false)
        -- 显示：已达到最大抽取次数
        local maxCount = XMLLabel()
        maxCount:setContentSize(cc.size(500,27))
        maxCount:setAlign("center","middle")
        self:addChild(maxCount)
        REL:toOther("center","middle",maxCount,diamondBottom,0,0)
        maxCount:setString(Language:get("uc510"))

    end

    -- 买买买
    local buyText = XMLLabel()
    buyText:setContentSize(cc.size(500,27))
    buyText:setAlign("center","middle")
    btn:addChild(buyText)
    REL:toOther("center","top",buyText,btn,0,15)
    buyText:setString(Language:get("uc508"))

    -- 钻石图片+文本
    local diamond = XMLLabel()
    diamond:setContentSize(cc.size(500,36))
    diamond:setAlign("center")
    btn:addChild(diamond)
    REL:toOther("center","top",diamond,btn,0,40)
    self.diamond = diamond
    diamond:setString(Language:get("uc509",self.needGems))

--    -- 钻石文本
--    local diamondText = XMLLabel()
--    diamondText:setContentSize(cc.size(500,27))
--    diamondText:setAlign("center")
--    btn:addChild(diamondText)
--    REL:toOther("center","top",diamondText,btn,140,42)
--    self.diamondText = diamondText
--    diamondText:setString(Language:get("uc509",self.needGems))

    -- 文本
    local text = XMLLabel()
    text:setContentSize(cc.size(500,27))
    text:setHAlign("center")
    self:addChild(text)
    REL:toOther("center","top",text,self.wheelBack,0,280)
    text:setString(Language:get("uc507"))


end

--- 开始转动
function T:startScroll()
    self.uiTopBorder.uiCloseButton:setIsGray(true)
    self.uiTopBorder.uiCloseButton:setTouchEnabled(false)
    self.uiTopBorder.uiDiamondButton:setIsGray(true)
    self.uiTopBorder.uiDiamondButton:setTouchEnabled(false)

    self.btn:setIsGray(true)
    self.btn:setTouchEnabled(false)

    local data = self.data
    local res = self.res
    local endValue = UnitConversion.getEveryBit(data.showDiamond) -- 返回一个表
    for i = 1,5,1 do
        res[i]:setScrollData(endValue[i],70 + (i - 1)*(-10))
        res[i]:runAct()
    end

end


--- 当所有数字转动完以后，刷新数据，（而且钻石的数字更新也是在这里设置）
function T:refresh()
    self.uiTopBorder.uiCloseButton:setIsGray(false)
    self.uiTopBorder.uiCloseButton:setTouchEnabled(true)
    self.uiTopBorder.uiDiamondButton:setIsGray(false)
    self.uiTopBorder.uiDiamondButton:setTouchEnabled(true)

    local data = self.data
    if data.isLottery then
        self.btn:setIsGray(false)
        self.btn:setTouchEnabled(true)
        self.needGems = data.needGems
        self.diamond:setString(Language:get("uc509",self.needGems))
    else
        self.btn:setVisible(false)
        -- 显示：已达到最大抽取次数
        local maxCount = XMLLabel()
        maxCount:setContentSize(cc.size(500,27))
        maxCount:setAlign("center","middle")
        self:addChild(maxCount)
        REL:toOther("center","middle",maxCount,self.diamondBottom,0,0)
        maxCount:setString(Language:get("uc510"))
    end

    PlayerInstance:setDiamond(data.diamond)


    self.data = nil
end



return T