--[[
 *	连续签到项

]]

local T = class("T",function() return ccui.Layout:create() end)

--- 奖励品数据解析器
T.AwardDataDecoder = require("src.app.communication.award.AwardDataDecoder")

--- 大小
T.size = nil

--- 领奖天数
T.day = nil

--- 领奖的数据
T.awardData = nil

--- 构造函数
function T:ctor(day,awardData)
    self.day = day
    self.awardData = mapping({}, awardData)  -- 这里必须深拷贝,因为要改变声望的值

    self:registerScriptHandler(
        function(eventType)
            if eventType == CocosEvent.ENTER then
                self:onEnter()
            elseif eventType == CocosEvent.EXIT then
                self:onExit()
            elseif eventType == CocosEvent.CLEAN_UP then
            end
        end
    )

    --- 如果有声望要特殊处理
    self:handleReputation()

    self.size = cc.size(622,143 - 3)
    self:setContentSize(self.size)

    self:init()

end

--- 进入
function T:onEnter()
    SetStates.signManager:luaAddEventListener(SetStates.signManager.EVT_CONTINUE_ATTENDANCE_COUNT_REFRESH,self,self.rightItem)
end

--- 退出
function T:onExit()
    SetStates.signManager:luaRemoveEventListener(SetStates.signManager.EVT_CONTINUE_ATTENDANCE_COUNT_REFRESH,self,self.rightItem)
end

function T:handleReputation()
    local awardData = self.awardData
    if awardData.reputation ~= nil then
        local level = UserInfo.level
        awardData.reputation = math.ceil(level/10) * awardData.reputation
    end
end

function T:init()
    --- 背景图片
    local back = ccui.ImageView:create()
    back:loadTexture("showBottom.png",ccui.TextureResType.plistType)
    self:addChild(back)
    REL:toOther("left","top",back,self,2,0)
    self.back = back

    -- 天数的背景图片
    local dayBack = ccui.ImageView:create()
    dayBack:loadTexture("daysBottom.png",ccui.TextureResType.plistType)
    self:addChild(dayBack)
    REL:toOther("left","top",dayBack,self.back,5,15)

    --- 天数文本
    local day = XMLLabel()
    day:setAlign("center","middle")
    day:setContentSize(cc.size(150,36))
    self:addChild(day)
    REL:toOther("center","middle",day,dayBack,0,0)
    day:setString(Language:get("uc427",self.awardData.day))
    
    --- 奖励品项
    self:addAward()

    --- 右边的显示项:签到按钮或者已经签到的钩钩或者明日签到的文本
    self:getRightWidget()  -- 隐藏
    self:rightItem() 

end

--- 奖励品项
function T:addAward()
    local awardList = self.AwardDataDecoder:awardResolve2(self.awardData)
    local num = #awardList
    for i = 1,num,1 do 
        local vipLevel,vipTimes = VipManager:getContinueSignVip(self.day,awardList[i].type,awardList[i].sid)

        local item = AwardItem.new({awardData = awardList[i],sizeScale = IconManager.baseScale,vipLevel = vipLevel,vipTimes = vipTimes})
        local width = item:getContentSize().width + 30
        local offset = (i - 1) * width + 30
        self:addChild(item)
        REL:toOther("left","top",item,self.back,offset,55)
    end
end

function T:getRightWidget()
    -- 勾
    local right = ccui.ImageView:create()
    right:loadTexture("needEnoughIcon.png",ccui.TextureResType.plistType)
    self:addChild(right)
    REL:toOther("right","middle",right,self.back,55,0)
    self.right = right
    right:setVisible(false)
    
    -- 签到按钮
    local sign = Button.new("false","true")
	sign:setButtonState("button_yellow_normal.png","button_yellow_normal.png")
	sign:setString(Language:get("uc425"))
	self:addChild(sign)
	REL:toOther("right","middle",sign,self.back,20,0)
    sign:setTextShadow()
	self.sign = sign
    sign:setVisible(false)
    sign:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local function finish(awardData)
                self.AwardDataDecoder:showSuccessWindow(awardData)
                self:rightItem()
            end
            Command:continueAttendance(self.day - 1,finish)
        end
    end)

end

--- 右边的项
function T:rightItem()
    local continueAttendanceCount = SetStates.signManager.continueAttendanceCount
    local continueAttendanceGet = SetStates.signManager.continueAttendanceGet

    local awardData = self.awardData
    local day = continueAttendanceCount:getCount()
    if day >= awardData.day then -- 可以领取
        if continueAttendanceGet:judgeIsContinueSign(self.day) then -- 领取
            self.right:setVisible(true)
            self.sign:setVisible(false)
            self:removeEff()
        else -- 未领取
            self.right:setVisible(false)
            self.sign:setVisible(true)
            self:addEff()
        end

    else
        self.right:setVisible(false)
        self.sign:setVisible(false)
        self:removeEff()
    end

end


--- 添加特效
function T:addEff()
    if not self.btnEff then
        if ResLoader:getAnimation("buttonPolishingEff") then
            self.btnEff = ResLoader:createAnimationSprite("buttonPolishingEff")
            self:addChild(self.btnEff)
		    REL:toOther("center", "middle", self.btnEff, self.sign, -4, 13)
        end
    else
        
    end
end

--- 移除特效
function T:removeEff()
    if self.btnEff then
        self.btnEff:removeFromParent()
        self.btnEff = nil
    end
end

return T
