--- 联盟(未加入)窗口
-- @author blank
local T = class("AllianceHasNotWindow", function()
	return AbstractWindow.new()
end)

--- 创建联盟视图
T.CreateAllianceView=require("src.app.uiScript.components.alliance.create.CreateAllianceView")

--- 联盟列表视图
T.AllianceListView=require("src.app.uiScript.components.alliance.list.AllianceListView")

--- 联盟邀请视图
T.AllianceInviteView=require("src.app.uiScript.components.alliance.invite.AllianceInviteView")

--- 联盟切换分页值
T.switchButtonIndex = nil

--- 构造函数
function T:ctor(switchButtonIndex)
	self.switchButtonIndex = switchButtonIndex or 1

	self:init()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	AudioManager:playEffect(SoundRes.OPEN_ALLIANCE)
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)
end

--- 内存清理
function T:cleanUp()
	AbstractWindow.cleanUp(self)
	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_JOINED_ALLIANCE,self,self.joinedAlliance)
	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 初始化
function T:init()
	self:setWindowTitle(Language:get("ug600"))

	local function allianceListTouchHandler()
		print("切换到联盟列表页面")
		self:switch2AllianceListView()
	end

	local function allianceInviteView()
		print("切换到联盟邀请页面")
		self:switch2AllianceInviteView()
	end

	local function allianceCreateTouchHandler()
		print("切换到联盟创建页面")
		self:switch2AllianceCreateView()
	end

	--按钮纹理
	local buttonTextures={"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"}

	-- 页签
	local uiSwitchButtons = SwitchButtonContainer.new()
	self:addChild(uiSwitchButtons)
	REL:relOther(uiSwitchButtons,self.uiContainer,0,0,0,0)
	uiSwitchButtons:init()
	uiSwitchButtons:initButtons(
		3,
		{buttonTextures,buttonTextures,buttonTextures},
		{80,22},
		{allianceListTouchHandler,allianceInviteView,allianceCreateTouchHandler},
		{Language:get("ug606"),Language:get("ug605"),Language:get("ug604")},
		self.switchButtonIndex
	)
	self.uiSwitchButtons = uiSwitchButtons

	--默认切换到"联盟列表"页面
	if self.switchButtonIndex == 1 then
		self:switch2AllianceListView()
	elseif self.switchButtonIndex == 2 then
		self:switch2AllianceInviteView()
	else
		self:switch2AllianceCreateView()
	end

	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_JOINED_ALLIANCE,self,self.joinedAlliance)
	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 检查是否在联盟中
function T:checkHasAlliance()
	local id=UserInfo.allianceBox.myAlliance.id
	if id and id > 0 then
		WindowManager:switch(require("src.app.uiScript.components.alliance.AllianceWindow").new())
	else

	end
end

--- 已成功加入联盟
function T:joinedAlliance()
	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_JOINED_ALLIANCE,self,self.joinedAlliance)

	--切换到联盟信息界面
	require("src.app.communication.alliance.GetMyAllianceInfoCommand")():exec(function(state)
		if state == ReturnCode.SUCCESS then
			WindowManager:switch(require("src.app.uiScript.components.alliance.AllianceWindow").new())
		else
			MUIAlter_singleLine.show({
				info=ErrorCode:getErrInfo(state)
			},WindowManager:getCurrentWindow())
		end
	end)
end

--- 切换到联盟列表视图
function T:switch2AllianceListView()
	local curView=self.curView
	if not curView or curView ~= self.uiAllianceListView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAllianceListView then
			curView=self.AllianceListView()
			self:addChild(curView)
			REL:relOther(curView,self.uiSwitchButtons.viewBack,0,0,20,0)
			curView:init()
			self.uiAllianceListView=curView
		end

		self.curView=self.uiAllianceListView
		self.curView:setVisible(true)
	end
end

--- 切换到联盟邀请视图
function T:switch2AllianceInviteView()
	local curView=self.curView
	if not curView or curView ~= self.uiAllianceInviteView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAllianceInviteView then
			curView=self.AllianceInviteView()
			self:addChild(curView)
			REL:relOther(curView,self.uiSwitchButtons.viewBack,0,0,0,0)
			curView:init()
			self.uiAllianceInviteView=curView
		end

		self.curView=self.uiAllianceInviteView
		self.curView:setVisible(true)
	end
end

--- 切换到联盟创建视图
function T:switch2AllianceCreateView()
	local curView=self.curView
	if not curView or curView ~= self.uiAllianceCreateView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAllianceCreateView then
			curView=self.CreateAllianceView()
			self:addChild(curView)
			REL:relOther(curView,self.uiSwitchButtons.viewBack,0,0,0,0)
			curView:init()
			self.uiAllianceCreateView=curView
		end

		self.curView=self.uiAllianceCreateView
		self.curView:setVisible(true)
	end
end

return T
