--- 联盟科技窗口
-- @author blank
local T = class("AllianceScienceWindow", function()
	return AbstractWindow.new(1)
end)

--- 科技信息项
T.AllianceScienceItem=require("src.app.uiScript.components.alliance.science.AllianceScienceItem")

--- 构造函数
function T:ctor()
	self:init()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)
end

--- 内存清理
function T:cleanUp()
	AbstractWindow.cleanUp(self)

	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_TODAY_CONTRIBUTION_POINT_CHANGE,self,self.updateTodayContributionPoint)
	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_DONATE_NOTICE,self,self.updateDonateNoti)

	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)
	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 初始化
function T:init()
	self:setWindowTitle(Language:get("ug900"))

	-- 背景图片
	local f = ResLoader:getSpriteFrame("viewBack.png")
	local s = f:getOriginalSize()
	local viewBack = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,s.height-1,1,1))
	self:addChild(viewBack)
	REL:relOther(viewBack,self.uiContainer,0,0,0,0)  -- 73是切换按钮的高度

	-- 科技动员口号背景
	local uiNoticeBack=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("allianceScienceNotiBack.png","h3"))
	self:addChild(uiNoticeBack)
	REL:relOther(uiNoticeBack,self.uiContainer,12,12,24,SIZE_NIL)

	-- 科技动员口号
	local uiNotice=XMLLabel()
	uiNotice:setAlign("left","middle")
	self:addChild(uiNotice)
	REL:relOther(uiNotice,uiNoticeBack,40,180,20,70)
	self.uiNotice=uiNotice

	-- 今日贡献
	local uiDailyContribution=XMLLabel()
	uiDailyContribution:setAlign("center","middle")
	uiDailyContribution:setHeight(36)
	self:addChild(uiDailyContribution)
	REL:relOther(uiDailyContribution,uiNoticeBack,40,180,SIZE_NIL,16)
	self.uiDailyContribution=uiDailyContribution

	-- 修改按钮
	local uiModifyNotiButton=BaseButton:createButton(Language:get("ug930"),"button_yellow_normal.png")
	self:addChild(uiModifyNotiButton)
	REL:toOther("right","bottom",uiModifyNotiButton,uiNoticeBack,24,10)
	uiModifyNotiButton:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			local setCvs=require("src.app.uiScript.components.alliance.science.DonateNotiSetCvs")()

			MUIAlter.show({
				info=setCvs,
				title=Language:get("ug928"),
				flag=4,
				callback=function()
					local mA=UserInfo.allianceBox.myAlliance
					self.uiNotice:setString(mA.donateNotice)
				end
			},WindowManager:getCurrentWindow())
		end
	end)

	local mA=UserInfo.allianceBox.myAlliance
	if mA.post == AlliancePost.LEADER then
		uiModifyNotiButton:setVisible(true)
		uiModifyNotiButton:setTouchEnabled(true)

		REL:relOther(uiNotice,uiNoticeBack,40,180,20,70)
		REL:relOther(uiDailyContribution,uiNoticeBack,40,180,SIZE_NIL,16)
	else
		uiModifyNotiButton:setVisible(false)
		uiModifyNotiButton:setTouchEnabled(false)

		REL:relOther(uiNotice,uiNoticeBack,40,40,20,70)
		REL:relOther(uiDailyContribution,uiNoticeBack,40,40,SIZE_NIL,16)
	end

	-- 科技项列表
	local uiScienceList=MUIListTree.extend(ccui.ListView:create())
	uiScienceList:setTouchEnabled(true)
	uiScienceList:setBounceEnabled(true)
	uiScienceList:setGravity(ccui.ListViewGravity.centerHorizontal)
	uiScienceList:setItemsMargin(4)
	self:addChild(uiScienceList)
	REL:relOther(uiScienceList,self.uiContainer,6,6,176,16)
	self.uiScienceList=uiScienceList

	--	uiScienceList:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	--	uiScienceList:setBackGroundColor(cc.c3b(random(0,255),0,0))
	--	uiScienceList:setBackGroundColorOpacity(100)

	self:updateScienceList()

	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_TODAY_CONTRIBUTION_POINT_CHANGE,self,self.updateTodayContributionPoint)
	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_DONATE_NOTICE,self,self.updateDonateNoti)

	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)
	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 检查是否在联盟中
function T:checkHasAlliance()
	local id=UserInfo.allianceBox.myAlliance.id
	if id and id > 0 then

	else
		--已离开联盟

		WindowManager:pop()
	end
end

--- 被踢出联盟
function T:byKickOut()
	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)

	local l=WindowManager:getLength()-1

	WindowManager:pop()
end

--- 更新捐献公告
function T:updateDonateNoti()
	local mA=UserInfo.allianceBox.myAlliance
	self.uiNotice:setString(mA.donateNotice)
end

--- 更新今日贡献点
function T:updateTodayContributionPoint()
	local mA=UserInfo.allianceBox.myAlliance
	self.uiDailyContribution:setString(Language:get("ug929",mA.todayContributionPoint))
end

--- 刷新科技列表
function T:updateScienceList()
	self:updateDonateNoti()
	self:updateTodayContributionPoint()

	local sList=AllianceScience_temp
	if sList then
		local i,len=1,#sList

		local itemAddEff=function(item)
			item:setOpacity(0)
			item:runAction(
				cc.FadeIn:create(0.5)
			)
		end

		self:runAction(
			cc.RepeatForever:create(
				cc.Sequence:create(
					cc.DelayTime:create(0.1),
					cc.CallFunc:create( function()
						if i <= len then
							local v=sList[i]

							local node=self.uiScienceList:getRootNode()
							local item=self.AllianceScienceItem(node,v)
							node:addChild(item)

							itemAddEff(node)
						else
							self:stopAllActions()
						end

						i=i+1
					end)
				)
			)
		)

		--		for _,v in ipairs(sList) do
		--			local node=self.uiScienceList:getRootNode()
		--			local item=self.AllianceScienceItem(node,v)
		--			node:addChild(item)
		--		end
	end
end



return T
