--- 联盟商城窗口
-- @author blank
local T = class("AllianceShopWindow", function()
	return AbstractWindow.new(1)
end)

--- 联盟商店商品项
T.AllianceGoodsItem=require("src.app.uiScript.components.alliance.shop.AllianceGoodsItem")

--- 构造函数
function T:ctor()
	self:init()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	self:updateBaseInfo()
	self:updateGoodsList()
	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_CHANGE_CONTRIBUTION_POINT,self,self.updateBaseInfo)
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)

	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_CHANGE_CONTRIBUTION_POINT,self,self.updateBaseInfo)
end

function T:cleanUp()
	AbstractWindow.cleanUp(self)

	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)
	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 初始化
function T:init()
	self:setWindowTitle(Language:get("ug899"))

	--边框
	local f = ResLoader:getSpriteFrame("viewBack.png")
	local s = f:getOriginalSize()
	local uiContentBack = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,s.height-1,1,1))
	self:addChild(uiContentBack)
	REL:relOther(uiContentBack,self.uiContainer,2,2,2,2)
	self.uiContentBack=uiContentBack

	--积分背景
	local uiContributionBack=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("captionBack.png"))
	self:addChild(uiContributionBack)
	REL:toOther("center","top",uiContributionBack,self.uiContainer,0,8)

	--联盟贡献
	local uiContribution=XMLLabel()
	uiContribution:setContentSize(cc.size(400,36))
	uiContribution:setAlign("center","middle")
	self:addChild(uiContribution)
	REL:relOther(uiContribution,uiContributionBack,0,0,0,0)
	self.uiContribution=uiContribution

	--商品列表
	local uiGoodsList=ccui.ListView:create()
	uiGoodsList:setTouchEnabled(true)
	uiGoodsList:setBounceEnabled(true)
	uiGoodsList:setItemsMargin(8)
	uiGoodsList:setGravity(ccui.ListViewGravity.centerHorizontal)
	self:addChild(uiGoodsList)
	REL:relOther(uiGoodsList,self.uiContainer,4,0,96,4)
	self.uiGoodsList = uiGoodsList

	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)
	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 检查是否在联盟中
function T:checkHasAlliance()
	local id=UserInfo.allianceBox.myAlliance.id
	if id and id > 0 then

	else
		--已离开联盟

		WindowManager:pop()
	end
end

--- 被踢出联盟
function T:byKickOut()
	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)

	local l=WindowManager:getLength()-1

	WindowManager:pop()
end

--- 更新基础信息
function T:updateBaseInfo()
	self.uiContribution:setString(Language:get("ug935",AllianceManager:getContributionPoint()))
end

--- 更新商品列表
function T:updateGoodsList()
	self.uiGoodsList:removeAllItems()

	local gList=AllianceManager:getAllianceShopGoodsList()
	if not gList then
		return
	end

	local itemAddEff=function(item)
		item:setOpacity(0)
		item:runAction(
			cc.FadeIn:create(0.5)
		)
	end

	local i,len=1,#gList
	self:runAction(
		cc.RepeatForever:create(cc.Sequence:create(
			cc.DelayTime:create(0.1),
			cc.CallFunc:create( function()
				if i <= len then
					local item=self.AllianceGoodsItem(gList[i])
					self.uiGoodsList:pushBackCustomItem(item)
					itemAddEff(item)
				else
					self:stopAllActions()
				end
				i=i+1
			end)
		))
	)
end

return T
