--- 联盟窗口
-- @author blank
local T = class("AllianceWindow", function()
	return AbstractWindow.new()
end)

--- 联盟信息视图
T.AllianceInfoView=require("src.app.uiScript.components.alliance.info.AllianceInfoView")

--- 联盟功能视图
T.AllianceMemberView=require("src.app.uiScript.components.alliance.member.AllianceMemberView")

--- 联盟其它视图
T.AllianceOtherView=require("src.app.uiScript.components.alliance.other.AllianceOtherView")

--- 构造函数
function T:ctor()
	self:init()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	AudioManager:playEffect(SoundRes.OPEN_ALLIANCE)
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)
end

--- 内存清理
function T:cleanUp()
	AbstractWindow.cleanUp(self)

	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)
	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 初始化
function T:init()
	self:setWindowTitle(Language:get("ug600"))

	local function allianceInfoTouchHandler()
		print("切换到联盟信息页面")
		self:switch2AllianceInfoView()
	end

	local function allianceMemberTouchHandler()
		print("切换到联盟成员页面")
		self:switch2AllianceMemberView()
	end

	local function allianceListTouchHandler()
		print("切换到联盟其它页面")
		self:switch2AllianceOtherView()
	end

	--按钮纹理
	local buttonTextures={"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"}

	-- 页签
	local uiSwitchButtons = SwitchButtonContainer.new()
	self:addChild(uiSwitchButtons)
	REL:relOther(uiSwitchButtons,self.uiContainer,0,0,0,0)
	uiSwitchButtons:init()
	uiSwitchButtons:initButtons(
		3,
		{buttonTextures,buttonTextures,buttonTextures},
		{70,15},
		{allianceInfoTouchHandler,allianceMemberTouchHandler,allianceListTouchHandler},
		{Language:get("ug601"),Language:get("ug602"),Language:get("ug603")}
	)
	self.uiSwitchButtons = uiSwitchButtons

	--默认切换到"联盟信息"页面
	self:switch2AllianceInfoView()

	AllianceManager:luaAddEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)
	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.checkHasAlliance)
end

--- 检查是否在联盟中
function T:checkHasAlliance()
	local id=UserInfo.allianceBox.myAlliance.id
	if id and id > 0 then

	else
		--已离开联盟

		WindowManager:pop()
	end
end

--- 被踢出联盟
function T:byKickOut()
	AllianceManager:luaRemoveEventListener(AllianceManager.EVT_TYPE_BY_KICK_OUT,self,self.byKickOut)

	WindowManager:pop()
end

--- 切换到联盟信息视图
function T:switch2AllianceInfoView()
	local curView=self.curView
	if not curView or curView ~= self.uiAllianceInfoView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAllianceInfoView then
			curView=self.AllianceInfoView()
			self:addChild(curView)
			REL:relOther(curView,self.uiSwitchButtons.viewBack,0,0,0,0)
			curView:init()
			self.uiAllianceInfoView=curView
		end

		self.curView=self.uiAllianceInfoView
		self.curView:setVisible(true)
	end
end

--- 切换到联盟功能视图
function T:switch2AllianceMemberView()
	local curView=self.curView
	if not curView or curView ~= self.uiAllianceMemberView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAllianceMemberView then
			curView=self.AllianceMemberView()
			self:addChild(curView)
			REL:relOther(curView,self.uiSwitchButtons.viewBack,0,0,6,0)
			curView:init()
			self.uiAllianceMemberView=curView
		end

		self.curView=self.uiAllianceMemberView
		self.curView:setVisible(true)
	end
end

--- 切换到联盟其它视图
function T:switch2AllianceOtherView()
	local curView=self.curView
	if not curView or curView ~= self.uiAllianceOtherView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAllianceOtherView then
			curView=self.AllianceOtherView()
			self:addChild(curView)
			REL:relOther(curView,self.uiSwitchButtons.viewBack,0,0,0,0)
			curView:init()
			self.uiAllianceOtherView=curView
		end

		self.curView=self.uiAllianceOtherView
		self.curView:setVisible(true)
	end
end

return T
