--- 竞技场窗口
-- @author blank
local T = class("ArenaWindow", function()
	return AbstractWindow.new()
end)

--- 竞技场挑战页面
T.ArenaChallengeView=require("src.app.uiScript.components.arena.challenge.ArenaChallengeView")

--- 部队设置视图
T.ArenaTeamSetView=require("src.app.uiScript.components.arena.teamSet.ArenaTeamSetView")

--- 竞技场排名页面
T.ArenaRankView=require("src.app.uiScript.components.arena.rank.ArenaRankView")

--- 构造函数
function T:ctor(defaultShowSubViewIndex)
	self.defaultShowSubViewIndex=defaultShowSubViewIndex or 1

	self:init()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	AudioManager:playEffect(SoundRes.OPEN_ARENA)

	if ArenaInfoMgr:getBaseInfo() then
		ArenaInfoMgr:getFightRecordListFromServer()
	end

	CommonBroadcast:luaAddEventListener(EVT_TYPE_SWITCH_VIEW,self,self.switchViewHandler)
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)

	ArenaInfoMgr:clearBaseInfo()
	ArenaInfoMgr:clearRankInfo()

	CommonBroadcast:luaRemoveEventListener(EVT_TYPE_SWITCH_VIEW,self,self.switchViewHandler)
end

--- 内存清理
function T:cleanUp()
	AbstractWindow.cleanUp(self)

	self:cleanupChild(self.uiChallengeView)
	self:cleanupChild(self.uiTeamSetView)
	self:cleanupChild(self.uiRankView)
end

function T:cleanupChild(child)
	if child then
		if child:getParent() then
			child:removeFromParent()
		end
		child:release()
	end
end

--- 初始化
function T:init()
	self:setWindowTitle(Language:get("ug1299"))

	local arenaInfoButtonTouchHandler=function()
		self:switchChallengeView()
	end

	local arenaTeamButtonTouchHandler=function()
		self:switchTeamSetView()
	end

	local arenaRankButtonTouchHandler=function()
		self:switchRankView()
	end

	--按钮纹理
	local buttonTextures={"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"}

	-- 创建3个
	local uiSwitchButtons = SwitchButtonContainer.new()
	self:addChild(uiSwitchButtons)
	REL:relOther(uiSwitchButtons,self.uiContainer,0,0,0,0)
	uiSwitchButtons:init()
	uiSwitchButtons:initButtons(
		3,
		{buttonTextures,buttonTextures,buttonTextures},
		{70,15},
		{arenaInfoButtonTouchHandler,arenaTeamButtonTouchHandler,arenaRankButtonTouchHandler},
		{Language:get("ug1300"),Language:get("ug1301"),Language:get("ug1302")},
		self.defaultShowSubViewIndex
	)
	self.uiSwitchButtons = uiSwitchButtons
	self.viewLayer=uiSwitchButtons.viewLayer
end

--- 切换到挑战信息视图
function T:switchChallengeView()
	--如果未设置参战部队,则不能切换到挑战信息视图
	if not ArenaInfoMgr:getBaseInfo() then
		--延迟1帧调用,以便本轮状态修改完成
		AbstractUI.timeout(
			self,
			function()
				self.uiSwitchButtons:switchButtonByIndex(2)
			end,
			0
		)

		MUIAlter_singleLine.show({
			info=Language:get("ug1317"),
			showSecond=1
		})
		return
	end

	local curView=self.curView
	if not curView or curView ~= self.uiChallengeView then
		if curView then
			--			self.viewLayer:removeChild(curView,false)
			curView:removeFromParent(false)
		end

		curView=self.uiChallengeView
		if not curView then
			curView=self.ArenaChallengeView()
			curView:retain()
			self.uiChallengeView=curView
		end

		self.viewLayer:addChild(curView)
		self.curView=curView
	end
end

function T:switchViewHandler(index)
	local curWin=WindowManager:getCurrentWindow()
	if curWin then
		if curWin.__cname == "ArenaWindow" then
			self.uiSwitchButtons:switchButtonByIndex(index)
		end
	end
end

--- 切换到部队设置视图
function T:switchTeamSetView()
	local curView=self.curView
	if not curView or curView ~= self.uiTeamSetView then
		if curView then
			--			self.viewLayer:removeChild(curView,false)
			curView:removeFromParent(false)
		end

		curView=self.uiTeamSetView
		if not curView then
			curView=self.ArenaTeamSetView()
			curView:retain()
			self.uiTeamSetView=curView
		end

		self.viewLayer:addChild(curView)
		self.curView=curView
	end
end

--- 切换到排行榜信息视图
function T:switchRankView()
	--如果未设置参战部队,则不能切换到排行榜视图
	if not ArenaInfoMgr:getBaseInfo() then
		--延迟1帧调用,以便本轮状态修改完成
		AbstractUI.timeout(
			self,
			function()
				self.uiSwitchButtons:switchButtonByIndex(2)
			end,
			0
		)

		MUIAlter_singleLine.show({
			info=Language:get("ug1317"),
			showSecond=1
		})
		return
	end

	local curView=self.curView
	if not curView or curView ~= self.uiRankView then
		if curView then
			--			self.viewLayer:removeChild(curView,false)
			curView:removeFromParent(false)
		end

		curView=self.uiRankView
		if not curView then
			curView=self.ArenaRankView()
			curView:retain()
			self.uiRankView=curView
		end

		self.viewLayer:addChild(curView)
		self.curView=curView
	end
end

return T
