--- 竞技场挑战页面
local T=Class.ArenaChallengeView(
	function()
		return ccui.Layout:create()
	end,
	AbstractUI
)

--- 挑战项
T.ArenaChallengeItem=require("src.app.uiScript.components.arena.challenge.ArenaChallengeItem")

--- 条目分隔线
T.ItemSep=require("src.app.uiScript.components.arena.challenge.ItemSep")

function T:def()
	AbstractUI.def(self)
end

function T:ctor()
	AbstractUI.ctor(self)

	self:_init()
end

function T:onEnter()
	AbstractUI.onEnter(self)

	self:updateBaseInfo()
	DailyStates:luaAddEventListener(DailyStates.EVT_TYPE_UPDATE_DAILY_STATES,self,self.updateBaseInfo)
	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_RESET,self,self.updateBaseInfo)
	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_INTEGRAL,self,self.updateBaseInfo)

	SetStates:luaAddEventListener(SetStates.EVT_TYPE_UPDATE_ARENA_AWARD_GET_Y_DAY,self,self.updateGetAwardInfo)

	self:updateChallengeList()
	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_RESET_BASE_INFO,self,self.updateChallengeList)

	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_FAIL_COOLDOWN,self,self.updateChallengePoint)
end

function T:onExit()
	AbstractUI.onExit(self)

	DailyStates:luaRemoveEventListener(DailyStates.EVT_TYPE_UPDATE_DAILY_STATES,self,self.updateBaseInfo)
	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_RESET,self,self.updateBaseInfo)
	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_INTEGRAL,self,self.updateBaseInfo)

	SetStates:luaRemoveEventListener(SetStates.EVT_TYPE_UPDATE_ARENA_AWARD_GET_Y_DAY,self,self.updateGetAwardInfo)

	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_RESET_BASE_INFO,self,self.updateChallengeList)

	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_FAIL_COOLDOWN,self,self.updateChallengePoint)
end

function T:_init()
	REL:relScreen(self,6,6,162,0)
	self:setTouchEnabled(true)

	-- 显示挑战点数
	local uiCPoint=XMLLabel(Language:get("ug1303",0,GameCfg.ARENA_CHALLENGE_POINT_LIMIT))
	uiCPoint:setAlign("left","middle")
	uiCPoint:setContentSize(cc.size(400,36))
	self:addChild(uiCPoint)
	REL:toOther("left","top",uiCPoint,self,20,20)
	self._uiChallengePoint=uiCPoint

	-- 增加挑战点数按钮
	local uiAddBtn=BaseButton:createButton("","addPlayerEnergy.png","iconButton_general_down.png")
	self:addChild(uiAddBtn)
	REL:toOther("right",nil,uiAddBtn,self,120,0)
	REL:toOther(nil,"middle",uiAddBtn,uiCPoint,0,0)
	self.uiAddBtn=uiAddBtn
	uiAddBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			ArenaInfoMgr:buyChallengePoint()
		end
	end)

	-- 信息按钮
	local uiInfoBtn=BaseButton:createButton("","infoButton_normal.png","iconButton_general_down.png")
	self:addChild(uiInfoBtn)
	REL:toOther("right",nil,uiInfoBtn,self,20,0)
	REL:toOther(nil,"middle",uiInfoBtn,uiAddBtn,0,-1)
	self.uiInfoBtn=uiInfoBtn
	uiInfoBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			MUIAlter_noTitle.show({
				info=Language:get("ug1308"),
				hAlign="left"
			})
		end
	end)

	local left,top,bottom=24,80,6

	-- 积分兑换按钮
	local uiIExchangeBtn=BaseButton:createButton(Language:get("ug1311"),"bigButton_yellow_normal.png","bigButton_general_down.png")
	self:addChild(uiIExchangeBtn)
	REL:toOther("right","bottom",uiIExchangeBtn,self,430,bottom)
	self.uiIExchangeBtn=uiIExchangeBtn

	uiIExchangeBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			require("src.app.communication.arena.GetIntegralShopInfoCommand")():exec(function(state)
				if state == ReturnCode.SUCCESS then
					WindowManager:push(
						require("src.app.uiScript.components.arena.integrelShop.IntegralShopWindow").new()
					)
				else
					MUIAlter_singleLine.show({
						info=ErrorCode:getErrInfo(state)
					},WindowManager:getCurrentWindow())
				end
			end)
		end
	end)

	-- 挑战记录按钮
	local uiRecordBtn=BaseButton:createButton(Language:get("ug1312"),"bigButton_green_normal.png","bigButton_general_down.png")
	self:addChild(uiRecordBtn)
	REL:toOther("right","bottom",uiRecordBtn,self,220,bottom)
	self.uiRecordBtn=uiRecordBtn

	uiRecordBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			WindowManager:push(
				require("src.app.uiScript.components.arena.fightRecord.FightRecordListWindow").new()
			)
		end
	end)

	-- 当前积分
	local uiIntegral=XMLLabel()
	uiIntegral:setAlign("center","middle")
	uiIntegral:setHeight(36)
	self:addChild(uiIntegral)
	REL:relOther(uiIntegral,uiIExchangeBtn,-20,-20,SIZE_NIL,uiIExchangeBtn:getContentSize().height)
	self.uiIntegral=uiIntegral

	-- 领取奖励按钮
	local uiGetAwardButton=BaseButton:createButton(Language:get("ug1337"),"bigButton_yellow_normal.png","bigButton_general_down.png")
	uiGetAwardButton.touchBeganSound=SoundRes.NONE
	self:addChild(uiGetAwardButton)
	REL:toOther("right","bottom",uiGetAwardButton,self,10,bottom)
	self.uiGetAwardButton=uiGetAwardButton

	uiGetAwardButton:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:getAward()
		end
	end)


    -- 添加感叹号按钮
	self.uiGetAwardButtonTag = ResLoader:createAnimationSprite("promptEffect")
	self:addChild(self.uiGetAwardButtonTag)
	REL:toOther("right", "top", self.uiGetAwardButtonTag, self.uiGetAwardButton, 20,15)


	-- 昨日排名文本框
	local uiesterdayRank=XMLLabel()
	uiesterdayRank:setAlign("center","middle")
	uiesterdayRank:setHeight(36)
	self:addChild(uiesterdayRank)
	REL:relOther(uiesterdayRank,uiGetAwardButton,-20,-20,SIZE_NIL,uiGetAwardButton:getContentSize().height)
	self.uiesterdayRank=uiesterdayRank

	bottom=bottom+uiGetAwardButton:getContentSize().height+uiesterdayRank:getHeight()+10

	--挑战列表标题>>-----------------------------------------------------------------
	--列表标题"排名"
	local uiCapRank=XMLLabel(Language:get("ug1304"))
	uiCapRank:setAlign("center","middle")
	uiCapRank:setContentSize(cc.size(96,36))
	self:addChild(uiCapRank)
	REL:toOther("left","top",uiCapRank,self,left,top+10)
	self.uiCapRank=uiCapRank
	left=left+uiCapRank:getWidth()

	--列表标题"名称"
	local uiCapPName=XMLLabel(Language:get("ug1305"))
	uiCapPName:setAlign("center","middle")
	uiCapPName:setContentSize(cc.size(180,36))
	self:addChild(uiCapPName)
	REL:toOther("left","top",uiCapPName,self,left,top+10)
	self.uiCapPName=uiCapPName
	left=left+uiCapPName:getWidth()

	--列表标题"等级"
	local uiCapPLv=XMLLabel(Language:get("ug1306"))
	uiCapPLv:setAlign("center","middle")
	uiCapPLv:setContentSize(cc.size(120,36))
	self:addChild(uiCapPLv)
	REL:toOther("left","top",uiCapPLv,self,left,top+10)
	self.uiCapPLv=uiCapPLv
	--<<挑战列表标题-----------------------------------------------------------------

	--挑战对象列表
	local uiChallengeList=ccui.ListView:create()
	uiChallengeList:setTouchEnabled(true)
	uiChallengeList:setBounceEnabled(true)
	uiChallengeList:setItemsMargin(4)
	uiChallengeList:setGravity(ccui.ListViewGravity.centerHorizontal)
	self:addChild(uiChallengeList)
	REL:relOther(uiChallengeList,self,4,4,top+46,bottom)
	self.uiChallengeList=uiChallengeList

	-- 挑战对象列表上边框
	local uiCListTopBorder=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("border_1_top.png"),"h3")
	self:addChild(uiCListTopBorder)
	REL:relOther(uiCListTopBorder,self,4,4,top,SIZE_NIL)

	-- 挑战对象列表下边框
	local uiCListBottomBorder=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("border_1_top.png"),"h3")
	uiCListBottomBorder:setAnchorPoint(cc.p(0.5,0.5))
	uiCListBottomBorder:setRotation(180)
	self:addChild(uiCListBottomBorder)
	REL:relOther(uiCListBottomBorder,self,4,4,SIZE_NIL,bottom)
end

--- 判断是否可购买挑战点数
function T:isAbleBuyChallengePoint()
	local bNum=DailyStates:getArenaBuyNum()
	local bLimit=VipManager:getArenaChallengePointBuyLimit()
	return bNum < bLimit
end

--- 更新挑战点数
function T:updateChallengePoint()
	local remainSec=ArenaInfoMgr:getFailCooldownSec()

	local pLimit=GameCfg.ARENA_CHALLENGE_POINT_LIMIT
	local point=pLimit-DailyStates:getArenaFightNum()
	if remainSec > 0 then
		self._uiChallengePoint:setString(Language:get("ug1303",point,pLimit,Language:get("ug1318",UnitConversion.handleTime(remainSec))))
	else
		self._uiChallengePoint:setString(Language:get("ug1303",point,pLimit,""))
	end
end

--- 更新奖励领取的相关信息
function T:updateGetAwardInfo()
	local yesterdayRank=ArenaInfoMgr:getYesterdayRank()
	self.uiesterdayRank:setString(Language:get("ug1338",yesterdayRank > 0 and yesterdayRank or Language:get("ug1339")))
	if yesterdayRank > 0 and SetStates:getArenaAwardGetYDay() ~= os.date("*t",TimeUtils:getSyncMillis()/1000).yday then
		self.uiGetAwardButton:setIsGray(false)
		self.uiGetAwardButton:setTouchEnabled(true)

        -- 添加感叹号按钮
        self.uiGetAwardButtonTag:setVisible(true)

	else
		self.uiGetAwardButton:setIsGray(true)
		self.uiGetAwardButton:setTouchEnabled(false)

        self.uiGetAwardButtonTag:setVisible(false)

	end
end

--- 更新挑战相关基础信息
function T:updateBaseInfo()
	self:updateChallengePoint()
	self.uiAddBtn:setIsGray(not (self:isAbleBuyChallengePoint() and DailyStates:getArenaFightNum() > 0))

	self.uiIntegral:setString(Language:get("ug1310",ArenaInfoMgr:getIntegral()))

	self:updateGetAwardInfo()
end

--- 更新挑战列表
function T:updateChallengeList()
	self.uiChallengeList:removeAllItems()

	local bInfo=ArenaInfoMgr:getBaseInfo()
	if not bInfo then
		return
	end

	local rl=bInfo.rankList
	if rl then
		--排序
		table.sort(rl,function(a,b)
			return a.rank < b.rank
		end)

		for k,v in ipairs(rl) do
			if #self.uiChallengeList:getItems() > 0 then
				local uiSep=self.ItemSep()
				self.uiChallengeList:pushBackCustomItem(uiSep)
			end

			local item=self.ArenaChallengeItem(v)
			self.uiChallengeList:pushBackCustomItem(item)
		end
	end
end

--- 领取奖励
function T:getAward()
	if ArenaInfoMgr:getYesterdayRank() <= 0 then
		return
	end

	if SetStates:getArenaAwardGetYDay() == os.date("*t",TimeUtils:getSyncMillis()/1000).yday then
		return
	end

	require("src.app.communication.arena.GetArenaRankAwardCommand")():exec(function(state,award)
		if state == ReturnCode.SUCCESS then
			MUIAlter_singleLine.show({
				info= require("src.app.communication.award.AwardDataDecoder"):award2HTML(award),
				showSecond=1.5,
				popSnd=SoundRes.SND_SND_GET_AWARD
			})
		else
			MUIAlter_singleLine.show({
				info=ErrorCode:getErrInfo(state)
			},WindowManager:getCurrentWindow())
		end
	end)
end

return T
