local T=Class.IntegralShopItem(
	function()
		return ccui.Layout:create()
	end,
	AbstractUI
)

function T:def()
	AbstractUI.def(self)

end

function T:ctor(data)
	self._data=data
	AbstractUI.ctor(self)

	self:_init()
	self:updateView()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_INTEGRAL,self,self.updateExchangeButtonState)
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)

	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_INTEGRAL,self,self.updateExchangeButtonState)
end

function T:cleanUp()
	AbstractUI.cleanUp(self)

	if self.uiMask then
		self.uiMask:release()
		self.uiMask=nil
	end
end

function T:_init()
	local f=ResLoader:getSpriteFrame("famBack_big.png")
	local fs=f:getOriginalSize()

	self:setContentSize(fs)

	--背景图片
	local uiBack=cc.Sprite:createWithSpriteFrame(f)
	self:addChild(uiBack)
	REL:toOther("center","middle",uiBack,self,0,0)
	self.uiBack=uiBack

	--物品图标
	local uiIcon=ccui.ImageView:create()
	self:addChild(uiIcon)
	self.uiIcon=uiIcon

	--标题
	local uiTitle=XMLLabel()
	uiTitle:setContentSize(cc.size(360,36))
	uiTitle:setAlign("left","middle")
	self:addChild(uiTitle)
	REL:toOther("left","top",uiTitle,self,110,2)
	self.uiTitle=uiTitle

	--描述
	local uiDes=XMLLabel()
	uiDes:setContentSize(cc.size(340,36))
	uiDes:setAlign("left","middle")
	self:addChild(uiDes)
	REL:toOther("left","top",uiDes,self,130,80)
	self.uiDes=uiDes

	--兑换需要的积分
	local uiNeed=XMLLabel()
	uiNeed:setContentSize(cc.size(120,72))
	uiNeed:setAlign("center","middle")
	self:addChild(uiNeed)
	REL:toOther("right","top",uiNeed,self,20,16)
	self.uiNeed=uiNeed

	--"兑换"按钮
	local uiExchangeButton=BaseButton:createButton(Language:get("ug1341"),"button_blue_normal.png","button_general_down.png")
	self:addChild(uiExchangeButton)
	REL:toOther("right","middle",uiExchangeButton,self,20,-40)
	self.uiExchangeButton=uiExchangeButton

	uiExchangeButton:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:exchangeGoods()
		end
	end)

	self:updateView()
end

--- 兑换商品
function T:exchangeGoods()
	local data=self._data
	if not data then
		return
	end

	if data.rankLimit ~= 0 and ArenaInfoMgr:getRank() > data.rankLimit then
		--排名不足
		MUIAlter_singleLine.show({
			info=Language:get("ug1344"),
			showSecond=0.8
		})

		return
	end

	if ArenaInfoMgr:getIntegral() < data.needPoint then
		--积分不足
		MUIAlter_singleLine.show({
			info=Language:get("ug1346"),
			showSecond=0.8
		})

		return
	end

	require("src.app.communication.arena.ExchangeGoodsCommand")():exec(
		data,
		function(state)
			if state == ReturnCode.SUCCESS then
				MUIAlter_singleLine.show({
					info=Language:get("ug1347"),
					showSecond=0.8
				})
			else
				MUIAlter_singleLine.show({
					info=ErrorCode:getErrInfo(state)
				},WindowManager:getCurrentWindow())
			end
		end
	)
end

--- 创建锁定图标
function T:createLockIcon()
	--锁定遮罩
	local uiMask=ccui.Layout:create()
	uiMask:setTouchEnabled(true)
	uiMask:retain()
	self:addChild(uiMask)
	REL:relOther(uiMask,self, 0, 0, 0, 0)
	uiMask:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid )
	uiMask:setBackGroundColor(cc.c3b(0,0,0))
	uiMask:setBackGroundColorOpacity(140)
	self.uiMask=uiMask

	--锁定后的提示文本的背景
	local uiLockedTipBack=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("lockedBack.png"))
	uiMask:addChild(uiLockedTipBack)
--	uiLockedTipBack:setIsGray(true)
	REL:toOther("center","middle",uiLockedTipBack,uiMask,0,0)

	--锁定后的提示文本
	local uiLockedTip=XMLLabel()
	uiLockedTip:setContentSize(cc.size(500,36))
	uiLockedTip:setAlign("center","middle")
	uiMask:addChild(uiLockedTip)
	REL:toOther("center","middle",uiLockedTip,uiLockedTipBack,0,-34)
	uiMask.uiLockedTip=uiLockedTip
end

--- 更新"兑换"按钮状态
function T:updateExchangeButtonState()
	local data=self._data
	if not data or ArenaInfoMgr:getIntegral() < data.needPoint then
		self.uiExchangeButton:setIsGray(true)
    else   
        self.uiExchangeButton:setIsGray(false)
	end
end

--- 更新视图
function T:updateView()
	self:updateExchangeButtonState()

	local data=self._data

	if data then
		local propTemp=Prop_temp[data.propSid]
		if propTemp then
			self.uiIcon:loadTexture(IconManager.iconPath..propTemp.propIcon,ccui.TextureResType.localType)
			REL:toOther("left","top",self.uiIcon,self,4,21)

			self.uiTitle:setString(Language:get("ug1342",Language:get(propTemp.prop),data.propCount))
			self.uiDes:setString(PlayerInstance:getPropDescripe2(data.propSid))
			self.uiNeed:setString(Language:get("ug1343",data.needPoint))

			if data.rankLimit ~= 0 and ArenaInfoMgr:getRank() > data.rankLimit then
				if self.uiMask then
					if not self.uiMask:getParent() then
						self:addChild(self.uiMask)
					end
				else
					self:createLockIcon()
				end

				self.uiMask.uiLockedTip:setString(Language:get("ug1344",data.rankLimit))
			else
				if self.uiMask then
					self.uiMask:removeFromParent()
				end
			end
		end
	end
end

function T:setData(value)
	if value ~= self._data then
		self._data=value

		self:updateView()
	end
end

function T:getData()
	return self._data
end

return T
