--- 竞技场排名页面
local T=Class.ArenaRankView(
	function()
		return ccui.Layout:create()
	end,
	AbstractUI
)

--- 排行榜最大显示数目
T.RANK_MAX_LENGTH=100

--- 获取竞技场排行榜信息
T.GetArenaRankListCommand=require("src.app.communication.arena.GetArenaRankListCommand")

--- 列表项
T.ArenaRankItem=require("src.app.uiScript.components.arena.rank.ArenaRankItem")

--- 加载更多
T.ArenRankLoadMore=require("src.app.uiScript.components.arena.rank.ArenRankLoadMore")

function T:def()
	AbstractUI.def(self)

	--- 正在加载更多数据
	self._isLoading=false

	--- 正在刷新数据项列表
	self._isUpdating=false
end

function T:ctor()
	AbstractUI.ctor(self)

	self:_init()
end

function T:onEnter()
	AbstractUI.onEnter(self)

	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_RANK_DATA,self,self.updateList)
end

function T:onExit()
	AbstractUI.onExit(self)

	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_RANK_DATA,self,self.updateList)
end

function T:_init()
	REL:relScreen(self,6,6,162,0)

	local top,bottom=0,2

	--标题背景
	local uiCapBack=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("rankTip.png"),"h3")
	self:addChild(uiCapBack)
	REL:relOther(uiCapBack,self,4,4,top,SIZE_NIL)
	self.uiCapBack=uiCapBack

	--挑战列表标题>>-----------------------------------------------------------------
	local left=4

	--列表标题"排名"
	local uiCapRank=XMLLabel(Language:get("ug1304"))
	uiCapRank:setAlign("center","middle")
	uiCapRank:setContentSize(cc.size(120,36))
	self:addChild(uiCapRank)
	REL:toOther("left","middle",uiCapRank,uiCapBack,left,0)
	self.uiCapRank=uiCapRank
	left=left+uiCapRank:getWidth()

	--列表标题"名称"
	local uiCapPName=XMLLabel(Language:get("ug1305"))
	uiCapPName:setAlign("center","middle")
	uiCapPName:setContentSize(cc.size(240,36))
	self:addChild(uiCapPName)
	REL:toOther("left","middle",uiCapPName,uiCapBack,left,0)
	self.uiCapPName=uiCapPName
	left=left+uiCapPName:getWidth()

	--列表标题"等级"
	local uiCapPLv=XMLLabel(Language:get("ug1306"))
	uiCapPLv:setAlign("center","middle")
	uiCapPLv:setContentSize(cc.size(254,36))
	self:addChild(uiCapPLv)
	REL:toOther("left","middle",uiCapPLv,uiCapBack,left,0)
	self.uiCapPLv=uiCapPLv
	--<<挑战列表标题-----------------------------------------------------------------

	--挑战对象列表
	local uiRankList=ccui.ListView:create()
	uiRankList:setTouchEnabled(true)
	uiRankList:setBounceEnabled(true)
	uiRankList:setItemsMargin(2)
	uiRankList:setGravity(ccui.ListViewGravity.centerHorizontal)
	self:addChild(uiRankList)
	REL:relOther(uiRankList,self,2,2,top+uiCapBack:getContentSize().height+2,bottom)
	self.uiRankList=uiRankList

	ccui.ScrollView.addEventListener(uiRankList,function(t,e)
		if t:getInnerContainer():getPositionY() >= 15 then
			if not self._isUpdating then
				self:loadMore()
			end
		end
	end)

	--有基础数据才能获取排行榜信息
	if ArenaInfoMgr:getBaseInfo() then
		if ArenaInfoMgr:getRankInfo() then
		--			self:updateList(1)
		else
			self:loadMore()
		end
	end
end

--- 更新排行榜列表
function T:updateList(startIndex)
	local rInfo=ArenaInfoMgr:getRankInfo()
	local uiList=self.uiRankList
	if not rInfo or not rInfo.rankList then
		uiList:removeAllItems()
		return
	end

	local rList=rInfo.rankList

	local i,len=startIndex or 1,#rList
	if i > len then
		return
	end

	local itemAddEff=function(item)
		item:setOpacity(0)
		item:runAction(
			cc.FadeIn:create(0.5)
		)
	end

	--"加载更多"提示项在列表中的位置
	local moreItemIdx=nil
	local lItem=self.loadMoreItem
	if lItem then
		moreItemIdx=uiList:getIndex(lItem)
	end

	print("开始刷新列表")
	self._isUpdating=true
	self.pushListItemAct=self:interval(
		function()
			if i <= len then
				--添加数据项
				local vItem=self.ArenaRankItem(rList[i])

				if moreItemIdx then
					uiList:insertCustomItem(vItem,moreItemIdx)
					moreItemIdx=moreItemIdx+1
				else
					uiList:pushBackCustomItem(vItem)
				end
				itemAddEff(vItem)
			elseif i == len + 1 then
				if rInfo.isReadEnd or #rList >= self.RANK_MAX_LENGTH then
					--已到列表末尾,移除"加载更多"提示
					if lItem then
						uiList:removeItem(uiList:getIndex(lItem))
					end
				elseif not lItem then
					--未到列表末尾,且无"加载更多"提示,创建一个"加载更多"提示到末尾
					lItem=self.ArenRankLoadMore()
					self.loadMoreItem=lItem
					lItem:setTouchEnabled(true)
					lItem:addTouchEventListener(function(t,e)
						if e == ccui.TouchEventType.ended then
							self:loadMore()
						end
					end)

					uiList:pushBackCustomItem(lItem)
				end
			else
				print("刷新列表完成")
				self._isUpdating=false

				self:clearScheduler(self.pushListItemAct)
			end

			i=i+1
		end,
		0.1
	)

	--	for i=startIndex,#rList,1 do
	--		local vItem=self.AllianceListItem(rList[i])
	--
	--		uiList:pushBackCustomItem(vItem)
	--	end
	--
	--	if not rBox.isReadEnd then
	--		uiList:pushBackCustomItem(lItem)
	--		lItem:release()
	--	end
end

--- 扩展排行榜列表(触发"点击加载更多")
function T:loadMore()
	if self._isLoading then
		return
	end

	local rInfo=ArenaInfoMgr:getRankInfo()
	if rInfo and rInfo.isReadEnd then
		return
	end

	local rList=rInfo and rInfo.rankList

	local s=rList and #rList+1 or 1
	if s > self.RANK_MAX_LENGTH then
		return
	end

	local e=s + 20
	if e > self.RANK_MAX_LENGTH + 1 then
		e = self.RANK_MAX_LENGTH
	end
	print("开始加载更多数据")
	self._isLoading=true
	self.GetArenaRankListCommand():exec(
		s,
		e,
		function(state,startIndex)
			print("加载更多数据完成")
			self._isLoading=false

			if state == ReturnCode.SUCCESS then

			else
				MUIAlter_singleLine.show({
					info=ErrorCode:getErrInfo(state)
				},WindowManager:getCurrentWindow())
			end
		end
	)
end

return T
