local TeamSetView=require("src.app.uiScript.components.team.TeamSetView")

--- 竞技场战斗阵形设置
local T=Class.ArenaTeamSetView(TeamSetView)

function T:def()
	TeamSetView.def(self)
end

function T:ctor()
	TeamSetView.ctor(self)
end

function T:_init()
	TeamSetView._init(self)

	--"最大战力"按钮
	local uiMaxFightButton=BaseButton:getGeneralButton(Language:get("ug320"),1)
	self:addChild(uiMaxFightButton)
	REL:toOther("right", "bottom", uiMaxFightButton, self, 385, 28)
	self.uiMaxFightButton=uiMaxFightButton
	uiMaxFightButton:addTouchEventListener( function(t, e)
		if e == ccui.TouchEventType.ended then
			self.uiTeamFormLocs:setTeam(PlayerInstance:getMaxFightTeam(self._setableAircraftInfos))
		end
	end )

	--"保存阵形"按钮
	local uiSaveTeamButton=BaseButton:getGeneralButton(Language:get("ug1313"),2)
	self:addChild(uiSaveTeamButton)
	REL:toOther("right","bottom",uiSaveTeamButton,self,25,28)
	self.uiSaveTeamButton=uiSaveTeamButton
	uiSaveTeamButton:addTouchEventListener( function(t, e)
		if e == ccui.TouchEventType.ended then
			local team=self.uiTeamFormLocs:getTeam()

			--检查阵形是否为空
			if not team or not self:hasLocSeted(team) then
				MUIAlter.show({
					info=Language:get("ug327")
				})
				return
			end

			local oldTeam=ArenaInfoMgr:getTeam()
			--如果有已保存的阵形,则判断当前要保存的阵形的战力否比它的80%还小
			if oldTeam then
				local savedFight=self:getTeamFight(oldTeam)
				if self:getTeamFight(team) < savedFight * 0.8 then
					MUIAlter.show({
						info=Language:get("ug1314"),
						flag=3,
						callback=function(flag)
							if flag == FLAG_OK then
								self:saveTeam(team)
							end
						end
					})
					return
				end
			end

			self:saveTeam(team)
		end
	end )

	--有基础数据才能获取部队信息
	if ArenaInfoMgr:getBaseInfo() then
		if not ArenaInfoMgr:getTeam() then
			self:getTeamFromServer()
		else
			self.uiTeamFormLocs:setTeam(ArenaInfoMgr:getTeam())
		end
	end
end

function T:getTeamFromServer()
	require("src.app.communication.arena.GetArenaTeamCommand")():exec(
		function(state)
			if state == ReturnCode.SUCCESS then
				self.uiTeamFormLocs:setTeam(ArenaInfoMgr:getTeam())
			else
				MUIAlter_singleLine.show({
					info=ErrorCode:getErrInfo(state)
				},WindowManager:getCurrentWindow())
			end
		end
	)
end

--- 设置阵形
function T:saveTeam(team)
	require("src.app.communication.arena.SaveArenaTeamCommand")():exec(team,function(state)
		if state == ReturnCode.SUCCESS then
			local lastBInfo=ArenaInfoMgr:getBaseInfo()
			require("src.app.communication.arena.GetArenaInfoCommand")():exec(function(state)
				if state == ReturnCode.SUCCESS then
					if not lastBInfo then
						CommonBroadcast:luaDispatchEvent(EVT_TYPE_SWITCH_VIEW,1)
					end
				else
					MUIAlter_singleLine.show({
						info=ErrorCode:getErrInfo(state)
					},WindowManager:getCurrentWindow())
				end
			end)
			--ArenaInfoMgr:getFightRecordListFromServer()

			MUIAlter_singleLine.show({
				info=Language:get("ug1315"),
				showSecond=1
			})
		else
			MUIAlter_singleLine.show({
				info=ErrorCode:getErrInfo(state)
			},WindowManager:getCurrentWindow())
		end
	end)
end

--- 获取阵形中是否有阵位被设置
function T:hasLocSeted(team)
	for _,v in pairs(team) do
		if v and v.sid > 0 and v.count > 0 then
			return true
		end
	end

	return false
end

--- 获取阵形的战力
function T:getTeamFight(team)
	local fight=0
	for k,v in pairs(team) do
		fight=fight+PlayerInstance:getAircraftFight(v,v.count)
	end

	return fight
end

return T
