---部队出征窗口
--@author blank
local T = class("ArmyWindow", function()
	return AbstractWindow.new()
end)

--- 部队设置视图
T.ArmyTeamSetView=require("src.app.uiScript.components.army.ArmyTeamSetView")

--- 部队任务视图
T.ArmyTaskView=require("src.app.uiScript.components.army.armyTask.ArmyTaskView")

--- 战机修复视图
T.AircraftFixView=require("src.app.uiScript.components.army.aircraftFix.AircraftFixView")

--- 视图切换按钮列表
--T.uiSwitchButtons=nil

--- 部队出征视图
--T.uiBuildView=nil

--- 部队任务视图
--T.uiArmyTaskView=nil

--- 战机修复视图
--T.uiAircraftFixView=nil

--- 当前视图
--T.curView=nil

--- 战斗数据
--T.battleInfo=nil

--- 信息
--T.info = nil

--- 上次播放的背景音乐的 url
--T._lastMusicUrl=nil

--- 构造函数
--- info:{tag = 1, detailInfo = ... } 1：表示正常进入，防守部队，2：表示进入战斗(关卡) 3：表示派遣任务,4:表示运输线挑战,5:补给线
function T:ctor(info,subViewIndex)
	subViewIndex=subViewIndex or 1
	self.info = info
	self.subViewIndex=subViewIndex

	self:init()
	self:updateArmyTaskTag()
	self:updateArmyRepairTag()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BATTLE_REMAIN_TIME, self, self.updateArmyTaskTag)
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.updateArmyTaskTag)

	AircraftManager:luaAddEventListener(AircraftManager.EVT_TYPE_REPAIR_ARMY, self, self.updateArmyRepairTag)

	local tag=self.info.tag
	if tag == 2 then
		--章节战斗
		local lMusic=AudioManager:getCurrentMusicUrl()
		if lMusic ~= SoundRes.MUSIC_CHAPTER then
			self._lastMusicUrl=lMusic
			AudioManager:playMusic(SoundRes.MUSIC_CHAPTER,true)
		end
	end
end

--- 退出场景
function T:onExit()
	AbstractWindow.onExit(self)

	--- 发送设置防御部队命令
	if self.info.tag == 1 and self.uiBuildView then
		local oldTeam=UserInfo.city.aircraftBox:getDefend()
		local team = self.uiBuildView.uiFormSet:getTeam()

		local isNeedSave=false
		if team and oldTeam then
			for i=1,6,1 do
				local item1,item2=oldTeam[i],team[i]

				if
					not item1 and item2
					or
					item1 and not item2
					or
					(
					item1 and item2
					and
					(item1.sid ~= item2.sid or item1.count ~= item2.count)
					)
				then
					isNeedSave=true
					break
				end
			end
		elseif team and not oldTeam then
			isNeedSave=true
		elseif not team and oldTeam then
			isNeedSave=true
		end

		if isNeedSave then
			print("保存防御阵形")
			Command:setDefence(team or {})
		end
	end

	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BATTLE_REMAIN_TIME, self, self.updateArmyTaskTag)
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.updateArmyTaskTag)

	AircraftManager:luaRemoveEventListener(AircraftManager.EVT_TYPE_REPAIR_ARMY, self, self.updateArmyRepairTag)

	local tag=self.info.tag
	if tag == 2 then
		if self._lastMusicUrl then
			AudioManager:playMusic(self._lastMusicUrl,self._lastMusicUrl ~= SoundRes.MUSIC_GENERAL)
		end
	end
end

--- 内存清理
function T:cleanUp()
	AbstractWindow.cleanUp(self)
end

--- 初始化
function T:init()
	local function armyBattleButtonTouchHandler()
		self:switchArmyBattleView()
	end

	local function execTaskButtonTouchHandler()
		self:switchArmyTaskView()
	end

	local function fixAircraftButtonTouchHandler()
		self:switchAircraftFixView()
	end

	--按钮纹理
	local buttonTextures={"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"}

	-- 创建3个
	local uiSwitchButtons = SwitchButtonContainer.new()
	self:addChild(uiSwitchButtons)
	REL:relOther(uiSwitchButtons,self.uiContainer,0,0,0,0)
	uiSwitchButtons:init()
	uiSwitchButtons:initButtons(
		3,
		{buttonTextures,buttonTextures,buttonTextures},
		{70,15},
		{armyBattleButtonTouchHandler,execTaskButtonTouchHandler,fixAircraftButtonTouchHandler},
		{Language:get("ug301"),Language:get("ug302"),Language:get("ug303")},
		self.subViewIndex
	)
	self.uiSwitchButtons = uiSwitchButtons
	self.viewLayer=uiSwitchButtons.viewLayer

	self:setData()

	local i=self.subViewIndex
	if i == 1 then
		--默认切换到"部队出征"页面
		self:switchArmyBattleView()
	elseif i == 2 then
		self:switchArmyTaskView()
	elseif i == 3 then
		self:switchAircraftFixView()
	end
end

function T:setData()
	local tag = self.info.tag
	if tag == 1 then
		self:setWindowTitle(Language:get("uc202"))
		self.uiSwitchButtons:setButtonText(1,Language:get("ug301"))
	elseif tag == 2 then
		self:setWindowTitle(Language:get("uc201"))
		self.uiSwitchButtons:setButtonText(1,Language:get("ug301"))
	elseif tag == 3 then
		self:setWindowTitle(Language:get("ug300"))
		self.uiSwitchButtons:setButtonText(1,Language:get("uc203"))
	else
		self:setWindowTitle(Language:get("uc387"))
		self.uiSwitchButtons:setButtonText(1,Language:get("uc388"))
	end
end

--- 更新任务标识
function T:updateArmyTaskTag()
	self.uiSwitchButtons:updateButton2Tag(ArmyTaskQueueManager:judgeIsTask())
end

--- 更新修理标识
function T:updateArmyRepairTag()
	self.uiSwitchButtons:updateButton3Tag(AircraftManager:judgeAirIsRepair())
end

--- 获得特效
function T:getEff(kind)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return
	end

	local ac = cc.RepeatForever:create(cc.Animate:create(ani))
	s:runAction(ac)
	return s

end

--- 切换到部队出征视图
function T:switchArmyBattleView()
	local curView=self.curView
	if not curView or curView ~= self.uiBuildView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiBuildView then
			curView=self.ArmyTeamSetView(self.info)
			self.viewLayer:addChild(curView)
			self.uiBuildView=curView
		end

		self.curView=self.uiBuildView
		self.curView:setVisible(true)
	end
end

--- 切换到部队任务视图
function T:switchArmyTaskView()
	local curView=self.curView
	if not curView or curView ~= self.uiArmyTaskView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiArmyTaskView then
			curView=self.ArmyTaskView()
			self.viewLayer:addChild(curView)
			REL:toOther("center","bottom",curView,self.uiContainer,0,10)
			self.uiArmyTaskView=curView
		end

		self.curView=self.uiArmyTaskView
		self.curView:setVisible(true)
	end
end

--- 切换到战机修复视图
function T:switchAircraftFixView()
	local curView=self.curView
	if not curView or curView ~= self.uiAircraftFixView then
		if curView then
			curView:setVisible(false)
		end

		if not self.uiAircraftFixView then
			curView=self.AircraftFixView()
			self.viewLayer:addChild(curView)
			REL:toOther("center","bottom",curView,self.uiContainer,0,10)
			self.uiAircraftFixView=curView
		end

		self.curView=self.uiAircraftFixView
		self.curView:setVisible(true)
	end
end

return T
