--[[
 *	阵型弹出窗口
]]
local T = class("T",function() return ccui.Layout:create() end)

--- 构造函数
function T:ctor(team)
    self.team = team

    self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)


    local size = cc.size(570,700)
    self.size = self:setContentSize(size)
    self.size = size

    self:setTouchEnabled(true)

    self:init()

end

--- 进入场景
function T:onEnter()
    local formationBox = FormationManager.formationBox
    formationBox:luaAddEventListener(formationBox.EVT_CHANGE_DEFAULT__FORMATION_REFRESH,self,self.defalutLogic)


end

--- 退出场景
function T:onExit()
    local formationBox = FormationManager.formationBox
    formationBox:luaRemoveEventListener(formationBox.EVT_CHANGE_DEFAULT__FORMATION_REFRESH,self,self.defalutLogic)

end

--- 内存清理
function T:cleanUp()
    
end


function T:init()
    self.chooseFormation = 0 -- 当前显示的阵型

    self:initCenter()
    self:initBottom()

    self:refreshData()

end

--- 初始化中间
function T:initCenter()
    local pageView = ccui.PageView:create()
    pageView:setTouchEnabled(true)
    pageView:setCustomScrollThreshold(60)
    pageView:setContentSize(cc.size(550,600))
    self:addChild(pageView)
    REL:toOther("center","top",pageView,self,0,0)
    self.pageView = pageView

    self.formationItems = {}
    for i = 1,4,1 do
        local formationItem = require("src.app.uiScript.components.army.fromation.FormationItem").new(i - 1)
        pageView:addPage(formationItem)
        self.formationItems[#self.formationItems + 1] = formationItem
    end 
    


    local function pageViewEvent(sender, eventType)
        if eventType == ccui.PageViewEventType.turning then
            local index =  pageView:getCurPageIndex()
            self.chooseFormation = index
            self:refreshData()
            self.formationItems[index + 1]:aircraftChange()
        end
    end

    pageView:addEventListener(pageViewEvent)

end


--- 初始化底部
function T:initBottom()
    -- 默认按钮
    local defaultBtn = Button.new("false","true")
	defaultBtn:setButtonState("longButton_green_normal.png","longButton_general_down.png")
    defaultBtn:setString(Language:get("uc953"))
	self:addChild(defaultBtn)
	REL:toOther("center","bottom",defaultBtn,self,-200,20)
    defaultBtn:setTextShadow()
    defaultBtn:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local function finish()
                local descripe = Language:get("uc949")
				MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
            end
            Command:setDefalutFormation(self.chooseFormation,finish)
        end
    end)
    self.defaultBtn = defaultBtn

    -- 覆盖按钮:设置按钮
    local setBtn = Button.new("false","true")
	setBtn:setButtonState("longButton_green_normal.png","longButton_general_down.png")
    setBtn:setString(Language:get("uc954"))
	self:addChild(setBtn)
	REL:toOther("center","bottom",setBtn,self,0,20)
    setBtn:setTextShadow()
    setBtn:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local function finish()
                local descripe = Language:get("uc956")
				MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
            end
            if self.team == nil then
                self.team = {}
            end
            Command:setFormation(self.chooseFormation,self.team,finish)
        end
    end)
    self.setBtn = setBtn

    -- 使用按钮
    local useBtn = Button.new("false","true")
	useBtn:setButtonState("longButton_green_normal.png","longButton_general_down.png")
    useBtn:setString(Language:get("uc955"))
	self:addChild(useBtn)
	REL:toOther("center","bottom",useBtn,self,200,20)
    useBtn:setTextShadow()
    useBtn:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local formationBox = FormationManager.formationBox
            -- 但是阵型不能超过战机数
            if formationBox:judgeAircraftNumIsEngough(self.chooseFormation) == true then
                local descripe = Language:get("uc948")
				MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

            else
                self.muialter:hide()
                formationBox:useFormation(self.chooseFormation)
                local descripe = Language:get("uc957")
				MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

            end
        end
    end)
    self.useBtn = useBtn

    -- vip特权按钮
    local vipBtn = Button.new("false","true")
	vipBtn:setButtonState("longButton_green_normal.png","longButton_general_down.png")
    vipBtn:setString(Language:get("uc332"))
	self:addChild(vipBtn)
	REL:toOther("center","bottom",vipBtn,self,0,20)
    vipBtn:setTextShadow()
    vipBtn:addTouchEventListener(function(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            self.muialter:hide()
            WindowManager:push(
				require("src.app.uiScript.components.vip.VIPWindow").new()
			)

        end
    end)
    self.vipBtn = vipBtn

end


--- 翻页后刷新数据
function T:refreshData()
    local data = FormationManager.formationBox:getFormationIsOpen(self.chooseFormation)
    if data.isOpen == true then -- 3个按钮
        self.defaultBtn:setVisible(true)
        self.setBtn:setVisible(true)
        self.useBtn:setVisible(true)
        self.vipBtn:setVisible(false)
    else -- 1个vip特权按钮
        self.defaultBtn:setVisible(false)
        self.setBtn:setVisible(false)
        self.useBtn:setVisible(false)
        self.vipBtn:setVisible(true)
    end

    self:defalutLogic()



end

--- 根据是否是默认阵型刷新
function T:defalutLogic()
    if self.chooseFormation == FormationManager.formationBox.defaultFormationId then
        self.defaultBtn:setTouchEnabled(false)
        self.defaultBtn:setIsGray(true)
    else
        self.defaultBtn:setTouchEnabled(true)
        self.defaultBtn:setIsGray(false)
    end

end


function T:setMUIAlter(muialter)
    self.muialter = muialter
end



return T
