local HollowMask=require("src.frameworks.cocos.ui.HollowMask")

--- 新手引导镂空遮罩
local T=Class.GuideHollowMask(HollowMask)

local ll,lr,lt,lb,lFT=0,0,0,0,nil

function T:def()
	HollowMask.def(self)

	--- 跳过按钮
	-- _uiSkipButton

	--- 引导箭头
	-- _uiGuideArrow

	--- 浮动提示框
	-- _uiFloatTipBox

	--- 镂空Action
	-- _hollowAct

	--- 在遮罩是点击的次数累计
	self._touchMaskCount=0
end

function T:ctor()
	HollowMask.ctor(self)
end

function T:onExit()
	HollowMask.onExit(self)

    local uiArrow=self._uiGuideArrow
	if uiArrow and uiArrow:getParent() then
		uiArrow:removeFromParent()
	end

    local uiFTBox=self._uiFloatTipBox
	if uiFTBox and uiFTBox:getParent() then
		uiFTBox:removeFromParent()
	end

	ll,lr,lt,lb,lFT=0,0,0,0,nil
end

function T:init()
	HollowMask.init(self)

	--	local _uiSkipButton=BaseButton:createButton(Language:get("ug1219"),"button_green_normal.png","button_general_down.png")
	--	self:addChild(_uiSkipButton,2)
	--	REL:toScreen("right","bottom",_uiSkipButton,4,4)
	--
	--	_uiSkipButton:addTouchEventListener(function(t,e)
	--		if e == ccui.TouchEventType.ended then
	--			BGMgr:skip()
	--		end
	--	end)
end

--- 释放
function T:releaseAll()
	self:_releaseGuideArrow()
	self:_releaseFloatTipBox()
	self:_realeaseGuideCircle()
end

--- 创建引导箭头
function T:_createGuideArrow()
	local uiArrow=cc.Sprite:create("res/images/guide/guideArrow.png")
	uiArrow:setScale(1.5)
	uiArrow:setAnchorPoint({x=0.5,y=0})
	REL:toScreen("center","middle",uiArrow,0,0)
	uiArrow:setRotation(180)

	uiArrow:setLocalZOrder(TOP_ORDER + 200)
	uiArrow:retain()

	self._uiGuideArrow=uiArrow

	return uiArrow
end

--- 释放引导箭头
function T:_releaseGuideArrow()
	local uiArrow=self._uiGuideArrow
	if uiArrow then
		if uiArrow:getParent() then
			uiArrow:removeFromParent()
		end
		uiArrow:release()
	end
	self._uiGuideArrow=nil
end

--- 创建圆圈
function T:_createGuideCircle()
	local uiGuideCircle=cc.Sprite:create("res/images/guide/guideCircle.png")
	uiGuideCircle:setAnchorPoint({x=0.5,y=0.5})
	REL:toScreen("center","middle",uiGuideCircle,0,0)

	uiGuideCircle:setLocalZOrder(TOP_ORDER + 200)
	uiGuideCircle:retain()
	self._uiGuideCircle=uiGuideCircle

	uiGuideCircle:runAction(cc.RepeatForever:create(cc.Sequence:create(
		cc.DelayTime:create(0.04),
		cc.CallFunc:create( function(t)
			t:setRotation((t:getRotation() + 10) % 360)
		end )
	)))

	return uiGuideCircle
end

--- 释放圆圈
function T:_realeaseGuideCircle()
	local uiGuideCircle=self._uiGuideCircle
	if uiGuideCircle then
		if uiGuideCircle:getParent() then
			uiGuideCircle:removeFromParent()
		end
		uiGuideCircle:release()
	end
	self._uiGuideCircle=nil
end

--- 创建浮动提示框
function T:_createFloatTipBox()
	local uiFTBox=require("src.app.uiScript.components.beginnersGuide.GuideFloatTipBox")()
	uiFTBox:retain()

	self._uiFloatTipBox=uiFTBox
	return uiFTBox
end

--- 释放浮动提示框
function T:_releaseFloatTipBox()
	local uiFTBox=self._uiFloatTipBox
	if uiFTBox then
		if uiFTBox:getParent() then
			uiFTBox:removeFromParent()
		end
		uiFTBox:release()
	end
	self._uiFloatTipBox=nil
end

--- 显示圆圈
function T:showGuideCircle(x,y,scale)
	local uiGuideCircle=self._uiGuideCircle or self:_createGuideCircle()

	if not uiGuideCircle:getParent() then
		self:addChild(uiGuideCircle,1)
	end

	uiGuideCircle:setScale(scale or 1)
	uiGuideCircle:setPosition(x,y)
end

--- 隐藏圆圈
function T:hideGuideCircle()
	local uiGuideCircle=self._uiGuideCircle
	if uiGuideCircle and uiGuideCircle:getParent() then
		uiGuideCircle:removeFromParent(false)
	end
end

--- 显示浮动提示框
function T:showFloatTipBox(tipText,x,y,rotation)
	local uiFTBox=self._uiFloatTipBox or self:_createFloatTipBox()

	if not uiFTBox:getParent() then
		self:addChild(uiFTBox,1)
		uiFTBox:stopAllActions()
		uiFTBox:runAction(cc.FadeIn:create(0.1))
	end

	uiFTBox:setCoordinate(tipText,x,y,rotation)
end

--- 隐藏浮动提示框
function T:hideFloatTipBox()
	local uiFTBox=self._uiFloatTipBox
	if uiFTBox and uiFTBox:getParent() then
		uiFTBox:stopAllActions()
		uiFTBox:runAction(cc.Sequence:create(
			cc.FadeOut:create(0.1),
			cc.CallFunc:create( function(t)
				t:removeFromParent(false)
			end)
		))
	end
end

--- 显示引导箭头
-- @param x 箭头指向点的横坐标
-- @param y 箭头指向点的纵坐标
-- @param rotation 旋转角度
-- @param floatTipText 浮动提示文本
function T:showGuideArrow(x,y,rotation,floatTipText)
	rotation=rotation or 0

	local uiArrow=self._uiGuideArrow or self:_createGuideArrow()
	if not uiArrow:getParent() then
		self:addChild(uiArrow)
	end
	uiArrow:stopAllActions()

	local s=uiArrow:getContentSize()
	local h=s.height
	local w=s.width
	local ot,dPt,tPt,fPt=20,{x=x,y=y},nil,nil
	if rotation == 0 then
		tPt={x=x,y=y+ot}
		fPt={x=x,y=y+ot+h}
	elseif rotation == 180 then
		tPt={x=x,y=y-ot}
		fPt={x=x,y=y-ot-h}
	elseif rotation == 90 then
		tPt={x=x+ot,y=y}
		fPt={x=x+ot+w,y=y}
	elseif rotation == 270 then
		tPt={x=x-ot,y=y}
		fPt={x=x-ot-w,y=y}
	end

	self:hideFloatTipBox()
	uiArrow:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.4,dPt),
		cc.CallFunc:create( function(target)
			if floatTipText and string.trim(floatTipText) ~= "" then
				self:showFloatTipBox(floatTipText,fPt.x,fPt.y,rotation)
			else
				self:hideFloatTipBox()
			end

			uiArrow:runAction(cc.RepeatForever:create(
				cc.Sequence:create(cc.MoveTo:create(0.2,tPt),cc.MoveTo:create(0.2,dPt))
			))
		end)
	))

	if rotation ~= uiArrow:getRotation() then
		uiArrow:runAction(cc.RotateTo:create(0.2,rotation))
	end
end
--- 隐藏引导箭头
function T:hideGuideArrow()
	local uiArrow=self._uiGuideArrow
	if uiArrow and uiArrow:getParent() then
		uiArrow:removeFromParent()
	end

	self:hideGuideCircle()
end

function T:setHollowAreas(areas)
	if not areas then
		if self._hollowAct then
			self:clearScheduler(self._hollowAct)
			self._hollowAct=nil
		end
		
		self:hideFloatTipBox()
		self:hideGuideArrow()
		
		ll,lr,lt,lb,lFT=0,0,0,0,nil
	end

	HollowMask.setHollowAreas(self,areas)
end

--- 获取镂空区域
-- @param triggerObj 被镂空对象
function T:__getHollowRect(triggerObj)
	assert(triggerObj,"被镂空对象不存在")

	local parent=triggerObj:getParent()
	assert(parent,"被镂空对象的父对象不存在")

	local p,s
	if triggerObj.getContentSize then
		s=triggerObj:getContentSize()
	else
		s=triggerObj:getContentSize()
	end
	local px,py=triggerObj:getPosition()
	local otherAnchor=triggerObj:getAnchorPoint()
	p={x=px-(s.width*otherAnchor.x),y=py-(s.height*otherAnchor.y)}
	local wp=parent:convertToWorldSpace(p)

	return wp.x,wp.x+s.width,wp.y+s.height,wp.y
end

--- 镂空触发对象
-- @param triggerObj 触发对象
-- @param offsetPts 相对于对象的中心点偏移的 point 集合
-- @param arrowOffset 引导箭头相对于对象中心点的偏移
-- @param arrowCenterOffset 引导箭头中心点偏移
-- @param floatTipText 浮动提示文本
-- @param circleInfo 圆圈信息
function T:_hollowTriggerObj(triggerObj,offsetPts,arrowOffset,arrowCenterOffset,floatTipText,circleInfo)
	local l,r,t,b=self:__getHollowRect(triggerObj)

	if ll ~= l or lr ~= r or lt ~= t or lb ~= b or lFT ~= floatTipText then
		local cx,cy=(l+r)/2,(b+t)/2
		if offsetPts then
			local pts={}
			for k,v in ipairs(offsetPts) do
				pts[#pts+1]={x=cx+v.x,y=cy+v.y}
			end

			HollowMask.setHollowAreas(self,{pts})
		else
			HollowMask.setHollowAreas(self,{{{x=l,y=b},{x=l,y=t},{x=r,y=t},{x=r,y=b},}})
		end
		if circleInfo then
			self:showGuideCircle(cx+circleInfo.x,cy+circleInfo.y,circleInfo.scale or 1)
		else
			self:hideGuideCircle()
		end

		if arrowOffset then
			arrowCenterOffset=arrowCenterOffset or {x=0,y=0}

			if V_SZ.height - t >= b-O_PT.x then
				self:showGuideArrow(cx+arrowCenterOffset.x,cy+arrowCenterOffset.y+arrowOffset.y,0,floatTipText)
			else
				self:showGuideArrow(cx+arrowCenterOffset.x,cy+arrowCenterOffset.y-arrowOffset.y,180,floatTipText)
			end
		else
			if V_SZ.height - t >= b-O_PT.x then
				self:showGuideArrow(cx,t,0,floatTipText)
			else
				self:showGuideArrow(cx,b,180,floatTipText)
			end
		end

		ll,lr,lt,lb,lFT=l,r,t,b,floatTipText
	end
end

--- 镂空触发对象
-- @param triggerObj 触发对象
-- @param offsetPts 相对于对象的中心点偏移的 point 集合
-- @param arrowOffset 引导箭头相对于对象中心点的偏移
-- @param arrowCenterOffset 引导箭头中心点偏移
-- @param floatTipText 浮动提示文本
-- @param circleInfo 圆圈信息
function T:hollowTriggerObj(triggerObj,offsetPts,arrowOffset,arrowCenterOffset,floatTipText,circleInfo)
	if self._hollowAct then
		self:clearScheduler(self._hollowAct)
		self._hollowAct=nil
	end

	self._hollowAct=self:interval(function()
		local b=CppCocos2dxUtils:execFunction(function()
			self:_hollowTriggerObj(triggerObj,offsetPts,arrowOffset,arrowCenterOffset,floatTipText,circleInfo)
		end)

		if not b then
			if self._hollowAct then
				self:clearScheduler(self._hollowAct)
				self._hollowAct=nil
			end

			local message="GuideHollowMask call cpp error"+"hollow trigger error! may be trigger is released"

			__print__("callCppErr",message)
			LogRecorder:push(message,LOG_LEVEL_ERROR,"callCppErr")
		end
	end,0)
end

return T
