--- 建筑物主窗口
local T = class(
	"BuildingWindow",
	function()
		local o = AbstractWindow.new()
		o.super = AbstractWindow
		return o
	end
)
--- 服务器数据
T.dataFromServer = nil

--- 建造页面变量
T.constructPage = nil

--- 说明页面变量
T.explainPage = nil

--- 科研中心研究界面
T.scienceResearchPage = nil

--- 这个页面的模板文件
T.modelResTable = nil

--- 构造函数
-- @param tag 分页
function T:ctor(data, tag)
	self.dataFromServer = data

	-- 测试标签页的例子
	local function callBack1()
		self:ConstructLogic()
	end

	local function callBack2()
		local sid = self.dataFromServer.sid
		if sid == 1 then
			self:ExplainLogic()
		end

		if sid == 3 then
			self:ScienceResearchLogic()
		end
	end

	if not tag then
		tag = 1
	end

	local sid = self.dataFromServer.sid
	if sid == 1 or sid == 3 then
		-- 创建2个
		local switchButtonContainer = SwitchButtonContainer.new()
		self:addChild(switchButtonContainer)
		REL:relOther(switchButtonContainer,self.uiContainer,0,0,0,0)
		switchButtonContainer:init()
		switchButtonContainer:initButtons(
			2,
			{{"viewSwitchButton_normal.png","viewSwitchButton_down.png"},{"viewSwitchButton_normal.png","viewSwitchButton_down.png"}},
			{110,15},
			{callBack1,callBack2},
			{Language:get("uc1"),Language:get("uc2")},
			tag
		)
		self.switchButtonContainer = switchButtonContainer
	else
		-- 创建1个
		local switchButtonContainer = SwitchButtonContainer.new()
		self:addChild(switchButtonContainer)
		REL:relOther(switchButtonContainer,self.uiContainer,0,0,0,0)
		switchButtonContainer:init()
		switchButtonContainer:initButtons(
			1,
			{{"viewSwitchButton_normal.png","viewSwitchButton_down.png"}},
			{110,15},
			{callBack},
			{Language:get("uc1")}
		)
		self.switchButtonContainer = switchButtonContainer
	end

	-- 初始化配置表:因为延迟了1帧，所以正确
	self:init()

	if tag == 1 then
		-- 资源建筑有这个
		self.constructPage = self.buildingConstruct.new(self.dataFromServer,self) -- 这个必须先要加载一次，不然位置要出错
		self:addChild(self.constructPage)
		REL:toOther("left","top",self.constructPage,self.uiContainer,3,115)
	elseif tag == 2 then
		local sid = self.dataFromServer.sid
		if sid == 1 then
			self:ExplainLogic()
		end

		if sid == 3 then
			self:ScienceResearchLogic()
		end
	end

	-- 进入页面显示数据
	self:ShowData()


	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	AbstractWindow.onEnter(self)

	local data=self.dataFromServer
	local sid=data.sid

	if sid == 1 then
		--指挥中心
		AudioManager:playEffect(SoundRes.OPEN_COMMAND_CENTER)
    elseif sid == 2 then 
        --- 机库
        AudioManager:playEffect(SoundRes.OPEN_AIRCRAFT_FACTORY)
	elseif sid == 3 then
		--科研中心
		AudioManager:playEffect(SoundRes.OPEN_TECH)
    elseif sid == 4 then 
        -- 改造工厂
        AudioManager:playEffect(SoundRes.OPEN_REFIT_FACTORY)
	elseif sid == 5 then
		--仓库
		AudioManager:playEffect(SoundRes.OPEN_WAREHOUSE)
	elseif sid == 6 then
		--铸币厂
		AudioManager:playEffect(SoundRes.OPEN_MINT_FACTORY)
	elseif sid == 7 then
		--资源建筑-铁
		AudioManager:playEffect(SoundRes.OPEN_RES_STEEL_FACTORY)
	elseif sid == 8 then
		--资源建筑-石油
		AudioManager:playEffect(SoundRes.OPEN_RES_OIL_FACTORY)
	elseif sid == 9 then
		--资源建筑-硅
		AudioManager:playEffect(SoundRes.OPEN_RES_ALUMINIUM_FACTORY)
	elseif sid == 10 then
		--资源建筑-钛
		AudioManager:playEffect(SoundRes.OPEN_RES_TITANIUM_FACTORY)
	end
end

--- 初始化数据
function T:init()
	-- 建造
	self.buildingConstruct = require "src.app.uiScript.components.building.BuildingConstruct"

	-- 科技研究
	self.scienceResearch = require "src.app.uiScript.components.building.ScienceResearch"

	self.modelResTable = PlayerInstance:getBuildingResTable(self.dataFromServer.sid)
end

--- 进入页面显示数据
function T:ShowData()
	local sid = self.dataFromServer.sid
	local location = self.dataFromServer.location
	if sid == 3 then
		self.switchButtonContainer:setButtonText(2,Language:get("uc22"))
	elseif sid == 1 then
		self.switchButtonContainer:setButtonText(2,Language:get("uc2"))
	end

	self:setTitle(self.dataFromServer.level)
end

--- 设置标题
function T:setTitle(level)
	local sid = self.dataFromServer.sid
	local location = self.dataFromServer.location
	local data = Build_temp[sid]
	if sid == 5 then
		-- 这里的仓库有2个,要处理就行了
		if location == 6 then
			self:setWindowTitle(Language:get("uc148"),level)
		else
			self:setWindowTitle(Language:get("uc149"),level)
		end
	else
		self:setWindowTitle(Language:get(data.name),level)
	end
end

--- 建造回调逻辑
function T:ConstructLogic()
	local sid = self.dataFromServer.sid
	if sid == 1 then --只有指挥中心才有这个操作
		if self.explainPage then
			self.explainPage:setVisible(false)
	end

	if self.constructPage == nil then
		self.constructPage = self.buildingConstruct.new(self.dataFromServer,self)
		self:addChild(self.constructPage)
		REL:toOther("left","top",self.constructPage,self.uiContainer,3,115)
	end

	if self.constructPage then
		self.constructPage:setVisible(true)
	end

	end

	if sid == 3 then --只有科研中心才有这个操作
		if self.scienceResearchPage then
			self.scienceResearchPage:setVisible(false)
	end

	if self.constructPage == nil then
		self.constructPage = self.buildingConstruct.new(self.dataFromServer,self)
		self:addChild(self.constructPage)
		REL:toOther("left","top",self.constructPage,self.uiContainer,3,115)
	end

	if self.constructPage then
		self.constructPage:setVisible(true)
	end
	end
end

--- 说明回调逻辑
function T:ExplainLogic()
	if self.constructPage then
		self.constructPage:setVisible(false)
	end

	if self.explainPage == nil then
		self.explainPage = self:GetCommandDescripLayout()
	end

	if self.explainPage then
		self.explainPage:setVisible(true)
	end
end

--- 科技建造逻辑
function T:ScienceResearchLogic()
	if self.constructPage then
		self.constructPage:setVisible(false)
	end

	if self.scienceResearchPage == nil then
		self.scienceResearchPage = self.scienceResearch.new(self.dataFromServer,self)
		self:addChild(self.scienceResearchPage)
		REL:relOther(self.scienceResearchPage,self.uiContainer,0,0,CommonCfg.TOP_CONTAINER_HEIGHT,CommonCfg.BOTTOM_CONTAINER_HEIGTH)
		self.scienceResearchPage:init()
	end

	if self.scienceResearchPage then
		self.scienceResearchPage:setVisible(true)
	end
end

--- 指挥中心介绍页面
function T:GetCommandDescripLayout()
	--描述，可以滚动的
	local uiDesc = XMLTextArea()
	self:addChild(uiDesc)
	REL:relOther(uiDesc,self.uiContainer,10,10,98,10)
	uiDesc:setString(Language:get("uc14"))

    local lineSpacing = uiDesc:getLineSpacing()
    uiDesc:setLineSpacing(lineSpacing + 5)


	return uiDesc
end

return T
