local PRE_RES_LIST={
	"res/images/generalTip.plist"
}

--- 加载所需要的plist
local function loadRes()
	ResLoader:syncLoad(PRE_RES_LIST)
end

loadRes()

local T=Class.AbstractAlter(
	function()
		local o=ccui.Layout:create()
		o.__addChild__=o.addChild

		return o
	end,
	AbstractUI,
	LuaEventDispatcher2
)

--- 内容的最大宽度
T.CONTENT_MAX_WIDTH=visibleSize.width - 0

--- 内容的最大高度
T.CONTENT_MAX_HEIGHT=visibleSize.height - 0

--- 触发关闭的方式-点击任何地方(如果传入的对象是一个 Layout 可以设置 Layout 的 setTouchEnabled(true) 来截获关闭的触发)
T.TRIGGER_CLOSE_METHOD_ANY="any"

--- 触发关闭的方式-点击内容区域(如果传入的对象是一个 Layout 可以设置 Layout 的 setTouchEnabled(true) 来截获关闭的触发)
T.TRIGGER_CLOSE_METHOD_CONTENT="content"

--- 触发关闭的方式-点击内容区域以外的地方
T.TRIGGER_CLOSE_METHOD_OUTSIDE="outside"

--- 触发关闭的方式-点击按钮
T.TRIGGER_CLOSE_METHOD_BUTTONS="buttons"

--- 触发关闭的方式-无遮罩
T.TRIGGER_CLOSE_METHOD_NONE="none"

function T:def()
	AbstractUI.def(self)
	LuaEventDispatcher2.def(self)

	--- 遮罩
	-- self.uiMask=nil

	--- 为了添加特效而添加的容器
	-- self.uiEffContainer=nil

	--- 提示框显示容器
	-- self.uiAlterContainer=nil

	--- 是否显示遮罩
	self.isMask=true

	--- 遮罩透明度
	self.maskOpacity=200

	--- 触发关闭的方式
	self.triggerCloseMethod=self.TRIGGER_CLOSE_METHOD_ANY
end

function T:ctor(zOrder)
	AbstractUI.ctor(self)
	LuaEventDispatcher2.ctor(self)

	self.zOrder=zOrder or TOP_ORDER + 100

	self:init()
end

--- 进入场景
function T:onEnter()
	AbstractUI.onEnter(self)
	self:luaDispatchEvent("onEnter",self)

	self:showEff()
end

--- 退出场景
function T:onExit()
	AbstractUI.onExit(self)
	self:luaDispatchEvent("onExit",self)
end

function T:init()
	REL:relScreen(self, 0, 0, 0, 0)

	self:setLocalZOrder(self.zOrder)

	if self.isMask then
		--- 遮罩
		local uiMask=ccui.Layout:create()
		self:__addChild__(uiMask)
		REL:relOther(uiMask,self, 0, 0, 0, 0)
		uiMask:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid )
		uiMask:setBackGroundColor(cc.c3b(0,0,0))
		uiMask:setBackGroundColorOpacity(self.maskOpacity)
		self.uiMask=uiMask
	end

	local uiEffContainer=cc.Node:create()
	self:__addChild__(uiEffContainer)
	REL:toScreen("center","middle",uiEffContainer,0,0)
	self.uiEffContainer=uiEffContainer

	local uiAlterContainer=ccui.Layout:create()
	uiAlterContainer:setTouchEnabled(true)
	uiEffContainer:addChild(uiAlterContainer)
	self.uiAlterContainer=uiAlterContainer

	-- 内容容器
	local uiContainer=ccui.Layout:create()
	self.uiAlterContainer:addChild(uiContainer,1)
	self.uiContainer=uiContainer
end

--- 更新关闭触发方法
function T:updateCloseTiggerMethod()
	if self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_NONE then
		self:setTouchEnabled(false)
		self.uiAlterContainer:setTouchEnabled(false)
	else
		self:setTouchEnabled(true)

		if self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_CONTENT then
			self:addTouchEventListener(function(t,e)
				if e == ccui.TouchEventType.ended then

				end
			end)
			self.uiAlterContainer:setTouchEnabled(true)
			self.uiAlterContainer:addTouchEventListener(function(t,e)
				if e == ccui.TouchEventType.ended then
					self:hide()
				end
			end)
		elseif self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_OUTSIDE then
			self:addTouchEventListener(function(t,e)
				if e == ccui.TouchEventType.ended then
					self:hide()
				end
			end)
			self.uiAlterContainer:setTouchEnabled(true)
			self.uiAlterContainer:addTouchEventListener(function(t,e)
				if e == ccui.TouchEventType.ended then

				end
			end)
		elseif self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_BUTTONS then
			self:addTouchEventListener(function(t,e)
				if e == ccui.TouchEventType.ended then

				end
			end)
			self.uiAlterContainer:setTouchEnabled(false)
		else
			self:addTouchEventListener(function(t,e)
				if e == ccui.TouchEventType.ended then
					self:hide()
				end
			end)
			self.uiAlterContainer:setTouchEnabled(false)
		end
	end
end

--- 显示特效
function T:showEff()
	local uiEffContainer=self.uiEffContainer

	uiEffContainer:setScaleY(0)
	uiEffContainer:runAction(cc.Sequence:create(
		cc.Spawn:create(
			cc.Sequence:create(
				cc.ScaleTo:create(0.1,1.05),
				cc.ScaleTo:create(0.2,1)
			),
			cc.FadeIn:create(0.4)
		)
	))

	if self.uiMask then
		self.uiMask:setOpacity(0)
		REL:relOther(self.uiMask,self, 0, 0, 0, 0)
		self.uiMask:runAction(cc.FadeIn:create(0.2))
	end
end

--- 隐藏时的特效
function T:hideEff()
	local uiEffContainer=self.uiEffContainer

	self:setEnabled(false)
	self:setTouchEnabled(false)
	uiEffContainer:runAction(cc.Sequence:create(
		cc.Spawn:create(
			cc.Sequence:create(
				cc.ScaleTo:create(0.2,1.05),
				cc.ScaleTo:create(0.1,0.75)
			),
			cc.FadeOut:create(0.3)
		),
		cc.CallFunc:create(function()
			self:removeFromParent()
		end)
	))

	if self.uiMask then
		self.uiMask:runAction(cc.FadeOut:create(0.15))
	end
end

----- 显示特效
--function P:showEff()
--	local uiEffContainer=self.uiEffContainer
--
--	uiEffContainer:setScaleY(0)
--	self.uiMask:setOpacity(0)
--	uiEffContainer:runAction(cc.Sequence:create(
--		cc.Spawn:create(
--			cc.Sequence:create(
--				cc.ScaleTo:create(0.1,1.05),
--				cc.CallFunc:create(function()
--					self.uiMask:runAction(cc.FadeIn:create(0.4))
--				end),
--				cc.ScaleTo:create(0.2,1)
--			),
--			cc.FadeIn:create(0.4)
--		)
--	))
--end
--
----- 隐藏时的特效
--function P:hideEff()
--	local uiEffContainer=self.uiEffContainer
--
--	self:setTouchEnabled(false)
--	uiEffContainer:runAction(cc.Sequence:create(
--		cc.Spawn:create(
--			cc.Sequence:create(
--				cc.ScaleTo:create(0.2,1.05),
--				cc.ScaleTo:create(0.1,0.75)
--			),
--			cc.FadeOut:create(0.3)
--		),
--		cc.CallFunc:create(function()
--			self.uiMask:runAction(cc.Sequence:create(
--				cc.FadeOut:create(0.15),
--				cc.CallFunc:create(function(target)
--					self:removeFromParent()
--				end)
--			))
--		end)
--	))
--end

function T:hide()
	self:hideEff()
end

function T:_show(data,triggerCloseMethod)
	self.data=data

	self.triggerCloseMethod=triggerCloseMethod or self.TRIGGER_CLOSE_METHOD_ANY
	self:updateCloseTiggerMethod()
end

--- 显示提示框
-- @param info 显示信息
-- @param parent 父对象,如果此值为nil,则显示到当前场景
function T:show(info,parent)
	local ins=self(info.zOrder)
	ins:_show(info)

	if not parent or type(parent) ~= "userdata" then
		parent=Director:getRunningScene()
	end
	if parent.__addChild__ then
		parent:__addChild__(ins)
	else
		parent:addChild(ins)
	end

	return ins
end

--- 更新正文内容
function T:updateCentent()
	local data=self.data

	local uiContainer=self.uiContainer

	--显示内容的区域的大小
	local cSize=uiContainer:getContentSize()
	--显示内容的区域的宽
	local cW=cSize.width
	--显示内容的区域的高
	local cH=cSize.height

	--显示内容
	local uiContent=self.uiContent
	if uiContent then
		uiContent:removeFromParent()
	end

	if type(data.info) == "string" then
		local uiTextArea=XMLTextArea(nil,26)
		uiTextArea:setContentSize(cc.size(cW,cH))
		uiTextArea:setLineSpacing(data.lineSpacing or 2)
		uiTextArea:setAlign(data.hAlign or "center",data.vAlign or "middle")
		uiTextArea:setString(data.info)
		self:addChild(uiTextArea)

		if self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_ANY or self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_NONE then
			uiTextArea:setTouchEnabled(false)
		end

		--强制立即刷新
		uiTextArea:updateView()

		local textHeight=uiTextArea:getTextHeight()

		if textHeight > uiTextArea:getContentSize().height then
			if textHeight >= self.CONTENT_MAX_HEIGHT then
				uiTextArea:setHeight(self.CONTENT_MAX_HEIGHT)

				uiTextArea:setTouchEnabled(true)
			else
				uiTextArea:setHeight(textHeight)

				if self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_ANY or self.triggerCloseMethod == self.TRIGGER_CLOSE_METHOD_NONE then
					uiTextArea:setTouchEnabled(false)
				end
			end
		end

		uiContent=uiTextArea
	else
		--内容
		local iContent=data.info

		function iContent.__hide__()
			self:hide()
		end

		local s=iContent:getContentSize()
		local w,h=s.width,s.height
		if w <= cW and h <= cH then
			uiContent=iContent
			self:addChild(uiContent)
			REL:toScreen("center","middle",uiContent,0,0)
		else
			local nW,nH=math.min(w,self.CONTENT_MAX_WIDTH),math.min(h,self.CONTENT_MAX_HEIGHT)

			if w > nW or h > nH then
				uiContent=ccui.ScrollView:create()
				uiContent:setBounceEnabled(true)
				uiContent:setContentSize(cc.size(nW,nH))
				self:addChild(uiContent)

				uiContent:addChild(iContent)
				uiContent:setInnerContainerSize(s)

				if w > nW and h > nH then
					uiContent:setDirection(ccui.ScrollViewDir.both)
				elseif  w > nW then
					uiContent:setDirection(ccui.ScrollViewDir.horizontal)
				elseif  h > nH then
					uiContent:setDirection(ccui.ScrollViewDir.vertical)
				end
			else
				uiContent=iContent
				uiContent:setContentSize(cc.size(nW,nH))
				self:addChild(uiContent)
			end
		end
	end

	self.uiContent=uiContent
end

return T
