--- 输入框
-- @author blank
local _T=Class.InputBox(LuaEventDispatcher2,AbstractUI)

local _print=print
local function print(...)
	_print(string.format("\t<%s>:%s",_T.__name,table.concat({...},"\t")))
end

local staticUiMask

--- 添加遮罩
local addMask=function()
	print("添加遮罩")

	local uiMask=staticUiMask
	if not uiMask then
		uiMask = ccui.Layout:create()
		Director:getRunningScene():addChild(uiMask)
		REL:relScreen(uiMask, 0, 0, 0, 0)
		uiMask:setLocalZOrder(1000000)
		uiMask:setTouchEnabled(true)

		staticUiMask = uiMask
	end
	
	REL:relScreen(uiMask, 0, 0, 0, 0)
	uiMask:setVisible(true)
end

--- 移除遮罩
local rmvMask=function()
	print("移除遮罩")

	local uiMask=staticUiMask
	if uiMask then
		uiMask:setVisible(false)
	end
end

function _T:def()
	LuaEventDispatcher2.def(self)
	AbstractUI.def(self)

	--- 是否显示遮罩(显示遮罩时,用户点击非键盘区域时,会触发键盘回收,而不是触发点击的UI组件)
	self._isShowMask=false

	--- 遮罩
	-- _uiMask
end

function _T:ctor()
	LuaEventDispatcher2.ctor(self)
	AbstractUI.ctor(self)

	self:setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE)
	self:setFontSize(defaultCssStyle.size)  		-- 如果不设置字号，则字体显示高度周文本框高度
	self:setFontName(defaultCssStyle.fontName)  	-- 不设置字体，就不会缩进

	self:registerScriptEditBoxHandler(function(e,t)
		print(string.format("editBox evt=>event:%s,target:%s",e,tostring(t)))

		if e == "began" then
			if self._isShowMask then
				addMask()
			end
		elseif e == "ended" then
			if self._isShowMask then
				self:timeout(function()
					rmvMask()
				end, 0)
			end
		elseif e == "changed" then
		elseif e == "return" then
		end

		self:luaDispatchEvent(e,t)
	end)
end

--- 设置是否显示遮罩
function _T:setIsShowMask(value)
	self._isShowMask=value
end

--- 获取是否显示遮罩
function _T:getIsShowMask()
	return self._isShowMask
end

--- 输入框
-- @author blank
local T={}

--- 创建输入框
-- @param size 输入框的大小
-- @param scale9Back 输入框的背景(ccui.Scale9Sprite 对象)
function T:create(size,scale9Back)
	size=size or cc.size(240,50)
	scale9Back=scale9Back or ccui.Scale9Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("inputBoxBack.png"),cc.rect(12,13,2,1))

	local _Class=Class.InputBox(
		function()
			return ccui.EditBox:create(size,scale9Back)
		end,
		_T
	)

	function _Class:def(def)
		_T.def(self)
	end

	function _Class:ctor()
		_T.ctor(self)
	end

	return _Class()
end

return T
