local AbstractAlter = require("src.app.uiScript.components.common.AbstractAlter")

--- 最小内容高度
local contentMinWidth=0

--- 最小内容高度
local contentMinHeight=0

local T=Class.T(AbstractAlter)

function T:def()
	AbstractAlter.def(self)

	--- 遮罩
	-- self.uiMask=nil

	--- 为了添加特效而添加的容器
	-- self.uiEffContainer=nil

	--- 提示框显示容器
	-- self.uiAlterContainer=nil

	--- 背景
	-- self.uiBack=nil

	--- 内容容器
	-- self.uiContainer=nil

	--- 显示时间(0表示 touch 触发关闭,-1表示程序触发关闭,大于0表示到时间后自动关闭)
	-- self.showSecond=0

	--- 是否显示遮罩
	self.isMask=false
end

--- 构造函数
function T:ctor()
	AbstractAlter.ctor(self)
end

--- 进入场景
function T:onEnter()
	AbstractAlter.onEnter(self)

	AudioManager:playEffect(self.data.popSnd or SoundRes.SND_SINGLE_LINE_TIP_POP)

	local sec=self.showSecond or 0
	if sec == 0 then
		self.showSecond=0
		self.uiAlterContainer:setTouchEnabled(true)
	else
		self.uiAlterContainer:setTouchEnabled(false)
		if sec > 0 then
			Cocos2dxUtils:timeOut(function()
				self:hide()
			end,sec)
		end
	end
end

--- 退出场景
function T:onExit()
	AbstractAlter.onExit(self)

	if self._closeCallback then
		local call=self._closeCallback
		local arg=self.flag
		Cocos2dxUtils:timeOut(function()
			call()
		end,0)
	end
end

--- 初始化
function T:init()
	AbstractAlter.init(self)

	self:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			if self.showSecond == 0 then
				self:hide()
			end
		end
	end)

	self.uiAlterContainer:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			if self.showSecond == 0 then
				self:hide()
			end
		end
	end)

	-- 背景
	local f=ResLoader:getSpriteFrame("singleTipBack.png")
	local fs=f:getOriginalSize()
	local uiBack=Scale9Sprite:createWithFrame(f,"v3")
	self.uiAlterContainer:addChild(uiBack)
	self.uiBack=uiBack

	contentMinWidth=fs.width
	contentMinHeight=fs.height

	self.uiContainer:setContentSize(fs)
end

--- 更新对齐
function T:updateAlign()
	local uiAlterContainer=self.uiAlterContainer

	local cs=self.uiContainer:getContentSize()

	uiAlterContainer:setContentSize(cc.size(cs.width+8,cs.height+28))
	REL:toOther("center","middle",uiAlterContainer,self.uiEffContainer,self.data.offsetX or 0,self.data.offsetY or 0)

	REL:relOther(self.uiBack,uiAlterContainer,0,0,0,0)
	REL:toOther("center","middle",self.uiContainer,uiAlterContainer,0,0)
end

function T:addChild(child)
	self.uiContainer:addChild(child)
end

--- 设置容器大小
function T:setContainerSize(width,height)
	width=width<contentMinWidth and contentMinWidth or width
	height=height<contentMinHeight and contentMinHeight or height

	local uiContainer=self.uiContainer

	uiContainer:setContentSize(cc.size(width,height))

	self:updateAlign()
end

--- 显示
function T:_show(data)
	local showSecond=data.showSecond or 1
	AbstractAlter._show(self,data,showSecond == 0 and self.TRIGGER_CLOSE_METHOD_ANY or self.TRIGGER_CLOSE_METHOD_NONE)

	self.showSecond=showSecond
	self._closeCallback=data.callback

	self:updateCentent()

	local uiContent=self.uiContent
	self:setContainerSize(uiContent:getContentSize().width,uiContent:getContentSize().height)

	REL:toOther("center","top",uiContent,self.uiContainer,0,0)
end

--- 无标题栏的通用提示框
-- 	弹窗接收一个table对象
-- 	{
-- 		info:显示信息（html文本或可显示对象）
-- 		showSecond:显示时间(0表示 touch 触发关闭,-1表示程序触发关闭,大于0表示到时间后自动关闭)
-- 		isMask:是否显示遮罩
--      offsetX:水平中间的距离
--      offsetY:高度中间的距离
-- 		vAlign：（只有当info为纯文字时，才有效，垂直对齐方式<默认居中>）
-- 		hAlign：（只有当info为纯文字时，才有效，水平对齐方式<默认居中>）
-- 		lineSpacing：行间距（默认为2）
-- 		callback：回调函数
--	}
function T.show(info,parent)
	return AbstractAlter.show(T,info,parent)
end

return T
