--- 滑块
-- @author blank
local T=Class.Slider(
	function()
		local o=ccui.Layout:create()
		o.__setSize__=o.setContentSize
		o.__setEnabled__=o.setEnabled
		o.__getEnabled__=o.getEnabled

		return o
	end,
	LuaEventDispatcher2
)

function T:def()
	LuaEventDispatcher2.def(self)

	--- 进度条裁剪器
	-- self.uiBarClipper=nil

	--- 进度条
	-- self.uiBar=nil

	--- 滑块按钮
	--self.uiSliderButton=nil

	--- 进度条纹理名
	-- self.barTexUrl=nil

	--- 进度条9格缩放的内部区域
	-- self.innerScaleRect=nil

	--- 滑块按钮纹理名-普通
	-- self.sliderButtonTexNormal=nil

	--- 滑块按钮纹理名-按下
	-- self.sliderButtonTexDown=nil

	--- 最大值
	self.maxValue=100

	--- 最小值
	self.minValue=0

	--- 当前值
	self.value=0

	--- 偏移
	--self.offsetX=nil

	--- 是否可用
	self._enabled=true
end

--- 构造函数
-- @param barTexUrl
-- @param innerScaleRect
function T:ctor(barTexUrl,innerScaleRect,sliderButtonTexNormal,sliderButtonTexDown)
	LuaEventDispatcher2.ctor(self)

	self.barTexUrl=barTexUrl
	self.innerScaleRect=innerScaleRect
	self.sliderButtonTexNormal=sliderButtonTexNormal
	self.sliderButtonTexDown=sliderButtonTexDown

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:init()
end

--- 进入
function T:onEnter()
end

--- 退出
function T:onExit()
end

--- 内存清理
function T:cleanUp()

end

--- 初始化
function T:init()
--	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
--	self:setBackGroundColor(cc.c3b(0,0,0xff))
--	self:setBackGroundColorOpacity(100)

	--进度条裁剪器
	local uiBarClipper=ccui.Layout:create()
	uiBarClipper:setContentSize(self:getContentSize())
	uiBarClipper:setClippingEnabled(true)
	self:addChild(uiBarClipper)
	self.uiBarClipper=uiBarClipper

	--进度条
	local uiBar=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame(self.barTexUrl),self.innerScaleRect)
	uiBar:setAnchorPoint(0,0)
	uiBarClipper:addChild(uiBar)
	self.uiBar=uiBar

	--滑块按钮
	local uiSliderButton=ccui.ImageView:create(self.sliderButtonTexNormal,ccui.TextureResType.plistType)
	uiSliderButton:setAnchorPoint(0.5,0.5)
	self:addChild(uiSliderButton)
	self.uiSliderButton=uiSliderButton

	self:initTouchListener()
	self:updateSliderButtonLoc()

	self:runAction(cc.Sequence:create(
		cc.DelayTime:create(0.01),
		cc.CallFunc:create(
			function()
				self:updateView()
			end
		)
	))
end

--- 设置滑块位置
function T:setSliderButtonPos(x)
	if x < 0 then
		x=0
	end

	local w=self:getContentSize().width
	if x > w then
		x=w
	end

	self.uiSliderButton:setPositionX(x)
	self.uiBarClipper:setContentSize(cc.size(x,self:getContentSize().height))

	local v=(self.maxValue-self.minValue)*(x/self:getContentSize().width)+self.minValue
	if v ~= self.value then
		self.value=v
		self:luaDispatchEvent(EVENT_CHANGE,self)
	end
end

--- 初始化 touch 事件侦听器
function T:initTouchListener()
	local uiBarClipper=self.uiBarClipper
	local uiSliderButton=self.uiSliderButton

	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	---开始拖动
	local onTouchBegan = function(t, e)
		if not self._enabled then
			return false
		end

		local p=t:getLocation()

		local localP=uiSliderButton:convertToNodeSpace(p)
		local contendS=uiSliderButton:getContentSize()

		local rect=cc.rect(0,0,contendS.width,contendS.height)
		if cc.rectContainsPoint(rect, localP) then
			self.offsetX=localP.x

			--- 按钮变为按下状态
			uiSliderButton:loadTexture(self.sliderButtonTexDown,ccui.TextureResType.plistType)

			return true
		else
			self.offsetX=nil

			localP=self:convertToNodeSpace(p)

			local s=self:getContentSize()

			local rect=cc.rect(0,0,s.width,s.height)
			if cc.rectContainsPoint(rect, localP) then
				self:setSliderButtonPos(localP.x)

				return true
			end
		end

		return false
	end

	---拖动
	local onTouchMoved = function(t, e)
		if not self._enabled then
			return false
		end

		local p=t:getLocation()

		if self.offsetX then
			local localP=self:convertToNodeSpace(p)
			self:setSliderButtonPos(localP.x-self.offsetX)
		end

		return true
	end

	--- 拖动结束
	local onTouchEnded = function(t, e)
		if not self._enabled then
			return false
		end

		local p=t:getLocation()

		--- 按钮变为普通状态
		uiSliderButton:loadTexture(self.sliderButtonTexNormal,ccui.TextureResType.plistType)

		self.offsetX=nil
		return true
	end

	local onTouchCanceled = function(t, e)
		if not self._enabled then
			return false
		end

		return true
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(onTouchCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)
	local eventDispatcher = self:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

function T:updateSliderButtonLoc()
	local x=self:getContentSize().width*(self.value-self.minValue)/(self.maxValue-self.minValue)

	self.uiSliderButton:setPositionX(x)
	self.uiBarClipper:setContentSize(cc.size(x,self:getContentSize().height))
end

function T:updateView()
	self.uiBar:setContentSize(self:getContentSize())
	self.uiBar:setPosition(cc.p(0,0))

	self.uiSliderButton:setPositionY(self:getContentSize().height/2)
	self:updateSliderButtonLoc()
end

function T:setMaxValue(value)
	if value ~= self.maxValue then
		self.maxValue=value

		self:updateSliderButtonLoc()
	end
end
function T:getMaxValue()
	return self.maxValue
end

function T:setMinValue(value)
	if value ~= self.minValue then
		self.minValue=value

		self:updateSliderButtonLoc()
	end
end
function T:getMinValue()
	return self.MinValue
end

function T:setValue(value)
	if value ~= self.value then
		self.value=value

		self:updateSliderButtonLoc()
	end
end
function T:getValue()
	return self.value
end

function T:setContentSize(value)
	local size=self:getContentSize()
	if value.width ~= size.width or value.height ~= size.height then
		self:__setSize__(value)
		self:updateView()
	end
end

function T:setEnabled(value)
	if value ~= self._enabled then
		self._enabled=value

		self.uiSliderButton:setIsGray(not self._enabled)
		self:__setEnabled__(self._enabled)
	end
end
function T:getEnabled()
	return self._enabled
end

--- 覆盖掉此方法(因为使用 SpriteBatchNode 对象时,设置 setIsGray 会导致位置发生错误)
function T:setIsGray()
end

return T
