local PRE_RES_LIST={
	"res/images/generalTip.plist",
	{type="animations",list={
		{name="touchEff",url="touchEff0",beginIndex=1,endIndex=10,extension=".png",delay=0.08},
		{name="loadingEff",url="loadingEff0",beginIndex=0,endIndex=9,extension=".png",delay=0.08},
	}},
}

--- 加载所需要的plist
local function loadRes()
	ResLoader:syncLoad(PRE_RES_LIST)
end

local T=Class.TouchEffLayer(
	function()
		return cc.Node:create()
	end,
	AbstractUI
)

function T:def()
	AbstractUI.def(self)

	--- touch特效
	-- _uiTouchEff

	--- touch特效Fade动作
	-- _touchEffFadeAct
end

function T:ctor()
	AbstractUI.ctor(self)

	loadRes()

	self:_init()
end

function T:_init()
	self:initTouchEff()

	self:_addTouchEvtListener()
end

--- 添加touch事件侦听
function T:_addTouchEvtListener()
	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(false)

	---开始拖动
	local onTouchBegan = function(t, e)
		local p=t:getLocation()

		--		self:showTouchEff(p)
		self:playTouchEff(p)

		return true
	end

	---拖动
	local onTouchMoved = function(t, e)
		local p=t:getLocation()

		--		self:showTouchEff(p)
		self:playTouchEff(p)

		return true
	end

	--- 拖动结束
	local onTouchEnded = function(t, e)
		local p=t:getLocation()

		self:hideTouchEff()
		self:playTouchEff(p)

		return true
	end

	local onTouchCanceled = function(t, e)
		self:hideTouchEff()

		return true
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(onTouchCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)
	self:getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, self)
end

--- 初始化touch特效
function T:initTouchEff()
	local uiEff=ResLoader:createAnimationSpriteRandom("touchEff")
	--	uiEff:setScale(2)
	uiEff:retain()
	self._uiTouchEff=uiEff
end

--- 显示touch特效
function T:showTouchEff(pt)
	local uiEff=self._uiTouchEff
	if uiEff then
		if not uiEff:getParent() then
			self:addChild(uiEff)

			uiEff:runAction(cc.Spawn:create(
				cc.Spawn:create(
					cc.FadeIn:create(0.2),
					cc.ScaleIn:create(0.2)
				),
				cc.ScaleIn:create(0.2)
			))
		end
		uiEff:setPosition(pt)
	end
end

--- 隐藏特效
function T:hideTouchEff()
	local uiEff=self._uiTouchEff
	if uiEff then
		if uiEff:getParent() then
			uiEff:runAction(cc.Sequence:create(
				cc.Spawn:create(
					cc.FadeIn:create(0.2),
					cc.ScaleIn:create(0.2)
				),
				cc.CallFunc:create( function(target)
					uiEff:removeFromParent()
				end)
			))
		end
	end
end

--- 播放点击特效
function T:playTouchEff(pt)
	local uiSp=cc.Sprite:create()
	uiSp:setScale(0)

	local animation=ResLoader:getAnimation("touchEff")
	if animation then
		uiSp:runAction(cc.Sequence:create(
			cc.Spawn:create(
				cc.Animate:create(animation),
				cc.FadeIn:create(0.2),
				cc.ScaleTo:create(0.2,1)
			),
			cc.FadeOut:create(0.2),
			cc.CallFunc:create(function(t)
				t:removeFromParent()
			end)
		))
	end

	uiSp:setPosition(pt)
	self:addChild(uiSp)
end

return T
