--- 阵营演播容器
local T = Class.AbstractCampPlayContainer(
	function()
		local o=ccui.Layout:create()
		o._addChild=o.addChild

		return o
	end,
	AbstractUI
)

--- 战场配置
T.FightCfg=require("src.app.configs.FightCfg")

--- 地图特效播放器
T.SeaPlayer=require("src.app.uiScript.components.fight.sea.SeaPlayer")

--- 云彩特效播放器
T.FightCloudPlayer=require("src.app.uiScript.components.fight.cloud.FightCloudPlayer")

--- 顶层云彩
T.TopCloudPlayer=require("src.app.uiScript.components.fight.cloud.TopCloudPlayer")

--- 战机阵位列表显示层
T.FormLocsLayer=require("src.app.uiScript.components.fight.FormLocsLayer")

--- 攻击特效播放的抽象类
T.AbstractATKEff=require("src.app.uiScript.components.fight.attackEffects.AbstractATKEff")

function T:def()
	AbstractUI.def(self)

	--- 阵营
	--self.camp=nil

	--- 阵营数据
	--self.campData=nil

	--- 剪裁器
	--self.uiClipper=nil

	--- 容器
	--self.uiContainer=nil

	--- 背景显示层
	--self.uiBackLayer=nil

	--- 底层特效显示层
	--self.uiBottomEffectLayer=nil

	--- 战机显示层
	--self.uiFormLocsLayer=nil

	--- 顶层特效显示层
	--self.uiTopEffectLayer=nil

	--- 云彩特效播放器
	--self.uiCloudEffectPlayer=nil

	--- 是否在震屏中
	self.isShocking=false

	--- 震屏结束时间
	-- self.shockEndTime=nil

	--- 震屏的原点
	-- self.shockOrigin=nil

end

--- 构造函数
function T:ctor(camp)
	AbstractUI.ctor(self)

	self.camp=camp

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:init()
end

--- 进入
function T:onEnter()

end

--- 退出
function T:onExit()

end

--- 内存清理
function T:cleanUp()
	self.uiLocer:release()
end

--- 初始化
function T:init()
	REL:relScreen(self,0,0,0,0)

	self:createClipper()

	self:initBackLayer()
	self:initBottomEffectLayer()
	self:initFormLocsLayer()
	self:initTopEffectLayer()
end

--- 创建被遮罩的容器
function T:createClipper()
	local uiClipper=cc.ClippingNode:create()

	local uiMask=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("campMask.png"))
	local w,h,a,b=self.FightCfg:getMaskSize(visibleSize.width,visibleSize.height)
	REL:relRect(uiMask,cc.rect(0,0,w,h),0,0,0,0)
	if self.camp == Camp.CAMP1 then
		uiMask:setAnchorPoint(cc.p(0,1))
		REL:toOther("left","top",uiMask,self,0,b)
	else
		uiMask:setAnchorPoint(cc.p(1,0))
		REL:toOther("right","bottom",uiMask,self,0,b)
	end
	uiMask:setRotation(self.FightCfg.SEP_ROTATION)

	uiClipper:setStencil(uiMask);
	uiClipper:setAlphaThreshold(0);

	local uiContainer=cc.Node:create()
	uiContainer:setPosition(0,0)

	uiClipper:addChild(uiContainer)
	self.uiContainer=uiContainer

	self:_addChild(uiClipper)
	self.uiClipper=uiClipper
end

--- 初始化背景显示层
function T:initBackLayer()
	local u=ccui.Layout:create()
	self:addChild(u)
	REL:relOther(u,self,0,0,0,0)
	self.uiBackLayer=u
end

--- 初始化底部特效显示层
function T:initBottomEffectLayer()
	local u=ccui.Layout:create()
	self:addChild(u)
	REL:relOther(u,self,0,0,0,0)
	self.uiBottomEffectLayer=u
end

--- 初始化战机显示层
function T:initFormLocsLayer()
	local u=self.FormLocsLayer()
	self:addChild(u)
	self.uiFormLocsLayer=u
end

--- 初始化顶部特效显示层
function T:initTopEffectLayer()
	local u=ccui.Layout:create()
	self:addChild(u)
	REL:relOther(u,self,0,0,0,0)
	self.uiTopEffectLayer=u
end

function T:addChild(child)
	self.uiContainer:addChild(child)
end

--- 设置阵营数据
function T:setCampData(value)
	self.campData=value

	self.uiFormLocsLayer:setCampData(value)
end

--- 获取阵营数据
function T:getCampData()
	return self.campData
end

--- 震动屏幕
-- @param keepSec 持续时间
-- @param strength 强度
function T:shockScreen(keepSec,strength)
	self.shockEndTime=TimeUtils:getSyncMillis()+keepSec*1000
	if self.isShocking then
		return
	end
	self.isShocking=true
	self.shockOrigin=cc.p(self:getPosition())

	self._act=self:interval(function()
		if TimeUtils:getSyncMillis()>= self.shockEndTime then
			self:clearScheduler(self._act)
			self.isShocking=false

			self:setPosition(self.shockOrigin)
			return
		end

		strength=strength or 5
		local offset=cc.p(random(-strength, strength),random(-strength, strength))
		self:setPosition(Cocos2dxUtils.offset(self.shockOrigin, offset))
	end,0)
end

return T
