--- 地面特效播放器
local T=Class.GroundPlayer(
	function()
		return ccui.Layout:create()
	end,
	AbstractUI
)

function T:def()
	AbstractUI.def(self)

	--- 加载进度条
	self.uiProgress=nil

	--- 需要加载的总的资源数
	self.needLoadCount=0

	--- 资源数所占的百分比
	self.maxPercent=40
end

function T:ctor()
	AbstractUI.ctor(self)

	self:init()
end

function T:onExit()
	AbstractUI.onExit(self)

	ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_ITEM_LOAD_COMPLETE,self,self.sceneResItemLoadCompleteHandler)
	ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_ADD_ITEM,self,self.addLoadItemHandler)
end

function T:init()
	REL:relScreen(self,0,0,0,0)
	self:setTouchEnabled(true)

	local scale=6

	local cloudFrame=ResLoader:getSpriteFrame("loadingCloud.png")
	local frameSize=cloudFrame:getOriginalSize()
	self.cloudWidth,self.cloudHeight=frameSize.width*scale,frameSize.height*scale

	local uiLBCloud=cc.Sprite:createWithSpriteFrame(cloudFrame)
	uiLBCloud:setScale(scale)
	self:addChild(uiLBCloud)
	uiLBCloud:setRotation(90)
	uiLBCloud:setPosition(-self.cloudHeight/2,V_SZ.height/2)
	self.uiLBCloud=uiLBCloud

	local uiRTCloud=cc.Sprite:createWithSpriteFrame(cloudFrame)
	uiRTCloud:setScale(scale)
	self:addChild(uiRTCloud)
	uiRTCloud:setRotation(250)
	uiRTCloud:setPosition(V_SZ.width+self.cloudHeight/2,V_SZ.height+self.cloudWidth/2)
	self.uiRTCloud=uiRTCloud

	local uiRBCloud=cc.Sprite:createWithSpriteFrame(cloudFrame)
	uiRBCloud:setScale(scale)
	self:addChild(uiRBCloud)
	uiRBCloud:setFlippedX(true)
	uiRBCloud:setRotation(120)
	uiRBCloud:setPosition(V_SZ.width+self.cloudHeight/2,-self.cloudWidth/2)
	self.uiRBCloud=uiRBCloud

	--进度条背景
	local uiProgress_back=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("loading_back.png"))
	self:addChild(uiProgress_back)
	REL:toScreen("center","bottom",uiProgress_back,0,0)

	--进度条
	local uiProgress_fore=cc.ProgressTimer:create(cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("loading_fore.png")))
	uiProgress_fore:setBarChangeRate(cc.p(1,0))--进度条动画方向，（1,0）表示横方向，（0,1）表示纵方向
	uiProgress_fore:setType(cc.PROGRESS_TIMER_TYPE_BAR)--进度条类型
	uiProgress_fore:setMidpoint(cc.p(0,1))--进度条的起始点，（0,y）表示最左边，（1,,y）表示最右边，（x,1）表示最上面，（x,0）表示最下面。
	uiProgress_fore:setPercentage(0)--初始值
	uiProgress_fore:setPosition(0,-50)
	self:addChild(uiProgress_fore)
	REL:toOther("center","bottom",uiProgress_fore,uiProgress_back,0,0)
	self.uiProgress=uiProgress_fore

	ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_ITEM_LOAD_COMPLETE,self,self.sceneResItemLoadCompleteHandler)
	ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_ADD_ITEM,self,self.addLoadItemHandler)
	self.needLoadCount=ResLoader:getRemaining()
end

--- 场景资源单项加载完成
function T:sceneResItemLoadCompleteHandler(item)
	self:addProgress(1/self.needLoadCount*self.maxPercent)
end

--T.curPercent=0

--- 添加处理进程的百分比
function T:addProgress(addPercent)
	--	self.curPercent=self.curPercent+addPercent
	--
	--	local curP=self.uiProgress:getPercentage()
	--	self.uiProgress:runAction(cc.ProgressFromTo:create(0.2,curP,self.curPercent))

	self.uiProgress:setPercentage(self.uiProgress:getPercentage()+addPercent)
end

--- 中途添加加载项
function T:addLoadItemHandler(item)
	self.needLoadCount=self.needLoadCount+1
end

--- 设置最大百分比
function T:setMaxPercent(value)
	self.maxPercent=value
end

function T:start(completeCall)
	AudioManager:playEffect(SoundRes.SND_CLOUD_MOVE)

	--	cc.Director:sharedDirector():getScheduler():setTimeScale(0.1)
	self.uiRTCloud:stopAllActions()
	self.uiRTCloud:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.4,cc.p(V_SZ.width,V_SZ.height)),
		cc.CallFunc:create(function()
			if completeCall then
				--				cc.Director:sharedDirector():getScheduler():setTimeScale(1)
				completeCall()
			end
		end)
	))

	self.uiRBCloud:stopAllActions()
	self.uiRBCloud:runAction(cc.MoveTo:create(0.4,cc.p(V_SZ.width,0)))

	self.uiLBCloud:stopAllActions()
	self.uiLBCloud:runAction(cc.MoveTo:create(0.4,cc.p(-647,self.uiLBCloud:getPositionY())))
end

function T:stop(completeCall)
	--	cc.Director:sharedDirector():getScheduler():setTimeScale(0.1)

	self.uiRTCloud:stopAllActions()
	self.uiRTCloud:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.4,cc.p(V_SZ.width+self.cloudHeight/2,V_SZ.height+self.cloudWidth/2)),
		cc.CallFunc:create(function()
			if completeCall then
				--				cc.Director:sharedDirector():getScheduler():setTimeScale(1)
				completeCall()
			end
		end)
	))

	self.uiRBCloud:stopAllActions()
	self.uiRBCloud:runAction(cc.MoveTo:create(0.4,cc.p(V_SZ.width+self.cloudHeight/2,-self.cloudWidth/2)))

	self.uiLBCloud:stopAllActions()
	self.uiLBCloud:runAction(cc.MoveTo:create(0.4,cc.p(-self.cloudHeight/2,V_SZ.height/2)))
end

return T
