---战斗窗口
--@author blank
local T = class("fightWindow", function()
	local super=require("src.app.uiScript.components.share.AbstractFullScreenWindow")

	local t = super.new(false)
	t.super = super
	return t
end)

--- 预加载资源列表
T.preResList={
	SoundRes.FSND_COMM_AIRCRAFT_DEAD,
	SoundRes.FSND_FIGHT_START,
	"res/images/fight/fightBack.plist",
	"res/images/fight/fight.plist",

	"res/images/fightEffect/aircraftExp.plist",
	{type="animations",list={
		--战机爆炸
		{name="aircraftBoom",url="boom_000",beginIndex=0,endIndex=10,extension=".png",delay=0.05},

		--战机碎片
		{name="aircraftPieces",url="suipian_000",beginIndex=0,endIndex=14,extension=".png",delay=0.10},
	}},

	"res/images/fightEffect/fightStart.plist",
	{type="animations",list={
		--开场动画
		{name="fightStart",url="fightStart_vs_000",beginIndex=1,endIndex=16,extension=".png",delay=0.06},
	}},

	--战斗胜利相关
	"res/images/fightEffect/fightVictoryEff.plist",
	{type="animations",list={
		--溜光
		{name="liuguang",url="liuguang_000",beginIndex=0,endIndex=7,extension=".png",delay=0.05},

		--旋转光束
		{name="xuanzhuanguangshu",url="xuanzhuanguangshu_000",beginIndex=0,endIndex=9,extension=".png",delay=0.10},
	}},
}

--- 战场演播层
T.FightPlayLayer=require("src.app.uiScript.components.fight.FightPlayLayer")

--- 战场菜单层
T.FightMenuLayer=require("src.app.uiScript.components.fight.FightMenuLayer")

--- 攻击特效播放的抽象类
T.AbstractATKEff=require("src.app.uiScript.components.fight.attackEffects.AbstractATKEff")

--- 战斗演播层
T.uiPlayLayer=nil

--- 菜单层
T.uiMenuLayer=nil

--- 上次播放的背景音乐的 url
T._lastMusicUrl=nil

--- 构造函数
function T:ctor()
	mapping( self, LuaEventDispatcher )

	self:init()

	self:initCompleteHandler()
end

--- 进入场景
function T:onEnter()
	--测试,放慢时间
	--	cc.Director:sharedDirector():getScheduler():setTimeScale(0.1)

	self:luaDispatchEvent(CocosEvent.ENTER)

	self._lastMusicUrl=AudioManager:getCurrentMusicUrl()
	--	AudioManager:stopMusic()
	AudioManager:playMusic(SoundRes.MUSIC_BATTLE,true)
end

--- 退出场景(关闭声音等)
function T:onExit()
	--测试,放慢时间
	--	cc.Director:sharedDirector():getScheduler():setTimeScale(1)

	self:luaDispatchEvent(CocosEvent.EXIT)

	AudioManager:stopMusic()
	if self._lastMusicUrl then
		AudioManager:playMusic(self._lastMusicUrl,self._lastMusicUrl ~= SoundRes.MUSIC_GENERAL)
	end
end

--- 内存清理
function T:cleanUp()
	self:releaseEventListeners()

	self.uiPlayLayer=nil
	self.uiMenuLayer=nil
end

--- 初始化
function T:init()
	--REL:relScreen(self,0,0,0,0)

	--战斗演播层
	local uiPlayLayer=self.FightPlayLayer()
	self:addChild(uiPlayLayer)
	self.uiPlayLayer=uiPlayLayer

	--菜单层
	local uiMenuLayer=self.FightMenuLayer()
	self:addChild(uiMenuLayer)
	self.uiMenuLayer=uiMenuLayer

	self.AbstractATKEff:setStaticUiMenuLayer(uiMenuLayer)
end

--- 初始化战斗数据
function T:initFightData(fightInitData)
	local camp1,camp2=fightInitData.camp1,fightInitData.camp2

	self.uiMenuLayer:setFightType(fightInitData.fightType)
	if camp2.id <= 0 then
		if camp2.name == "enemy" then
			camp2.name = Language:get("uc1015")
		else
			local first,second = CommonMethod:getCharFromStr(camp2.name)

			local realTitle1 = CommonMethod:getRealText(first)
			if second then
				camp2.name = realTitle1 .. " Lv." .. second
			else
				camp2.name = realTitle1
			end
		end

	end

	self.uiMenuLayer:setPlayerInfo(camp1.name,camp1.level,camp2.name,camp2.level)
	self.uiMenuLayer:setPropBuffCode(camp1.playerBuffCode,camp2.playerBuffCode)

	self.uiPlayLayer:initAircrafts(camp1.aircrafts,camp2.aircrafts)
end

--- 战斗开始
function T:start(data)
	AudioManager:playEffect(SoundRes.FSND_FIGHT_START)

	--- 遮罩
	local uiMask=ccui.Layout:create()
	self:addChild(uiMask)
	REL:relOther(uiMask,self, 0, 0, 0, 0)
	uiMask:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid )
	uiMask:setBackGroundColor(cc.c3b(0,0,0))
	uiMask:setBackGroundColorOpacity(100)
	self.uiMask=uiMask

	AbstractUI.timeout(self,function()
		uiMask:runAction(cc.Sequence:create(
			cc.FadeOut:create(0.15),
			cc.CallFunc:create(function(t)
				t:removeFromParent()
			end)
		))
	end,1)

	--播放开场动画
	local uiFightStart=cc.Sprite:create()
	uiFightStart:setScale(2.381)
	self:addChild(uiFightStart)
	REL:toOther("center","middle",uiFightStart,self,0,0)
	self.uiFightStart=uiFightStart

	local animation=ResLoader:getAnimation("fightStart")
	if animation then
		uiFightStart:runAction(cc.Sequence:create(
			cc.Animate:create(animation),
			cc.CallFunc:create(function(t)
				t:removeFromParent()
			end)
		))
	end
end

--- 设置当前的回合数
function T:setRound(round)
	self.uiMenuLayer:setRound(round)
end

--- 攻击
function T:attack(attackData)
	self.uiPlayLayer:attack(attackData)

	self.uiMenuLayer:attack(attackData)
end

--- 被攻击
function T:beAttack(beAttackData)
	self.uiPlayLayer:beAttack(beAttackData)

	self.uiMenuLayer:beAttack(beAttackData)
end

return T
