--- 战机阵位
local T=Class.FormLoc(
	function()
		return cc.Node:create()
	end,
	AbstractUI
)

--- 战机数量显示框
T.FormLocCountBox=require("src.app.uiScript.components.fight.FormLocCountBox")

T.MODEL_NAME_TEMPLATE="src.app.uiScript.components.fight.aircraftModels.{modelName}_camp{camp}"

--- 特效状态-无
T.EFFECT_STATUS_NONE=0

--- 特效状态-飞行
T.EFFECT_STATUS_FLYING=1

--- 特效状态-摇晃
T.EFFECT_STATUS_SHOCKING=2

function T:def()
	AbstractUI.def(self)

	--- 所属阵营
	--self.camp=nil

	--- 战机的逻辑数据
	--self.aircraftData=nil

	--- 战机的初始坐标
	--self.initPosition=nil

	--- 战机显示对象
	--self.uiAircraft=nil

	--- 数量显示对象
	--self.uiCount=nil

	--- 特效状态
	self.effectStatus=self.EFFECT_STATUS_NONE

	--- 当前正在执行的动作
	-- _act

	--- 震屏结束时间
	-- self.shockEndTime=nil
end

--- 构造函数
function T:ctor(camp,aircraftData)
	AbstractUI.ctor(self)

	self.camp=camp
	self.aircraftData=aircraftData
	self.dataTemplate=Aircraft_temp[aircraftData.sid]

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:init()
end

--- 进入
function T:onEnter()

end

--- 退出
function T:onExit()
end

--- 内存清理
function T:cleanUp()

end

--- 初始化
function T:init()
	self:initAircraft()
	self:initCount()

	self:setCount(self.aircraftData.count)
end

--- 初始化战机显示
function T:initAircraft()
	local T=GameUtils:getAicraftModelUrl(self.camp,self.aircraftData.sid)
	if T then
		local o=T()
		o:setScale(0.8)
		self:addChild(o)
		self.uiAircraft=o
	end
end

--- 初始化战机数量显示组件
function T:initCount()
	local o=self.FormLocCountBox(self.camp)
	self:addChild(o)
	self.uiCount=o
end

--- 战机死亡
function T:act_die()
	if self.uiCount then
		self.uiCount:removeFromParent()
	end

	local uiAircraft=self.uiAircraft
	if uiAircraft then
		uiAircraft:act_die()
	end
end

--- 飞行(前后晃动)
function T:toFlying()
	if self.effectStatus ~= self.EFFECT_STATUS_FLYING then
		if self._act then
			self:clearScheduler(self._act)
		end
		self.effectStatus=self.EFFECT_STATUS_FLYING
	end
	self.shockEndTime=nil

	--基于初始位置移动
	local baseX,baseY=self.initPosition.x,self.initPosition.y
	local direct=self.camp == Camp.My

	local r=random(2,4)
	self._act=self:interval(function()
		local x, y = self:getPosition()

		if direct then
			x=x+0.07
			y=y+0.05
			if(x > baseX + r) then
				direct = false
				r=random(2,4)
			end
		else
			x=x-0.07
			y=y-0.05
			if(x < baseX - r) then
				direct = true
				r=random(2,4)
			end
		end

		self:setPosition(x,y)
	end,0)
end

--- 震动
-- @param keepSec 持续时间
-- @param strength 强度
function T:shock(keepSec,strength)
	keepSec=keepSec or 2
	strength=strength or 3

	self.shockEndTime=TimeUtils:getSyncMillis()+keepSec*1000
	if self.effectStatus == self.EFFECT_STATUS_SHOCKING then
		return
	else
		if self._act then
			self:clearScheduler(self._act)
		end
		self.effectStatus=self.EFFECT_STATUS_SHOCKING
	end

	self._act=self:interval(function()
		if TimeUtils:getSyncMillis() >= self.shockEndTime then
			self:setPosition(self.initPosition)
			self:toFlying()
			return
		end

		local offset=cc.p(random(-strength, strength),random(-strength, strength))
		self:setPosition(cc.pAdd(self.initPosition, offset))
	end,0)
end

--- 设置战机数据
function T:setAircraftData(camp,aircraftData)
	self.camp,self.aircraftData=camp,aircraftData
end

--- 获取战机数据
function T:getAircraftData()
	return self.camp,self.aircraftData
end

--- 设置战机的初始位置
function T:setInitPosition(position)
	self.initPosition=position
end

--- 获取战机的初始位置
function T:getInitPosition()
	return self.initPosition
end

--- 设置战机数据
function T:setCount(value)
	self.uiCount:setAircraftData(self.camp,self.dataTemplate.aircraftQuality,value)
end

--- 获取战机模型
function T:getAircaftModel()
	return self.uiAircraft
end

return T
