--- 战机阵位列表
local T=Class.FormLocsLayer(
	function()
		local o=ccui.Layout:create()
		o._addChild=o.addChild

		return o
	end
)

--- 战场配置
T.FightCfg=require("src.app.configs.FightCfg")

--- 战场战机显示对象
T.FormLoc=require("src.app.uiScript.components.fight.FormLoc")

function T:def()
	--- 阵营数据
	--self.campData=nil

	--- 战机阵位列表
	--self.uiFormLocs=nil

	--- 震屏计时器SID
	self.isShocking=false

	--- 震屏结束时间
	-- self.shockEndTime=nil

	--- 震屏的原点
	-- self.shockOrigin=nil
end

--- 构造函数
-- @param campData 战机数据
function T:ctor(campData)
	self.campData=campData

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:init()
end

--- 进入
function T:onEnter()

end

--- 退出
function T:onExit()

end

--- 内存清理
function T:cleanUp()

end

--- 初始化
function T:init()

end

--- 创建战机阵位列表
function T:createFormLocs()
	local campData=self.campData
	if not campData then
		return
	end

	local aircrafts=campData.aircrafts
	if not aircrafts then
		return
	end

	self.uiFormLocs={}
	if aircrafts then
		for k,v in pairs(aircrafts) do
			self:createFormLoc(k,v)
		end
	end

end

--- 创建战机阵位
-- @param loc 位置
-- @param aircraftData 初始化战机数据
function T:createFormLoc(loc,aircraftData)
	if aircraftData then
		local uiFLoc=self.FormLoc(self.campData.camp,aircraftData)
		local p
		if self.campData.camp == Camp.CAMP1 then
			self:addChild(uiFLoc,self.FightCfg.AIRCRAFT_Z_ORDER_CAMP_1[loc])
			p=self.FightCfg.AIRCRAFT_LOCS_CAMP_1[loc]
		else
			self:addChild(uiFLoc,self.FightCfg.AIRCRAFT_Z_ORDER_CAMP_2[loc])
			p=self.FightCfg.AIRCRAFT_LOCS_CAMP_2[loc]
		end
		REL:toScreen("center","middle",uiFLoc,p.x,p.y)

		--设置战机的初始位置
		uiFLoc:setInitPosition(cc.p(uiFLoc:getPosition()))
		uiFLoc:toFlying()

		self.uiFormLocs[loc]=uiFLoc
	end
end

--- 移除战机阵位
function T:removeFormLoc(loc)
	--	local o=self.uiFormLocs[loc]
	--	if o then
	--		self:removeChild(o)
	--	end

	self.uiFormLocs[loc]=nil
end

--- 更新战机阵位列表
function T:updateFormLocs()
	local campData=self.campData
	if not campData then
		self:createFormLocs()
		return
	end

	local aircrafts=campData.aircrafts
	if not aircrafts then
		self:createFormLocs()
		return
	end

	local uiFormLocs=self.uiFormLocs
	if not uiFormLocs then
		self:createFormLocs()
		return
	end

	local camp=campData.camp
	for k,v in pairs(self.AIRCRAFT_LOCS) do
		local data=aircrafts[k]

		if uiFormLocs[k] then
			if data then
				uiFormLocs[k]:setAircraftData(campData.camp,data)
			else
				self:removeFormLoc(k)
			end
		else
			if data then
				self:createFormLoc(k,aircrafts[k])
			end
		end
	end
end

--- 获取指定位置上的战机阵位对象
-- @param loc 位置
function T:getFormLoc(loc)
	return self.uiFormLocs[loc]
end

--- 设置阵营数据
function T:setCampData(value)
	self.campData=value

	self:updateFormLocs()
end

--- 获取阵营数据
function T:getCampData()
	return self.campData
end

--- 更新战机数量
function T:updateCount(loc,count)
	local uiAircraft=self.uiFormLocs[loc]
	if uiAircraft then
		uiAircraft:setCount(count)
		if count == 0 then
			self:removeFormLoc(loc)
			uiAircraft:act_die()
			return
		end
	end
end

--- 震动屏幕
-- @param keepSec 持续时间
-- @param strength 强度
function T:shockScreen(keepSec,strength)
	if self.isShocking then
		self.shockEndTime=TimeUtils:getSyncMillis()+keepSec*1000
		return
	end

	self.shockOrigin=cc.p(self:getPosition())

	self.shockEndTime=TimeUtils:getSyncMillis()+keepSec*1000
	self.isShocking=true
	self:scheduleUpdateWithPriorityLua(function()
		if TimeUtils:getSyncMillis()>= self.shockEndTime then
			self:unscheduleUpdate()
			self.isShocking=false

			self:setPosition(self.shockOrigin)
			return
		end

		strength=strength or 5
		local offset=cc.p(random(-strength, strength),random(-strength, strength))
		self:setPosition(Cocos2dxUtils.offset(self.shockOrigin, offset))
	end, 0)
end

return T
