local AbstractATKEff=require("src.app.uiScript.components.fight.attackEffects.AbstractATKEff")

--- 激光
local T=Class.Laser(AbstractATKEff)

T.preResList = {
	SoundRes.FSND_LASER_FIRE,
	SoundRes.FSND_LASER_HIT,
	"res/images/fightEffect/laser.plist",
	{
		type = "animations",
		list ={
			-- 发射
			{ name = "jiguangxuliqishou", url = "jiguangxuliqishou_000", beginIndex = 0, endIndex = 17, extension = ".png", delay = 0.06 },
			-- 飞行
			{ name = "jiguangfeixing", url = "jiguangfeixing_000", beginIndex = 0, endIndex = 7, extension = ".png", delay = 0.12 },
			-- 爆炸
			{ name = "jiguangshouji", url = "jiguangshouji_000", beginIndex = 0, endIndex = 9, extension = ".png", delay = 0.08 },
		}
	},
}

function T:_playAttackEffect(camp, mountTgt, sendPt)
	AudioManager:playEffect(SoundRes.FSND_LASER_FIRE, false)

	local emissionEff = ResLoader:createAnimationSpriteRunOnce("jiguangxuliqishou")
	mountTgt:addChild(emissionEff,mountTgt:getFireMountChildIndex())
	
	emissionEff:setScale(1.388888)
	emissionEff:setRotation(camp == Camp.CAMP1 and self.FCfg.BULLET_ROTATION_CAMP_1 or self.FCfg.BULLET_ROTATION_CAMP_2)
	emissionEff:setAnchorPoint(cc.p(0.2, 0.46))
	emissionEff:setPosition(sendPt)
end

function T:_allBeAttackEff(camp, campUIs, sendPt, endPt, scale)
	AudioManager:playEffect(SoundRes.FSND_LASER_HIT, false)

	local beAtkEff = ResLoader:createAnimationSpriteRunOnce("jiguangfeixing")
	campUIs.uiTopEffectLayer:addChild(beAtkEff)

	beAtkEff:setScale(1.388888)
	beAtkEff:setRotation(camp == Camp.CAMP1 and self.FCfg.BULLET_ROTATION_CAMP_2 or self.FCfg.BULLET_ROTATION_CAMP_1)
	beAtkEff:setAnchorPoint(cc.p(0.39, 0.3))
	beAtkEff:setPosition(endPt)
end

function T:_playBeAttackEff(beAttackData)
	local camp = beAttackData.camp
	local loc = beAttackData.loc
	local count = beAttackData.count
	local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
	local fLoc=campUIs.uiFormLocsLayer:getFormLoc(loc)
	if not fLoc then
		return
	end

	local aModel=fLoc:getAircaftModel()
	if aModel then
		local endPt=cc.p(self:getAircaraftCoordinate(camp, loc, campUIs.uiTopEffectLayer))

		if beAttackData.isHit then
			self:playExpEffect(beAttackData, camp, aModel)

			-- 显示伤害
			self:getStaticUiMenuLayer():playMinHPWordEff(beAttackData.damage, endPt, camp, beAttackData.isCrit)

			-- 更新战机数量
			campUIs.uiFormLocsLayer:updateCount(loc, count)
		else
			--闪避
			self:getStaticUiMenuLayer():playMissEff(endPt,camp)
		end
	end
end

--- 获取爆炸特效
-- @param self
-- @param beAttackData 被攻击数据
-- @param camp 阵营
-- @param hitTgt 被攻击对象
function T:playExpEffect(beAttackData, camp, hitTgt)
	if beAttackData.count <= 0 then
		local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
		campUIs.uiCampContainer:shockScreen(1)
	else
		-- 爆炸音效
		AudioManager:playEffect(SoundRes.FSND_LASER_HIT)

		self:getFormLoc(beAttackData.camp, beAttackData.loc):shock(0.5,5)
	end

	-- 爆炸效果
	local eff = ResLoader:createAnimationSpriteRunOnce("jiguangshouji")
	eff:setAnchorPoint(cc.p(0.55, 0.4))
	hitTgt:addChild(eff,150)
	eff:setScale(1.388888)
	if camp == Camp.CAMP2 then
		eff:setFlippedX(true)
	end
end

--- 播放被攻击数据
-- @param hitData 被攻击数据
function T:playBeAttackEff(hitData)
	local desList = hitData.desList
	local firstDes = desList[1]
	local campUIs = firstDes.camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs

	local endPt = cc.p(self:getAircaraftCoordinate(firstDes.camp, firstDes.loc % 3, campUIs.uiTopEffectLayer))
	self:_allBeAttackEff(firstDes.camp,campUIs,nil,endPt,1)

	--延迟后播放被攻击效果
	campUIs.uiCampContainer:timeout(function()
		for k,beAttackData in pairs(desList) do
			self:_playBeAttackEff(beAttackData)
		end
	end,0.15)
end

return T
