local AbstractATKEff=require("src.app.uiScript.components.fight.attackEffects.AbstractATKEff")

--- 机枪
local T=Class.MachineGun(AbstractATKEff)

T.preResList={
	SoundRes.FSND_MACHINE_GUN_FIRE,
	SoundRes.FSND_MACHINE_GUN_HIT,
	"res/images/fightEffect/machineGun.plist",
	{type="animations",list={
		--发射
		{name="jiqiangfashe_dan",url="jiqiangfashe_dan_000",beginIndex=0,endIndex=5,extension=".png",delay=0.08},
		--爆炸
		{name="jiqiangshouji_dan",url="jiqiangshouji_dan_000",beginIndex=0,endIndex=9,extension=".png",delay=0.1},
	}},
}

function T:def()
	AbstractATKEff.def(self)
	self.attactIntervalInMount={0.01,0.02}
end

--- 播放机枪攻击特效
-- @param camp 阵营
-- @param mountTgt 挂载目标
-- @param sendPt 发射点(坐标)
function T:_playAttackEffect(camp, mountTgt, sendPt)
	AudioManager:playEffect(SoundRes.FSND_MACHINE_GUN_FIRE)

	local campUIs,rotation
	if camp == Camp.CAMP1 then
		campUIs,rotation=self.camp1UIs,self.FCfg.BULLET_ROTATION_CAMP_1
	else
		campUIs,rotation=self.camp2UIs,self.FCfg.BULLET_ROTATION_CAMP_2
	end

	local emissionEff=cc.Sprite:create()
	mountTgt:addChild(emissionEff,mountTgt:getFireMountChildIndex())

	local isEnd=false
	emissionEff:runAction(cc.Sequence:create(
		cc.Animate:create(ResLoader:getAnimation("jiqiangfashe_dan")),
		cc.Animate:create(ResLoader:getAnimation("jiqiangfashe_dan")),
		cc.CallFunc:create(function(target)
			target:removeFromParent()
			isEnd=true
		end)
	))

	emissionEff:setRotation(rotation)
	emissionEff:setAnchorPoint(cc.p(0.15,0.4))
	emissionEff:setPosition(sendPt)

	--子弹>>-----------------------------------------------------------------------------
	local bulletC=campUIs.uiBottomEffectLayer

	local wPt=mountTgt:convertToWorldSpace(sendPt)
	local lPt=bulletC:convertToNodeSpace(wPt)

	--子弹发射位置
	local bsPt=cc.pAdd(lPt,MathUtil.getPointOfCircle(math.rad(-rotation),50))
	--子弹结束位置
	local bePt=cc.pAdd(lPt,MathUtil.getPointOfCircle(math.rad(-rotation),500))

	--子弹精灵帧
	local bf=ResLoader:getSpriteFrame("jiqiangzidanfeixing_00072.png")

	local delayFunc
	delayFunc=function()
		campUIs.uiCampContainer:timeout(function()
			local bullet=cc.Sprite:createWithSpriteFrame(bf)
			bullet:setRotation(rotation)
			bulletC:addChild(bullet)
			bullet:setPosition(bsPt)

			bullet:runAction(cc.Sequence:create(
				cc.MoveTo:create(0.15,bePt),
				cc.CallFunc:create(function(target)
					target:removeFromParent()
				end)
			))
			if not isEnd then
				delayFunc()
			end
		end,randomFloat(0.1,0.2))
	end
	delayFunc()
	--<<子弹-----------------------------------------------------------------------------
end

--- 被单颗子弹击中
--
function T:_singleBeAttackEff(camp,endPt,loc,isHit)
	local campUIs,rotation
	if camp == Camp.CAMP1 then
		campUIs,rotation=self.camp1UIs,self.FCfg.BULLET_ROTATION_CAMP_1
	else
		campUIs,rotation=self.camp2UIs,self.FCfg.BULLET_ROTATION_CAMP_2
	end

	local bSendPt,bEndPt
	bSendPt=cc.pAdd(endPt,MathUtil.getPointOfCircle(math.rad(-rotation),500))
	if isHit then
		bEndPt=cc.pAdd(endPt,MathUtil.getPointOfCircle(math.rad(-rotation),50))
	else
		bEndPt=cc.pAdd(endPt,MathUtil.getPointOfCircle(math.rad(-rotation),-450))
	end

	local bulletC=campUIs.uiTopEffectLayer

	--子弹
	local bullet=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("jiqiangzidanfeixing_00072.png"))
	bullet:setRotation(rotation+180)--子弹从对面打来,所有要加上180度
	bullet:setPosition(bSendPt)
	bulletC:addChild(bullet)

	bullet:runAction(cc.Sequence:create(
		cc.MoveTo:create(isHit and 0.15 or 0.3,bEndPt),
		cc.CallFunc:create(function(target)
			target:removeFromParent()

			if isHit then
				local fLoc=campUIs.uiFormLocsLayer:getFormLoc(loc)
				if not fLoc then
					--目标阵位不存在
					return
				end

				--子弹击中
				local exp=ResLoader:createAnimationSpriteRunOnce("jiqiangshouji_dan")
				exp:setScale(2)
				exp:setAnchorPoint(cc.p(0.53,0.6))
				exp:setRotation(rotation)

				--爆炸的容器
				local aModel=fLoc:getAircaftModel()
				if aModel then
					local wPt=bulletC:convertToWorldSpace(endPt)
					local lPt=aModel:convertToNodeSpace(wPt)

					exp:setPosition(lPt)
					aModel:addChild(exp,150)
                else
                    exp:setPosition(endPt)
				    bulletC:addChild(exp)
				end
			end
		end)
	))
end

function T:_playBeAttackEff(beAttackData)
	local camp = beAttackData.camp
	local loc = beAttackData.loc
	local count = beAttackData.count
	local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
	local fLoc=campUIs.uiFormLocsLayer:getFormLoc(loc)
	if not fLoc then
		--目标阵位不存在
		return
	end

	local aModel=fLoc:getAircaftModel()
	if aModel then
		local endPt=cc.p(self:getAircaraftCoordinate(camp, loc, campUIs.uiTopEffectLayer))

		--被攻击效果>>-----------------------------------------------------------------------------------------------
		local hitPs,allPts={},aModel:getHitPoints()
		for i=1,6,1 do
			local trangle=randomSelect(allPts,1)
			table.insert(hitPs,MathUtil:randomInTriangle(unpack(trangle[1])))
		end

		--每间隔0.1-0.2秒发射一颗子弹
		local i, c = 1, #hitPs
		local delayFunc
		delayFunc=function()
			campUIs.uiCampContainer:timeout(function()
				self:_singleBeAttackEff(camp,cc.pAdd(endPt,hitPs[i]),loc,beAttackData.isHit)

				i = i + 1
				if i <= c then
					delayFunc()
				end
			end,randomFloat(0.1,0.2))
		end
		delayFunc()
		--<<被攻击效果-----------------------------------------------------------------------------------------------

		if beAttackData.isHit then
			AudioManager:playEffect(SoundRes["FSND_MACHINE_GUN_HIT"])

			campUIs.uiCampContainer:timeout(function()
				-- 显示伤害
				self:getStaticUiMenuLayer():playMinHPWordEff(beAttackData.damage, endPt, camp, beAttackData.isCrit)

				-- 更新战机数量
				local uiFormLocsLayer=campUIs.uiFormLocsLayer
				uiFormLocsLayer:updateCount(loc, count)
			end,0.35)

			campUIs.uiCampContainer:timeout(function()
				if count <= 0 then
					local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
					campUIs.uiCampContainer:shockScreen(1)
				end
			end,0.55)
		else
			--闪避
			self:getStaticUiMenuLayer():playMissEff(endPt,camp)
		end
	end
end

return T
