local AbstractATKEff=require("src.app.uiScript.components.fight.attackEffects.AbstractATKEff")

--- 激光
local T=Class.Missile(AbstractATKEff)

T.preResList={
	SoundRes.FSND_MISSILE_FIRE,
	SoundRes.FSND_MISSILE_HIT,
	"res/images/fightEffect/missile.plist",
	{type="animations",list={
		--导弹发射
		{name="daodanfashe_dan",url="daodanfashe_dan_000",beginIndex=0,endIndex=9,extension=".png",delay=0.12},
		--导弹飞行
		{name="daodanfeixing_dan",url="daodanfeixing_dan_000",beginIndex=0,endIndex=9,extension=".png",delay=0.1},
		--导弹爆炸
		{name="daodanshouji_dan",url="daodanshouji_dan_000",beginIndex=0,endIndex=12,extension=".png",delay=0.1},
	}},
}

--- 播放机枪攻击特效
-- @param camp 阵营
-- @param mountTgt 挂载目标
-- @param sendPt 发射点(坐标)
function T:_playAttackEffect(camp,mountTgt,sendPt)
	AudioManager:playEffect(SoundRes.FSND_MISSILE_FIRE)

	local atkEff=ResLoader:createAnimationSpriteRunOnce("daodanfashe_dan")
	atkEff:setScale(1.388888)
	mountTgt:addChild(atkEff,mountTgt:getFireMountChildIndex())

	if random(0,100) > 50 then
		atkEff:setFlippedX(true)
	end

	atkEff:setAnchorPoint(cc.p(0.5,0.2))
	atkEff:setPosition(sendPt)
end

--- 获取爆炸特效
-- @param self
-- @param beAttackData 被攻击数据
function T:playExpEffect(beAttackData)
	AudioManager:playEffect(SoundRes.FSND_MISSILE_HIT)

	local camp,loc=beAttackData.camp,beAttackData.loc
	local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs

	-- 爆炸效果
	local eff = ResLoader:createAnimationSpriteRunOnce("daodanshouji_dan")
	eff:setScale(1.388888)
	if camp == Camp.CAMP2 then
		eff:setFlippedX(true)
	end

	local uiC,endPt
	if beAttackData.isHit then
		local fLoc=campUIs.uiFormLocsLayer:getFormLoc(loc)
		if fLoc then
			local aModel=fLoc:getAircaftModel()
			if aModel then
				uiC=aModel
				uiC:addChild(eff,150)
				
				fLoc:shock(0.7,3)
			end
		end
	end
	if not uiC then
		uiC=campUIs.uiBottomEffectLayer
		uiC:addChild(eff)
		eff:setPosition(cc.p(self:getAircaraftCoordinate(camp, loc, uiC)))
	end

	if beAttackData.count and beAttackData.count <= 0 then
		campUIs.uiCampContainer:shockScreen(1)
	end
end

function T:_singleBeAttackEff(camp,endPt,hitTgt,campUIs)
	local beAtkEff=ResLoader:createAnimationSpriteRunOnce("daodanfeixing_dan")
	beAtkEff:setScale(1.388888)
	campUIs.uiTopEffectLayer:addChild(beAtkEff)
	beAtkEff:setAnchorPoint(cc.p(0.5,0.07))
	beAtkEff:setPosition(endPt)
	if camp == Camp.CAMP2 then
		beAtkEff:setFlippedX(true)
	end
end

function T:_playBeAttackEff(beAttackData)
	local camp = beAttackData.camp
	local loc = beAttackData.loc
	local count = beAttackData.count
	local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
	local fLoc=campUIs.uiFormLocsLayer:getFormLoc(loc)
	--	if not fLoc then
	--		return
	--	end

	local endPt=cc.p(self:getAircaraftCoordinate(camp, loc, campUIs.uiTopEffectLayer))
	self:_singleBeAttackEff(camp,endPt,nil,campUIs)

	campUIs.uiCampContainer:timeout(function()
		self:playExpEffect(beAttackData)
	end,0.5)

	--击中>>-----------------------------------------------------------------------------------------------
	campUIs.uiCampContainer:timeout(function()
		local evtWpt=campUIs.uiTopEffectLayer:convertToWorldSpace(endPt)

		if beAttackData.isHit then
			-- 显示伤害
			self:getStaticUiMenuLayer():playMinHPWordEff(beAttackData.damage, evtWpt, camp, beAttackData.isCrit)

			-- 更新战机数量
			local uiFormLocsLayer=campUIs.uiFormLocsLayer
			uiFormLocsLayer:updateCount(loc, count)
		else
			if beAttackData.count then
				--闪避
				self:getStaticUiMenuLayer():playMissEff(evtWpt,camp)
			end
		end
	end,0.35)
end

--- 播放被攻击数据
-- @param hitData 被攻击数据
function T:playBeAttackEff(hitData)
	local desList = hitData.desList

	local firstDes = desList[1]
	local campUIs = firstDes.camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
	
	local i=0
	local intervalAct
	intervalAct=campUIs.uiCampContainer:interval(function()
		local data
		for k,v in pairs(desList) do
			if v.loc == i then
				data=v
				break;
			end
		end
		self:_playBeAttackEff(data or {camp=firstDes.camp,loc=i})
		
		i = i + 1
		if i > 5 then
			campUIs.uiCampContainer:clearScheduler(intervalAct)
		end
	end,0.1)
end

return T
