local AbstractATKEff=require("src.app.uiScript.components.fight.attackEffects.AbstractATKEff")

--- 粒子炮
local T=Class.ParticleGun(AbstractATKEff)

T.preResList = {
	SoundRes.FSND_PARTICLE_GUN_FIRE,
	SoundRes.FSND_PARTICLE_GUN_HIT,
	"res/images/fightEffect/particleGun.plist",
	{
		type = "animations",
		list ={
			{ name = "lizi_ready", url = "lizi_ready_000", beginIndex = 0, endIndex = 13, extension = ".png", delay = 0.1 },
			-- 导弹飞行
			{ name = "lizi_fly", url = "lizi_fly_000", beginIndex = 0, endIndex = 4, extension = ".png", delay = 0.06 },
			-- 导弹飞行
			{ name = "lizi_fly2", url = "lizi_fly_000", beginIndex = 0, endIndex = 4, extension = ".png", delay = 0.04 },
			-- 导弹爆炸
			{ name = "lizi_exp", url = "lizi_exp_000", beginIndex = 0, endIndex = 9, extension = ".png", delay = 0.06 },
		}
	},
}

--- 播放粒子炮攻击特效
-- @param camp 阵营
-- @param mountTgt 挂载目标
-- @param sendPt 发射点(坐标)
function T:_playAttackEffect(camp, mountTgt, sendPt)
	local campUIs,rotation
	if camp == Camp.CAMP1 then
		campUIs,rotation=self.camp1UIs,self.FCfg.BULLET_ROTATION_CAMP_1
	else
		campUIs,rotation=self.camp2UIs,self.FCfg.BULLET_ROTATION_CAMP_2
	end

	--蓄力效果>>----------------------------------------------------------------------
	local readyEff = ResLoader:createAnimationSpriteRunOnce("lizi_ready")
	readyEff:setScale(1.9999)
	mountTgt:addChild(readyEff,mountTgt:getFireMountChildIndex())
	readyEff:setRotation(rotation+30)

	readyEff:setAnchorPoint(cc.p(0.6, 0.6))
	readyEff:setPosition(sendPt)
	--<<蓄力效果----------------------------------------------------------------------

	--粒子飞行>>----------------------------------------------------------------------
	local bulletC=campUIs.uiBottomEffectLayer

	local wPt=mountTgt:convertToWorldSpace(sendPt)
	local lPt=bulletC:convertToNodeSpace(wPt)

	campUIs.uiCampContainer:timeout(function()
		AudioManager:playEffect(SoundRes.FSND_PARTICLE_GUN_FIRE)
	end,0.15)

	campUIs.uiCampContainer:timeout(function()
		local bulletEff = ResLoader:createAnimationSpriteRunOnce("lizi_fly")
		bulletEff:setScale(1.9999)
		bulletC:addChild(bulletEff)
		bulletEff:setRotation(rotation)

		bulletEff:setAnchorPoint(cc.p(0.2, 0.47))
		bulletEff:setPosition(lPt)
	end,0.45)
	--<<粒子飞行----------------------------------------------------------------------
end

function T:_singleBeAttackEff(desCamp,endPt,hitTgt,campUIs)
	local beAtkEff = ResLoader:createAnimationSpriteRunOnce("lizi_fly2")
	beAtkEff:setScale(1.9999)
	campUIs.uiTopEffectLayer:addChild(beAtkEff)
	beAtkEff:setRotation(desCamp == Camp.CAMP1 and self.FCfg.BULLET_ROTATION_CAMP_2 or self.FCfg.BULLET_ROTATION_CAMP_1)

	beAtkEff:setAnchorPoint(cc.p(1, 0.5))
	beAtkEff:setPosition(endPt)
end

function T:_playBeAttackEff(beAttackData)
	local camp = beAttackData.camp
	local loc = beAttackData.loc
	local count = beAttackData.count
	local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
	local fLoc=campUIs.uiFormLocsLayer:getFormLoc(loc)
	if not fLoc then
		return
	end

	local aModel=fLoc:getAircaftModel()
	if aModel then
		local endPt=cc.p(self:getAircaraftCoordinate(camp, loc, campUIs.uiTopEffectLayer))

		--被攻击效果>>-----------------------------------------------------------------------------------------------
		self:_singleBeAttackEff(camp,endPt,aModel,campUIs)
		--<<被攻击效果-----------------------------------------------------------------------------------------------

		campUIs.uiCampContainer:timeout(function()
			local evtWpt=campUIs.uiTopEffectLayer:convertToWorldSpace(endPt)
			
			if beAttackData.isHit then
				self:playExpEffect(beAttackData, camp, aModel)

				-- 显示伤害
				self:getStaticUiMenuLayer():playMinHPWordEff(beAttackData.damage, endPt, camp, beAttackData.isCrit)

				-- 更新战机数量
				campUIs.uiFormLocsLayer:updateCount(loc, count)
			else
				--闪避
				self:getStaticUiMenuLayer():playMissEff(endPt,camp)
			end
		end,0.15)
	end
end

--- 获取爆炸特效
-- @param self
-- @param beAttackData 被攻击数据
-- @param camp 阵营
-- @param hitTgt 被攻击对象
function T:playExpEffect(beAttackData, camp, hitTgt)
	if beAttackData.count <= 0 then
		local campUIs = camp == Camp.CAMP1 and self.camp1UIs or self.camp2UIs
		campUIs.uiCampContainer:shockScreen(1)
	else
		-- 爆炸音效
		AudioManager:playEffect(SoundRes.FSND_PARTICLE_GUN_HIT,false)

		self:getFormLoc(beAttackData.camp, beAttackData.loc):shock(0.7,5)
	end

	-- 爆炸效果
	local eff = ResLoader:createAnimationSpriteRunOnce("lizi_exp")
	eff:setAnchorPoint(cc.p(0.55, 0.4))
	hitTgt:addChild(eff,150)
	eff:setScale(1.9999)

	if camp == Camp.CAMP2 then
		eff:setFlippedX(true)
	end
end

return T
