--[[
 *	战机强化
]]
local T = class("T",function() return ccui.Layout:create() end)

--- tag的基础值
T.baseTagValue = 10000

-- tag:哪一个等级的标记，默认是1(就是等级)
T.tag = 1

--- 构造函数
function T:ctor()
    
end

function T:init()
    self:registerScriptHandler(
        function(eventType)
            if eventType == CocosEvent.ENTER then
                self:onEnter()
            elseif eventType == CocosEvent.EXIT then
                self:onExit()
            elseif eventType == CocosEvent.CLEAN_UP then
            end
        end
    )

    self.viewTag = AircraftStrengManager:getSids()
    self.viewTagName = {Language:get("uc370"),Language:get("uc371"),Language:get("uc372"),Language:get("uc373"),Language:get("uc1075")}
    self.btns = {}
    
    local listView = ListView.new()
    self:addChild(listView)
    listView:setContentSize(cc.size(620,80))
    REL:toOther("center","top",listView,self,30,-35)
    listView:setDirection(ccui.ScrollViewDir.horizontal)
    self.listView = listView
    listView:setTouchEnabled(false)

    self:_addListView()
    self:_setButton()
    
    self:addPageView()

    self:addBtn()
    self:addBottom()
    self:refreshData()

end

--- 进入
function T:onEnter()
    AircraftStrengManager:luaAddEventListener(AircraftStrengManager.EVT_ATTACK_STRENGTH,self,self.updateAttack)
    AircraftStrengManager:luaAddEventListener(AircraftStrengManager.EVT_HP_STRENGTH,self,self.updateHp)
end

--- 退出
function T:onExit()
    AircraftStrengManager:luaRemoveEventListener(AircraftStrengManager.EVT_ATTACK_STRENGTH,self,self.updateAttack)
    AircraftStrengManager:luaRemoveEventListener(AircraftStrengManager.EVT_HP_STRENGTH,self,self.updateHp)
end

function T:_addListView()
    local viewTagName = self.viewTagName
    for i = 1,5,1 do -- 现在只有5种
        local btn = Button.new("false","true")
	    btn:setButtonState("littleSwitchDown.png","littleSwitchNormal.png")
        btn:setTag(self.baseTagValue + i)
	    btn:setString(viewTagName[i])
        btn:setTextCenter()
        self.listView:pushBackCustomItem(btn)
        self.btns[i] = btn
    end

end

--- 回调监听
function T:_setButton()
    local function callback(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local tag = sender:getTag()
            self.tag = tag - self.baseTagValue
            self:refreshData()
        end
    end


    self.btns[1]:addTouchEventListener(callback)
    self.btns[2]:addTouchEventListener(callback)
    self.btns[3]:addTouchEventListener(callback)
    self.btns[4]:addTouchEventListener(callback)
    self.btns[5]:addTouchEventListener(callback)

end

function T:refreshData()
    local tag = self.tag
    local viewTag = self.viewTag
    local btns = self.btns

    self.pageView:scrollToPage(tag - 1)

    for i = 1,5,1 do       
        if i == tag then -- 设置为显示
            btns[i]:setDownState() -- 如果有按下状态，则设置按下状态
        else
            btns[i]:setNormalState()
        end
    end

    if tag == 5 then
        self.rightBtn:setVisible(false)
        self.leftBtn:setVisible(true)
    elseif tag == 1 then
        self.leftBtn:setVisible(false)
        self.rightBtn:setVisible(true)
    else
        self.rightBtn:setVisible(true)
        self.leftBtn:setVisible(true)
    end

    self:updateAttack()
    self:updateHp()

end

--- 翻页控件
function T:addPageView()
    local pageView = ccui.PageView:create()
    pageView:setTouchEnabled(true)
    pageView:setCustomScrollThreshold(60)
    pageView:setContentSize(cc.size(570, 554))
    self:addChild(pageView)
    REL:toOther("center","top",pageView,self,0,30)
    self.pageView = pageView


    local btns = self.btns
    local viewTag = self.viewTag

    for i = 1,5,1 do
        local labAllItemView = require("src.app.uiScript.components.laboratory.LabAircraftAllItemView").new(i,viewTag[i])
        pageView:addPage(labAllItemView)
        btns[i].allItemView = labAllItemView
    end 
    

    local function pageViewEvent(sender, eventType)
        if eventType == ccui.PageViewEventType.turning then
            local index =  pageView:getCurPageIndex()
            self.tag = index + 1
            self:refreshData()
        end
    end 

    pageView:addEventListener(pageViewEvent)
end

function T:addBtn()
    local leftBtn = Button.new("false","false")
	leftBtn:setButtonState("arrowLeft.png","arrowLeft.png")
    self:addChild(leftBtn)
    REL:toOther("left","middle",leftBtn,self.pageView,-40,0)
    leftBtn:addTouchEventListener(function (sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local tag = self.tag
            if tag > 1 then
                self.tag = tag - 1
                self:refreshData()
            end
        end
    end)
    self.leftBtn = leftBtn

    local rightBtn = Button.new("false","false")
	rightBtn:setButtonState("arrowRight.png","arrowRight.png")
    self:addChild(rightBtn)
    REL:toOther("right","middle",rightBtn,self.pageView,-40,0)
    rightBtn:addTouchEventListener(function (sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local tag = self.tag
            if tag < 5 then
                self.tag = tag + 1
                self:refreshData()
            end
        end
    end)
    self.rightBtn = rightBtn

end

--- 添加底部
function T:addBottom()
     -- 背景图片
    local back = Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("cvsBack3.png"))
    back:setContentSize(cc.size(600,180))
	self:addChild(back)
    REL:toOther("center","top",back,self,0,40 + 554)

    -- 信息按钮
    local info = Button.new("false","false")
	info:setButtonState("infoButton_normal.png","iconButton_general_down.png")
    self:addChild(info)
    REL:toOther("left","top",info,back,15,10)
    info:addTouchEventListener(function(sender, eventType) 
        if eventType == ccui.TouchEventType.ended then
            local layout = ccui.Layout:create()
            layout:setContentSize(cc.size(544,226))

            local text = XMLLabel()
            text:setContentSize(cc.size(500,36))
            text:setVAlign("middle")
            layout:addChild(text)
            REL:toOther("left","middle",text,layout,30,-30)
            text:setString(Language:get("uc811"))

            MUIAlter_noTitle.show(
                {
                    info = layout,
                    flag = 4,
                }
            )
        end
    end)

    -- 拥有的道具：数据核心数量
    local icon = ccui.ImageView:create()

    local iconPath = IconManager.iconPath
    icon:loadTexture(iconPath .. Prop_temp[204].propIcon)
    icon:setScale(IconManager.halfScale[1],IconManager.halfScale[2])

    self:addChild(icon)
    REL:toOther("center","top",icon,info,0,100)

    -- 拥有数量
    local ownNum = XMLLabel()
    ownNum:setContentSize(cc.size(100,27))
    self:addChild(ownNum)
    REL:toOther("left","middle",ownNum,icon,55,0)
    ownNum:setString(Language:get("uc527",PropsInfoMgr:getPlayerPropByID(204)))
    self.ownNum = ownNum

    -- 攻击等级
    local attackLevel = XMLLabel()
    attackLevel:setContentSize(cc.size(300, 36))
    attackLevel:setHAlign("center")
    self:addChild(attackLevel)
    REL:toOther("center", "top", attackLevel, back,-50,10)
    self.attackLevel = attackLevel

    -- 攻击强化按钮
    local attackBtn = Button.new("false","true")
	attackBtn:setButtonState("bigButton_yellow_normal.png","bigButton_general_down.png")
	attackBtn:setString(Language:get("uc375"))
    attackBtn:setTextCenter()
    self:addChild(attackBtn)
    REL:toOther("center", "top", attackBtn, attackLevel,0,20)
    attackBtn:addTouchEventListener(function(sender, eventType) 
        if eventType == ccui.TouchEventType.ended then
--            -- 上一级别对应属性等级没有强化满时，当前强化等级不能超过上一级的最大等级
--            if self.tag ~= 1 then
--                local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
--                local lastInfo = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag - 1)
--                if lastInfo.atkLevel < lastInfo:getMAXLevel() and info.atkLevel >= lastInfo:getMAXLevel() then
--                    MUIPop.lastLevelNotFull()
--                    return
--                end

--            end

            -- 不能超过上个级别对应属性等级,但是上一个等级强化满时不限制
            if self.tag ~= 1 then
                local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
                local lastInfo = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag - 1)
                if lastInfo.atkLevel < lastInfo:getMAXLevel() and lastInfo.atkLevel <= info.atkLevel then
                    MUIPop.lastLevelNotFull()
                    return
                end

            end


            -- 判断sid=204道具的数量是否足够
            local ownNum = PropsInfoMgr:getPlayerPropByID(204)
            local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
            local needNum = info:getAttackNeedPropNum()
            if needNum > ownNum then
                local price = Prop_temp[204].price * 0.8
                local nowNeedNum = needNum - ownNum
                local needPrice = price * nowNeedNum

                local function confirm()
                    local function finish()
                        local descripe = Language:get("uc379")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                    Command:UpgradeAttrNeedGmesCommand(self.tag,0,needNum,needPrice,finish)
                end
                MUIPop.MUIImmediate(Language:get("uc380",nowNeedNum,needPrice),confirm)

            else
                --local function confirm()
                    local function finish()
                        local descripe = Language:get("uc379")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                    Command:UpgradeAttrNoGmesCommand(self.tag,0,needNum,finish)
                --end
                --MUIPop.MUIImmediate(Language:get("uc381",needNum),confirm)

            end


        end
    end)
    self.attackBtn = attackBtn

    -- 血量等级
    local hpLevel = XMLLabel()
    hpLevel:setHAlign("center")
    hpLevel:setContentSize(cc.size(300, 36))
    self:addChild(hpLevel)
    REL:toOther("center", "top", hpLevel, back,150,10)
    self.hpLevel = hpLevel

    -- 血量强化按钮
    local hpBtn = Button.new("false","true")
	hpBtn:setButtonState("bigButton_yellow_normal.png","bigButton_general_down.png")
	hpBtn:setString(Language:get("uc376"))
    hpBtn:setTextCenter()
    self:addChild(hpBtn)
    REL:toOther("center", "top", hpBtn, hpLevel,0,20)
    hpBtn:addTouchEventListener(function(sender, eventType) 
        if eventType == ccui.TouchEventType.ended then
--            -- 上一级别对应属性等级没有强化满时，当前强化等级不能超过上一级的最大等级
--            if self.tag ~= 1 then
--                local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
--                local lastInfo = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag - 1)
--                if lastInfo.hpLevel < lastInfo:getMAXLevel() and info.hpLevel >= lastInfo:getMAXLevel() then
--                    MUIPop.lastLevelNotFull()
--                    return
--                end
--            end


            -- 不能超过上个级别对应属性等级
            if self.tag ~= 1 then
                local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
                local lastInfo = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag - 1)
                if lastInfo.hpLevel < lastInfo:getMAXLevel() and  lastInfo.hpLevel <= info.hpLevel then
                    MUIPop.lastLevelNotFull()
                    return
                end
            end


            local ownNum = PropsInfoMgr:getPlayerPropByID(204)
            local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
            local needNum = info:getHpNeedPropNum()

            if needNum > ownNum then
                local price = Prop_temp[204].price * 0.8
                local nowNeedNum = needNum - ownNum
                local needPrice = price * nowNeedNum

                local function confirm()
                    local function finish()
                        local descripe = Language:get("uc378")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                    Command:UpgradeAttrNeedGmesCommand(self.tag,1,needNum,needPrice,finish)
                end
                MUIPop.MUIImmediate(Language:get("uc380",nowNeedNum,needPrice),confirm)


            else    
                --local function confirm()
                    local function finish()
                        local descripe = Language:get("uc378")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                    Command:UpgradeAttrNoGmesCommand(self.tag,1,needNum,finish)
                --end
                --MUIPop.MUIImmediate(Language:get("uc381",needNum),confirm)
            end


        end
    end)
    self.hpBtn = hpBtn


    -- 添加消耗
    local attackCostText = XMLLabel()
    attackCostText:setVAlign("middle")
    attackCostText:setContentSize(cc.size(300, 36))
    self:addChild(attackCostText)
    REL:toOther("left", "top", attackCostText, attackBtn,20,80)
    self.attackCostText = attackCostText
    attackCostText:setString(Language:get("uc528"))

    attackCostText:updateView()
	local attackCostTextWidth = attackCostText:getTextWidth()


    local attackCostIcon = ccui.ImageView:create()
    attackCostIcon:loadTexture(iconPath .. Prop_temp[204].propIcon)
    attackCostIcon:setScale(IconManager.halfScale[1],IconManager.halfScale[2])
    self:addChild(attackCostIcon)
    REL:toOther("left","middle",attackCostIcon,attackCostText,attackCostTextWidth + 5,0)
    self.attackCostIcon = attackCostIcon

    local attackCost = XMLLabel()
    attackCost:setVAlign("middle")
    attackCost:setContentSize(cc.size(300, 36))
    self:addChild(attackCost)
    REL:toOther("left", "middle", attackCost, attackCostIcon,55,0)
    self.attackCost = attackCost

    --
    local hpCostText = XMLLabel()
    hpCostText:setVAlign("middle")
    hpCostText:setContentSize(cc.size(300, 36))
    self:addChild(hpCostText)
    REL:toOther("left", "top", hpCostText, hpBtn,20,80)
    self.hpCostText = hpCostText
    hpCostText:setString(Language:get("uc528"))


    hpCostText:updateView()
	local hpCostTextWidth = hpCostText:getTextWidth()

    local hpCostIcon = ccui.ImageView:create()
    hpCostIcon:loadTexture(iconPath .. Prop_temp[204].propIcon)
    hpCostIcon:setScale(IconManager.halfScale[1],IconManager.halfScale[2])
    self:addChild(hpCostIcon)
    REL:toOther("left","middle",hpCostIcon,hpCostText,hpCostTextWidth + 5,0)
    self.hpCostIcon = hpCostIcon


    local hpCost = XMLLabel()
    hpCost:setVAlign("middle")
    hpCost:setContentSize(cc.size(300, 36))
    self:addChild(hpCost)
    REL:toOther("left", "middle", hpCost, hpCostIcon,55,0)
    self.hpCost = hpCost

end


--- 更新攻击等级，按钮显示
function T:updateAttack()
    local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)
    
    local maxLevel = info:getMAXLevel()
    self.attackLevel:setString(Language:get("uc377",info.atkLevel,maxLevel))

    self.ownNum:setString(Language:get("uc527",PropsInfoMgr:getPlayerPropByID(204)))
    
    if info.atkLevel >= maxLevel then
        self.attackBtn:setIsGray(true)
        self.attackBtn:setTouchEnabled(false)

        self.attackCostText:setVisible(false)
        self.attackCostIcon:setVisible(false)
        self.attackCost:setVisible(false)
    else
        self.attackBtn:setIsGray(false)
        self.attackBtn:setTouchEnabled(true)

        self.attackCostText:setVisible(true)
        self.attackCostIcon:setVisible(true)
        self.attackCost:setVisible(true)

        local needNum = info:getAttackNeedPropNum()
        self.attackCost:setString(Language:get("uc527",needNum))
    end

end

--- 更新血量等级，按钮显示
function T:updateHp()
    local info = AircraftStrengManager.aircraftStrengBox:getStrengInfo(self.tag)

    local maxLevel = info:getMAXLevel()
    self.hpLevel:setString(Language:get("uc377",info.hpLevel,maxLevel))

    self.ownNum:setString(Language:get("uc527",PropsInfoMgr:getPlayerPropByID(204)))

    if info.hpLevel >= maxLevel then
        self.hpBtn:setIsGray(true)
        self.hpBtn:setTouchEnabled(false)

        self.hpCostText:setVisible(false)
        self.hpCostIcon:setVisible(false)
        self.hpCost:setVisible(false)
    else
        self.hpBtn:setIsGray(false)
        self.hpBtn:setTouchEnabled(true)

        self.hpCostText:setVisible(true)
        self.hpCostIcon:setVisible(true)
        self.hpCost:setVisible(true)

        local needNum = info:getHpNeedPropNum()
        self.hpCost:setString(Language:get("uc527",needNum))
    end

end


return T