--- 语言列表画布
local T=Class.LanguageListCvs(
	function()
		return ccui.Layout:create()
	end,
	AbstractUI
)

--- 语言工具类
T.LanguageUtils=require("src.app.utils.LanguageUtils")

--语言列表项
T.LanguageItem=require("src.app.uiScript.components.login.LanguageItem")

function T:def()
	AbstractUI.def(self)

	--- 语言项列表
	self.languageIitemList={}

	--- 当前选择项
	-- _selectedItem
	
	--- 语言改变的回调
	-- _changedCallback
end

function T:ctor(changedCallback)
	self._changedCallback=changedCallback
	AbstractUI.ctor(self)

	self:_init()
end

function T:_init()
	self:setContentSize(cc.size(560,512))

	--	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	--	self:setBackGroundColor(cc.c3b(random(0,255),0,0))
	--	self:setBackGroundColorOpacity(100)

	--列表背景
	local uiListBack=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("worldPopMiddleFrame.png"))
	uiListBack:setContentSize(cc.size(502,330))
	self:addChild(uiListBack)
	REL:relOther(uiListBack,self,10,10,10,100)
	self.uiListBack=uiListBack

	--列表
	local uiLanguageList=ccui.ScrollView:create()
	self:addChild(uiLanguageList)
	REL:relOther(uiLanguageList,uiListBack,0,0,4,4)
	self.uiLanguageList=uiLanguageList

	--"确定"按钮
	local uiConfirmButton=BaseButton:createButton(Language:get("ug101"),"bigButton_green_normal.png","bigButton_general_down.png")
	self:addChild(uiConfirmButton)
	REL:toOther("center","bottom",uiConfirmButton,self,0,10)
	self.uiConfirmButton=uiConfirmButton

	uiConfirmButton:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:changeLanguageHandler()
		end
	end)

	self:updateLanguageList()
	self:updateCurrentLanguage()
end

--- 刷新语言列表
function T:updateLanguageList()
	local uiC = self.uiLanguageList:getInnerContainer()

	local items=self.languageIitemList
	local lList=self:getLanuageList()
	for k,v in ipairs(lList) do
		local uiItem=self.LanguageItem()
		uiItem:setData(v)
		uiC:addChild(uiItem)

		uiItem:setTouchEnabled(true)
		uiItem:addTouchEventListener(function(t,e)
			if e == ccui.TouchEventType.ended then
				self:setSelectedItem(t)
			end
		end)

		items[#items+1]=uiItem
	end

	-- 以下为刷新列表项的位置
	local len=#lList
	local w, h=self.LanguageItem.WIDTH,self.LanguageItem.HEIGHT

	-- 项与项之间的间隔
	local W_INTERVAL,H_INTERVAL = 8, 6

	-- 更新显示的容器size
	local s=self.uiLanguageList:getContentSize()
	local cW = s.width
	-- 列数
	local colsNum = math.floor(cW /(w + W_INTERVAL))
	local tW =(w + W_INTERVAL) * colsNum + W_INTERVAL
	if tW > cW then
		colsNum = colsNum - 1
		tW =(w + W_INTERVAL) * colsNum + W_INTERVAL
	end
	local cH = math.ceil(len / colsNum) *(h + H_INTERVAL) + H_INTERVAL
	cH=cH<s.height and s.height or cH
	uiC:setContentSize(cc.size(cW, cH))
	--	uiC:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	--	uiC:setBackGroundColor(cc.c3b(100, 0, 0))
	--	uiC:setBackGroundColorOpacity(100)

	local offsetX = tW / 2

	-- 更新项的显示位置
	local top
	for i = 0, len - 1, 1 do
		local item = items[i + 1]

		local m = i % colsNum
		if m == 0 then
			top =(i / colsNum) *(h + H_INTERVAL) + H_INTERVAL
		end
		REL:toOther("center", "top", item, uiC,(m *(w + W_INTERVAL) + W_INTERVAL + w / 2) - offsetX, top)
	end
end

--- 刷新当前语言
function T:updateCurrentLanguage()
	local curId=self.LanguageUtils:getId()

	for k,v in ipairs(self.languageIitemList) do
		if v:getData().id == curId then
			self:setSelectedItem(v)
		else
			v:setIsSelected(false)
		end
	end
end

--- 设置当前选择项
function T:setSelectedItem(uiItem)
	if uiItem == self._selectedItem then
		return
	end

	if self._selectedItem then
		self._selectedItem:setIsSelected(false)
	end

	local uiCBtn=self.uiConfirmButton
	if uiItem then
		uiItem:setIsSelected(true)

		if uiItem:getData().id ~= self.LanguageUtils:getId() then
			uiCBtn:setIsGray(false)
			uiCBtn:setTouchEnabled(true)
		else
			uiCBtn:setIsGray(true)
			uiCBtn:setTouchEnabled(false)
		end
	else
		uiCBtn:setIsGray(true)
		uiCBtn:setTouchEnabled(false)
	end
	self._selectedItem=uiItem
end

--- 获取语言列表
function T:getLanuageList()
	local l=require("res.data.lang.languageList")
	unrequire("res.data.lang.languageList")

	local lList={}
	for k,v in pairs(l) do
		if v.isOpen then
			table.insert(lList,v)
		end
	end

	table.sort(lList,function(a,b)
		return a.priority < b.priority
	end)

	return lList
end

--- 处理语言的改变
function T:changeLanguageHandler()
	if not self._selectedItem then
		return 
	end

	--- 图片字体配置
	unrequire("src.app.BMFontCfg")
	BMFontCfg=require("src.app.BMFontCfg")

	self.LanguageUtils:init(self._selectedItem:getData())
	self.LanguageUtils:reloadFirstFiles()
	
	self.__hide__()
	
	if self._changedCallback then
		self._changedCallback()
	end
end

return T
