local AbstractLoginView=require("src.app.uiScript.components.login.AbstractLoginView")

--- 易接登陆视图
local T=Class.LoginView_1(AbstractLoginView)

function T:def()
	AbstractLoginView.def(self)
end

function T:ctor()
	AbstractLoginView.ctor(self)
end

function T:onEnter()
	AbstractLoginView.onEnter(self)

	SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_LOGIN_SUCCESS,self,self.sdkLoginSuccessHandler)
	SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_LOGIN_FAILURE,self,self.sdkLoginFailureHandler)
	SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_LOGINOUT,self,self.sdkLogoutHandler)
end

function T:onExit()
	AbstractLoginView.onExit(self)

	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_LOGIN_SUCCESS,self,self.sdkLoginSuccessHandler)
	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_LOGIN_FAILURE,self,self.sdkLoginFailureHandler)
	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_LOGINOUT,self,self.sdkLogoutHandler)
end

function T:init()
	AbstractLoginView.init(self)

	self.uiEnter:setString(Language:get("ug_eg_startGame"))
end

--- 获取登陆信息
function T:doGetLogin()
	local lstate=SDK_Delegate:getLoginState()

	if lstate == SDK_Delegate.LOGIN_STATE_LOGIN or lstate == SDK_Delegate.LOGIN_STATE_RELOGIN then
		SDK_Delegate:doLogin()
	else
		SDK_Delegate:doSwitchAccount()
	end
end

--- 登陆服务器成功
function T:loginSuccHandler()
	AbstractLoginView.loginSuccHandler(self)

	--- 开始状态监听
	require("src.app.logic.platformSDKs.sdk_1.SDKStateDetector"):start()
end

--- 进入用户名输入视图
function T:enterUNameInputView()
	--因为可能会卡进程,会造成上退出特效没有播放完成就播放进入特效,导致按钮不能复位
	self:playExitEff(function()
		SDK_Delegate:doSwitchAccount()
	end)
end

--- 使用SDK登陆完成
function T:sdkLoginSuccessHandler()
	print("1 sdk 登陆成功,刷新用户信息")

	local data=DataStorage:getLoginData() or {serverId=ServerListMgr:getDefaultServerData().id}
	if SDK_Delegate:getIsTourist() then
		data.isTourist=true
	else
		data.isTourist=nil
		data.uid=SDK_Delegate:getUserName()
	end

	self.loginData=data

	self.uiEnter:setString(Language:get("ug17"))
	self:getLoginDataComplete()
end

--- 登陆失败
function T:sdkLoginFailureHandler()
	print("1 sdk 登陆失败")

	self.loginData=nil

	self.uiEnter:setString(Language:get("ug_eg_startGame"))
	self:showLoginButton()
end

--- 用户登出
function T:sdkLogoutHandler()
	self.loginData=nil

	self.uiEnter:setString(Language:get("ug_eg_startGame"))
	self:showLoginButton()
end

--- 开始切换账号
function T:startSwitchAccountHnalder()
	--因为可能会卡进程,会造成上退出特效没有播放完成就播放进入特效,导致按钮不能复位
	--这个是ios才有的功能,但是ios不会卡进程,所以没上面描述的问题
	self:playExitEff()
end

--- 进入游戏
function T:enterGame()
	if self.loginData == nil then
		self:enterUNameInputView()
		return
	end

	AbstractLoginView.enterGame(self)
end

--- 显示登陆按钮
function T:showLoginButton()
	local uiEnter,uiUName,uiSName=self.uiEnter,self.uiUName,self.uiSName

	uiUName:setVisible(false)
	uiSName:setVisible(false)

	uiEnter:setVisible(true)
	REL:toOther("center","bottom",uiEnter,self,-2,86)
	uiEnter:runAction(cc.FadeIn:create(0.2))
end

--- 登陆服务器失败
function T:loginFailHandler(isForceUpdate,errInfo)
	if isForceUpdate then
		self:removeLoginListeners()
		NetManager:close()

		self.loginTip:hide()
	else
		self.loginData=nil

		self.uiEnter:setString(Language:get("ug_eg_startGame"))
		self:showLoginButton()
		
		AbstractLoginView.loginFailHandler(self,errInfo)
	end
end

return T
