local AbstractLoginView=require("src.app.uiScript.components.login.AbstractLoginView")

local T=Class.LoginView_360(AbstractLoginView)

function T:def()
	AbstractLoginView.def(self)
end

function T:ctor()
	AbstractLoginView.ctor(self)
end

--- 获取登陆信息
function T:doGetLogin()
	local lstate=SDK_Delegate:getLoginState()

	self:addLoginListeners()
	if lstate == SDK_Delegate.LOGIN_STATE_LOGIN or lstate == SDK_Delegate.LOGIN_STATE_RELOGIN then
		SDK_Delegate:doLogin()
	else
		SDK_Delegate:doSwitchAccount()
	end
end

--- 进入用户名输入视图
function T:enterUNameInputView()
	SDK_Delegate:doSwitchAccount()
end

--- 添加登陆的相关侦听
function T:addLoginListeners()
	local lstate=SDK_Delegate:getLoginState()
	if lstate == SDK_Delegate.LOGIN_STATE_LOGIN or lstate == SDK_Delegate.LOGIN_STATE_RELOGIN then
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_LOGIN_SUCCESS,self,self.sdkLoginSuccessHandler)
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_LOGIN_FAILURE,self,self.sdkLoginFailureHandler)
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_LOGIN_CANCEL,self,self.sdkLoginCancelHandler)
	else
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_SWITCH_ACCOUNT_SUCCESS,self,self.sdkLoginSuccessHandler)
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_SWITCH_ACCOUNT_FAILURE,self,self.sdkLoginFailureHandler)
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_SWITCH_ACCOUNT_CANCEL,self,self.sdkLoginCancelHandler)
	end
end

--- 移除登陆的相关侦听
function T:rmvLoginListeners()
	local lstate=SDK_Delegate:getLoginState()
	if lstate == SDK_Delegate.LOGIN_STATE_LOGIN or lstate == SDK_Delegate.LOGIN_STATE_RELOGIN then
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_LOGIN_SUCCESS,self,self.sdkLoginSuccessHandler)
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_LOGIN_FAILURE,self,self.sdkLoginFailureHandler)
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_LOGIN_CANCEL,self,self.sdkLoginCancelHandler)
	else
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_SWITCH_ACCOUNT_SUCCESS,self,self.sdkLoginSuccessHandler)
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_SWITCH_ACCOUNT_FAILURE,self,self.sdkLoginFailureHandler)
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_SWITCH_ACCOUNT_CANCEL,self,self.sdkLoginCancelHandler)
	end
end

--- 使用SDK登陆完成
function T:sdkLoginSuccessHandler()
	print("登陆成功,刷新用户信息")

	self:rmvLoginListeners()

	local data=DataStorage:getLoginData() or {serverId=ServerListMgr:getDefaultServerData().id}
	data.isTourist=nil
	data.uid=SDK_Delegate:getUserName()

	self.loginData=data

	self:getLoginDataComplete()

	SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_GET_USER_INFO_SUCCESS,self,self.getUserInfoSuccessHandler)
	SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_GET_USER_INFO_FAILURE,self,self.getUserInfoFailureHandler)
end

--- 登陆失败
function T:sdkLoginFailureHandler()
	self:rmvLoginListeners()

	print("登陆失败")
end

--- 登陆被取消
function T:sdkLoginCancelHandler()
	self:rmvLoginListeners()

	print("登陆被取消")

	local data=DataStorage:getLoginData() or {serverId=ServerListMgr:getDefaultServerData().id}
	data.isTourist=nil
	data.uid=SDK_Delegate:getUserName()

	self.loginData=data

	self:getLoginDataComplete()
end

--- 获取用户信息成功
function T:getUserInfoSuccessHandler()
	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_GET_USER_INFO_SUCCESS,self,self.getUserInfoSuccessHandler)
	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_GET_USER_INFO_FAILURE,self,self.getUserInfoFailureHandler)

	print("获取用户信息成功")
end

--- 获取用户信息失败
function T:getUserInfoFailureHandler()
	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_GET_USER_INFO_SUCCESS,self,self.getUserInfoSuccessHandler)
	SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_GET_USER_INFO_FAILURE,self,self.getUserInfoFailureHandler)

	MUIAlter_singleLine.show({
		info=Language:get("sdk_360_getUserInfoFail")
	})
end

return T
