local _print=print
local function print(...)
	_print("\t<ServerListCvs>: ",table.concat({...},"\t"))
end

--- 服务器列表
local T=Class.ServerListCvs(
	function()
		return ccui.Layout:create()
	end,
	AbstractUI
)

--- 服务器列表项
T.ServerListItem=require("src.app.uiScript.components.login.ServerListItem")

function T:def()
	AbstractUI.def(self)

	--- 服务器项列表
	self._uiSList={}
	
	--- 当前选择的服务器列表项
	-- _curItem
end

function T:ctor(loginData)
	AbstractUI.ctor(self)

	self:init()
	
	self:setLoginData(loginData)
end

function T:init()
	self:setContentSize(cc.size(630,790))

	-- 背景图片
	local f = ResLoader:getSpriteFrame("worldPopFrame.png")
	local s = f:getOriginalSize()
	local uiBack = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,216,1,1))
	self:addChild(uiBack)
	REL:relOther(uiBack,self,0,0,0,0)
	self.uiBack = uiBack

	-- 标题背景
	local uiTitleBack=cc.Sprite:createWithSpriteFrameName("worldPopTopTag.png")
	self:addChild(uiTitleBack)
	REL:toOther("center","top",uiTitleBack,uiBack,0,75)
	self.uiTitleBack=uiTitleBack

	-- 标题
	local uiTitle=XMLLabel(Language:get("ug_selServerTitle"))
	uiTitle:setAlign("center","middle")
	uiTitle:setCssStyle(CssStyles.alterTitle)
	self:addChild(uiTitle)
	REL:relOther(uiTitle,uiTitleBack,10,10,4,4)
	self.uiTitle=uiTitle

	-- 服务器列表
	local uiSList=ccui.ScrollView:create()
	uiSList:setBounceEnabled(true)
	self:addChild(uiSList)
	REL:relOther(uiSList,uiBack,10,10,140,90)
	self.uiSList=uiSList

	self:updateSList()
end

--- 创建服务器列表项
function T:createSLItem()
	local uiItem=self.ServerListItem()

	uiItem:setTouchEnabled(true)
	uiItem:addTouchEventListener( function(t, e)
		if e == ccui.TouchEventType.ended then
			self:selectedSLItem(t)
			self.__hide__()
		end
	end)

	return uiItem
end

--- 选择服务器列表项
function T:selectedSLItem(sLitem)
	if not sLitem then
		return
	end

	if sLitem ~= self._curItem then
		if self._curItem then
			self._curItem:setIsSelected(false)
		end
		self._curItem=sLitem
		self._curItem:setIsSelected(true)
		
		if self._loginData then
			self._loginData.serverId=self._curItem:getData().id
		end
	end
end

--- 创建指定数目的服务器列表项
function T:createSList(len)
	local uiSList=self._uiSList
	local l1 = #uiSList
	local c = self.uiSList:getInnerContainer()

	if len > l1 then
		for i = l1 + 1, len, 1 do
			local uiItem = self:createSLItem()
			c:addChild(uiItem)

			uiSList[i] = uiItem
		end
	elseif len < l1 then
		for i = l1, len + 1, -1 do
			local uiItem = uiSList[i]

			uiItem:removeFromParent()
			uiSList[i] = nil
		end
	end

	-- 以下为刷新列表项的位置
	if not uiSList[1] then
		return
	end

	local firstItemSize=uiSList[1]:getContentSize()
	local h, w=firstItemSize.height,firstItemSize.width

	-- 项与项之间的间隔
	local W_INTERVAL,H_INTERVAL = 40, 10

	-- 更新显示的容器size
	local s=self.uiSList:getContentSize()
	local cW = s.width
	-- 列数
	local colsNum = math.floor(cW /(w + W_INTERVAL))
	local tW =(w + W_INTERVAL) * colsNum + W_INTERVAL
	if tW > cW then
		colsNum = colsNum - 1
		tW =(w + W_INTERVAL) * colsNum + W_INTERVAL
	end
	local cH = math.ceil(len / colsNum) *(h + H_INTERVAL) + H_INTERVAL
	cH=cH<s.height and s.height or cH
	c:setContentSize(cc.size(cW, cH))

	--	c:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	--	c:setBackGroundColor(cc.c3b(100, 0, 0))
	--	c:setBackGroundColorOpacity(100)

	local offsetX = tW / 2

	-- 更新项的显示位置
	local top
	for i = 0, len - 1, 1 do
		local item = uiSList[i + 1]

		local m = i % colsNum
		if m == 0 then
			top =(i / colsNum) *(h + H_INTERVAL) + H_INTERVAL
		end
		REL:toOther("center", "top", item, c,(m *(w + W_INTERVAL) + W_INTERVAL + w / 2) - offsetX, top)
	end
end

--- 更新服务器列表
function T:updateSList()
	local sList=ServerListMgr:getServerList()
	local uiSList=self._uiSList

	self:createSList(#sList)
	for k,v in ipairs(sList) do
		uiSList[k]:setData(v)
	end
end

--- 更新当前选择的服务器列表项
function T:updateSelectedSLItem()
	local lData=self._loginData
	if lData then
		for k,v in ipairs(self._uiSList) do
			if v:getData().id == self._loginData.serverId then
				self:selectedSLItem(v)
			end
		end
	end
end

--- 设置当前的登陆数据
function T:setLoginData(value)
	if value ~= self._loginData then
		self._loginData=value
		
		self:updateSelectedSLItem()
	end
end
--- 获取当前的登陆数据
function T:getLoginData()
	return self._loginData
end

return T
