--[[
 *	邮箱主窗口
]]
local T = class(
    "T",
    function() 
        local o = AbstractWindow.new("windowBack2.jpg")
        o.super = AbstractWindow
        return o
    end
)

--- 哪个标签页的标记,1:普通,2：报告,3：发件箱
T.tag = nil


--- 构造函数
function T:ctor()
    local function callBack1()
        self:ordinaryLogic()
        self:refreshData()
    end

    local function callBack2()
        self:reportLogic()
        self:refreshData()
    end

    local function callBack3()
        self:sendLogic()
        self:refreshData()
    end
    -- 创建3个
    local switchButtonContainer = SwitchButtonContainer.new("windowBack2.jpg")
    self:addChild(switchButtonContainer)
    REL:relOther(switchButtonContainer,self.uiContainer,0,0,0,0)
    switchButtonContainer:init()
    switchButtonContainer:initButtons(
        3,
        {{"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"},{"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"},{"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"}},
        {70,15},
        {callBack1,callBack2,callBack3},
        {Language:get("uc67"),Language:get("uc68"),Language:get("uc69")}
    )
    self.switchButtonContainer = switchButtonContainer

    -- 得到控件
    self:GetWidget()

    -- 进入页面显示数据
    self:ShowData()

    -- 按钮回调设置
    self:SetButton()

    self:ordinaryLogic()

    -- 调用一次，透明度才没问题
	self:initCompleteHandler()
end


function T:onEnter()
    self.super.onEnter(self)
    MailManager:luaAddEventListener(MailManager.EVT_TYPE_DELETE_ALL_MAIL,self,self.refreshData)
    MailManager:luaAddEventListener(MailManager.EVT_TYPE_DELETE_ONE_MAIL,self,self.refreshData)
    MailManager:luaAddEventListener(MailManager.EVT_TYPE_READ_MAIL,self,self.refreshData)
    MailManager:luaAddEventListener(MailManager.EVT_TYPE_RECEIVE_MAIL,self,self.refreshData)
    MailManager:luaAddEventListener(MailManager.EVT_TYPE_GET_TEN_MAIL,self,self.refreshData)
end

function T:onExit()
    self.super.onExit(self)
    MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_DELETE_ALL_MAIL,self,self.refreshData)
    MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_DELETE_ONE_MAIL,self,self.refreshData)
    MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_READ_MAIL,self,self.refreshData)
    MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_RECEIVE_MAIL,self,self.refreshData)
    MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_GET_TEN_MAIL,self,self.refreshData)
end

--- 得到控件
function T:GetWidget()
    -- 底部发邮件按钮
    local btnBottomSendMail = Button.new("true","false")
    btnBottomSendMail:setName("btnBottomSendMail")
    btnBottomSendMail:setButtonState("button_green_normal.png","button_general_down.png")
    btnBottomSendMail:setButtonPic("editEMailIcon.png")
    self:addChild(btnBottomSendMail)
    REL:toOther("right","bottom",btnBottomSendMail,self,110,-15) 
    self._btnBottomSendMail = btnBottomSendMail
    REL:toOther("center","middle",btnBottomSendMail.buttonPic,btnBottomSendMail,0,15) 

    -- 底部删除按钮
    local btnBottomDeleteMail = Button.new("true","false")
    btnBottomDeleteMail:setName("btnBottomDeleteMail")
    btnBottomDeleteMail:setButtonState("button_red_normal.png","button_general_down.png")
    self:addChild(btnBottomDeleteMail)
    REL:toOther("right","bottom",btnBottomDeleteMail,self._btnBottomSendMail,self._btnBottomSendMail:getContentSize().width + 40,0)
    btnBottomDeleteMail:setButtonPic("delete.png") 
    REL:toOther("center","middle",btnBottomDeleteMail.buttonPic,btnBottomDeleteMail,0,15) 
    self._btnBottomDeleteMail = btnBottomDeleteMail

    -- 0条信息
    local labelInfo = XMLLabel()
    labelInfo:setContentSize(cc.size(200,27))
    self:addChild(labelInfo)
    REL:toOther("left","bottom",labelInfo,self,60,20)
    self._labelInfo = labelInfo

    -- 0条未读
    local labelIsNotRead = XMLLabel()
    labelIsNotRead:setContentSize(cc.size(200,27))
    self:addChild(labelIsNotRead)
    REL:toOther("left","bottom",labelIsNotRead,self,60,55)
    self._labelIsNotRead = labelIsNotRead
end

--- 进入页面显示数据
function T:ShowData()
    self:setWindowTitle(Language:get("uc66"))

    self.tag = 1 -- 设置默认值为1

    self:refreshData()
end

function T:refreshData()
    local isNotRead = 0
    local info = 0

    local mailInfosTable = MailManager:getMailBoxByIndex(self.tag)
    if mailInfosTable[1] then
        if mailInfosTable[3] then
            isNotRead = mailInfosTable[3]
        end
        
        if mailInfosTable[4] then
            info = mailInfosTable[4]
        end
        
    end
    
    if self.tag == 3 then
        self._labelIsNotRead:setVisible(false)
    else
        self._labelIsNotRead:setVisible(true)
        self._labelIsNotRead:setString(Language:get("uc70",isNotRead))
    end
    self._labelInfo:setString(Language:get("uc71",info))


    --- 标记
    self:updateBtn1()
    self:updateBtn2()

end

--- 更新普通标识
function T:updateBtn1()
    self.switchButtonContainer:updateButton1Tag(MailManager:judgeNormalIsRead())
end

--- 更新战报标识
function T:updateBtn2()
    self.switchButtonContainer:updateButton2Tag(MailManager:judgeFightMailIsRead())
end

--- 按钮回调设置
function T:SetButton()
    local function callback(sender, eventType)
        if eventType == ccui.TouchEventType.ended then 
            local name = sender:getName()
            if name == "btnBottomDeleteMail" then   
                local mailInfosTable = MailManager:getMailBoxByIndex(self.tag)
                if mailInfosTable[4] and mailInfosTable[4] > 0 then  -- 如果没有邮件则不能发送命令
                    self:DeleteMailLogic()
                else
                    local descripe = Language:get("uc461")
		            MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                end
                
            else
                self:SendMailLogic()
            end
        end
    end
    self._btnBottomDeleteMail:addTouchEventListener(callback)
    self._btnBottomSendMail:addTouchEventListener(callback)
end

function T:DeleteMailLogic()
    local view = nil
    if self.tag == 1 then
        view = self.ordinary
    elseif self.tag == 2 then
        view = self.report
    else
        view = self.send
    end


    if self.tag == 2 then
        local idScout,idReport = view:getChooseIds()

        if #idReport > 0 or #idScout > 0 then
            
            local info = Language:get("uc659")
            local function confirm()
                
                if #idReport > 0 then
                    local function finish()
                        local descripe = Language:get("uc660")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                    BuildingCommand:batchDeleteMail(self.tag,idReport,finish)
                end

                if #idScout > 0 then
                    MailManager:batchDeleteScoutMail(idScout)

                    if #idReport <= 0 then
                        local descripe = Language:get("uc660")
		                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                    end
                end

            end

            MUIPop.MUIImmediate(info,confirm)

        end

        if #idScout <= 0 and #idReport <= 0 then
            local info = Language:get("uc87")
            local function confirm()
                local function finish()
                    local descripe = Language:get("uc660")
		            MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                end
                BuildingCommand:DeleteAllMail(self.tag,finish)
            end
            MUIPop.MUIImmediate(info,confirm)
        
        end 
 

    else
        if view:getChooseIds() then
            local info = Language:get("uc659")
            local function confirm()
                local function finish()
                    local descripe = Language:get("uc660")
		            MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                end
                BuildingCommand:batchDeleteMail(self.tag,view:getChooseIds(),finish)
            end
            MUIPop.MUIImmediate(info,confirm)

        else
            local info = Language:get("uc87")
            local function confirm()
                local function finish()
                    local descripe = Language:get("uc660")
		            MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                end
                BuildingCommand:DeleteAllMail(self.tag,finish)
            end
            MUIPop.MUIImmediate(info,confirm)

        end

    end




end

-- 进入发送邮件窗口
function T:SendMailLogic()
    local temp = require("src.app.uiScript.components.mail.SendMailWindow").new(1)
    WindowManager:push(temp)
end

function T:ordinaryLogic()
    self.tag = 1

    if self.ordinary == nil then
        self.ordinary = require("src.app.uiScript.components.mail.MailView").new(1)
        self:addChild(self.ordinary)
        REL:relOther(self.ordinary,self.uiContainer,0,0,90,80)
        self.ordinary:init()
    end

    if self.ordinary then
        self.ordinary:setVisible(true)
    end

    if self.report then
        self.report:setVisible(false)
    end

    if self.send then
        self.send:setVisible(false)
    end
end

function T:reportLogic()
    self.tag = 2

    if self.ordinary then
        self.ordinary:setVisible(false)
    end

    if self.report == nil then
        self.report = require("src.app.uiScript.components.mail.MailView").new(2)
        self:addChild(self.report)
        REL:relOther(self.report,self.uiContainer,0,0,90,80)
        self.report:init()
    end

    if self.report then
        self.report:setVisible(true)
    end

    if self.send then
        self.send:setVisible(false)
    end
end

function T:sendLogic()
    self.tag = 3

    if self.ordinary then
        self.ordinary:setVisible(false)
    end

    if self.report then
        self.report:setVisible(false)
    end

    if self.send == nil then
        self.send = require("src.app.uiScript.components.mail.MailView").new(3)
        self:addChild(self.send)
        REL:relOther(self.send,self.uiContainer,0,0,90,80)
        self.send:init()
    end

    if self.send then
        self.send:setVisible(true)
    end

end


return T