--[[
 * 道具中的每一项
]]
local T = class("T",function() return ccui.Layout:create() end)

--- 客户端的道具的sid
T.sid = nil

--- 传入的data：1是背包，2是商城 一级菜单标志
T.oneMenuTag = nil

--- 窗口大小
T.size = nil

--- 构造函数
function T:ctor(sid,oneMenuTag,listView)
	self.sid = sid
	self.oneMenuTag = oneMenuTag
	self.listView = listView

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
			end
		end
	)

	local listItem = ListItem3.new()
	listItem:init()
	self:addChild(listItem)

	self.size = listItem:getContentSize()
	self:setContentSize(self.size)

	REL:toOther("left","top",listItem,self,0,0)
	self.listItem = listItem

	-- 得到json中的控件
	self:GetWidget()
	-- 设置数据
	self:SetData()
	-- 按钮回调设置
	self:SetButton()

end

--- 进入
function T:onEnter()
	if self.oneMenuTag == 1 then -- 使用道具刷新
		local function update()
			local data = Prop_temp[self.sid]
			local num = PropsInfoMgr:getPlayerPropByID(self.sid)
			if num == 0 then
				local index = self.listView:getIndex(self)
				local listView = self.listView
				self.listView:removeItem(index)
				listView:refreshView()
			else
				self._hasUse:setString(Language:get("uc277",num))
			end

		end
		self.update = update
		PropsInfoMgr:luaAddEventListener(PlayerInstance.noticeMessage.prop .. self.sid,nil,self.update)
	end
end

--- 退出
function T:onExit()
	PropsInfoMgr:luaRemoveEventListener(PlayerInstance.noticeMessage.prop .. self.sid,nil,self.update)
end

--- 得到json中的控件
function T:GetWidget()
	-- 头像
	self._imgHead = self.listItem.icon

	-- 购买或者使用按钮
	self._btnBuy = self.listItem.btnUse

	-- 名字
	self._labelName = self.listItem.title

	-- 描述
	self._labelDescribe = self.listItem.describe

	-- 拥有
	self._hasUse = self.listItem.hasUse

	-- 消耗钻石
	local labelCostDiamond = XMLLabel()
	labelCostDiamond:setHAlign("center")
	labelCostDiamond:setVAlign("middle")
	labelCostDiamond:setContentSize(cc.size(300, 27))
	self:addChild(labelCostDiamond)
	self._labelCostDiamond = labelCostDiamond
end

function T:SetData()
	local data = Prop_temp[self.sid]
	-- 头像
	local iconPath = IconManager.iconPath
	self._imgHead:loadTexture(iconPath .. data.propIcon)

	self._labelName:setString(Language:get("uc276",Language:get(data.prop)))

	-- 名字
	if self.oneMenuTag == 1 then -- 使用
		local num = PropsInfoMgr:getPlayerPropByID(self.sid)

		self._hasUse:setString(Language:get("uc277",num))

		self._labelCostDiamond:setVisible(false)

		self._btnBuy:loadTextures("button_blue_normal.png","button_blue_normal.png","",ccui.TextureResType.plistType)
		self._btnBuy:setString(Language:get("uc33")) -- 按钮上的字设置为"使用"
		REL:toOther("right", "bottom", self._btnBuy, self.listItem.back, 32, 30)
		REL:toOther("center","middle",self._btnBuy.buttonText,self._btnBuy,0,12)
	else
		self._hasUse:setVisible(false)

		self._labelCostDiamond:setVisible(true)
		self._labelCostDiamond:setString(Language:get("uc111",data.price))

		self._btnBuy:loadTextures("useDiamonButton_normal.png","useDiamonButton_normal.png","",ccui.TextureResType.plistType)
		self._btnBuy:setString(Language:get("uc28")) -- 按钮上的字设置为"购买"
		REL:toOther("right", "bottom", self._btnBuy, self.listItem.back, 5, 25)
		REL:toOther("center","middle",self._btnBuy.buttonText,self._btnBuy,0,-10)
		REL:toOther("center", "bottom", self._labelCostDiamond, self._btnBuy, 15, 90)
	end

	-- 描述:这个要根据情况设置
	local description = PlayerInstance:getPropDescripe2(self.sid)
	self._labelDescribe:setString(description)


	--- 不能直接使用的道具
	local isDirectUse = data.isDirectUse
	if self.oneMenuTag == 1 and isDirectUse ~= "nil" then
		self._btnBuy:setVisible(false)
	end

end

--- 按钮回调设置
function T:SetButton()
	--进入游戏回调
	local function callback(sender, eventType)
		if eventType == ccui.TouchEventType.ended then
			if self.oneMenuTag == 1 then
				self:UseProp()
			elseif self.oneMenuTag == 2 then
				self:BuyProp()
			else
			end
		end
	end
	self._btnBuy:addTouchEventListener(callback)
end

--- 使用逻辑
function T:UseProp()
	local sid = self.sid
	if sid == 202 then
		-- 高级
		if ArmyTaskQueueManager:selfIsQueue() then
			MUIPop.selfIsArmyTast()
			return
		end

		local layout = HighFlyPop.new()
		local function confirm()
			local mapX = layout.posX
			local mapY = layout.posY
			if mapX and mapY then
				local num = PropsInfoMgr:getPlayerPropByID(sid)
				if num <= 0 then
					-- 道具数量不足，请购买道具
					MUIPop.PropNumNotEnough()
				else
					require("src.app.communication.worldMap.WorldMapFlyCommand").new():exec(
						{x = mapX, y = mapY},
						false,  -- 用钻石
						function(state)
							if state == ReturnCode.SUCCESS then
								PropsInfoMgr:setPlayerPropByID(sid, num - 1)

								local oldX, oldY = PlayerInstance:getWorldPosition()
								PlayerInstance:setWorldPosition(mapX,mapY)
								WorldMapMgr:updateNodesDataForced({
									{x = oldX, y = oldY},
									{x = mapX, y = mapY},
								})
							else
								WorldMapMgr:updateNodesDataForced({
									{x = mapX, y = mapY},
								})
							end
						end
					)
				end
			else
				local descripe = Language:get("uc449")
				MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
			end
		end
		MUIPop.HighFly(layout,confirm)


	elseif sid == 203 then
		-- 随机
		if ArmyTaskQueueManager:selfIsQueue() then
			MUIPop.selfIsArmyTast()
			return
		end


		local num = PropsInfoMgr:getPlayerPropByID(sid)
		if num <= 0 then
			-- 道具数量不足，请购买道具
			MUIPop.PropNumNotEnough()
		else
			local oldX, oldY = PlayerInstance:getWorldPosition()
			require("src.app.communication.worldMap.WorldMapFlyRandomCommand").new():exec(function(state)
				if state == ReturnCode.SUCCESS then
					PropsInfoMgr:setPlayerPropByID(sid, num - 1)

					local mapX,mapY = PlayerInstance:getWorldPosition()
					WorldMapMgr:updateNodesDataForced({
						{x = oldX, y = oldY},
						{x = mapX, y = mapY},
					})
				else
					MUIPop.UsePropFail(state)
				end
			end)
		end

    elseif sid == 211 then
        VipManager:enterResetDoubleRechagerPopWindow()
	else
		BuildingCommand.UseProp(self.sid,finish)
	end

end

--- 购买逻辑
function T:BuyProp()
	local payState = SetStates.setPayState
	local value = payState:getClosedValue(payState.CLOSE_BUY_GOODS)

	local data = Prop_temp[self.sid]
	local allDiamond = data.price
	if value then
		BuildingCommand.BuyProp(self.sid,allDiamond)
	else
		local info = Language:get("uc63",allDiamond)
		local layout = DiamondPopInfo.new(info)
		local function confirm()
			SetStates.setPayState:setClosedValue(layout.judge,SetStates.setPayState.CLOSE_BUY_GOODS)
			if layout.judge == true then
				Command:SetPayCommand()
			end

			BuildingCommand.BuyProp(self.sid,allDiamond)
		end
		MUIPop.DiamondPopInfo(layout,confirm)
	end

end


return T
