--[[
 * 物品主窗口，有商城和背包窗口
]]
local T = class(
    "RankWindow",
    function() 
        local o = AbstractWindow.new(1)
        o.super = AbstractWindow
        return o
    end
)


--- 构造函数 
function T:ctor(tag)
    self.tag = tag or 1

    -- 得到控件
    self:_getWidget()

    --- 设置数据
    self:_setData()

    self:_setButton()

    -- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

function T:onEnter()
    self.super.onEnter(self)

end

function T:onExit()
    self.super.onExit(self)
    RankManager.rankCheckPoint.startPos = 1
    RankManager.rankAchieve.startPos = 1 
    RankManager.rankFight.startPos = 1 
end

--- 得到控件
function T:_getWidget()
    -- 背景图片
    local f = ResLoader:getSpriteFrame("viewBack.png")
	local s = f:getOriginalSize()
	local viewBack = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,s.height-1,1,1))
	self:addChild(viewBack)
    REL:relOther(viewBack,self.uiContainer,0,0,0,0)
	self.viewBack = viewBack

    -- 标签按钮：战斗力
    local btnFight = Button.new("false","true")
    btnFight:setName("fight")
    btnFight:loadTextures("panelSwitchButton_down.png","panelSwitchButton_normal.png","",ccui.TextureResType.plistType)
    self:addChild(btnFight)
    REL:toOther("left", "top", btnFight, viewBack,20,10)
    self.btnFight = btnFight
    btnFight.buttonText:setHAlign("center")
    btnFight.buttonText:setVAlign("middle")
    REL:toOther("center", "middle", btnFight.buttonText, btnFight,0,0)

    -- 标签按钮：关卡
    local btnPoint = Button.new("false","true")
    btnPoint:setName("point")
    btnPoint:loadTextures("panelSwitchButton_down.png","panelSwitchButton_normal.png","",ccui.TextureResType.plistType)
    self:addChild(btnPoint)
    REL:toOther("center", "top", btnPoint, viewBack,0,10)
    self.btnPoint = btnPoint
    btnPoint.buttonText:setHAlign("center")
    btnPoint.buttonText:setVAlign("middle")
    REL:toOther("center", "middle", btnPoint.buttonText, btnPoint,0,0)

    -- 标签按钮：荣誉（成就）
    local btnAchieve = Button.new("false","true")
    btnAchieve:setName("achieve")
    btnAchieve:loadTextures("panelSwitchButton_down.png","panelSwitchButton_normal.png","",ccui.TextureResType.plistType)
    self:addChild(btnAchieve)
    REL:toOther("right", "top", btnAchieve, viewBack,20,10)
    self.btnAchieve = btnAchieve
    btnAchieve.buttonText:setHAlign("center")
    btnAchieve.buttonText:setVAlign("middle")
    REL:toOther("center", "middle", btnAchieve.buttonText, btnAchieve,0,0)

    -- rankBack图片
    local rankBackFrame = ResLoader:getSpriteFrame("rankTip.png")
    local rankBackFrameSize = rankBackFrame:getOriginalSize()
    local rankBack = Scale9Sprite:createWithFrame(rankBackFrame,cc.rect(rankBackFrameSize.width/2,rankBackFrameSize.height/2,1,1))
	self:addChild(rankBack)
    rankBack:setContentSize(cc.size(640 - 12,35))
    REL:toOther("center", "top",rankBack,btnPoint,0,btnPoint:getContentSize().height + 10)
	self.rankBack = rankBack

    -- 排名
    local uiRank = XMLLabel()
    uiRank:setAlign("center","middle")
    uiRank:setContentSize(cc.size(200,36))
    self:addChild(uiRank)
    REL:toOther("center","middle",uiRank,rankBack,-250,0)
    self.uiRank = uiRank

    -- 姓名
    local uiName = XMLLabel()
    uiName:setAlign("center","middle")
    uiName:setContentSize(cc.size(200,36))
    self:addChild(uiName)
    REL:toOther("center","middle",uiName,rankBack,-80,0)
    self.uiName = uiName

    -- 等级
    local uiLevel = XMLLabel()
    uiLevel:setAlign("center","middle")
    uiLevel:setContentSize(cc.size(200,36))
    self:addChild(uiLevel)
    REL:toOther("center","middle",uiLevel,rankBack,100,0)
    self.uiLevel = uiLevel

    -- 最后一个
    local uiLast = XMLLabel()
    uiLast:setAlign("center","middle")
    uiLast:setContentSize(cc.size(200,36))
    self:addChild(uiLast)
    REL:toOther("center","middle",uiLast,rankBack,250,0)
    self.uiLast = uiLast


--    local text = XMLLabel()
--    text:setContentSize(cc.size(1500,27))
--    self:addChild(text)
--    REL:toOther("left","middle",text,rankBack,20,0)
--    self.text = text

end

function T:_setData()
    self:setWindowTitle(Language:get("uc259"))

    self.btnFight:setString(Language:get("uc260"))
    self.btnPoint:setString(Language:get("uc261"))
    self.btnAchieve:setString(Language:get("uc262"))

    --self.text:setString(Language:get("uc263"))
    self.uiRank:setString(Language:get("uc263"))
    self.uiName:setString(Language:get("uc264"))
    self.uiLevel:setString(Language:get("uc265"))


    if self.tag == 1 then
        self:fight()
        self:setDownState(self.btnFight)
        self:setNormalState(self.btnPoint)
        self:setNormalState(self.btnAchieve)
    elseif self.tag == 2 then
        self:point()
        self:setNormalState(self.btnFight)
        self:setDownState(self.btnPoint)
        self:setNormalState(self.btnAchieve)
    else
        self:achieve()
        self:setNormalState(self.btnFight)
        self:setNormalState(self.btnPoint)
        self:setDownState(self.btnAchieve)
    end
    
end

--- 设置为正常状态
function T:setNormalState(btn)
    if btn then
        btn:loadTextureNormal("panelSwitchButton_down.png",ccui.TextureResType.plistType)
        btn:setTouchEnabled(true)
    end
end


--- 设置为按下状态
function T:setDownState(btn)
    if btn then
        btn:loadTextureNormal("panelSwitchButton_normal.png",ccui.TextureResType.plistType)
        btn:setTouchEnabled(false)
    end
end

function T:_setButton()
    local function callback(sender, eventType)
        if eventType == ccui.TouchEventType.ended then 
            local name = sender:getName()
            if name == "fight" then
                self:setDownState(self.btnFight)
                self:setNormalState(self.btnPoint)
                self:setNormalState(self.btnAchieve)
                self:fight()
            elseif name == "point" then 
                self:setNormalState(self.btnFight)
                self:setDownState(self.btnPoint)
                self:setNormalState(self.btnAchieve)
                self:point()
            else
                self:setNormalState(self.btnFight)
                self:setNormalState(self.btnPoint)
                self:setDownState(self.btnAchieve)
                self:achieve()
            end
        end
    end
    self.btnFight:addTouchEventListener(callback)
    self.btnPoint:addTouchEventListener(callback)
    self.btnAchieve:addTouchEventListener(callback)
end

function T:fight()
    --self.text:setString(Language:get("uc263"))
    self.uiLast:setString(Language:get("uc306"))
    self.uiLevel:setString(Language:get("uc265"))

    if self.fightView == nil then
        self.fightView = require("src.app.uiScript.components.rank.RankFightView").new()
        self.fightView:setLocalZOrder(20000)
        self:addChild(self.fightView)
        REL:relOther(self.fightView,self.viewBack,0,0,120,CommonCfg.BOTTOM_CONTAINER_HEIGTH)
        self.fightView:init()
    end

    if self.fightView then
        self.fightView:setVisible(true)
    end

    if self.checkPointView then
        self.checkPointView:setVisible(false)
    end

    if self.achieveView then
        self.achieveView:setVisible(false)
    end
end

function T:point()
    --self.text:setString(Language:get("uc264"))
    self.uiLast:setString(Language:get("uc307"))
    self.uiLevel:setString(Language:get("uc265"))

    if self.fightView then
        self.fightView:setVisible(false)
    end

    if self.checkPointView == nil then
        local function finish()
            self.checkPointView = require("src.app.uiScript.components.rank.RankCheckPointView").new()
            self:addChild(self.checkPointView)
            REL:relOther(self.checkPointView,self.viewBack,0,0,120,CommonCfg.BOTTOM_CONTAINER_HEIGTH)
            self.checkPointView:init()
        end
        Command:SendRank(2,1,finish)
    end

    if self.checkPointView then
        self.checkPointView:setVisible(true)
    end

    if self.achieveView then
        self.achieveView:setVisible(false)
    end
end

function T:achieve()
    --self.text:setString(Language:get("uc265"))
    self.uiLast:setString(Language:get("uc308"))
    self.uiLevel:setString(Language:get("uc982"))

    if self.fightView then
        self.fightView:setVisible(false)
    end

    if self.checkPointView then
        self.checkPointView:setVisible(false)
    end

    if self.achieveView == nil then
        local function finish()
            self.achieveView = require("src.app.uiScript.components.rank.RankItemAchieveView").new()
            self:addChild(self.achieveView)
            REL:relOther(self.achieveView,self.viewBack,0,0,120,CommonCfg.BOTTOM_CONTAINER_HEIGTH)
            self.achieveView:init()
        end
        Command:SendRank(3,1,finish)

    end

    if self.achieveView then
        self.achieveView:setVisible(true)
    end

end


return T