local WindowTitleBar=require("src.app.uiScript.components.share.WindowTitleBar")

local super=require("src.app.uiScript.components.share.AbstractFullScreenWindow")

--- 抽象窗口
-- @author blank
local T = class(
	"AbstractWindow",
	function()
		local t = super.new()

		-- 把layout的属性addChild存储起来，以便于避免本身的窗口与属于这个窗口的容器的addChild冲突
		t.__addChild__ = t.addChild

		return t
	end
)

--- 父类
T.super=super

--- 背景图片
--T.uiBack = nil

--- 上边框,包括容器
--T.uiTopBorder = nil

--- 下边框
--T.uiBottomBorder = nil

--- 标题
--T.uiTitle=nil

--- 关闭按钮
--T.uiCloseButton=nil

--- 钻石按钮
--T.uiDiamondButton=nil

--- 钻石数量
--T.uiDiamondCount=nil

--- 内容容器
--T.uiContainer = nil

--- 构造函数
function T:ctor(windowBackUrlOrType)
	self.windowBackUrlOrType=windowBackUrlOrType or 0
	mapping(self,require("src.app.uiScript.components.share.WindowEffTemplate"))

	self:init()
end

--- 进入
function T:onEnter()
	super.onEnter(self)

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.diamond,self,self.updateDiamondCount)

    PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.fighting,self,self.fightRefresh)

    UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET, self, self.updateDiamondCount2)
end

--- 退出
function T:onExit()
	super.onExit(self)

	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.diamond,self,self.updateDiamondCount)

    PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.fighting,self,self.fightRefresh)

    UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET, self, self.updateDiamondCount2)

end

--- 内存清理
function T:cleanUp()
	super.cleanUp(self)
end

--- 初始化
function T:init()
--	--标题栏背景
--	local uiTitleBack=cc.Sprite:createWithSpriteFrameName("borderTop.png")
--	self:__addChild__(uiTitleBack)
--	REL:toScreen("center","top",uiTitleBack,0,0)
--	self.uiTitleBack=uiTitleBack

	--上边框
	local uiTopBorder = WindowTitleBar()
	self:__addChild__(uiTopBorder)
	-- 由于这里顶部和底部有动画，所以还是要设置宽度和高度，所以还是用toScreen
	REL:toScreen("center", "top", uiTopBorder, 0, 0)
	self.uiTopBorder = uiTopBorder
	
	self:setWindowBack(self.windowBackUrlOrType)

	--内容容器
	local uiContainer = ccui.Layout:create()
	self:__addChild__(uiContainer)
	REL:relScreen(uiContainer,0,0,65,0)
	self.uiContainer = uiContainer

	--	uiContainer:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	--	uiContainer:setBackGroundColor(cc.c3b(0xff,0,0))
	--	uiContainer:setBackGroundColorOpacity(100)

	--关闭按钮
	local uiCloseButton = uiTopBorder:getCloseButton()
	uiCloseButton:addTouchEventListener(function(t,e)
		self:onTouchCloseButtonHandler(t,e)
	end)

	--钻石按钮
	local uiDiamondButton = uiTopBorder:getDiamondButton()
	self.uiDiamondButton=uiDiamondButton
	uiDiamondButton:addTouchEventListener(function(t,e)
		self:onTouchDiamondButtonHandler(t,e)
	end)

	self:initBottomBorder()

	if BGMgr:isGuide() then
		BGMgr:regGuideObj(BGMgr.TGR_WINDOW_CLOSE_BUTTON,uiCloseButton)
	end
end

--- 设置背景图片
-- @param urlOrType 背景图片的url或类型
-- @param textureType 当 urlOrType 指定为url时,此值为纹理类型
function T:setWindowBack(urlOrType,textureType)
	local uiBack=self.uiBack
	if uiBack then
		uiBack:removeFromParent()
	end

	local t=type(urlOrType)
	if t == "string" then
		if textureType == ccui.TextureResType.localType then
			uiBack = cc.Sprite:create(urlOrType)
		else
			uiBack = cc.Sprite:createWithSpriteFrameName(urlOrType)
		end
	else
		if urlOrType == 0 then
			return
		elseif urlOrType == 1 then
			uiBack = cc.Sprite:createWithSpriteFrameName("windowBack.jpg")
		elseif urlOrType == 2 then
			uiBack = cc.Sprite:createWithSpriteFrameName("windowBack2.jpg")
		elseif urlOrType == 3 then
			uiBack = cc.Sprite:createWithSpriteFrameName("windowBack3.jpg")
		elseif urlOrType == 4 then
			uiBack = cc.Sprite:createWithSpriteFrameName("windowBack4.jpg")
		end
	end

	self:__addChild__(uiBack)
	REL:relScreen(uiBack,0,0,65,0)
	self.uiBack = uiBack
end

--- 初始化下边框
function T:initBottomBorder()
	local layout = ccui.Layout:create()
	layout:setContentSize(cc.size(1, 136))
	self:__addChild__(layout)
	REL:relScreen(layout,0,0,SIZE_NIL,0)

	local uiLeftBottomWraper = ccui.ImageView:create("borderLeftBottom.png", ccui.TextureResType.plistType)
	layout:addChild(uiLeftBottomWraper)
	REL:toOther("left", "bottom", uiLeftBottomWraper, layout, 0, 0)

	local uiRightBottomWraper = ccui.ImageView:create("borderLeftBottom.png", ccui.TextureResType.plistType)
	uiRightBottomWraper:setFlippedX(true)
	layout:addChild(uiRightBottomWraper)
	REL:toOther("right", "bottom", uiRightBottomWraper, layout, 0, 0)

	self.uiBottomBorder = layout
end

--- 关闭按钮被点击
function T:onTouchCloseButtonHandler(t,e)
	if e == ccui.TouchEventType.ended then
		self:close()
	end
end

--- 钻石按钮被点击
function T:onTouchDiamondButtonHandler(t,e)
	if e == ccui.TouchEventType.ended then
	--跳转到钻石充值页面
	end
end

--- 设置背景
-- @param url 背景图片地址
-- @param textureResType 资源类型,默认为本地加载
function T:setBackGround(url,textureResType)
	self.uiBack:loadTexture(url,textureResType or ccui.TextureResType.localType)
end

--- 设置窗口标题
-- @param name 窗口标题
-- @param leval 等级,如果不指定,则不显示等级
function T:setWindowTitle(name,level)
	if level then
		self.uiTopBorder:setTitle(Language:get("uc100", name, level))
	else
		self.uiTopBorder:setTitle(name)
	end
end

--- 更新钻石数量
-- @param count 不指定则从用户信息中获取
function T:updateDiamondCount(count)
	self.uiTopBorder:updateDiamondCount(count)
    CommonMethod:diamondEff("left")

end

--- 没有特效的
function T:updateDiamondCount2(count)
	self.uiTopBorder:updateDiamondCount(count)
end


--- 给容器添加孩子
function T:addChild(child)
	self.uiContainer:addChild(child)
end


--- 战斗力刷新特效显示
function T:fightRefresh()
    CommonMethod:fightRefresh()
end



return T
