--[[
 *	建筑物相关的升级命令
]]
local T = {}

--[[
 *	@param dataFromServer：建筑物所对应的数据
 *	@param modelResTable：建筑物所对应的资源表
 *	@param finish：完成后的回调
]]
----------------------------------------- 普通建筑物的建造页面升级相关 ------------------------------------------------------------

--- 设置资源项的中本地内存的存储is = 0 表示没有resTable，is = 1 表示有resTable
function T.SetListViwData(dataFromServer,ModelResTable,is,resTable)
    local level = dataFromServer.level

    local buildCommandData = ModelResTable[level].need
    local num = #buildCommandData

    for i = 1,num,1 do
        local resData = buildCommandData[i] -- {2,100}
        local nowNum = UserInfo.city[Resource_temp[resData[1]].attributeName]  -- 玩家当前拥有数量
        local maxNum = resData[2] -- 等级资源需求数量

        if is == 0 then
            UserInfo.city[Resource_temp[resData[1]].attributeName] = nowNum - maxNum -- 本地内存维护一份
        else
            if resTable[i] then
                UserInfo.city[resTable[i]] = 0
            else
                UserInfo.city[Resource_temp[resData[1]].attributeName] = nowNum - maxNum
            end
        end

    end

    ResourceManager:luaDispatchEvent(ResourceManager.EVT_TYPE_RESOURCE_REFRESH)
end

--- 立刻完成命令
function T.ImmediateFinish(dataFromServer,modelResTable)
    -- 先判断钻石是否足够，如果不足够则弹出到钻石不够啦，去购买;当前拥有的钻石，和等级公式算出的钻石想比较
    local nowDiamond = UserInfo.gems
    local allDiamond = BuildingDiamondCalculation.FreeTimeGetImmediateFinishDiamond(dataFromServer,modelResTable)

    if allDiamond <= nowDiamond then
        -- 下面的state是从encode()函数里面返回的数据传到这个function里面的，瞬间完成，更新显示和数据
        UpgradeImmediatelyCommand:new():exec(dataFromServer,function(state,diamond)
            if state == ReturnCode.SUCCESS then
                PlayerInstance:setDiamond(diamond)

                local name = BuildingsInfoMgr:_getBuildName(dataFromServer.id)
                local descripe = Language:get("uc12",name,dataFromServer.level)
		        MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )
                --MUIPop.MUIUpGradeSuccess()
            else
            -- 升级失败
                --MUIPop.MUIUpGradeFail(state)
                ErrorCode:popErrorCodeTip(state)
            end
        end)

    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

--- 普通升级无钻石
function T.UpGradeNoDiamond(dataFromServer,ModelResTable,finish)
    UpgradeCommand:new():exec(dataFromServer,false,function(state,diamond)
        if state == ReturnCode.SUCCESS then
            PlayerInstance:setDiamond(diamond)
            BuildingCommand.SetListViwData(dataFromServer,ModelResTable,0)  -- 这里刷新内存数据
            finish()
        else
            ErrorCode:popErrorCodeTip(state)
        end
    end)
end

--- allDiamond为补足资源所算出来的钻石数目
--- 普通升级有钻石,补足钻石完成普通升级逻辑
function T.UpGradeHasDiamond(dataFromServer,ModelResTable,resTable,finish,allDiamond)
    local nowDiamond = UserInfo.gems

    if allDiamond <= nowDiamond then
        -- 发出用钻石升级的命令
        UpgradeCommand:new():exec(dataFromServer,true,function(state,diamond)
            if state == ReturnCode.SUCCESS then
                PlayerInstance:setDiamond(diamond)
                BuildingCommand.SetListViwData(dataFromServer,ModelResTable,1,resTable)  -- 这里刷新内存数据
                finish()
            else
                ErrorCode:popErrorCodeTip(state)
            end

        end)

    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

--- 取消升级逻辑，返回比例
function T.CancleUpGradeLogic(dataFromServer,ModelResTable,finish)
        CancleUpgradeCommand:new():exec(dataFromServer,function(state)
        if state == ReturnCode.SUCCESS then 
            --MUIPop.CancleUpGradeSuccess()

            -- 刷新内存数据:计算返回的资源比例
--            local level = dataFromServer.level
--            local buildCommandData = ModelResTable[level].need
--            local num = #buildCommandData

--            local allTime = ModelResTable[level].upgradeTime -- 总时间
--            allTime = BuildingDiamondCalculation:BuildConstructActualTime(allTime)

--            local item = BuildingsInfoMgr:getBuildQueue()[dataFromServer.id]
--            local remainTime = item.remainingSec -- 剩余时间

--            for i = 1,num,1 do
--                local resData = buildCommandData[i] -- {2,100}
--                local nowNum = UserInfo.city[Resource_temp[resData[1]].attributeName]
--                local maxNum = resData[2] 

--                local backRes = math.floor(maxNum*remainTime/allTime)
--                UserInfo.city[Resource_temp[resData[1]].attributeName] = nowNum + backRes
--            end

--            ResourceManager:luaDispatchEvent(ResourceManager.EVT_TYPE_RESOURCE_REFRESH)


            finish()
        else
            ErrorCode:popErrorCodeTip(state)
        end

    end)
end

--- 加速升级逻辑
function T.SpeedUpGradeLogic(dataFromServer)
    local item = BuildingsInfoMgr:getBuildQueue()[dataFromServer.id]
    local allDiamond = BuildingDiamondCalculation.FreeTimeGetSpeedFinishDiamond(item)
    local nowDiamond = UserInfo.gems

    if allDiamond <= nowDiamond then
        SpeedUpgradeCommand:new():exec(dataFromServer,function(state,diamond)
            if state == ReturnCode.SUCCESS then
                PlayerInstance:setDiamond(diamond)
                --MUIPop.SpeedUpGradeSuccess()

                local name = BuildingsInfoMgr:_getBuildName(dataFromServer.id)
                local descripe = Language:get("uc12",name,dataFromServer.level)
		        MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

                Command:buildQueueTimeFinish()

            else
                ErrorCode:popErrorCodeTip(state)
            end

        end)

    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

--- 加速升级逻辑免费
function T:SpeedUpGradeFreeLogic(dataFromServer)
    SpeedUpgradeCommand:new():exec(dataFromServer,function(state,diamond)
        if state == ReturnCode.SUCCESS then
            PlayerInstance:setDiamond(diamond)

            local name = BuildingsInfoMgr:_getBuildName(dataFromServer.id)
            local descripe = Language:get("uc12",name,dataFromServer.level)
		    MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

            Command:buildQueueTimeFinish()

        else
            ErrorCode:popErrorCodeTip(state)
        end

    end)
end

--- 建筑物建造无钻石，从0到1,军团是不会发到这里来的,false不使用钻石
function T.constructBuildingNoDiamond(dataFromServer,ModelResTable,finish)
    ConstructCommand:new():exec(dataFromServer.sid,dataFromServer.location,false,function(state)
        if state == ReturnCode.SUCCESS then
            BuildingCommand.SetListViwData(dataFromServer,ModelResTable,0)  -- 这里刷新内存数据
            if finish then
                finish()
            end
        else
            ErrorCode:popErrorCodeTip(state)
        end
    end)
end

--- 建筑物建造有钻石，从0到1,军团是不会发到这里来的,true:使用钻石
function T.constructBuildingHasDiamond(dataFromServer,ModelResTable,resTable,allDiamond,finish)
    local nowDiamond = UserInfo.gems

    if allDiamond <= nowDiamond then
        ConstructCommand:new():exec(dataFromServer.sid,dataFromServer.location,true,function(state)
            if state == ReturnCode.SUCCESS then
                BuildingCommand.SetListViwData(dataFromServer,ModelResTable,1,resTable)  -- 这里刷新内存数据
                if finish then
                    finish()
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end)
    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end


end

--- 购买建筑队列
function T:BuyBuildQueue(finish)
    BuyBuildingQueueCommand:new():exec(function(state)
        if state == ReturnCode.SUCCESS then
            if finish then
                finish()
            end
        else
            ErrorCode:popErrorCodeTip(state)
        end
    end)
end



----------------------------------------- 普通建筑物的建造页面升级相关 ------------------------------------------------------------


----------------------------------------- 科技研究的升级相关 ------------------------------------------------------------

--- 研究升级(无钻石)传入的是科技信息的sid
function T.ScienceUpGradeNoDiamond(sid,cityRes,finish)
    LevelUpTechnologyCommand:new():exec(sid,false,function(state)
            if state == ReturnCode.SUCCESS then
                -- 要扣除对应等级的消耗的资源数目
                local num = #Resource_temp
                for i = 1,num,1 do
                    local attributeName = Resource_temp[i].attributeName
                    if cityRes[i] then
                        UserInfo.city[attributeName] = cityRes[i]
                    end

                end
                ResourceManager:luaDispatchEvent(ResourceManager.EVT_TYPE_RESOURCE_REFRESH)
                finish()
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 研究升级(有钻石),传入的是科技信息的sid
function T.ScienceUpGradeHasDiamond(sid,finish,cityRes,allDiamond)
    local nowDiamond = UserInfo.gems

    if allDiamond <= nowDiamond then
        LevelUpTechnologyCommand:new():exec(sid,true,function(state)
            if state == ReturnCode.SUCCESS then
                -- 要扣除对应等级的消耗的资源数目
                local num = #Resource_temp
                for i = 1,num,1 do
                    local attributeName = Resource_temp[i].attributeName
                    if cityRes[i] then
                        UserInfo.city[attributeName] = cityRes[i]
                    end
                end
                ResourceManager:luaDispatchEvent(ResourceManager.EVT_TYPE_RESOURCE_REFRESH)
                finish()
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )

    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

--- 取消研究升级:传入的是队列的id
function T.ScienceCancleUpGradeLogic(queueItem,finish)
    CancelLevelUpTechnologyCommand.new():exec(queueItem,function(state)
            if state == ReturnCode.SUCCESS then
                --MUIPop.CancleUpGradeSuccess()
                finish()
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 加速研究升级:传入的是队列的id
function T.ScienceSpeedUpGradeLogic(queueItem,finish)
    ClearTechnologyTimeCammand.new():exec(queueItem,function(state)
            if state == ReturnCode.SUCCESS then
                --MUIPop.SpeedUpGradeSuccess()
                finish()
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

----------------------------------------- 科技研究的升级相关 ------------------------------------------------------------

----------------------------------------- 道具相关的命令 ------------------------------------------------------------
--- 使用道具
function T.UseProp(propSid,finish)
    local num = PropsInfoMgr:getPlayerPropByID(propSid)
    if num <= 0 then -- 道具数量不足，请购买道具
        MUIPop.PropNumNotEnough()
    else
        UsePropCommand:new():exec(propSid,function(state)
            if state == ReturnCode.SUCCESS then 
                local remainProp = num - 1
                PropsInfoMgr:setPlayerPropByID(propSid, remainProp)
                if finish then
                    finish()
                end
                local descripe = Language:get("uc35",Language:get(Prop_temp[propSid].prop))
                MUIAlter_singleLine.show( {info = descripe,showSecond = 1 } )

            else
                ErrorCode:popErrorCodeTip(state)
            end
                                    
        end)
    end

end

--- 购买并且使用道具
function T.BuyAndUseProp(propSid,needDiamond,finish)
    local nowDiamond = UserInfo.gems

    if needDiamond <= nowDiamond then
        UseAndBuyPropCommand:new():exec(propSid,function(state,diamond)
            if state == ReturnCode.SUCCESS then 
                PlayerInstance:setDiamond(diamond)
                if finish then
                    finish() 
                end              
                MUIPop.BuyAndUsePropSuccess(Language:get(Prop_temp[propSid].prop))
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end)

    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

--- 购买道具
function T.BuyProp(propSid,needDiamond,finish)
    local nowDiamond = UserInfo.gems

    if needDiamond <= nowDiamond then
        BuyPropCommand:new():exec(propSid,function(state)
            if state == ReturnCode.SUCCESS then 
                local num = PropsInfoMgr:getPlayerPropByID(propSid)
                PropsInfoMgr:addPlayerPropByID(propSid, 1)

                if finish then
                    finish()
                end
                MUIPop.BuyPropSuccess(Language:get(Prop_temp[propSid].prop))
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end)

    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

----------------------------------------- 道具相关的命令 ------------------------------------------------------------


----------------------------------------- 招募或者改造的命令 ------------------------------------------------------------
--- 普通生产或者改造
function T.NomalRecruitCommand(sid,bid,num,recruitType,finish,resTable,upCraftCosts,propCosts)
    if num == 0 then  -- 为0的时候资源不足，则不发送命令
        MUIPop.ResourceNotEnough()
        return
    end

    RecruitCommand:new():exec(sid,bid,num,recruitType,function(state)
            if state == ReturnCode.SUCCESS then    
                local length = #resTable
                for i = 1,length,1 do
                    local resData = resTable[i] 
                    local maxNum = resData[2] 
                    local allNum = num * maxNum
                    -- 服务器
                    local nowNum = UserInfo.city[Resource_temp[resData[1]].attributeName]  -- 玩家当前拥有数量
                    UserInfo.city[Resource_temp[resData[1]].attributeName] = nowNum - allNum
                end

                ResourceManager:luaDispatchEvent(ResourceManager.EVT_TYPE_RESOURCE_REFRESH)

                if upCraftCosts then
                    local nowNum = UserInfo.city.aircraftBox.infos[upCraftCosts[1]].num  -- 玩家当前拥有数量
                    local allNum = num * upCraftCosts[2]
                    UserInfo.city.aircraftBox.infos[upCraftCosts[1]].num = nowNum - allNum

                    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH) -- add by yuding
                end

                if propCosts then
                    local nowNum = PropsInfoMgr:getPlayerPropByID(propCosts[1])
                    local allNum = num * propCosts[2]
                    PropsInfoMgr:setPlayerPropByID(propCosts[1],nowNum - allNum)
                end

                if finish then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 加速改造或者生产
function T.SpeedNomalRecruitCommand(qid,needDiamond,finish)
    local nowDiamond = UserInfo.gems

    if needDiamond <= nowDiamond then
        SpeedRecruitCommand:new():exec(qid,function(state)
                if state == ReturnCode.SUCCESS then
                    --PlayerInstance:setDiamond(nowDiamond - needDiamond)
                    --MUIPop.SpeedNomalRecruitSuccess()

                    finish()
                else
                    ErrorCode:popErrorCodeTip(state)
                end
            end
        )
    else -- 弹出充值页面
        MUIPop.MUIToReCharge()
    end
end

--- 取消生产或者改造
function T.CancleNomalRecruitCommand(qid,finish)
    CancleRecruitCommand:new():exec(qid,function(state)
            if state == ReturnCode.SUCCESS then
                --MUIPop.CancleNomalRecruitSuccess()
                finish()
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 一键完成生产用（钻石）（改造暂时没有）
function T.GemsNomalRecruitCommand(sid,bid,num,needDiamond,finish,aircraftProduct)
    local nowDiamond = UserInfo.gems

    if needDiamond <= nowDiamond then
        GemsRecruitCommand:new():exec(sid,bid,num,function(state)
                if state == ReturnCode.SUCCESS then
                    -- 更新数量到内存中去
                    local nowNum = UserInfo.city.aircraftBox.infos[sid].num
                    UserInfo.city.aircraftBox.infos[sid].num = num + nowNum

                    AircraftManager:setAddAircraftNumBySid(sid,num)

                    AircraftManager:luaDispatchEvent(AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH) -- add by yuding
                    PlayerInstance:setPlayerFightLogic()
                    --MUIPop.GemsNomalRecruitSuccess()
                    finish()
                else
                    --MUIPop.GemsNomalRecruitFail(state)
                    ErrorCode:popErrorCodeTip(state)
                end
            end
        )
    else -- 弹出充值页面
        MUIAlter.show(
		{
			info = Language:get("uc15"),
			size = 30,
			flag = 3,
			confirmText = Language:get("ug103"),
			cancelText = Language:get("ug104"),
			callback = function(flag)
				if flag == 0 then
                    if aircraftProduct then
                        aircraftProduct.muialter2:hide()
                    end
					VipManager:enterIapWindow()
				end
			end
		}
	)
    end
end



----------------------------------------- 招募或者改造的命令 ------------------------------------------------------------

----------------------------------------- 邮件相关的命令 ------------------------------------------------------------
--- 查看邮件
function T:ReadMail(msgType,id,finish)
        ReadMailCommand:new():exec(msgType,id,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()  
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

----- 回放战斗录像
--function T:PlayBack(messageId,finish)
--        PlayBackCommand:new():exec(messageId,function(state)
--            if state == ReturnCode.SUCCESS then
--                finish()
--            else
--                MUIPop.PlayBackFail(state)
--            end
--        end
--    )
--end

--- 删除邮件
function T:DeleteMail(msgType,id,finish)
        DeleteMailCommand:new():exec(msgType,id,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()  
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 删除所有的邮件
function T:DeleteAllMail(msgType,finish)
        DeleteAllMailCommand:new():exec(msgType,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()  
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 批量删除邮件命令
function T:batchDeleteMail(msgType,ids,finish)
        BatchDeleteMailCommand:new():exec(msgType,ids,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 发送邮件
function T:SendMail(targetName,title,content,finish)
        SendMailCommand:new():exec(targetName,title,content,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()  
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 获得系统邮件奖励
function T:GetSysAward(messageId,finish)
        GetSysAwardCommand:new():exec(messageId,function(state,data)
            if state == ReturnCode.SUCCESS then
                if finish and data then
                    finish(data)
                end
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 查看它人录像
function T:ReadOtherReport(chatInfo,finish)
        ReadOtherReportCommand:new():exec(chatInfo,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    if finish then
                        finish()
                    end
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

----- 回放他人录像 
--function T:PlayBackOther(otherId,messageId,finish)
--        PlayBackOtherCommand:new():exec(otherId,messageId,function(state)
--            if state == ReturnCode.SUCCESS then
--                finish()
--            else
--                MUIPop.PlayBackOtherFail(state)
--            end
--        end
--    )
--end

--- 得到接下来的10封邮件
function T:GetTenMail(msgType,endId,finish)
        GetTenMailCommand:new():exec(msgType,endId,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 复仇
function T:RevengeLogic(x,y,targetId,finish)
        RevengeCommand:new():exec(x,y,targetId,function(state,tb)
            if state == ReturnCode.SUCCESS then
                if finish and tb then
                    finish(tb)
                end
            elseif state == 2 then  -- 如果对方已经不在原地，返回的状态码是2，弹出提示:目标已不在原地
                MUIPop.TargetNotAuto()
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 发送联盟邮件
function T:SendAllianceMail(title,content,finish)
        SendAllianceMailCommand():exec(title,content,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
                
            else
                
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

----------------------------------------- 邮件相关的命令 ------------------------------------------------------------

----------------------------------------- 聊天相关的命令 ------------------------------------------------------------
--- 发送世界聊天命令
function T:SendWorldChat(msg,finish)
        WorldChatCommand:new():exec(msg,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
            else
--                if state == 6 then
--                    MUIPop.Gag()
--                end
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 发送联盟聊天命令
function T:SendGroupChat(msg,finish)
        GroupChatCommand:new():exec(msg,function(state)
            if state == ReturnCode.SUCCESS then
                finish()
            else
--                if state == 6 then
--                    MUIPop.Gag()
--                end
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 发送私人聊天命令
function T:SendPrivateChat(msg,targetName,finish,confirm)
        PrivateChatCommand:new():exec(msg,targetName,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
                
            else
--                if state == 29 then -- 用户没上线，不能私聊
--                    MUIPop.UserNotExist(confirm)
--                end
--                if state == 6 then
--                    MUIPop.Gag()
--                end
                -- 输入的用户名不正确，请重新输入
                --MUIPop.InputUserNameAgain()
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 分享战报
-- @param channelType 
-- @param battleReportId 战报ID
-- @param msgType 信息类型(2表示普通战报,4表示竞技场战报)
function T:ShareReport(channelType,battleReportId,msgType,finish)
    ShareReportCommand:new():exec(channelType,battleReportId,msgType,function(state)
            if state == ReturnCode.SUCCESS then

                if finish then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end


----------------------------------------- 聊天相关的命令 ------------------------------------------------------------


----------------------------------------- 聊天相关的命令 ------------------------------------------------------------

--- 查看角色信息
function T:CheckChatInfo(playerId,finish)
    CheckChatInfoCommand:new():exec(playerId,function(state,dataTable)
            if state == ReturnCode.SUCCESS then
                if finish and dataTable then
                    finish(dataTable)
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end

--- 添加黑名单
function T:AddBlackList(targetName,targetId,finish)
    AddBlackListCommand:new():exec(targetName,targetId,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end
--- 添加好友
function T:AddFriend(targetName,targetId,finish)
    AddFriendCommand:new():exec(targetName,targetId,function(state)
            if state == ReturnCode.SUCCESS then
                if finish then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end
--- 删除黑名单
function T:DeleteBlackList(targetId,finish)
    DeleteBlackListCommand:new():exec(targetId,function(state)
            if state == ReturnCode.SUCCESS then
                if finish  then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end
--- 删除好友
function T:DeleteFriend(targetId,finish)
    DeleteFriendCommand:new():exec(targetId,function(state)
            if state == ReturnCode.SUCCESS then
                if finish  then
                    finish()
                end
                
            else
                ErrorCode:popErrorCodeTip(state)
            end
        end
    )
end


----------------------------------------- 聊天相关的命令 ------------------------------------------------------------


return T