--[[
 *	建筑物中钻石的计算
]]
local T = { }

--[[
 *	@param dataFromServer：建筑物所对应的数据
 *	@param modelResTable：建筑物所对应的资源表
]]

--- 那些建筑物建造加上科研中心研究 立刻完成所需钻石，时间要减去免费时间
function T.FreeTimeGetImmediateFinishDiamond(dataFromServer, modelResTable)
    local level = dataFromServer.level
    local buildCommandData = modelResTable[level].need
    local num = #buildCommandData

    local allDiamond = 0
    local diamondPerMinute = CommonConfigs_temp.minute2Diamond
    -- 每分钟钻石数目

    local upgradeTime = modelResTable[level].upgradeTime

    local techAdd = TechnologiesInfoMgr:getBuildingSpeedAdd()/100
    upgradeTime = upgradeTime/(1 + techAdd)

    if ActivityManager.rapidDevelopmentInfo:getOpenValue() == true then
        local add = ActivityManager.rapidDevelopmentInfo:getReducePercent()
        upgradeTime = upgradeTime * add
    end

    if upgradeTime >= 0 and upgradeTime <= 1 then
        upgradeTime = 1
    end


    local freeTime = BuildingDiamondCalculation.GetVipFreeTime()
    local nowTime = upgradeTime - freeTime
    if nowTime > 0 then
        local minute = nowTime / 60
        -- 分钟数目
        allDiamond = minute * diamondPerMinute
    end

    for i = 1, num, 1 do
        local resData = buildCommandData[i]
        -- {2,100}
        local maxNum = resData[2]
        -- 等级资源需求数量
        local resPerDiamond = Resource_temp[resData[1]].ResPerDiamond
        -- 每钻石资源数目基数
        allDiamond = allDiamond +(maxNum * CommonConfigs_temp.molecular) / resPerDiamond
    end

    if allDiamond > 0 and allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
    end

    return allDiamond
end

--- 立即完成
-- @param remainTime  剩余的时间
-- @param vipFree 是否 减去vip的特权 (暂时不用)
function T.TimeSpeedFinishDiamond(remainTime, vipFree)
    local allDiamond = 0
    local diamondPerMinute = CommonConfigs_temp.minute2Diamond
    -- 每分钟钻石数目
    local minute = math.floor(remainTime / 60)
    -- 剩余分钟数目
    allDiamond = minute * diamondPerMinute

    if allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
    end

    return allDiamond
end


--- 那些建筑物建造加上科研中心研究 加速所需钻石，时间要减去免费时间
function T.FreeTimeGetSpeedFinishDiamond(item)
    local allDiamond = 0
    local diamondPerMinute = CommonConfigs_temp.minute2Diamond
    -- 每分钟钻石数目
    local remainTime = item.remainingSec
    -- 剩余时间
    local freeTime = BuildingDiamondCalculation.GetVipFreeTime()
    local nowTime = remainTime - freeTime
    if nowTime > 0 then
        local minute = math.ceil(nowTime / 60)
        -- 剩余分钟数目
        allDiamond = minute * diamondPerMinute
    end

    return allDiamond
end


--- 得到vip免费的时间
function T.GetVipFreeTime()
    local vipFreeTime = VipManager.getVipFreeTime()

    return vipFreeTime
end

--- 立刻完成所需要的钻石,我这里的每种资源的每钻石资源数目不一样
function T.GetImmediateFinishDiamond(dataFromServer, modelResTable)
    local level = dataFromServer.level
    local buildCommandData = modelResTable[level].need
    local num = #buildCommandData

    local allDiamond = 0
    local diamondPerMinute = CommonConfigs_temp.minute2Diamond
    -- 每分钟钻石数目
    local minute = modelResTable[level].upgradeTime / 60
    -- 分钟数目
    allDiamond = minute * diamondPerMinute
    for i = 1, num, 1 do
        local resData = buildCommandData[i]
        -- {2,100}
        local maxNum = resData[2]
        -- 等级资源需求数量
        local resPerDiamond = Resource_temp[resData[1]].ResPerDiamond
        -- 每钻石资源数目基数
        allDiamond = allDiamond +(maxNum * CommonConfigs_temp.molecular) / resPerDiamond
    end

    if allDiamond > 0 and allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
    end

    return allDiamond
end

--- 通过这个方法得到的结果判断资源是否足够，升级资源不足时，返回的资源表和所需要的钻石
function T.JudgeResIsEnough(dataFromServer, modelResTable)
    -- 判断资源是否足够升级
    local level = dataFromServer.level
    local buildCommandData = modelResTable[level].need
    local num = #buildCommandData

    local allDiamond = 0
    local resTable = { }
    -- 更新资源时用的表

    for i = 1, num, 1 do
        local resData = buildCommandData[i]
        -- {2,100}
        -- 服务器
        local nowNum = UserInfo.city[Resource_temp[resData[1]].attributeName]
        -- 玩家当前拥有数量
        -- 本地
        local maxNum = resData[2]
        -- 等级资源需求数量
        if nowNum < maxNum then
            local tempNum = maxNum - nowNum
            resTable[i] = Resource_temp[resData[1]].attributeName
            -- 存储的是遍历id，还有资源的属性名字
            local resPerDiamond = Resource_temp[resData[1]].ResPerDiamond
            allDiamond = allDiamond +(tempNum * CommonConfigs_temp.molecular) / resPerDiamond
        end
    end

    if allDiamond > 0 and allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
    end

    return resTable, allDiamond
end

--- 加速升级所需要的钻石
function T.GetSpeedFinishDiamond(item)
    local allDiamond = 0
    local diamondPerMinute = CommonConfigs_temp.minute2Diamond
    -- 每分钟钻石数目
    local remainTime = item.remainingSec
    -- 剩余时间
    local minute = math.floor(remainTime / 60)
    -- 剩余分钟数目
    allDiamond = minute * diamondPerMinute

    if allDiamond >= 0 and allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
    end

    return allDiamond
end

--- 判断科技研究的资源是否足够，升级资源不足时，返回的所需要的钻石;sid为研究项的模板id
function T.JudgeResIsEnoughInScience(sid)
    local level = nil
    if TechnologiesInfoMgr:getInfos()[sid] then
        level = TechnologiesInfoMgr:getInfos()[sid].level
    else
        level = 0
    end

    local data = Science_research_temp[sid]
    local num = #Resource_temp

    local allDiamond = 0
    local cityRes = { }
    -- 存储的是当前资源的数值,这里如果不存在这种资源的话是nil

    for i = 1, num, 1 do
        local attributeName = Resource_temp[i].attributeName
        local tp=data[attributeName]
        if tp then
            -- 因为每种科技所需要的资源的种类是不一样的,所以要判断是否存在
            -- 等级资源需求数量
            local maxNum
            if level < #tp then
                maxNum = tp[level + 1]
            else
                maxNum = tp[#tp]
            end
            local nowNum = UserInfo.city[attributeName]
            -- 玩家当前拥有数量

            if nowNum < maxNum then
                local tempNum = maxNum - nowNum
                local resPerDiamond = Resource_temp[i].ResPerDiamond
                allDiamond = allDiamond +(tempNum * CommonConfigs_temp.molecular) / resPerDiamond
                cityRes[i] = 0
            else
                cityRes[i] = nowNum - maxNum
            end
        end
    end

    if allDiamond > 0 and allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
        -- 向下取整
    end

    return cityRes, allDiamond
end

--- 得到一键完成招募所需要的钻石数目,根据传入的战机的sid，数量num,返回所需要的钻石
function T.GemsNomalRecruitLogic(allNum, sid, location)
    local allDiamond = 0

    local data = Aircraft_temp[sid]

    local upgradeTime = nil
    if location == 5 then
        upgradeTime = BuildingDiamondCalculation:AircraftProActualTime(location, data.time)
    else
        upgradeTime = BuildingDiamondCalculation:AircraftProActualTime(location, data.upgradeTime)
        -- 要得到真正的时间
    end


    local allTime = upgradeTime * allNum
    local diamondPerMinute = CommonConfigs_temp.minute2Diamond
    -- 每分钟钻石数目
    local minute = allTime / 60
    -- 分钟数目
    allDiamond = minute * diamondPerMinute

    local upResourceCosts = data.upResourceCosts
    local num = #upResourceCosts
    for i = 1, num, 1 do
        local resData = upResourceCosts[i]
        -- {2,100}
        local maxNum = resData[2] * allNum
        -- 等级资源需求数量(要乘以数量)
        local resPerDiamond = Resource_temp[resData[1]].ResPerDiamond
        -- 每钻石资源数目基数
        allDiamond = allDiamond +(maxNum * CommonConfigs_temp.molecular) / resPerDiamond
    end

    if allDiamond > 0 and allDiamond < 1 then
        -- 0.5 取1
        allDiamond = 1
    else
        allDiamond = math.floor(allDiamond)
    end

    return allDiamond

end

--- 科技科研实际时间
function T:ScienceActualTime(allTime)
    local time = allTime
    local tech = 0
    if BuildingsInfoMgr:getBuildingInfos()[4] then
        local techInfo = BuildingsInfoMgr:getBuildingInfos()[4]
        tech = 0.05 * techInfo.level
    end

    -- vip特权
    local techSpeed = VipManager.getVipTechSpeed()
    time = allTime /(1 + tech + techSpeed)

    -- 活动
    if ActivityManager.rapidDevelopmentInfo:getOpenValue() == true then
        local activityAdd = ActivityManager.rapidDevelopmentInfo:getReducePercent()
        time = time * activityAdd
    end


    return time

end


--- 科技科研实际时间，无活动
function T:ScienceActualTimeNoActivity(allTime)
    local time = allTime
    local tech = 0
    if BuildingsInfoMgr:getBuildingInfos()[4] then
        local techInfo = BuildingsInfoMgr:getBuildingInfos()[4]
        tech = 0.05 * techInfo.level
    end

    -- vip特权
    local techSpeed = VipManager.getVipTechSpeed()
    time = allTime /(1 + tech + techSpeed)

    return time

end


--- 飞机或者改造该工厂 生产实际时间
function T:AircraftProActualTime(location, allTime)
    local time = allTime
    local air = 0
    if BuildingsInfoMgr:getBuildingInfos()[location] then
        local airInfo = BuildingsInfoMgr:getBuildingInfos()[location]
        air = 0.05 * airInfo.level
    end

    -- vip
    local speed = 0
    if location == 5 then
        -- 改造工厂
        speed = VipManager.getVipReconstructSpeed()
    else
        -- 机库
        speed = VipManager.getVipProduceSpeed()
    end

    time = allTime /(1 + air + speed)

    -- 活动
    if ActivityManager.rapidDevelopmentInfo:getOpenValue() == true then
        local activityAdd = ActivityManager.rapidDevelopmentInfo:getReducePercent()
        time = time * activityAdd
    end

    return time
end

--- 飞机或者改造该工厂 生产实际时间(没有活动)
function T:AircraftProActualTimeNoAcitvity(location, allTime)
    local time = allTime
    local air = 0
    if BuildingsInfoMgr:getBuildingInfos()[location] then
        local airInfo = BuildingsInfoMgr:getBuildingInfos()[location]
        air = 0.05 * airInfo.level
    end

    -- vip
    local speed = 0
    if location == 5 then
        -- 改造工厂
        speed = VipManager.getVipReconstructSpeed()
    else
        -- 机库
        speed = VipManager.getVipProduceSpeed()
    end

    time = allTime /(1 + air + speed)

    return time
end


--- 建筑建造实际时间
function T:BuildConstructActualTime(allTime)
    local time = allTime

    local techAdd = TechnologiesInfoMgr:getBuildingSpeedAdd()/100

    time = allTime /(1 + techAdd)

    if ActivityManager.rapidDevelopmentInfo:getOpenValue() == true then
        local add = ActivityManager.rapidDevelopmentInfo:getReducePercent()
        time = time * add
    end

    if time >= 0 and time <= 1 then
        time = 1
    end


    return time
end

--- 没有活动的时间建筑建造实际时间
function T:BuildConstructActualTimeNoActivity(allTime)
    local time = allTime

    local techAdd = TechnologiesInfoMgr:getBuildingSpeedAdd()/100

    time = allTime /(1 + techAdd)

    if time >= 0 and time <= 1 then
        time = 1
    end


    return time
end

--- 修复单个飞机部队的钻石
function T:repairSingleAirGems(sid)
    local repairAircrafts = UserInfo.city.aircraftBox:getRepair()
    if repairAircrafts[sid] and repairAircrafts[sid].num > 0 then
        local cost = Aircraft_temp[sid].repairGems * repairAircrafts[sid].num
        return self:getRealCost(cost)
    else
        return 0
    end
end

--- 修复单个飞机部队的钻石:先不取
function T:repairSingleAirGems2(sid)
    local repairAircrafts = UserInfo.city.aircraftBox:getRepair()
    if repairAircrafts[sid] and repairAircrafts[sid].num > 0 then
        local cost = Aircraft_temp[sid].repairGems * repairAircrafts[sid].num
        return cost
    else
        return 0
    end
end

--- 修复单个飞机部队的金币
function T:repairSingleAirMoney(sid)
    local repairAircrafts = UserInfo.city.aircraftBox:getRepair()
    if repairAircrafts[sid] and repairAircrafts[sid].num > 0 then
        local cost = Aircraft_temp[sid].repairMoney * repairAircrafts[sid].num
        return self:getRealCost(cost)
    else
        return 0
    end
end

--- 修复单个飞机部队的金币
function T:repairSingleAirMoney2(sid)
    local repairAircrafts = UserInfo.city.aircraftBox:getRepair()
    if repairAircrafts[sid] and repairAircrafts[sid].num > 0 then
        local cost = Aircraft_temp[sid].repairMoney * repairAircrafts[sid].num
        return cost
    else
        return 0
    end
end



--- 修复所有飞机部队的钻石
function T:repairAllAirGems()
    local repairAircrafts = UserInfo.city.aircraftBox:getRepair()
    local cost = 0
    for k, v in pairs(repairAircrafts) do
        if v then
            cost = cost + self:repairSingleAirGems2(v.sid)
        end
    end
    return self:getRealCost(cost)
end


--- 修复所有飞机部队的金币
function T:repairAllAirMoney()
    local repairAircrafts = UserInfo.city.aircraftBox:getRepair()
    local cost = 0
    for k, v in pairs(repairAircrafts) do
        if v then
            cost = cost + self:repairSingleAirMoney(v.sid)
        end
    end
    return self:getRealCost(cost)

end


--- 得到修理的实际要的花费
function T:getRealCost(cost)
    if cost then
        if cost < 1 and cost > 0 then
            return 1
        else
            return math.floor(cost)
        end
    else
        return 0
    end

end



return T