--[[
 *	列表中的项
]]
local T = class("T",function() return ccui.Layout:create() end)

--- 标题位置:左边
T.titlePos = nil

--- 中间部分的位置:水平，横向,有可能有进度条
T.centerHPos = nil

--- 中间部分的位置:垂直，纵向,有可能有进度条
T.centerVPos = nil

--- 右边只有1个按钮的位置:
T.rightOnePos = nil

--- 右边只有2个按钮的位置:从右向左开始
T.rightTwoPos = nil

--- lable的tag基础值
T.baseValue = 5000

--- 构造函数
function T:ctor()
    self:init()
end

function T:init()
    -- 背景图片
    local back = cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("frameBack_middle.png"))
    self:addChild(back)

    local size = back:getContentSize()
    self:setContentSize(size)

    REL:toOther("center","middle",back,self,0,0)
    self.back = back

    -- 头像
    local leftHead = ccui.ImageView:create()
    local iconPath = IconManager.iconPath
    leftHead:loadTexture(iconPath .. Prop_temp[1].propIcon) -- 随便找的一个头像，但是要存在
    self:addChild(leftHead)
    REL:toOther("left","top",leftHead,back,4,13)
    self.leftHead = leftHead

    -- 标题:左边
    local title = XMLLabel()
    title:setContentSize(cc.size(600, 36))
    self:addChild(title)
    self.title = title

    self.titlePos = {110,4}     -- 相对位置为左顶
    self.centerHPos = {{130,60},{325,60}}   -- 相对位置为离左边和顶部
    self.centerVPos = {{120,45},{150,85}}     -- 相对位置为离左边和顶部
    self.rightOnePos = {15,8}     -- 相对位置为离右边和底部
    -- 有2种
    self.rightTwoPos = {{15 + 80,8},{15,8}}     -- 相对位置为离右边和底部......
    self.rightTwoPos2 = {{385,8},{385 + 74,-3}}     -- 相对位置为离左边和底部......
end

--- 添加进度条:位置
function T:addLoadingBar(centerPos)
    -- 背景图片
    local backImg = ccui.ImageView:create()
    backImg:loadTexture("productProgressBack.png",ccui.TextureResType.plistType)
    self:addChild(backImg)
    REL:toOther("left","top",backImg,self.back,centerPos[1],centerPos[2])
    self.backImg = backImg
    -- 进度条图片
    local uiProgressFore = cc.ProgressTimer:create(cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("productProgressFore.png")))
	uiProgressFore:setBarChangeRate(cc.p(1,0))--进度条动画方向，（1,0）表示横方向，（0,1）表示纵方向
	uiProgressFore:setType(cc.PROGRESS_TIMER_TYPE_BAR)--进度条类型
	uiProgressFore:setMidpoint(cc.p(0,1))--进度条的起始点，（0,y）表示最左边，（1,,y）表示最右边，（x,1）表示最上面，（x,0）表示最下面。
	uiProgressFore:setPercentage(0)--初始值
	backImg:addChild(uiProgressFore)
    REL:toOther("center","middle",uiProgressFore,backImg,0,0)
    self.loadingBar = uiProgressFore
end

--[[
    设置项的属性
    centerPosDir:V:纵向，H为横向,中间几个控件的位置方向
    centerTable:{"label"}label为文本
    centerTable:{"progress"}progress为进度条
    rightTable:现在只考虑2种情况
    {
        {normalState="",downState="",isHasPic="true",isHasText="true"} -- 最后2个表示，按钮是否有图片或者文字
        {normalState="",downState="",isHasPic="false",isHasText="false"},没有的话传入false
    }
]]
function T:setListItemAttribute(centerPosDir,centerTable,rightTable)
    -- 标题
    REL:toOther("left","top",self.title,self.back,self.titlePos[1],self.titlePos[2])

    -- 中间
    if centerPosDir and centerTable then
        local num = #centerTable
        local centerPos = nil
        if centerPosDir == "H" then
            centerPos = self.centerHPos
        else
            centerPos = self.centerVPos
        end

        for i = 1,num,1 do  -- 这个i可以认为是对应位置上的lable
            if centerTable[i] == "label" then -- 文本
                local label = XMLLabel()
                label:setContentSize(cc.size(250, 36))  -- 要考虑换行的
                label:setTag(self.baseValue + i)
                self:addChild(label)
                REL:toOther("left", "top", label, self.back, centerPos[i][1], centerPos[i][2])
            else    -- 进度条
                self:addLoadingBar(centerPos[i])
            end
        end

    end

    -- 右边
    if rightTable then
        local num = #rightTable
        if num == 1 then
            local button1 = Button.new(rightTable[1][3],rightTable[1][4])
            button1:loadTextures(rightTable[1][1],rightTable[1][2] , "",ccui.TextureResType.plistType)
            self:addChild(button1)
            REL:toOther("right", "bottom", button1, self.back, self.rightOnePos[1], self.rightOnePos[2])
            self.button1 = button1
        elseif num == 2 then
            local button1 = Button.new(rightTable[1][3],rightTable[1][4]) -- 左1
            button1:loadTextures(rightTable[1][1],rightTable[1][2] , "",ccui.TextureResType.plistType)
            self:addChild(button1)
            REL:toOther("right", "bottom", button1, self.back, self.rightTwoPos[1][1], self.rightTwoPos[1][2])
            self.button1 = button1

            local button2 = Button.new(rightTable[2][3],rightTable[2][4]) -- 左2
            button2:loadTextures(rightTable[2][1],rightTable[2][2] , "",ccui.TextureResType.plistType)
            self:addChild(button2)
            REL:toOther("right", "bottom", button2, self.back, self.rightTwoPos[2][1], self.rightTwoPos[2][2])
            self.button2 = button2
        end
    end
end

--- 根据位置得到对应的label控件，从1开始的
function T:getLabel(location)
    if self:getChildByTag(self.baseValue + location) then
        local label = self:getChildByTag(self.baseValue + location)
        return label
    end
end

--- 设置文本字符串
function T:setString(xmlLabelName,text)
    if xmlLabelName == self.title then
        xmlLabelName:setString(Language:get("uc276",text))
    else
        xmlLabelName:setString(tostring(text))
    end
end

--- 针对120的icon设置为100*100,单独加载
function T:iconLoadTexture(path)
    self.leftHead:loadTexture(path)
    self.leftHead:setScale(IconManager.buildScale[1],IconManager.buildScale[1])
end

return T
