--- 窗口特效
local T={}

--- 上下进来时间
T.inTopAndBottomTime = 0.2
--- 内容进来时间（缩放和渐变）
T.inContainerTime = 0.2
--- 背景进来时间
T.inBackTime = 0.2

--- 上下退出时间
T.OutTopAndBottomTime = 0.2
--- 内容退出时间（缩放和渐变）
T.OutContainerTime = 0.2
--- 背景退出时间
T.OutBackTime = 0.2

--- 进入特效
function T:playOpenEffect()
	self:luaDispatchEvent(EVENT_START_OPEN,self)

	local c = 0
	local function complete()
		c = c + 1
		-- 如果有几个特效模块，那么这里的数字就是几
		if c >= 4 then
			self:playOpenEffectComplete()
		end
	end

	-- 顶部
	local uiT = self.uiTopBorder
	if uiT then
		local endX, endY = uiT:getPosition()

		uiT:setPosition(endX, endY + (uiT.getContentSize and uiT:getContentSize().height or uiT:getContentSize().height))
		uiT:runAction(
			cc.Sequence:create(
				cc.MoveTo:create(self.inTopAndBottomTime, cc.p(endX, endY)),
				cc.CallFunc:create( function(target)
					complete()
				end )
			)
		)
	else
		complete()
	end

	-- 底部
	local uiB = self.uiBottomBorder
	if uiB then
		local endX, endY = uiB:getPosition()

		uiB:setPosition(endX, endY - (uiB.getContentSize and uiB:getContentSize().height or uiB:getContentSize().height))
		uiB:runAction(
			cc.Sequence:create(
				cc.MoveTo:create(self.inTopAndBottomTime, cc.p(endX, endY)),
				cc.CallFunc:create( function(target)
					complete()
				end )
			)
		)
	else
		complete()
	end

	-- 中间
	local uiC = self.uiContainer

	if uiC then
		uiC:setAnchorPoint(cc.p(0.5, 0.5))
		uiC:setOpacity(0)

		local x, y = uiC:getPosition()
		local size = uiC:getContentSize()
		uiC:setPosition(x + size.width / 2, y + size.height / 2)

		local endScale = uiC:getScale()

		uiC:setScale(0.5)
		uiC:runAction(
			cc.Spawn:create(
				cc.ScaleTo:create(self.inContainerTime, endScale),
				cc.FadeIn:create(self.inContainerTime),
				cc.CallFunc:create( function(target)
					complete()
				end )
			)

		)
	else
		complete()
	end

	-- 背景
	local uiB = self.uiBack
	if uiB then
		uiB:setAnchorPoint(cc.p(0.5, 0.5))
		uiB:setOpacity(0)
		uiB:runAction(cc.Sequence:create(
			cc.Spawn:create(
				cc.FadeIn:create(self.inBackTime)
			),
			cc.CallFunc:create( function(target)
				complete()
			end )
		))
	else
		complete()
	end

end

--- 退出特效
function T:playColseEffect()
	self:luaDispatchEvent(EVENT_START_CLOSE,self)

	local c = 0
	local function complete()
		c = c + 1
		-- 如果有几个特效模块，那么这里的数字就是几
		if c >= 4 then
			self:playCloseEffectComplete()
		end
	end

	-- 顶部
	local uiT = self.uiTopBorder
	if uiT then
		local x, y = uiT:getPosition()

		uiT:runAction(
			cc.Sequence:create(
				cc.MoveTo:create(self.OutTopAndBottomTime, cc.p(x, y + (uiT.getContentSize and uiT:getContentSize().height or uiT:getContentSize().height))),
				cc.CallFunc:create( function(target)
					-- 还原
					target:setPosition(x, y)

					complete()
				end )
			)
		)
	else
		complete()
	end

	-- 底部
	local uiB = self.uiBottomBorder
	if uiB then
		local x, y = uiB:getPosition()

		uiB:runAction(cc.Sequence:create(
			cc.MoveTo:create(self.OutTopAndBottomTime, cc.p(x, y - (uiB.getContentSize and uiB:getContentSize().height or uiB:getContentSize().height))),
			cc.CallFunc:create( function(target)
				-- 还原
				target:setPosition(x, y)

				complete()
			end )
		))
	else
		complete()
	end

	-- 中间
	local uiC = self.uiContainer
	if uiC then
		uiC:setAnchorPoint(cc.p(0.5, 0.5))
		local scale = uiC:getScale()

		uiC:runAction(cc.Sequence:create(
			cc.Spawn:create(
				cc.ScaleTo:create(self.OutContainerTime, 0.5),
				cc.FadeOut:create(self.OutContainerTime)
			),
			cc.CallFunc:create( function(target)
				-- 还原
				target:setScale(scale)

				complete()
			end )
		))
	else
		complete()
	end

	-- 背景
	local uiB = self.uiBack
	if uiB then
		uiB:setAnchorPoint(cc.p(0.5, 0.5))
		uiB:runAction(cc.Sequence:create(
			cc.Spawn:create(
				cc.FadeOut:create(self.OutBackTime)
			),
			cc.CallFunc:create( function(target)
				complete()
			end )
		))
	else
		complete()
	end

end

return T