--region TaskWindow.lua
--Author    : YuDing
--Date      : 2015/1/10
--Introduce : 任务界面

local T = class(
	"T",
	function()
		local o = AbstractWindow.new()
		o.super = AbstractWindow
		return o
	end
)

--- 三级页面
T.switchButtonContainer = nil
--- 配置文件
T.cfg = nil
--- 主线任务节点
T.mainTask = nil
--- 日常任务节点
T.dailyTask = nil
--- 成就节点
T.achieve = nil
--- 上一次点击的节点
T.lastClick = nil


function T:ctor(tag)
	mapping(self, LuaEventDispatcher)
	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:initCfg()               -- 初始化配置文件
	self:initThreeContainer(tag)    -- 初始化三级页面
end

function T:onEnter()
	self.super.onEnter(self)

	self:registerPublicEvent()
end

function T:onExit()
	self.super.onExit(self)
end

function T:cleanUp()

	self:releasePageNode()

	self:releasePublicEvent()
	self:removeFlushTask()
end
--- 初始化配置文件
function T:initCfg()
	self.cfg = require("src.app.uiScript.components.task.TaskWindowCfg")
end

--- 创建UI
function T:initThreeContainer(tag)
	-- 创建3个

	local callBack1= function()
		self:switchMainTaskPage()
	end

	local callBack2 = function()
		self:switchDailyTaskPage()
	end

	local callBack3 = function()
		self:switchAchievePage()
	end

	--- 初始化分页
	if not tag then tag = 1 end
	self.menuTag = tag

	local switchButtonContainer = SwitchButtonContainer.new()
	self:addChild(switchButtonContainer)
	REL:relOther(switchButtonContainer,self.uiContainer,0,0,0,0)
	switchButtonContainer:init()
	switchButtonContainer:initButtons(
		2,
		{{"viewSwitchButton_normal.png","viewSwitchButton_down.png"},{"viewSwitchButton_normal.png","viewSwitchButton_down.png"}},
		{110,15},
		{callBack1, callBack2},
		{Language:get("uy52"), Language:get("uy53")},
		self.menuTag
	)
	self.switchButtonContainer = switchButtonContainer

	if self.menuTag == 1 then
		self:switchMainTaskPage()
	end

	-- 标题
	self:setWindowTitle(Language:get("uy115"))

	--self:updateDailyTip() -- 更新标记

	self:flushTask()

	-- 调用一次，透明度才没问题
	self:initCompleteHandler()
end

--- 初始化 日常任务 提示
function T:updateDailyTip()
	local normal = TaskManager:getDailyNormalTip()
	local alliance = TaskManager:getDailyAllianceTip()

	if normal == true or alliance == true then -- 有任务
		self:createDailyTip()
	else
		self:removeDailyTip()
	end
end

--- 创建 日常 提示
function T:createDailyTip()
	if self.dailyTip then
		return
	end

	local tag = HomePageManager:getEff("promptEffect")
	self:addChild(tag)
	REL:toOther("right", "top", tag, self.switchButtonContainer:getButton(2), 20, 15)

	if self.switchButtonContainer:getClickTag() == 2 then  -- 正好在第二个按钮状态 提示是不出现的
	--tag:setVisible(false)
	end

	self.dailyTip = tag
end

--- 移除 日常提示
function T:removeDailyTip()
	if self.dailyTip then
		self.dailyTip:removeFromParent()
		self.dailyTip = nil
	end
end

--- 设置标记可见
function T:setDailyVisible(v)
	if self.dailyTip then
	--self.dailyTip:setVisible(v)
	end
end



function T:registerPublicEvent()
--TaskManager:luaAddEventListener(TaskManager.noticeMessage.dailyTaskFresh, self, self.updateDailyTip)
end

function T:releasePublicEvent()
--TaskManager:luaAddEventListener(TaskManager.noticeMessage.dailyTaskFresh, self, self.updateDailyTip)
end


--- 切换到任务界面
function T:switchMainTaskPage()
	if not self.mainTask then
		self.mainTask = require("src.app.uiScript.components.task.taskMain.TaskMainNode").new()
		self:addChild(self.mainTask)
		REL:toOther("left", "top", self.mainTask, self.switchButtonContainer, 10, 90)
	end

	if self.lastClick == self.mainTask then
		return
	else
		if self.lastClick then -- 如果上一次点击的存在
			self.lastClick:setVisible(false)
		end
		self.lastClick = self.mainTask
		self.mainTask:setVisible(true)
		self:setDailyVisible(true)       -- 日常标记可见
	end
end


--- 切换到日常任务界面
function T:switchDailyTaskPage()
	if not self.dailyTask then
		self.dailyTask = require("src.app.uiScript.components.task.TaskDailyNode").new()
		self:addChild(self.dailyTask)
		REL:toScreen("center", "middle", self.dailyTask, 0, 0)
	end
	if self.lastClick == self.dailyTask then
		return
	else
		if self.lastClick then
			self.lastClick:setVisible(false)
		end
		self.lastClick = self.dailyTask
		self.dailyTask:setVisible(true)
		self:setDailyVisible(false)     -- 日常标记不可见
	end
end

--- 获得任务 额外提示 完成的按钮
function T:getTaskExtraFinishBtn()
	return self.mainTask:getExtraFinishBtnObject()
end

--- 切换到成就界面
function T:switchAchievePage()
	if not self.achieve then
		-- self.achieve = require("src.app.uiScript.components.task.AchieveNode").new()
		self.achieve = ccui.Layout:create()

		self:addChild(self.achieve)
		--self.achieve:retain()
	end

	if self.lastClick == self.achieve then
		return
	else
		if self.lastClick then
			--self.lastClick:removeFromParent()
			self.lastClick:setVisible(false)
		end
		self.lastClick = self.achieve
		self.achieve:setVisible(true)
	end

end
-- 移除节点
function T:releasePageNode()
--    if self.mainTask then self.mainTask:release() end
--    if self.dailyTask then self.dailyTask:release() end
--    if self.achieve then self.achieve:release() end
end

function T:putWindowToManager()
--TaskManager
end


--- 因为时间的缘故
function T:flushTask()
	if self.taskSchduleId then
		return
	end

	local func = function()
		self:updateDailyTip()
	end

	self.taskSchduleId = Cocos2dxUtils:interval(func, 0.5)
end

function T:removeFlushTask()
	if self.taskSchduleId then
		Cocos2dxUtils:clearScheduler(self.taskSchduleId)
		self.taskSchduleId = nil
	end
end



return T



--endregion
