--Author    : YuDing
--Date      : 2015/1/10
--Introduce : 任务条

local T = class("T", function() return ccui.Layout:create() end)

function T:ctor(taskSid, finish, kind, extra, delay)
	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self.taskSid = taskSid
	self.kind = kind
	self.extra = extra
	self.finish = finish

	if not self.extra then -- 普通的
		self:setContentSize(cc.size(619, 129))
	else -- 特别的
		self:setContentSize(cc.size(620, 185))
	end

	self:delayShow(delay)
end

--- 延迟显示
function T:delayShow(d)
	if d == true then  -- 显示
		if not self.widget then -- 确保第二次进来 不会重复加载
			self:loadJson()
			self:init()
	end
	end
end

--- 得到显示效果的主体
function T:getEffRender()
	local tb = {}
    self.uiChild = self.uiChild or {}

	for k, v in pairs(self.uiChild) do
		if v:getDescription() == "Button" or v:getDescription() == "Label" or v:getDescription() == "ImageView" then
			table.insert(tb, v:getVirtualRenderer())
		end
	end
	return tb
end

function T:initEffRender()
    local tb = self:getEffRender()

    for i = 1, #tb do 
        tb[i]:setOpacity(255)
    end
end

function T:onEnter()
    self:initEffRender() -- 确保第二次添加的时候 透明度是正确的 因为在刚开始的时候 特效还没播放完就移除了 会有透明度问题

end

function T:onExit()

end

function T:cleanUp()

end

--- 设置完成状态
function T:setFinishState(t)
	if t == true then
		if self.finish == true then return end

		self.finish = true
	else
		if self.finish == false then return end

		self.finish = false
	end
end

-- 加载ui
function T:loadJson()
	local widget
	if not self.extra then
		widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.taskItem)
	else
		widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.taskExtraTag)
	end

	self.widget = widget
	self:setContentSize(widget:getContentSize())
	self:addChild(widget)

	CCUIREL:toOther("left", "bottom", widget, self, 0, 0)
end

function T:initUI()

	local w = self.widget
	if not self.uiChild then self.uiChild = {} end
	local c = self.uiChild

	-- 任务类型 图片
	c.imgKind = Cocos2dxUtils:getChildByName(w, "imgKind")
	-- 任务名字
	c.taskName = Cocos2dxUtils:getChildByName(w, "taskName")
	-- 任务现有进度
	c.taskCondition = Cocos2dxUtils:getChildByName(w, "taskCondition")
	-- 任务详细信息 btn
	c.info = Cocos2dxUtils:getChildByName(w, "info")
	-- 任务完成 btn
	c.finish = Cocos2dxUtils:getChildByName(w, "finish")
	-- 任务完成情况
	c.finishInfo = Cocos2dxUtils:getChildByName(w, "finishInfo")

	if self.extra then
		c.head = Cocos2dxUtils:getChildByName(w, "head")
         
		c.headBackPanel = Cocos2dxUtils:getChildByName(w, "headBackPanel")
        --c.headBack:removeFromParent()

        local t = SecretaryGenerator:getBust("susy_clothing_1",SecretaryGenerator.DEFAULT_BUST_RECT)
        c.headBackPanel:addChild(t)
        --t:setGlobalZOrder(1)  -- 设置全局
        --t:setScaleX(0.75)
        --t:setScaleY(0.8)
        t:setAnchorPoint(cc.p(0, 0)) 
        t:setScale(0.5)
        REL:toOther("left", "bottom", t, c.headBackPanel, -20, 0)
        c.headBack = t

        --c.headBack:getVirtualRenderer():setTexture(t)

		if BGMgr:isGuide() then
			BGMgr:regGuideObj(BGMgr.TGR_TASK_WINDOW_FINISH_BUTTON,c.finish)
			
			BGMgr:fire(BGMgr.EVT_TASK_MAIN_CREATED)
		end
	end


	self:initFixedUI()
end

-- 初始化
function T:init()
	self:initUI()

	self:initView()
	self:registerUIEvent()
end

-- 初始化固定的属性
function T:initFixedUI()

	if self.extra then
		if self.uiChild.headBack then
			local scaleX = self.uiChild.headBack:getScaleX()
			local scaleY = self.uiChild.headBack:getScaleY()
			reductionScale(self.uiChild.headBack, scaleX, scaleY)
		end
	end

	local tb = {}
	for k, v in pairs(self.uiChild) do
		if v:getDescription() == "Label" then
			table.insert(tb, v)
		end
	end

	Cocos2dxUtils:setTextFont(tb)
end

function T:updateView(taskSid, finish, kind, extra)
	self.taskSid = taskSid
	self.kind = kind
	self.extra = extra
	self.finish = finish

	if not self.widget then  -- 万一没有经过上面的步骤
		self:loadJson()
		self:initUI()
		self:registerUIEvent()
	end

	self:initView()
end

function T:getFinishState()
	return self.finish
end


function T:updateFinishState()
	if self.finish == false then
		self.uiChild.finishInfo:setString(Language:get("uy58"))
	end

	if self.finish == true then
		self.uiChild.finishInfo:setString(Language:get("uy59"))
	end
end

function T:initView()
	local c = self.uiChild
    local name = TaskManager:getTaskNameBySid(self.taskSid)
	c.taskName:setString(name) -- 任务名字
	c.taskCondition:setString(TaskManager:getCurrentTaskInfo(self.taskSid, self.finish)) -- 完成状态
	if self.extra then -- 额外的提示
		c.head:setString(Language:get("uy60"))
        c.head:setFontSize(CssStyles.default26.size)
	else
		local res, scale = TaskManager:getTaskHeadRes(self.taskSid)
		c.imgKind:loadTexture(res)
		c.imgKind:setScale(scale[1], scale[2])
	end

	if self.finish == false then
		c.finishInfo:setString(Language:get("uy58"))
		c.finish:loadTextures("button_blue_normal.png","button_general_down.png","",ccui.TextureResType.plistType)
		if c.finish.eff then  -- 完成的一个特效
			c.finish.eff:removeFromParent()
			c.finish.eff = nil
		end
	end

	if self.finish == true then
		c.finishInfo:setString(Language:get("uy59"))
		c.finish:loadTextures("button_yellow_normal.png","button_general_down.png","",ccui.TextureResType.plistType)
		if c.finish.eff then -- 先移除特效 可能会出现特效 停止不动的现象
			c.finish.eff:removeFromParent()
			c.finish.eff = nil
		end

		if not c.finish.eff then -- 调加特效
			c.finish.eff = HomePageManager:addEffect(c.finish, "buttonPolishingEff")
		end
	end
end

function T:registerUIEvent()
	local c = self.uiChild


	local func = function(t, e)
		if e ~= ccui.TouchEventType.ended then return end
		if t == c.finish then
			if BGMgr:isGuide() then
				BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_TASK_WINDOW_FINISH_BUTTON)
			end
		
			self:onFinishClick()
		elseif t == c.info then
			self:onInfoClick()
		end
	end

	BaseButton:extend(c.finish)
	BaseButton:extend(c.info)

	c.finish:addTouchEventListener(func)
	c.info:addTouchEventListener(func)
end
-- 信息按钮点击
function T:onInfoClick()
	local tb = TaskManager:getTaskReward(self.taskSid)
	TaskManager:showRewardInfoWindow(self.taskSid, self.finish, tb)
end

-- 完成按钮点击
function T:onFinishClick()
	if self.finish == true then -- 完成任务
		TaskManager:finishTask(self.taskSid)
	elseif self.finish == false then -- 打开相应的建筑
		TaskManager:openWindowBySid(self.taskSid, self.kind)
	end
end

--- 获取完成按钮对象
function T:getFinishBtnObject()
	if self.uiChild then
		return self.uiChild.finish
	end
	return nil
end

return T
