--region TaskMainNode.lua
--Author    : YuDing
--Date      : 2015/1/10
--Introduce : 主任务节点
local T = class("T", function() return ccui.Layout:create() end)

-- 条目集合
T.itemTb = nil

local itemTask = require("src.app.uiScript.components.task.taskItem.TaskItem")

function T:ctor()
    mapping(self, LuaEventDispatcher)

    self:registerScriptHandler(
        function(eventType)
            if eventType == CocosEvent.ENTER then
                self:onEnter()
            elseif eventType == CocosEvent.EXIT then
                self:onExit()
            elseif eventType == CocosEvent.CLEAN_UP then
                self:cleanUp()
            end
        end
    )

    self:init()
end

function T:onEnter() 
    if self.scroll then 
        if self.exitPosition then 
            self.scroll:getInnerContainer():setPosition(self.exitPosition) 
        end
    end

    --TaskManager:setMainNodeObject(self)
end

function T:onExit()
    if self.scroll then 
       local px, py = self.scroll:getInnerContainer():getPosition() 
       self.exitPosition = cc.p(px, py)
    end
end

function T:cleanUp()
    print("enter the taskMain cleanUp!!!")
    self:releasePublicEvent()

    self.itemTb = nil

    for k, v in pairs(self.itemPool) do -- 释放
        v.node:release()
    end

    --TaskManager:releaseMainNodeObject()
end

function T:init()
    self:initCfg()
    self:initScroll()
    self:updateView()
    self:registerPublicEvent()

end

----- 初始化通信锁
--function T:initCommunicationLock()
--    self.communicationLock = false -- 默认是关闭的
--end

----- 设置锁的状态 false true
--function T:setCommunicationState(v)
--    self.communicationLock = v
--end


--- 延迟加上item
function T:delayAddItem(delay)
    local itemAddEff = function(item)
		item:setOpacity(0)
		item:runAction(
			cc.FadeIn:create(0.5)
		)
	end

--    local tmp = {}
--    for k, v in pairs(delay) do 
--        table.insert(tmp, v)
--    end

    local len = #delay
   

    if self.firstEffect then -- 直接加载
        for i = 1, len do 
            delay[i]:delayShow(true) -- 加载
        end
    else -- 第一次进来
        local i = 1
	    self:runAction(
	    	cc.RepeatForever:create(
	    		cc.Sequence:create(
	    			cc.DelayTime:create(0.1),
	    			cc.CallFunc:create(function()
	    				if i <= len then
                            delay[i]:delayShow(true)

                            local tb = delay[i]:getEffRender()
                            for k, v in pairs(tb) do 
                                itemAddEff(v)
                            end
	    				else
	    					--self:stopAllActions()  -- 结束所有的动作
	    				end
	    				i=i+1
	    			end)
	    		)
	    	)
	    )

        self.firstEffect = true -- 赋值为真
    end

    

end

function T:registerPublicEvent()
    TaskManager:luaAddEventListener(TaskManager.noticeMessage.finishMainTask, self, self.updateView)

    TaskManager:luaAddEventListener(TaskManager.noticeMessage.aircraftNumTask, self, self.updateAircraftNumTask)

    TaskManager:luaAddEventListener(TaskManager.noticeMessage.resourceTask, self, self.updateResouceTask)

    TaskManager:luaAddEventListener(TaskManager.noticeMessage.pointTask, self, self.updatePointTask)

    TaskManager:luaAddEventListener(TaskManager.noticeMessage.maxAttackNPCTask, self, self.updateMaxAttackNPCTask)

    TaskManager:luaAddEventListener(TaskManager.noticeMessage.buildingLvlTask, self, self.updateBuildingLvlTask)
    --TaskManager:luaAddEventListener(TaskManager.noticeMessage.finishSuccess, self, self.finishItem)
end

function T:releasePublicEvent()
    TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.finishMainTask, self, self.updateView)

    TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.aircraftNumTask, self, self.updateAircraftNumTask)

    TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.resourceTask, self, self.updateResouceTask)

    TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.pointTask, self, self.updatePointTask)
                   
    TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.maxAttackNPCTask, self, self.updateMaxAttackNPCTask)

    TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.buildingLvlTask, self, self.updateBuildingLvlTask)
    --TaskManager:luaRemoveEventListener(TaskManager.noticeMessage.finishSuccess, self, self.finishItem)
end

--- 更新可完成条目
function T:updateWaitFinishItem(tb)
    for k, v in pairs(tb) do
        if self.itemTb[k] then 
            self.itemTb[k]:setFinishState()
        end
    end
end

--- 完成任务
function T:finishItem(sid)
    if self.itemTb[sid] then 
        self.itemTb[sid]:removeFromParent()
        self.itemTb[sid] = nil
    end
end

--- 创建条目
function T:createItem(sid)
    local tb = TaskManager:getPlayerMainTask()

    local item = itemTask.new(sid, tb[sid].finish, "main", false)

    return item
end

function T:initScroll()
    if not self.scroll then 
        self.scroll = ccui.ScrollView:create()
        self.scroll:setContentSize(self.cfg.scrollSize)
        self:addChild(self.scroll)
        self.scroll:setDirection(cc.SCROLLVIEW_DIRECTION_VERTICAL)
        self.scroll:setClippingEnabled(true)
        self.scroll:setBounceEnabled(true)
        self.scroll:setTouchEnabled(true)
    end
end

function T:initCfg()
    if not self.cfg then 
        self.cfg = require("src.app.uiScript.components.task.taskMain.TaskMainNodeCfg")
    end
end


--- 更新条目 
-- @return change true 改变了，位置需要更新 未改变位置不需要更新 只需要更新状态
function T:updateItem()
    if not self.itemTb then self.itemTb = {} end
    local tb = TaskManager:getPlayerMainTask()

    self:putItemToPoll()
    self.itemTb = {}

    for k, v in pairs(tb) do 
--        if not self.itemTb[k] then
--            self.itemTb[k] = self:createItem(k)
--            self.scroll:addChild(self.itemTb[k])
--        else
--            self.itemTb[k]:updateView(k, tb[k].finish, "main", false)
--        end

        if not self.itemTb[k] then 
            local item, f = self:getItemFromPoll(k) 

            if f == true then --- 缓存的
                item:updateView(k, tb[k].finish, "main", false)
            end

            self.scroll:addChild(item) -- 加载
            self.itemTb[k] = item
        end
     end
--    end

--    local tmp = {}
--    for k, v in pairs(self.itemTb) do -- 判断itemtb中多余的条目 比如是完成任务成功之后 
--        if not tb[k] then 
--            table.insert(tmp, k)
--        end
--    end

--    if #tmp > 0 then 
--        for i = 1, #tmp do 
--          self.itemTb[tmp[i]]:removeFromParent()  
----          if delay[k] then 
----             delay[k] = nil
----          end
--          self.itemTb[tmp[i]] = nil

--        end
--    end

end

--- 从poll中得到item对象
function T:getItemFromPoll(sid)
    self.itemPool = self.itemPool or {}
    --local t = {node = 0x0000, use = true}

    --print("池子里的item数目为: " .. #self.itemPool)

    local item = nil
    for k, v in pairs(self.itemPool) do 
        if v.use == false then -- 未使用 
            item = v.node
            v.use = true      -- 标记成使用
            return item, true  -- true 表示新创建的
        end
    end

    if not item then  -- 创建一个
        item = self:createItem(sid)
        item:retain() -- 缓存
        table.insert(self.itemPool, {node = item, use = true})
        return item, false
    end

    
end

--- 将item对象放入到poll中
function T:putItemToPoll()
    self.itemPool = self.itemPool or {}

    for k, v in pairs(self.itemPool) do
        v.node:removeFromParent()
        v.use = false
    end

end



--- 更新
function T:updateView()
    local base = {}
    local resource = {}
    local role = {}
    local extraTag = {} -- 额外的标记

    local tb = TaskManager:getPlayerMainTask()

    local sortTb = {}

    self:updateItem()

    for k, v in pairs(self.itemTb) do 
        --table.insert(sortTb, {k = v})
        if TaskManager:getTaskKind(k) == "base" then 
            table.insert(base, {sid = k, node = v})
        elseif TaskManager:getTaskKind(k) == "role" then 
            table.insert(role, {sid = k, node = v})
        elseif TaskManager:getTaskKind(k) == "resource" then 
            table.insert(resource, {sid = k, node = v})
        end

        if TaskData[k].extraTipSids > 0 then 
            table.insert(extraTag, k)
        end
    end

    -- 排序
    if #base > 1 then 
        table.sort(base, function(x, y) return x.sid < y.sid end)
    end
    if #role > 1 then 
        table.sort(role, function(x, y) return x.sid < y.sid end)
    end
    if #resource > 1 then 
        table.sort(resource, function(x, y) return x.sid < y.sid end)
    end
    if #extraTag > 1 then 
        table.sort(extraTag, function(x, y) return x < y end)
    end

    self:checkTag(base, role, resource, extraTag)
    self:addItemToScroll(base, role, resource, extraTag)
end

function T:checkTag(base, role, resource, extraTag)
    local tb = TaskManager:getPlayerMainTask()

    local finishTb = TaskManager:getFinishedTask()

    if #finishTb > 0 then -- 完成的任务也显示在上面
         if not self.extraTag then
             self.extraTag = self:createExtraTag(finishTb[1])
             self.extraTag:delayShow(true) -- 立马显示
             self:addChild(self.extraTag)
         else
             self.extraTag:updateView(finishTb[1], tb[finishTb[1]].finish, "main", true)
         end
    else  -- 未完成
         if #extraTag > 0 then -- 额外的提示 （优先提示）
             if not self.extraTag then
                 self.extraTag = self:createExtraTag(extraTag[1])
                 self.extraTag:delayShow(true) -- 立马显示

                 self:addChild(self.extraTag)
             else
                 self.extraTag:updateView(extraTag[1], tb[extraTag[1]].finish, "main", true)
             end
         else -- 优先提示的没有了 剩下的是 从任务id从小开始往大的排
             local minId = TaskManager:getMinSidTask()
             if minId == 0 then -- 没有了任务  删除
                if self.extraTag then 
                    self.extraTag:removeFromParent()
                    self.extraTag = nil
                end
             else 
                 if not self.extraTag then
                     self.extraTag = self:createExtraTag(minId)
                     self.extraTag:delayShow(true) -- 立马显示

                     self:addChild(self.extraTag)
                 else
                     self.extraTag:updateView(minId, tb[minId].finish, "main", true)
                 end
             end
             
         end
    end

    if #base > 0 then
        if not self.baseTag then 
            self.baseTag = self:createOtherTag("base", self.scroll)
            --self.scroll:addChild(self.baseTag)
        end
    else
        if self.baseTag then 
            self.baseTag:removeFromParent()
            self.baseTag = nil
        end
    end

    if #role > 0 then 
        if not self.roleTag then 
            self.roleTag = self:createOtherTag("role", self.scroll)
            --self.scroll:addChild(self.roleTag)
        end
    else
        if self.roleTag then 
            self.roleTag:removeFromParent()
            self.roleTag = nil
        end
    end

    if #resource > 0 then 
        if not self.resourceTag then 
            self.resourceTag = self:createOtherTag("resource", self.scroll)
            --self.scroll:addChild(self.resourceTag)
        end
    else
        if self.resourceTag then 
            self.resourceTag:removeFromParent()
            self.resourceTag = nil
        end
    end

    if self.extraTag then 
        local height = UI_VSIZE.height - self.cfg.scrollDT - self.cfg.scrollDB
        self.scroll:setContentSize(cc.size(self.cfg.scrollWidth, height))

        REL:toOther("left", "top", self.extraTag, self, 0, 0)
        REL:toOther("left", "top", self.scroll, self, self.cfg.scrollDistance.x, self.cfg.scrollDistance.y)

    else
        local height = UI_VSIZE.height - self.cfg.scrollDTNoTag - self.cfg.scrollDB
        self.scroll:setContentSize(cc.size(self.cfg.scrollWidth, height))

        REL:toOther("left", "top", self.scroll, self, 0, 0)
    end
   
end

-- 添加内容到scroll
function T:addItemToScroll(base, role, resource, extraTag)
    local distance = self.cfg.itemDistance      -- 两个组件之间的距离
    local scroll = self.scroll

    local height = 0
    local num = 0
    if self.resourceTag then 
        local sz = self.resourceTag:getContentSize()
        height = height + sz.height
        num = num + 1
    end 

    if self.roleTag then 
        local sz = self.roleTag:getContentSize()
        height = height + sz.height
        num = num + 1
    end 

    if self.baseTag then 
        local sz = self.baseTag:getContentSize()
        height = height + sz.height
        num = num + 1
    end 
    
    for i = 1, #base do 
        local sz = base[i].node:getContentSize()
        height = height + sz.height
        num = num + 1
    end

    for i = 1, #role do 
        local sz = role[i].node:getContentSize()
        height = height + sz.height
        num = num + 1
    end

    for i = 1, #resource do 
        local sz = resource[i].node:getContentSize()
        height = height + sz.height
        num = num + 1
    end

    if num > 1 then -- 加上距离
        height = distance * (num - 1) + height
    end

    local inner = scroll:getInnerContainer()

    local s = scroll:getContentSize()
    if height >= s.height then
        inner:setContentSize(cc.size(s.width, height))    -- 下一次进入这个函数 需要重置scroll的高度
    else
        inner:setContentSize(cc.size(s.width, s.height))
    end

    local first = false -- scroll的第一个
    local s = inner:getContentSize()
    local calHeight = function(tmp)
        if not tmp then return end 
        
        -- 依次放入 inner中
        if type(tmp) ~= "table" then 
            local sz = tmp:getContentSize()
            if first == false then  
                s.height = s.height - sz.height 
                first = true
            else
                s.height = s.height - sz.height - distance
            end
            --inner:addChild(tmp)
            tmp:setPosition(cc.p(0, s.height))
        else
            for i = 1, #tmp do  
                local sz = tmp[i].node:getContentSize()
                if first == false then 
                    s.height = s.height - sz.height 
                    first = true
                else
                    s.height = s.height - sz.height - distance
                end
                --inner:addChild(tmp[i])
                tmp[i].node:setPosition(cc.p(0, s.height))
            end


        end
    end

    if self.roleTag then
        calHeight(self.roleTag)
    end

    if #role > 0 then
        calHeight(role)
    end
    
    if self.baseTag then
        calHeight(self.baseTag)
    end

    if #base then
        calHeight(base)
    end

    if self.resourceTag then
        calHeight(self.resourceTag)
    end

    if #resource then 
        calHeight(resource)
    end

    scroll:jumpToTop()   -- scroll到顶部

    self:delaySortItem(base, role, resource) -- 位置调整之后 开始实现效果
end

--- 调整之后的显示
function T:delaySortItem(base, role, resource)
    local tmp = {}
    if #role > 0 then
        for i = 1, #role do 
            table.insert(tmp, role[i].node)
        end
    end
    if #base then
        for i = 1, #base do 
            table.insert(tmp, base[i].node)
        end
    end
    if #resource then 
        for i = 1, #resource do 
            table.insert(tmp, resource[i].node)
        end
    end

    self:delayAddItem(tmp)
end

function T:createExtraTag(sid)
    local tb = TaskManager:getPlayerMainTask()
    
    return itemTask.new(sid, tb[sid].finish, "main", true)
end

function T:createOtherTag(content, parent)
    local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.taskTag)
    local title = Cocos2dxUtils:getChildByName(widget, "title")
    local back = Cocos2dxUtils:getChildByName(widget, "back")

    Cocos2dxUtils:setTextFont(title) -- 设置字体

--    local uiLayout = ccui.Layout:create()
--    uiLayout:setContentSize(620, 43)
--    if parent then 
--        parent:addChild(uiLayout)
--    end

--    local back = Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("taskTagBack.png"), cc.rect(20, 0, 1, 43)) -- 三格缩放

--	uiLayout:addChild(back)

--	back:setContentSize(cc.size(620, 43))
--	REL:toOther("left", "bottom", uiBottomBorder, uiLayout, 0, 0)

    local str = nil
    if content == "base" then 
        str = Language:get("uy56")
    elseif content == "role" then 
        str = Language:get("uy55")
    elseif content == "resource" then 
        str = Language:get("uy57")
    end

    title:setString(str)

    if parent then 
        parent:addChild(widget)
    end

    return widget
    --return uiLayout
end
--- 更新飞机数量任务（这是已有的任务） 还有飞机是不用额外表示的 所以不用判定
function T:updateAircraftNumTask()
    self:updateTask("aircraftNumTask")
end

--- 更新资源任务
function T:updateResouceTask()
    self:updateTask("resouceTask")
end

--- 更新关卡任务
function T:updatePointTask()
    self:updateTask("pointTask")
end

--- 更新最大攻击NPC任务
function T:updateMaxAttackNPCTask()
    self:updateTask("npcTask")
end 

function T:updateBuildingLvlTask()
    self:updateTask("buildingLvlTask")
end

--- 更新
function T:updateTask(task)
    if not self.itemTb then self.itemTb = {} end
    local tb = TaskManager:getPlayerMainTask()

    local func   -- 判断sid的函数
    if task == "aircraftNumTask" then
        func = TaskManager.isAircraftNumTaskSid
    elseif task == "resouceTask" then
        func = TaskManager.isResourceTaskSid
    elseif task == "pointTask" then 
        func = TaskManager.isPointTaskSid
    elseif task == "npcTask" then
        func = TaskManager.isMaxAttackNPCTaskSid
    elseif task == "buildingLvlTask" then
        func = TaskManager.isBuildingTaskSid
    end
    
    if not func then
        return
    end

    for k, v in pairs(self.itemTb) do 
        if func(TaskManager, k) then  -- 判定 任务
            if self.itemTb[k] and tb[k] then -- 节点存在
                if self.itemTb[k]:getFinishState() ~= tb[k].finish then  -- 完成状态的标识不一样 就更新
                    self.itemTb[k]:updateView(k, tb[k].finish, "main", false)
                end
            end
        end
    end
end

--- 获取额外显示的 完成 按钮对象
function T:getExtraFinishBtnObject()
    if self.extraTag then 
        return self.extraTag:getFinishBtnObject()
    else 
        return nil
    end
end

return T
--endregion
