local TeamFormLocs=require("src.app.uiScript.components.team.TeamFormLocs")

local T=Class.TeamFormLocsSetable(
	TeamFormLocs,
	LuaEventDispatcher2
)

--- 战机选择视图
T.AircraftSelectWindow=require("src.app.uiScript.components.army.AircraftSelectWindow")

function T:def()
	TeamFormLocs.def(self)
	LuaEventDispatcher2.def(self)

	--- 可上阵的战机信息列表
	self._setableAircraftInfos={}
end

function T:ctor(camp)
	TeamFormLocs.ctor(self,camp)
	LuaEventDispatcher2.ctor(self)
end

function T:init()
	TeamFormLocs.init(self)

	self:initTouchListener()
end

--- 初始化战机阵位列表
function T:initFormLocs()
	for k,v in ipairs(self.formLocList) do
		local coordinate=v.coordinate

		local x,y=coordinate.x,coordinate.y
		v.hitArea={
			cc.p(x+6,y+88),
			cc.p(x+138,y+12),
			cc.p(x-18,y-102),
			cc.p(x-156,y-22)
		}

		--		local dn=cc.DrawNode:create()
		--		dn:clear()
		--		self:addChild(dn)
		--		dn:drawPolygon(v.hitArea,4,cc.c4b(0xff,0x00,0xff,0x44),1,cc.c4b(0xff,0x00,0x00,0x44))

		--战机阵形位置显示项
		local formLoc=self.FormLoc(k)
		self:addChild(formLoc)
		local s=formLoc:getContentSize()
		formLoc:setPosition(coordinate.x-s.width/2,coordinate.y-s.height/2)
		v.formLoc=formLoc
	end
end

--- 初始化 touch 事件侦听器
function T:initTouchListener()
	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	---按下
	local onTouchBegan = function(t, e)
		local p=t:getLocation()
		local localP=self:convertToNodeSpace(p)

		for k,v in ipairs(self.formLocList) do
			if MathUtil.containsPointInPolygon(localP,v.hitArea) then
				return true
			end
		end

		return false
	end

	---移动
	local onTouchMoved = function(t, e)
		return true
	end

	--- 释放
	local onTouchEnded = function(t, e)
		local p=t:getLocation()
		local localP=self:convertToNodeSpace(p)

		for k,v in ipairs(self.formLocList) do
			if MathUtil.containsPointInPolygon(localP,v.hitArea) then
				self:formLocTouch(v.formLoc)

				return true
			end
		end

		return false
	end

	local onTouchCanceled = function(t, e)
		return true
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(onTouchCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)
	local eventDispatcher = self:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

--- 更新可选择战机信息
-- @param data 战机信息
-- @param isToSelected 是否切换到被选择状态
function T:updateSetableAircraftInfo(data,isToSelected)
	if not data then
		return
	end

	local info=self._setableAircraftInfos[data.sid]
	if info then
		local c=data.count

		local freeCount,totalCount=info:getFreeingNum(),info:getNum()
		if isToSelected then
			if freeCount < c then
				info:setBusingNum(totalCount)
			else
				info:toBusingNum(c)
			end
		else
			if freeCount + c > totalCount then
				info:setBusingNum(0)
			else
				info:toFreeingNum(c)
			end
		end
	end
end

function T:formLocTouch(formLoc)
	local data=formLoc:getData()

	--此阵位上如果有数据,则将数据清除,否则弹出战机选择页面
	if data then
		formLoc:setData(nil)
		self:updateSetableAircraftInfo(data,false)

		self:luaDispatchEvent("change")
	else
		local aircraftSelectWindow=self.AircraftSelectWindow.new(function(data)
			formLoc:setData(data)
			self:updateSetableAircraftInfo(data,true)

			self:luaDispatchEvent("change")
		end,self._setableAircraftInfos)
		WindowManager:push(aircraftSelectWindow)
	end
end

--- 设置阵形
-- @param team 阵形数据
function T:setTeam(team)
	team=team or {}
	for k,v in ipairs(self.formLocList) do
		local loc=v.formLoc

		local oldData=loc:getData()
		self:updateSetableAircraftInfo(oldData,false)

		local data=team[k]
		loc:setData(data)
		self:updateSetableAircraftInfo(data,true)
	end

	self:luaDispatchEvent("change")
end

--- 设置可选择战机信息列表
function T:setSetableAircraftInfos(value)
	local l=self._setableAircraftInfos

	--去除新列表中不存在的项
	for k,v in pairs(l) do
		if not value[k] then
			l[k]=nil
		end
	end

	--将新列表中的数据拷贝到可选择战机信息列表
	for k,v in pairs(value) do
		if l[k] then
			l[k].num=v.num
		else
			l[k]=mapping({},v)
		end
	end

	--更新已使用战机信息
	for k,v in ipairs(self.formLocList) do
		local loc=v.formLoc

		local data=loc:getData()
		self:updateSetableAircraftInfo(data,false)
		self:updateSetableAircraftInfo(data,true)
	end
end
--- 获取可选择战机信息列表
function T:getSetableAircraftInfos()
	return self._setableAircraftInfos
end

return T
