--- 地图网格逻辑处理器
local T=Class.MapGridLogicHandler()

function T:def()
	--- 地图节点被点击特效
	-- _uiNodeTouchEff

	--- 是否正在处理中
	self._isProcessing=false
end

function T:ctor()

end

--- 播放地图节点被点击的特效
function T:playNodeTouchEff(mapX,mapY,completeCallback)
	local uiNodeTouchEff=self._uiNodeTouchEff
	if not uiNodeTouchEff then
		uiNodeTouchEff=cc.Sprite:createWithSpriteFrameName("worldSelectedPoint.png")
		self:addChild(uiNodeTouchEff)
		self._uiNodeTouchEff=uiNodeTouchEff
	else
		uiNodeTouchEff:setVisible(true)
	end
	uiNodeTouchEff:setPosition(self:coordinateMap2Device(mapX,mapY))
	uiNodeTouchEff:runAction(cc.Sequence:create(
		cc.Blink:create(0.4, 2),
		cc.CallFunc:create(function()
			uiNodeTouchEff:setVisible(false)

			if completeCallback then
				completeCallback()
			end
		end)
	))
end

--- 处理地图节点被点击事件
function T:handleTouchEvent(mapX,mapY)
	if self._isProcessing then
		return
	end
	self._isProcessing=true

	print(string.format("点击:%d,%d",mapX,mapY))

	AudioManager:playEffect(SoundRes.SOUND_COMMON_BUTTON)
	self:playNodeTouchEff(mapX,mapY,function()
		local nData=WorldMapMgr:getNodeData(mapX,mapY)

		if nData then
			local cellInfo = require("src.app.uiScript.scenes.homePage.world.HomePageWorldCellInfo").new()
			cellInfo:initView(nData, HomePageManager)    -- 加载到world上

			MUIAlter_custom.show({
				info=cellInfo,
				flag=4,
				isShowCloseButton = true,
				callback=function()
					self._isProcessing=false
				end
			})
		else
			self:moveCity(mapX,mapY)
		end
	end)
end

--- 移城
function T:moveCity(mapX,mapY)
	if ArmyTaskQueueManager:selfIsQueue() then
		MUIAlter.show({
			info = Language:get("uc451"),
			size = 30,
			callback=function()
				self._isProcessing=false
			end
		})
	else
		local num = PropsInfoMgr:getPlayerPropByID(202) -- 迁移令
		if num >= 1 then
			MUIAlter.show({
				info = Language:get("uy101", mapX,mapY),
				flag = 3,
				callback = function(flag)
					self._isProcessing=false

					if flag == FLAG_OK then
						self:_moveCity(false,mapX,mapY)
					end
				end
			})
		else
			if PlayerInstance:getDiamond() >= GameCfg.COST_FLY_COMMAND then
				MUIAlter.show({
					info = Language:get("uy100", mapX,mapY),
					flag = 3,
					callback = function(flag)
						self._isProcessing=false

						if flag == FLAG_OK then
							self:_moveCity(true,mapX,mapY)
						end
					end
				})
			else
				MUIAlter.show({
					info = Language:get("uy98", 98 - PlayerInstance:getDiamond()),
					flag = 3,
					callback = function(flag)
						self._isProcessing=false

						if flag == FLAG_OK then -- 打开钻石购买界面
							MUIPop.MUIToReCharge()
						end
					end
				})
			end
		end
	end
end

--- 迁移城市
-- @param isCostDiamond 是否花费钻石
-- @param x
-- @param y
function T:_moveCity(isCostDiamond,mapX,mapY)
	print(string.format("moveCity=>x:%d,y:%d",mapX,mapY))

	require("src.app.communication.worldMap.WorldMapFlyCommand").new():exec(
		{x = mapX, y = mapY},
		isCostDiamond,  -- 用钻石
		function(state)
			if state == ReturnCode.SUCCESS then
				local oldX, oldY = PlayerInstance:getWorldPosition()
				PlayerInstance:setWorldPosition(mapX,mapY)
				WorldMapMgr:updateNodesDataForced({
					{x = oldX, y = oldY},
					{x = mapX, y = mapY},
				})
			else
				WorldMapMgr:updateNodesDataForced({
					{x = mapX, y = mapY},
				})
			end
		end
	)
end

return T
