local AbstractGroundGrid=require("src.frameworks.cocos.ui.ground.AbstractGroundGrid")

--- 网格尺寸
local CELL_WIDTH,CELL_HEIGHT=1118,1118

--- 地图背景网格
local T=Class.MapGround(
	function()
		return cc.Node:create()
	end,
	AbstractUI,
	AbstractGroundGrid
)

function T:def()
	AbstractUI.def(self)
	AbstractGroundGrid.def(self)

	--- 背景源
	self.backFrame = ResLoader:getSpriteFrame("worldMapBg.jpg")

	local s=self.backFrame:getOriginalSize()

	--- 背景缩放比X
	self.backScaleX=CELL_WIDTH / s.width
	--- 背景缩放比Y
	self.backScaleY=CELL_HEIGHT / s.height

	--- 单元格背景对象池
	self.cellBackPool = Stack()
end

--- 构造函数
-- @param visibleRect 可见区域
-- @param cellSize 单元格尺寸
function T:ctor(visibleRect)
	AbstractUI.ctor(self)
	AbstractGroundGrid.ctor(self,visibleRect,{width=CELL_WIDTH,height=CELL_HEIGHT})
end

--- 显示单元格
-- @param cell
-- @return cell
function T:_showCell(cell)
	cell.isShow=true

	local x,y=cell.x,cell.y

	local els={}

	local uiBack=self.cellBackPool:pop()
	if not uiBack then
		uiBack=cc.Sprite:createWithSpriteFrame(self.backFrame)
		uiBack:setScale(self.backScaleX,self.backScaleY)
		self:addChild(uiBack)
		uiBack:setAnchorPoint(cc.p(0, 0))
	end
	uiBack:setPosition(x * CELL_WIDTH, y * CELL_HEIGHT)
	els.uiBack=uiBack

	cell.elements=els

	return cell
end

--- 隐藏单元格
-- @param cell
-- @return cell
function T:_hideCell(cell)
	cell.isShow=false

	local els=cell.elements
	if els then
		local uiBack=els.uiBack
		if uiBack then
			self.cellBackPool:push(uiBack)

			els.uiBack=nil
		end
	end

	return cell
end

return T
