local MapNode=require("src.app.uiScript.components.worldMap.MapNode")

--- 地图显示节点-玩家
local T=Class.MapNodePlayer(MapNode)

--- 当前静态对象被引用的数目
T.relCount=0

--- 保护罩对象池
T.PROTECTION_POOL=Stack()

--- 联盟图标对象池
T.ALLIANCE_ICON_POOL=Stack()

--- 翻译对象池
function T:releasePool()
	print(string.format("%s=>清理静态对象池",T.__name))

	local stack=self.ALLIANCE_ICON_POOL
	while not stack:isEmpty() do
		stack:pop():release()
	end

	local stack=self.PROTECTION_POOL
	while not stack:isEmpty() do
		stack:pop():release()
	end
end

function T:def()
	MapNode.def(self)

	--- 图标
	-- _uiIcon

	--- 玩家名称背景
	-- _uiNameBack

	--- 联盟图标
	-- _allianceIcon

	--- 玩家名
	-- _uiName

	--- 保护罩
	-- _uiProtection
end

function T:ctor(...)
	MapNode.ctor(self,...)

	T.relCount = T.relCount + 1
end

function T:cleanUp()
	MapNode.cleanUp(self)

	local aSocial = UserInfo.allianceBox.myAlliance.allianceSocial
	aSocial:luaRemoveEventListener(aSocial.EVT_ADD_FRIEND_REFRESH,self,self.updateAllianceIcon)
	aSocial:luaRemoveEventListener(aSocial.EVT_DELETE_FRIEND_REFRESH,self,self.updateAllianceIcon)
	aSocial:luaRemoveEventListener(aSocial.EVT_ADD_ENEMY_REFRESH,self,self.updateAllianceIcon)
	aSocial:luaRemoveEventListener(aSocial.EVT_DELETE_ENEMY_REFRESH,self,self.updateAllianceIcon)

	T.relCount = T.relCount - 1
	if T.relCount == 0 then
		T:releasePool()
	end
end

function T:_init()
	MapNode._init(self)

	local uiIcon=cc.Sprite:createWithSpriteFrameName("worldAgainst.png")
	self:addChild(uiIcon,1)
	REL:toOther("center","middle",uiIcon,self,0,0)
	self._uiIcon=uiIcon

	local uiNameBack=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("nameBack2.png"))
	self:addChild(uiNameBack,2)
	self._uiNameBack=uiNameBack

	local uiName=XMLLabel()
	uiName:setCssStyle(CssStyles.default20)
	uiName:setAutoSizeLabel("width")
	uiName:setIsSingleLine(true)
	uiName:setAlign("center","middle")
	self:addChild(uiName,4)
	self._uiName=uiName

	local aSocial = UserInfo.allianceBox.myAlliance.allianceSocial
	aSocial:luaAddEventListener(aSocial.EVT_ADD_FRIEND_REFRESH,self,self.updateAllianceIcon)
	aSocial:luaAddEventListener(aSocial.EVT_DELETE_FRIEND_REFRESH,self,self.updateAllianceIcon)
	aSocial:luaAddEventListener(aSocial.EVT_ADD_ENEMY_REFRESH,self,self.updateAllianceIcon)
	aSocial:luaAddEventListener(aSocial.EVT_DELETE_ENEMY_REFRESH,self,self.updateAllianceIcon)
end

function T:updateView()
	MapNode.updateView(self)

	local data=self._data

	local uiIcon,uiNameBack, uiName= self._uiIcon, self._uiNameBack, self._uiName

	local sType=data.showType
	if sType == 101 then
		--暗黑
		uiIcon:setSpriteFrame(ResLoader:getSpriteFrame("worldAgainst_1.png"))
	elseif sType == 102 then
		--冰晶
		uiIcon:setSpriteFrame(ResLoader:getSpriteFrame("worldAgainst_2.png"))
	elseif sType == 103 then
		--黄金
		uiIcon:setSpriteFrame(ResLoader:getSpriteFrame("worldAgainst_3.png"))
	elseif sType == 104 then
		--土豪金
		uiIcon:setSpriteFrame(ResLoader:getSpriteFrame("worldAgainst_4.png"))
	else
		--普通
		uiIcon:setSpriteFrame(ResLoader:getSpriteFrame("worldAgainst.png"))
	end

	uiName:setString(data.name)
	uiName:updateView()

	REL:relOther(uiNameBack,uiName,-14,-14,-6,SIZE_NIL)
	REL:toOther("center","middle",uiNameBack,self,0,-30)
	REL:toOther("center","middle",uiName,uiNameBack,0,0)

	self:updateAllianceIcon()
	self:updateProtection()
end

--- 更新联盟图标
function T:updateAllianceIcon()
	local data=self._data
	local aId=data.allianceId or 0

	if aId ~= 0 then
		local myA=UserInfo.allianceBox.myAlliance
		local myAId=myA.id

		if myAId == aId then
			self:getAllianceIcon("allianceLabel_my.png")
		else
			local aSocial = myA.allianceSocial
			if aSocial:judgeIsAtEnemyList(aId) then
				--自己的敌对联盟
				self:getAllianceIcon("allianceLabel_hostile.png")
			elseif aSocial:judgeIsAtFrendList(aId) then
				--自己的友联盟
				self:getAllianceIcon("allianceLabel_friendly.png")
			else
				--中立
				self:putAllianceIcon()
			end
		end
	else
		self:putAllianceIcon()
	end
end

--- 获取联盟图标
function T:getAllianceIcon(iconUrl)
	local uiAIcon=self._allianceIcon

	if not uiAIcon then
		uiAIcon=self.ALLIANCE_ICON_POOL:pop()
		if not uiAIcon then
			uiAIcon=cc.Sprite:create()
			uiAIcon:retain()
		end
		uiAIcon:setSpriteFrame(ResLoader:getSpriteFrame(iconUrl))
		self:addChild(uiAIcon,3)
		uiAIcon:release()
		self._allianceIcon=uiAIcon

		local uiNameBack, uiName= self._uiNameBack, self._uiName
		REL:relOther(uiNameBack,uiName,-24,-12,-4,SIZE_NIL)
		REL:toOther("center","middle",uiNameBack,self,20,-30)
		REL:toOther("center","middle",uiName,uiNameBack,2,0)
		REL:toOther("left","middle",uiAIcon,uiNameBack,-26,0)
	end

	return uiAIcon
end

--- 存储联盟图标
function T:putAllianceIcon()
	local uiAIcon=self._allianceIcon

	if uiAIcon then
		uiAIcon:retain()
		uiAIcon:removeFromParent(false)
		self._allianceIcon=nil

		self.ALLIANCE_ICON_POOL:push(uiAIcon)

		local uiNameBack, uiName= self._uiNameBack, self._uiName
		REL:relOther(uiNameBack,uiName,-14,-14,-6,SIZE_NIL)
		REL:toOther("center","middle",uiNameBack,self,0,-30)
		REL:toOther("center","middle",uiName,uiNameBack,0,0)
	end
end

--- 更新保护罩
function T:updateProtection()
	local data=self._data

	if data.isProtection then
		self:getUiProtection()
	else
		self:putUiProtection()
	end
end

--- 获取保护罩对象
function T:getUiProtection()
	local uiP=self._uiProtection

	if not uiP then
		uiP=self.PROTECTION_POOL:pop()
		if not uiP then
			uiP=HomePageManager:getEff("protectCoverEff")
			uiP:setOpacity(255 * 0.8)

			uiP:retain()
		end
		self:addChild(uiP,5)
		uiP:release()
		REL:toOther("center", "middle", uiP, self, 8, 0)
		self._uiProtection=uiP
	end
end

--- 存入保护罩对象
function T:putUiProtection()
	local uiP=self._uiProtection

	if uiP then
		uiP:retain()
		uiP:removeFromParent(false)
		self._uiProtection=nil

		self.PROTECTION_POOL:push(uiP)
	end
end

function T:reset()
	MapNode.reset(self)

	self:putAllianceIcon()
	self:putUiProtection()
end

return T
