local MapNode=require("src.app.uiScript.components.worldMap.MapNode")
local MapNodeResLocType=require("src.app.model.WorldMap.MapNodeResLocType")

--- 地图显示节点-资源点
local T=Class.MapNodeResLoc(MapNode)

--- 当前静态对象被引用的数目
T.relCount=0

--- 占领图标对象池
T.OCCUPATION_ICON_POOL=Stack()

--- 翻译对象池
function T:releasePool()
	print(string.format("%s=>清理静态对象池",T.__name))

	local stack=self.OCCUPATION_ICON_POOL
	while not stack:isEmpty() do
		stack:pop():release()
	end
end

function T:def()
	MapNode.def(self)

	--- 图标
	-- _uiIcon

	--- 等级背景
	-- _uiLvBack

	--- 等级
	-- _uiLv

	--- 占领图标
	-- uiOccupationIcon
end

function T:ctor(...)
	MapNode.ctor(self,...)
end

function T:cleanUp()
	MapNode.cleanUp(self)

	T.relCount = T.relCount - 1
	if T.relCount == 0 then
		T:releasePool()
	end
end

function T:_init()
	MapNode._init(self)

	local uiIcon=cc.Sprite:create()
	self:addChild(uiIcon)
	self._uiIcon=uiIcon
	REL:toOther("center","middle",uiIcon,self,0,0)

	local uiLvBack=cc.Sprite:createWithSpriteFrameName("homePageLvlFrame.png")
	self:addChild(uiLvBack,3)
	REL:toOther("center","bottom",uiLvBack,self,0,0)
	self._uiLvBack=uiLvBack

	local uiLv=XMLLabel()
	uiLv:setAlign("center","middle")
	self:addChild(uiLv,4)
	REL:relOther(uiLv,uiLvBack,-2,-2,0,0)
	self._uiLv=uiLv
end

function T:updateView()
	MapNode.updateView(self)

	local data=self._data
	local resType=data.resType
	local url
	if resType == MapNodeResLocType.RES_STEEL then
		url="worldFieldSteel.png"
	elseif resType == MapNodeResLocType.RES_OIL then
		url="worldFieldOil.png"
	elseif resType == MapNodeResLocType.RES_SILICON then
		url="worldFieldSilicon.png"
	elseif resType == MapNodeResLocType.RES_TATI then
		url="worldFieldTati.png"
	else
		url="worldFieldGold.png"
	end
	self._uiIcon:setSpriteFrame(ResLoader:getSpriteFrame(url))

	self._uiLv:setString(data.lvl)
end

--- 更新占领图标
function T:updateOccupationIcon(isOccupation)
	if isOccupation then
		self:getOccupationIcon()
	else
		self:putOccupationIcon()
	end
end

function T:getOccupationIcon()
	local uiOIcon=self.uiOccupationIcon

	if not uiOIcon then
		uiOIcon=self.OCCUPATION_ICON_POOL:pop()
		if not uiOIcon then
			uiOIcon=cc.Sprite:createWithSpriteFrameName("homeWorldOccupy.png")
			uiOIcon:retain()
		end

		self:addChild(uiOIcon,2)
		uiOIcon:release()
		REL:toOther("center","middle",uiOIcon,self,0,0)
		self.uiOccupationIcon=uiOIcon
	end
end

function T:putOccupationIcon()
	local uiOIcon=self.uiOccupationIcon
	if uiOIcon then
		self.uiOccupationIcon=nil

		uiOIcon:retain()
		uiOIcon:removeFromParent(false)
		self.OCCUPATION_ICON_POOL:push(uiOIcon)
	end
end

function T:reset()
	MapNode.reset(self)

	self:putOccupationIcon()
end

return T
