local MapGround = require("src.app.uiScript.components.worldMap.MapGround")
local MapGrid = require("src.app.uiScript.components.worldMap.MapGrid")

--- 网格尺寸
local CELL_WIDTH, CELL_HEIGHT = MapGrid.CELL_WIDTH, MapGrid.CELL_HEIGHT

--- 网格单元预创建距离
local PREV_CRETATE_DIS = 60

--- 世界地图
local T = Class.WorldMap(
	function()
		return ccui.ScrollView:create()
	end ,
	AbstractUI,
	LuaEventDispatcher2
)

--- 地图的中心点发生改变
T.EVT_TYPE_VISIBLE_RECT_CENTER_CHANGED = "evt_type_visible_rect_center_changed"

--- 地图的边框-左
T.BORDER_LEFT = CELL_WIDTH

--- 地图的边框-右
T.BORDER_RIGHT = CELL_WIDTH

--- 地图的边框-上
T.BORDER_TOP = CELL_HEIGHT

--- 地图的边框-下
T.BORDER_BOTTOM = CELL_HEIGHT

function T:def()
	AbstractUI.def(self)
	LuaEventDispatcher2.def(self)

	--- 地面可视区域矩形
	self.groundVisibleRect = { x = 0, y = 0, width = 0, height = 0 }

	--- 地图网格可视区域矩形
	self.gridVisibleRect = { x = 0, y = 0, width = 0, height = 0 }

	--- 地面网格
	-- uiGround

	--- 地图网格
	-- uiGrid

	--- 可视区中心点的地图坐标-x
	self.visible_rect_map_x = 0

	--- 可视区中心点的地图坐标-y
	self.visible_rect_map_y = 0

	--- 此页面被显示列表移除时内容容器的坐标-x
	-- _innerContainerLeaveX

	--- 此页面被显示列表移除时内容容器的坐标-y
	-- _innerContainerLeaveY
end

function T:ctor()
	AbstractUI.ctor(self)
	LuaEventDispatcher2.ctor(self)

	self:_init()
end

function T:onEnter()
	AbstractUI.onEnter(self)

	if self._innerContainerLeaveX and self._innerContainerLeaveY then
		self:getInnerContainer():setPosition(self._innerContainerLeaveX,self._innerContainerLeaveY)
	end

	--	self:_updateVisibleRect()
	self:updateShowingRect()
end

function T:onExit()
	AbstractUI.onExit(self)

	self._innerContainerLeaveX,self._innerContainerLeaveY=self:getInnerContainer():getPosition()
end

--- 初始化
function T:_init()
	self:setDirection(3)
	self:setClippingEnabled(true)
	self:setInnerContainerSize(cc.size(
		WorldMapMgr.GRID_COL_NUM * CELL_WIDTH + self.BORDER_LEFT + self.BORDER_RIGHT,
		WorldMapMgr.GRID_ROW_NUM * CELL_HEIGHT + self.BORDER_TOP + self.BORDER_BOTTOM
	))

	-- 背景
	local uiGround = MapGround(self.groundVisibleRect)
	self:addChild(uiGround)
	self.uiGround = uiGround

	-- 地图节点网格
	local uiGrid = MapGrid(self.gridVisibleRect, WorldMapMgr.GRID_ROW_NUM, WorldMapMgr.GRID_COL_NUM)
	self:addChild(uiGrid)
	self.uiGrid = uiGrid
	uiGrid:setPosition(self.BORDER_LEFT, self.BORDER_BOTTOM)

	local groundVRect, gridVRect = self.groundVisibleRect, self.gridVisibleRect
	ccui.ScrollView.addEventListener(self, function(t, e)
		self:_updateVisibleRect()
	end )
end

--- 更新可视化区域
function T:_updateVisibleRect()
	local uiIC = self:getInnerContainer()
	local s = self:getContentSize()

	local x, y = uiIC:getPosition()
	local w, h = s.width, s.height

	local groundVRect = self.groundVisibleRect
	groundVRect.x = - x - PREV_CRETATE_DIS
	groundVRect.y = - y - PREV_CRETATE_DIS
	groundVRect.width = w + PREV_CRETATE_DIS * 2
	groundVRect.height = h + PREV_CRETATE_DIS * 2

	local gridVRect = self.gridVisibleRect
	gridVRect.x = - x - self.BORDER_LEFT - PREV_CRETATE_DIS
	gridVRect.y = - y - self.BORDER_BOTTOM - PREV_CRETATE_DIS
	gridVRect.width = w + PREV_CRETATE_DIS * 2
	gridVRect.height = h + PREV_CRETATE_DIS * 2

	self.uiGround:updateGrid()
	self.uiGrid:updateGrid()

	self:updateCurCoordinate()
end

--- 更新正在显示的地图网格区域
function T:updateShowingRect()
	self.uiGrid:updateShowingCelles()
end

--- 更新当前坐标值(视图中心点坐标)
function T:updateCurCoordinate()
	local s = self:getContentSize()
	local uiIC = self:getInnerContainer()

	-- 屏幕中心点(世界坐标)
	local cwpt = self:convertToWorldSpace( { x = s.width / 2, y = s.height / 2 })

	-- 转换到内容滚动容器的坐标
	local lpt = self.uiGrid:convertToNodeSpace(cwpt)

	local x, y = self.uiGrid:coordinateDevice2Map(lpt.x, lpt.y)

	if x ~= self.visible_rect_map_x or y ~= self.visible_rect_map_y then
		self.visible_rect_map_x,self.visible_rect_map_y=x,y
		self:luaDispatchEvent(self.EVT_TYPE_VISIBLE_RECT_CENTER_CHANGED, x, y)
	end
end

--- 定位到指定坐标
-- @param x
-- @param y
function T:toCoordinate(mapX, mapY)
	local uiIC = self:getInnerContainer()
	local s = self:getContentSize()

	-- 地图网格中的坐标
	local x, y = self.uiGrid:coordinateMap2Device(mapX, mapY)

	-- 地面的坐标
	x, y = x + self.BORDER_LEFT, y + self.BORDER_BOTTOM

	-- 指定地图节点到正中央的坐标
	local px, py = - x + s.width / 2, - y + s.height / 2

	local is = uiIC:getContentSize()
	-- 坐标修正,地图显示不能超出边框以外
	if px > 0 then
		px = 0
	elseif px < - (is.width - s.width) then
		px = - (is.width - s.width)
	end
	if py > 0 then
		py = 0
	elseif py < - (is.height - s.height) then
		py = - (is.height - s.height)
	end

	uiIC:setPosition(px, py)
	self:_updateVisibleRect()
end

--- 获取正在显示的指定单元格显示对象
function T:getMapNode(mapX,mapY)
	local cell=self.uiGrid:getCell(mapX,mapY)
	if cell then
		return cell.uiNode
	end
end

return T
