local WorldMap=require("src.app.uiScript.components.worldMap.WorldMap")

-- 预先加载的资源
local PRE_RESLIST = {
	"res/images/worldMap/worldMap.plist",
	{type="animations",list={
		{name = "boundaryEff",url = "boundary", beginIndex = 1, endIndex = 3, extension=".png", delay = 0.08},
	}},
	"res/images/worldMap/worldMapBack.plist",
}

local T=Class.WorldMapView(
	function()
		return cc.Node:create()
	end,
	AbstractUI,
	LuaEventDispatcher2
)

function T:def()
	AbstractUI.def(self)
	LuaEventDispatcher2.def(self)

	--- 地图
	-- _uiMap

	--- 渐变遮罩
	-- _uiGradualMask

	--- 小地图
	-- _uiSmallMap
end

--- 构造函数
function T:ctor()
	AbstractUI.ctor(self)
	LuaEventDispatcher2.ctor(self)

	ResLoader:syncLoad(PRE_RESLIST)

	--	WorldMapMgr:addArea(100,100,110,110)

	self:_init()
end

function T:onEnter()
	AbstractUI.onEnter(self)

	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_SWTICH_TO_WORLD_MAP)
	end
end

--- 回收
function T:cleanUp()
	AbstractUI.cleanUp(self)

	self._uiMap:luaRemoveEventListener(WorldMap.EVT_TYPE_VISIBLE_RECT_CENTER_CHANGED,self,self.handleVisibleRectCenterChanged)
	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.showSmallMap, self, self.toggleSmallMap)

	--断线重连/或重新登陆
	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateWorldMapGrid)

	local uiSM=self._uiSmallMap
	if uiSM then
		self._uiSmallMap=nil

		uiSM:release()
	end

	ResLoader:unloadReses(PRE_RESLIST)
end

function T:_init()
	REL:relScreen(self,0,0,0,0)

	self:initMap()
end

--- 初始化地图
function T:initMap()
	local uiMap=WorldMap()
	uiMap:setTouchEnabled(true)
	uiMap:setBounceEnabled(false)
	self:addChild(uiMap)
	REL:relOther(uiMap,self,0,0,108,78)
	self._uiMap=uiMap

	uiMap:luaAddEventListener(WorldMap.EVT_TYPE_VISIBLE_RECT_CENTER_CHANGED,self,self.handleVisibleRectCenterChanged)

	local uiGradualMask=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("worldSurfaceMask.png"))
	self:addChild(uiGradualMask)
	REL:relOther(uiGradualMask,uiMap,0,0,0,0)
	self._uiGradualMask=uiGradualMask

	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.showSmallMap, self, self.toggleSmallMap)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateWorldMapGrid)

	if BGMgr:isGuide() then
		BGMgr:regGuideObj(BGMgr.TGR_WORLD_MAP_VIEW,self)
	end
end

--- 处理地图可视区中心点的地图坐标改变
function T:handleVisibleRectCenterChanged(x,y)
	HomePageManager:updateCoordinate(x,y)
end

--- 显示/隐藏小地图
function T:toggleSmallMap()
	local uiSM=self._uiSmallMap
	if not uiSM then
		uiSM= require("src.app.uiScript.components.worldMap.smallMap.SmallMap")(self._uiMap)
		uiSM:setLocalZOrder(TOP_ORDER)
		uiSM:retain()

		self._uiSmallMap=uiSM
	end

	if uiSM:getParent() then
		if not uiSM:getIsLocationEnabled() then
			uiSM:setIsLocationEnabled(true)

			REL:toOther("center", "top", uiSM,self, 0, 200)
		else
			uiSM:removeFromParent(false)
		end
	else
		self:addChild(uiSM)
		uiSM:setIsLocationEnabled(false)

		REL:toOther("center", "top", uiSM,self, 0, 140)
	end
end

--- 更新世界地图网格
function T:updateWorldMapGrid()
	self._uiMap:updateShowingRect()
end

--- 获取正在显示的指定单元格显示对象
function T:getMapNode(mapX,mapY)
	return self._uiMap:getMapNode(mapX,mapY)
end

--- 跳转到指定位置
function T:gotoPosition(mapX,mapY)
	self._uiMap:toCoordinate(mapX,mapY)
end

--- 设置是否可滑动
function T:setIsScrollable(value)
	CppCocos2dxUtils:setIsScrollEnabled(self._uiMap,value)
end

return T
