local Point=require("src.app.uiScript.components.worldMap.mapMarchRoute.Point")
local Route=require("src.app.uiScript.components.worldMap.mapMarchRoute.Route")

--- 行军路线
local T=Class.MapMarchRoute(
	function()
		return cc.Node:create()
	end,
	AbstractUI
)

--- 从起始坐标到终止坐标逼近的步长
T.APPROACH_STEP=28

function T:def()
	AbstractUI.def(self)

	--- 地图网格
	-- _mapGrid

	--- 路线字典
	self.routeDict={}

	--- 占领字典
	self.occupyDict={}

	--- 路线点对象池
	self.routePtsPool=Stack()

	--- 部队图标对象池
	self.armyIconPool=Stack()

	--- 路线箭头对象池
	self.routeArrowPool=Stack()
end

function T:ctor(mapGrid)
	AbstractUI.ctor(self)

	self._mapGrid=mapGrid

	self:_init()
end

function T:cleanUp()
	AbstractUI.cleanUp(self)

	--开始出征
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_START_BATTLE_REFRESH, self, self.handleStartBattle)
	-- 出征结束
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BATTLE_OVER, self, self.handleEndBattle)
	-- 开始返回
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_START_BACK_REFRESH, self, self.handleStartBack)
	-- 返回结束
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.handleEndBack)

	--出征到达倒计时刷新
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BATTLE_REMAIN_TIME, self, self.handleUpdateStartBattleRemainingTime)
	--返回到达倒计时刷新
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BACK_REMAIN_TIME, self, self.handleUpdateBackBattleRemainingTime)

	--占领倒计时刷新
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_STATION_REMAIN_TIME, self, self.occupationStart)
	--占领结束
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_STATION_OVER, self, self.occupationEnd)

	--断线重连/或重新登陆
	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateRoutes)

	--释放路线点对象池
	print("释放路线点对象池")
	local stack=self.routePtsPool
	while not stack:isEmpty() do
		stack:pop():release()
	end

	--释放部队图标对象池
	print("释放部队图标对象池")
	local stack=self.armyIconPool
	while not stack:isEmpty() do
		stack:pop():release()
	end

	--释放路线箭头对象池
	print("释放路线箭头对象池")
	local stack=self.routeArrowPool
	while not stack:isEmpty() do
		stack:pop():release()
	end
end

--- 初始化
function T:_init()
	local infos=ArmyTaskQueueManager:getTaskInfos()
	for _,v in pairs(infos) do
		local status=v.status
		if status == 1 then
			self:addRoute(v,false)--出征信息列表
		elseif status == 2 then
			self:occupationStart(v)--占领信息列表

			self.occupyDict[v.id]=v
		elseif status == 3 then
			self:addRoute(v,true)--返回信息列表
		end
	end

	--开始出征
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_START_BATTLE_REFRESH, self, self.handleStartBattle)
	-- 出征结束
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BATTLE_OVER, self, self.handleEndBattle)
	-- 开始返回
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_START_BACK_REFRESH, self, self.handleStartBack)
	-- 返回结束
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.handleEndBack)

	--出征到达倒计时刷新
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BATTLE_REMAIN_TIME, self, self.handleUpdateStartBattleRemainingTime)
	--返回到达倒计时刷新
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BACK_REMAIN_TIME, self, self.handleUpdateBackBattleRemainingTime)

	--占领倒计时刷新
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_STATION_REMAIN_TIME, self, self.occupationStart)
	--占领结束
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_STATION_OVER, self, self.occupationEnd)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateRoutes)
end

--- 更新路线列表
function T:updateRoutes()
	local infos=ArmyTaskQueueManager:getTaskInfos()

	local infosDict={}
	for _,v in pairs(infos) do
		infosDict[v.id]=v
	end

	for _,v in pairs(self.routeDict) do
		local id = v.id
		local info=infosDict[id]
		if info and info.status == 1 or info.status == 3 then
			infosDict[id]=nil
		else
			self:_rmvRoute(id)
		end
	end
	for _,v in pairs(self.occupyDict) do
		local id = v.id
		local info=infosDict[id]
		if info and info.status == 2 then
			infosDict[id]=nil
		else
			self:occupationEnd(info)
		end
	end

	for _,v in pairs(infosDict) do
		local status=v.status
		if status == 1 then
			self:addRoute(v,false)--出征信息列表
		elseif status == 2 then
			self:occupationStart(v)--占领信息列表

			self.occupyDict[v.id]=v
		elseif status == 3 then
			self:addRoute(v,true)--返回信息列表
		end
	end
end

--- 处理开始出征
function T:handleStartBattle(routeInfo)
	self:addRoute(routeInfo,false)
end

--- 处理出征结束
function T:handleEndBattle(routeInfo)
	self:_rmvRoute(routeInfo.id)
end

--- 处理开始返回
function T:handleStartBack(routeInfo)
	self:addRoute(routeInfo,true)

	self:occupationEnd(routeInfo)
end

--- 处理返回结束
function T:handleEndBack(routeInfo)
	self:_rmvRoute(routeInfo.id)
end

--- 处理更新出征到达倒计时更新事件
function T:handleUpdateStartBattleRemainingTime(routeInfo)
	local route=self.routeDict[routeInfo.id]
	if route then
		self:_updateArmyIcon(route,routeInfo.movingCost,routeInfo.remainingSec)
	end
end

--- 处理更新返回到达倒计时更新事件
function T:handleUpdateBackBattleRemainingTime(routeInfo)
	local route=self.routeDict[routeInfo.id]
	if route then
		self:_updateArmyIcon(route,routeInfo.movingCost,routeInfo.remainingSec)
	end
end

--- 添加路线
-- @param routeInfo 路线信息
-- @param isBack 是否是回城
function T:addRoute(routeInfo,isBack)
	local sx,sy,ex,ey
	if isBack then
		ex,ey=PlayerInstance:getWorldPosition()
		sx,sy=routeInfo.posX,routeInfo.posY
	else
		ex,ey=routeInfo.posX,routeInfo.posY
		sx,sy=PlayerInstance:getWorldPosition()
	end
	local route=self:_addRoute(routeInfo.id,ex,ey,sx,sy)

	--刷新当前正在显示单元格,以便将路径显示出来
	local angle=route.angle
	for _,v in pairs(self._mapGrid.showingCellDict) do
		local pts=route:getRefCellPoits(v.x,v.y)
		if pts then
			for _,pt in ipairs(pts) do
				self:_getRoutePt(pt,angle)
			end
		end
	end

	--刷新部队图标位置
	self:_updateArmyIcon(route,routeInfo.movingCost,routeInfo.remainingSec)
end

--- 添加路线
-- @param id 编号
-- @param targetX 目标坐标-x
-- @param targetY 目标坐标-y
-- @param sourceX 源坐标-x
-- @param sourceY 源坐标-y
function T:_addRoute(id,targetX,targetY,sourceX,sourceY)
	local mapGrid=self._mapGrid

	local dvcSx,dvcSy=mapGrid:coordinateMap2Device(sourceX,sourceY)
	local dvcEx,dvcEy=mapGrid:coordinateMap2Device(targetX,targetY)

	local radian=math.atan2(dvcEy-dvcSy,dvcEx-dvcSx)
	local distance=MathUtil.pointToPointDistance(dvcSx,dvcSy,dvcEx,dvcEy)

	local step=self.APPROACH_STEP

	local angle=-math.deg(radian) --计算机和数学的角度旋转方向相反

	--路线
	local route=Route(id,targetX,targetY,angle,distance)
	self.routeDict[id]=route

	local lt=TimeUtils:getCurrentMillis()

	local sin,cos=math.sin,math.cos
	local d2g=mapGrid.coordinateDevice2Grid

	--添加路线点到相关节点中
	for i=step,distance-(step * 2),step do
		local x,y=dvcSx+i*cos(radian),dvcSy+i*sin(radian)
		local grdX,grdY=d2g(mapGrid,x,y)
		route:addPoint2RefCell(x,y,grdX,grdY)
	end

	--添加中缝箭头到最后一个节点中
	local x,y=dvcSx+distance*cos(radian),dvcSy+distance*sin(radian)
	local grdX,grdY=d2g(mapGrid,x,y)
	route:addPoint2RefCell(x,y,grdX,grdY,Point.TYPE_ARROW)

	print("计算耗时:",TimeUtils:getCurrentMillis()-lt)

	return route
end

--- 移除路线
-- @param id 编号
function T:_rmvRoute(id)
	--路线
	local route=self.routeDict[id]
	if route then
		for _,pt in pairs(route:getAllPoints()) do
			self:_puRoutePt(pt)
		end

		self:_updateArmyIcon(route,0,0)
	end

	self.routeDict[id]=nil
end

--- 更新部队图标
function T:_updateArmyIcon(route,allSec,remainingSec)
	local uiArmyIcon=route.uiArmyIcon
	if remainingSec > 0 then
		local angle=route.angle

		if not uiArmyIcon then
			uiArmyIcon=self.armyIconPool:pop()
			if not uiArmyIcon then
				uiArmyIcon=cc.Sprite:createWithSpriteFrameName("homePageArmy.png")
				uiArmyIcon:setAnchorPoint({x=0.5,y=0.5})
				uiArmyIcon:retain()
			end

			self:addChild(uiArmyIcon,2)
			uiArmyIcon:release()
			uiArmyIcon:setRotation(angle+90)--原图片需要转90度角度才归零
			route.uiArmyIcon=uiArmyIcon
		end
		local dvcEx,dvcEy=self._mapGrid:coordinateMap2Device(route.targetX,route.targetY)

		uiArmyIcon:stopAllActions()

		--弧度
		--注意:计算机和数学的角度旋转方向相反
		local r=math.rad(-angle+180)
		--距离
		local d=route.distance

		local offset=MathUtil.getPointOfCircle(r,d*(remainingSec/allSec))
		uiArmyIcon:setPosition(dvcEx+offset.x,dvcEy+offset.y)

		--移动动画效果
		local offset=MathUtil.getPointOfCircle(r,d*((remainingSec-1)/allSec))
		uiArmyIcon:runAction(cc.MoveTo:create(1,{x=dvcEx+offset.x,y=dvcEy+offset.y}))
	else
		if uiArmyIcon then
			uiArmyIcon:retain()
			uiArmyIcon:removeFromParent(false)
			self.armyIconPool:push(uiArmyIcon)

			route.uiArmyIcon=nil
		end
	end
end

--- 占领开始
function T:occupationStart(routeInfo)
	self:_updateOccupationIcon(routeInfo,true)
end

--- 占领结束
function T:occupationEnd(routeInfo)
	self:_updateOccupationIcon(routeInfo,false)
end

--- 更新占领图标
function T:_updateOccupationIcon(routeInfo,isOccupation)
	--	print("更新占领图标",routeInfo.posX..","..routeInfo.posY)

	local cell=self._mapGrid.showingCellDict[routeInfo.posX..","..routeInfo.posY]
	if cell and cell.uiNode and cell.uiNode.updateOccupationIcon then
		--		print("更新占领图标2",routeInfo.posX..","..routeInfo.posY,cell.x,cell.y)
		cell.uiNode:updateOccupationIcon(isOccupation)
	end
end

--- 显示单元格
function T:showCell(grdX,grdY)
	for _,route in pairs(self.routeDict) do
		local angle=route.angle

		local pts=route:getRefCellPoits(grdX,grdY)
		if pts then
			for _,pt in ipairs(pts) do
				self:_getRoutePt(pt,angle)
			end
		end
	end
end

--- 隐藏单元格
function T:hideCell(grdX,grdY)
	for _,route in pairs(self.routeDict) do
		local pts=route:getRefCellPoits(grdX,grdY)
		if pts then
			for _,pt in ipairs(pts) do
				self:_puRoutePt(pt)
			end
		end
	end
end

--- 获取路径点显示对象
function T:_getRoutePt(pt,angle)
	local uiPt
	if pt.type == Point.TYPE_POINT then
		uiPt=self.routePtsPool:pop()
		if not uiPt then
			uiPt=cc.Sprite:createWithSpriteFrameName("homeWorldRoadArrow.png")
			uiPt:retain()
		end
	else
		uiPt=self.routeArrowPool:pop()
		if not uiPt then
			uiPt=cc.Sprite:createWithSpriteFrameName("worldRoadArrow.png")
			uiPt:setAnchorPoint({x=1,y=0.5})
			uiPt:retain()
		end
	end
	self:addChild(uiPt)
	uiPt:release()
	uiPt:setRotation(angle)
	uiPt:setPosition(pt)

	pt.uiPt=uiPt
end

--- 存入路径点显示对象
function T:_puRoutePt(pt)
	local uiPt=pt.uiPt
	if uiPt then
		pt.uiPt=nil

		uiPt:retain()
		uiPt:removeFromParent(false)

		if pt.type == Point.TYPE_POINT then
			self.routePtsPool:push(uiPt)
		else
			self.routeArrowPool:push(uiPt)
		end
	end
end

return T
