--- 小地图
local T=Class.SmallMap(
	function()
		return cc.Node:create()
	end,
	AbstractUI
)

--- 点击的事件的触发的最大距离
local TOUCH_MAX_DIS = 16

--- 边框尺寸
T.BORDER_SIZE=10

function T:def()
	AbstractUI.def(self)

	--- 是否可定位
	self._isLocationEnabled=false

	--- 点击开始位置
	-- _touchSPt

	--- 行军路线字典
	--	self.armyRouteDict={}

	--- 小地图的原始尺寸
	-- _originalSize

	--- 地图视图
	-- _uiMapView

	--- 背景
	-- _uiBack

	--- 当前显示区域的位置
	-- _uiCurLoc

	--- 自己城堡在地图中的位置
	-- _uiHomeIcon
end

function T:ctor(uiMapView)
	AbstractUI.ctor(self)

	self._uiMapView=uiMapView

	self:_init()
end

function T:onEnter()
	AbstractUI.onEnter(self)

	self:updateCurLoc()
	self:updateHomeIcon()

	-- 地图的显示区域发生改变
	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.worldMoving, self, self.updateCurLoc)
	-- 玩家的世界坐标变化
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.changeWorldPositon, self, self.updateHomeIcon)

	--	self:updateArmyRoutes()
	--	self:addArmyTaskListeners()
end

function T:onExit()
	AbstractUI.onExit(self)

	-- 地图的显示区域发生改变
	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.worldMoving, self, self.updateCurLoc)
	-- 玩家的世界坐标变化
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.changeWorldPositon, self, self.updateHomeIcon)

	--	self:rmvArmyTaskListeners()
end

function T:_init()
	local backFrame=ResLoader:getSpriteFrame("worldSmallMap.png")
	self._originalSize=backFrame:getOriginalSize()

	self:setContentSize(self._originalSize)

	-- 背景
	local uiBack=cc.Sprite:createWithSpriteFrame(backFrame)
	uiBack:setOpacity(125)
	self:addChild(uiBack)
	REL:relOther(uiBack,self,0,0,0,0)
	self._uiBack=uiBack

	-- 当前显示区域的位置
	local uiCurLoc=cc.Sprite:createWithSpriteFrameName("worldSmallMove.png")
	uiCurLoc:setOpacity(125)
	self:addChild(uiCurLoc)
	self._uiCurLoc=uiCurLoc

	-- 自己城堡在地图中的位置
	local uiHomeIcon=cc.Sprite:createWithSpriteFrameName("homePageSwitchActive.png")
	uiHomeIcon:setOpacity(125)
	self:addChild(uiHomeIcon)
	self._uiHomeIcon=uiHomeIcon

	self:_addTouchEvtListener()
end

--function T:addArmyTaskListeners()
--	--开始出征
--	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_START_BATTLE_REFRESH, self, self.handleStartBattle)
--	-- 出征结束
--	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BATTLE_OVER, self, self.handleEndBattle)
--	-- 开始返回
--	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_START_BACK_REFRESH, self, self.handleStartBack)
--	-- 返回结束
--	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.handleEndBack)
--end
--
--function T:rmvArmyTaskListeners()
--	--开始出征
--	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_START_BATTLE_REFRESH, self, self.handleStartBattle)
--	-- 出征结束
--	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BATTLE_OVER, self, self.handleEndBattle)
--	-- 开始返回
--	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_START_BACK_REFRESH, self, self.handleStartBack)
--	-- 返回结束
--	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.handleEndBack)
--end

--- 设置是否可定位(是否启用定位功能)
function T:setIsLocationEnabled(value)
	if value ~= self._isLocationEnabled then
		self._isLocationEnabled=value

		local uiBack=self._uiBack
		if value then
			local w=V_SZ.width - 180

			local scale=w/self._originalSize.width
			self:setContentSize(cc.size(w,self._originalSize.height * scale))

			uiBack:setOpacity(255)
		else
			self:setContentSize(self._originalSize)

			uiBack:setOpacity(125)
		end
		REL:relOther(uiBack,self,0,0,0,0)

		self:updateCurLoc()
		self:updateHomeIcon()

		--		self:updateArmyRoutes()
	end
end
--- 是否可定位
function T:getIsLocationEnabled()
	return self._isLocationEnabled
end

--- 更新当前显示区域位置
function T:updateCurLoc()
	local uiMV=self._uiMapView
	self._uiCurLoc:setPosition(self:coordinateMap2Dvice(uiMV.visible_rect_map_x,uiMV.visible_rect_map_y))
end

--- 更新自己城堡在地图中的位置
function T:updateHomeIcon()
	self._uiHomeIcon:setPosition(self:coordinateMap2Dvice(PlayerInstance:getWorldPosition()))
end

----- 更新行军路线列表
--function T:updateArmyRoutes()
--	for k,uiRoute in pairs(self.armyRouteDict) do
--		self.armyRouteDict[k]=nil
--
--		if uiRoute then
--			uiRoute:removeFromParent()
--		end
--	end
--
--	--出征信息列表
--	for _,v in pairs(ArmyTaskQueueManager:getBattleInfos()) do
--		self:handleStartBattle(v)
--	end
--
--	--返回信息列表
--	for _,v in pairs(ArmyTaskQueueManager:getBackInfos()) do
--		self:handleStartBack(v,true)
--	end
--end
--
----- 处理开始出征
--function T:handleStartBattle(routeInfo)
--	if routeInfo then
--		local uiRoute = self.armyRouteDict[routeInfo.id]
--		if uiRoute then
--			uiRoute:removeFromParent()
--		end
--
--		local uiRoute=self:drawRoute(routeInfo.posX,routeInfo.posY)
--		self:addChild(uiRoute)
--
--		self.armyRouteDict[routeInfo.id]=uiRoute
--	end
--end
--
----- 处理出征结束
--function T:handleEndBattle(routeInfo)
--	if routeInfo then
--		local uiRoute = self.armyRouteDict[routeInfo.id]
--		if uiRoute then
--			self.armyRouteDict[routeInfo.id]=nil
--
--			uiRoute:removeFromParent()
--		end
--	end
--end
--
----- 处理开始返回
--function T:handleStartBack(routeInfo)
--	if routeInfo then
--		local uiRoute = self.armyRouteDict[routeInfo.id]
--		if uiRoute then
--			uiRoute:removeFromParent()
--		end
--
--		local uiRoute=self:drawRoute(routeInfo.posX,routeInfo.posY)
--		self:addChild(uiRoute)
--
--		self.armyRouteDict[routeInfo.id]=uiRoute
--	end
--end
--
----- 处理返回结束
--function T:handleEndBack(routeInfo)
--	if routeInfo then
--		local uiRoute = self.armyRouteDict[routeInfo.id]
--		if uiRoute then
--			self.armyRouteDict[routeInfo.id]=nil
--
--			uiRoute:removeFromParent()
--		end
--	end
--end
--
----- 绘制路线
--function T:drawRoute(targetX,targetY)
--	--	local hDvcX,hDvcY=self:coordinateMap2Dvice(PlayerInstance:getWorldPosition())
--	--	local tDvcX,tDvcY=self:coordinateMap2Dvice(targetX,targetY)
--	--
--	--	local uiDN = cc.DrawNode:create()
--	--	uiDN:drawSegment(cc.p(hDvcX,hDvcY), cc.p(tDvcX,tDvcY), 0.5, cc.c4f(1, 0, 0, 1))
--	--
--	--	return uiDN
--
--	local hx,hy=PlayerInstance:getWorldPosition()
--	return self:createLine(hx,hy,targetX,targetY)
--end
--
----- 创建一条直线
---- @param startX
---- @param startY
---- @param endX
---- @param endY
---- @param color3b
---- @return
--function T:createLine(sMapX,sMapY,eMapX,eMapY,color3b)
--	local uiLine=cc.Sprite:createWithSpriteFrameName("line_noColor.png")
--	uiLine:setAnchorPoint({x=0,y=0.5})
--
--	local dvcSx,dvcSy=self:coordinateMap2Dvice(sMapX,sMapY)
--	local dvcEx,dvcEy=self:coordinateMap2Dvice(eMapX,eMapY)
--
--	local distance=MathUtil.pointToPointDistance(dvcSx,dvcSy,dvcEx,dvcEy)
--	uiLine:setScaleX(distance/uiLine:getContentSize().width)
--	uiLine:setRotation(-math.deg(math.atan2(dvcEy-dvcSy,dvcEx-dvcSx))) --计算机和数学的角度旋转方向相反
--	uiLine:setPosition(dvcSx,dvcSy)
--	uiLine:setColor(color3b or cc.c3b(255,0,0))
--
--	return uiLine
--end

--- 地图坐标转换为当前小地图的设备坐标
-- @param mapX
-- @param mapY
-- @return dvcX,dvcY
function T:coordinateMap2Dvice(mapX,mapY)
	local s=self:getContentSize()

	local uiMV=self._uiMapView
	local uiMapGrid=uiMV.uiGrid
	local grdX,grdY=uiMapGrid:coordinateMap2Grid(mapX,mapY)
	local numCol,numRow=WorldMapMgr.GRID_COL_NUM - 1,WorldMapMgr.GRID_ROW_NUM - 1

	local bSize=self.BORDER_SIZE
	return bSize + (s.width-bSize * 2)*(grdX/numCol),bSize + (s.height - bSize * 2)*(grdY/numRow)
end

--交互事件处理相关>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--- 添加touch事件侦听
function T:_addTouchEvtListener()
	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	---开始拖动
	local onTouchBegan = function(t, e)
		if not self._isLocationEnabled then
			return false
		end

		local pt=t:getLocation()
		local localPt=self:convertToNodeSpace(pt)

		local s=self:getContentSize()
		local rect=cc.rect(0,0,s.width,s.height)
		if cc.rectContainsPoint(rect, localPt) then
			self._touchSPt=pt

			return true
		end
		return false
	end

	--- 拖动结束
	local onTouchEnded = function(t, e)
		local s,e=self._touchSPt,t:getLocation()
		if MathUtil.pointToPointDistance(s.x,s.y,e.x,e.y) > TOUCH_MAX_DIS then
			return
		end

		local localPt=self:convertToNodeSpace(e)

		local s=self:getContentSize()
		local rect=cc.rect(0,0,s.width,s.height)
		if cc.rectContainsPoint(rect, localPt) then
			self:toCoordinate(localPt.x,localPt.y)
		end
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	self:getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, self)
end
--交互事件处理相关<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

--- 定位
function T:toCoordinate(localDvcX,localDvcY)
	local uiMV=self._uiMapView
	local uiMapGrid=uiMV.uiGrid

	local numCol,numRow=WorldMapMgr.GRID_COL_NUM - 1,WorldMapMgr.GRID_ROW_NUM - 1

	local s=self:getContentSize()
	local bSize=self.BORDER_SIZE

	local grdX=math.floor((localDvcX-bSize)/(s.width-bSize * 2) * numCol)
	if grdX < 0 then
		grdX = 0
	elseif grdX > numCol then
		grdX = numCol
	end

	local grdY=math.floor((localDvcY-bSize)/(s.height-bSize * 2) * numRow)
	if grdY < 0 then
		grdY=0
	elseif grdY > numRow then
		grdY=numRow
	end

	local mapX,mapY=uiMapGrid:coordinateGrid2Map(grdX,grdY)

	MUIAlter.show({
		info=Language:get("ug_mapLocTip",mapX,mapY),
		confirmText=Language:get("ug103"),
		cancelText=Language:get("ug104"),
		flag=3,
		callback=function(flag)
			if flag == FLAG_OK then
				uiMV:toCoordinate(mapX,mapY)
			end
		end
	})
end

return T
