local _print=print
local function print(...)
	_print("<Loading>: ",table.concat({...},"\t"))
end

--- 资源加载及热更新检查
local T=Class.LoadingScene(
	function()
		return cc.Scene:create()
	end,
	AbstractUI
)

----- 资源更新器
--T.ResUpdate=require("src.frameworks.cocos.ResUpdate")

--- 登陆处理的扩展
local login_sdkExtend=nil
if CUR_SDK == nil then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_normal")
elseif CUR_SDK == SDKType.PLATFORM_360 then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_360")
elseif CUR_SDK == SDKType.PLATFORM_ENJOY_GAME then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_enjoyGame")
elseif CUR_SDK == SDKType.PLATFORM_TENGLONG_1 then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_1")
elseif CUR_SDK == SDKType.PLATFORM_UC then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_uc")
elseif CUR_SDK == SDKType.PLATFORM_VIVO then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_vivo")
elseif CUR_SDK == SDKType.PLATFORM_HUA_WEI then
	T.LoginView=require("src.app.uiScript.components.login.LoginView_huaWei")
end

--- 预加载资源列表
T.PRE_RES_LIST={
	"res/images/loadingBack.plist",
	"res/images/loading/loading.plist",

	"res/images/loading/loadingEff.plist",
	{type="animations",list={
		{name="deng1",url="deng1_000",beginIndex=0,endIndex=6,extension=".png",delay=0.12},
		{name="deng2",url="deng2_000",beginIndex=0,endIndex=7,extension=".png",delay=0.12},
		{name="dian",url="dian_000",beginIndex=0,endIndex=9,extension=".png",delay=0.1},
		{name="xian1",url="xian1_000",beginIndex=0,endIndex=9,extension=".png",delay=0.1},
		{name="xian2",url="xian2_000",beginIndex=0,endIndex=5,extension=".png",delay=0.1},
		{name="xian3",url="xian3_000",beginIndex=0,endIndex=7,extension=".png",delay=0.1},
	}},
}

--- 加载预加载资源
function T:loadPreRes()
	ResLoader:syncLoad(self.PRE_RES_LIST)
end
--- 卸载预加载资源
function T:unloadPreRes()
	ResLoader:unloadReses(self.PRE_RES_LIST)
end

function T:def()
	AbstractUI.def(self)

	--- 需要加载的资源数
	self.loadItemCount=0

	--- 日志记录解锁器
	-- self.logUnlocker=nil

	--- 欢迎界面
	-- _uiWelcomePage

	--- 延迟移除欢迎界面的 action
	-- _hideWelcomePageDelayAct
end

function T:ctor()
	AbstractUI.ctor(self)

	self:loadPreRes()

	self:init()
end

--- 进入场景(播放声音,刷新数据等)
function T:onEnter()
	AbstractUI.onEnter(self)

	LogIcon:show()

	AudioManager:playMusic(SoundRes.MUSIC_LOGIN,true)

	--显示一些特效,播放视频等
end

--- 清理
function T:cleanUp()
	AbstractUI.cleanUp(self)

	self:unloadPreRes()
end

function T:init()
	--背景
	local uiBack=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("big_startscene.jpg"))
	REL:relRect(uiBack,cc.rect(0,0,640,1136),0,0,0,0)
	self:addChild(uiBack)
	REL:toScreen("center","top",uiBack,0,0)
	self.uiBack=uiBack

	--显示游戏名称>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	local LanguageUtils=require("src.app.utils.LanguageUtils")
	if LanguageUtils:getId() ~= cc.LANGUAGE_CHINESE then
		local resPath=string.format("res/images/loading/appName_%s.jpg",LanguageUtils:getCode())

		local uiAppName=cc.Sprite:create(resPath)
		self:addChild(uiAppName)
		REL:toOther("center","top",uiAppName,uiBack,0,0)
		self.uiAppName=uiAppName
	end
	--显示游戏名称<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

	--版本号
	local uiVersion=XMLLabel()
	uiVersion:setOpacity(100)
	uiVersion:setHeight(36)
	self:addChild(uiVersion)
	REL:relScreen(uiVersion,10,10,SIZE_NIL,4)
	self.uiVersion=uiVersion

	self:updateVersion()

	local uiEffContainer=cc.Node:create()
	self:addChild(uiEffContainer)
	self.uiEffContainer=uiEffContainer

	AbstractUI.interval(self,function()
		self:playLoadingEff()
	end,3)
	self:playLoadingEff(true)

	self:playCloudEff()

	local uiProgressBar=cc.Node:create()
	self:addChild(uiProgressBar)
	self.uiProgressBar=uiProgressBar

	--进度条背景
	local uiProgressBack=cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("loadingProgressBack.png"))
	uiProgressBar:addChild(uiProgressBack)
	REL:toScreen("center","middle",uiProgressBack,0,0)
	self.uiProgressBack=uiProgressBack

	--进度条
	local uiProgressFore=cc.ProgressTimer:create(cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("loadingProgressFore.png")))
	uiProgressFore:setBarChangeRate(cc.p(1,0))--进度条动画方向，（1,0）表示横方向，（0,1）表示纵方向
	uiProgressFore:setType(cc.PROGRESS_TIMER_TYPE_BAR)--进度条类型
	uiProgressFore:setMidpoint(cc.p(0,1))--进度条的起始点，（0,y）表示最左边，（1,,y）表示最右边，（x,1）表示最上面，（x,0）表示最下面。
	uiProgressFore:setPercentage(0)--初始值
	uiProgressBar:addChild(uiProgressFore)
	REL:toOther("center","middle",uiProgressFore,uiProgressBack,0,0)
	self.uiProgressFore=uiProgressFore

	--提示文本
	local uiTip=XMLLabel(Language:get("ug2"))
	uiTip:setAlign("center","middle")
	uiProgressBar:addChild(uiTip)
	REL:relOther(uiTip,uiProgressBack,0,0,0,0)
	self.uiTip=uiTip

	--日志记录解锁器
	local logUnlocker=require("src.frameworks.cocos.log.LogRecordUnlocker")()
	self:addChild(logUnlocker)
	REL:toScreen("center","top",logUnlocker,0,0)
	self.logUnlocker=logUnlocker

	--飞机小贴士
	local des = require("src.app.logic.AirSmallStick")():getDes()
	local uAirSmallStick=XMLLabel()
	uAirSmallStick:setAlign("center","top")
	uAirSmallStick:setContentSize(cc.size(600,36))
	self:addChild(uAirSmallStick)
	REL:toScreen("center","middle",uAirSmallStick,0,-70)
	self.uAirSmallStick=uAirSmallStick
	uAirSmallStick:setString(Language:get(des))

	if CUR_SDK == SDKType.PLATFORM_ENJOY_GAME and C_PFM == cc.PLATFORM_OS_ANDROID then
		self:showWelcomePage()
		self._hideWelcomePageDelayAct=self:timeout(function()
			self._hideWelcomePageDelayAct=nil

			self:hideWelcomePage()
		end,0.6)
	end

--	if CUR_SDK == SDKType.PLATFORM_TENGLONG_1 or CUR_SDK == SDKType.PLATFORM_UC or CUR_SDK == SDKType.PLATFORM_VIVO or CUR_SDK == SDKType.PLATFORM_HUA_WEI then
--		self:showWelcomePage()
--		self._hideWelcomePageDelayAct=self:timeout(function()
--			self._hideWelcomePageDelayAct=nil
--
--			self:hideWelcomePage()
--		end,0.6)
--	end

	if CUR_SDK == SDKType.PLATFORM_TENGLONG_1 and SDK_Delegate and not SDK_Delegate:getIsInited() then
		SDK_Delegate:luaAddEventListener(SDK_Delegate.EVT_TYPE_INIT_COMPLETE,self,self.sdkInitComplete)
	else
		self:getServerListData()
	end
end

if CUR_SDK == SDKType.PLATFORM_TENGLONG_1 and SDK_Delegate then
	--- SDK初始化完成
	function T:sdkInitComplete()
		SDK_Delegate:luaRemoveEventListener(SDK_Delegate.EVT_TYPE_INIT_COMPLETE,self,self.sdkInitComplete)

		self:getServerListData()
	end
end

--- 获取服务器列表信息
function T:getServerListData()
	ServerListMgr:luaAddEventListener(ServerListMgr.EVT_TYPE_GET_SERVER_LIST_COMPLETE,self,self.loadCommonRes)
	ServerListMgr:luaAddEventListener(ServerListMgr.EVT_TYPE_UPDATE_START,self,self.luaUpdateStartHandler)

	ServerListMgr:getServerListData()
end

if CUR_SDK == SDKType.PLATFORM_ENJOY_GAME then
	--- 显示欢迎界面
	function T:showWelcomePage()
		--背景
		local uiBack=cc.Sprite:create("sdk_res/enjoyGame/welcome.png")
		REL:relRect(uiBack,cc.rect(0,0,640,1136),0,0,0,0)
		self:addChild(uiBack)
		REL:toScreen("center","middle",uiBack,0,0)
		self._uiWelcomePage=uiBack
	end

	--- 隐藏欢迎界面
	function T:hideWelcomePage()
		local uiPage=self._uiWelcomePage
		if uiPage then
			uiPage:runAction(cc.Sequence:create(
				cc.FadeOut:create(0.2),
				cc.CallFunc:create(function()
					uiPage:removeFromParent()
					self._uiWelcomePage=nil
				end)
			))
		end
	end
end

if CUR_SDK == SDKType.PLATFORM_TENGLONG_1 or CUR_SDK == SDKType.PLATFORM_UC or CUR_SDK == SDKType.PLATFORM_VIVO or CUR_SDK == SDKType.PLATFORM_HUA_WEI then
	--- 显示欢迎界面
	function T:showWelcomePage()
		--背景
		local uiBack=cc.Sprite:create("sdk_res/tonglong/flashScreen.png")
		REL:relRect(uiBack,cc.rect(0,0,640,1136),0,0,0,0)
		self:addChild(uiBack)
		REL:toScreen("center","middle",uiBack,0,0)
		self._uiWelcomePage=uiBack
	end

	--- 隐藏欢迎界面
	function T:hideWelcomePage()
		local uiPage=self._uiWelcomePage
		if uiPage then
			uiPage:runAction(cc.Sequence:create(
				cc.FadeOut:create(0.2),
				cc.CallFunc:create(function()
					uiPage:removeFromParent()
					self._uiWelcomePage=nil
				end)
			))
		end
	end
end

--- 开始lua代码及资源更新
function T:luaUpdateStartHandler()
	ServerListMgr:luaRemoveEventListener(ServerListMgr.EVT_TYPE_UPDATE_START,self,self.luaUpdateStartHandler)

	self.uiTip:setString(Language:get("ug_updating"))

	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_START,self,self.updateStartHandler)
	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_PROGRESS,self,self.updateProgressHandler)
	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_ITEM_START,self,self.updateItemStartHandler)
	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_ITEM_COMPLETE,self,self.updateItemCompleteHandler)
	ResUpdate:luaAddEventListener(ResUpdate.EVT_TYPE_UPDATE_COMPLETE,self,self.updateCompleteHandler)
end

--- 开始更新
function T:updateStartHandler()
	self.totalUpdateCount=ResUpdate:getUpdateQueue():getLength()
	self.curUpdateIndex=1

	self.uiProgressFore:stopAllActions()
	self.uiProgressFore:setPercentage(0)
end

--- 更新过程
function T:updateProgressHandler(perant)
	self.uiProgressFore:setPercentage(perant)

	--	local curP=self.uiProgressFore:getPercentage()
	--	self.uiProgressFore:runAction(cc.ProgressFromTo:create(0.1,curP,perant))
	--
	--	self.uiTip:setString(Language:get("ug_updatingProgress",self.curUpdateIndex,self.totalUpdateCount))
end

--- 单项更新开始
function T:updateItemStartHandler(updateItem)
	self.uiProgressFore:setPercentage(0)
	print("单项更新开始")
end

--- 更新单项完成
function T:updateItemCompleteHandler(updateItem)
	self.uiProgressFore:stopAllActions()
	self.uiProgressFore:setPercentage(100)
	self.curUpdateIndex=self.curUpdateIndex+1
	print("更新单项完成")
end

--- 更新完成
function T:updateCompleteHandler()
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_START,self,self.updateStartHandler)
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_PROGRESS,self,self.updateProgressHandler)
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_ITEM_START,self,self.updateItemStartHandler)
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_ITEM_COMPLETE,self,self.updateItemCompleteHandler)
	ResUpdate:luaRemoveEventListener(ResUpdate.EVT_TYPE_UPDATE_COMPLETE,self,self.updateCompleteHandler)

	self:updateVersion()
end

--- 更新当前version显示
function T:updateVersion()
	print(string.format("version:[%s]%s",CFG.CPP_VERSION,CFG:getVersion()))
	self.uiVersion:setString(Language:get("ug_version",CFG.CPP_VERSION,CFG:getVersion()))
end

function T:playLoadingEff(isFirst)
	local uiBack=self.uiBack
	local uiEffContainer=self.uiEffContainer

	if isFirst then
		local uiDian1=ResLoader:createAnimationSpriteRandom("dian",true)
		uiEffContainer:addChild(uiDian1)
		REL:toOther("center","middle",uiDian1,uiBack,-173,261)

		local uiDian2=ResLoader:createAnimationSpriteRandom("dian",true)
		uiDian2:setFlippedX(true)
		uiEffContainer:addChild(uiDian2)
		REL:toOther("center","middle",uiDian2,uiBack,196,261)
	end

	local uiXian1=ResLoader:createAnimationSpriteRunOnce("xian1")
	uiEffContainer:addChild(uiXian1)
	REL:toOther("center","middle",uiXian1,uiBack,-158,278)

	local uiXian2=ResLoader:createAnimationSpriteRunOnce("xian1")
	uiXian2:setFlippedX(true)
	uiEffContainer:addChild(uiXian2)
	REL:toOther("center","middle",uiXian2,uiBack,170,278)

	AbstractUI.timeout(self,function()
		local uiMiddleXian1=ResLoader:createAnimationSpriteRunOnce("xian2")
		uiEffContainer:addChild(uiMiddleXian1)
		REL:toOther("center","middle",uiMiddleXian1,uiBack,-13,272)

		local uiMiddleXian2=ResLoader:createAnimationSpriteRunOnce("xian2")
		uiMiddleXian2:setFlippedX(true)
		uiEffContainer:addChild(uiMiddleXian2)
		REL:toOther("center","middle",uiMiddleXian2,uiBack,16,272)
	end,0.6)

	AbstractUI.timeout(self,function()
		local uiBottomXian=ResLoader:createAnimationSpriteRunOnce("xian3")
		uiEffContainer:addChild(uiBottomXian)
		REL:toOther("center","middle",uiBottomXian,uiBack,-0,203)
	end,0.9)

	AbstractUI.timeout(self,function()
		local uiDeng1=ResLoader:createAnimationSpriteRunOnce("deng1")
		uiEffContainer:addChild(uiDeng1)
		REL:toOther("center","middle",uiDeng1,uiBack,-2,-22)
	end,1.2)

	AbstractUI.timeout(self,function()
		local uiDeng2=ResLoader:createAnimationSpriteRunOnce("deng2")
		uiEffContainer:addChild(uiDeng2)
		REL:toOther("center","middle",uiDeng2,uiBack,-0,-294)
	end,1.9)
end

--- 播放云彩特效
function T:playCloudEff()
	local uiCloudEffPlayer=require("src.app.uiScript.components.loading.CloudPlayer")()
	self.uiEffContainer:addChild(uiCloudEffPlayer)
	REL:relOther(uiCloudEffPlayer,self.uiBack,0,0,0,0)

	AbstractUI.interval(self,function()
		uiCloudEffPlayer:play()
	end,3)
	uiCloudEffPlayer:play()
end

--- 创建登陆视图
function T:createLoginView()
	local uiLogin=self.LoginView()
	self:addChild(uiLogin)
	REL:toScreen("center","middle",uiLogin,0,0)
	self.uiLogin=uiLogin
end

---加载公共资源
function T:loadCommonRes()
	ServerListMgr:luaRemoveEventListener(ServerListMgr.EVT_TYPE_GET_SERVER_LIST_COMPLETE,self,self.loadCommonRes)

	self.uiTip:setString(Language:get("ug16"))

	-- 资源相关配置
	require("src.app.ResConfig")

	print("加载公共资源")
	ResLoader:loadReses(CommonRes.preResList)

	self.uiProgressFore:setPercentage(0)
	local count=ResLoader:getRemaining()
	if count > 0 then
		self.loadItemCount=count

		ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_ITEM_LOAD_COMPLETE,self,self.itemLoadedHandler)
		ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,self,self.allLoadedHandler)
	else
		local curP=self.uiProgressFore:getPercentage()
		self.uiProgressFore:runAction(cc.ProgressFromTo:create(0.2,curP,100))
	end
end

function T:itemLoadedHandler(item)
	local count=self.loadItemCount

	local curP=self.uiProgressFore:getPercentage()
	self.uiProgressFore:runAction(cc.ProgressFromTo:create(0.2,curP,(count-ResLoader:getRemaining())/count*100))
end

function T:allLoadedHandler()
	ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_ITEM_LOAD_COMPLETE,self,self.itemLoadedHandler)
	ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,self,self.allLoadedHandler)

	self.uiProgressBar:runAction(cc.Sequence:create(
		cc.FadeOut:create(0.6),
		cc.CallFunc:create(function(t)
			t:removeFromParent()
		end)
	))

	-- 初始化UI相关的文件引入
	print("导入UI相关文件")
	require("src.app.UIRequires")

	-- 移除飞机小贴士
	self.uAirSmallStick:removeFromParent()


	self:createLoginView()

	CommonBroadcast:luaAddEventListener(NET_LOGIN_SUCC, self, self.loginSuccessHandler)
end

--- 登陆服务器成功
function T:loginSuccessHandler()
	CommonBroadcast:luaRemoveEventListener(NET_LOGIN_SUCC, self, self.loginSuccessHandler)

	self:enterMainScene()
end

--- 进入主场景
function T:enterMainScene()
	-- 发送获取所有世界聊天信息
	GetAllWorldChatCommand = require("src.app.communication.chat.GetAllWorldChatCommand")
	GetAllWorldChatCommand:new():exec( function(state)
		if state == ReturnCode.SUCCESS then
			print("获取聊天信息成功")
		else

		end
	end )

	-- 发送获取邮箱命令
	MailManager = require("src.app.model.mail.MailManager")()      -- 必须在这里初始化
	GetMailListCommand = require("src.app.communication.mail.GetMailListCommand")

	GetMailListCommand:new():exec( function(state)
		if state == ReturnCode.SUCCESS then
			print("获取邮箱成功")
			-- 邮箱检测
			-- MailManager = require "src.app.model.mail.MailManager"
			-- MailManager:startUpdateCheck()
		else

		end
	end )

	-- 进入主城
	SceneManager:switch(ScenesList.HOME_PAGE)
end

return T
