--Author    : YuDing
--Date      : 2014/11/10
--Introduce : 模拟的滑动容器

--- 模拟的滑动容器
--未实现边缘裁切的功能
-- 1种方案是: 固定的缩放，  不一样的速率
-- 2种方案是: 固定的速率，  不一样的缩放
local T = class("HomePageBaseSimulateScroll")

-- 滑动的框架
T.scroll = nil
-- 滑动的容器 container需要是scroll的直接子类， 东西的添加都需要在container中添加
T.container = nil

-- 四个方向的距离
T.top = nil
T.bottom = nil
T.left = nil
T.right = nil

-- 速率
T.rate = 1

T.max = 1.75 -- 放大的最大比 为了适应主界面
T.min = 0.7

--- 自动滑动 默认是不开启的
T._autoScroll = false

--- 自动滑动 增加的时间
T._autoScrollAddUpTime = 0

--- 加速度
T._autoScrollAcceleration = -1000

--- 最大速度
T.AUTOSCROLLMAXSPEED = 1000

--- 滑动 灵敏度
T.moveSensitive = 2.0

--- 点击 灵敏度 判断此次动作是点击 还是滑动 值取小一点 不然在边界的地方很容易引起bug
T.clickSensitive = 16.0

--- 左边界 事件
T.LEFT_EDGE_EVENT = "LEFT_EDGE_EVENT"
--- 右边界
T.RIGHT_EDGE_EVENT = "RIGHT_EDGE_EVENT"
--- 上边界
T.TOP_EDGE_EVENT = "TOP_EDGE_EVENT"
--- 下边界
T.BOTTOM_EDGE_EVENT = "BOTTOM_EDGE_EVENT"
--- 每一次 滑动之后 派发的事件
T.MOVING_AFTER_EVENT = "MOVING_AFTER_EVENT"
--- 滑动结束事件
T.MOVING_END_EVENT = "MOVING_END_EVENT"

--- 忽略 最后的滑动
T.ignoreElid = true

-- 构造函数
-- @param scroll  框架
-- @param container 包含内容的容器， 为scroll的直接子类
function T:ctor(scroll, container)
	mapping(self, LuaEventDispatcher) -- 继承 dispatcher

	self.scroll = scroll
	self.container = container
	self:startSlidUpdate()
end

--- 设置代理
function T:setScrollDelegete(scroll, container)
	self.scroll = scroll
	self.container = container
end

--- 根据container的坐标 更新四个方向
--- @param x 当前container的x坐标
--- @param y 当前container的y坐标
function T:updateDirection(x, y)
	-- 没有传入x, y参数， 那代表  container某些特性已经改变， 直接getPosition获取
	-- 传入了x, y代表还未改变， 先判断再改变
	if not x or not y then
		x, y = self.container:getPosition()
	end

	local cSize = self.container:getContentSize()

	local scaleX = self.container:getScaleX()
	local scaleY = self.container:getScaleY()
	cSize.width = scaleX * cSize.width       -- 因为这里有缩放的因子在里面
	cSize.height = scaleY * cSize.height

	local size = self.scroll:getContentSize()  -- scroll本身不会受scale的影响

	local enableX = true
	local enableY = true
	local left = 0 - x  -- 因为前面是left, bottom的判断， 所以不影响后面的实现
	if left <= 0 then
		left = 0
		enableX = false
	end
	self.left = left

	local bottom = 0 - y
	if bottom <= 0 then
		bottom = 0
		enableX = false
	end
	self.bottom = bottom

	local right = cSize.width - self.left - size.width
	if right <= 0 then
		right = 0
		self.left = cSize.width - size.width -- 更新
		enableY = false
	end
	self.right = right -- 如果没有进入到if里面 就表示 scroll 在 container 的里面

	local up = cSize.height - self.bottom - size.height
	if up <= 0 then
		up = 0
		self.bottom = cSize.height - size.height
		enableY = false
	end
	self.up = up

	self.container:setPosition(cc.p(0 - self.left, 0 - self.bottom)) -- 因为相对于scroll来说， container的坐标总是负数

	if enableX == false and enableY == false then  -- 两边都要 到达头
		return false
	else
		return true
	end

end

--- 获得缩放的比例
function T:getScrollScale()
	return self.container:getScale()
end

--- 获得滑动的容器
function T:getContainer()
	return self.container
end

--- 获得滑动容器的位置
function T:getContainerPosition()
	return self.container:getPosition()
end

--- 设置 滑动容器的位置
function T:setContainerPosition(x, y)
	self.container:setPosition(cc.p(x, y))
end

function T:getScroll()
	return self.scroll
end

--- 获得内部容器
function T:getInnerContainer()
	return self.container
end

--- 移动的距离
function T:getMoveTouchOffset()
	return self.touchOffset
end

function T:setMoveTouchOffset(offset)
	self.touchOffset = offset
end

--- 获得scroll 和 container的长度和高度的差距
function T:getDistance()
	local cSize = self.container:getContentSize()

	local scale = self.container:getScale()
	cSize.width = scale * cSize.width       -- 因为这里有缩放的因子在里面
	cSize.height = scale * cSize.height

	local sSize = self.scroll:getContentSize()

	return cSize.width - sSize.width, cSize.height - sSize.height
end

--- 获得 x 轴的距离差量
function T:getXDistance()
	local s = self.container:getContentSize()
	local cw = s.width
	s = self.scroll:getContentSize()
	local w = s.width

	local scale = self.container:getScale()

	local px = self.container:getPositionX()


	return math.abs(px), math.abs(cw * scale - math.abs(px) - w)                          -- 左边, 右边
end

--- 获得x轴 偏移
-- @return px container 距离scroll的距离
-- @return cSize.width - sSize.width x轴 container和scroll的宽度差
function T:getLeftXDistance()
	local cSize = self.container:getContentSize()

	local scale = self.container:getScale()
	cSize.width = scale * cSize.width       -- 因为这里有缩放的因子在里面

	local sSize = self.scroll:getContentSize()

	local px, py = self.container:getPosition()

	return math.abs(px), cSize.width - sSize.width
end

function T:getMaxScale()
	return self.max
end

function T:getMinScale()
	return self.min
end

--- 设置缩放率
function T:setScaleWithOutDistance(scale)
	local c = self.container
	c:setScale(scale)
end

--- 设置滑动速率， 1种方案是: 固定的缩放，  不一样的速率
function T:setScrollRate(rateX, rateY)
	self.rateX = rateX
	self.rateY = rateY
end

--- 得到滑动速率
function T:getScrollRate()
	return self.rateX or 1, self.rateY or 1
end

--- 设置容器里面的比例
-- @param scale 增量
function T:setScrollScale(scale, mid)
	local c = self.container

	if scale > 0 then
		if c:getScale() == self.max then
			return 0
		end
	end

	if scale < 0 then
		if c:getScale() == self.min then
			return 0
		end
	end

	scale = c:getScale() + scale

	if scale >= self.max then
		scale = self.max
	end

	if scale <= self.min then
		scale = self.min
	end

	--    local size=c:getContentSize()

	--    if size.width * scale < UI_VSIZE.width then  -- 宽度缩小到最小宽度
	--        scale = UI_VSIZE.width / size.width
	--    end

	--    if size.height * scale < UI_VSIZE.height then -- 高度缩小到最小高度
	--       scale = UI_VSIZE.height / size.height
	--    end

	if mid then
		local scale = c:getScale()
		local size = c:getContentSize()
		local new = cc.size(size.width * scale, size.height * scale)
		local anchor = cc.p(mid.x / new.width, mid.y / new.height)
		c:setAnchorPoint(anchor)
	end

	local oScale = c:getScale()
	c:setScale(scale)

	if mid then
		c:setAnchorPoint(cc.p(0, 0))
	end

	self:updateDirection()

	return scale - oScale  -- 变化的scale
end

--function T:setScaleWidthDistance(dx, dy, scale)
--    local c = self.container
--
--    local cx, cy = self:getDistance()
--
----    local scaleX = cx / dx * scale
----    local scaleY = cy / dy * scale
--
----    scaleX = c:getScaleX() + scaleX
----    scaleY = c:getScaleY() + scaleY
--
----    c:setScale(scaleX, scaleY)
--
--
--    self:updateDirection()
--
--end

--- x, y 增量
function T:addScrollPosition(ax, ay)
	local px, py = self.container:getPosition()
	local x = px + ax * self.rate
	local y = py + ay * self.rate



	-- 更新
	return self:updateDirection(x, y)

end

--- 设置最小的变化率
function T:setMinScale(min)
	self.min = min
end

--- 设置最大的变化率
function T:setMaxScale(max)
	self.max = max
end

--- 自动滑动
function T:autoScrollChildren(dt)
	local dt = dt / 1000
	local lastTime = self._autoScrollAddUpTime
	self._autoScrollAddUpTime = self._autoScrollAddUpTime + dt
	local nowSpeed = self._autoScrollOriginalSpeed + self._autoScrollAcceleration * self._autoScrollAddUpTime
	--print("nowSpeed is " .. nowSpeed)
	--print("_autoScrollOriginalSpeed is   " .. self._autoScrollOriginalSpeed)

	if nowSpeed <= 0.0 then  -- 结束偏移
		self:stopAutoScrollChildren()
		self:execMoveEndEvent()  -- 结束运动事件

	else
		local timeParam = lastTime * 2 + dt
		local offset = (self._autoScrollOriginalSpeed + self._autoScrollAcceleration * timeParam * 0.5) * dt
		local offsetX = offset * self._autoScrollDir.x
		local offsetY = offset * self._autoScrollDir.y

		--print("offsetx is " .. offsetX)
		--print("offsety is " .. offsetY)

		local mx, my = self:getDistance()

		local d = self:checkEdge(offsetX, offsetY)

		if self._onTouchMoved then -- 存在 普通的move事件
			--            if not ((d.l == true or d.r == true) and (d.t == true or d.b == true)) then  -- 两个没同时到达边界
			--                self._onTouchMoved(d)
			--            end
			self._onTouchMoved(d, cc.p(offsetX, offsetY))
		end


		if not self:addScrollPosition(offsetX, offsetY) then  -- 到达末尾
			self:stopAutoScrollChildren()
		else -- 在滑动
			if self.moveCallBack then  -- 有额外的 惯性回调
				if self.moveCallBackTarget then  -- 是一个self:func()
					self.moveCallBack(self.moveCallBackTarget, offsetX, offsetY, mx, my)
			else  -- 是一个 func() 类型 闭包
				self.moveCallBack(offsetX, offsetY, mx, my)
			end
		end

		end

		if self._moveAfterEvent then  -- 运动之后的事件
			self._moveAfterEvent(cc.p(offsetX, offsetY))
		end

	end

end

--- 停止滑动
function T:stopAutoScrollChildren()
	self._autoScroll = false
	self._autoScrollOriginalSpeed = 0.0
	self._autoScrollAddUpTime = 0.0
end

--- 滑动 更新
--  每一次的间隔大概在15-20ms之间
function T:slidUpdate()
	local sys = TimeUtils:getSyncMillis()
	--print("the distance is " .. sys - self.lastSysTime)

	self._slidTime = sys - self.lastSysTime

	if self._autoScroll then
		self:autoScrollChildren(sys - self.lastSysTime)
	end

	self.lastSysTime = TimeUtils:getSyncMillis()
	--self:recordSlidTime(dt)
end

--- 开启定时器
function T:startSlidUpdate()
	self.lastSysTime = TimeUtils:getSyncMillis()

	local func = function()
		self:slidUpdate()
	end

	Cocos2dxUtils:interval(func, 0) -- 每一帧都调用
end

---  滑动
function T:startRecordSlidAction()
	--    if self._slidTime <= 0.016 then  -- 大于这个时间 姑且认为是大于1帧的时间 待定 这里
	--        self._delta = cc.p(0, 0)
	--        return
	--    end

	--local subVector =  cc.pSub(self._touchEndedPoint, self._touchBeganPoint) -- 距离

	--local totalDis = cc.pGetLength(subVector) -- 起点到终点

	if self.ignoreElid == false then  -- 被忽略
		return
	end


	if not self._delta then
		self._delta = cc.p(0, 0)
	end

	local dis = cc.pGetLength(self._delta) -- 最后的 两点距离

	--print("move dis: " .. dis)
	if dis >= self.moveSensitive then
		local time = self._slidTime / 1000
		local orSpeed = math.min(dis / time, self.AUTOSCROLLMAXSPEED);
		--print("self._slidTime is " .. time)
		--print("orSpeed is  " .. orSpeed)
		self:startAutoScrollChildrenWithOriginalSpeed(cc.pNormalize(self._delta), orSpeed)
		self._slidTime = 0.0
	else -- 这里没有到最后的偏移 也算结束
		self:execMoveEndEvent()
	end

	self._delta = cc.p(0, 0)
end

--- 开始滑动
function T:startAutoScrollChildrenWithOriginalSpeed(dir, v)
	self:stopAutoScrollChildren()

	self._autoScrollOriginalSpeed = v
	self._autoScroll = true
	self._autoScrollDir = dir
end

---  滑动
function T:endRecordSlidAction()
	if self._autoScroll then
		self:stopAutoScrollChildren()
	end

	self._slidTime = 0.0
end

function T:setBeginPoint(p)
	self._touchBeganPoint = p
end

function T:setEndPoint(p)
	self._touchEndedPoint = p
end

function T:setDelta(d)
	self._delta = d
end

--- 设置加速度 为负值
function T:setAcceleration(num)
	if num >= 0 then
		num = -1000
	end

	self._autoScrollAcceleration = num
end

--- 注册 自动滑动 事件
function T:registerAutoElidEvent(e, t)
	if e then
		self.moveCallBack = e
		self.moveCallBackTarget = t
	end
end

--- 退出时调用
function T:exitHandler(target)
	if CppCocos2dxUtils.getTouchedTarget then
		if CppCocos2dxUtils:getTouchedTarget() == target then
			CppCocos2dxUtils:setTouchedTarget(nil)
		end
	else
		self.touchedTarget=nil
	end
end

function T:registerTouchEvent(onTouchBegan, onTouchMoved, onTouchCanceled, onTouchEnded)

	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	local funcBegin = function(t, e)
		if CppCocos2dxUtils.getTouchedTarget then
			if CppCocos2dxUtils:getTouchedTarget() then
				return true
			end
			CppCocos2dxUtils:setTouchedTarget(self.scroll)
		else
			if self.touchedTarget then
				return true
			end

			self.touchedTarget=t
		end
		
		self:onTouchBegan(t, e)
		return true
	end
	local funcMoved = function(t, e)
		self:onTouchMoved(t, e)
	end
	local funcCanceled = function(t, e)
		if CppCocos2dxUtils.getTouchedTarget then
			if CppCocos2dxUtils:getTouchedTarget() ~= self.scroll then
				return
			end
			CppCocos2dxUtils:setTouchedTarget(nil)
		else
			self.touchedTarget=nil
		end

		self:onTouchCanceled(t, e)
	end
	local funcEnded = function(t, e)
		if CppCocos2dxUtils.getTouchedTarget then
			if CppCocos2dxUtils:getTouchedTarget() ~= self.scroll then
				return
			end
			CppCocos2dxUtils:setTouchedTarget(nil)
		else
			if self.touchedTarget ~= t then
				return
			end
			self.touchedTarget=nil
		end

		self:onTouchEnded(t, e)
	end

	listener:registerScriptHandler(funcBegin, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(funcMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(funcEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(funcCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)

	local eventDispatcher = self.scroll:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self.scroll)

	self._onTouchBegan = onTouchBegan
	self._onTouchMoved = onTouchMoved
	self._onTouchCanceled = onTouchCanceled
	self._onTouchEnded = onTouchEnded
end

--- 运动之后的事件
function T:registerMoveAfterEvent(func)
	self._moveAfterEvent = func
end

function T:releaseMoveAfterEvent()
	self._moveAfterEvent = nil
end

--- 滑动结束的事件
function T:registerMoveEndEvent(func)
	self._moveEndEvent = func
end

function T:releaseMoveEndEvent()
	self._moveEndEvent = nil
end

--- 执行
function T:execMoveEndEvent()
	if self._moveEndEvent then
		self._moveEndEvent()
	end
end

--- 先进行父类的函数
function T:onTouchBegan(t, e)
	if self._autoScroll then
		self.needFlush = true
	else
		self.needFlush = false
	end

	self:endRecordSlidAction() -- 停止滑动
	self._onTouchBegan(t, e)
end

function T:onTouchMoved(t, e)
	self._delta = t:getDelta()

	local d = self:checkEdge(self._delta.x, self._delta.y)    -- 需要在滑动之前判断派发边界事件
	self._onTouchMoved(d, self._delta)  -- 调用 注册的事件  因为在这里 会进行图片的一些翻转操作

	self:addScrollPosition(self._delta.x, self._delta.y)

	if self._moveAfterEvent then
		self._moveAfterEvent(self._delta)
	end

end

function T:onTouchEnded(t, e)
	--self._delta = t:getDelta() -- 获取到最后一次的距离 总是为0

	local bp = t:getStartLocation() -- 开始的点
	local ep = t:getLocation()
	local px = math.abs(bp.x - ep.x)
	local py = math.abs(bp.y - ep.y)
	local click = false

	if px < self.clickSensitive and py < self.clickSensitive then
		click = true
		if self.needFlush == true then
			self:execMoveEndEvent()
		end
	end

	self._onTouchEnded(click, bp)

	if click == false then -- 不是点击 是滑动
		self:startRecordSlidAction() -- 开始加速度滑动
		if self.ignoreElid == false then -- 偏移被忽略
			self:execMoveEndEvent()
		end
	end
end

function T:onTouchCanceled(t, e)
	--self._lastDelta = t:getDelta()

	self:startRecordSlidAction() -- 开始滑动

	self._onTouchCanceled()
end

--- 检查边界
function T:checkEdge(offx, offy)
	local px, py = self.container:getPosition()
	local cSize = self.container:getContentSize()
	local size = self.scroll:getContentSize()

	local scale = self.container:getScale()

	cSize = cc.size(cSize.width * scale, cSize.height * scale)

	px = px + offx
	py = py + offy

	local _l, _r, _t, _b = false, false, false, false  -- 四个方向
	if px >= 0 then  -- 右滑动 即将达到左边界
		--print("即将达到左边界")
		--self:luaDispatchEvent(self.LEFT_EDGE_EVENT)
		_l = true
	end

	if px <= -(cSize.width - size.width) then -- 左滑动 即将达到右边界
		--print("即将达到右边界")
		--self:luaDispatchEvent(self.RIGHT_EDGE_EVENT)
		_r = true
	end

	if py >= 0 then -- 上滑动 即将到达下边界
		-- print("即将到达下边界")
		--self:luaDispatchEvent(self.BOTTOM_EDGE_EVENT)
		_b = true
	end

	if py <= -(cSize.height - size.height) then -- 下滑动 即将达到上边界
		-- print("即将达到上边界")
		--self:luaDispatchEvent(self.TOP_EDGE_EVENT)
		_t = true
	end

	return {l = _l, r = _r, t = _t, b = _b}
end

--- 忽略  最后的滑行 默认是关闭的
-- false 不 忽略滑动
-- true  忽略 滑动
function T:setIgnoreElid(v)
	self.ignoreElid = v
end

function T:releaseAutoElidEvent()
	self.moveCallBack = nil
	self.moveCallBackTarget = nil
end


return T
