-- Author    : YuDing
-- Date      : 2014/10/23
-- Introduce : 主界面管理类

local T = Class.HomePageManager(LuaEventDispatcher2)

-- 基地名字显示时间
T.nameShowTime = 0.5

--- 世界地图一些配置数据
T.wmCfg = require("src.app.uiScript.scenes.homePage.world.HomePageWorldConfig")
--- 野外的一些配置数据
T.outCfg = require("src.app.uiScript.scenes.homePage.outSide.HomePageOutSideConfig")

T.noticeMessage = {
	switchBase = "switchBase", -- 切换到主基地
	switchOutSide = "switchOutSide", -- 切换到野外
	switchWorld = "switchWorld", -- 切换到世界
	openWindow = "openWindow", -- 打开窗口
	worldMoving = "worldMoving", -- 世界地图移动
	showSmallMap = "showSmallMap", -- 显示小地图

	scaleAddEffect = "scaleAddEffect", -- 主界面放大事件
	scaleReduceEffect = "scaleReduceEffect", -- 主界面缩小事件

	cloudMove = "cloudMove", -- 世界地图 云动的事件

	enterEffectComplete = "enterEffectComplete", -- 完成 进入特效

	enterEffectMainStage = "enterEffectMainState", -- 主界面 进入特效阶段
}

T.preResList = {}

function T:def()
	LuaEventDispatcher2.def(self)
	--- 代理
	self.delegete = nil

	--- 世界地图
	self.world = nil

	-- 界面点击标示
	self.viewIndex = 1
	-- 基地
	self.base = nil
	-- 郊外
	self.outSide = nil
	-- 世界
	self.world = nil

	-- 当前显示的界面
	self.currentViewTb = { }

	-- 坐标
	self.currentX = nil
	self.currentY = nil

	-- 地图数据相关
	self.mapData = { }
	-- 监测碎片地图数据是否缓存
	self.mapData.checkList = { }
	-- 获取地图数据时， 存碎片地图区域的表
	self.mapData.posLists = { }
	-- 地图数据
	self.mapData.map = { }

	self.isFirstEnterWorld = nil

	-- 世界地图收藏
	self.worldCollection = { }
	self.firstEnterWorldCollection = false
	-- 最大收集
	self.maxCollection = 30
	--- 世界地图三级开放
	self.worldOpenLvl = 3

	--- 打开建筑时产生的触摸层（避免重复点击）
	self.bldMask = nil
end

--- 构造函数
function T:ctor()
	mapping(self, LuaEventDispatcher)
	local handler = self.registerScriptHandler
	self:loadRes()
end

function T:loadRes()
	ResLoader:syncLoad(self.preResList)
end

function T:cleanUp()
	ResLoader:unloadReses(self.preResList)

	if self.base then
		self.base:cleanUpAll()
		self.base:release()
		self.base=nil
	end

	--    if self.outSide then
	--        self.outSide:release()
	--    end

	if self.world then
		self.world:setTag(23560)
		print("调试:",self.world:getTag())

		self.world:release()
		self.world=nil
	end

	if self.bottomBar then
		self.bottomBar:releaseEvent()
	end

	if self.topBar then
		self.topBar:cleanup()
		self.topBar:release()
		self.topBar=nil
	end

	if self.topWorldBar then
		self.topWorldBar:release()
		self.topWorldBar=nil
	end

	self.leftBar = nil
	self.rightBar = nil

end

--- 设置代理
function T:setDelegete(d)
	self.delegete = d
end

--- 得到代理
function T:getDelegete()
	return self.delegete
end

--- 初始化
function T:init()
	print("__________________________HomePageManager:init____________________________")

	local delegete = self.delegete
	local container = delegete:getContainer()

	-- 基地
	self.base = require("src.app.uiScript.scenes.homePage.base.HomePageBase").new()
	self.base:retain()
	self.base:setManager(self)
	self.base:initView()

	-- 下方工具条
	self.bottomBar = require("src.app.uiScript.scenes.homePage.bar.HomePageBottomBar").new()
	self.bottomBar:setManager(self)
	self.bottomBar:initView()
	container:addChild(self.bottomBar)
	-- 加入到homescene里面
	self.bottomBar:setLocalZOrder(100)

	-- 基地顶部条
	self.topBar = require("src.app.uiScript.scenes.homePage.bar.HomePageTopBar")()
	self.topBar:retain()
	self.topBar:setManager(self)
	self.topBar:initView()
	REL:toScreen("center","top",self.topBar,0,0)
	self.topBar:setLocalZOrder(100)

	--	-- 世界地图顶部条
	--	self.topWorldBar = require("src.app.uiScript.scenes.homePage.bar.HomePageTopWorldBar").new()
	--	self.topWorldBar:retain()
	--	self.topWorldBar:setManager(self)
	--	self.topWorldBar:initView()
	--	-- delegete:addChild(self.topWorldBar)   -- 加入到homescene里面
	--	self.topWorldBar:setPositionY(UI_VSIZE.height - self.topWorldBar:getHeight())
	--	self.topWorldBar:setLocalZOrder(100)

	local uiTopBar=require("src/app/uiScript/scenes/homePage/bar/WorldMapTopBar.lua")()
	uiTopBar:retain()
	uiTopBar:setManager(self)
	uiTopBar:setLocalZOrder(100)
	REL:toScreen("center","top",uiTopBar,0,0)
	self.topWorldBar=uiTopBar

	-- 左边工具条
	self.leftBar = require("src.app.uiScript.scenes.homePage.bar.HomePageLeftBar").new()
	self.leftBar:initView(container)
	container:addChild(self.leftBar)
	self.leftBar:setLocalZOrder(100)

	-- 右边工具条
	self.rightBar = require("src.app.uiScript.scenes.homePage.bar.HomePageRightBar").new()
	self.rightBar:initView(container)
	container:addChild(self.rightBar)
	self.rightBar:setLocalZOrder(100)

	self:initMapConnect()
	-- 初始化世界地图数据

	--- 派发进入主场景事件
	CommonBroadcast:luaDispatchEvent(MAINSCENE_ENTER)
end

--- 创建郊外
function T:createOutSide()
	-- 郊外
	if self.outSide then
		return
	end

	local container = self.delegete:getContainer()

	xpcall(
		function()
			self.outSide = require("src.app.uiScript.scenes.homePage.outSide.HomePageOutSide").new()
			self.outSide:initView()
			container:addChild(self.outSide)
		end,
		function(errStr)
			if self.outSide then
				self.outSide:removeFromParent()
			end

			local message=tostring(errStr).."\n"..debug.traceback()

			__print__("InitOutSideErr",message)
			LogRecorder:push(message,LOG_LEVEL_ERROR,"InitOutSideErr")
		end
	)
end

--- 移除郊外
function T:removeOutSide()
	if self.outSide then
		self.outSide:removeFromParent()
		self.outSide = nil
	end
end

--- 创建世界地图
function T:createWorld()
	local container = self.delegete:getContainer()

	if not self.world then
		self.world = require("src.app.uiScript.components.worldMap.WorldMapView")()
		self.world:retain()

		container:addChild(self.world)
		self.world:gotoPosition(PlayerInstance:getWorldPosition())
	else
		if not self.world:getParent() then
			container:addChild(self.world)
		end
	end
end

--- 移除世界地图
function T:removeWorld()
	local container = self.delegete:getContainer()

	if self.world and self.world:getParent() then
		container:removeChild(self.world, false)
	end
end

--- 基地名字显示时间
function T:getNameShowTime()
	return self.nameShowTime
end

--- 设置主基地是否能滑动
-- @param able false不能滑动 true能滑动
function T:setHomeBaseMoveEable(able)
	if self.base then
		self.base:setHomeBaseMoveEnable(able)
	end
end

--- 设置郊外是否能滑动
-- @param able false不能滑动 true能滑动
function T:setOutSideMoveEable(able)
	if self.outSide then
		self.outSide:setHomeBaseMoveEnable(able)
	end
end

--- 设置底部条是否能滑动
-- @param able false不能滑动 true能滑动
function T:setBottomBarMoveEable(able)
	if self.bottomBar then
		self.bottomBar:setMoveEable(able)
	end
end


--- 设置主基地滑动
-- @param ax,ay 为增量
function T:setHomeBaseMove(ax, ay)
	if self.base then
		self.base:excutMove(ax, ay)
	end
end

--- 设置郊外滑动
-- @param ax, ay 为增量
function T:setHomeOutSideMove(ax, ay)
	if self.outSide then
		self.outSide:excutMove(ax, ay)
	end
end

--- 增加打开建筑时的触摸层(避免多次点击)
function T:addOpenBldMask(isPatch)
	if self.bldMask then
		self.bldMask:removeFromParent()
		self.bldMask = nil
	end

	if not self.bldMask then
		self.bldMask = ccui.Layout:create()
		self.bldMask:setTouchEnabled(true)
		self.bldMask:setLocalZOrder(200)
		self.bldMask:setContentSize(UI_VSIZE)
		local container = self.delegete:getContainer()
		container:addChild(self.bldMask)
	end

	self:luaAddEventListener(self.noticeMessage.openWindow, self, self.removeOpenBldMask) -- 添加删除遮罩的事件
end
--- 移除点击的遮罩层
function T:removeOpenBldMask()
	if self.bldMask then
		self.bldMask:removeFromParent()
		self.bldMask = nil

		WindowManager:getCurrentSceneMgr():luaRemoveEventListener(EVENT_START_OPEN, self, self.removeOpenBldMask) -- 添加删除遮罩的事件

	end
end

--- 设置默认的界面
function T:setDefaultView()
	local delegete = self.delegete
	local container = delegete:getContainer()
	container:addChild(self.base)
	container:addChild(self.topBar)

end

--- 主场景刚开始时  特效的时间
function T:setStartEffectTime(t)
	self.startEffectTime = t
end

function T:getStartEffectTime()
	return self.startEffectTime
end

--- 顺序切换
function T:switchView()
	local delegete = self.delegete
	local container = delegete:getContainer()

	if self.viewIndex == 1 then
		self:switchViewToWorld()
	elseif self.viewIndex == 3 then
		self:switchViewToBase()
	end

	return self.viewIndex
end

--- 进入到主界面
function T:switchViewToBase()
	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_HOMEPAGE_SWITCH_BUTTON)
	end

	MUIAlter_singleLine.show({
		info=Language:get("uc822"),
		showSecond=1
	})

	local delegete = self.delegete
	local container = delegete:getContainer()
	if self.viewIndex == 1 then return end -- 进入当前

	if self.viewIndex == 2 then  -- 从郊外到基地
		self:removeOutSide()
		container:addChild(self.base)
	elseif self.viewIndex == 3 then -- 从世界到基地
		self:removeWorld()

		container:removeChild(self.topWorldBar, false)

		if not self.base:getParent() then
			container:addChild(self.base)
		end
		if not self.topBar:getParent() then
			container:addChild(self.topBar)
		end
	end

	self.viewIndex = 1

	self:luaDispatchEvent(self.noticeMessage.switchBase) -- 派发事件
end

--- 进入到郊外
function T:switchViewToOutSide()
	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_HOMEPAGE_OUTSIDE)
	end

	MUIAlter_singleLine.show({
		info=Language:get("uc823"),
		showSecond=1
	})

	local delegete = self.delegete
	local container = delegete:getContainer()

	if self.viewIndex == 2 then return end -- 进入当前

	if self.viewIndex == 1 then  -- 从主基地到郊外
		container:removeChild(self.base, false)
		self:createOutSide()
	elseif self.viewIndex == 3 then -- 从世界到郊外
		self:removeWorld()

		container:removeChild(self.topWorldBar, false)
		self:createOutSide()

		container:addChild(self.topBar)
	end

	self.viewIndex = 2
	self:luaDispatchEvent(self.noticeMessage.switchOutSide) -- 派发事件
end
--- 进入到世界
function T:switchViewToWorld()
	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_HOMEPAGE_SWITCH_BUTTON)
	end

	MUIAlter_singleLine.show({
		info=Language:get("uc821"),
		showSecond=1
	})

	local delegete = self.delegete
	local container = delegete:getContainer()

	if self.viewIndex == 3 then return end -- 进入当前

	if self.viewIndex == 1 then
		container:removeChild(self.base, false)  -- 移除主界面
		self:createWorld()

		container:removeChild(self.topBar, false)
		container:addChild(self.topWorldBar)

	elseif self.viewIndex == 2 then
		self:removeOutSide()
		self:createWorld()

		container:removeChild(self.topBar, false)
		container:addChild(self.topWorldBar)
	end

	self.viewIndex = 3

	self:luaDispatchEvent(self.noticeMessage.switchWorld) -- 派发事件
end

--- 增加特效
-- @param 增加的对象
-- @kind 对象是哪一种种类
-- @wr withoutRandom 不要随机 默认是随机的
function T:addEffect(t, kind, wr)
	local eff

	-- 名字或者是sid
	if kind == "oil" or kind == 8 then
		eff = self:getEff("oilEffect")
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, -14, 24)
	elseif kind == "steel" or kind == 7 then
		eff = self:getEff("steelEffect")
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, 10, -27)
	elseif kind == "silicon" or kind == 9 then
		eff = self:getEff("siliconEffect")
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, 0, 0)

	elseif kind == "tati" or kind == 10 then
		eff = self:getEff("tatiEffect")
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, 16, -3)

	elseif kind == "transfer" then
		eff = self:getEff("transferEffect")
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, 5, 15)
	elseif kind == "smooth" then -- 溜光
		eff = self:getEff("smoothEffect", 3) -- 3秒一次
		eff:setScale(0.95)
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, 0, -3)
	elseif kind == "buttonPolishingEff" then  -- 有阴影的按钮特效, 1秒一次
		eff = self:getEff("buttonPolishingEff", 1)
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, -4, 12)
	elseif kind == "rolePhotoTipEff" then   -- 主界面提示 可升级统率
		eff = self:getEff("rolePhotoTipEff", 1)
		t:addChild(eff)
		CCUIREL:toOther("center", "middle", eff, t, 0, 0)
	elseif kind == "button6TipEff" then  -- 可升级的 按钮提示 6边形的
		if wr == true then -- 不要随机
			eff = self:getEffWithOutRandom("button6TipEff", 2)
	else
		eff = self:getEff("button6TipEff", 1)
	end

	t:addChild(eff)
	CCUIREL:toOther("center", "middle", eff, t, -2, 10)
	end

	return eff
end

--- 更新坐标
function T:updateCoordinate(x, y)
	self.currentX = x
	self.currentY = y

	self.topWorldBar:setCurrentPos(x,y)
	self:luaDispatchEvent(self.noticeMessage.worldMoving, x, y)
end

function T:getCurrentCoordinate()
	return self.currentX, self.currentY
end

--- 世界地图是否开放
function T:isWorldOpen()
	local lvl = PlayerInstance:getPlayerLevel()
	if lvl >= self.worldOpenLvl then
		return true
	else
		return false
	end
end

--- 播放点击声音
function T:playClickSound()
	AudioManager:playEffect(SoundRes.SOUND_COMMON_BUTTON)
end

--- 跳转到世界位置
function T:gotoWorldPosition(x, y)
	if not self.world then
		return
	end

	self.world:gotoPosition(x, y)
end
--- 跳转到当前位置
function T:gotoCurrentWorld()
	self.world:gotoPosition(self.currentX, self.currentY)
end

--- 切换世界文字信息
function T:switchWorldText()
	self.world:switchText()
end

function T:switchSmallMap()
	self:luaDispatchEvent(self.noticeMessage.showSmallMap)
end

--- 一张碎片图最大的cell x个数
function T:getWorldMaxCellX()
	return self.wmCfg.cXN
end

--- 一张碎片图最大的cell y个数
function T:getWorldMaxCellY()
	return self.wmCfg.cYN
end

--- 检查缓存里是否有数据
function T:checkMapDataCache(ax, ay)
	local key = ax .. "," .. ay

	local status,curSec=nil,TimeUtils:getSyncMillis() / 1000
	if not self.mapData.checkList[key] then
		self.mapData.checkList[key] = curSec
		status = true
	elseif self.mapData.checkList[key] > 1 then
		if curSec - self.mapData.checkList[key] >= 100 then
			-- 大于100s就重新获取一次数据
			self.mapData.checkList[key] = curSec
			status = true
		end
	end

	if status then
		table.insert(self.mapData.posLists, { x = ax, y = ay })
		self:requestServerMapData()

		return false
			-- 没数据或者是数据已过期
	else
		return true
			-- 数据有且未过期
	end
end

function T:checkManyMapDataCache(tb)
	-- self:luaDispatchEvent("mapUpdate", self)  -- 观察
	-- 先刷新数据

	local status, curSec = nil, TimeUtils:getSyncMillis() / 1000

	for _, v in ipairs(tb) do
		-- v[1] = x v[2] = y
		local ax = v[1]
		local ay = v[2]
		local key = v[1] .. "," .. v[2]
		local insert = false
		-- 是否可插入
		if not self.mapData.checkList[key] then
			self.mapData.checkList[key] = curSec
			status = true
			insert = true
		elseif self.mapData.checkList[key] > 1 then
			if curSec - self.mapData.checkList[key] >= 100 then
				-- 大于100s就重新获取一次数据
				self.mapData.checkList[key] = curSec
				status = true
				insert = true
			end
		end

		if status == true and insert == true then
			-- 同时满足
			table.insert(self.mapData.posLists, { x = ax, y = ay })
		end

	end

	-- 请求数据
	if status == true and #self.mapData.posLists >= 1 then
		self:requestServerMapData()
	else -- 刷新更新
		self:luaDispatchEvent("mapUpdate", self)
	end

end

--- 检查缓存中的世界地图数据
function T:checkWorldDataInMap(ax, ay)
	local key = ax .. "," .. ay
	local status,curSec=nil,TimeUtils:getSyncMillis() / 1000
	if not self.mapData.checkList[key] then
		self.mapData.checkList[key] = curSec
		status = true
	elseif self.mapData.checkList[key] > 1 then
		if curSec - self.mapData.checkList[key] >= 100 then
			-- 大于100s就重新获取一次数据
			self.mapData.checkList[key] = curSec
			status = true
		end
	end
	if status == true then
		return false
			-- 数据过期或者是没有数据
	else
		return true
			-- 有数据
	end

end

--- 请求一块碎地图的数据
function T:requestServerMapData()
	local cfg = self.wmCfg

	local maxx,maxy=cfg.horizonMWidth + 1,cfg.verticalMheight + 1
	local sx,sy,ex,ey=maxx,maxy,1,1

	-- posLists 保存有地图区域坐标
	for k, v in pairs(self.mapData.posLists) do
		local x,y=v.x,v.y

		if x < sx then
			sx=x
		end
		if y < sy then
			sy=y
		end

		if x > ex then
			ex=x
		end
		if y > ey then
			ey=y
		end
	end

	ex = ex + cfg.cXN - 1
	-- 请求的是一个范围数据
	ey = ey + cfg.cYN - 1

	self.mapData.posLists = { }
	if ex >= maxx - 1 then
		ex = maxx
	end
	if ey >= maxy - 1 then
		ey = maxy
	end
end

--- 缓存地图数据
--    map 里面保存的是 x,y 对应点的
--    mapnode 里面含有的元素 x, y , rType, lvl, npcType
--    1,2,3,4,5分别代表金矿，铁矿，石油，硅矿，钛矿
function T:cacheMapData(data, sp, ep)
	local mapSizeX = self.wmCfg.cXN
	local mapSizeY = self.wmCfg.cYN
	local map = self.mapData.map

	for i = 1, #data do
		local tag = data[i].x .. "," .. data[i].y
		map[tag] = data[i]
	end

	for x = sp.x, ep.x - mapSizeX, mapSizeX do
		for y = sp.y, ep.y - mapSizeY, mapSizeY do
			self:signMapCache(x, y)
			-- 对应的地图块
		end
	end
	-- self:dispatchListener(Event.new(MAP_UPDATA))
	self:luaDispatchEvent("mapUpdate", self)
end

--- 刷新单个数据节点数据
function T:updataTheMapNode(x, y, mapnode)
	self.mapData.map[x .. "," .. y] = mapnode
	if mapnode then  -- 如果有数据
		mapnode.x = x -- 将节点内的数据改变
		mapnode.y = y
	end
	self:luaDispatchEvent("mapUpdateNode", x, y)
end

--- 标记区域已被缓存
function T:signMapCache(x, y)
	local curSec = TimeUtils:getSyncMillis() / 1000

	print("signMapaCache : " .. x .. "," .. y .. "curSec = " .. curSec)

	self.mapData.checkList[x .. "," .. y] = curSec
	-- 写入当前的时间
end

function T:checkSignDataOutTime(x, y)

end

--- 获取地图缓存
function T:getMapCache()
	return self.mapData.map
end
--- 清楚地图数据
function T:cleanMapData()
	self.mapData.map = {}
	self.mapData.checkList = {}
	self.mapData.posLists = {}
end

--- 初始化请求大地图数据
function T:initMapConnect()
	local px, py = PlayerInstance:getWorldPosition()
	self.currentX = px
	-- 初始化当前的坐标
	self.currentY = py
end

--- 增加
function T:addWorldCollection(info)
	local index = info.x .. "," .. info.y
	self.worldCollection[index] = info
end

--- 检查是否有收集
function T:checkWorldCollection(x, y)
	local index = x .. "," .. y
	if self.worldCollection[index] then
		return true
	else
		return false
	end

end

-- 是否为收集最大值  30 个
function T:isWorldCollectionMax()
	local num = 0
	for k, v in pairs(self.worldCollection) do
		if v then
			num = num + 1
		end
	end

	if num >= self.maxCollection then
		return true
	else
		return false
	end
end

--- 收集的初始化
function T:initWorldCollection()
	if self.firstEnterWorldCollection == false then
		-- 表示第一次进入
		self.firstEnterWorldCollection = true
		require("src.app.communication.worldMap.WorldMapGetCollectionCommand").new():exec( function(state, cc, length)
			if state == ReturnCode.SUCCESS then
				self.worldCollection = cc
				self:luaDispatchEvent("worldMapCollection")
				-- 派发初始化事件
			end
		end )
	else
		self:luaDispatchEvent("worldMapCollection")
	end
end

--- 获得收集的信息
function T:getWorldCollection()
	local n = 0
	if not self.worldCollection then return { } end
	for k, v in pairs(self.worldCollection) do
		n = n + 1
	end
	if n > 0 then
		return mapping( { }, self.worldCollection)
			-- 避免被修改
	else
		return { }
	end

	return self.worldCollection
end

--- 删除收集的
function T:removeWorldCollection(x, y)
	local index = x .. "," .. y
	if self.worldCollection[index] then
		self.worldCollection[index] = nil
		self:luaDispatchEvent("worldMapCollection")
		-- 删除之后派发更新事件
	end
end

--- 修改数据
function T:modifyWorldCollection(data)
	if not data then return end

	for i = 1, #data do
		local index = data[i].x .. "," .. data[i].y
		if self.worldCollection[index] then
			self.worldCollection[index] = data[i]
		end
	end
end

--- 获得具体位置的信息
function T:getNodeByPosition(x, y)
	return self.mapData.map[x .. "," .. y]
end

--- 获得开放的建筑
function T:getOpenBuilding()
	return { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 }
end

--- 得到当前的viewindex
-- 1 代表当前是基地
-- 2 代表是郊外
-- 3 代表是世界地图
function T:getSwitchView()
	return self.viewIndex
end

--- 获得特效
function T:getEff(kind, time)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return
	end

	local dur=ani:getDuration()

	--一个周期的播放持续时间,(1/4之内偏移)
	local animate=cc.Animate:create(ani)
	local offset=randomFloat(0,dur/4)


	animate:setDuration(dur+offset-dur/8)


	local funcBegin = function()
		s:setVisible(true)
	end

	local funcEnd = function()
		s:setVisible(false)
	end

	if time then
		s:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.CallFunc:create(funcBegin), animate, cc.CallFunc:create(funcEnd), cc.DelayTime:create(time))))
	else
		s:runAction(cc.RepeatForever:create(animate))
	end
	--播放的开始时间,在整个持续时间内随机
	CppCocos2dxUtils:setActionInterValElapsed(animate,randomFloat(0,dur));

	return s

end

--- 获得 不用随机帧的 特效
function T:getEffWithOutRandom(kind, time)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return
	end

	local animate = cc.Animate:create(ani)

	local funcBegin = function()
		s:setVisible(true)
	end

	local funcEnd = function()
		s:setVisible(false)
	end

	if time then
		s:runAction(cc.RepeatForever:create(cc.Sequence:create(
			cc.CallFunc:create(funcBegin),
			animate,
			cc.CallFunc:create(funcEnd),
			cc.DelayTime:create(time)
		)))
	else
		s:runAction(cc.RepeatForever:create(animate))
	end

	return s
end

--- 运行 n 次之后 自动删除
function T:getEffWithDelete(kind, num)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return
	end

	local animate = cc.Animate:create(ani)

	local func = function() -- 删除自身
		s:removeFromParent()
		s = nil
	end

	local ac = cc.Sequence:create(cc.Repeat:create(animate, num), cc.CallFunc:create(func)) -- 运行完动作之后 运行函数

	s:runAction(ac)
	return s
end

--- 是否正在播放主界面入场动画
function T:getIsPlayingEnterEff()
	return self.base.isPlayingEnterEff
end

return T
