--Author    : YuDing
--Date      : 2014/10/24
--Introduce : 下方工具条

local T = class("HomePageBottomBar",function()return ccui.Layout:create() end)

--- 管理
T.manager = nil

T.widget = nil

--- 上一次激活点
T.lastActive = nil

--- 进入主场景的特效时间
T.enterEffectTime = 0.5

--- 开放配件的 等级
T.openPart = 10

--- 构造函数
function T:ctor()
	mapping( self, LuaEventDispatcher )

	local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function( eventType )
				print( "HomePage:" .. eventType )
				if CocosEvent.ENTER == eventType then
					self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
				end

				self:luaDispatchEvent( eventType )
			end
		)
	end

end

function T:onEnter()
	if self.uiBottomButtonList and self.exitPosition then
		self.uiBottomButtonList:getInnerContainer():setPosition(self.exitPosition)
	end
end

function T:onExit()
	if self.uiBottomButtonList then
		local px, py = self.uiBottomButtonList:getInnerContainer():getPosition()
		self.exitPosition = cc.p(px, py)
	end
end

--- 回收
function T:cleanUp()
	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_UPDATE_WORLD_CHAT,self,self.updateWorldChatContent)
	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_GET_WORLD_QUEUE_CHAT,self,self.updateWorldChatContent)

	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_UPDATE_GROUP_CHAT,self,self.updateAllianceChatContent)
	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_UPDATE_PRIVATE_CHAT,self,self.updatePrivateChatContent)
end

--- 初始化
function T:initView()
	self:loadJson()
	self:initUI()
	self:registerEvent()
	self:initActive()
	self:updateContentTag()
	self:enterEffect()
end

function T:loadJson()
	local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.homePageBottomBar)
	self.widget = widget
	self:setContentSize(widget:getContentSize())
	self:addChild(widget)
end

--- 获取底部按钮
-- @param name 名称
-- @param textId 标题在语言包中的编号
-- @return 按钮
function T:extendBottomButton(name,textId)
	local btn=BaseButton:extend(Cocos2dxUtils:getChildByName(self.widget,name))
	if textId then
		btn:setString(Language:get(textId))
	end

	local uiText = Cocos2dxUtils:getChildByName(btn, "uiText") or btn.uiText
	if uiText then
		Cocos2dxUtils:setTextFont(uiText)
	end

	btn.touchBeganSound=SoundRes.NONE
	btn.touchEndedSound=SoundRes.OPEN_FUNC_OF_BAR

	return btn
end

function T:initUI()
	--切换按钮
	self.uiSwitchBtn=self:extendBottomButton("uiSwitchBtn","uy105")
	self.uiSwitchBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventSwitchClick()
		end
	end)

	--"关卡"按钮
	self.uiChapterBtn=self:extendBottomButton("uiChapterBtn","uy90")
	self.uiChapterBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventPointClick()
		end
	end)

	--"部队"按钮
	self.uiDefenceArmyBtn=self:extendBottomButton("uiDefenceArmyBtn","uy91")
	self.uiDefenceArmyBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventArmyClick()
		end
	end)

	--"角色"按钮
	self.uiRoleBtn=self:extendBottomButton("uiRoleBtn","ug_role")
	self.uiRoleBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			WindowManager:push(require("src.app.uiScript.scenes.playerInfo.PlayerInfoWindow").new(1))
		end
	end)

	--"背包和商店"按钮
	self.uiBagAndShopBtn=self:extendBottomButton("uiBagAndShopBtn","uy92")
	self.uiBagAndShopBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventBackPackClick()
		end
	end)

	--"配件"按钮
	self.uiPartBtn=self:extendBottomButton("uiPartBtn","uy93")
	self.uiPartBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventPartClick()
		end
	end)

	--"邮件"按钮
	self.uiEMailBtn=self:extendBottomButton("uiEMailBtn","uy107")
	self.uiEMailBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventMailClick()
		end
	end)

	--"排行"按钮
	self.uiRankBtn=self:extendBottomButton("uiRankBtn","uy94")
	self.uiRankBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventRankClick()
		end
	end)

	--"帮助"按钮
	self.uiHelpBtn=self:extendBottomButton("uiHelpBtn","uy95")
	self.uiHelpBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventHelpClick()
		end
	end)

	--"设置"按钮
	self.uiSetBtn=self:extendBottomButton("uiSetBtn","uy110")
	self.uiSetBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventSetClick()
		end
	end)

	--聊天按钮
	self.uiChatBtn=BaseButton:extend(Cocos2dxUtils:getChildByName(self.widget,"uiChatBtn"))
	self.uiChatBtn:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventChatClick()
		end
	end)

	--聊天文本容器
	local uiChatTextContainer=BaseButton:extend(Cocos2dxUtils:getChildByName(self.widget,"uiChatTextContainer"))
	uiChatTextContainer:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:onEventChatClick()
		end
	end)
	self.uiChatTextContainer=uiChatTextContainer

	--主基地激活显示
	self.uiActiveBase=Cocos2dxUtils:getChildByName(self.widget,"uiActiveBase")

	--世界地图激活显示
	self.uiActiveWorld=Cocos2dxUtils:getChildByName(self.widget,"uiActiveWorld")

	--底部按钮列表
	self.uiBottomButtonList = Cocos2dxUtils:getChildByName(self.widget, "uiBottomButtonList") -- 滑动

	-- 创建char xmlLabel
	local uiChat=XMLLabel()
	uiChat:setIsSingleLine(true)
	uiChat:setClippingEnabled(true)
	uiChat:setAlign("left","middle")
	uiChatTextContainer:addChild(uiChat)
	self.labelChat = uiChat
	if C_PFM == cc.PLATFORM_OS_IPHONE or C_PFM == cc.PLATFORM_OS_IPAD then
		REL:relOther(uiChat,uiChatTextContainer,0,0,0,0)
	elseif C_PFM == cc.PLATFORM_OS_ANDROID then
		REL:relOther(uiChat,uiChatTextContainer,0,0,0,0)
	else
		REL:relOther(uiChat,uiChatTextContainer,0,0,0,0)
	end

	if BGMgr:isGuide() then
		BGMgr:regGuideObj(BGMgr.TGR_HOMEPAGE_SWITCH_BUTTON,self.uiSwitchBtn)

		BGMgr:regGuideObj(BGMgr.TGR_HOMEPAGE_CHATPER_BUTTON,self.uiChapterBtn)
	end
end
--- 注册事件
function T:registerEvent()
	self:registerLocalEvent()
	self:registerPublicEvent()
end

function T:releaseEvent()
	self:releaseLocalEvent()
	self:releasePulicEvent()
end

--- 本地事件
function T:registerLocalEvent()
	ChatManager:luaAddEventListener(ChatManager.EVT_TYPE_UPDATE_WORLD_CHAT,self,self.updateWorldChatContent)

	ChatManager:luaAddEventListener(ChatManager.EVT_TYPE_UPDATE_GROUP_CHAT,self,self.updateAllianceChatContent)
	ChatManager:luaAddEventListener(ChatManager.EVT_TYPE_UPDATE_PRIVATE_CHAT,self,self.updatePrivateChatContent)

	local queue = ChatManager.chatBox.chatWorldInfosQueue
	local length = queue:getLength()
	if length > 0 then
		self:updateWorldChatContent()
	else
		ChatManager:luaAddEventListener(ChatManager.EVT_TYPE_GET_WORLD_QUEUE_CHAT,self,self.updateWorldChatContent)
	end
end

function T:releaseLocalEvent()
	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_UPDATE_WORLD_CHAT,self,self.updateWorldChatContent)

	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_UPDATE_GROUP_CHAT,self,self.updateAllianceChatContent)
	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_UPDATE_PRIVATE_CHAT,self,self.updatePrivateChatContent)

	ChatManager:luaRemoveEventListener(ChatManager.EVT_TYPE_GET_WORLD_QUEUE_CHAT,self,self.updateWorldChatContent)
end

function T:updateWorldChatContent()
	local queue = ChatManager.chatBox.chatWorldInfosQueue
	local length = queue:getLength()
	if length <= 0 then
		return
	end

	local chatInfo = queue:look(length)
	self:setChatName(chatInfo,1)
end

function T:updateAllianceChatContent()
	local queue = ChatManager.chatGroupInfosQueue
	local length = queue:getLength()
	if length <= 0 then
		return
	end

	local chatInfo = queue:look(length)
	self:setChatName(chatInfo,2)
end

function T:updatePrivateChatContent()
	AbstractUI.timeout(self,function()
		local queue = ChatManager.chatPrivateInfosQueue
		local length = queue:getLength()
		if length <= 0 then
			return
		end

		local chatInfo = queue:look(length)
		self:setChatName(chatInfo,3)
	end, 0)
end

function T:setChatName(chatInfo,tag)
	local chatInfo = chatInfo
	local sourceName = chatInfo.sourceName
	local content = nil

	if chatInfo.contentType == 2 then
		-- 战报
		if chatInfo.fightReportType == 4 then
			--竞技场战报
			content = Language:get("ug1349", chatInfo.content)
		else
			if chatInfo.resultType == 1 or chatInfo.resultType == 2 then
				if chatInfo.realTitle1 then
					content = Language:get("uc254",chatInfo.realTitle2)
				else
					content = Language:get("uc254",chatInfo.content)
				end

			elseif chatInfo.resultType == 3 or chatInfo.resultType == 4 then
				if chatInfo.realTitle1 then
					content = Language:get("uc834",chatInfo.realTitle2)
				else
					content = Language:get("uc255",chatInfo.content)
				end
			else
				content = Language:get("uc700")
			end
		end
	elseif chatInfo.contentType == 1 then -- 系统信息
		content = Language:get("uc708",chatInfo:getContent())
	else
		content = Language:get("uc704",chatInfo:getContent())
	end

	local description = nil
	if tag == 1 then
		-- 世界
		if chatInfo.sourceId <= 0 then
			description = Language:get("uy144",Language:get("uc922"),content)
		else
			description = Language:get("uy144",sourceName,content)
		end
	elseif tag == 2 then
		-- 军团
		description = Language:get("uc826",sourceName,content)
	else
		-- 私聊
		if UserInfo.id == chatInfo.sourceId then
			description = Language:get("uc827",chatInfo.chatUserName,content)
		else
			description = Language:get("uc828",chatInfo.sourceName,content)
		end
	end

	self.labelChat:setString(description)
end

--- 切换 事件
function T:onEventSwitchClick()
	local index = HomePageManager:getSwitchView()

	if index == 2 then
		-- 由资源地图 跳入
		if self.lastIndex == 1 then
			-- 上次是从主城进入
			HomePageManager:switchViewToBase()
		elseif self.lastIndex == 3 then
			-- 是从世界地图进入
			HomePageManager:switchViewToWorld()
		end
	else
		-- 由正常地方跳入
		HomePageManager:switchView()
	end
end

--- 邮件 事件
function T:onEventMailClick()
	WindowManager:push(
		require("src.app.uiScript.components.mail.MailWindow").new()
	)
end

--- 关卡 事件
function T:onEventPointClick()
	--新手引导中,此外流程被改变了
	if BGMgr:isGuide() then
		if BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_HOMEPAGE_CHATPER_BUTTON) then
			return
		end
	end

	WindowManager:push(
		require("src.app.uiScript.components.battle.AllCampaignWindow").new()
	)
end

--- 军队 事件
function T:onEventArmyClick()
	local judge = ArmyTaskQueueManager:getData()
	if judge then
		WindowManager:push(
			require("src.app.uiScript.components.army.ArmyWindow").new({["tag"] = 1},2)
		)
	else
		WindowManager:push(
			require("src.app.uiScript.components.army.ArmyWindow").new({["tag"] = 1})
		)
	end

end

--- 背包 事件
function T:onEventBackPackClick()
	if ActivityManager.timeLimitBuyInfo:getOpenValue() == true or ActivityManager.timeLimitBuyInfo2:getOpenValue() == true then
		WindowManager:push(
			require("src.app.uiScript.components.prop.PropWindow").new(1,true)
		)
	else
		WindowManager:push(
			require("src.app.uiScript.components.prop.PropWindow").new(1)
		)
	end

end

--- 配件 事件
function T:onEventPartClick()
	local lvl = PlayerInstance:getPlayerLevel()
	if lvl < self.openPart then
		MUIAlter_singleLine.show( {info = Language:get("uy199"), showSecond = 1 } )
	else
		WindowManager:push(
			require("src.app.uiScript.components.accessory.accessorySystem.AccessorySystemWindow").new()
		)
	end
end

--- 聊天 事件
function T:onEventChatClick()
	WindowManager:push(
		require("src.app.uiScript.components.chat.ChatWindow").new(1)
	)
end

--- 帮助 事件
function T:onEventHelpClick()
	WindowManager:push(
		require("src.app.uiScript.components.help.HelpWindow").new()
	)
end

--- 排行榜 事件
function T:onEventRankClick()
	local function finish()
		WindowManager:push(
			require("src.app.uiScript.components.rank.RankWindow").new()
		)
	end
	Command:SendRank(1,1,finish)
end

--- 设置 事件
function T:onEventSetClick()
	WindowManager:push(
		require("src.app.uiScript.components.set.SystemSetWindow").new()
	)
end

--- 释放
function T:releasePulicEvent()
	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.switchBase, self, self.activeToBase)
	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.switchWorld, self, self.activeToWorld)

	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BATTLE_REMAIN_TIME, self, self.updateArmyTag) -- 部队
	ArmyTaskQueueManager:luaRemoveEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.updateArmyTag)
	AircraftManager:luaRemoveEventListener(AircraftManager.EVT_TYPE_REPAIR_ARMY, self, self.updateArmyTag)

	MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_FIRST_GET_MAIL_LIST, self, self.updateMailTag)  -- 邮件

	MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_DELETE_ALL_MAIL, self, self.updateMailTag)  -- 邮件
	MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_DELETE_ONE_MAIL , self, self.updateMailTag)
	MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_READ_MAIL, self, self.updateMailTag)
	MailManager:luaRemoveEventListener(MailManager.EVT_TYPE_RECEIVE_MAIL, self, self.updateMailTag)

	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.enterEffectMainStage, self, self.enterComplete)

	SupplyLineManager:luaRemoveEventListener(SupplyLineManager.EVT_STARNUN_CHANGE,self,self.updatePartTag)    -- 配件
	SupplyLineManager:luaRemoveEventListener(SupplyLineManager.EVT_SUPPLY_LINE_FINISH,self,self.updatePartTag)
	SupplyLineManager:luaRemoveEventListener(SupplyLineManager.EVT_RESET_ALL_POINT,self,self.updatePartTag)

	DailyStates:luaRemoveEventListener(DailyStates.EVT_TYPE_UPDATE_DAILY_STATES,self,self.updatePartTag)



    local slotBox = AccessorySystemManager:getSlotBox()
    slotBox:luaRemoveEventListener(slotBox.EVT_INSTALL_ACCESSORY_REFRESH,self,self.updatePartTag)
    slotBox:luaRemoveEventListener(slotBox.EVT_UNLOAD_ACCESSORY_REFRESH,self,self.updatePartTag)
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    accessoryBox:luaRemoveEventListener(accessoryBox.EVT_LEVEL_UP_SUCCESS_REFRESH,self,self.updatePartTag)
    accessoryBox:luaRemoveEventListener(accessoryBox.EVT_REFORM_SUCCESS_REFRESH,self,self.updatePartTag)
    accessoryBox:luaRemoveEventListener(accessoryBox.EVT_DECOMPOSE_ACCESSORY_REFRESH,self,self.updatePartTag)
    accessoryBox:luaRemoveEventListener(accessoryBox.EVT_PORT_ACCESSORY_REFRESH,self,self.updatePartTag)
    PropsInfoMgr:luaRemoveEventListener(PlayerInstance.noticeMessage.prop,self,self.updatePartTag)



	--技能更新标识相关
	PropsInfoMgr:luaRemoveEventListener(PlayerInstance.noticeMessage.prop .. PlayerInstance:getMedalPropID(), self, self.updateRoleAndSkillTag)   -- 荣誉值的变化 在当前页面 等级不会发生改变 所以不用关心 人物等级带来的变化
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.skillLvl, self, self.updateRoleAndSkillTag) -- 注册技能等级改变事件
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.skillReset, self, self.updateRoleAndSkillTag)

	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateContentTag)
end

function T:registerPublicEvent()
	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.switchBase, self, self.activeToBase)       -- 跳转
	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.switchWorld, self, self.activeToWorld)


	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BATTLE_REMAIN_TIME, self, self.updateArmyTag) -- 部队
	ArmyTaskQueueManager:luaAddEventListener(ArmyTaskQueueManager.EVT_BACK_OVER_TIME, self, self.updateArmyTag)
	AircraftManager:luaAddEventListener(AircraftManager.EVT_TYPE_REPAIR_ARMY, self, self.updateArmyTag)

	MailManager:luaAddEventListener(MailManager.EVT_TYPE_FIRST_GET_MAIL_LIST, self, self.updateMailTag)  -- 邮件

	MailManager:luaAddEventListener(MailManager.EVT_TYPE_DELETE_ALL_MAIL, self, self.updateMailTag)  -- 邮件
	MailManager:luaAddEventListener(MailManager.EVT_TYPE_DELETE_ONE_MAIL , self, self.updateMailTag)
	MailManager:luaAddEventListener(MailManager.EVT_TYPE_READ_MAIL, self, self.updateMailTag)
	MailManager:luaAddEventListener(MailManager.EVT_TYPE_RECEIVE_MAIL, self, self.updateMailTag)

	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.enterEffectMainStage, self, self.enterComplete)

	SupplyLineManager:luaAddEventListener(SupplyLineManager.EVT_STARNUN_CHANGE,self,self.updatePartTag)    -- 配件
	SupplyLineManager:luaAddEventListener(SupplyLineManager.EVT_SUPPLY_LINE_FINISH,self,self.updatePartTag)
	SupplyLineManager:luaAddEventListener(SupplyLineManager.EVT_RESET_ALL_POINT,self,self.updatePartTag)

	DailyStates:luaAddEventListener(DailyStates.EVT_TYPE_UPDATE_DAILY_STATES,self,self.updatePartTag)


    local slotBox = AccessorySystemManager:getSlotBox()
    slotBox:luaAddEventListener(slotBox.EVT_INSTALL_ACCESSORY_REFRESH,self,self.updatePartTag)
    slotBox:luaAddEventListener(slotBox.EVT_UNLOAD_ACCESSORY_REFRESH,self,self.updatePartTag)
    local accessoryBox = AccessorySystemManager:getAccessoryBox()
    accessoryBox:luaAddEventListener(accessoryBox.EVT_LEVEL_UP_SUCCESS_REFRESH,self,self.updatePartTag)
    accessoryBox:luaAddEventListener(accessoryBox.EVT_REFORM_SUCCESS_REFRESH,self,self.updatePartTag)
    accessoryBox:luaAddEventListener(accessoryBox.EVT_DECOMPOSE_ACCESSORY_REFRESH,self,self.updatePartTag)
    accessoryBox:luaAddEventListener(accessoryBox.EVT_PORT_ACCESSORY_REFRESH,self,self.updatePartTag)
    PropsInfoMgr:luaAddEventListener(PlayerInstance.noticeMessage.prop,self,self.updatePartTag)


	--技能更新标识相关
	PropsInfoMgr:luaAddEventListener(PlayerInstance.noticeMessage.prop .. PlayerInstance:getMedalPropID(), self, self.updateRoleAndSkillTag)   -- 荣誉值的变化 在当前页面 等级不会发生改变 所以不用关心 人物等级带来的变化
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.skillLvl, self, self.updateRoleAndSkillTag) -- 注册技能等级改变事件
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.skillReset, self, self.updateRoleAndSkillTag)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateContentTag)
end

--- 更新部队 标识
function T:updateArmyTag()
	self:updateBottomButtonUpadteTagEff(self.uiDefenceArmyBtn,AircraftManager:judgeAirIsRepair() or ArmyTaskQueueManager:judgeIsTask())
end

--- 更新角色和角色技能 标识
function T:updateRoleAndSkillTag()
	self:updateBottomButtonUpadteTagEff(self.uiRoleBtn,PlayerInstance:judgeIsSkillUpgrade())
end

--- 更新邮件 标识
function T:updateMailTag()
	self:updateBottomButtonUpadteTagEff(self.uiEMailBtn,MailManager:judgeMailIsRead())
end

--- 更新配件 标示
function T:updatePartTag()
    local judge1 = SupplyLineManager:judgeIsAttack()
    local judge2 = AccessorySystemManager:judgeIsHasAccessoryOrBlueInstall()

	self:updateBottomButtonUpadteTagEff(self.uiPartBtn,judge1 or judge2)

end

--- 更新底部按钮的更新标记特效
-- @param uiButton 按钮
-- @param isHasUpdated 是否有更新
function T:updateBottomButtonUpadteTagEff(uiButton,isHasUpdated)
	local uiUpadteEff=uiButton.uiUpadteEff

	if isHasUpdated then
		if not uiUpadteEff then
			uiUpadteEff=HomePageManager:getEff("promptEffect")
			uiButton:addChild(uiUpadteEff)
			REL:toOther("right","top",uiUpadteEff,uiButton,20,20)
			uiButton.uiUpadteEff=uiUpadteEff
		end
	else
		if uiUpadteEff then
			uiUpadteEff:removeFromParent()
			uiButton.uiUpadteEff=nil
		end
	end
end

function T:updateContentTag()
	self:updateArmyTag()
	self:updateRoleAndSkillTag()
	self:updateMailTag()
	self:updatePartTag()
end

--- 激活点
function T:activeToBase()
	self.uiSwitchBtn:loadTexture("homePageBtnSwitchBase.png", ccui.TextureResType.plistType)
	self.lastIndex = HomePageManager:getSwitchView()
	self.uiActiveBase:setVisible(true)
	self.uiActiveWorld:setVisible(false)

	self.uiSwitchBtn:setString(Language:get("uy105"))
end

function T:activeToWorld()
	self.uiSwitchBtn:loadTexture("homePageBtnSwitchWorld.png", ccui.TextureResType.plistType)
	self.lastIndex = HomePageManager:getSwitchView()
	self.uiActiveBase:setVisible(false)
	self.uiActiveWorld:setVisible(true)

	self.uiSwitchBtn:setString(Language:get("uy155"))
end

--- 初始化激活点
function T:initActive()

	self.lastIndex = 1
	self.uiActiveBase:setVisible(true)
	self.uiActiveWorld:setVisible(false)
end

function T:setManager(m)
	self.manager = m
end

--- 设置是否能滑动
function T:setMoveEable(able)
	if self.uiBottomButtonList then
		CppCocos2dxUtils:setIsScrollEnabled(self.uiBottomButtonList,able)
	end
end

function T:enterEffect()
	local widget = self.widget
	local size = widget:getContentSize()
	local px, py = widget:getPosition()
	widget:setPosition(px, py - size.height) -- 刚开始在实现之外

	self:setTouchCanNotTouch() -- 刚开始 屏蔽消息

	local func = function() -- 派发 主界面开始的效果
		HomePageManager:luaDispatchEvent(HomePageManager.noticeMessage.enterEffectMainStage) -- 派发事件
	end

	local ac = cc.Sequence:create(cc.MoveTo:create(self.enterEffectTime, cc.p(px, py)), cc.CallFunc:create(func))
	widget:runAction(ac)
end

--- 设置不能点击
--- 主容器 屏蔽下面的事件  就可以实现这个效果
function T:setTouchCanNotTouch()
	self.widget:setTouchEnabled(true)
end

function T:setTouchCanTouch()
	self.widget:setTouchEnabled(false)
end

--- 进入完成
function T:enterComplete()
	self:setTouchCanTouch()
end

return T
