--Author    : YuDing
--Date      : 2014/10/23
--Introduce : 主界面基地

local T = class(
	"HomePageBase",
	function()
		return cc.Layer:create()
	end
)

--指挥中心   l=1
--1号机库  l=2
--2号机库  l=3
--研究所   l=4
--改造工厂 l=5
--仓库1   l=6
--仓库2   l=7
--金币    l=8

--- 管理
T.manager = nil

--- json
T.widget = nil

--- ui组
T.uiChild = {}

-- 中景
T.middleScroll = nil

--- 正在触摸的点数
T._touchingCount=0

-- 展示迷你
T._nameShowState = 0

-- 界面缩放速度
T.scaleAcc = 0.05
-- 滑动误差
T.scrollError = 20

-- 进入效果 修正
T.enterFixed = nil

--- 显示建筑名称的最小缩放比
T.showBuildingNameScale=1000

--- 滑动容器距离顶部的距离
T.scrollDistanceOfTop = 114
--- 滑动容器距离底部的距离
T.scrollDistanceOfBottom = 78

--- 模拟的scroll
local HomePageBaseSimulateScroll = require("src.app.uiScript.scenes.homePage.HomePageBaseSimulateScroll")

--- 构造函数
function T:ctor()
	mapping( self, LuaEventDispatcher )

	self.recordedTouchs={}

	local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function( eventType )
				print( "HomePageBase:" .. eventType )

				if CocosEvent.ENTER == eventType then
					self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
				end

				self:luaDispatchEvent( eventType )
			end
		)
	end

	if UI_VSIZE.height == 960 then
		self.enterFixed = cc.p(1, 8)
	else
		self.enterFixed = cc.p(1, 7)
	end

	self:initCfg()
end

--- 初始化配置文件
function T:initCfg()
	self.cfg = require("src.app.uiScript.scenes.homePage.base.HomePageBaseConfig")
end

function T:onEnter()
	--    AbstractUI.timeout(self,function()
	--        MUIAlter_singleLine.show({
	--		    info=Language:get("uc822"),
	--		    showSecond=1
	--	    })
	--    end, 0)

	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_SWTICH_TO_BASE)
	end
end

function T:onExit()
	--离开此页面后,必须对存储的 touch 进行处理,因为离开页面后,将不会再收到对此页面侦听的任何 touch 事件,即使有 touch 事件处理按下状态
	local tbTouch = self.recordedTouchs
	for k,v in ipairs(tbTouch) do
		table.remove(tbTouch,k)
		v:release()
	end
end

--- 回收
function T:cleanUp()

end

function T:cleanUpAll()
	self:releasePublicEvent()  -- 释放建筑节点上的侦听
	self:releaseBldObjectEvent()
end

function T:updatePlaneNum()
	local num = #self.cfg.AIRCRAFT_LOCS
	for i = 1, num do
		local info=UserInfo.city.aircraftBox.infos[i]

		local c = 0
		if info then
			c = info:getNum()
		end
		self.homePlaneObject:showAircraft(i, c) -- 展示飞机
	end
end

--- 多点触摸
function T:multiEvent()
	-- 记录多点触控距离
	local distance
	local touchTag
	local movePoint

	local delayEffAct

	local startEff=function()
		delayEffAct=AbstractUI.timeout(self,function()
			delayEffAct=nil

			--拖动时显示建筑名称
			if self._nameShowState == 0 then
				self._nameShowState = 1
				HomePageManager:luaDispatchEvent(HomePageManager.noticeMessage.scaleAddEffect)
			end
		end,0.2)
	end

	local endEff=function()
		if delayEffAct then
			AbstractUI.clearScheduler(self,delayEffAct)
			delayEffAct=nil
		end

		--当前的缩放比
		local s=math.floor(self.middleScroll:getScrollScale() * 1000)
		if s >= self.showBuildingNameScale then
			return
		end

		--隐藏建筑名称的显示
		if self._nameShowState == 1 then
			self._nameShowState = 0
			HomePageManager:luaDispatchEvent(HomePageManager.noticeMessage.scaleReduceEffect)
		end
	end

	-- 触摸组
	local tbTouch = self.recordedTouchs
	-- 触摸开始监听
	local function onTouchesBegan(touches,event)
		self._touchingCount=self._touchingCount + 1

		self.middleScroll:endRecordSlidAction() -- 有触摸开始事件就 停止滑动

		local size = #touches
		if size > 1 then
			local t1, t2 = touches[1]:getLocation(), touches[2]:getLocation()
			distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)

			for k,v in ipairs(touches) do
				v:retain()
				table.insert(tbTouch, v)
			end
		else
			local fTouch=touches[1]
			fTouch:retain()
			table.insert(tbTouch, fTouch)

			local l=#tbTouch
			if l == 1 then
				movePoint = touches[1]:getLocation()
				touchTag = true
			elseif l == 2 then -- 两个点
				touchTag = false
				local t1, t2 = tbTouch[1]:getLocation(), tbTouch[2]:getLocation()
				distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)
			elseif l > 2 then   -- 大于两个点就不处理
				touchTag = false
				distance = nil
			end
		end

		startEff()
	end
	--触摸移动监听
	local function onTouchesMoved(touches,event)
		local size = #touches
		if size > 1 then
			--缩放
			touchTag = false

			if #touches > 2 then   -- 大于两指
				return
			end

			if not distance then
				return
			end -- 没有距离也退出

			local t1, t2 = touches[1]:getLocation(), touches[2]:getLocation()
			local tempdistance = math.pow(t1.x - t2.x, 2) + math.pow(t1.y - t2.y, 2)
			local temp = math.sqrt(math.abs(tempdistance - distance))

			if not self.midPoint then  -- 如果没有中间点 表示第一次开始放大的时候， 就是我们需要关注的点， 最好是在滑动的时候判定
				self.midPoint = cc.pMidpoint(t1, t2)
				self.containerMidPoint = self.middleScroll:getContainer():convertToNodeSpace(self.midPoint)
				self.backContainerMidPoint = self.backScroll:getContainer():convertToNodeSpace(self.midPoint)
			end

			if self.moveEable == false then  -- 不能滑动
				return
			end

			if temp > 2 then -- 2个像素的差异 就是缩放
				if tempdistance > distance then
					self:excutScale(self.scaleAcc, self.midPoint)
			else
				self:excutScale(-self.scaleAcc, self.midPoint)
			end

			distance = tempdistance
			end

		else
			if not self.midPoint then self.midPoint = nil end
			if #tbTouch >= 2 then return end                         -- 单点滑动  但是触摸点有多个

			local now = touches[1]:getLocation()
			local old = touches[1]:getPreviousLocation()

			self.middleScroll:setDelta(touches[1]:getDelta()) -- touches[k]

			if touchTag == true then -- 表示当前还是单点
				if movePoint then
					local dx = math.abs(now.x - movePoint.x)
					local dy = math.abs(now.y - movePoint.y)
					if dx > 3 or dy > 3 then
						touchTag = false -- 表示点击的范围属于滑动
					else
						touchTag = true  -- 属于点击事件
						movePoint = now
					end
			else
				touchTag = true  -- 属于点击事件
				movePoint = now
			end
			end

			local ax = now.x - old.x
			local ay = now.y - old.y

			if self.moveEable == false then  -- 不能滑动
				return
			end

			self:excutMove(ax, ay) -- 移动
		end
	end

	local function onTouchesCancel(touches, event)
		self._touchingCount=self._touchingCount - 1

		movePoint = nil

		local isRemoved=false
		for k,v in ipairs(touches) do

			for m,n in ipairs(tbTouch) do
				if n == v then
					table.remove(tbTouch, m)
					n:release()

					isRemoved=true
				end

				break
			end
		end

		if isRemoved then
			if #tbTouch == 2 then
				local t1, t2=tbTouch[1]:getLocation(), tbTouch[2]:getLocation()
				distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)
			else
				if #tbTouch == 0 then
					self.middleScroll:startRecordSlidAction()
				end
				distance = nil
				self.midPoint = nil -- 不是两点的 中间点都置空
			end
		end

		endEff()
	end

	--触摸结束监听
	local function onTouchesEnded(touches, event)
		self._touchingCount=self._touchingCount - 1

		movePoint = nil
		for k,v in ipairs(touches) do
			for m,n in ipairs(tbTouch) do
				if n == v then
					table.remove(tbTouch, m)
					n:release()

					if #tbTouch == 2 then
						-- 由三个点变成两个点 算出两点的距离
						local t1, t2 = tbTouch[1]:getLocation(), tbTouch[2]:getLocation()
						distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)
					elseif #tbTouch == 0 then
						-- 这个就是点击事件
						if touchTag == true then
							-- 表示可以处理单击事件
							self:onTouchEvent(touches[1]:getLocation()) -- 点击事件
						end
						distance = nil
						self.midPoint = nil   -- 中间点置空

						self.middleScroll:startRecordSlidAction() -- 开始滑动

					else
						distance = nil
						self.midPoint = nil  -- 中间点置空
					end

					break
				end
			end
		end

		endEff()
	end

	local listener = cc.EventListenerTouchAllAtOnce:create()
	listener:registerScriptHandler(onTouchesBegan,cc.Handler.EVENT_TOUCHES_BEGAN)
	listener:registerScriptHandler(onTouchesMoved,cc.Handler.EVENT_TOUCHES_MOVED)
	listener:registerScriptHandler(onTouchesEnded,cc.Handler.EVENT_TOUCHES_ENDED)
	listener:registerScriptHandler(onTouchesCancel,cc.Handler.EVENT_TOUCHES_CANCELLED)

	local eventDispatcher = self:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

--- 放大
-- @param scale 增量
-- @param mid 中间点
function T:excutScale(scale, mid)
	self:setScrollScale(self.middleScroll, self.backScroll, self.farScroll, scale, mid)

	if self._touchingCount == 0 then
		self:excutScaleEffect()  -- 需要在改变了scale之后调用
	end

	if self.haloTwo then
		self.haloTwo:excutMove(self.middleScroll:getLeftXDistance())
	end

end

--- scale变化时产生的效果
function T:excutScaleEffect()
	self._nameShowState = self._nameShowState or 1   -- 初始化为true 这时候放大就不会有出现的效果， 因为界面刚出来的时候 名字是能看见的

	--当前的缩放比
	local s=math.floor(self.middleScroll:getScrollScale() * 1000)

	--大于等于此值显示建筑名称,否则不显示
	if s >= self.showBuildingNameScale then
		if self._nameShowState == 0 then
			self._nameShowState = 1
			HomePageManager:luaDispatchEvent(HomePageManager.noticeMessage.scaleAddEffect)
		end
	else
		if self._nameShowState == 1 then
			self._nameShowState = 0
			HomePageManager:luaDispatchEvent(HomePageManager.noticeMessage.scaleReduceEffect)
		end
	end
end

--- 增加倍率
-- @param scroll 主滑动容器
-- @param backScroll back滑动容器
-- @param farScroll far滑动容器
-- @param scale 增量
-- @param mid 世界中间点
function T:setScrollScale(scroll, backScroll, farScroll, scale, mid)

	local container = scroll:getContainer()

	local dx, dy = scroll:getDistance()

	local change = scroll:setScrollScale(scale)

	if change ~= 0 then
		self:excutMoveByScroll(scroll, mid) -- 偏移
	end

	if backScroll then
		if change ~= 0 then  -- 如果是有变化
			backScroll:setScrollScale(change * 0.8)
			self:excutMoveByScroll(backScroll, mid) -- 偏移
		end
	end

	--- 重置速率
	self:updateScrollRate()
end

--- 更新滑动速率
function T:updateScrollRate()
	local mx, my = self.middleScroll:getDistance() -- 获得长和宽的差距（含有scale的计算）
	local bx, by = self.backScroll:getDistance()
	--local cx, cy = self.farScroll:getDistance()

	local px, py = self.middleScroll:getContainerPosition()

	local xRate = math.abs(bx / mx)
	local yRate = math.abs(by / my)
	self.backScroll:setScrollRate(xRate, yRate)
	self.backScroll:setContainerPosition(xRate * px, yRate * py) -- 放大或者缩小之后 位置会发生偏差 此处是校正
end

-- 移动的增量
-- @param ax , ay 增量
function T:excutMove(ax, ay)
	if not ax or not ay then return end

	local c = self.uiChild

	-- 中景移动
	local mx, my = self.middleScroll:getDistance() -- 获得长和宽的差距（含有scale的计算）

	self.middleScroll:addScrollPosition(ax, ay)

	local bx, by = self.backScroll:getDistance()

	-- 背景移动
	self.backScroll:addScrollPosition(bx / mx * ax, by / my * ay) -- 达到一起达到边缘的效果， 中景带动背景滑动的效果

	local left, right = self.backScroll:getXDistance()

	if self.haloTwo then
		self.haloTwo:excutMove(self.middleScroll:getLeftXDistance())
	end
end

--- 中间层滑动事件
function T:middleScrollEvent(ax, ay, mx, my)
	local bx, by = self.backScroll:getDistance()
	-- 背景移动
	self.backScroll:addScrollPosition(bx / mx * ax, by / my * ay) -- 达到一起达到边缘的效果， 中景带动背景滑动的效果

	local left, right = self.backScroll:getXDistance()

	if self.haloTwo then
		self.haloTwo:excutMove(self.middleScroll:getLeftXDistance())
	end
end


--- 根据scroll 移动
function T:excutMoveByScroll(scroll, mid)
	local container = scroll:getContainer()

	local wp = nil
	if scroll == self.middleScroll then
		wp = container:convertToWorldSpace(self.containerMidPoint) -- 转换为世界坐标
	elseif scroll == self.backScroll then
		wp = container:convertToWorldSpace(self.backContainerMidPoint) -- 转换为世界坐标
	end

	local p = cc.pSub(mid, wp)  -- 计算偏移相应的距离
	scroll:addScrollPosition(p.x, p.y)
end

function T:initScrollState()
	if UI_VSIZE.width == 640 and UI_VSIZE.height == 960 then  -- 640 和 960 的分辨率
		self.middleScroll:setMinScale(0.95)
		self.backScroll:setMinScale(0.95)

		local max = self.middleScroll:getMaxScale() - 0.1

		self.middleScroll:setScaleWithOutDistance(max) -- 设置 刚开始时 主界面的缩放率 0.85
		self.backScroll:setScaleWithOutDistance(max) -- 0.88

		self.middleScroll.container:setPosition(cc.p(-714.5, -1847.8))
		self.backScroll.container:setPosition(cc.p(-319.5, -512.5))

		if self.haloTwo then
			self.haloTwo:excutMove(self.middleScroll:getLeftXDistance())
		end

	else
		self.middleScroll:setMinScale(0.95)
		self.backScroll:setMinScale(0.95)

		local max = self.middleScroll:getMaxScale() - 0.1

		self.middleScroll:setScaleWithOutDistance(max) -- 设置 刚开始时 主界面的缩放率 0.90
		self.backScroll:setScaleWithOutDistance(max) -- 0.92

		self.middleScroll.container:setPosition(cc.p(-714, -1866)) -- 调整 刚开始进入主界面的时候的 缩放率
		self.backScroll.container:setPosition(cc.p(-319, -507))

		if self.haloTwo then
			self.haloTwo:excutMove(self.middleScroll:getLeftXDistance())
		end
	end

	self:updateScrollRate()
end

--- 点击事件
function T:onTouchEvent(touch)
	local c = self.uiChild
	local buildingLocs=PlayerInstance.fixedBldLocation

	-- 跳转
	if self.toResourceObject:checkTouch(touch) then
		self:playClickSound()

		self:onEventToOutSideClick()
		return
	end

	-- 研究中心
	if self.studyObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.study) then -- 未开放
			MUIAlter_singleLine.show( {info = Language:get("fy1010"),showSecond = 1 } )
		else -- 开放
			self:onEventStudyCenterClick()
		end
		return
	end

	-- 许愿池
	if self.wishObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.wish) then
			return
		end
		self:onWishClick()
		return
	end

	-- 制造工厂
	if self.makeObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.make) then
			MUIAlter_singleLine.show( {info = Language:get("fy1012"),showSecond = 1 } )
		else
			self:onEventMakeFactoryClick()
		end

		return
	end

	-- 主基地
	if self.centerObject:checkTouch(touch) then
		self:playClickSound()

		if BGMgr:isGuide() then
			BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_HOMEPAGE_COMMAND_CENTER)
		end

		self:onEventCenterClick()
		return
	end

	-- 作战中心
	if self.commandObject:checkTouch(touch) then
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.command) then
			MUIAlter_singleLine.show( {info = Language:get("uc929",7), showSecond = 1 } )
		else
			self:playClickSound()
			AudioManager:playEffect(SoundRes.OPEN_AIRCRAFT_LAB)
			self:onEventCommandClick()
		end
		return
	end

	-- 仓库1
	if self.storeOneObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.storeOne) then
			MUIAlter_singleLine.show( {info = Language:get("fy1014"),showSecond = 1 } )
		else
			self:onEventStoreHouseOneClick()
		end

		return
	end

	-- 仓库2
	if self.storeTwoObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.storeTwo) then
			MUIAlter_singleLine.show( {info = Language:get("fy1015"),showSecond = 1 } )
		else
			self:onEventStoreHouseTwoClick()
		end

		return
	end
	-- 钱工厂
	if self.moneyObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.gold) then
			MUIAlter_singleLine.show( {info = Language:get("fy1013"),showSecond = 1 } )
		else
			self:onEventMoneyFactoryClick()
		end

		return
	end
	-- 机库1
	if self.planeOneObject:checkTouch(touch) then
		self:playClickSound()
		--		if not PlayerInstance:getBuildingIsOpen(buildingLocs.planeOne) then
		--		else
		self:onEventPlaneFactoryOneClick()
		--		end

		return
	end
	-- 机库2
	if self.planeTwoObject:checkTouch(touch) then
		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.planeTwo) then
			MUIAlter_singleLine.show( {info = Language:get("fy1011"),showSecond = 1 } )
		else
			self:onEventPlaneFactoryTwoClick()
		end

		return
	end
	-- 联盟
	if self.regionObject:checkTouch(touch) then

		self:playClickSound()
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.region) then
			MUIAlter_singleLine.show( {info = Language:get("fy1016"),showSecond = 1 } )
		else
			self:onEventLegionClick()
		end

		return
	end

	--- 竞技场
	if self.fightingObject:checkTouch(touch) then
		if not PlayerInstance:getBuildingIsOpen(buildingLocs.arena) then
			MUIAlter_singleLine.show( {info = Language:get("uc929",15), showSecond = 1 } )
		else
			self:playClickSound()
			self:onEventFightingClick()
		end

		return
	end

	if self.homePlaneObject:onTouch(touch) then
		return
	end
end

-- 得到对应target的内部坐标 和 自身区域
function T:getTouchRect(target, touch)
	local locationInNode = target:convertToNodeSpace(touch)
	local s = target:getContentSize()
	local rect = cc.rect(0, 0, s.width, s.height)
	return locationInNode, rect
end

function T:loadJson()
	local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.homePageBase)
	self.widget = widget
	self:addChild(widget)
	local eventDispatcher = self:getEventDispatcher()
	eventDispatcher:removeEventListenersForTarget(self.widget)  -- 删除这个节点上的listener.. 貌似加载的时候会加一个listener。
end


--- 飞机特效
function T:planeEffect()
	local fire = self.uiChild.panelFire
	local all = self.uiChild.panelAll

	local sizeFire = fire:getContentSize()
	local x, y = fire:getPosition()

	local func = function ()
		fire:setPosition(cc.p(x, y))
	end

	local size = all:getContentSize()
	local seq = cc.Sequence:create(cc.MoveTo:create(3, cc.p(x + size.width + sizeFire.width, y)), cc.CallFunc:create(func))

	local rt = cc.RepeatForever:create(seq)
	fire:runAction(rt)
end

--- 检查是否能点击
function T:checkBuildingOpenTouchVisible(t)
	if not t.openState then return true end  -- 代表一出来就有的建筑
	if t.openState == 0 then
		return false
	else
		return true
	end
end


--- 点击声音
function T:playClickSound()
--    AudioManager:playEffect(SoundRes.SOUND_COMMON_BUTTON)
end


function T:onEventToOutSideClick()
	HomeOpenWindowManager:jumpToOutSide(self.uiChild.toOutSide)
end

--- 许愿池
function T:onWishClick()
	HomeOpenWindowManager:openWishEvent(self.uiChild.imgWish)
end

--- 指挥中心
function T:onEventCenterClick()
	HomeOpenWindowManager:openCenterEvent(self.uiChild.imgCenter)
end

--- 作战中心
function T:onEventCommandClick()
	HomeOpenWindowManager:openCommandEvent(self.uiChild.imgCommand)
end

--- 仓库1
function T:onEventStoreHouseOneClick()
	HomeOpenWindowManager:openStoreHouseOneEvent(self.uiChild.imgStoreOne)
end

--- 研究中心
function T:onEventStudyCenterClick()
	HomeOpenWindowManager:openStudyCenterEvent(self.uiChild.imgStudy)
end
--- 造钱厂
function T:onEventMoneyFactoryClick()
	HomeOpenWindowManager:openMoneyFactoryEvent(self.uiChild.imgMoney)
end
--- 机库2
function T:onEventPlaneFactoryTwoClick()
	HomeOpenWindowManager:openPlaneFacrtoryTwoEvent(self.uiChild.imgPlaneTwo)
end

--- 联盟
function T:onEventLegionClick()
	HomeOpenWindowManager:openLegionEvent(self.uiChild.imgRegion)
end

--- 机库1
function T:onEventPlaneFactoryOneClick()
	HomeOpenWindowManager:openPlaneFactoryOneEvent(self.uiChild.imgPlaneOne)
end

--- 改造工厂
function T:onEventMakeFactoryClick()
	HomeOpenWindowManager:openMakeFactoryEvent(self.uiChild.imgMake)
end
--- 仓库2
function T:onEventStoreHouseTwoClick()
	HomeOpenWindowManager:openStoreHouseTwoEvent(self.uiChild.imgStoreTwo)
end

--- 竞技场
function T:onEventFightingClick()
	HomeOpenWindowManager:openArenaEvent(self.uiChild.imgFighting)
end

--- 加载ui
function T:loadUI()

	local c = self.uiChild
	local w = self.widget

	c.panelMiddle = Cocos2dxUtils:getChildByName(w, "panelMiddle") -- 中景
	c.imgMiddle = Cocos2dxUtils:getChildByName(w, "imgMiddle")
	c.middleContainer = Cocos2dxUtils:getChildByName(w, "paneMiddleContainer")

	c.panelBack = Cocos2dxUtils:getChildByName(w, "panelBack")
	c.imgBack = Cocos2dxUtils:getChildByName(w, "imgBack") -- 海滩层

	c.imgWater = Cocos2dxUtils:getChildByName(w, "panelWater") -- 带水的背景
	c.panelWaterfall = Cocos2dxUtils:getChildByName(w, "panelWaterfall") -- 带水的背景

	c.imgWish       = Cocos2dxUtils:getChildByName(w, "imgWish")                -- 许愿
	c.imgMake       = Cocos2dxUtils:getChildByName(w, "imgMake")                -- 制造工厂
	c.imgCenter     = Cocos2dxUtils:getChildByName(w, "imgCenter")              -- 主基地
	c.imgCommand    = Cocos2dxUtils:getChildByName(w, "imgCommand")             -- 作战中心
	c.imgMoney      = Cocos2dxUtils:getChildByName(w, "imgMoney")               -- 钱工厂
	c.imgPlaneOne   = Cocos2dxUtils:getChildByName(w, "imgPlaneOne")            -- 机库1
	c.imgPlaneTwo   = Cocos2dxUtils:getChildByName(w, "imgPlaneTwo")            -- 机库2
	c.imgRegion     = Cocos2dxUtils:getChildByName(w, "imgRegion")              -- 联盟
	c.imgStoreOne   = Cocos2dxUtils:getChildByName(w, "imgStoreOne")            -- 仓库1
	c.imgStoreTwo   = Cocos2dxUtils:getChildByName(w, "imgStoreTwo")            -- 仓库2
	c.imgStudy      = Cocos2dxUtils:getChildByName(w, "imgStudy")               -- 研究中心

	c.toOutSide = Cocos2dxUtils:getChildByName(w, "toOutSide")                  -- 跳转到资源地图
	c.imgFighting = Cocos2dxUtils:getChildByName(w, "imgFighting")              -- 竞技场
	c.robotPanel = Cocos2dxUtils:getChildByName(w, "robotPanel")                -- 机器人容器
	c.haloPanel = Cocos2dxUtils:getChildByName(w, "panelHalo")                -- 光晕容器
	c.panelPlane = Cocos2dxUtils:getChildByName(w, "panelPlane")                -- 飞机容器

	if BGMgr:isGuide() then
		BGMgr:regGuideObj(BGMgr.TGR_HOMEPAGE_AIRCRAFT_FACTORY_1,c.imgPlaneOne)
		BGMgr:regGuideObj(BGMgr.TGR_HOMEPAGE_COMMAND_CENTER,c.imgCenter)
		BGMgr:regGuideObj(BGMgr.TGR_HOMEPAGE_OUTSIDE,c.toOutSide)
	end
end

--- 初始化 建筑对象
function T:initBuildingObject()
	local c = self.uiChild
	local fixedLt = PlayerInstance.fixedBldLocation

	self.centerObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingCenter")(fixedLt.center,c.imgCenter)
	self.commandObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingLaboratory")(fixedLt.command,c.imgCommand)
	self.makeObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingMake")(fixedLt.make,c.imgMake)
	self.moneyObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingMoney")(fixedLt.gold,c.imgMoney)
	self.planeOneObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingPlane")(fixedLt.planeOne,c.imgPlaneOne)
	self.planeTwoObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingPlane")(fixedLt.planeTwo,c.imgPlaneTwo)
	self.regionObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingRegion")(fixedLt.region,c.imgRegion)
	self.storeOneObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingStore")(fixedLt.storeOne,c.imgStoreOne)
	self.storeTwoObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingStore")(fixedLt.storeTwo,c.imgStoreTwo)
	self.studyObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingStudy")(fixedLt.study,c.imgStudy)
	self.wishObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingWish")(fixedLt.wish,c.imgWish)
	self.toResourceObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingJumpToResource")(fixedLt.toResouce,c.toOutSide)
	self.fightingObject=require("src.app.uiScript.scenes.homePage.base.building.BuildingFighting")(fixedLt.arena,c.imgFighting)

	self.backCliffObject = mapping(c.imgWater, require("src.app.uiScript.scenes.homePage.base.BackCliff")) -- 带水的背景
	self.backBeachObject = mapping(c.imgBack, require("src.app.uiScript.scenes.homePage.base.BackBeach")) -- 海滩的背景

	self.robotActionObject = mapping(c.robotPanel, require("src.app.uiScript.scenes.homePage.base.PatrolRobotExtend")) -- 机器人 动作
	self.robotActionObject:create()
	self.robotActionObject:patrol()

	self.homePlaneObject = mapping(c.panelPlane, require("src.app.uiScript.scenes.homePage.base.AircraftLocsContainerExtend")) -- 主城飞机

	self.backCliffObject:initView()
	self.backBeachObject:initView()

	self.homePlaneObject:init(self.cfg.AIRCRAFT_LOCS)
end

--- 创建模拟的scroll
function T:createSimulateScroll()
	local c = self.uiChild

	self:initMiddleScroll(c.panelMiddle)
	self.middleScroll = HomePageBaseSimulateScroll.new(c.panelMiddle, c.middleContainer)

	self:initBackScroll(c.panelBack)
	self.backScroll = HomePageBaseSimulateScroll.new(c.panelBack, c.imgBack)

	self.middleScroll:registerAutoElidEvent(self.middleScrollEvent, self)
end

function T:initMiddleScroll(scroll)
	local height = UI_VSIZE.height - self.scrollDistanceOfBottom - self.scrollDistanceOfTop
	scroll:setPositionY(self.scrollDistanceOfBottom)
	scroll:setContentSize(cc.size(UI_VSIZE.width, height))
end

function T:initFarScroll(scroll)
end

function T:initBackScroll(scroll)
	local px, py = scroll:getPosition()

	local size = cc.size(640, 400)
	scroll:setContentSize(size)

	local msize = self.uiChild.panelMiddle:getContentSize()
	local mx, my = self.uiChild.panelMiddle:getPosition()

	local height = msize.height + my - size.height
	scroll:setPositionY(height)
end

function T:registerPublicEvent()
	local event = {AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH, AircraftManager.EVT_TYPE_DEFENCE_ARMY, AircraftManager.EVT_TYPE_REPAIR_ARMY}
	for i = 1, #event do
		AircraftManager:luaAddEventListener(event[i], self, self.updatePlaneNum)
	end

	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.enterEffectMainStage, self, self.enterEffect) -- 开始进入特效
	CommonBroadcast:luaAddEventListener(EVT_TYPE_NET_TIME_OUT,self,self.netTimeoutHandler)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updatePlaneNum)
end

function T:releasePublicEvent()
	local event = {AircraftManager.EVT_TYPE_AIRCRAFT_REFRESH, AircraftManager.EVT_TYPE_DEFENCE_ARMY, AircraftManager.EVT_TYPE_REPAIR_ARMY}
	for i = 1, #event do
		AircraftManager:luaRemoveEventListener(event[i], self, self.updatePlaneNum)
	end

	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.enterEffectMainStage, self, self.enterEffect)
	CommonBroadcast:luaRemoveEventListener(EVT_TYPE_NET_TIME_OUT,self,self.netTimeoutHandler)

	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updatePlaneNum)
end

--- 通讯超时
function T:netTimeoutHandler()
	MUIAlter.show({
		info=Language:get("ug_netException"),
		zOrder=TOP_ORDER + 200+10,
		callback=function()
			print("通讯超时,退回到登陆界面")
			if BGMgr:isGuide() then
				--退回到登陆界面
				require("src.app.controler.Back2LoginScenceCommand")()
			end
		end
	})
end

--- 释放建筑对象的事件
function T:releaseBldObjectEvent()
	self.centerObject:rmvEvtListeners()
	self.commandObject:rmvEvtListeners()
	self.makeObject:rmvEvtListeners()
	self.moneyObject:rmvEvtListeners()
	self.planeOneObject:rmvEvtListeners()
	self.planeTwoObject:rmvEvtListeners()
	self.regionObject:rmvEvtListeners()
	self.storeOneObject:rmvEvtListeners()
	self.storeTwoObject:rmvEvtListeners()
	self.studyObject:rmvEvtListeners()
	self.wishObject:rmvEvtListeners()
	self.toResourceObject:rmvEvtListeners()
	self.fightingObject:rmvEvtListeners()

	self.robotActionObject:create()
	self.robotActionObject:patrol()
end

function T:setManager(m)
	self.manager = m
end

--- 初始化
function T:initView()
	self:loadJson()                      -- 加载json
	self:loadUI()                        -- 加载ui
	self:createSimulateScroll()          -- scroll创建
	self:multiEvent()                    -- 多点事件的发生
	self:initBuildingObject()            -- 初始化build对象

	self:initHaloEff()                   -- 光效

	self:updatePlaneNum()                -- 更新飞机数量
	self:registerPublicEvent()           -- 注册公共事件
	self:initScrollState()               -- 初始化状态
	self:setHomeBaseMoveEnable(false)

	self:initEnterEffect()  -- 初始化

	self:initSecret()       -- 小秘书

	--	self:testScroll() -- 测试使用
end

function T:initSecret()
	self.homeSecret = require("src.app.uiScript.scenes.homePage.base.HomeSecret").new(self, self.widget)
end

--- 测试
function T:testScroll()

	local add = ccui.Button:create()
	add:loadTextures("buttonRed_normal.png", "buttonRed_normal.png", "",ccui.TextureResType.plistType)
	self.middleScroll.scroll:addChild(add)
	REL:toOther("right", "middle", add, self.middleScroll.scroll, 0, -200)
	add:setLocalZOrder(10000)
	add:addTouchEventListener(function(t, e)
		if ccui.TouchEventType.ended ~= e then return end
		self:subScrollEvent()
		--self:setHomeBaseMoveEnable(true)
		--HomePageManager:setBottomBarMoveEable(true)
		--print("回复滑动")
	end)

	local sub = ccui.Button:create()
	sub:loadTextures("buttonInc_normal.png", "buttonInc_normal.png", "", ccui.TextureResType.plistType)
	self.middleScroll.scroll:addChild(sub)
	REL:toOther("right", "middle", sub, self.middleScroll.scroll, 0, -250)
	sub:setLocalZOrder(10000)
	sub:addTouchEventListener(function(t, e)
		if ccui.TouchEventType.ended ~= e then return end
		self:addScrollEvent()
		--self:setHomeBaseMoveEnable(false)
		--HomePageManager:setBottomBarMoveEable(false)
		--print("禁用滑动")
	end)

end

function T:addScrollEvent(scale)
	local sp = cc.p(320, 400)
	self.containerMidPoint = self.middleScroll:getContainer():convertToNodeSpace(sp)
	self.backContainerMidPoint = self.backScroll:getContainer():convertToNodeSpace(sp)

	if not scale then
		scale = self.scaleAcc
	end

	self:excutScale(scale, sp)
end

function T:subScrollEvent(scale)

	local sp
	if UI_VSIZE.height == 960 then
		sp = cc.p(320, 0)
	else  -- 已经到底部了
		sp = cc.p(320, 400)
	end

	self.containerMidPoint = self.middleScroll:getContainer():convertToNodeSpace(sp)
	self.backContainerMidPoint = self.backScroll:getContainer():convertToNodeSpace(sp)

	if not scale then
		scale = self.scaleAcc
	end

	self:excutScale(-scale, sp)
end

function T:initEnterEffect()
	self.startEffMinChangeScale = self.middleScroll:getMinScale() + 0.06 -- 变化的最小尺寸
	self.startEffMaxChangeScale = self.middleScroll:getMaxScale() - 0.05 -- 变化最大的尺寸
	self.startEffAddTime = 0.01   -- 每一次 的 增加 放大的时间
	self.startEffChangeNum = 0.01 -- 每一次 增大的值

	local allTime = 1.10

	HomePageManager:setStartEffectTime(allTime)
	HomePageManager:setStartEffectTime(allTime) -- 系统的时间有误差 不是每一次 都是按照0.03这个值来 所以 allTime 是一个估量的值 根据游戏效果调整而来

	if self.halo then
		self.halo:setShowHaloTime(allTime) -- 光晕开始
	end
end

--- 是否正在播放主界面入场动画
T.isPlayingEnterEff=false

--- 进入效果
function T:enterEffect()
	local min = self.startEffMinChangeScale

	local change = self.startEffMaxChangeScale - self.startEffMinChangeScale

	local changeNum = self.startEffChangeNum -- 变化值
	local time = self.startEffAddTime

	self:setHomeBaseMoveEnable(false)
	self.haloTwo:setVisible(false)

	local effAct

	self.isPlayingEnterEff=true
	effAct=AbstractUI.interval(self,function()
		self:excutMove(self.enterFixed.x, self.enterFixed.y)
		self:subScrollEvent(changeNum)
		local scale = self.middleScroll:getScrollScale()

		if not self.firstStartHalo then -- 只触发一次
			self.halo:startAutoShowHaloTime() -- 光晕开始
			self.firstStartHalo = true
		end

		if scale <= min then
			AbstractUI.clearScheduler(self,effAct)

			self.isPlayingEnterEff=false

			self.haloTwo:setVisible(true) -- 第二类光效出现
			if not BGMgr:isGuide() then -- 没有在新手引导阶段 回复滑动
				self:setHomeBaseMoveEnable(true)
			end

			self.middleScroll:setMaxScale(1.4)
			self.backScroll:setMaxScale(1.4)

			HomePageManager:luaDispatchEvent(HomePageManager.noticeMessage.enterEffectComplete) -- 派发 进入界面完成的事件
		end
	end,0.01)
end

function T:setHomeBaseMoveEnable(able)
	self.moveEable = able
	self.middleScroll:setIgnoreElid(able)
end

function T:initHaloEff()
	local c = self.uiChild
	local size = c.panelMiddle:getContentSize()
	c.haloPanel:setContentSize(size)

	--- 创建第一类的光晕 并运行

	self.haloTwo = require("src.app.uiScript.scenes.homePage.base.HomeHaloTwo").new(c.haloPanel)
	self.halo = require("src.app.uiScript.scenes.homePage.base.HomeHalo").new(c.haloPanel)
end

return T
