--region HomeSecret.lua
--Author    : YuDing
--Date      : 2015/6/8
--Introduce : 主界面小秘书

local T = class( "HomeHalo", function() return ccui.Layout:create() end )

--- 两类提示的标记 只能其中一个为true 但是 lottery 优先
T.lottery = false 
T.resource = false 

--- 创建
--- @param parent 父类
--- @param relNode 相对于位置的节点
function T:ctor(parent, relNode)
    local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function( eventType )
                if CocosEvent.ENTER == eventType then
                    self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
				end
			end
		)
	end

    self:setContentSize(cc.size(100, 100)) -- 默认大小
    parent:addChild(self)

    REL:toOther("left", "bottom", self, relNode, 90, 140) -- 设置位置

    self:initCfg()
    self:registerPublicEvent()
end

--- 每进入一次 这个场景就刷新一次
function T:onEnter()
    self:refreshSecret()
end

function T:onExit()
end

function T:cleanUp()
    self:releasePublicEvent()
end

function T:registerPublicEvent()
    SetStates.luckyLotteryState:luaAddEventListener(SetStates.luckyLotteryState.EVT_FREE_LUCKY_LOTTERY_REFRESH, self, self.refreshSecret)
    PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl, self, self.refreshSecret)
    SetStates.setCommonState:luaAddEventListener(SetStates.setCommonState.EVT_CLOSE_OPEN_NEW_RESOURCE, self, self.refreshSecret)
    BGMgr:luaAddEventListener(BGMgr.EVT_TYPE_GUIDE_COMPLETE, self, self.refreshSecret)
end

function T:releasePublicEvent()
    SetStates.luckyLotteryState:luaRemoveEventListener(SetStates.luckyLotteryState.EVT_FREE_LUCKY_LOTTERY_REFRESH, self, self.refreshSecret)
    PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl, self, self.refreshSecret)
    SetStates.setCommonState:luaRemoveEventListener(SetStates.setCommonState.EVT_CLOSE_OPEN_NEW_RESOURCE, self, self.refreshSecret)
    BGMgr:luaRemoveEventListener(BGMgr.EVT_TYPE_GUIDE_COMPLETE, self, self.refreshSecret)
end

--- 更新小秘书
function T:updateSecret()
    if BGMgr:isGuide() then -- 正在新手指引
        self:removeSecret() -- 应该没有 秘书出现
        return
    end

    if SetStates.luckyLotteryState:judgeIsLottrey() then -- 能抽取
        self.resource = false
        self.lottery = true
        self:createSecret("lottery")
    else 
        self.lottery = false
        local state = SetStates.setCommonState:getClosedValue(SetStates.setCommonState.CLOSE_OPEN_NEW_RESOURCE)
	    if state == true then  -- 关闭
            self.resource = false
	    	self:removeSecret()
	    else
	    	if self:getNotBuildNum() == 0 then  -- 已经建造完了
                self.resource = false
	    		self:removeSecret()
	    	else
                self.resource = true
	    		self:createSecret("resource")
	    	end
	    end
    end
end

--- 获取还未建造的数目
function T:getNotBuildNum()
	local num = 0

    local b = self.cfg.posBegin
    local e = self.cfg.posEnd

    for i = b, e, 1 do 
        if self:getIsBuildedOpen(i) and PlayerInstance:getBuildingLvl(i) == 0 then  -- 位置 开放 等级 正在升级
			num = num + 1
		end
    end

	--print("建筑物数量为: " .. num)

	return num
end

function T:initCfg()
    self.cfg = HomePageManager.outCfg
end


--- 检测是否开放
-- @param l  位置
function T:getIsBuildedOpen(l)
	local cLvl = PlayerInstance:getBuildingLvl(PlayerInstance.fixedBldLocation.center)
	local lv = self.cfg.posReverse[l]
	if cLvl >= lv then
		return true
	else
		return false
	end
end

--- 创建小秘书
function T:createSecret()
	if not self.secret then  -- 头像
		local head = ccui.ImageView:create()
		head:loadTexture("secretBigger.png",ccui.TextureResType.plistType)

		self:addChild(head)
		REL:toOther("left", "bottom", head, self, 0, 0)

		self.secret = head

		BaseButton:extend(head)
		head:addTouchEventListener(function(t, e)
			if e ~= ccui.TouchEventType.ended then
				return
			end
			self:onEventHeadClick()
		end)

		-- 文本框
		local defalutSize = cc.size(480,42)
		local spriteFrame = ResLoader:getSpriteFrame("guideTipBack.png")
		-- 这个Scale9Sprite是cocos2dx库所带的九宫格，因为前面有cc
		--local scale9Sprite = cc.Scale9Sprite:createWithSpriteFrame(spriteFrame,cc.rect(spriteFrameSize.width/2,37,1,1))
		local scale9Sprite = Scale9Sprite:createWithFrame(spriteFrame)
		scale9Sprite:setContentSize(defalutSize)
		head:addChild(scale9Sprite)

		head.s9Sprite = scale9Sprite
		--REL:toOther("left", "middle", scale9Sprite, head, 120, 0)

		-- 文本内容
		local str
        if self.lottery == true then 
            str = Language:get("uy197")
        elseif self.resource == true then 
            str = Language:get("uy198")
        end

		local content = XMLLabel()
		content:setWidth(250)
		content:setString(str)
		content:updateView()

		local width = content:getWidth()
		local height = content:getHeight()

		head:addChild(content)

		head.content = content

		scale9Sprite:setContentSize(cc.size(width + 40, height + 20))

		REL:toOther("left", "middle", scale9Sprite, head, 140, 0)

		REL:toOther("left", "middle", content, scale9Sprite, 20, 0)

		--- 触摸载体
		local lay = ccui.Layout:create()
		lay:setTouchEnabled(true)
		BaseButton:extend(lay)
		lay:addTouchEventListener(function(t, e)    -- 事件
			if e ~= ccui.TouchEventType.ended then
				return
		    end
		    self:onEventHeadClick()
		end)
		head:addChild(lay)

		REL:toOther("left", "middle", lay, head, 140, 0) -- 位置
		REL:relOther(lay, scale9Sprite, 0, 0, 0, 0) -- 大小

		head.touchLay = lay

		local close = ccui.ImageView:create()
		close:loadTexture("homeRightTopClose.png", ccui.TextureResType.plistType)
		head:addChild(close)

		BaseButton:extend(close)
		close:addTouchEventListener(function(t, e)    -- 事件
			if e ~= ccui.TouchEventType.ended then
				return
		    end
		    local func = function(s)
		    	if s == FLAG_OK then
		    		SetStates.setCommonState:setClosedValue(true, SetStates.setCommonState.CLOSE_OPEN_NEW_RESOURCE)
		    		Command:SetCommonCommand()
		    	end

		    end

		    MUIAlter.show(
		    	{
		    		info = Language:get("uy128"),
		    		size = 30,
		    		flag = 3,
		    		callback = func,
		    	}
		    )

		end)

        local cs = ResLoader:getSpriteFrame("homeRightTopClose.png"):getOriginalSize()
		REL:toOther("right", "top", close, lay, -9, -30) -- 位置

		head.closeBtn = close

        if self.lottery == true then 
            close:setVisible(false)
        elseif self.resource == true then 
            close:setVisible(true)
        end

	end
end

function T:removeSecret()
	if self.secret then
		self.secret:removeFromParent()
		self.secret = nil
	end
end

function T:refreshSecret()
	-- 首先获取文本内容的高度，宽度
	self:updateSecret()
	--print("secret is fresh!")
	self:updateContent()
end

--- 更新内容
function T:updateContent()
    if self.secret then
		local head = self.secret

		local str
        if self.lottery == true then 
            str = Language:get("uy197")
        elseif self.resource == true then 
            str = Language:get("uy198")
        end

		local content = self.secret.content
		local scale9Sprite = self.secret.s9Sprite
		local lay = self.secret.touchLay
		local close = self.secret.closeBtn
		content:setString(str)
		content:updateView()

		local width = content:getWidth()
		local height = content:getHeight()

		scale9Sprite:setContentSize(cc.size(width + 40, height + 20))

		REL:toOther("left", "middle", scale9Sprite, head, 140, 0)

		REL:toOther("left", "middle", content, scale9Sprite, 20, 0)

		REL:toOther("left", "middle", lay, head, 140, 0) -- 位置
		REL:relOther(lay, scale9Sprite, 0, 0, 0, 0) -- 大小

		local cs = ResLoader:getSpriteFrame("homeRightTopClose.png"):getOriginalSize()
		REL:toOther("right", "top", close, lay, -9, -30) -- 位置

        if self.lottery == true then 
            close:setVisible(false)
        elseif self.resource == true then 
            close:setVisible(true)
        end
	end
end

-- 点击
function T:onEventHeadClick()
    if self.lottery == true then        -- 幸运彩票抽奖
        HomeOpenWindowManager:openWishEvent()
    elseif self.resource == true then   -- 资源
        HomePageManager:switchViewToOutSide()
    end    
end

return T

--endregion
