local Direct=require("src.app.uiScript.scenes.homePage.base.RobotDirect")

local T=Class.Robot(
	function()
		return cc.Sprite:create()
	end,
	AbstractUI,
	LuaEventDispatcher2
)

--- 行走状态-站立
T.STATUS_WALK_STANDING=0

--- 行走状态-行走
T.STATUS_WALK_WALKING=1

--- 行走状态-跑动
T.STATUS_WALK_RUNING=2

--- 行走状态-转向
T.STATUS_WALK_TURNING=3

--- 行走状态-蹲下中
T.STATUS_WALK_SQUAT_DNING=4

--- 行走状态-站起中
T.STATUS_WALK_STAND_UPING=5

--- 行走状态-蹲着
T.STATUS_WALK_SQUALING=6

--- 事件类型-行走状态被改变
T.EVT_TYPE_WALK_STATUS_CHANGE="evt_type_walk_status_change"

function T:def()
	AbstractUI.def(self)
	LuaEventDispatcher2.def(self)

	--- 当前状态
	self._walkStatus=self.STATUS_WALK_STANDING

	--- 行走速度
	self._walkSpeed=0.02

	--- 8向的资源
	self._direct8Reses={}

	--- 当前行走状态的action
	self._walkAction=nil

	--- 当前行走移动的action
	self._moveAction=nil

	--- 当前的方向
	self._direct=Direct.R
end

function T:ctor(drect8Reses)
	AbstractUI.ctor(self)
	LuaEventDispatcher2.ctor(self)

	self._direct8Reses=drect8Reses

	self:_init()
end

function T:_init()

end

--- 站立
-- @param pt 位置
-- @param direct 方向
function T:standing(pt,direct)
	local lWStatus=self._walkStatus
	self._walkStatus=self.STATUS_WALK_STANDING

	local lDirect=self._direct
	self._direct=direct or lDirect

	--状态或方向改变,都需要更新站立动画
	if lWStatus ~= self.STATUS_WALK_STANDING or lDirect ~= self._direct then
		if self._walkAction then
			self:stopAction(self._walkAction)
			self._walkAction=nil
		end

		local standRes=self._direct8Reses[self._direct].stand

		self:setFlippedX(standRes.isFlippedX)
		self._walkAction=cc.RepeatForever:create(cc.Animate:create(standRes.animation))
		self._moveAction=nil

		self:runAction(self._walkAction)
	end

	if pt then
		self:setPosition(pt)
	end

	self:luaDispatchEvent(self.EVT_TYPE_WALK_STATUS_CHANGE,self,lWStatus)
end

--- 行走到
-- @param desPt 目标地点
function T:walkTo(desPt)
	if self._walkAction then
		self:stopAction(self._walkAction)
		self._walkAction=nil
	end

	local lWStatus=self._walkStatus
	self._walkStatus=self.STATUS_WALK_WALKING

	--- 当前位置
	local sPt,ePt=cc.p(self:getPosition()),desPt
	local angle=MathUtil.angleOfDoudlePoint(sPt,ePt)
	local dirct=Direct:angle2direct(angle)
	self._direct=dirct

	--- 行走持续的时间
	local keepMS=self._walkSpeed*MathUtil.pointToPointDistance(sPt.x,sPt.y,ePt.x,ePt.y)

	--- 行走的资源
	local walkRes=self._direct8Reses[dirct].walk
	self:setFlippedX(walkRes.isFlippedX)
	self._walkAction=cc.RepeatForever:create(cc.Animate:create(walkRes.animation))
	self._moveAction=cc.Sequence:create(
		cc.MoveTo:create(keepMS,ePt),
		cc.CallFunc:create(function(t)
			if t._walkAction then
				t:stopAction(t._walkAction)
				t._walkAction=nil
			end

			t:standing()
		end)
	)

	self:runAction(self._walkAction)
	self:runAction(self._moveAction)

	self:luaDispatchEvent(self.EVT_TYPE_WALK_STATUS_CHANGE,self,lWStatus)
end

--- 转向
-- @param direct
function T:turnTo(direct)
	if direct and direct ~= self._direct then
		local lDirect=self._direct

		local turnRes=self._direct8Reses[lDirect].turn--[lDirect*10+direct]
		if turnRes then
			local lWStatus=self._walkStatus
			self._walkStatus=self.STATUS_WALK_TURNING

			if self._walkAction then
				self:stopAction(self._walkAction)
				self._walkAction=nil
			end

			self:setFlippedX(turnRes.isFlippedX)
			self._walkAction=cc.Sequence:create(
				cc.Animate:create(turnRes.animation),
				cc.CallFunc:create(function(t)
					t._direct=direct
					t:standing()
				end)
			)
			self:runAction(self._walkAction)

			self:luaDispatchEvent(self.EVT_TYPE_WALK_STATUS_CHANGE,self,lWStatus)
		else
			self._direct=direct
			self:standing()
		end
	end
end

--- 蹲着
function T:squating()
	local standRes=self._direct8Reses[self._direct].squating
	if standRes then
		local lWStatus=self._walkStatus
		self._walkStatus=self.STATUS_WALK_SQUALING

		if self._walkAction then
			self:stopAction(self._walkAction)
			self._walkAction=nil
		end

		self:setFlippedX(standRes.isFlippedX)
		self._walkAction=cc.RepeatForever:create(cc.Animate:create(standRes.animation))
		self._moveAction=nil

		self:runAction(self._walkAction)

		self:luaDispatchEvent(self.EVT_TYPE_WALK_STATUS_CHANGE,self,lWStatus)
	end
end

--- 蹲下
function T:squatDn()
	local turnRes=self._direct8Reses[self._direct].squatDn
	if turnRes then
		local lWStatus=self._walkStatus
		self._walkStatus=self.STATUS_WALK_SQUAT_DNING

		if self._walkAction then
			self:stopAction(self._walkAction)
			self._walkAction=nil
		end

		self:setFlippedX(turnRes.isFlippedX)
		self._walkAction=cc.Sequence:create(
			cc.Animate:create(turnRes.animation),
			cc.CallFunc:create(function(t)
				t:squating()
			end)
		)
		self:runAction(self._walkAction)

		self:luaDispatchEvent(self.EVT_TYPE_WALK_STATUS_CHANGE,self,lWStatus)
	else
		self:squating()
	end
end

--- 站起
function T:standUp()
	local turnRes=self._direct8Reses[self._direct].standUp
	if turnRes then
		local lWStatus=self._walkStatus
		self._walkStatus=self.STATUS_WALK_STAND_UPING

		if self._walkAction then
			self:stopAction(self._walkAction)
			self._walkAction=nil
		end

		self:setFlippedX(turnRes.isFlippedX)
		self._walkAction=cc.Sequence:create(
			cc.Animate:create(turnRes.animation),
			cc.CallFunc:create(function(t)
				t:standing()
			end)
		)
		self:runAction(self._walkAction)

		self:luaDispatchEvent(self.EVT_TYPE_WALK_STATUS_CHANGE,self,lWStatus)
	else
		self:standing()
	end
end

--- 当前的行走状态
function T:getWalkStatus()
	return self._walkStatus
end

return T
