--Author    : YuDing
--Date      : 2015/4/13
--Introduce : 机器人动作

local T = {}

--- 领头机器人的 起始坐标
T.walkBegin = {x=1152, y=861}
T.walkEnd = {x=581, y=584}

--- 机器人队伍的 起始坐标 和 结束坐标
T.robotTeam = {
	{{x=T.walkBegin.x+13, y=T.walkBegin.y+19}, {x=T.walkEnd.x-28, y=T.walkEnd.y-6}},
	{{x=T.walkBegin.x+25, y=T.walkBegin.y+11}, {x=T.walkEnd.x-15, y=T.walkEnd.y-17}}
}

mapping(T, LuaEventDispatcher)

--- 机器人的放大倍数
T.robotScale = 1

--- 初始化
function T:initView()
	self:initWalkTime()
	self:createLeaderRobot("front", self.walkBegin, self.walkEnd)

	self:initFollowRobotTime()
	self:createFollowRobot("front", "one")
	self:createFollowRobot("front", "two")

	--    for k, v in ipairs(self.robotTeam) do
	--        self:createRobot("front", v[1], v[2], "kindTwo")
	--    end

end

--- 获得特效的一次时间
function T:getAniDurTime(kind)
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return 0
	end

	local dur = ani:getDuration()
	return dur
end

--- 获得特效 以及运行次数
function T:getEffWithTime(kind, time, func)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return
	end

	local animate = cc.Animate:create(ani)

	local ac
	if func then
		ac = cc.Sequence:create(cc.Repeat:create(animate, time), cc.CallFunc:create(func)) -- 运行完动作之后 运行函数
	else
		ac = cc.Repeat:create(animate, time)
	end

	s:runAction(ac)
	return s
end

--- 没有次数限制
function T:getEffWithOutTime(kind)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(kind)
	if not ani then
		return
	end

	local animate = cc.Animate:create(ani)

	s:runAction(cc.RepeatForever:create(animate))
	return s
end


--- 创建领队机器人
function T:createLeaderRobot(direction)
	local endPic = nil
	local effName = nil
	local playEffName = nil -- 玩耍 特效

	local beginPos = nil
	local endPos = nil

	if direction == "front" then -- 向前走
		endPic = "robotOneWalk_00030.png"
		effName = "robotOneWalkFrontEff"

		playEffName = "robotOnePlayBackTwoEff" -- 到终点了 就是向后走的 玩耍特效

		beginPos = self.walkBegin
		endPos = self.walkEnd

		direction = "back" -- 方向重置
	else -- 向后走
		endPic = "robotOneWalk_00010.png"
		effName = "robotOneWalkBackTwoEff"

		playEffName = "robotOnePlayFrontEff"  -- 到起点了 就是想前走的 玩耍特效

		beginPos = self.walkEnd
		endPos = self.walkBegin

		direction = "front"
	end

	local eff = self:getEffWithTime(effName, self.walkNum)   -- 次数
	self:addChild(eff)
	CCUIREL:toOther("left", "bottom", eff, self, beginPos.x, beginPos.y)

	eff:setScale(self.robotScale)

	local func = function()
		self:removeChild(eff)

		local playEff = nil

		playEff = self:getEffWithTime(playEffName, 1, function()
			playEff:removeFromParent()
			playEff = nil

			local sprite = cc.Sprite:create() -- 转身
			sprite:setSpriteFrame(ResLoader:getSpriteFrame(endPic))
			local size = sprite:getContentSize()
			self:addChild(sprite)
			CCUIREL:toOther("left", "bottom", sprite, self, endPos.x - size.width / 2, endPos.y - size.height / 2)

			self.leaderSprite = sprite
			sprite:setScale(self.robotScale)

			if direction == "front" then
				self.leaderSprite:setVisible(false)
			end

			--self:createLeaderRobot(direction) -- 递归

		end) --　1次之后 停止

		if direction == "front" then  -- 特效也隐藏
			playEff:setVisible(false)
		end

		self:addChild(playEff)
		CCUIREL:toOther("left", "bottom", playEff, self, endPos.x, endPos.y)

		playEff:setScale(self.robotScale)
	end

	local ac = cc.Sequence:create(cc.MoveTo:create(self.walkTime, endPos), cc.CallFunc:create(func)) -- 后转
	eff:runAction(ac)

end

--- 初始化走动时间

-- 用时间作为标准 是有问题的
-- 用速度作为标准
function T:initWalkTime()
	self.walkTime = 10
	local dur = self:getAniDurTime("robotOneWalkFrontEff") -- 因为走动的特效 都是10帧 所以选一个就行
	self.walkNum = math.ceil(self.walkTime / dur)  -- 特效的运动次数
	--self.walkTime = self.walkNum * dur -- 时间为 特效时间的整数倍

	-- self.standardDistance = math.pow(math.abs(self.walkBegin.x - self.walkEnd.x)) + math.pow(math.abs(self.walkBegin.y - self.walkEnd.y)) -- 直线距离

	--self.standardDistance = math.sqrt(self.standardDistance) -- 开方 标准距离
end

function T:initFollowRobotTime()

	local beginPos = self.robotTeam[1][1]
	local endPos = self.robotTeam[1][2]
	self.followRobotTime = self.followRobotTime or {}
	--self.followRobotWalkNum = self.followRobotWalkNum or {}

	-- 标准距离
	local standardDistance = math.pow(math.abs(self.walkBegin.x - self.walkEnd.x), 2) + math.pow(math.abs(self.walkBegin.y - self.walkEnd.y), 2)
	standardDistance = math.sqrt(standardDistance) -- 开方 标准距离

	-- 相对距离
	local rel = math.pow(math.abs(beginPos.x - endPos.x), 2) + math.pow(math.abs(beginPos.y - endPos.y), 2)
	rel = math.sqrt(rel) -- 开方 标准距离

	self.followRobotTime.one = self.walkTime * rel / standardDistance -- 算出时间

	-- 第二个机器人
	beginPos = self.robotTeam[2][1]
	endPos = self.robotTeam[2][2]
	rel = math.pow(math.abs(beginPos.x - endPos.x), 2) + math.pow(math.abs(beginPos.y - endPos.y), 2)
	rel = math.sqrt(rel) -- 开方 标准距离
	self.followRobotTime.two = self.walkTime * rel / standardDistance -- 算出时间

	--local dur = self:getAniDurTime("robotTwoWalkFrontEff") -- 因为走动的特效 都是10帧 所以选一个就行
	--self.followRobotWalkNum.one = math.ceil(self.followRobotTime.one / dur)  -- 特效的运动次数 向上取整
end

--- 创建跟随机器人
function T:createFollowRobot(direction, kind)

	local endPic = nil
	local effName = nil
	local beginPos = nil
	local endPos = nil
	local walkTime = nil -- 行走的时间

	if kind == "one" then -- 第一个机器人 转身的图片
		if self.followOneSprite then
			self.followOneSprite:removeFromParent()
			self.followOneSprite = nil
	end
	elseif kind == "two" then
		if self.followTwoSprite then
			self.followTwoSprite:removeFromParent()
			self.followTwoSprite = nil
		end
	end

	if direction == "front" then -- 向前走
		endPic = "robotTwoWalk_00010.png"
		effName = "robotTwoWalkBackTwoEff"

		if kind == "one" then  -- 第一个 跟随 机器人
			beginPos = self.robotTeam[1][1]
			endPos = self.robotTeam[1][2]
			walkTime = self.followRobotTime.one
		elseif kind == "two" then
			beginPos = self.robotTeam[2][1]
			endPos = self.robotTeam[2][2]
			walkTime = self.followRobotTime.two
		end

		direction = "back" -- 方向重置
	else -- 向后走
		endPic =  "robotTwoWalk_00030.png"
		effName = "robotTwoWalkFrontEff"

		if kind == "one" then -- 第一个 跟随 机器人
			beginPos = self.robotTeam[1][2]
			endPos = self.robotTeam[1][1]
			walkTime = self.followRobotTime.one
		elseif kind == "two" then
			beginPos = self.robotTeam[2][2]
			endPos = self.robotTeam[2][1]
			walkTime = self.followRobotTime.two
		end

		direction = "front"
	end

	local eff = self:getEffWithOutTime(effName)
	self:addChild(eff)

	CCUIREL:toOther("left", "bottom", eff, self, beginPos.x, beginPos.y)
	eff:setScale(self.robotScale)

	local func = function()
		self:removeChild(eff)

		local sprite = cc.Sprite:create() -- 转身
		sprite:setSpriteFrame(ResLoader:getSpriteFrame(endPic))
		local size = sprite:getContentSize()
		self:addChild(sprite)
		CCUIREL:toOther("left", "bottom", sprite, self, endPos.x - size.width / 2, endPos.y - size.height / 2)
		sprite:setScale(self.robotScale)

		if kind == "one" then  -- 第一个机器人
			self.followOneSprite = sprite
		elseif kind == "two" then
			self.followTwoSprite = sprite
		end

		if direction == "front" then  -- 隐藏
			sprite:setVisible(false)
		end

		self:noticeLeaderRobotChange(direction)
	end
	-- cc.DelayTime:create(0.1)
	local ac = cc.Sequence:create(cc.MoveTo:create(walkTime, endPos), cc.CallFunc:create(func)) -- 先转
	eff:runAction(ac)

end

--- 通知 跟随机器人改变方向
function T:noticeFollowRobotChange(direction)
	self:createFollowRobot(direction, "one")
	self:createFollowRobot(direction, "two")
end

--- 通知 领头机器人改变方向
function T:noticeLeaderRobotChange(direction)
	local lSprite=self.leaderSprite

	if lSprite then
		local func = function()
			lSprite:removeFromParent()
			self.leaderSprite=nil

			self:noticeFollowRobotChange(direction) -- 通知跟随机器人换方向走
			self:createLeaderRobot(direction) -- 递归
		end

		if direction == "front" then -- 隐藏10秒 才开始走
			lSprite:runAction(cc.Sequence:create(cc.DelayTime:create(10), cc.CallFunc:create(func))) -- 10 秒之后开始行走
		else
			lSprite:runAction(cc.Sequence:create(cc.DelayTime:create(0.5), cc.CallFunc:create(func))) -- 0.5 秒之后开始行走
		end
	end
end


return T
