--region UpgradeBuilding.lua
--Author    : YuDing
--Date      : 2015/3/27
--Introduce : 建筑升级 共有的事件


local T = {}

--- 建筑的位置 唯一标示
T.l = nil

--- 开放状态 0-未开放 1-待开放 2-开放 (默认开放)
T.openState = nil

--- 建筑标记
T.buildTag = nil

--- 升级条
T.bar = nil

--- tag容器 标识
T.tagPanel = nil

--- 进度条容器
T.barPanel = nil

--- 注册共同的函数
function T:registerCommonEvent()
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.unLock .. self.l, self, self.setWaitUnLockState)  -- 增加解锁侦听
	BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME_2 .. self.l, self, self.upgrading) -- 剩余时间
	BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE .. self.l, self, self.upgradingComplete)
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl..self.l, self, self.upgradingComplete) -- 也是监听等级的变化
	BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_CANCEL_LEVEL_UP .. self.l, self, self.upgradingCancel)

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl .. self.l, self, self.updateLvl) -- 注册更新等级的消息

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateBuildState)
end

function T:releaseCommonEvent()
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.unLock .. self.l, self, self.setWaitUnLockState) -- 增加解锁侦听
	BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME_2 .. self.l, self, self.upgrading) -- 剩余时间
	BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE .. self.l, self, self.upgradingComplete)
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl..self.l, self, self.upgradingComplete) -- 也是监听等级的变化
	BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_CANCEL_LEVEL_UP .. self.l, self, self.upgradingCancel)

	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl .. self.l, self, self.updateLvl) -- 注册更新等级的消息

	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateBuildState)
end

--- 更新建造状态
function T:updateBuildState()
	local lvl = PlayerInstance:getBuildingLvl(self.l)

	if lvl == 0 then
		-- 0J 表示未开放或者待开放
		if not PlayerInstance:getBuildingIsOpen(self.l) then
			self:setLockState()
		else
			self:setWaitUnLockState()
		end
	else
		self:clearLockState()
		local item = BuildingsInfoMgr:checkBuildingUpgrade(self.l) -- 检查该建筑是否在升级
		if item then -- 有建筑在升级
			self:upgrading(item)
		else
			self:switchEffState("free") -- 切换特效状态
			self:removeBar()
		end
	end

	self:updateLvl()
end

--- 切换特效状态 子类覆盖
function T:switchEffState(state)

end

--- 待接锁状态
function T:setWaitUnLockState()
	self.openState = 1         -- 待开放

	self:removeLockTag() -- 移除锁的标记
	self:createBuildTag()  --  增加一个待建筑的标记

	if self.nameDelegete then
		self.nameDelegete:setVisible(true)
	end
end

--- 解锁状态
function T:setUnLockState()
	self.openState = 2

	self:removeLockTag() -- 移除锁的标记
	self:switchEffState("free")

	if self.nameDelegete then
		self.nameDelegete:setVisible(true)
	end
	if self.lvlDelegete then
		self.lvlDelegete:setVisible(true)
	end
end

-- 检查是否解锁
function T:checkLockState()
	local lvl = PlayerInstance:getBuildingLvl(self.l)

	if lvl == 0 then
		-- 0J 表示未开放或者待开放
		if not PlayerInstance:getBuildingIsOpen(self.l) then
			self:setLockState()
		else
			self:setWaitUnLockState()
		end
	else
		-- 开放
		self:setUnLockState()
		local item = BuildingsInfoMgr:checkBuildingUpgrade(self.l) -- 检查该建筑是否在升级
		if item then -- 有建筑在升级
			self:upgrading(item)
		end
	end
end

-- 未解锁状态
function T:setLockState()
	--self:setColor(cc.c3b(150, 150, 150))
	self.openState = 0

	self:createLockTag()

	if self.nameDelegete then self.nameDelegete:setVisible(false) end
	if self.lvlDelegete then self.lvlDelegete:setVisible(false) end
end

--- 清除 锁状态
function T:clearLockState()
	self:removeLockTag()
	self:removeBuildTag()
	self:setUnLockState()
end

--- 正在升级
function T:upgrading(item)
	if self.openState == 0 then
		-- 还未开启
		return
	end

	self:switchEffState("busy")

	if not self.bar then -- 创建进度条
		self:createBar()
	end

	if self.bar then
		self:updateBar(item)
	end
end

--- 升级完成
function T:upgradingComplete()
	self:switchEffState("free") -- 切换特效状态
	self:removeBar() -- 移除 进度条

	if self.openState ~= 2 then
		-- 清楚解锁的状态
		self:clearLockState()
	end
end

--- 升级取消
function T:upgradingCancel()
	self:switchEffState("free") -- 切换特效状态
	self:removeBar() -- 移除 进度条
end

--- 创建进度条
function T:createBar()
	if self.bar then return end

	local size = ResLoader:getSpriteFrame("homePageBldLevelUpFrame.png"):getOriginalSize()

	local c = ccui.Layout:create()
	c:setContentSize(size)

	local back = ccui.ImageView:create()
	back:loadTexture("homePageBldLevelUpFrame.png", ccui.TextureResType.plistType)
	c:addChild(back)
	back:setPosition(cc.p(size.width / 2, size.height / 2))

	local uiProgress = cc.ProgressTimer:create(cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("homePageBldLevelupBar.png")))
	uiProgress:setBarChangeRate(cc.p(1,0))--进度条动画方向，（1,0）表示横方向，（0,1）表示纵方向
	uiProgress:setType(cc.PROGRESS_TIMER_TYPE_BAR)--进度条类型
	uiProgress:setMidpoint(cc.p(0,1))--进度条的起始点，（0,y）表示最左边，（1,,y）表示最右边，（x,1）表示最上面，（x,0）表示最下面。
	uiProgress:setPercentage(0)--初始值
	uiProgress:setAnchorPoint(0, 0)

	local us = uiProgress:getContentSize()
	uiProgress:setPosition((size.width - us.width) / 2, (size.height - us.height) / 2)

	back:addChild(uiProgress)

	c.progress = uiProgress

	function c.setPercent(t, num)     -- 重新赋值一个函数
		if num > 100 then
			num = 100
	end
	t.progress:setPercentage(num)
	end

	self.bar = c
	self.bar:setPercent(0)

	self.barPanel:addChild(self.bar)

	CCUIREL:toOther("center", "middle", self.bar, self.barPanel, 0, 0) -- 相对于bar容器定位

    -- 添加时间显示
    local time = XMLLabel()
    time:setHAlign("center")
    time:setContentSize(cc.size(500, 36))
    self.bar:addChild(time)
    self.time = time
    CCUIREL:toOther("center", "top", self.time, self.bar, 0, 20)



end

--- 移除进度条
function T:removeBar()
	if self.bar then
		self.bar:removeFromParent()
		self.bar = nil
	end
end

--- 更新进度条
function T:updateBar(item)
	local allTime = PlayerInstance:getBuildingCurrentUpgradeTime(self.l) -- 总的时间
	local percent = (allTime - item.remainingSec) / allTime * 100
	if percent > 100 then percent = 100 end
	self.bar:setPercent(percent)

    self.time:setString(UnitConversion.handleTime4(item.remainingSec))

end

--- 创建建筑标识
function T:createBuildTag()
	if self.buildTag then
		self.buildTag:removeFromParent()
	end
	local res = "homePageWaitBuilding.png"
	local tag = ccui.ImageView:create()

	tag:loadTexture(res, ccui.TextureResType.plistType)

	local eff = HomePageManager:getEff("smoothEffect", 3) -- 闪光特效
	eff:setScale(0.55) -- 设置比例 比例也是微调
	tag:addChild(eff)

	local back = ccui.ImageView:create()
	back:loadTexture("homeBaseBldTagBack.png", ccui.TextureResType.plistType)

	self.tagPanel:addChild(back)  -- 加入到t中
	back:addChild(tag)

	CCUIREL:toOther("center", "middle", back, self.tagPanel, 0, 0) -- 相对于tag容器定位
	CCUIREL:toOther("center", "middle", tag, back, 0, 0)

	REL:toOther("center", "middle", eff, tag, 0, -2)  -- 位置为微调

	self.buildTag = back
end

--- 创建 锁的 标识
function T:createLockTag()
	if self.lockTag then self.lockTag:removeFromParent() end

	local tag = ccui.ImageView:create()
	tag:loadTexture("homeBldLock.png", ccui.TextureResType.plistType)

	self.tagPanel:addChild(tag)  -- 加入到t中

	CCUIREL:toOther("center", "middle", tag, self.tagPanel, 0, 0) -- 相对于tag容器定位

	self.lockTag = tag
end

function T:removeLockTag()
	if self.lockTag then
		self.lockTag:removeFromParent()
		self.lockTag = nil
	end
end

--- 移除建造标识
function T:removeBuildTag()
	if self.buildTag then
		self.buildTag:removeFromParent()
		self.buildTag = nil
	end
end

--- 更新等级
function T:updateLvl()
	local lvl = PlayerInstance:getBuildingLvl(self.l)
	if lvl == 0 then -- 0J -- 等级载体不可见
		self.lvlDelegete:setVisible(false)
	else -- 等级载体可见
		self.lvlDelegete:setVisible(true)
		self.lvl:setString(tostring(lvl))
	end
end

--- 设置唯一的位置
function T:setUniquePosition(l)
	self.l = l
end

--- 设置等级阴影
function T:setLvlShadow()
--    local shadowColor = cc.c4b(0, 0, 0, 125)  -- 阴影
--    local shadowOffset = cc.size(3, -2)

--    self.lvl:getVirtualRenderer():enableShadow(shadowColor, shadowOffset)
end

function T:getOpenState()
	return self.openState
end

--- 检查状态
function T:checkState()

end


return T





