local _print=print
local function print(...)
	_print("\t<AbstractBuilding>: ",table.concat({...},"\t"))
end

local BuildingNameBox_level=require("src.app.uiScript.scenes.homePage.base.building.BuildingNameBox_level")
local BuildingLvUpProgressBar=require("src.app.uiScript.scenes.homePage.base.building.BuildingLvUpProgressBar")

--- 抽象建筑
local T=Class.AbstractBuilding()

--- 建筑状态-未开放
T.BUILD_STATE_LOCKED=0

--- 建筑状态-已开放
T.BUILD_STATE_UNLOCK=1

--- 建筑状态-已建造
T.BUILD_STATE_BUILDED=2

--- 建筑状态-正在升级
T.BUILD_STATE_LEVELUPING=3

function T:def()
	--- 建筑位置
	-- _loc

	--- 当前的建筑状态
	self._build_state=-1

	--- 点击热点区域
	-- _touchHitArea

	--- 显示容器
	-- _uiContainer

	--- 建筑名称
	-- _uiBuildingName

	--- 建筑升级进度条
	-- _uiBuildingLvUpProgressBar

	--- 状态图标基于中心点的位置偏移
	self.stateIconPosition={x=100,y=100}

	--- 状态图标背景
	-- _uiStateIconBack

	--- 状态图标
	-- _uiStateIcon

	--- 状态图标附加特效
	-- _uiStateIconEff
end

--- 构造函数
-- @param buildLoc 建筑位置
-- @param uiContainer ui组件容器
function T:ctor(buildLoc,uiContainer)
	self._loc=buildLoc
	self._uiContainer=uiContainer

	self:_init()

	self:addEvtListeners()
end

function T:_init()
	self:initTouchHitArea()

	self:updateOpenState()
end

--- 添加事件侦听器
function T:addEvtListeners()
	local loc=self._loc
--	print("-------------------------------------添加事件侦听器-------------------------------------",tostring(loc))

	if loc then
		PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.unLock .. loc, self, self.updateOpenState)  -- 增加解锁侦听
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME_2 .. loc, self, self.updateBuildProgress) -- 剩余时间
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE .. loc, self, self.buildingLvUpCompleteHandler)
		PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl..loc, self, self.buildingLvUpCompleteHandler) -- 也是监听等级的变化
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_CANCEL_LEVEL_UP .. loc, self, self.buildingLvUpCancelHandler)

		PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl .. loc, self, self.updateOpenState) -- 注册更新等级的消息

		--断线重连/或重新登陆
		UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateOpenState)
	end
end

--- 移除事件侦听器
function T:rmvEvtListeners()
	local loc=self._loc
--	print("-------------------------------------移除事件侦听器-------------------------------------",tostring(loc))

	if loc then
		PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.unLock .. loc, self, self.updateOpenState) -- 增加解锁侦听
		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME_2 .. loc, self, self.updateBuildProgress) -- 剩余时间
		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE .. loc, self, self.buildingLvUpCompleteHandler)
		PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl..loc, self, self.buildingLvUpCompleteHandler) -- 也是监听等级的变化
		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_CANCEL_LEVEL_UP .. loc, self, self.buildingLvUpCancelHandler)

		PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl .. loc, self, self.updateOpenState) -- 注册更新等级的消息

		UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateOpenState)
	end
end

--- 创建建筑名称显示项
function T:createBuildingName()
	local uiC=self._uiContainer

	local uiBName=BuildingNameBox_level()
	uiC:addChild(uiBName,10)
	uiBName:setPosition(120,120)

	return uiBName
end

--- 创建建筑升级进度条
function T:createLvUpProgressBar()
	local uiC=self._uiContainer

	local uiPBar=BuildingLvUpProgressBar()
	uiC:addChild(uiPBar)

	return uiPBar
end

--- 更新当前建筑的开放状态
function T:updateOpenState()
	print("更新当前建筑的开放状态")

	if not self._loc then
		if not self._uiBuildingName then
			self._uiBuildingName=self:createBuildingName()
		end

		return
	end

	local lv=PlayerInstance:getBuildingLvl(self._loc)

	if PlayerInstance:getBuildingIsOpen(self._loc) then
		if not self._uiBuildingName then
			self._uiBuildingName=self:createBuildingName()
		end

		local lv=PlayerInstance:getBuildingLvl(self._loc)
		if lv == 0 then
			--建筑已开放,但未建造
			self:switchWaitBuildState()
		else
			--建筑已建造
			local buildInfo=BuildingsInfoMgr:checkBuildingUpgrade(self._loc)
			if buildInfo then
				--正在升级中
				self:switchLevelupingState()
			else
				--空闲
				self:switchBuildedState()
			end

			self:updateBuildLv()
			self:updateBuildProgress(buildInfo)
		end
	else
		--建筑未开放
		self:switchLockedState()
	end
end

--- 建筑升级完成
function T:buildingLvUpCompleteHandler()
	print("建筑升级完成")

	if not self._uiBuildingName then
		self._uiBuildingName=self:createBuildingName()
	end

	self:updateBuildLv()
	self:updateBuildProgress()
end

--- 建筑升级被取消
-- @param buildingLoc 建筑位置
function T:buildingLvUpCancelHandler(buildingLoc)
	print("建筑升级被取消",buildingLoc)

	if buildingLoc == self._loc then
		self:updateBuildProgress()
	end
end

--- 创建状态背景
function T:createStateIconBack()
	local uiBack=self._uiStateIconBack
	if not uiBack then
		uiBack=cc.Sprite:create()
		self._uiContainer:addChild(uiBack,9)
		local offset=self.stateIconPosition
		self._uiStateIconBack=uiBack
	end

	return uiBack
end

--- 释放状态背景
function T:releaseStateIconBack()
	local uiBack=self._uiStateIconBack
	if uiBack then
		uiBack:removeFromParent()
		self._uiStateIconBack=nil
	end
end

--- 创建状态图标
function T:createStateIcon()
	local uiIcon=self._uiStateIcon
	if not uiIcon then
		uiIcon=cc.Sprite:create()
		self._uiContainer:addChild(uiIcon,9)
		self._uiStateIcon=uiIcon
	end

	return uiIcon
end

--- 释放状态图标
function T:releaseStateIcon()
	local uiIcon=self._uiStateIcon
	if uiIcon then
		uiIcon:removeFromParent()
		self._uiStateIcon=nil
	end
end

--- 创建流光特效
function T:createSmoothEff()
	local uiEff=self._uiStateIconEff
	if not uiEff then
		uiEff=HomePageManager:getEff("smoothEffect", 3)
		uiEff:setScale(0.55)
		self._uiContainer:addChild(uiEff,9)
		self._uiStateIconEff=uiEff
	end

	return uiEff
end

--- 释放流光特效
function T:releaseSmoothEff()
	local uiEff=self._uiStateIconEff
	if uiEff then
		uiEff:removeFromParent()

		self._uiStateIconEff=nil
	end
end

--- 切换到锁定状态
function T:switchLockedState()
	if self._build_state == self.BUILD_STATE_LOCKED then
		return
	end

	local uiBack=self:createStateIconBack(),self:releaseStateIcon(),self:releaseSmoothEff()
	uiBack:setSpriteFrame(ResLoader:getSpriteFrame("homeBldLock.png"))
	uiBack:setPosition(self.stateIconPosition)

	self._build_state = self.BUILD_STATE_LOCKED
end

--- 切换到等待建造状态
function T:switchWaitBuildState()
	if self._build_state == self.BUILD_STATE_UNLOCK then
		return
	end

	local uiBack,uiIcon,uiEff=self:createStateIconBack(),self:createStateIcon(),self:createSmoothEff()
	uiBack:setVisible(true)
	uiIcon:setVisible(true)

	uiBack:setSpriteFrame(ResLoader:getSpriteFrame("homeBaseBldTagBack.png"))
	uiBack:setPosition(self.stateIconPosition)

	uiIcon:setSpriteFrame(ResLoader:getSpriteFrame("homePageWaitBuilding.png"))
	uiIcon:setPosition(self.stateIconPosition)

	uiEff:setPosition(self.stateIconPosition)

	self._build_state = self.BUILD_STATE_UNLOCK
end

--- 切换到已建造状态
function T:switchBuildedState()
	if self._build_state == self.BUILD_STATE_BUILDED then
		return
	end

	self:releaseStateIconBack()
	self:releaseStateIcon()
	self:releaseSmoothEff()

	self:playFreeEff()

	self._build_state = self.BUILD_STATE_BUILDED
end

--- 切换到正在升级状态
function T:switchLevelupingState()
	if self._build_state == self.BUILD_STATE_LEVELUPING then
		return
	end

	self:releaseStateIconBack()
	self:releaseStateIcon()
	self:releaseSmoothEff()

	self:playLevelUpingEff()

	self._build_state = self.BUILD_STATE_LEVELUPING
end

--- 播放空闲状态时的特效
function T:playFreeEff()

end

--- 播放升级状态中的特效
function T:playLevelUpingEff()

end

--- 更新建筑等级
function T:updateBuildLv()
	self._uiBuildingName:setBuildingLv(PlayerInstance:getBuildingLvl(self._loc))
end

--- 更新建造信息
function T:updateBuildProgress(buildInfo)
	--print("更新建筑升级进度信息:",self._loc,tostring(buildInfo))

	local rSec,percent=0,0

	if buildInfo then
		if self._build_state ~= self.BUILD_STATE_LEVELUPING then
			self:switchLevelupingState()
		end

		rSec=buildInfo.remainingSec

		local allSec = PlayerInstance:getBuildingCurrentUpgradeTime(self._loc)
		percent = (allSec - rSec) / allSec * 100

		if not self._uiBuildingLvUpProgressBar then
			self._uiBuildingLvUpProgressBar=self:createLvUpProgressBar()
		end
	end

	if rSec == 0 then
		if self._build_state ~= self.BUILD_STATE_BUILDED then
			self:switchBuildedState()
		end
	end

	if self._uiBuildingLvUpProgressBar then
		self._uiBuildingLvUpProgressBar:setProgress(percent,rSec)
	end
end

--- 初始化触摸热点区域
function T:initTouchHitArea()
--显示热点区域>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--	local pts={}
--
--	local uiC=self._uiContainer
--	local aPt=uiC:getAnchorPoint()
--	local offsetX,offsetY=aPt.x*uiC:getContentSize().width,aPt.y*uiC:getContentSize().height
--	print("offsetX,offsetY",offsetX,offsetY)
--	for k,v in ipairs(self._touchHitArea) do
--		pts[#pts+1]={x=v.x+offsetX,y=v.y+offsetY}
--	end
--
--	local dn=cc.DrawNode:create()
--	dn:clear()
--	uiC:addChild(dn)
--	dn:drawPolygon(pts,#pts,cc.c4b(0xff,0x00,0xff,0x44),1,cc.c4b(0xff,0x00,0x00,0x44))
--<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
end

--- 检查点击
-- @param touchPt 点击位置
function T:checkTouch(touchPt)
	local p = self._uiContainer:convertToNodeSpaceAR(touchPt)

	if MathUtil.containsPointInPolygon(p, self._touchHitArea) then
		return true
	end
	return false
end

return T
