local BuildingNameBox=require("src.app.uiScript.scenes.homePage.base.building.BuildingNameBox")

local AbstractBuilding=require("src.app.uiScript.scenes.homePage.base.building.AbstractBuilding")

--- 竞技场
local T=Class.BuildingFighting(AbstractBuilding)

function T:def()
	AbstractBuilding.def(self)

	--- 第一名背景
	-- _uifirstBack

	--- 第一名标题
	-- _uiFirstCaption

	--- 第一名名称
	-- _uiFirstName
end

--- 构造函数
-- @param buildLoc 建筑位置
-- @param uiContainer ui组件容器
function T:ctor(buildLoc,uiContainer)
	AbstractBuilding.ctor(self,buildLoc,uiContainer)
end

function T:_init()
	self.stateIconPosition={x=132,y=70}

	AbstractBuilding._init(self)

	local uiC=self._uiContainer

	self._uifirstBack = Cocos2dxUtils:getChildByName(uiC, "fightingRanking")      -- 竞技牌

	-- "第一名"标题
	local uiFC = Cocos2dxUtils:getChildByName(uiC, "rankText")
	uiFC:setSkewY(24.5)
	uiFC:setString(Language:get("uy166"))
	self._uiFirstCaption=uiFC

	self._uiFirstName = Cocos2dxUtils:getChildByName(uiC, "rankName")
	self._uiFirstName:setSkewY(24.5)

	self:updateFirstName()

	self:playEff()
end

--- 创建建筑名称显示项
function T:createBuildingName()
	local uiC=self._uiContainer

	local uiBName=BuildingNameBox()
	uiBName:setBuildingName(Language:get("uy141"))
	uiC:addChild(uiBName,10)
	uiBName:setPosition(132,126)

	return uiBName
end

--- 添加事件侦听器
function T:addEvtListeners()
	AbstractBuilding.addEvtListeners(self)

	ArenaInfoMgr:luaAddEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_FIRST_NAME, self, self.updateFirstName)
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.updateOpenState)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateFirstName)
end

--- 移除事件侦听器
function T:rmvEvtListeners()
	AbstractBuilding.rmvEvtListeners(self)
	
	ArenaInfoMgr:luaRemoveEventListener(ArenaInfoMgr.EVT_TYPE_CHANGE_FIRST_NAME, self, self.updateFirstName)
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.updateOpenState)

	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateFirstName)
end

--- 初始化触摸热点区域
function T:initTouchHitArea()
	local s=self._uiContainer:getContentSize()
	local ox,oy=s.width/2,s.height/2

	self._touchHitArea = {
		{x=117+ox, y=-13+oy},
		{x=43+ox, y=-77+oy},
		{x=-64+ox, y=-46+oy},
		{x=-74+ox, y=26+oy},
		{x=-8+ox, y=73+oy},
		{x=55+ox, y=40+oy},
	}

	AbstractBuilding.initTouchHitArea(self)
end

--- 切换到等待建造状态
function T:switchWaitBuildState()
	if self._build_state == self.BUILD_STATE_UNLOCK then
		return
	end

	self:releaseStateIconBack()
	self:releaseStateIcon()
	self:releaseSmoothEff()

	self._build_state = self.BUILD_STATE_UNLOCK
end

--- 播放特效
function T:playEff()
	local uiEff1=self._uiEff1
	if uiEff1 then
		return
	end

	local uiC=self._uiContainer

	uiEff1 = HomePageManager:getEffWithOutRandom("fightingNormalEff")
	uiEff1:setBlendFunc({src=0x0302, dst=1})
	uiC:addChild(uiEff1,1)
	uiEff1:setPosition(126,103.5)
	self._uiEff1=uiEff1

	local uiEff2 = HomePageManager:getEffWithOutRandom("arenaEff", 4)
	self._uifirstBack:addChild(uiEff2,1)
	uiEff2:setScale(0.8)
	uiEff2:setBlendFunc({src=0x0302, dst=1})
	uiEff2:setPosition(63,67)
	self.uiEff2=uiEff2
end

--- 更新第一名姓名
function T:updateFirstName()
    self._uiFirstName:setString(ArenaInfoMgr:getFirstName())
end

return T
