local BuildingNameBox_level=require("src.app.uiScript.scenes.homePage.base.building.BuildingNameBox_level")

local AbstractBuilding=require("src.app.uiScript.scenes.homePage.base.building.AbstractBuilding")

--- 改造工厂
local T=Class.BuildingMake(AbstractBuilding)

function T:def()
	AbstractBuilding.def(self)
end

--- 构造函数
-- @param buildLoc 建筑位置
-- @param uiContainer ui组件容器
function T:ctor(buildLoc,uiContainer)
	AbstractBuilding.ctor(self,buildLoc,uiContainer)
end

function T:_init()
	self.stateIconPosition={x=120,y=90}

	AbstractBuilding._init(self)

	--- 升级时的特效
	-- _uiLevelupEff

	--- 生产中特效
	-- _uiProdunctingEff

	--- 上一次显示的战机 sid
	-- _lastShowAircraftSid
end

--- 创建建筑名称显示项
function T:createBuildingName()
	local uiC=self._uiContainer

	local uiBName=BuildingNameBox_level()
	uiBName:setBuildingName(Language:get("uy120"))
	uiC:addChild(uiBName,10)
	uiBName:setPosition(120,146)

	return uiBName
end

--- 创建建筑升级进度条
function T:createLvUpProgressBar()
	local uiPBar=AbstractBuilding.createLvUpProgressBar(self)
	uiPBar:setPosition(120,10)
	
	return uiPBar
end

--- 显示正在生产的战机的图标
function T:showProductingAircraftIcon(productingList)
	-- 根据战机类型排序
	table.sort(productingList,function(a,b)
		return a.unitType < b.unitType
	end)

	local aircraftSid=productingList[1].unitType
	if self._lastShowAircraftSid and self._lastShowAircraftSid == aircraftSid then
		return
	end
	self._lastShowAircraftSid=aircraftSid

	local uiBack,uiIcon=self:createStateIconBack(),self:createStateIcon(),self:releaseSmoothEff()
	uiBack:setSpriteFrame(ResLoader:getSpriteFrame("homePlaneProduceBack.png"))
	local offset=self.stateIconOffsetOfCenter
	uiBack:setPosition(self.stateIconPosition)

	uiIcon:setTexture(IconManager.iconPath..Aircraft_temp[aircraftSid].icon)
	uiIcon:setScale(0.5)
	uiIcon:setPosition(self.stateIconPosition)
end

--- 隐藏正在生产的战机的图标
function T:hideProductingAircraftIcon()
	self:releaseStateIconBack()
	self:releaseStateIcon()
	self:releaseSmoothEff()
end

--- 更新改造工厂状态
function T:updateMakeState()
	if PlayerInstance:getBuildingLvl(self._loc) <= 0 then
		return
	end

	local productingList={}

	-- 当前正在生产的项
	local list = AircraftProduceQueueMgr:getQueueItems()
	for _,v in pairs(list) do
		if v.location == self._loc then
			table.insert(productingList,v)
		end
	end

	-- 当前没有正在生产的项
	if #productingList == 0 then
		self._lastShowAircraftSid=nil
	
		self:stopProductingEff()
		self:hideProductingAircraftIcon()
		return
	end

	self:playProductingEff()
	self:showProductingAircraftIcon(productingList)
end

--- 添加事件侦听器
function T:addEvtListeners()
	AbstractBuilding.addEvtListeners(self)

	AircraftProduceQueueMgr:luaAddEventListener(AircraftProduceQueueMgr.EVT_TYPE_AIRCRAFTFACTORY_REFRESH, self, self.updateMakeState)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateMakeState)
end

--- 移除事件侦听器
function T:rmvEvtListeners()
	AbstractBuilding.rmvEvtListeners(self)

	AircraftProduceQueueMgr:luaRemoveEventListener(AircraftProduceQueueMgr.EVT_TYPE_AIRCRAFTFACTORY_REFRESH, self, self.updateMakeState)

	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateMakeState)
end

--- 初始化触摸热点区域
function T:initTouchHitArea()
	local s=self._uiContainer:getContentSize()
	local ox,oy=s.width/2,s.height/2

	self._touchHitArea = {
		{x=24+ox, y=68+oy},
		{x=108+ox, y=-35+oy},
		{x=51+ox, y=-64+oy},
		{x=-60+ox, y=-20+oy},
		{x=-60+ox, y=26+oy}
	}

	AbstractBuilding.initTouchHitArea(self)
end

--- 播放空闲状态时的特效
function T:playFreeEff()
	self:stopLevelUpingEff()
end

--- 播放升级状态中的特效
function T:playLevelUpingEff()
	local uiLEff=self._uiLevelupEff
	if uiLEff then
		return
	end

	local uiC=self._uiContainer

	local flash=function(uiEff)
		uiEff:runAction(cc.RepeatForever:create(
			cc.Sequence:create(
				cc.FadeIn:create(0.15),
				cc.DelayTime:create(0.2),
				cc.FadeOut:create(0.15),
				cc.DelayTime:create(0.2)
			)
		))
	end

	local uiLEff = cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("planeFreetimeEff_00000.png"))
	uiLEff:setBlendFunc({src=0x0302, dst=1})
	uiC:addChild(uiLEff,1)
	CCUIREL:toOther("center", "middle", uiLEff, uiC, -3, 33)
	flash(uiLEff)
	self._uiLevelupEff=uiLEff
end

--- 停止播放升级状态中的特效
function T:stopLevelUpingEff()
	local uiLEff=self._uiLevelupEff
	if uiLEff then
		uiLEff:removeFromParent()
		self._uiLevelupEff=nil
	end
end

--- 播放生产中特效
function T:playProductingEff()
	local uiPEff=self._uiProdunctingEff
	if uiPEff then
		return
	end

	local uiC=self._uiContainer

	uiPEff = HomePageManager:getEffWithOutRandom("planeProduceEff")
	uiPEff:setBlendFunc({src=0x0302, dst=1})
	uiC:addChild(uiPEff,1)
	uiPEff:setPosition(190.5,36)
	self._uiProdunctingEff=uiPEff
end

--- 停止播放生产中特效
function T:stopProductingEff()
	local uiPEff=self._uiProdunctingEff
	if uiPEff then
		uiPEff:removeFromParent()
		self._uiProdunctingEff=nil
	end
end

return T
