--- 建筑名称显示框
local T=Class.BuildingNameBox(
	function()
		return cc.Node:create()
	end,
	AbstractUI
)

function T:def()
	AbstractUI.def(self)

	--- 名称背景的原始大小
	-- _uiNameBackOriginalSize

	--- 名称容器
	-- _uiNameContainer

	--- 名称背景
	-- _uiNameBack

	--- 名称背景
	-- _uiNameBack

	--- 名称下边框
	-- _uiNameBBorder

	--- 名称
	-- _uiName

	--- 名称显示/隐藏特效
	-- _nameFadeAct

	--- 当前名称的显示/隐藏状态
	self._nameIsHided=false
end

function T:ctor()
	AbstractUI.ctor(self)

	self:_init()
end

function T:cleanUp()
	AbstractUI.cleanUp(self)

	self:releaseScaleEvent()
end

function T:_init()
	--名称容器
	local uiNameC=cc.Node:create()
	self:addChild(uiNameC)
	self._uiNameContainer=uiNameC

	local f=ResLoader:getSpriteFrame("buildNameBack.png")
	self._uiNameBackOriginalSize=f:getOriginalSize()

	--名称背景
	local uiNBack=Scale9Sprite:createWithFrame(f,"h3")
	uiNameC:addChild(uiNBack)
	self._uiNameBack=uiNBack

	--名称下边框
	local uiNBBorder=Scale9Sprite:createWithFrame(ResLoader:getSpriteFrame("homePageNameBack.png"),"h3")
	uiNameC:addChild(uiNBBorder)
	self._uiNameBBorder=uiNBBorder

	--名称
	local uiName=XMLLabel()
	uiName:setAlign("center","middle")
	uiName:setCssStyle(CssStyles.buildingName)
	uiName:setIsSingleLine(true)
	uiNameC:addChild(uiName)
	self._uiName=uiName

	self:registerScaleEvent()
	
	self:updateNameShowState()
end

function T:registerScaleEvent()
	local host=SetStates.setCommonState
	host:luaAddEventListener(host.EVT_CLOSE_OPEN_BUILD_NAME, self, self.updateNameShowState)

	--	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.scaleAddEffect, self, self.showBuildingName)
	--	HomePageManager:luaAddEventListener(HomePageManager.noticeMessage.scaleReduceEffect, self, self.hideBuildingName)
end

function T:releaseScaleEvent()
	local host=SetStates.setCommonState
	host:luaAddEventListener(host.EVT_CLOSE_OPEN_BUILD_NAME, self, self.updateNameShowState)

	--	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.scaleAddEffect, self, self.showBuildingName)
	--	HomePageManager:luaRemoveEventListener(HomePageManager.noticeMessage.scaleReduceEffect, self, self.hideBuildingName)
end

--- 设置建筑名称
function T:setBuildingName(value)
	local uiName=self._uiName
	if value ~= uiName:getString() then
		uiName:setString(value)
		uiName:updateView()

		local uiNBack=self._uiNameBack
		local ts,os=uiName:getContentSize(),self._uiNameBackOriginalSize
		if ts.width > os.width or ts.height > os.height then
			REL:relOther(uiNBack,uiName,0,0,SIZE_NIL,SIZE_NIL)
		else
			uiNBack:setContentSize(os)
		end
		REL:toOther("center","middle",uiNBack,self,0,0)
		REL:relOther(self._uiNameBBorder,uiNBack,0,0,uiNBack:getContentSize().height-2,SIZE_NIL)
		REL:toOther("center","middle",uiName,uiNBack,0,0)
	end
end

function T:updateNameShowState()
	local host=SetStates.setCommonState
	if host:getClosedValue(host.CLOSE_BUILD_NAME) then
		self:hideBuildingName()
	else
		self:showBuildingName()
	end
end

--- 显示建筑名称
function T:showBuildingName()
	if not self._nameIsHided then
		return
	end

	local uiNC=self._uiNameContainer

	if self._nameFadeAct then
		uiNC:stopAction(self._nameFadeAct)
	end
	self._nameFadeAct=cc.Sequence:create(
		cc.FadeIn:create(0.15),
		cc.CallFunc:create(function()
			self._nameFadeAct=nil
		end)
	)
	uiNC:runAction(self._nameFadeAct)

	self._nameIsHided=false
end

--- 隐藏建筑名称
function T:hideBuildingName()
	if self._nameIsHided then
		return
	end

	local uiNC=self._uiNameContainer

	if self._nameFadeAct then
		uiNC:stopAction(self._nameFadeAct)
	end
	self._nameFadeAct=cc.Sequence:create(
		cc.FadeOut:create(0.15),
		cc.CallFunc:create(function()
			self._nameFadeAct=nil
		end)
	)
	uiNC:runAction(self._nameFadeAct)

	self._nameIsHided=true
end

return T
