local BuildingNameBox_level=require("src.app.uiScript.scenes.homePage.base.building.BuildingNameBox_level")

local AbstractBuilding=require("src.app.uiScript.scenes.homePage.base.building.AbstractBuilding")

--- 科研中心
local T=Class.BuildingStudy(AbstractBuilding)

function T:def()
	AbstractBuilding.def(self)

	--- 升级时的特效1
	-- _uiLevelupEff

	--- 升级时的特效2
	-- _uiLevelupEff2

	--- 生产中特效
	-- _uiProdunctingEff

	--- 上一次显示的研究项 sid
	-- _lastShowTechSid
end

--- 构造函数
-- @param buildLoc 建筑位置
-- @param uiContainer ui组件容器
function T:ctor(buildLoc,uiContainer)
	AbstractBuilding.ctor(self,buildLoc,uiContainer)
end

function T:cleanUp()
	AbstractBuilding.cleanUp(self)

	if self.effAct then
		if self.effAct:getParent() then
			self.effAct:removeFromParent()
		end
		self.effAct:release()
		self.effAct=nil
	end
end

function T:_init()
	self.stateIconPosition={x=144,y=150}

	local uiC=self._uiContainer

	local uiLEff=cc.Sprite:createWithSpriteFrameName("studyFreeTimeEff_00001.png")
	uiLEff:setScale(1/0.73)
	--	uiLEff:setBlendFunc({src=0x0302, dst=1})
	uiC:addChild(uiLEff,1)
	uiLEff:setPosition(130.5,149.5)
	self._uiLevelupEff=uiLEff

	local animation=ResLoader:getAnimation("studyFreeTimeEff")
	if animation then
		local effAct=cc.RepeatForever:create(cc.Animate:create(animation))
		effAct:retain()
		self.effAct=effAct
	end

	AbstractBuilding._init(self)

	--建筑等级大于1才能研究哦
	local lv=PlayerInstance:getBuildingLvl(self._loc)
	if lv > 0 then
		self:updateStudyState()
	end
end

--- 创建建筑名称显示项
function T:createBuildingName()
	local uiC=self._uiContainer

	local uiBName=BuildingNameBox_level()
	uiBName:setBuildingName(Language:get("uy117"))
	uiC:addChild(uiBName,10)
	uiBName:setPosition(144,206)

	return uiBName
end

--- 创建建筑升级进度条
function T:createLvUpProgressBar()
	local uiPBar=AbstractBuilding.createLvUpProgressBar(self)
	uiPBar:setPosition(144,40)
	
	return uiPBar
end

--- 显示研究项图标
function T:showStudyItemIcon(studingList)
	-- 根据战机类型排序
	table.sort(studingList,function(a,b)
		return a.technologySid < b.technologySid
	end)

	local techSid=studingList[1].technologySid
	if self._lastShowTechSid and self._lastShowTechSid == aircraftSid then
		return
	end
	self._lastShowTechSid=techSid

	local uiBack,uiIcon=self:createStateIconBack(),self:createStateIcon(),self:releaseSmoothEff()
	uiBack:setSpriteFrame(ResLoader:getSpriteFrame("homePlaneProduceBack.png"))
	local offset=self.stateIconOffsetOfCenter
	uiBack:setPosition(self.stateIconPosition)

	uiIcon:setTexture(IconManager.iconPath..Science_research_temp[techSid].iconUrl)
	uiIcon:setScale(0.5)
	uiIcon:setPosition(self.stateIconPosition)
end

--- 隐藏研究项图标
function T:hideStudyItemIcon()
	self:releaseStateIconBack()
	self:releaseStateIcon()
	self:releaseSmoothEff()
end

--- 更新技能研究状态
function T:updateStudyState()
	local list = TechnologiesInfoMgr:getQueue()

	local studingList={}
	for _,v in pairs(list) do
		if v.remainingSec > 0 then
			table.insert(studingList, v)
		end
	end

	if #studingList == 0 then
		self:stopProductingEff()
		self:hideStudyItemIcon()
		return
	end

	self:playProductingEff()
	self:showStudyItemIcon(studingList)
end

--- 添加事件侦听器
function T:addEvtListeners()
	AbstractBuilding.addEvtListeners(self)

	TechnologiesInfoMgr:luaAddEventListener(TechnologiesInfoMgr.EVT_TYPE_CANCEL_STUDY, self, self.updateStudyState)
	TechnologiesInfoMgr:luaAddEventListener(TechnologiesInfoMgr.EVT_TYPE_STUDY_COMPLETE, self, self.updateStudyState)
	TechnologiesInfoMgr:luaAddEventListener(TechnologiesInfoMgr.EVT_TYPE_START_STUDY, self, self.updateStudyState)
	TechnologiesInfoMgr:luaAddEventListener(TechnologiesInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME, self, self.updateStudyState)

	--断线重连/或重新登陆
	UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateStudyState)
end

--- 移除事件侦听器
function T:rmvEvtListeners()
	AbstractBuilding.rmvEvtListeners(self)

	TechnologiesInfoMgr:luaRemoveEventListener(TechnologiesInfoMgr.EVT_TYPE_CANCEL_STUDY, self, self.updateStudyState)
	TechnologiesInfoMgr:luaRemoveEventListener(TechnologiesInfoMgr.EVT_TYPE_STUDY_COMPLETE, self, self.updateStudyState)
	TechnologiesInfoMgr:luaRemoveEventListener(TechnologiesInfoMgr.EVT_TYPE_START_STUDY, self, self.updateStudyState)
	TechnologiesInfoMgr:luaRemoveEventListener(TechnologiesInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME, self, self.updateStudyState)

	UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,self,self.updateStudyState)
end

--- 初始化触摸热点区域
function T:initTouchHitArea()
	local s=self._uiContainer:getContentSize()
	local ox,oy=s.width/2,s.height/2

	self._touchHitArea = {
		{x=73+ox, y=106+oy},
		{x=92+ox, y=-51+oy},
		{x=-65+ox, y=-64+oy},
		{x=-54+ox, y=124+oy}
	}

	AbstractBuilding.initTouchHitArea(self)
end

--- 播放空闲状态时的特效
function T:playFreeEff()
	self:stopLevelUpingEff()
end

--- 播放升级状态中的特效
function T:playLevelUpingEff()
	local uiLEff=self._uiLevelupEff
	if uiLEff and not uiLEff.isRuningEff and self.effAct then
		uiLEff:runAction(self.effAct)

		uiLEff.isRuningEff=true
	end
end

--- 停止播放升级状态中的特效
function T:stopLevelUpingEff()
	local uiLEff=self._uiLevelupEff
	if uiLEff and self.effAct and uiLEff.isRuningEff then
		uiLEff:stopAction(self.effAct)

		uiLEff.isRuningEff=false
	end
end

--- 播放生产中特效
function T:playProductingEff()
	self:playLevelUpingEff()
end

--- 停止播放生产中特效
function T:stopProductingEff()
	self:stopLevelUpingEff()
end

return T
