--Author    : YuDing
--Date      : 2015/3/9
--Introduce : 神秘礼物窗口

local T = class("MysterousGiftWindow",function()return ccui.Layout:create() end)

--- 是否已被 cleanup
T.isNotCleanuped=true

function T:ctor()
	mapping(self, LuaEventDispatcher)

	local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function(eventType)
				print(string.format("MysterousGiftWindow=>ect:%s",eventType))

				if CocosEvent.ENTER == eventType then
					self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
					self.isNotCleanuped=nil
				end

				self:luaDispatchEvent(eventType)
			end
		)
	end

	self:initUI()
	self:initReward()
	self:updateTime()
	self:registerPublicEvent()

end

function T:onEnter()
end

function T:onExit()
end

function T:cleanUp()
	self:releasePublicEvent()
end

function T:registerPublicEvent()
	SetStates.giftState:luaAddEventListener(SetStates.giftState.REMAIN_TIME, self, self.updateTime)   -- 礼物时间
end

function T:releasePublicEvent()
	SetStates.giftState:luaRemoveEventListener(SetStates.giftState.REMAIN_TIME, self, self.updateTime)   -- 礼物时间
end

function T:initUI()
	self:setContentSize(cc.size(520, 400))

	local confirm = ccui.Button:create()    -- 确认按钮
	confirm:loadTextures("button_green_normal.png", "button_general_down.png", "",ccui.TextureResType.plistType)
	confirm:setIsGray(false)
	BaseButton:extend(confirm)
	confirm:addTouchEventListener(function(t, e)
		if e ~= ccui.TouchEventType.ended then return end

		self.__hide__() -- 关闭窗口

		GetMysteriousGiftCommand.new():exec(function(state)
			if state == ReturnCode.SUCCESS then -- 成功
			else
				MUIAlter_singleLine.show({
					info=ErrorCode:getErrInfo(state)
				})
			end
		end)
	end)


	self:addChild(confirm)

	REL:toOther("center", "bottom", confirm, self, 0, 67)
	self.confirm = confirm

	local confirmText = XMLLabel()          -- 按钮文字
	confirmText:setContentSize(cc.size(150, 27))
	confirmText:setHAlign("center")
	confirmText:setVAlign("middle")
	confirmText:setString(Language:get("uy196")) -- "领 奖"
	self:addChild(confirmText)
	REL:toOther("center", "middle", confirmText, confirm, 0, 15)
	self.confirmText = confirmText

	local rewardTime = XMLLabel()           -- 倒计时
	rewardTime:setContentSize(cc.size(150, 27))
	rewardTime:setHAlign("center")
	rewardTime:setVAlign("middle")
	rewardTime:setString("00:00:00")
	self:addChild(rewardTime)
	REL:toOther("center", "bottom", rewardTime, self, 0, 170)
	self.rewardTime = rewardTime

end

function T:initReward()
	local r = SetStates.giftState:getGift()
	local awardDataDecoder = require"src.app.communication.award.AwardDataDecoder"

	if not r then
		CheckMysteriousGiftCommand.new():exec(function(state, data)
			if state == ReturnCode.SUCCESS then
				SetStates.giftState:setGift(data)

				if not self.isNotCleanuped then
					return
				end

				local layout = awardDataDecoder:award2LayoutContainer(data)
				self:addChild(layout)
				CCUIREL:toOther("center", "bottom", layout, self, 0, 230)
			else
				MUIAlter_noTitle.show({
					info = Language:get("uy136"),
				})

				return
			end

		end)
	else
		local layout = awardDataDecoder:award2LayoutContainer(r)  -- 得到一个物品的layout
		self:addChild(layout)
		CCUIREL:toOther("center", "bottom", layout, self, 0, 230)
	end
end

function T:updateTime(t)
	if not t then
		t = SetStates.giftState:getTime()
	end

	local h = math.floor(t / 3600)
	local m = math.floor(math.floor(t % 3600) / 60)
	local n = math.floor(t % 60)

	if m == 60 then
		m = 59
	end

	if n == 60 then
		n = 59
	end

	self.rewardTime:setString(string.format("%02d", h) .. ":" .. string.format("%02d", m) .. ":" .. string.format("%02d", n))

	if t == 0 then
		self.confirm:setTouchEnabled(true)
		self.confirm:setIsGray(false)
		self.confirm:setEnabled(true)
	else

		self.confirm:setTouchEnabled(false)
		self.confirm:setIsGray(true)
		self.confirm:setEnabled(false)
	end

end

return T













