--region HomePageManagerOutSide.lua
--Author    : YuDing
--Date      : 2014/10/23
--Introduce : 郊外

local HomePageOutSide = class("HomePageOutSide",function()
	local lay = ccui.Layout:create()
	lay:setContentSize(UI_VSIZE)
	return lay
end)

--- 管理
HomePageOutSide.manager = nil

--- 预先加载的资源
HomePageOutSide.preResList = {
	"res/images/homePageOutSide/homePageOutSide.plist", -- 资源地图
}

--- 缩放速度
HomePageOutSide.scaleAcc = 0.05
--- 存放底框的容器
HomePageOutSide.bldBottom = nil

HomePageOutSide.scrollDistanceOfBottom = 77

HomePageOutSide.scrollDistanceOfTop = 114

--- 构造函数
function HomePageOutSide:ctor()
	mapping(self, LuaEventDispatcher)

	local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function( eventType )
				if CocosEvent.ENTER == eventType then
					self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
				end

				self:luaDispatchEvent( eventType )
			end
		)
	end

	self:loadRes()
end

function HomePageOutSide:loadRes()
	--ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE, self, self.loadCompleteHandler)
	self.cfg = HomePageManager.outCfg -- 配置数据

	ResLoader:syncLoad(self.preResList)
end

function HomePageOutSide:onEnter()
	--    AbstractUI.timeout(self,function()
	--        MUIAlter_singleLine.show({
	--		    info=Language:get("uc823"),
	--		    showSecond=1
	--	    })
	--    end, 0)

	if #self.bldBottom > 0 then -- 地板已经创建好
		self:autoTips() -- 自动闪烁
	end

	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_SWITCH_TO_OUTSIDE)
	end

	self:setSchduleSecret()
end

function HomePageOutSide:onExit()
	self:releaseSecretSchdule() -- 放在这里移除就不会出现问题;
end

function HomePageOutSide:initScroll(scroll)
	local height = UI_VSIZE.height - self.scrollDistanceOfBottom - self.scrollDistanceOfTop
	scroll:setPositionY(self.scrollDistanceOfBottom)
	scroll:setContentSize(cc.size(UI_VSIZE.width, height))
end

--- 回收
function HomePageOutSide:cleanUp()
	--PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl .. PlayerInstance.fixedBldLocation.command, self, self.updateOpenBuilding)

	for i = 1, #self.bldBottom do
		self.bldBottom[i]:releaseEvent()   -- 释放 图形界面上的 节点的 事件
	end

	ResLoader:unloadReses(self.preResList)  -- 释放资源
end

--- 初始化
function HomePageOutSide:initView()
	self:createSimulateScroll()     --
	self:initScrollState()        -- 初始化位置


	self:registerPublicEvent()
	self:multiEvent()               -- 多点触摸
	--self:updateOpenBuilding()
	self:initBuilding()

	--self:updateSecret()

	self:initWaterEffect()

	--self:refreshSecret()  -- 更新小秘书

	self:setHomeBaseMoveEnable(true) -- 是否能滑动

	--self:testScroll()
end

function HomePageOutSide:loadJson()
--    local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.homePageOutSide)
--    self.widget = widget
--    self:addChild(widget)
end

--- 点击事件
function HomePageOutSide:onTouchEvent(touch)
	local c = self.uiChild

	-- 传送门
	if self:isTouchRect(self.center, touch) then
		if BGMgr:isGuide() then
			BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_OUTSIDE_HOMEPAGE)
		end

		self:playClickSound() -- 播放声音
		self:onEventTransmit()

		return
	end

	-- 资源建筑
	for i = 1, #self.bldBottom do
		if self:isTouchRect(self.bldBottom[i].touchDelegete, touch) then
			local uiBld=self.bldBottom[i]
			if not uiBld.touch then -- 还未开放
				MUIAlter_singleLine.show({
					info = Language:get("uy104"),
					showSecond = 1,
					popSnd=SoundRes.SND_DISABLED
				})
			else
				if uiBld.build then
					-- 有建筑
					self:onEventOpenBuilded(uiBld)
				else
					-- 没有建筑
					if BGMgr:isGuide() then
						BGMgr:fire(BGMgr.EVT_TGR_UI,uiBld)
					end

					self:onEventOpenWaitBuilding(uiBld)
				end
			end

			return -- 返回 不再进行区域判断
		end
	end
end

--- 点击声音
function HomePageOutSide:playClickSound()
	AudioManager:playEffect(SoundRes.SOUND_COMMON_BUTTON)
end

-- 打开 已经建好了的
function HomePageOutSide:onEventOpenBuilded(t)
	local sid = t.sid
	local l = t.l
	HomeOpenWindowManager:openBuildedResource(sid, l, t.build)
end

-- 打开 正在建设当中的
function HomePageOutSide:onEventOpenWaitBuilding(t)
	local res = self.cfg.pos[t.order].res
	local l = self.cfg.pos[t.order].pId
	HomeOpenWindowManager:openWaitBuildingResource(res, l, t)
end

--- 地板闪烁
function HomePageOutSide:onEventBottomFrameBink(t)
	HomeOpenWindowManager:openWaitBuildingResource(nil, nil, t, true)
end

--- 传送门事件
function HomePageOutSide:onEventTransmit()
	--HomePageManager:switchViewToBase()
	HomeOpenWindowManager:jumpToBase(self.center)
end

--- 是否点击中区域
function HomePageOutSide:isTouchRect(target, touch)
	local locationInNode, rect = self:getTouchRect(target, touch)
	if cc.rectContainsPoint(rect, locationInNode) then
		if target.l then

		end

		return true
	end
	return false
end

--- 获得点击的区域
-- @param target 点击的载体
-- @param touch 世界点
-- @return locationInNode 相对节点坐标系的 点
-- @return rect 点击的有效区域
function HomePageOutSide:getTouchRect(target, touch)
	local locationInNode = target:convertToNodeSpace(touch)   -- 无视锚点
	local s = target:getContentSize()
	local rect = cc.rect(0, 0, s.width, s.height)
	return locationInNode, rect
end

--- 初始化建筑
function HomePageOutSide:initBuilding()
	local cfg = self.cfg
	local center = ccui.ImageView:create()       -- center是固定的
	center:loadTexture(cfg.centerPos.pic, ccui.TextureResType.plistType)
	center:setPosition(cfg.centerPos.p)
	center:setLocalZOrder(1)                     -- 保存当前地图绘制最高
	self.center = center
	self.bld:addChild(center)

	HomePageManager:addEffect(self.center, "transfer")

	self.bldBottom = self.bldBottom or {}

	for k, v in pairs(self.cfg.pos) do
		local b = cc.Sprite:create()
		b:setSpriteFrame(cfg.bottomFrame)
		b:setLocalZOrder(v.show)
		local s = b:getContentSize()

		self.bld:addChild(b)

		REL:toOther("left", "bottom", b, self.bld, v.p.x - s.width / 2, v.p.y - s.height / 2)
		table.insert(self.bldBottom, b)     -- 放入容器里
		self:relationUpgrade(b, v.pId, k)
	end

	if BGMgr:isGuide() then
		BGMgr:regGuideObj(BGMgr.TGR_OUTSIDE_BUILD_LOC_1,self.bldBottom[1])
		BGMgr:regGuideObj(BGMgr.TGR_OUTSIDE_BUILD_LOC_2,self.bldBottom[2])
		BGMgr:regGuideObj(BGMgr.TGR_OUTSIDE_BUILD_LOC_3,self.bldBottom[3])

		BGMgr:regGuideObj(BGMgr.TGR_OUTSIDE_HOMEPAGE,self.center)
	end
end

-- 关联升级条
-- @param node 建筑node
-- @param l location 建筑位置
function HomePageOutSide:relationUpgrade(node, l, order)
	local cls = {}
	cls.bar = nil -- 升级条
	cls.l = l     -- 位置
	cls.touch = false  -- 默认是不可点击
	cls.build = nil   -- 对应的建筑
	cls.order = order -- 遍历表的序号(位置序号)
	cls.mask = nil    -- 创建遮罩

	cls.lvlContent = nil -- 等级内容
	cls.touchDelegete = node -- 触摸载体

	cls.initUnOpenState = function(t) -- 未开放状态
		t.touch = false           -- 不能触摸
		t.touchDelegete = t       -- 触摸载体为自身

		if not t.mask then  -- 创建一个黑色的遮罩
			local m = cc.Sprite:create()
			m:setSpriteFrame(self.cfg.mask)
			t:addChild(m)
			REL:toOther("center", "middle", m, t, 0, 0)
			t.mask = m
		end

		if t.build then  -- 如果有建筑（为以后拆除 功能保留）
			t:removeChild(t.build)
			t.build = nil
		end
	end

	cls.getTouchDelegete = function(t)
		return t.touchDelegete
	end

	cls.initOpenState = function(t)
		t.touch = true -- 可点击
		local lvl = PlayerInstance:getBuildingLvl(t.l)
		if t.mask then -- 移除遮罩
			t.mask:removeFromParent()
			t.mask = nil
		end
		print("建筑已开放,",lvl)
		if lvl >= 1 then
			t:createBuild()    -- 创建建筑
			t:createLvlContent() -- 创建等级描述
		else -- 未有建筑
			t.touchDelegete = t
		end
		if not BuildingsInfoMgr:checkBuildingUpgrade(t.l) then
			t:removeBar()
		end
	end

	--- 根据sid建造
	cls.createBuild = function(t)
		if t.build then
			return
		end

		local sid = PlayerInstance:getBuildingSid(t.l)
		if sid == 0 then
			return
		end

		local res = self.cfg.sidRes[sid]
		if not res then return nil end

		local v = ccui.ImageView:create()
		v:loadTexture(res, ccui.TextureResType.plistType)
		v:setLocalZOrder(1)

		t:addChild(v)
		local s = v:getContentSize()
		v:setPosition(cc.p(self.cfg.bldDs.x + s.width / 2, self.cfg.bldDs.y + s.height / 2)) -- 这里要setPosition才没有问题

		--- 增加特效
		HomePageManager:addEffect(v, sid)

		t.touchDelegete = t               -- 触摸代理
		t.build = v                       -- 建筑
		t.sid = sid                       -- 建筑sid
	end

	--- 创建进度条
	cls.createBar = function(t)
		if t.bar then
			t.bar:removeFromParent()
		end -- 为了容错 才这样写

		t.bar = self:createProgressBar()
		t.build:addChild(t.bar)
		t.bar:setPercent(0)

		REL:toOther("center", "middle", t.bar, t, -5, -20) -- 相对于地板  位置 好控制
	end

	--- 创建等级描述
	cls.createLvlContent = function(t)
		if t.textContent then
			return
		end
		local lvl = PlayerInstance:getBuildingLvl(t.l) -- 得到等级
		if lvl == 0 then
			return
		end

		local sid = PlayerInstance:getBuildingSid(t.l)

		t.textContent = self:createLvlDt(t.build, lvl)
		t.textContent:setLvlText(lvl)
		local size = t.build:getContentSize()

		self:setLvlDtPosition(t.textContent, t.build, sid)
	end

	--- 移除进度条
	cls.removeBar = function(t)
		if not t.bar then
			return
		end
		t.bar:removeFromParent()
		t.bar = nil
	end

	--- 更新进度条
	cls.updateBar = function(t, item)
		if not t.bar or not item then
			return
		end

		local allTime = PlayerInstance:getBuildingCurrentUpgradeTime(t.l) -- 总的时间
		local percent = (allTime - item.remainingSec) / allTime * 100
		if percent > 100 then
			percent = 100
		end
		t.bar:setPercent(percent)
	end

	--- 更新开放
	cls.updateOpen = function(t, sid)
		-- 已经建造好了的
		if t.build then
			return
		end

		-- 表示此处已经开放
		if t.touch then
			return
		end

		if self:getIsBuildedOpen(t.l) then
			t.touch = true
			t:setVisible(true) -- 显示
			t.touchDelegete = t

			if t.mask then
				t.mask:removeFromParent()
				t.mask = nil
			end

		end
	end

	--- 更新升级
	cls.upgrading = function(t, item)
		if not t.sid then
			local sid = PlayerInstance:getBuildingSid(t.l)
			if sid == 0 then return end -- 没有此建筑
			t.sid = sid
		end
		if not t.build then -- 第一次升级时 会建造
			t:createBuild()
		end

		if not t.bar then   -- 升级条创建
			t:createBar()
		end

		t:updateBar(item) -- 实时更新进度条
	end

	--- 更新取消升级
	cls.upgradingCancel = function(t)
		if t.bar then
			t:removeBar()
		end
	end

	--- 升级完成
	cls.upgradingCompelte = function(t)
		if t.bar then
			t:removeBar()
		end

		if not t.build then  -- 表示此处还没有建筑
			t:createBuild()
		end
	end

	--- 更新等级
	cls.updateLvl = function(t)
		local lvl = PlayerInstance:getBuildingLvl(t.l)
		if lvl == 0 then return end
		if not t.textContent then  -- 创建等级描述
			t:createLvlContent()
		else
			t.textContent:setLvlText(lvl)
			REL:toOther("center", "middle", t.textContent.lvlNode, t.textContent, 0, 0)
		end
	end

	--- 检查开放状态
	cls.checkLockState = function(t)
		print("检查开放状态")
		if self:getIsBuildedOpen(t.l) then -- 开放
			t:initOpenState()
		else -- 还未开放
			t:initUnOpenState()
		end
	end

	--- 增加事件
	cls.addEvent = function(t)
		PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl .. PlayerInstance.fixedBldLocation.center, t, t.updateOpen) -- 开放资源点
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_START_LEVEL_UP .. t.l, t, t.upgrading)

		-- 监听指挥中心升级
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME_2 .. t.l, t, t.upgrading) -- 剩余时间
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE .. t.l, t, t.upgradingCompelte) -- 完成
		PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl..t.l, t, t.upgradingCompelte) -- 完成
		PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl .. t.l, t, t.updateLvl) -- 更新等级监听
		BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_CANCEL_LEVEL_UP .. t.l, t, t.upgradingCancel) -- 取消升级

		--断线重连/或重新登陆
		UserInfoMgr:luaAddEventListener(UserInfoMgr.EVT_TYPE_RESET,t,t.checkLockState)
	end
	--- 移除事件
	cls.releaseEvent = function(t)
		PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl .. PlayerInstance.fixedBldLocation.center, t, t.updateOpen)
		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_START_LEVEL_UP .. t.l, t, t.upgrading)

		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME_2 .. t.l, t, t.upgrading) -- 剩余时间
		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE .. t.l, t, t.upgradingCompelte) -- 完成
		PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl..t.l, t, t.upgradingCompelte) -- 完成
		PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl .. t.l, t, t.updateLvl) -- 更新等级监听
		BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_CANCEL_LEVEL_UP .. t.l, t, t.upgradingCancel) -- 取消升级

		UserInfoMgr:luaRemoveEventListener(UserInfoMgr.EVT_TYPE_RESET,t,t.checkLockState)
	end

	mapping(node, cls)      -- 复制cls中元素到node中

	node:checkLockState()   -- 先初始化
	node:addEvent()         -- 增加事件
end

--- 设置等级载体位置
-- @param lvlDt     等级载体
-- @param toNode    相对的节点
-- @param sid
function HomePageOutSide:setLvlDtPosition(lvlDt, toNode, sid)
	lvlDt:setScale(0.8)
	REL:toOther("left", "bottom", lvlDt, toNode, 126, 10)

	REL:toOther("center", "middle", lvlDt.lvlNode, lvlDt, 0, 0)
	--end
end

function HomePageOutSide:createProgressBar()
	--- 创建进度条
	--if t.bar then return end

	local size = ResLoader:getSpriteFrame("homePageBldLevelUpFrame.png"):getOriginalSize()

	local c = ccui.Layout:create()
	c:setContentSize(size)

	local back = ccui.ImageView:create()
	back:loadTexture("homePageBldLevelUpFrame.png", ccui.TextureResType.plistType)
	c:addChild(back)
	back:setPosition(cc.p(size.width / 2, size.height / 2))


	local uiProgress = cc.ProgressTimer:create(cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame("homePageBldLevelupBar.png")))
	uiProgress:setBarChangeRate(cc.p(1,0))--进度条动画方向，（1,0）表示横方向，（0,1）表示纵方向
	uiProgress:setType(cc.PROGRESS_TIMER_TYPE_BAR)--进度条类型
	uiProgress:setMidpoint(cc.p(0,1))--进度条的起始点，（0,y）表示最左边，（1,,y）表示最右边，（x,1）表示最上面，（x,0）表示最下面。
	uiProgress:setPercentage(0)--初始值
	uiProgress:setAnchorPoint(0, 0)


	local us = uiProgress:getContentSize()
	uiProgress:setPosition((size.width - us.width) / 2, (size.height - us.height) / 2)

	back:addChild(uiProgress)

	c.progress = uiProgress

	function c.setPercent(t, num)     -- 重新赋值一个函数
		if num > 100 then
			num = 100
	end
	t.progress:setPercentage(num)
	end

	return c
end


--- 创建等级载体
function HomePageOutSide:createLvlDt(parent)
	local lvlDt = ccui.ImageView:create()
	lvlDt:loadTexture(self.cfg.lvlRes, ccui.TextureResType.plistType)
	lvlDt:setLocalZOrder(2)
	local size = lvlDt:getContentSize()

	local lvl = ccui.Text:create()
	lvl.fontType = "default" -- 可以为数字 也可以为其他标识
	Cocos2dxUtils:setTextFont(lvl)
	lvl:setLocalZOrder(2)
	--lvlDt:addChild(lvl)
	parent:addChild(lvlDt)
	parent:addChild(lvl)

	lvlDt.lvlNode = lvl

	REL:toOther("center", "middle", lvl, lvlDt, 0, 0)  -- -8,-16为微调 总感觉这里的addChild会出问题

	lvlDt.setLvlText = function(t, num)     -- 设置文本
		t.lvlNode:setString(tostring(num))
	end

	return lvlDt
end


--- 检测是否开放
-- @param l  位置
function HomePageOutSide:getIsBuildedOpen(l)
	local cLvl = PlayerInstance:getBuildingLvl(PlayerInstance.fixedBldLocation.center)
	local lv = self.cfg.posReverse[l]
	if cLvl >= lv then
		return true
	else
		return false
	end
end

--- 多点触摸
function HomePageOutSide:multiEvent()
	-- 记录多点触控距离
	local distance
	local touchTag
	local movePoint

	-- 触摸组
	local tbTouch = {}
	-- 触摸开始监听
	local function onTouchesBegan(touches,event)
		local size = #touches
		if size > 1 then
			--print("touch begin size = " .. size)
			local t1 = touches[1]:getLocation()
			local t2 = touches[2]:getLocation()

			distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)

			for i = 1, #touches do  -- ios 一次能传多个点?
				table.insert(tbTouch, touches[i])
			end
		else
			--print("simple touch....")
			if #tbTouch == 0 then
				--self.midPoint = nil
				table.insert(tbTouch, touches[1])
				movePoint = touches[1]:getLocation()
				touchTag = true
			elseif #tbTouch == 1 then -- 两个点
				touchTag = false
				table.insert(tbTouch, touches[1])
				local t1 = tbTouch[1]:getLocation()
				local t2 = tbTouch[2]:getLocation()
				distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)
			elseif #tbTouch > 1 then   -- 大于两个点就不处理
				--self.midPoint = nil
				touchTag = false
				table.insert(tbTouch, touches[1])
				distance = nil
			end
		end
	end
	--触摸移动监听
	local function onTouchesMoved(touches,event)
		--print("move  = " .. #touches)
		local size = #touches
		if size > 1 then
			--缩放
			touchTag = false

			if #touches > 2 then   -- 大于两指
				return
			end

			if not distance then return end -- 没有距离也退出

			local t1 = touches[1]:getLocation()
			local t2 = touches[2]:getLocation()
			local tempdistance = math.pow(t1.x - t2.x, 2) + math.pow(t1.y - t2.y, 2)
			local temp = math.sqrt(math.abs(tempdistance - distance))

			if not self.midPoint then  -- 如果没有中间点 表示第一次开始放大的时候， 就是我们需要关注的点， 最好是在滑动的时候判定
				self.midPoint = cc.pMidpoint(t1, t2)
				self.containerMidPoint = self.scroll:getContainer():convertToNodeSpace(self.midPoint)
			end

			if self.moveEable == false then  -- 不能滑动
				return
			end

			if temp > 2 then -- 2个像素的差异 就是缩放
				if tempdistance > distance then
					self:excutScale(self.scaleAcc, self.midPoint)
			else
				self:excutScale(-self.scaleAcc, self.midPoint)
			end

			distance = tempdistance
			end

		else
			if not self.midPoint then self.midPoint = nil end
			if #tbTouch >= 2 then return end                         -- 单点滑动  但是触摸点有多个


			local now = touches[1]:getLocation()
			local old = touches[1]:getPreviousLocation()

			if movePoint then
				if touchTag == true then -- 表示当前还是单点
					local dx = math.abs(now.x - movePoint.x)
					local dy = math.abs(now.y - movePoint.y)
					if dx > 2.0 or dy > 2.0 then
						touchTag = false -- 表示点击的范围属于滑动
					else
						touchTag = true  -- 属于点击事件
						movePoint = now
					end
				end
			end

			local ax = now.x - old.x
			local ay = now.y - old.y

			--            local px, py = self.scroll.container:getPosition()
			--            print("outside scroll, p = " .. px .. ", " .. py)

			if self.moveEable == false then  -- 不能滑动
				return
			end

			self:excutMove(ax, ay)
			oneTouch = touches[1]:getLocation()
		end
	end

	local function onTouchesCancel(touches, event)
		movePoint = nil
		for k = 1, #touches do
			for i = 1, #tbTouch do
				if touches[k] == tbTouch[i] then
					table.remove(tbTouch, i)

					if #tbTouch == 2 then -- 由三个点变成两个点 算出两点的距离
						local t1 = tbTouch[1]:getLocation()
						local t2 = tbTouch[2]:getLocation()
						distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)
					else
						distance = nil
						self.midPoint = nil -- 不是两点的 中间点都置空
					end

					break
				end
			end
		end
	end

	--触摸结束监听
	local function onTouchesEnded(touches, event)
		movePoint = nil
		--print("touches end = " .. #touches)
		for k = 1, #touches do
			for i = 1, #tbTouch do
				if touches[k] == tbTouch[i] then
					table.remove(tbTouch, i)

					if #tbTouch == 2 then -- 由三个点变成两个点 算出两点的距离
						local t1 = tbTouch[1]:getLocation()
						local t2 = tbTouch[2]:getLocation()
						distance = math.pow(t2.x - t1.x, 2) + math.pow(t2.y - t1.y, 2)
					elseif #tbTouch == 0 then  -- 这个就是点击事件
						if touchTag == true then -- 表示可以处理单击事件
							self:onTouchEvent(touches[1]:getLocation()) -- 点击事件
					end
					distance = nil
					self.midPoint = nil   -- 中间点置空
					else
						distance = nil
						self.midPoint = nil  -- 中间点置空
					end

					break
				end
			end

		end


	end

	local listener = cc.EventListenerTouchAllAtOnce:create()
	listener:registerScriptHandler(onTouchesBegan,cc.Handler.EVENT_TOUCHES_BEGAN)
	listener:registerScriptHandler(onTouchesMoved,cc.Handler.EVENT_TOUCHES_MOVED)
	listener:registerScriptHandler(onTouchesEnded,cc.Handler.EVENT_TOUCHES_ENDED)
	listener:registerScriptHandler(onTouchesCancel,cc.Handler.EVENT_TOUCHES_CANCELLED)

	local eventDispatcher = self:getEventDispatcher()
	--eventDispatcher:removeEventListenersForTarget(self.widget)  -- 删除这个节点上的listener.. 貌似加载的时候会加一个listener。
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

--- 初始化水波特效
function HomePageOutSide:initWaterEffect()
	local size = self.backWater:getContentSize()

	--关闭抗锯齿
	ResLoader:getSpriteFrame("waterFlush_00000.png"):getTexture():setAliasTexParameters()

	local widthNum = math.floor(size.width / 128)
	local heightNum = math.floor(size.height / 128)

	for i = 1, heightNum do  -- 暂时使用全覆盖的方式
		for j = 1, widthNum do
			local eff = HomePageManager:getEffWithOutRandom("waterFlushEffect")
			self.backWater:addChild(eff)
			eff:setPosition(cc.p((j - 1) * 128 + 128 / 2, (i - 1) * 128 + 128 / 2))
	end
	end

	--    --显示热点区域
	--    local dn=cc.DrawNode:create()
	--    dn:clear()
	--    self:addChild(dn)
	--    dn:drawPolygon(v.hitArea,4,cc.c4b(0xff,0x00,0xff,0x44),1,cc.c4b(0xff,0x00,0x00,0x44))
	--    v.hitArea={
	--			cc.p(x+6,y+88),
	--			cc.p(x+138,y+12),
	--			cc.p(x-18,y-102),
	--			cc.p(x-156,y-22)
	--		}

end

--- 注册事件
function HomePageOutSide:registerPublicEvent()
--PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl .. PlayerInstance.fixedBldLocation.center, self, self.updateOpenBuilding)  -- 指挥中心
--   PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.buildingLvl, self, self.refreshSecret)

--SetStates.setCommonState:luaAddEventListener(SetStates.setCommonState.EVT_CLOSE_OPEN_NEW_RESOURCE, self, self.updateSecret)  -- 小秘书

-- BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME, self, self.refreshSecret) -- 剩余时间
-- BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE, self, self.refreshSecret) -- 完成
-- BuildingsInfoMgr:luaAddEventListener(BuildingsInfoMgr.EVT_TYPE_START_LEVEL_UP, self, self.refreshSecret) -- 完成
end

function HomePageOutSide:releasePublicEvent()
--PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl .. PlayerInstance.fixedBldLocation.center, self, self.updateOpenBuilding)  -- 指挥中心
--  PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.buildingLvl, self, self.refreshSecret)

--SetStates.setCommonState:luaRemoveEventListener(SetStates.setCommonState.EVT_CLOSE_OPEN_NEW_RESOURCE, self, self.updateSecret)  -- 小秘书

-- BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_UPDATE_REMAINING_TIME, self, self.refreshSecret) -- 剩余时间
-- BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_LEVEL_UP_COMPLETE, self, self.refreshSecret) -- 完成
-- BuildingsInfoMgr:luaRemoveEventListener(BuildingsInfoMgr.EVT_TYPE_START_LEVEL_UP, self, self.refreshSecret) -- 完成
end


--- 初始化位置
function HomePageOutSide:initScrollState()
	self.scroll.container:setPosition(cc.p(-37, -298))

	if UI_VSIZE.width == 640 and UI_VSIZE.height == 960 then
		self.scroll.container:setPosition(cc.p(-21, -265))
	else
		self.scroll:setMinScale(0.85)

		self.scroll.container:setPosition(cc.p(-10, -72))

	end
end


function HomePageOutSide:excutScale(scale, mid)
	self:setScrollScale(self.scroll, scale, mid)
	--self:excutScaleEffect()  -- 需要在改变了scale之后调用
end

--- 增加倍率
-- @param scroll 滑动容器
-- @param scale 增量
-- @param mid 世界中间点
function HomePageOutSide:setScrollScale(scroll, scale, mid)

	local container = scroll:getContainer()

	local os = scroll:getScrollScale()
	scroll:setScrollScale(scale)
	local ns = scroll:getScrollScale()


	if scale > 0 then -- 放大， scroll向负方向移动（自建坐标系观察）
		if ns > os then -- 有效的缩放（考虑到有最大值的情况） math.abs(os - ns) 可能不等于scale
			local wp = container:convertToWorldSpace(self.containerMidPoint) -- 转换为世界坐标
			local p = cc.pSub(mid, wp)  -- 计算偏移相应的距离
			self:excutMove(p.x, p.y) -- 偏移
	end
	elseif scale < 0 then
		if ns < os then -- 缩小， scroll向正方向移动
			local wp = container:convertToWorldSpace(self.containerMidPoint) -- 转换为世界坐标
			local p = cc.pSub(mid, wp) -- 计算偏移相应的距离
			self:excutMove(p.x, p.y) -- 偏移
		end
	end
end

-- 移动的增量
function HomePageOutSide:excutMove(ax, ay)
	if not ax or not ay then return end
	self.scroll:addScrollPosition(ax, ay)
end

--- 创建小秘书
function HomePageOutSide:createSecret()

	if not self.secret then  -- 头像
		local head = ccui.ImageView:create()
		head:loadTexture("secretBigger.png",ccui.TextureResType.plistType)

		self:addChild(head)
		REL:toOther("left", "bottom", head, self, 70, 140)

		self.secret = head

		BaseButton:extend(head)
		head:addTouchEventListener(function(t, e)
			if e ~= ccui.TouchEventType.ended then
				return
			end
			self:autoOpen()
		end)

		-- 文本框
		local defalutSize = cc.size(480,42)
		local spriteFrame = ResLoader:getSpriteFrame("guideTipBack.png")
		local spriteFrameSize = spriteFrame:getOriginalSize()
		-- 这个Scale9Sprite是cocos2dx库所带的九宫格，因为前面有cc
		--local scale9Sprite = cc.Scale9Sprite:createWithSpriteFrame(spriteFrame,cc.rect(spriteFrameSize.width/2,37,1,1))
		local scale9Sprite = Scale9Sprite:createWithFrame(spriteFrame,cc.rect(spriteFrameSize.width/2,37,1,1))
		scale9Sprite:setContentSize(defalutSize)
		head:addChild(scale9Sprite)


		head.s9Sprite = scale9Sprite
		REL:toOther("left", "middle", scale9Sprite, head, 140, 0)

		-- 文本内容
		local str = Language:get("uy127", self:getNotBuildNum())
		local content = XMLLabel()
		content:setWidth(300)
		content:setString(str)
		content:updateView()

		local width = content:getWidth()
		local height = content:getHeight()

		head:addChild(content)

		head.content = content

		scale9Sprite:setContentSize(cc.size(width + 40, height + 20))

		REL:toOther("left", "middle", scale9Sprite, head, 140, 0)

		REL:toOther("left", "middle", content, scale9Sprite, 20, 0)

		--- 触摸载体
		local lay = ccui.Layout:create()
		lay:setTouchEnabled(true)
		BaseButton:extend(lay)
		lay:addTouchEventListener(function(t, e)    -- 事件
			if e ~= ccui.TouchEventType.ended then
				return
		end
		self:autoOpen()
		end)
		head:addChild(lay)

		REL:toOther("left", "middle", lay, head, 140, 0) -- 位置
		REL:relOther(lay, scale9Sprite, 0, 0, 0, 0) -- 大小

		head.touchLay = lay

		local close = ccui.ImageView:create()
		close:loadTexture("homeRightTopClose.png", ccui.TextureResType.plistType)
		head:addChild(close)

		BaseButton:extend(close)
		close:addTouchEventListener(function(t, e)    -- 事件
			if e ~= ccui.TouchEventType.ended then
				return
		end
		local func = function(s)
			if s == FLAG_OK then
				SetStates.setCommonState:setClosedValue(true, SetStates.setCommonState.CLOSE_OPEN_NEW_RESOURCE)
				Command:SetCommonCommand()
			end

		end

		MUIAlter.show(
			{
				info = Language:get("uy128"),
				size = 30,
				flag = 3,
				callback = func,
			}
		)

		end)

		local cs = ResLoader:getSpriteFrame("homeRightTopClose.png"):getOriginalSize()
		REL:toOther("right", "top", close, lay, -9, -30) -- 位置

		head.closeBtn = close

	end

end

--- 移除小秘书
function HomePageOutSide:removeSecret()
	if self.secret then
		self.secret:removeFromParent()
		self.secret = nil
	end
end

--- 更新小秘书
function HomePageOutSide:updateSecret()
	local state = SetStates.setCommonState:getClosedValue(SetStates.setCommonState.CLOSE_OPEN_NEW_RESOURCE)
	if state == true then  -- 关闭
		self:removeSecret()
	else
		if self:getNotBuildNum() == 0 then  -- 已经建造完了
			self:removeSecret()
		else
			self:createSecret()
		end
	end
end

--- 获取还未建造的数目
function HomePageOutSide:getNotBuildNum()
	local num = 0

	for k, v in ipairs(self.bldBottom) do
		if self:getIsBuildedOpen(v.l) and PlayerInstance:getBuildingLvl(v.l) == 0 and not v.build  then  -- 位置 开放 等级 正在升级
			num = num + 1
		end
	end

	--print("建筑物数量为: " .. num)

	return num
end

--- 设置小秘书 计时器
function HomePageOutSide:setSchduleSecret()
	if self.secretSchduleId then
		return
	end

	local func = function()
		self:refreshSecret()
	end

	self.secretSchduleId = Cocos2dxUtils:interval(func, 0.5)
end

--- 释放定时器
function HomePageOutSide:releaseSecretSchdule()
	if self.secretSchduleId then
		print("移除 小秘书 定时器")
		Cocos2dxUtils:clearScheduler(self.secretSchduleId)
		self.secretSchduleId = nil
	end
end

function HomePageOutSide:refreshSecret()
	-- 首先获取文本内容的高度，宽度

	self:updateSecret()

	--print("secret is fresh!")
	if self.secret then
		local head = self.secret
		local str = Language:get("uy127", self:getNotBuildNum())
		local content = self.secret.content
		local scale9Sprite = self.secret.s9Sprite
		local lay = self.secret.touchLay
		local close = self.secret.closeBtn
		content:setString(str)
		content:updateView()

		local width = content:getWidth()
		local height = content:getHeight()

		scale9Sprite:setContentSize(cc.size(width + 40, height + 20))

		REL:toOther("left", "middle", scale9Sprite, head, 140, 0)

		REL:toOther("left", "middle", content, scale9Sprite, 20, 0)

		REL:toOther("left", "middle", lay, head, 140, 0) -- 位置
		REL:relOther(lay, scale9Sprite, 0, 0, 0, 0) -- 大小

		local cs = ResLoader:getSpriteFrame("homeRightTopClose.png"):getOriginalSize()
		REL:toOther("right", "top", close, lay, -9, -30) -- 位置
	end

end


--- 自动打开 建筑物列表
function HomePageOutSide:autoOpen()
	local normal = {}
	local extra = {}

	for k, v in ipairs(self.bldBottom) do
		if v.l < 130 and self:getIsBuildedOpen(v.l) and PlayerInstance:getBuildingLvl(v.l) == 0 and not v.build  then  -- 位置 开放 等级 正在升级
			table.insert(normal, v)
		elseif v.l >= 130 and self:getIsBuildedOpen(v.l) and PlayerInstance:getBuildingLvl(v.l) == 0 and not v.build then
			table.insert(extra, v)
		end
	end

	if #extra >= 1 then
		self:onEventOpenWaitBuilding(extra[1])
	elseif #normal >= 1 then
		self:onEventOpenWaitBuilding(normal[1])
	end
end

--- 切换到资源地图 自动提示
function HomePageOutSide:autoTips()
	local normal = {}
	local extra = {}

	for k, v in ipairs(self.bldBottom) do
		if v.l < 130 and self:getIsBuildedOpen(v.l) and PlayerInstance:getBuildingLvl(v.l) == 0 and not v.build  then  -- 位置 开放 等级 正在升级
			table.insert(normal, v)
		elseif v.l >= 130 and self:getIsBuildedOpen(v.l) and PlayerInstance:getBuildingLvl(v.l) == 0 and not v.build then
			table.insert(extra, v)
		end
	end

	if #extra >= 1 then  -- 地板闪烁
		self:onEventBottomFrameBink(extra[1])
	elseif #normal >= 1 then
		self:onEventBottomFrameBink(normal[1])
	end
end

--- 创建模拟的scroll
function HomePageOutSide:createSimulateScroll()
	local cfg = self.cfg

	local panelMain = ccui.Layout:create()
	panelMain:setContentSize(cc.size(UI_VSIZE.width, UI_VSIZE.height))
	self:addChild(panelMain)

	--    local backWater = ccui.ImageView:create()  -- 背景中的水
	--    backWater:loadTexture(self.cfg.backWaterImg, ccui.TextureResType.plistType)
	--    backWater:setAnchorPoint(cc.p(0, 0))
	--    self.backWater = backWater

	--- 896 1080
	self.backWater = ccui.Layout:create() -- 特效 和 碎片图的容器
	self.backWater:setContentSize(cc.size(896, 1080))

	--local size = backWater:getContentSize()
	local container = ccui.Layout:create() -- 主容器
	container:setContentSize(cc.size(896, 1080))
	container:addChild(self.backWater)

	REL:toOther("left", "bottom", self.backWater, container, 0, 0)

	local fragWater = cfg.fragWater -- 添加碎片图
	for i = 1, #fragWater do
		local img = ccui.ImageView:create()  -- 背景中的 建筑地板
		img:loadTexture(fragWater[i].name, ccui.TextureResType.plistType)
		self.backWater:addChild(img)
		REL:toOther("left", "bottom", img, self.backWater, fragWater[i].x, fragWater[i].y)
	end

	local bld = ccui.ImageView:create()  -- 背景中的 建筑地板
	bld:loadTexture(self.cfg.backImg, ccui.TextureResType.plistType)
	bld:setAnchorPoint(cc.p(0, 0))
	self.bld = bld
	container:addChild(bld)

	panelMain:addChild(container)

	self:initScroll(panelMain)

	self.scroll = require("src.app.uiScript.scenes.homePage.HomePageBaseSimulateScroll").new(panelMain, container)

	self.scroll:setMinScale(0.8) -- 设置缩小比

	self.container = container

end


function HomePageOutSide:setManager(m)
	self.manager = m
end


function HomePageOutSide:setHomeBaseMoveEnable(able)
	self.moveEable = able
end

function HomePageOutSide:testScroll()

	local add = ccui.Button:create()
	add:loadTextures("buttonRed_normal.png", "buttonRed_normal.png", "",ccui.TextureResType.plistType)
	self.scroll.scroll:addChild(add)
	REL:toOther("right", "middle", add, self.scroll.scroll, 0, -200)
	add:setLocalZOrder(10000)
	add:addTouchEventListener(function(t, e)
		if ccui.TouchEventType.ended ~= e then return end
		self:subScrollEvent()
		--self:setHomeBaseMoveEnable(true)
		--HomePageManager:setBottomBarMoveEable(true)
		--print("回复滑动")
	end)

	local sub = ccui.Button:create()
	sub:loadTextures("buttonInc_normal.png", "buttonInc_normal.png", "", ccui.TextureResType.plistType)
	self.scroll.scroll:addChild(sub)
	REL:toOther("right", "middle", sub, self.scroll.scroll, 0, -250)
	sub:setLocalZOrder(10000)
	sub:addTouchEventListener(function(t, e)
		if ccui.TouchEventType.ended ~= e then return end
		self:addScrollEvent()
		--self:setHomeBaseMoveEnable(false)
		--HomePageManager:setBottomBarMoveEable(false)
		--print("禁用滑动")
	end)

end

function HomePageOutSide:addScrollEvent(scale)
	local sp = cc.p(320, 300)
	self.containerMidPoint = self.scroll:getContainer():convertToNodeSpace(sp)

	if not scale then
		scale = self.scaleAcc
	end

	self:excutScale(scale, sp)
end

function HomePageOutSide:subScrollEvent(scale)
	local sp = cc.p(320, 300)

	self.containerMidPoint = self.scroll:getContainer():convertToNodeSpace(sp)

	if not scale then
		scale = self.scaleAcc
	end

	self:excutScale(-scale, sp)

	--    print("middle scale is -->" .. self.middleScroll:getScrollScale())
	--    print("back scale is -->" .. self.backScroll:getScrollScale())
	--    print("far scale is -->" .. self.farScroll:getScrollScale())

end

return HomePageOutSide
--endregion
