--region HomePageReinforceWindow.lua
--Author    : YuDing
--Date      : 2015/3/12
--Introduce : 援军窗口


local T = class("HomePageReinforceWindow", function() return ccui.Layout:create() end)

--- json文件
T.widget = nil

--- 子类容器
T.uiChild = nil

function T:ctor()
    mapping( self, LuaEventDispatcher)
    
    local handler = self.registerScriptHandler
    if handler then
        self:registerScriptHandler(
            function(eventType)
               if CocosEvent.ENTER == eventType then
                    self:onEnter()
               elseif CocosEvent.EXIT == eventType then
                    self:onExit()
               elseif eventType == CocosEvent.CLEAN_UP then
                    self:cleanUp()
                end
               self:luaDispatchEvent(eventType)
            end
         )    
    end

    self:loadJson()
    self:loadUI()
    self:updateReinforce()
    self:registerLocalEvent()
    self:registerPublicEvent()
    self:initReward()  -- 初始化奖品

end

function T:onEnter()

end

function T:onExit()

end

function T:cleanUp()
    self:releasePublicEvent()
end

function T:loadUI()
    self.uiChild = self.uiChild or {}
    local c = self.uiChild  
    local w = self.widget
    
    c.title = Cocos2dxUtils:getChildByName(w, "title") -- 标题
    c.content = Cocos2dxUtils:getChildByName(w, "content")        -- 描述内容
    c.receiveTag = Cocos2dxUtils:getChildByName(w, "receiveTag")   -- 标签
    c.receive = Cocos2dxUtils:getChildByName(w, "receive")        -- 领取 按钮
    c.receiveText = Cocos2dxUtils:getChildByName(w, "receiveText")        -- 领取文字
    c.close = Cocos2dxUtils:getChildByName(w, "close") -- 关闭按钮
    c.name = Cocos2dxUtils:getChildByName(w, "name") -- 名字
    c.planeContainer = Cocos2dxUtils:getChildByName(w, "planeContainer") -- 飞机容器

    self:initFixedUI()
end

--- 加载json
function T:loadJson()
    local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.homePageReinforceWindow)

    self.widget = widget
    self:setContentSize(widget:getContentSize())
    self:addChild(widget)
    CCUIREL:toOther("center", "middle", widget, self, 0, 0)

end

function T:initFixedUI()
    local c = self.uiChild

    local tb = {}
    for k, v in pairs(self.uiChild) do
        if v:getDescription() == "Label" then
            table.insert(tb, v)
        end
    end

    c.title:setString(Language:get("uy132"))
    c.content:setString(Language:get("uy133"))
    c.receiveTag:setString(Language:get("uy134"))
    c.receiveText:setString(Language:get("uy135"))
    c.name:setString(Language:get("uy134")) -- "援 军"

    Cocos2dxUtils:setTextFont(tb)
end

function T:registerLocalEvent()
    local c = self.uiChild 

    BaseButton:extend(c.receive)
    c.receive:addTouchEventListener(function(t, e)
            if e ~= ccui.TouchEventType.ended then return end 
            self.__hide__() -- 关闭窗口

            GetReinforceCommand.new():exec(function(state)
                 if state == ReturnCode.SUCCESS then -- 成功
                     
                 end
            end)
    end)

    BaseButton:extend(c.close)
    c.close:addTouchEventListener(function(t, e)
            if e ~= ccui.TouchEventType.ended then return end 
            self.__hide__() -- 关闭窗口
    end)
end

function T:registerPublicEvent()
    SetStates.reinforceState:luaAddEventListener(SetStates.reinforceState.REMAIN_TIME, self, self.updateReinforce) -- 援军

    PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.fighting,self,self.fightRefresh) -- 战斗力刷新
end

function T:releasePublicEvent()
    SetStates.reinforceState:luaRemoveEventListener(SetStates.reinforceState.REMAIN_TIME, self, self.updateReinforce) -- 援军

    PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.fighting,self,self.fightRefresh)
end

function T:fightRefresh()
    CommonMethod:fightRefresh()
end

--- 更新
function T:updateReinforce(t)
    local c = self.uiChild
    if not t then 
        t = SetStates.reinforceState:getTime()
    end

    if t == 0 then  -- 已到达
        c.receive:setTouchEnabled(true)
        c.receive:setIsGray(false)
        c.receive:setEnabled(true)
        c.content:setString(Language:get("uy133"))
    else -- 没有到达
        c.receive:setTouchEnabled(false)
        c.receive:setIsGray(true)
        c.receive:setEnabled(false)
        c.content:setString(Language:get("uy192"))
    end
end

function T:initReward()
    local r = SetStates.reinforceState:getGift()
    local awardDataDecoder = require"src.app.communication.award.AwardDataDecoder"

    if not r then 
        CheckReinforceCommand.new():exec(function(state, data)

            if state == ReturnCode.SUCCESS then
                r = data
                SetStates.reinforceState:setGift(data)

                local layout = awardDataDecoder:award2LayoutContainer(r)
                self.uiChild.planeContainer:addChild(layout)
                CCUIREL:toOther("center", "middle", layout, self.uiChild.planeContainer, 0, 0) 

            else
--                MUIAlter_noTitle.show(
--                    {
--                        info = Language:get("uy136"),
--                    }
--                )
                return
            end

        end)
    else 
        local layout = awardDataDecoder:award2LayoutContainer(r)  -- 得到一个物品的layout
        self.uiChild.planeContainer:addChild(layout)
        CCUIREL:toOther("center", "middle", layout, self.uiChild.planeContainer, 0, 0)       
    end
end

return T

--endregion
