--region HomePageWorldCollectionView.lua
--Author    : YuDing
--Date      : 2014/11/19
--Introduce : 收集界面

local HomePageWorldCollectionView = class("HomePageWorldCollectionView", function() 
        local o = AbstractWindow.new(1)
        o.super = AbstractWindow
        return o
    end)

local cellView = require("src.app.uiScript.scenes.homePage.world.HomePageWorldCollectionCellView")

-- ui组
HomePageWorldCollectionView.uiChild = {}
-- 管理
HomePageWorldCollectionView.manager = nil
-- 根json
HomePageWorldCollectionView.widget = nil
-- 点击的标示
HomePageWorldCollectionView.clickType = nil
-- 上次点击
HomePageWorldCollectionView.lastClick = nil
-- 缓冲栈
HomePageWorldCollectionView.stack = nil
-- cell容器
HomePageWorldCollectionView.listViewContainer = nil

function HomePageWorldCollectionView:ctor()
    mapping(self, LuaEventDispatcher)
    local handler = self.registerScriptHandler
    if handler then
        self:registerScriptHandler(
            function(eventType)
               if CocosEvent.ENTER == eventType then
                    --self:onEnter()
               elseif CocosEvent.EXIT == eventType then
                    --self:onExit()
               elseif eventType == CocosEvent.CLEAN_UP then
				    self:cleanUp()
               end
               
               self:luaDispatchEvent(eventType)
            end
         )    
    end

    local f = ResLoader:getSpriteFrame("viewBack.png")
    local s = f:getOriginalSize()
    local viewBack = Scale9Sprite:createWithFrame(f,cc.rect(s.width/2,s.height-1,1,1))
	self:addChild(viewBack)
    REL:relOther(viewBack,self,0,0,73 - 4,0)  -- 73是切换按钮的高度
	self.viewBack = viewBack

end

function HomePageWorldCollectionView:cleanUp()
    self:cleanEvent()
end


-- 加载ui
function HomePageWorldCollectionView:loadJson()
    local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.homePageWorldCollectionView)
    self.widget = widget
    self:addChild(widget)

    self.manager = HomePageManager -- 暂时适应
end

function HomePageWorldCollectionView:loadUI()
    local c = self.uiChild 
    local w = self.widget
    --c.imgClose = Cocos2dxUtils:getChildByName(w, "imgClose")
    c.imgAll = Cocos2dxUtils:getChildByName(w, "imgAll")   -- 所有选项
    c.allText = Cocos2dxUtils:getChildByName(w, "allText")

    c.imgResource = Cocos2dxUtils:getChildByName(w, "imgResource")  -- 资源
    c.resourceText = Cocos2dxUtils:getChildByName(w, "resourceText")

    c.imgAgainst = Cocos2dxUtils:getChildByName(w, "imgAgainst") -- 敌对
    c.againstText = Cocos2dxUtils:getChildByName(w, "againstText")

    c.imgFriend = Cocos2dxUtils:getChildByName(w, "imgFriend") -- 朋友
    c.friendText = Cocos2dxUtils:getChildByName(w, "friendText")

    c.bottom = Cocos2dxUtils:getChildByName(w, "bottom") -- 底部容器

    self:initScroll()
    
    self:initFixedUI()

    self:updateClickContent()
end

function HomePageWorldCollectionView:initFixedUI()
    local c = self.uiChild

    local tb = {} 
    for k, v in pairs(self.uiChild) do 
        if v:getDescription() == "Label" then 
            table.insert(tb, v)
        end
    end
    Cocos2dxUtils:setTextFont(tb)

    self:setWindowTitle(Language:get("uy82"))  -- 标题

    c.allText:setString(Language:get("uy83"))
    c.resourceText:setString(Language:get("uy84"))
    c.friendText:setString(Language:get("uy85"))
    c.againstText:setString(Language:get("uy86"))
    
    -- 底部
    REL:toScreen("left", "bottom", c.bottom, 0, 0)
end

function HomePageWorldCollectionView:registerLocalEvent()
    local c = self.uiChild
    local clickHandler = function(t, e)
        if e ~= ccui.TouchEventType.ended then return end 

        if t == c.imgAll then
            self:imgAllEventClick()
        elseif t == c.imgResource then
            self:imgResourceEventClick()
        elseif t == c.imgAgainst then
            self:imgAgainstEventClick()
        elseif t == c.imgFriend then
            self:imgFriendEventClick()
        end

    end

    BaseButton:extend(c.imgAll)
    BaseButton:extend(c.imgResource)
    BaseButton:extend(c.imgAgainst)
    BaseButton:extend(c.imgFriend)

    c.imgAll:addTouchEventListener(clickHandler)
    c.imgResource:addTouchEventListener(clickHandler)
    c.imgAgainst:addTouchEventListener(clickHandler)
    c.imgFriend:addTouchEventListener(clickHandler)

end

function HomePageWorldCollectionView:initScroll()
    if not self.scroll then 
        self.scroll = ccui.ScrollView:create()
        self:addChild(self.scroll)
        self.scroll:setDirection(cc.SCROLLVIEW_DIRECTION_VERTICAL)
        self.scroll:setClippingEnabled(true)
        self.scroll:setBounceEnabled(true)
        self.scroll:setTouchEnabled(true)

        -- scroll 232 的高度 left top toscreen 232

--        self.scroll:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
--		self.scroll:setBackGroundColor(cc.c3b(0xff,0,0))
--		self.scroll:setBackGroundColorOpacity(100)

        REL:relScreen(self.scroll, 0, 0, 90, 130)       -- 上边90 下边130
    end
end


function HomePageWorldCollectionView:registerPublicEvent()
     HomePageManager:luaAddEventListener("worldMapCollection", self, self.updateViewData)  -- 增加更新的派发
end

function HomePageWorldCollectionView:releasePublicEvent()
     HomePageManager:luaRemoveEventListener("worldMapCollection", self, self.updateViewData)  -- 删除更新的派发
end

function HomePageWorldCollectionView:imgAllEventClick()
    if self.clickType.all == true then 
        return
    else 
        self.clickType.all = true
        self:updateClickType(self.uiChild.imgAll)
        self:updateViewData()
    end
end

function HomePageWorldCollectionView:imgResourceEventClick()
    if self.clickType.resource == true then 
        return
    else 
        self.clickType.resource = true
        self:updateClickType(self.uiChild.imgResource)
        self:updateViewData()
    end
end

function HomePageWorldCollectionView:imgAgainstEventClick()
    if self.clickType.against == true then 
        return
    else 
        self.clickType.against = true
        self:updateClickType(self.uiChild.imgAgainst)
        self:updateViewData()
    end
    
end

function HomePageWorldCollectionView:imgFriendEventClick()
    if self.clickType.friend == true then 
        return 
    else 
        self.clickType.friend = true
        self:updateClickType(self.uiChild.imgFriend)
        self:updateViewData()
    end
end

function HomePageWorldCollectionView:updateClickType(t)
    local c = self.uiChild
    local ct = self.clickType 

    if c.imgAll == self.lastClick  then
        ct.all = false
    elseif c.imgResource == self.lastClick  then
        ct.resource = false 
    elseif c.imgAgainst == self.lastClick  then
        ct.against = false 
    elseif c.imgFriend == self.lastClick  then
        ct.friend = false 
    end

    self.lastClick = t
    self:updateClickContent()

end

-- 更新点击的内容
function HomePageWorldCollectionView:updateClickContent()
    local c = self.uiChild
    if self.clickType == nil then 
        self.clickType = {all = true, resource = false, against = false, friend = false}
        self.lastClick = self.uiChild.imgAll
    end
    local clickType = self.clickType

    local cfg = HomePageManager.wmCfg.collection

    if clickType.all then 
        c.imgAll:loadTexture(cfg.all[2], ccui.TextureResType.plistType)
    else 
        c.imgAll:loadTexture(cfg.all[1], ccui.TextureResType.plistType)
    end

    if clickType.resource then 
        c.imgResource:loadTexture(cfg.resource[2], ccui.TextureResType.plistType)
    else 
        c.imgResource:loadTexture(cfg.resource[1], ccui.TextureResType.plistType)
    end

    if clickType.against then 
        c.imgAgainst:loadTexture(cfg.against[2], ccui.TextureResType.plistType)
    else 
        c.imgAgainst:loadTexture(cfg.against[1], ccui.TextureResType.plistType)
    end

    if clickType.friend then 
        c.imgFriend:loadTexture(cfg.friend[2], ccui.TextureResType.plistType)
    else 
        c.imgFriend:loadTexture(cfg.friend[1], ccui.TextureResType.plistType)
    end

end

function HomePageWorldCollectionView:closeEventClick()
    WindowManager:pop()
end

function HomePageWorldCollectionView:cleanEvent()
    self:releaseCellView() -- stack中cell的 引用变量减1

    self:updateWorldCollectionData()

    self:releasePublicEvent()
end

function HomePageWorldCollectionView:registerEvent()
    self:registerLocalEvent()
end

--- 更新收集的内容
function HomePageWorldCollectionView:updateWorldCollectionData()
    local data = self.manager:getWorldCollection()
    local manager = self.manager

    local sendData = {}

    for k, v in pairs(self.tmpCollectionData) do 
        if v.isDirty then 
            if v.resType ~= data[k].resType then  -- 改变类型
                table.insert(sendData, v)
            elseif v.desc ~= data[k].desc then
                table.insert(sendData, v)  -- 改变描述
            end
        end
    end

    if #sendData >= 1 then 
        require("src.app.communication.worldMap.WorldMapModifyCollectionCommand").new():exec(
        sendData, 
        function(state) 
            if state == ReturnCode.SUCCESS then 
                -- 成功 修改数据
                for i = 1, #sendData do 
                    manager:modifyWorldCollection(sendData)
                end
            end
        end)

    end
end

-- 删除条目
function HomePageWorldCollectionView:delItem(x, y)
    local manager = self.manager
    require("src.app.communication.worldMap.WorldMapRemoveCollectionCommand").new():exec(
    {x = x, y = y}, 
    function(state)
        if state == ReturnCode.SUCCESS then  -- 成功
            local index = x .. "," .. y
            self.tmpCollectionData[index] = nil     -- 本类的备份删除  
            manager:removeWorldCollection(x, y)     -- 管理器的删除， 成功之后会派发更新事件
        end
    end)
end
-- 设置管理器
function HomePageWorldCollectionView:setManager(m)
    self.manager = m
end
-- 初始化数据
function HomePageWorldCollectionView:initData()
    self:registerPublicEvent()

    if self.manager then 
        self.manager:initWorldCollection()     -- 初始化
    end
end
--- 获得本类中缓存的收藏数据
function HomePageWorldCollectionView:getCollectionData()
   self.tmpCollectionData =  self.tmpCollectionData or self.manager:getWorldCollection()
   return self.tmpCollectionData
end
--- 更新界面数据
function HomePageWorldCollectionView:updateViewData()
    local data = self:getCollectionData()

    local c = self.uiChild

    self:removeCellView(self.scroll)               -- 先缓存数据
    
    local tb = {}
    if self.lastClick == c.imgAll then   -- 上次点击的是所有选项
        for k, v in pairs(data) do 
            table.insert(tb, v)
        end
    elseif self.lastClick == c.imgResource then 
        for k, v in pairs(data) do 
            if v.resType == 1 or v.resType == 3 or v.resType == 5 or v.resType == 7 then  
                table.insert(tb, v)
            end
        end
    elseif self.lastClick == c.imgAgainst then 
        for k, v in pairs(data) do 
            if v.resType == 2 or v.resType == 3 or v.resType == 6 or v.resType == 7  then  
                table.insert(tb, v)
            end
        end
    elseif self.lastClick == c.imgFriend then 
        for k, v in pairs(data) do 
            if v.resType == 4 or v.resType == 5 or v.resType == 6 or v.resType == 7 then 
                table.insert(tb, v)
            end
        end
    end

    local uTb = {} -- 创建cell
    for i = 1, #tb do 
        local view = self:getCellView(tb[i])
        table.insert(uTb, view)
    end

    self:addItemToScroll(self.scroll, uTb)

end
--- 初始化栈
function HomePageWorldCollectionView:initStack()
    self.stack = self.stack or Stack()
end

-- 改变数据
function HomePageWorldCollectionView:changeCollectionData(nodeInfo)
    local index = nodeInfo.x .. "," .. nodeInfo.y
    if self.tmpCollectionData[index] then     -- 改变数据 
        self.tmpCollectionData[index] = nodeInfo
    end
end


--- 获得cellview
function HomePageWorldCollectionView:getCellView(info)
--    local view = require("src.app.uiScript.scenes.homePage.HomePageWorldCollectionCellView").new()
--    view:initView()
--    view:setNodeInfo(info)
--    return view
    
    self.stack = self.stack or Stack()

    if self.stack:isEmpty() then
        self.listViewContainer = self.listViewContainer or {}  
        local view = cellView.new()
        view:initView()
        view:setNodeInfo(info, self)
        view:retain()
        table.insert(self.listViewContainer, view)
        return view
    else 
        local view = self.stack:pop()
        view:setNodeInfo(info, self)
        return view
    end

end
-- 释放cellview
function HomePageWorldCollectionView:releaseCellView()
    self.listViewContainer = self.listViewContainer or {}
    local length = #self.listViewContainer
    if length == 0 then return end
    for i = 1, length do 
        self.listViewContainer[i]:release()
    end

    self.stack = nil
    self.listViewContainer = nil
end
-- 删除cell
function HomePageWorldCollectionView:removeCellView(scroll)
    local inner = scroll:getInnerContainer()
    local view = inner:getChildren()
    if #view == 0 then return end 
    self.stack = self.stack or Stack()

    for i = 1, #view do 
        self.stack:push(view[i])
    end

    inner:removeAllChildren()
end

--- 添加内容到scroll
function HomePageWorldCollectionView:addItemToScroll(scroll, tb)
    local height = 0
    local distance = 5 -- 两个组件之间的距离 5 个像素
    local allDistance = 0
    if #tb == 0 then return end
    --scroll:getInnerContainer():removeAllChildren()
   
    for i = 1, #tb do  
        local sz = tb[i]:getContentSize()
        height = height + sz.height
    end

    if #tb > 1 then -- 加上距离
        allDistance = distance * (#tb - 1)
        height = allDistance + height
    end

    local inner = scroll:getInnerContainer()
    local s = scroll:getContentSize()
    if height >= s.height then
        inner:setContentSize(cc.size(s.width, height))    -- 下一次进入这个函数 需要重置scroll的高度
    else
        inner:setContentSize(cc.size(s.width, s.height))
    end
    local s = inner:getContentSize()


    -- 依次放入 inner中
    local n = #tb
    for i = 1, n do  
        local sz = tb[i]:getContentSize()
        if i > 1 then 
            s.height = s.height - sz.height - distance
        else
            s.height = s.height - sz.height
        end
        inner:addChild(tb[i])
        tb[i]:setPosition(cc.p((s.width - sz.width) / 2, s.height))
    end


    scroll:jumpToTop()   -- scroll到顶部
end

function HomePageWorldCollectionView:initView()
    self:loadJson()
    self:loadUI()
    self:registerEvent()

    self:initData()

    self:initCompleteHandler() -- 透明度
end

return HomePageWorldCollectionView
--endregion
