--Author    : YuDing
--Date      : 2014/12/1
--Introduce : 玩家信息 技能条

local PlayerInfoSkillItem = class("PlayerInfoSkillItem", function() return ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.playerInfoSkillItem) end)

PlayerInfoSkillItem.uiChild = nil

-- 技能id
PlayerInfoSkillItem.id = nil

function PlayerInfoSkillItem:ctor()
	mapping(self, LuaEventDispatcher)
	local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function(eventType)
				if CocosEvent.ENTER == eventType then
					self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
				else

				end

				self:luaDispatchEvent(eventType)
			end
		)
	end

	self:loadUI()

end

function PlayerInfoSkillItem:onEnter()

end

function PlayerInfoSkillItem:cleanUp()
	self:releaseDispatchEvent()
end

function PlayerInfoSkillItem:onExit()

end

function PlayerInfoSkillItem:loadUI()
	if not self.uiChild then self.uiChild = {} end
	local c = self.uiChild

	c.imgKind = Cocos2dxUtils:getChildByName(self, "imgKind")
	c.name = Cocos2dxUtils:getChildByName(self, "name")                -- 名字和等级
	c.attribute = Cocos2dxUtils:getChildByName(self, "attribute")      -- 属性
	c.btnInfo = Cocos2dxUtils:getChildByName(self, "btnInfo")          -- 信息
	c.btnAdd = Cocos2dxUtils:getChildByName(self, "btnAdd")            -- 升级按钮
	c.back = Cocos2dxUtils:getChildByName(self, "imgBack")             -- 背景

	self:initFixedUI()
end
--- 注册事件
function PlayerInfoSkillItem:registerEvent()
	local c = self.uiChild

	local func = function(t, eventType)
		if ccui.TouchEventType.ended ~= eventType then return end
		if c.btnInfo == t then
			self:onEventInfoClick()
		elseif c.btnAdd == t then
			self:onEventLevelUpClick()
		end
	end

	BaseButton:extend(c.btnInfo)
	BaseButton:extend(c.btnAdd)

	c.btnInfo:addTouchEventListener(func)
	c.btnAdd:addTouchEventListener(func)


	self:registerDispatchEvent()
end

--- 详细信息点击
function PlayerInfoSkillItem:onEventInfoClick()
	local skillId=self.id
	local name = PlayerInstance:getPlayerSkillNameByID(skillId)
	local sLvl = PlayerInstance:getPlayerSkillLevelByID(skillId)
	local extra = Language:get(Skill_temp[skillId].extra)

	local nextLv=sLvl + 1
	local playerLv=PlayerInstance:getPlayerLevel()
	local skillPont=PlayerInstance:getSkillPoint()

	MUIAlter_noTitle.show({
		info=Language:get("uy111",
			name,
			sLvl,
			extra,
			nextLv <= playerLv and nextLv or CssStyles.span(tostring(nextLv),"red24"),
			nextLv <= skillPont and nextLv or CssStyles.span(tostring(nextLv),"red24")
		),
		hAlign="left"
	})
end

function PlayerInfoSkillItem:initFixedUI()
	local c = self.uiChild

	local tb = {}
	for k, v in pairs(self.uiChild) do
		if v:getDescription() == "Label" then
			table.insert(tb, v)
		end
	end
	Cocos2dxUtils:setTextFont(tb)

end


function PlayerInfoSkillItem:registerDispatchEvent()
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.skillReset, self, self.updateCondition)
	PropsInfoMgr:luaAddEventListener(PlayerInstance.noticeMessage.prop .. PlayerInstance:getMedalPropID(), self, self.updateCondition)   -- 荣誉值的变化 在当前页面 等级不会发生改变 所以不用关心 人物等级带来的变化
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.skillLvl .. self.id, self, self.updateSkillLevelAndDes) -- 注册技能等级改变事件
end

function PlayerInfoSkillItem:releaseDispatchEvent()
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.skillReset, self, self.updateCondition)
	PropsInfoMgr:luaRemoveEventListener(PlayerInstance.noticeMessage.prop .. PlayerInstance:getMedalPropID(), self, self.updateCondition)   -- 荣誉值的变化 在当前页面 等级不会发生改变 所以不用关心 人物等级带来的变化
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.skillLvl .. self.id, self, self.updateSkillLevelAndDes) -- 注册技能等级改变事件
end

--- 升级点击
function PlayerInfoSkillItem:onEventLevelUpClick()
	local tb = self:isCanLevelUp()
	local c = self.uiChild
	if tb[1] == false then
		return
	end

	local useDiamond = false
	if tb[2] == false then
		useDiamond = true
	end

	local func = function()
		SkillLevelUpCommand.new():exec(self.id,
			function(state, data)
				local lvl = PlayerInstance:getPlayerSkillLevelByID(self.id)
				local name = PlayerInstance:getPlayerSkillNameByID(self.id)
				local des = name .. "Lv." .. lvl
				if state == ReturnCode.SUCCESS then
					MUIAlter_singleLine.show( {info = des .. Language:get("fy1002"), showSecond = 1 } )
				else
				--MUIAlter_singleLine.show( {info = des .. Language:get("fy1003"), showSecond = 1 } )
				end
			end, useDiamond)
	end

	if useDiamond == true then -- 需要钻石
		local costGems = (PlayerInstance:getSkillLevelupSkillPointNeedByID(self.id) - PlayerInstance:getSkillPoint()) * PlayerInstance:getSkillPointPrice()
		local gems = UserInfo.gems
		if costGems > gems then
			-- 钻石不足,-- 弹出充值页面
			MUIPop.MUIToReCharge()
			return
		end

		local payState = SetStates.setPayState
		local value = payState:getClosedValue(payState.CLOSE_SKILL_LEVELUP)

		if value then
			func()
		else
			local info = Language:get("uy189", costGems)
			local layout = DiamondPopInfo.new(info)
			local function confirm()
				SetStates.setPayState:setClosedValue(layout.judge, SetStates.setPayState.CLOSE_SKILL_LEVELUP)
				if layout.judge == true then
					Command:SetPayCommand()
				end
				func()
			end
			MUIPop.DiamondPopInfo(layout,confirm)
		end

		--        local info = Language:get("uy189", costGems)
		--		MUIAlter.show(
		--            {
		--                info = info,
		--                size = 30,
		--                flag = 3,
		--                callback = function(flag)
		--	               	if flag == 0 then -- 0为确定，1为取消
		--                        func()
		--	                end
		--                end
		--            }
		--        )
	else -- 不需要钻石
		func()
	end

end
--- 更新条件
function PlayerInfoSkillItem:updateCondition()
	local tb = self:isCanLevelUp()
	local c = self.uiChild

	if tb[1] == false then -- 变灰 不可点击 移除特效
		c.btnAdd:setIsGray(true)
		c.btnAdd:setTouchEnabled(false)
		if c.btnAdd.eff then -- 按钮 特效
			c.btnAdd.eff:removeFromParent()
			c.btnAdd.eff = nil
		end
	elseif tb[1] == true and tb[2] == false then -- 变灰 可点击 移除特效
		c.btnAdd:setIsGray(true)
		c.btnAdd:setTouchEnabled(true)
		if c.btnAdd.eff then -- 按钮 特效
			c.btnAdd.eff:removeFromParent()
			c.btnAdd.eff = nil
		end
	elseif tb[1] == true and tb[2] == true then  -- 不变化 可点击 添加特效
		c.btnAdd:setIsGray(false)
		c.btnAdd:setTouchEnabled(true)
		if not c.btnAdd.eff then -- 按钮 特效
			c.btnAdd.eff = HomePageManager:addEffect(c.btnAdd, "button6TipEff", true)
		end
	end

end
--- 更新等级和描述
function PlayerInfoSkillItem:updateSkillLevelAndDes()
	local lvl = PlayerInstance:getPlayerSkillLevelByID(self.id)
	local name = PlayerInstance:getPlayerSkillNameByID(self.id)
	self.uiChild.name:setString(Language:get("uy89",name,lvl))

	local des = PlayerInstance:getPlayerSkillDesByID(self.id, lvl)
	self.uiChild.attribute:setString(des)
end

function PlayerInfoSkillItem:updateHead()
	local dt=IconManager.skillRes[self.id]

	if dt then
		self.uiChild.imgKind:loadTexture(IconManager.iconPath..dt.res)
	end
end

--- 检测是否能升级
-- @param [1] false 达到等前人物最大等级
-- @param [2] false 技能点不够
function PlayerInfoSkillItem:isCanLevelUp()
	return {PlayerInstance:checkSkillLevelupLevelByID(self.id),
		PlayerInstance:checkSkillLevelupSkillPointNeedByID(self.id)}
end
--- 注册事件
function PlayerInfoSkillItem:registerItem()

end

-- 得到特效的载体
function PlayerInfoSkillItem:getEffRender()
	local tb = {}
	for k, v in pairs(self.uiChild) do
		if v:getDescription() == "Button" or v:getDescription() == "Label" or v:getDescription() == "ImageView" then
			table.insert(tb, v:getVirtualRenderer())
		end
	end
	return tb
end

-- 初始化
function PlayerInfoSkillItem:initView(id, d)
	self.id = id
	self.delegete = d

	self:registerEvent()
	self:updateCondition()                  -- 先更新
	self:updateSkillLevelAndDes()
	self:updateHead()

end

return PlayerInfoSkillItem
