--region T.lua
--Author    : YuDing
--Date      : 2014/11/29
--Introduce : 玩家信息界面
local T = class(
    "PlayerInfoWindow",
    function() 
        local o = AbstractWindow.new()
        o.super = AbstractWindow
        return o
    end
)

--- 三级页面
T.switchButtonContainer = nil

function T:ctor(tag)
    mapping(self, LuaEventDispatcher)
    self:registerScriptHandler(
        function(eventType)
            if eventType == CocosEvent.ENTER then
                self:onEnter()
            elseif eventType == CocosEvent.EXIT then
                self:onExit()
            elseif eventType == CocosEvent.CLEAN_UP then
                self:cleanUp()
            end
        end
    )

    self:initThreeContainer(tag)    -- 初始化三级页面

    self:registerPublicEvent()
    self:updateSkillUpgradeTag()    -- 更新提示
end

function T:onEnter()
    self.super.onEnter(self)
end

function T:onExit()
    self.super.onExit(self)
end

function T:cleanUp()
	AbstractWindow.cleanUp(self)
    --self:releasePageNode()
    self:releasePublicEvent()
end

--- 创建UI
function T:initThreeContainer(tag)
    -- 创建3个

    local callBack1= function()
        self:switchIntroducePage()
    end

    local callBack2 = function()
        self:switchSkillPage()
    end

    local callBack3 = function()
        self:switchFriendPage()
    end

    --- 初始化分页
    self.menuTag = tag or 1

    local switchButtonContainer = SwitchButtonContainer.new()
    self:addChild(switchButtonContainer)
    REL:relOther(switchButtonContainer,self.uiContainer,0,0,0,0)
    switchButtonContainer:init()
    switchButtonContainer:initButtons(
        3,
        {{"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"},{"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"},{"viewSwitchButton2_normal.png","viewSwitchButton2_down.png"}},
        {70,15},
        {callBack1, callBack2, callBack3},
        {Language:get("uy30"), Language:get("uy31"), Language:get("uy32")},
        self.menuTag
    )

    self.switchButtonContainer = switchButtonContainer

    if self.menuTag == 1 then 
        self:switchIntroducePage()
    elseif self.menuTag == 2 then
        self:switchSkillPage()
    else
        self:switchFriendPage()
    end

    -- 设置标题
    self:setWindowTitle(Language:get("uy114"))

    -- 调用一次，透明度才没问题  
	self:initCompleteHandler()
end

function T:switchIntroducePage()
    if self.introducePage == nil then
        self.introducePage = require("src.app.uiScript.scenes.playerInfo.PlayerInfoIntroduce").new()
        self:addChild(self.introducePage)
        REL:toScreen("middle", "top", self.introducePage, 0, 160)
    end

    if self.introducePage then
        self.introducePage:setVisible(true)
    end

    if self.skillPage then
        self.skillPage:setVisible(false)
    end

    if self.socialPage then
        self.socialPage:setVisible(false)
    end
end

function T:switchSkillPage()
    if self.introducePage then
        self.introducePage:setVisible(false)
    end

    if self.skillPage == nil then
        self.skillPage = require("src.app.uiScript.scenes.playerInfo.PlayerInfoSkill").new(self)
        self:addChild(self.skillPage)

        REL:toScreen("left", "bottom", self.skillPage, 0, 0)
    end

    if self.skillPage then
        self.skillPage:setVisible(true)
    end

    if self.socialPage then
        self.socialPage:setVisible(false)
    end
end

function T:switchFriendPage()
    if self.introducePage then
        self.introducePage:setVisible(false)
    end

    if self.skillPage then
        self.skillPage:setVisible(false)
    end

    if self.socialPage == nil then
        self.socialPage = require("src.app.uiScript.scenes.playerInfo.PlayerInfoSocial").new()
        self:addChild(self.socialPage)
        REL:relOther(self.socialPage,self.uiContainer,0,0,CommonCfg.TOP_CONTAINER_HEIGHT,CommonCfg.BOTTOM_CONTAINER_HEIGTH)
        self.socialPage:init()
    end

    if self.socialPage then
        self.socialPage:setVisible(true)
    end
end

--- 更新技能升级提示
function T:updateSkillUpgradeTag()
    
    self:initCompleteHandler() -- test
    
	if PlayerInstance:judgeIsSkillUpgrade() then -- 能升级
		if not self.skillUpgradeTag then -- 增加特效
			self.skillUpgradeTag = HomePageManager:getEff("promptEffect")
			self.switchButtonContainer:getButton(2):addChild(self.skillUpgradeTag)
			CCUIREL:toOther("right", "top", self.skillUpgradeTag, self.switchButtonContainer:getButton(2), 22, 15)
		end

	else
		if self.skillUpgradeTag then
			self.skillUpgradeTag:removeFromParent()
			self.skillUpgradeTag = nil
		end
	end

    self:initCompleteHandler() -- test
end

--- 注册事件
function T:registerPublicEvent()
    PropsInfoMgr:luaAddEventListener(PlayerInstance.noticeMessage.prop .. PlayerInstance:getMedalPropID(), self, self.updateSkillUpgradeTag)   -- 荣誉值的变化 在当前页面 等级不会发生改变 所以不用关心 人物等级带来的变化
    PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.skillLvl, self, self.updateSkillUpgradeTag) -- 注册技能等级改变事件
    PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.skillReset, self, self.updateSkillUpgradeTag)
end

--- 销毁事件
function T:releasePublicEvent()
    PropsInfoMgr:luaRemoveEventListener(PlayerInstance.noticeMessage.prop .. PlayerInstance:getMedalPropID(), self, self.updateSkillUpgradeTag)   -- 荣誉值的变化 在当前页面 等级不会发生改变 所以不用关心 人物等级带来的变化
    PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.skillLvl, self, self.updateSkillUpgradeTag) -- 注册技能等级改变事件    
    PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.skillReset, self, self.updateSkillUpgradeTag)
end

--- 移除节点
function T:releasePageNode()
--    if self.introducePage then self.introducePage:release() end
--    if self.skillPage then self.skillPage:release() end
--    if self.friendPage then self.friendPage:release() end
end

return T

--endregion
