--region PromoteCommandWindow.lua
--Author    : YuDing
--Date      : 2015/4/2
--Introduce : 提升统御（指挥）

local T = class("PromoteCommandWindow", function() return ccui.Layout:create() end)


function T:ctor()
	mapping(self, LuaEventDispatcher)
	local handler = self.registerScriptHandler
	if handler then
		self:registerScriptHandler(
			function(eventType)
				if CocosEvent.ENTER == eventType then
					self:onEnter()
				elseif CocosEvent.EXIT == eventType then
					self:onExit()
				elseif eventType == CocosEvent.CLEAN_UP then
					self:cleanUp()
				else

				end

				self:luaDispatchEvent(eventType)
			end
		)
	end

	self:loadJson()
	self:initUI()
	self:registerEvent()
	self:updateLuckyNum()
	self:updateDominanceNum()
	self:updateMaxNum()
	self:updateCurrentNum()
	self:updateArmyNum()
	self:updateUpgradeBtn()

end

function T:onEnter()
	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_LV_UP_DOMINANCE_VIEW_CREATED)
	end
end

function T:onExit()

end

function T:cleanUp()
	self:releasePublicEvent()
end

function T:loadJson()
	local widget = ccs.GUIReader:getInstance():widgetFromJsonFile(UIJsonMgr.promoteCommand)
	self.widget = widget
	self:addChild(widget)
	self:setContentSize(widget:getContentSize())
end

function T:initUI()
	self.uiChild = self.uiChild or {}
	local w = self.widget
	local c = self.uiChild

	c.head = Cocos2dxUtils:getChildByName(w, "head") -- 标题
	c.info = Cocos2dxUtils:getChildByName(w, "info")  -- 介绍
	c.dominanceHead = Cocos2dxUtils:getChildByName(w, "dominanceHead") -- 头像
	c.luckyInfo = Cocos2dxUtils:getChildByName(w, "luckyInfo") -- 幸运介绍
	c.barBack = Cocos2dxUtils:getChildByName(w, "barBack") -- 进度条 背景
	c.barLucky = Cocos2dxUtils:getChildByName(w, "barLucky") -- 进度条
	local px, py = c.barLucky:getPosition()
	c.barLucky:setPosition(cc.p(px + 6, py)) -- 微调
	c.luckyNum = Cocos2dxUtils:getChildByName(w, "luckyNum") -- 幸运值
	c.haveText = Cocos2dxUtils:getChildByName(w, "haveText") -- 身上已有的
	c.haveHead = Cocos2dxUtils:getChildByName(w, "haveHead") -- 身上已有的
	c.haveNum = Cocos2dxUtils:getChildByName(w, "haveNum") -- 身上已有的
	c.costText = Cocos2dxUtils:getChildByName(w, "costText") -- 消耗的
	c.costHead = Cocos2dxUtils:getChildByName(w, "costHead")-- 消耗的
	c.costNum = Cocos2dxUtils:getChildByName(w, "costNum")-- 消耗的

	c.normalBtn = Cocos2dxUtils:getChildByName(w, "normalBtn") -- 升级1次
	c.normalText = Cocos2dxUtils:getChildByName(w, "normalText")

	c.currentText = Cocos2dxUtils:getChildByName(w, "currentText") -- 当前等级
	c.currentNum = Cocos2dxUtils:getChildByName(w, "currentNum")
	c.maxText = Cocos2dxUtils:getChildByName(w, "maxText")         -- 最大限制等级
	c.maxNum = Cocos2dxUtils:getChildByName(w, "maxNum")

	c.armyNumText = Cocos2dxUtils:getChildByName(w, "armyNumText")  -- 军队数量
	c.armyNum = Cocos2dxUtils:getChildByName(w, "armyNum")
	c.armyNumExtra = Cocos2dxUtils:getChildByName(w, "armyNumExtra")

	self:initFixedUI()

	if BGMgr:isGuide() then
		BGMgr:regGuideObj(BGMgr.TGR_PLAYER_INFO_LV_UP_DOMINANCE_CONFIRM_BUTTON,c.normalBtn)
	end
end

function T:initFixedUI()
	local c = self.uiChild

	local tb = {}
	for k, v in pairs(self.uiChild) do
		if v:getDescription() == "Label" then
			table.insert(tb, v)
		end
	end
	Cocos2dxUtils:setTextFont(tb)

	c.head:setString(Language:get("uy145"))
	c.info:setString(Language:get("uy146"))
	c.luckyInfo:setString(Language:get("uy147"))
	c.haveText:setString(Language:get("uy148"))
	c.costText:setString(Language:get("uy149"))
	c.normalText:setString(Language:get("uy150"))
	c.costNum:setString("1")
	c.currentText:setString(Language:get("uy152"))
	c.maxText:setString(Language:get("uy153"))
	c.armyNumText:setString(Language:get("uy154"))

	local headRes = IconManager.iconPath .. "propIcon/governingbook.png"

	c.dominanceHead:loadTexture(headRes)
	c.haveHead:loadTexture(headRes)
	c.haveHead:setScale(0.4)
	c.costHead:loadTexture(headRes)
	c.costHead:setScale(0.4)
end

--- 更新带兵数量
function T:updateArmyNum()
	local c = self.uiChild

	local ownPlane = PlayerInstance:getPlayerOwnPlane()
	local extraPlane = PlayerInstance:getDominanceExtraPlane()

	c.armyNum:setString(ownPlane)
	c.armyNumExtra:setString("+" .. extraPlane)

	local size = c.armyNum:getContentSize()
	local p = c.armyNum:getPosition()

	c.armyNumExtra:setPositionX(p + size.width + 2)

end

--- 注册本地事件
function T:registerLocalEvent()
	local c = self.uiChild

	local clickHandler = function(t, e)
		if e ~= ccui.TouchEventType.ended then return end
		if t == c.normalBtn then
			if BGMgr:isGuide() then
				BGMgr:fire(BGMgr.EVT_TGR_UI,BGMgr.TGR_PLAYER_INFO_LV_UP_DOMINANCE_CONFIRM_BUTTON)
			end

			self:onEventNormalBtnClick()
		end
	end

	BaseButton:extend(c.normalBtn)

	c.normalBtn:addTouchEventListener(clickHandler)

end

function T:onEventNormalBtnClick()
	local s = PlayerInstance:checkDominanceLevelupBookNeed()

	if s == true then  -- 有书 直接发送命令
		self:sendDominanceUpgradeCommand(false) -- 使用书籍升级
	else
		local needDiamond=PlayerInstance:getDominanceLevelUpDiamond()
		local st = self:checkDimondIsEnough(needDiamond)
		if st == true then  -- 钻石是足够的
			local payState = SetStates.setPayState
			local value = payState:getClosedValue(payState.CLOSE_UPGRADE_COMMAND)

			if value then
				self:sendDominanceUpgradeCommand(false)
			else
				local info = Language:get("fy1001", needDiamond)
				local layout = DiamondPopInfo.new(info)
				local function confirm()
					SetStates.setPayState:setClosedValue(layout.judge, SetStates.setPayState.CLOSE_UPGRADE_COMMAND)
					if layout.judge == true then
						Command:SetPayCommand()
					end
					self:sendDominanceUpgradeCommand(false)
				end
				MUIPop.DiamondPopInfo(layout,confirm)
			end
		else
			MUIAlter.show({
				info=Language:get("v32",needDiamond),
				flag = 3,
				confirmText = Language:get("ug103"),
				cancelText = Language:get("ug104"),
				callback=function(flag)
					if flag == FLAG_OK then
						VipManager:enterIapWindow()

						self.__hide__()
					end
				end
			})
		end
	end
end

--- 查看钻石是否足够
function T:checkDimondIsEnough(cost)
	local total = PlayerInstance:getDiamond()
	if cost > total then
		return false
	else
		return true
	end
end

--- 发送指挥升级
-- @param 是否 是连抽
function T:sendDominanceUpgradeCommand(s)

	--self.__hide__()

	DominanceLevelUpCommand.new():exec(s,
		function(state, s)
			if BGMgr:isGuide() then
				if BGMgr:fire(BGMgr.EVT_LV_UP_DOMINANCE_COMPLETE) then
					self.__hide__()
				end
			end

			if s == true then
				self:showUpgradeEff()
			else -- 失败
			--MUIAlter_singleLine.show( {info = Language:get("fy1003"),showSecond = 1 } )
			end
		end)
end


--- 更新幸运值
function T:updateLuckyNum()
	local c = self.uiChild
	local num = PlayerInstance:getDominanceLucky()

	local have = SetStates.dominanceLuckyState:getDominanceLucky()

	local per = have / num  * 100
	if per > 100 then
		per = 100
	end

	c.luckyNum:setString(have .. "/" .. num) -- 文字

	c.barLucky:setPercent(per) -- 百分比
end

--- 更新指挥书的数量
function T:updateDominanceNum()
	local c = self.uiChild
	local num = PlayerInstance:getDominanceBook()
	c.haveNum:setString(num)
end

--- 更新最大限制等级
function T:updateMaxNum()
	local c = self.uiChild
	local lvl = PlayerInstance:getPlayerLevel()
	c.maxNum:setString(Language:get("uy18", lvl))

end

--- 更新升级按钮
function T:updateUpgradeBtn()
	local c = self.uiChild
	local dLvl = PlayerInstance:getDominanceLevel()
	local pLvl = PlayerInstance:getPlayerLevel()



	if dLvl >= pLvl then
		c.normalBtn:setIsGray(true)
		c.normalBtn:setTouchEnabled(false)
	else
		c.normalBtn:setIsGray(false)
		c.normalBtn:setTouchEnabled(true)
		--        local s = PlayerInstance:checkDominanceLevelupBookNeed()
		--        if s == false then
		--            c.normalBtn:setIsGray(true)
		--        else
		--            c.normalBtn:setIsGray(false)
		--        end
	end

end

--- 当前等级
function T:updateCurrentNum()
	local c = self.uiChild
	local lvl = PlayerInstance:getDominanceLevel()
	c.currentNum:setString(Language:get("uy18", lvl))

end

function T:registerPublicEvent()
	SetStates.dominanceLuckyState:luaAddEventListener(SetStates.dominanceLuckyState.CHANGE, self, self.updateLuckyNum)
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.dominanceNum, self, self.updateDominanceNum)

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.updateMaxNum)          --最大限制等级
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.dominanceLvl, self, self.updateCurrentNum)   --当前等级

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.dominanceLvl, self,  self.updateArmyNum) -- 带兵数量

	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.updateUpgradeBtn)          --最大限制等级
	PlayerInstance:luaAddEventListener(PlayerInstance.noticeMessage.dominanceLvl, self, self.updateUpgradeBtn)   --当前等级


end

function T:releasePublicEvent()
	SetStates.dominanceLuckyState:luaRemoveEventListener(SetStates.dominanceLuckyState.CHANGE, self, self.updateLuckyNum)
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.dominanceNum, self, self.updateDominanceNum)

	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.updateMaxNum)
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.dominanceLvl, self, self.updateCurrentNum)

	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.dominanceLvl, self,  self.updateArmyNum) -- 带兵数量

	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.playerLvl, self, self.updateUpgradeBtn)      --最大限制等级
	PlayerInstance:luaRemoveEventListener(PlayerInstance.noticeMessage.dominanceLvl, self, self.updateUpgradeBtn)   --当前等级
end

function T:registerEvent()
	self:registerLocalEvent()
	self:registerPublicEvent()
end

--- 显示特效
function T:showUpgradeEff()
	local c = self.uiChild
	local eff = HomePageManager:getEffWithDelete("upGradeEff", 1)

	local scale = 0.76
	c.dominanceHead:addChild(eff)
	eff:setScale(scale)
	CCUIREL:toOther("center", "middle", eff, c.dominanceHead, 1, 10)
end


return T



























































































































































































































--endregion
