local put=function(name,byteArray)
	CppCocos2dxUtils:storeageData(CFG.APP_NAME.."_"..name, byteArray)
end

local get=function(name)
	return CppCocos2dxUtils:restoreData(CFG.APP_NAME.."_"..name)
end

--- 游戏数据存储
local T={}

local KEY_LANG_ID="key_lang_id"
--- 存储游戏语言
-- @param appLangId 游戏语言string
function T:putAPPLang(appLangId)
	local data = ByteArray:new()
	data:writeInt(appLangId)
	data:setPosition(0)

	put(KEY_LANG_ID,data)
end
--- 获取游戏语言
function T:getAPPLang()
	local data=get(KEY_LANG_ID)

	if data then
		data:setPosition(0)

		return data:readInt()
	end

	return nil
end

local KEY_UNIQUE_ID="key_unique_id"
--- 存储唯一ID
-- @param uniqueId 唯一ID
function T:putUniqueId(uniqueId)
	KeyChainUtils:getInstance():setString(CFG.APP_NAME.."_"..KEY_UNIQUE_ID,uniqueId)
end
--- 获取唯一ID
function T:getUniqueId()
	local s=KeyChainUtils:getInstance():getString(CFG.APP_NAME.."_"..KEY_UNIQUE_ID)

	if s and string.trim(s) ~= "" then
		return s
	end

	return nil
end

local KEY_LOGIN_DATA="key_login_data"
--- 存储登陆数据
-- @param loginData 登陆数据
function T:putLoginData(loginData)
	local s = obj2str(loginData)

	KeyChainUtils:getInstance():setString(CFG.APP_NAME.."_"..KEY_LOGIN_DATA,s)
end
--- 获取登陆数据
function T:getLoginData()
	local s=KeyChainUtils:getInstance():getString(CFG.APP_NAME.."_"..KEY_LOGIN_DATA)

	if s and string.trim(s) ~= "" then
		return str2obj(s)
	end

	return nil
end

local KEY_HISTORY_LOGIN_DATA="key_history_login_data"
--- 存储历史登陆数据
function T:putHistoryLoginData(historyLoginData)
	local s = obj2str(historyLoginData)

	KeyChainUtils:getInstance():setString(CFG.APP_NAME.."_"..KEY_HISTORY_LOGIN_DATA,s)
end
--- 获取历史登陆数据
function T:getHistoryLoginData()
	local s=KeyChainUtils:getInstance():getString(CFG.APP_NAME.."_"..KEY_HISTORY_LOGIN_DATA)

	if s and string.trim(s) ~= "" then
		return str2obj(s)
	end

	return nil
end

local KEY_TEST_SWITCH_DATA="key_test_switch_data"
--- 存储测试开关的相关数据
function T:putTestSwitchData(testSwitchData)
	local s = obj2str(testSwitchData)

	KeyChainUtils:getInstance():setString(CFG.APP_NAME.."_"..KEY_TEST_SWITCH_DATA,s)
end
--- 获取测试开关的相关数据
function T:getTestSwitchData()
	local s=KeyChainUtils:getInstance():getString(CFG.APP_NAME.."_"..KEY_TEST_SWITCH_DATA)

	if s and string.trim(s) ~= "" then
		return str2obj(s)
	end

	return nil
end

local KEY_AUDIO="key_audio"
--- 存储音频状态
function T.putAudioStatus()
	local data = ByteArray:new()

	local aMgr=AudioManager
	data:writeBoolean(aMgr:getEnabledPlayEffect())
	data:writeFloat(aMgr:getEffectsVolume())
	data:writeBoolean(aMgr:getEnabledMusic())
	data:writeFloat(aMgr:getMusicVolume())
	data:setPosition(0)

	put(KEY_AUDIO,data)
end
--- 恢复音频状态
function T:getAudioStatus()
	local data=get(KEY_AUDIO)
	if data then
		data:setPosition(0)

		local aMgr=AudioManager
		aMgr:setEnabledPlayEffect(data:readBoolean())
		aMgr:getEffectsVolume(data:readFloat())
		aMgr:setEnabledMusic(data:readBoolean())
		aMgr:setMusicVolume(data:readFloat())
	end
end

--- 存储公告校验值
local KEY_ANNOUNCE_SRC="key_announce_src"
function T:putAnnounceSrc(userId,announceCrc)
	local data = ByteArray:new()
	data:writeInt(announceCrc)

	data:setPosition(0)

	put(userId..KEY_ANNOUNCE_SRC,data)
end

function T:getAnnounceSrc(userId)
	local data = get(userId..KEY_ANNOUNCE_SRC)
	if data then
		data:setPosition(0)
		return data:readInt()
	else
		return 0
	end
end

local KEY_IAP_RECEIPTS="key_iap_receipts"
--- 存储内购验证密钥
function T:putIapReceipts(userId,iapReceipts)
	local data=ByteArray:new()
	for k,v in pairs(iapReceipts) do
		data:writeString(obj2str(v))  -- 每一项是一个表
	end
	data:setPosition(0)

	put(userId..KEY_IAP_RECEIPTS,data)
end
--- 获取存储的内购验证密钥
function T:getIapReceipts(userId)
	local data=get(userId..KEY_IAP_RECEIPTS)
	if data then
		data:setPosition(0)

		local iapReceipts={}
		while(data:bytesAvailable() > 0) do
			iapReceipts[#iapReceipts+1]=str2obj(data:readString())
		end

		return iapReceipts
	end
	return nil
end
--- 添加一个验证密钥
function T:addReceipt(userId,iapReceipt)
	local iapReceipts = self:getIapReceipts(userId) or {}
	for k,v in pairs(iapReceipts) do
		if v.id == iapReceipt.id then
			return
		end
	end

	iapReceipts[#iapReceipts+1]=iapReceipt
	self:putIapReceipts(userId,iapReceipts)
end
--- 删除存储的某一个验证密钥,并且把最新的更新回去
function T:delReceipt(userId,iapReceipt)
	local iapReceipts = self:getIapReceipts(userId) or {}
	for k,v in pairs(iapReceipts) do
		if v.id == iapReceipt.id then
			iapReceipts[k] = nil
			break
		end
	end

	self:putIapReceipts(userId,iapReceipts)
end

--- 存储KEY-内购订单号
local KEY_IAP_ORDER_ID="key_iap_order_id"
--- 获取内购订单号
function T:getIapOrderId()
	local data=get(KEY_IAP_ORDER_ID)

	local orderId=1
	if data then
		data:setPosition(0)

		orderId=data:readInt()+1
	end

	self:_putIapOrderId(orderId)

	return orderId
end
--- 存储内购订单号
function T:_putIapOrderId(orderId)
	local data = ByteArray:new()
	data:writeInt(orderId)
	data:setPosition(0)

	put(KEY_IAP_ORDER_ID,data)
end

--- 存储KEY-唯一ID
local KEY_ID_UNIQUE_ID="key_id_unique_id"
--- 生成一个唯一ID
function T:generateUniqueId()
	local id=1

	local data=get(KEY_ID_UNIQUE_ID)
	if data then
		data:setPosition(0)
		id=data:readInt()+1
		data:setPosition(0)
	else
		data=ByteArray:new()
	end
	data:writeInt(id)
	put(KEY_ID_UNIQUE_ID,data)

	return id
end

--- 存储KEY-本地推送信息
local KEY_ID_LOCAL_NOTIFICTION_INFO="key_id_local_notifiction_info"
--- 获取本地推送信息
function T:getLocalNotificationInfo()
	local data=get(KEY_ID_LOCAL_NOTIFICTION_INFO)
	if data then
		data:setPosition(0)
	
		return str2obj(data:readString())
	end
	return nil
end
--- 存储本地推送信息
function T:putLocalNotificationInfo(info)
	local data = ByteArray:new()
	data:writeString(obj2str(info))
	data:setPosition(0)

	put(KEY_ID_LOCAL_NOTIFICTION_INFO,data)
end

return T
