--- 语言包文件模板
local T=Class.LanguageUtils()

--- 模板,其中 {code} 表示语言代码,如果有 android 平台的代码,以 android 的代码优先,其次为 ios
T.TEMPLATE={
	firstFiles={
		"res.data.lang.{code}.ui",
	},
	secondFiles={
		"res.data.lang.{code}.fix",
		"res.data.lang.{code}.var"
	}
}

T.defaultLanguageId=cc.LANGUAGE_ENGLISH

function T:def()
--- 当前语言数据
-- _languageData
end

--- 初始化
-- @param languageData [可选]语言数据
function T:init(languageData)
	if not languageData then
		--语言列表
		local l=require("res.data.lang.languageList")
		unrequire("res.data.lang.languageList")

		-- 获取上次登陆的语言配置
		local lastLangId = DataStorage:getAPPLang()
		if lastLangId then
			languageData=l[lastLangId] or l[cc.Application:getInstance():getCurrentLanguage()]
        else
            languageData=l[cc.Application:getInstance():getCurrentLanguage()]
		end

        if not languageData or not languageData.isOpen then
			languageData=l[self.defaultLanguageId]
		end
	end

	self._languageData=mapping({},languageData)
	mapping(self._languageData,self.TEMPLATE)
	
	Language:init()

	DataStorage:putAPPLang(self._languageData.id)
end

--- 加载第一次文件
function T:loadFirstFiles()
	self:_loadFiles(self._languageData.firstFiles)
end

--- 加载第二次文件
function T:loadSecondFiles()
	self:_loadFiles(self._languageData.secondFiles)
end

--- 重新加载第一次文件
function T:reloadFirstFiles()
	Language:init()
	self:loadFirstFiles()
end

--- 加载文件
-- @param filesUrl 文件url
function T:_loadFiles(filesUrl)
	local code=self:getCode()
	for _,v in pairs(filesUrl) do
		local path=v:gsub("{code}",code)
		Language:addLang(require(path))
		unrequire(path)
	end
end

--- 获取当前语言的 code
function T:getCode()
	local l=self._languageData
	
	local code=l.code
	if l.longCode_android and string.trim(l.longCode_android) ~= "" then
		code=l.longCode_android
	elseif l.longCode_ios and string.trim(l.longCode_ios) ~= "" then
		code=l.longCode_ios
	end

	return code
end

--- 获取当前语言的ID
function T:getId()
	return self._languageData.id
end

return T()
