-- 注册本地信息推送
LocalPushProxy:regLocalNotification()

--- 信息列表
local _list={}

--- 排除的分类列表
local _includeCategory={}

--- 本地信息推类
local T={}

--- 建筑/生产推送
T.CATEGORY_BUILD="category_build"

--- 精力点回复提示
T.PVE_POINT_TIP="pve_point_tip"

--- 防护罩状态推送
T.PROTECT_COVER="protect_cover"

--- 本地推送
-- @param alterConent 推送信息的内容
-- @param afterSec 指定时间(秒)后推送本地信息
-- @param category 分类
-- @param id 信息的唯一编号
-- @return 本次推送的唯一ID,记录此ID,以便取消此信息的推送
function T:push(alterConent,afterSec,category,id)
	print(string.format("添加本地推送:\nalterContent:%s\nafterSec:%s\ncategory:%s\nid:%s",alterConent,tostring(afterSec),category,id))

	_list[id]={
		alterContent=alterConent,
		category=category,
		id=id,
		scheduleEndSec=TimeUtils:getSyncMillis()/1000+afterSec, --计划到期时间(1970年到现在的秒)
	}

	return id
end

--- 取消本地推送
-- @param localPushId 推送的唯一ID
function T:cancel(id)
	print("取消本地推送:",obj2str(_list[id]))
	_list[id]=nil
end

--- 包含指定分类
-- @param category 类型名
function T:includeCategory(category)
	_includeCategory[category]=true
end

--- 排除指定分类
-- @param category 类型名
function T:excludeCategory(category)
	_includeCategory[category]=nil
end

--- 应用程序进入后台
function T:app2background()
	local curSec=TimeUtils:getSyncMillis()/1000
	for k,v in pairs(_list) do
		print("检查:",obj2str(v))
		if v.scheduleEndSec < curSec then
			print("已过期:",obj2str(v))
			--计划已过期
			_list[k]=nil
		else
			local isIncude=false
			local category=v.category
			for c,_ in pairs(_includeCategory) do
				if c == category then
					isIncude=true
					break
				end
			end

			if isIncude then
				print("添加本地推送:",obj2str(v))
				print(math.ceil(v.scheduleEndSec-curSec))
				LocalPushProxy:scheduleLlocalNotification(
					v.alterContent,
					math.ceil(v.scheduleEndSec-curSec),
					CFG.APP_NAME.."_user",
					tostring(v.id)
				)
			end
		end
	end
end

--- 应用程序进入前台
function T:app2foreground()
	LocalPushProxy:cancelNotifications(CFG.APP_NAME.."_user")
	LocalPushProxy:clearIconBadgeNumber()
end

return T
