-- 注册本地信息推送
LocalPushProxy:regLocalNotification()

--- 信息列表
_list={}

--- 排除的分类列表
_includeCategory={}

--- 检查到期的推送信息的计时器ID
_checkScheduleId=nil

--- 本地信息推类
local T={}

--- 检查推送信息是否到期
function T:check()
	local curSec=TimeUtils:getSyncMillis()/1000
	for k,v in pairs(_list) do
		if v.scheduleEndSec <= curSec then
			_list[k]=nil

			LocalPushProxy:cancelNotification(CFG.APP_NAME.."_"..v.category,k)
		end
	end
end

--- 本地推送
-- @param alterConent 推送信息的内容
-- @param afterSec 指定时间(秒)后推送本地信息
-- @param category 分类
-- @param id 信息的唯一编号
-- @return 本次推送的唯一ID,记录此ID,以便取消此信息的推送
function T:push(alterConent,afterSec,category,id)
	if afterSec <= 0 then
		return -1
	end

	_list[id]={
		alterContent=alterConent,
		category=category,
		id=id,
		scheduleEndSec=TimeUtils:getSyncMillis()/1000+afterSec, --计划到期时间(1970年到现在的秒)
	}

	local isInclude=false
	for c,_ in pairs(_includeCategory) do
		if c == category then
			isInclude=true
			break
		end
	end
	if isInclude then
		LocalPushProxy:scheduleLlocalNotification(
			alterConent,
			afterSec+3,
			CFG.APP_NAME.."_"..category,
			_id
		)
	end

	if not _checkScheduleId then
		Cocos2dxUtils:interval(
			function()
				self:check()
			end,
			1
		)
	end

	return id
end

--- 取消本地推送
-- @param localPushId 推送的唯一ID
function T:cancel(id)
	local info=_list[id]
	if info then
		LocalPushProxy:cancelNotification(CFG.APP_NAME.."_"..info.category,id)

		_list[id]=nil
	end
	
	for _ in pairs(_list) do
		return
	end
	
	if _checkScheduleId then
		Cocos2dxUtils:clearScheduler(_checkScheduleId)
	end
end

--- 包含指定分类
-- @param category 类型名
function T:includeCategory(category)
	if _includeCategory[category] then
		return
	end

	local curSec=TimeUtils:getSyncMillis()/1000
	for k,v in pairs(_list) do
		if v.category == category then
			local afterSec=math.ceil(v.scheduleEndSec-curSec)
			if afterSec > 0 then
				LocalPushProxy:scheduleLlocalNotification(
					v.alterConent,
					afterSec+3,
					CFG.APP_NAME.."_"..category,
					k
				)
			end
		end
	end

	_includeCategory[category]=true
end

--- 排除指定分类
-- @param category 类型名
function T:excludeCategory(category)
	if _includeCategory[category] then
		LocalPushProxy:cancelNotifications(CFG.APP_NAME.."_"..category)
	
		_includeCategory[category]=nil
	end
end

return T
