--- 数据存储
DataStorage=require("src.app.utils.DataStorage")

local luaj = LuaJavaBridge
local LuaInterface_LocalNotification="org.cocos2dx.lua.LuaInterface_LocalNotification"

local T=Class.LocalNotification_android()

local _print=print
local function print(...)
	_print(string.format("\t<%s>:%s",T.__name,table.concat({...},"\t")))
end

--- 触发的延迟时间(秒)
T.TRIGGER_DELAY_SEC=3

--- 建筑/生产推送
T.CATEGORY_BUILD="category_build"

--- 精力点回复提示
T.PVE_POINT_TIP="pve_point_tip"

--- 防护罩状态推送
T.PROTECT_COVER="protect_cover"

function T:def()
	--- 信息列表
	self._list={}

	--- 包含的分类列表
	self._includeCategory={}
	
	--- 推送信息列表
	self._notificationList=DataStorage:getLocalNotificationInfo() or {}
end

function T:ctor()
	self:_check()
end

function T:_push(notificationInfo)
	local nInfo=notificationInfo
	local strId=nInfo.strId
	
	local nList=self._notificationList

	--如果已在推送列表中
	if nList[strId] then
		return
	end

	local delaySec=math.floor(nInfo.scheduleEndSec - TimeUtils:getSyncMillis()/1000)
	--如果信息已经过期
	if delaySec < 0 then
		return
	end

	nList[strId]=nInfo
	DataStorage:putLocalNotificationInfo(nList)

	luaj.callStaticMethod(
		LuaInterface_LocalNotification,
		"push",
		{
			json:encode({strId=nInfo.strId,id=nInfo.id,content=nInfo.content,delaySec=delaySec + self.TRIGGER_DELAY_SEC})
		},
		"(Ljava/lang/String;)V"
	)
end
function T:_cancel(notificationInfo)
	local nInfo=notificationInfo
	local strId=nInfo.strId
	
	local nList=self._notificationList

	--如果未在推送列表中
	if not nList[strId] then
		return
	end

	local delaySec=math.floor(nInfo.scheduleEndSec - TimeUtils:getSyncMillis()/1000)
	--	--如果信息已经过期
	--	if delaySec > 0 then
	--		return
	--	end

	nList[strId]=nil
	DataStorage:putLocalNotificationInfo(nList)

	luaj.callStaticMethod(
		LuaInterface_LocalNotification,
		"cancel",
		{
			json:encode({strId=nInfo.strId,id=nInfo.id,content=nInfo.content,delaySec=delaySec + self.TRIGGER_DELAY_SEC})
		},
		"(Ljava/lang/String;)V"
	)
end

function T:_check()
	local nList=self._notificationList

	for k,nInfo in pairs(nList) do
		local delaySec=math.floor(nInfo.scheduleEndSec - TimeUtils:getSyncMillis()/1000)
		--如果信息已经过期
		if delaySec < 0 then
			nList[k]=nil

			self:_cancel(nInfo)
		end
	end
end

--- 本地推送
-- @param alterConent 推送信息的内容
-- @param afterSec 指定时间(秒)后推送本地信息
-- @param category 分类
-- @param id 信息的唯一编号
-- @return 本次推送的唯一ID,记录此ID,以便取消此信息的推送
function T:push(alterConent,afterSec,category,id)
	print(string.format("添加本地推送=>\nalterContent:%s\nafterSec:%s\ncategory:%s\nid:%s",alterConent,tostring(afterSec),category,id))

	if afterSec <= 0 then
		return -1
	end

	local intId=DataStorage:generateUniqueId(id)

	local l=self._list[category]
	if not l then
		l={}
		self._list[category]=l
	end

	local isIncluded=self._includeCategory[category]

	-- 如果在包含的分类中,则取消以前的推送信息
	if isIncluded then
		self:cancel(id)
	end

	--- 通知信息
	local nInfo={
		strId=id,
		id=intId,
		content=alterConent,
		scheduleEndSec=TimeUtils:getSyncMillis()/1000+afterSec, --计划到期时间(1970年到现在的秒)
	}
	l[id]=nInfo

	if isIncluded then
		self:_push(nInfo)
	end

	return id
end

--- 取消本地推送
-- @param localPushId 推送的唯一ID
function T:cancel(id)
	print(string.format("取消信息推送=>id:%s",id))

	--- 通知信息
	local nInfo
	for _,l in pairs(self._list) do
		nInfo=l[id]
		if nInfo then
			l[id]=nil
			break
		end
	end

	if nInfo then
		self:_cancel(nInfo)
	end
end

--- 包含指定分类
-- @param category 类型名
function T:includeCategory(category)
	local ic=self._includeCategory
	if ic[category] then
		return
	end
	ic[category]=true

	local l=self._list[category]
	if l then
		for _,nInfo in pairs(l) do
			self:_push(nInfo)
		end
	end
end

--- 排除指定分类
-- @param category 类型名
function T:excludeCategory(category)
	local ic=self._includeCategory
	if not ic[category] then
		return
	end
	ic[category]=nil

	local l=self._list[category]
	if l then
		for _,nInfo in pairs(l) do
			_cancel(nInfo)
		end
	end
end

--- 应用程序进入后台
function T:app2background()
end

--- 应用程序进入前台
function T:app2foreground()
end

return T
