require("src.cfg.Serialization")

local T={}

--- 应用名
T.APP_NAME="airwar"

--cpp核心版本
T.CPP_VERSION=str2obj(CppCocos2dxUtils:getUserData()).cppVersion

--当前lua代码和资源版本
T.LUA_VERSION="4.0.11"

--- 文件操作工具
T.fUtils=cc.FileUtils:getInstance()

--- 获取更新文件存储目录
function T:getUpdateDir()
	return self.fUtils:getWritablePath()..self.APP_NAME.."_update_"..self.CPP_VERSION
end

--- 更新搜索目录,检查更新目录是否存在,如果存在,具更新目录文件的lua版本号大于当前lua版本号,将更新目录加入搜索目录
function T:updateSearchDir()
	local updateDir=self:getUpdateDir()

	if not CppCocos2dxUtils:isDirExist(updateDir) then
		print("创建目录:",updateDir)
		CppCocos2dxUtils:createDir(updateDir)

		self:putUpdateCfg({})
	end

	if self:compareVersion(self:getVersion(),self.LUA_VERSION) >= 0 then
		--将更新目录插入到搜索目录开始位置
		local paths=self.fUtils:getSearchPaths()
		table.insert(paths,1,updateDir)
		self.fUtils:setSearchPaths(paths)
		print("搜索目录:",obj2str(paths))
	else
		--一般不会出现这种情况,如果有此种情况,需要下面任意一种方法进行处理
		--1.删除更新目录(功能未实现)
		--2.修改更新目录位置(目前依赖于cpp的版本号来实现)
	end
end

--- 获取存储在更新目录中的配置数据
function T:getUpdateCfg()
	local updateDir=self:getUpdateDir()
	local updateCfgUrl=updateDir.."/udateCfg.dat"

	if self.fUtils:isFileExist(updateCfgUrl) then
		local data=ByteArray:new()
		if CppCocos2dxUtils:fRead(updateCfgUrl,data) > 0 then
			data:setPosition(0)

			local s=data:readString()
			print("更新配置数据:",s)
			return str2obj(s)
		end
	end

	return {}
end

--- 获取最新的版本号
function T:getVersion()
	local updatedVersion=self:getUpdateCfg().updatedVersion
	if self:compareVersion(updatedVersion,self.LUA_VERSION) >= 0 then
		return updatedVersion
	end
	return self.LUA_VERSION
end

--- 存储更新配置数据到更新目录
-- @param updateCfg 更新配置数据
-- @return 是否写入成功
function T:putUpdateCfg(updateCfg)
	local updateDir=self:getUpdateDir()
	local updateCfgUrl=updateDir.."/udateCfg.dat"

	local data=ByteArray:new()
	data:writeString(obj2str(updateCfg))

	return CppCocos2dxUtils:fWrite(updateCfgUrl,data) > 0
end

--- 比较版本(如果 version1 版本号大于 version2 则返回正数,否则为负数,相等则返回为 0,返回的数值表示比较到第 n 级时不一致)
-- @param version1 版本号1
-- @param version2 版本号1
-- @return 如果 version1 版本号大于 version2 则返回正数,否则为负数,相等则返回为 0,返回的数值表示比较到第 n 级时不一致
function T:compareVersion(version1,version2)
	version1,version2=version1 or "0.0.0",version2 or "0.0.0"

	local ver1Arr,ver2Arr=self.split(version1,"."),self.split(version2,".")

	local l=math.max(#ver1Arr,#ver2Arr)
	for i=1,l,1 do
		local d=(tonumber(ver1Arr[i]) or 0) - (tonumber(ver2Arr[i]) or 0)

		if d > 0 then
			return i
		elseif d < 0 then
			return -i
		end
	end

	return 0
end

--- 劈分字符串
-- @param text 字符串
-- @param sep 基于此符号劈分
function T.split(text, sep)
	local sep, fields = sep or "\t", {}
	local pattern = string.format("([^%s]+)", sep)
	string.gsub(text,pattern, function(c)
		fields[#fields+1] = c
	end)
	return fields
end

T:updateSearchDir()

rawset(_G,"CFG",T)