--- 保留的方法名/属性名
local reservedKey ={
	__index			= true,
	__newindex		= true,
	__type			= true,
}

--- 浅表克隆继承
-- @param ... [className,][createFunction,][bases]
-- className 类名
-- createFunction 实例创建函数,创建 tolua 中的对象(C++对象)
-- bases 基类列表
local function extend(...)
	local cls = {__type = 'class'}

	local bases={...}

	-- 如果第一个参数类型为 string ,则设置为类名
	if type(bases[1]) == 'string' then
		cls.__name = bases[1]
		table.remove(bases, 1)
	else
		cls.__name = cls
	end

	-- 如果第一个参数类型为 function ,则实例创建函数,创建 tolua 中的对象(C++对象)
	if type(bases[1]) == 'function' then
		cls.__createFunction = bases[1]
		table.remove(bases, 1)
	end

	-- 模糊方法/属性(多个基类的同名方法或属性) key 列表
	local ambiguousKeys={}

	-- 拷贝基类的方法/静态属性
	for i,base in ipairs(bases) do
		assert(type(base) == "table","the base class must be \"table\"")

		for k,v in pairs(base) do
			if type(v) == "function" and not reservedKey[k] then
				if not ambiguousKeys[k] then
					cls[k]=v
				else
					cls[k]=nil
				end
			end
		end
	end
	cls.__index = cls
	cls.__bases=bases

	-- 默认构造方法(如果不被重写,则默认会调用此方法)
	function cls:ctor() end

	--- 将类名作为实现创建方法
	local function __call(cls,...)
		local ins=nil
		local __createFunction=rawget(cls,"__createFunction")
		if __createFunction then
			ins=__createFunction()

			local t = tolua.getpeer(ins)
			if not t then
				t = {}
				tolua.setpeer(ins, t)
			end
			setmetatable(t,cls)
		else
			ins=setmetatable({},cls)
		end
		ins.__index=cls
		ins.__type="Object"
		ins.class=cls

		--成员变量定义函数
		if ins.def then
			ins:def()
		end

		--构造函数
		if ins.ctor then
			ins:ctor(...)
		end

		return ins
	end

	local n=#bases
	if n == 1 then
		local firstBase=bases[1]
		if not firstBase.def then
			function cls:def() end
		end
		
		setmetatable(cls, {__call = __call,__index=firstBase})
	elseif n > 1 then
		local __index=function(o,k)
			for _,base in pairs(o.__bases) do
				local val=base[k]
				if val then
					return val
				end
			end
			return nil
		end

		-- 默认变量定义方法(如果不被重写,则默认会调用此方法)
		function cls:def()
			local bases=self.__bases
			for _,base in ipairs(bases) do
				local def=base.def
				if def then
					def(self)
				end
			end
		end

		setmetatable(cls, {__call = __call,__index=__index})
	else
		setmetatable(cls, {__call = __call})
	end

	return cls
end

rawset(_G,"Class",{})
local mt={}

setmetatable(Class,mt)

function mt:__index(name)
	return function(...)
		local c = extend(name, ...)
		--getfenv()[name] = c
		return c
	end
end

return Class
