--- 事件派发器
local T=Class.LuaEventDispatcher2()

function T:def()
	--- 事件侦听器列表
	self.types={}

	--- 是否正在回调
	self.isCalling=false
end

--- 添加事件侦听器
-- @param self
-- @param type 事件类型
-- @param host 宿主
-- @param listener 回调函数
function T:luaAddEventListener(type,host,listener)
	host=host or 0

	local list=self.types[type]
	if not list then
		list={}
		self.types[type]=list
	end

    xpcall(
        function()
            list[tostring(host)..tostring(listener)]={host=host,listener=listener}
        end,
        function(errStr)
           
			local message=tostring(errStr).."\n"..debug.traceback()

			__print__("InitDispatchErr",message)
			LogRecorder:push(message,LOG_LEVEL_ERROR,"InitDispatchErr")
		end
    )

end

--- 移除事件侦听器
-- @param self
-- @param type 事件类型
-- @param host 宿主
-- @param listener 回调函数
function T:luaRemoveEventListener(type,host,listener)
	local list=self.types[type]
	if not list then
		return
	end

	host=host or 0
	local key=tostring(host)..tostring(listener)

	if self.isCalling then
		local t=list[key]
		if t then
			t.listener=nil
		end
	end
	list[key]=nil
end

--- 移除事件侦听器
-- @param self
-- @param type 事件类型
function T:luaRemoveEventListenersByType(type)
	self.types[type]=nil
end

--- 派发事件
-- @param self
-- @param type 事件类型
-- @param ... 数据
function T:luaDispatchEvent(type,...)
	local list=self.types[type]
	if not list then
		return
	end
	local tempList,i={},1
	for _,v in pairs(list) do
		tempList[i]=v
		i=i+1
	end

	self.isCalling=true
	local args={...}
	for _,v in ipairs(tempList) do
		local f=v.listener
		if f then
			local h=v.host

			local h=v.host

            if h == 0 then
				f(...)
			else
				f(h,...)
			end
		end
	end
	self.isCalling=false
end

--- 释放事件侦听器
function T:releaseEventListeners()
	self.types={}
end

return T
