--- 字符串处理的一些方法
local StringUtils={}

--- 劈分字符串
-- @param text 字符串
-- @param sep 基于此符号劈分
function StringUtils.split(text, sep)
	local sep, fields = sep or "\t", {}
	local pattern = string.format("([^%s]+)", sep)
	string.gsub(text,pattern, function(c)
		fields[#fields+1] = c
	end)
	return fields
end

--- 获取宽字节字符的数目
-- @param 字符串
function StringUtils.getWCharCount(text)
	local c = 0

	local strByte=string.byte
	local strSub=string.sub

	local len = string.len(text)
	local i, asciiCode = 1, nil
	while i <= len do
		asciiCode = strByte(text, i)

		if asciiCode >= 0xfc then
			i = i + 6
		elseif asciiCode >= 0xf8 then
			i = i + 5
		elseif asciiCode >= 0xf0 then
			i = i + 4
		elseif asciiCode >= 0xe0 then
			i = i + 3
		elseif asciiCode >= 0xc0 then
			i = i + 2
		else
			i = i + 1
		end

		c = c + 1
	end

	return c
end

--- 获取宽字节字符的数目(中文算2个字符,英文算1个字符)
-- @param 字符串
function StringUtils.getCount(text)
	local c = 0

	local strByte=string.byte
	local strSub=string.sub

	local len = string.len(text)
	local i, asciiCode = 1, nil
	while i <= len do
		asciiCode = strByte(text, i)

		if asciiCode >= 0xfc then
			i = i + 6

			c = c + 2
		elseif asciiCode >= 0xf8 then
			i = i + 5

			c = c + 2
		elseif asciiCode >= 0xf0 then
			i = i + 4

			c = c + 2
		elseif asciiCode >= 0xe0 then
			i = i + 3

			c = c + 2
		elseif asciiCode >= 0xc0 then
			i = i + 2

			c = c + 2
		else
			i = i + 1

			c = c + 1
		end
	end

	return c
end

--- 获取指定长度的字符(中文算2个字符,英文算1个字符)
-- @param text 字符串
-- @param count 长度(中文算2个字符,英文算1个字符)
function StringUtils.getCountChar(text,count)
	local c = 0

	local strByte=string.byte
	local strSub=string.sub

	local len = string.len(text)
	local i, asciiCode, t = 1, nil, 1
	while i <= len do
		asciiCode = strByte(text, i)

		if asciiCode >= 0xfc then
			t, c = 6, c + 2
		elseif asciiCode >= 0xf8 then
			t, c = 5, c + 2
		elseif asciiCode >= 0xf0 then
			t, c = 4, c + 2
		elseif asciiCode >= 0xe0 then
			t, c = 3, c + 2
		elseif asciiCode >= 0xc0 then
			t, c = 2, c + 2
		else
			t, c = 1, c + 1
		end
		i = i + t

		if c == count then
			return string.sub(text,1,i-1),string.sub(text,i)
		elseif c > count then
			return string.sub(text,1,i-t-1),string.sub(text,i - t)--截取的位置刚好是半个汉字
		end
	end

	return text,nil
end

--- 切分宽字节字符
-- @param 字符串
-- @return 宽字节字符数组,字符数量(数组长度)
function StringUtils.str2WCharArray(text)
	local wChars, c= { }, 0

	local strByte=string.byte
	local strSub=string.sub

	local len = string.len(text)
	local i, asciiCode = 1, nil
	while i <= len do
		c=c+1

		asciiCode = strByte(text, i)

		if asciiCode >= 0xfc then
			wChars[c] = strSub(text, i, i + 5)
			i = i + 6
		elseif asciiCode >= 0xf8 then
			wChars[c] = strSub(text, i, i + 4)
			i = i + 5
		elseif asciiCode >= 0xf0 then
			wChars[c] = strSub(text, i, i + 3)
			i = i + 4
		elseif asciiCode >= 0xe0 then
			wChars[c] = strSub(text, i, i + 2)
			i = i + 3
		elseif asciiCode >= 0xc0 then
			wChars[c] = strSub(text, i, i + 1)
			i = i + 2
		else
			wChars[c] = strSub(text, i, i)
			i = i + 1
		end
	end

	return wChars,c
end

--- 获取字符串中的前n个字符
-- @param text 字符串
-- @param n
-- @return
function StringUtils.getNWChar(text,n)
	local c=0

	local strByte=string.byte

	local len = string.len(text)
	if len <= n then
		return text
	end

	local i, asciiCode, t= 1, nil,1
	while i <= len do
		asciiCode = strByte(text, i)

		if asciiCode < 0xc0 then
			--英文字符
			i = i + 1
		else
			--非英文字符
			if asciiCode >= 0xfc then
				i = i + 6
			elseif asciiCode >= 0xf8 then
				i = i + 5
			elseif asciiCode >= 0xf0 then
				i = i + 4
			elseif asciiCode >= 0xe0 then
				i = i + 3
			else
				i = i + 2
			end
		end
		c = c + 1
		if c == n then
			return string.sub(text,1,i-1),string.sub(text,i)
		end
	end

	return text
end

--- 返回指定宽度内的字符串
-- @param text 字符串
-- @param 最大长度
-- @param 字号
-- @return 指定长度内的字符串,超出长度的字符串
function StringUtils.limitWChar(text,widthLimit,size)
	local w=0

	local strByte=string.byte

	local len = string.len(text)
	if len * size <= widthLimit then
		return text,nil
	end

	local i, asciiCode, t= 1, nil,1
	while i <= len do
		asciiCode = strByte(text, i)

		if asciiCode < 0xc0 then
			--英文字符
			w=w+size/2

			t=1
		else
			--非英文字符

			w=w+size
			if asciiCode >= 0xfc then
				t=6
			elseif asciiCode >= 0xf8 then
				t=5
			elseif asciiCode >= 0xf0 then
				t=4
			elseif asciiCode >= 0xe0 then
				t=3
			else
				t=2
			end
		end
		i=i+t

		if w == widthLimit then
			return string.sub(text,1,i-1),string.sub(text,i)
		elseif w > widthLimit then
			return string.sub(text,1,i-t-1),string.sub(text,i - t)--截取的位置刚好是半个汉字
		end
	end

	return text,nil
end

--- 验证是否为邮件地址
function StringUtils:isEMailUrl(str)
	if string.len(str or "") < 6 then return false end
	local b,e = string.find(str or "", '@')
	local bstr = ""
	local estr = ""
	if b then
		bstr = string.sub(str, 1, b-1)
		estr = string.sub(str, e+1, -1)
	else
		return false
	end

	-- check the string before '@'
	local p1,p2 = string.find(bstr, "[%w_]+")
	if (p1 ~= 1) or (p2 ~= string.len(bstr)) then return false end

	-- check the string after '@'
	if string.find(estr, "^[%.]+") then return false end
	if string.find(estr, "%.[%.]+") then return false end
	if string.find(estr, "@") then return false end
	if string.find(estr, "[%.]+$") then return false end

	_,count = string.gsub(estr, "%.", "")
	if (count < 1 ) or (count > 3) then
		return false
	end

	return true
end

return StringUtils
