--- 单位转化
local UnitConversion={}

--- 小数保留位数，如果小数位数大于3位，就保留3位小数,这里不四舍五入的
function UnitConversion.handleDecimal(value)
    local num = math.floor(value * 1000)
    return num/1000
end


--- 得到一个小于等于5位数的每一位，前面不够的补0
function UnitConversion.getEveryBit(value)
    if not value then return end
    local data = {0,0,0,0,0} -- 从左到右，从大的到个位
    if value >= 0 and value < 10 then -- 1位
        data = {0,0,0,0,value}
    elseif value >= 10 and value < 100 then -- 2位
        local two = math.floor(value/10)
        local one = value%10
        data = {0,0,0,two,one}
    elseif value >= 100 and value < 1000 then -- 3位
        local three = math.floor(value/100)
        local temp = value%100
        local two = math.floor(temp/10)
        local one = temp%10
        data = {0,0,three,two,one}
    elseif value >= 1000 and value < 10000 then -- 4位
        local four = math.floor(value/1000)
        local temp1 = value%1000

        local three = math.floor(temp1/100)
        local temp = temp1%100
        local two = math.floor(temp/10)
        local one = temp%10
        data = {0,four,three,two,one}

    elseif value >= 10000 and value < 100000 then -- 5位
        local five = math.floor(value/10000)
        local temp2 = value%10000

        local four = math.floor(temp2/1000)
        local temp1 = temp2%1000

        local three = math.floor(temp1/100)
        local temp = temp1%100
        local two = math.floor(temp/10)
        local one = temp%10
        data = {five,four,three,two,one}
    end

    return data

end


--- 如果是数字是1位数则返前面加个0
function UnitConversion.date(num)
    if num > 0 and num < 10 then
        return '0' .. num
    elseif num == 0 then  -- 比如11:00
        return '00'
    else
        return num
    end
end

--- 得到今天是今年的第几天
function UnitConversion.getYday()
    local t = os.date("*t",TimeUtils:getSyncMillis()/1000)
    return t.yday

end

--- 根据时间来算离现在相差多少天
function UnitConversion.getDay(time)
    local lastTime = time
    local lastData = os.date("*t",lastTime)
    local nowData = os.date("*t",TimeUtils:getSyncMillis()/1000)
    if lastData.year == nowData.year then
        local day = nowData.yday - lastData.yday
        return day
    else -- 昨年(或者前年，或者前上年)创建的号码,今年还在玩,现在暂时考虑为1年,2015年(365天)申请的号，2016年在玩
        local day = nowData.yday + (365 - lastData.yday)
        return day
        
    end

end

--- 1小时20分
function UnitConversion.handleTime(time)
    if time < 60 and time >= 0 then
        local data = string.format("%0.0f", time) .. Language:get("uc10") 
        return data
    elseif time < 3600 and 60 <= time then
        local minute = math.floor(time/60) 
        local second = time%60
        if second == 0 then 
            local data = string.format("%0.0f", minute) ..Language:get("uc9")
            return data
        else
            local data = minute ..Language:get("uc9") .. string.format("%0.0f", second) .. Language:get("uc10")
            return data
        end
    elseif time >= 3600 then
        local hour = math.floor(time/3600)
        local timeRest = time%3600

        if math.floor(timeRest/60) == 0 then 
            local second = timeRest%60
            if second == 0 then
                return string.format("%0.0f", hour) .. Language:get("uc8")
            else 
                return hour .. Language:get("uc8") .. string.format("%0.0f", second) .. Language:get("uc10")
            end
            
        elseif math.floor(timeRest/60) > 0 then 
            local minute = math.floor(timeRest/60)
            if timeRest%60 == 0 then 
                return hour .. Language:get("uc8") .. string.format("%0.0f", minute) .. Language:get("uc9")
            else
                local second = timeRest%60
                return hour .. Language:get("uc8") .. minute .. Language:get("uc9") .. string.format("%0.0f", second) .. Language:get("uc10")
            end
        end
    else
        print("输入的数字为不合法的")
    end
end

-- 美国科学计数法
function UnitConversion.USASciCount(num)
    if num >= 0 and num < 1000 then
        return num
    elseif num >= 1000 and num <  1000000 then
        local newData = math.floor(num/1000)
        local ramain = num%1000
        return newData .. ',' .. ramain
    elseif num >= 1000000 and num < 1000000000 then
        local newData1 = math.floor(num/1000)
        local ramain1 = num%1000

        local newData2 = math.floor(newData1/1000)
        local ramain2 = newData1%1000

        return newData2 .. ',' .. ramain2 .. ',' .. ramain1

    elseif num >= 1000000000 then
        local newData1 = math.floor(num/1000)
        local ramain1 = num%1000

        local newData2 = math.floor(newData1/1000)
        local ramain2 = newData1%1000

        local newData3 = math.floor(newData2/1000)
        local ramain3 = newData2%1000

        return newData3 .. ',' .. ramain3 .. ',' .. ramain2 .. ',' .. ramain1
    else
        print("输入的数字为不合法的")
    end
end


-- 单位转化,小数点前后一共3位数字,并且进位时不四舍五入,后面要用0补充
function UnitConversion.unitTrans(num)  -- 这里的num要考虑小数的情况
    if num > 0 and num < 1000 then 
--        local newData = UnitConversion.changeNumber(num)
--        return newData
        local newData = math.floor(num)  -- 这里向下取整
        return newData
    elseif num >= 1000 and num < 1000000  then
        local data = num/1000
        local newData = UnitConversion.changeNumber(data)
        return newData .. 'K' 
    elseif num >= 1000000 then
        local data = num/1000000
        local newData = UnitConversion.changeNumber(data)
        return newData .. 'M'
    elseif num == 0 then
        return num
    else
         print("输入的数字为不合法的")
    end
end

-- 单位转化,小数点前后一共3位数字,并且进位时不四舍五入,后面不用0补充
function UnitConversion.unitTrans2(num)  -- 这里的num要考虑小数的情况
    if num > 0 and num < 1000 then 
        local newData = UnitConversion.changeNumber(num)
        return UnitConversion.removeRemainder(newData)
    elseif num >= 1000 and num < 1000000  then
        local data = num/1000
        local newData = UnitConversion.changeNumber(data)
        return UnitConversion.removeRemainder(newData) .. 'K' 
    elseif num >= 1000000 then
        local data = num/1000000
        local newData = UnitConversion.changeNumber(data)
        return UnitConversion.removeRemainder(newData) .. 'M'
    elseif num == 0 then
        return num
    else
         print("输入的数字为不合法的")
    end
end

--- 判断一个最多2位的小数，后面如果余数有0，把0取消掉
function UnitConversion.removeRemainder(num)
    local num = num
    local integer = math.floor(num)
    local remain = num - integer

    local tempNum = remain * 100
    if tempNum%10 == 0 then -- 第2位是0
        local newData = tempNum * 10
        if newData == 0 then
            return string.format("%0.0f", num)
        else
            return string.format("%0.1f", num)
        end
    else -- 第2位不是0
        return num
    end

end

--- 把一个数改成3位（这里的数的整数最多3位）
function UnitConversion.changeNumber(num)
    local num = num

    local newNum = nil
    -- 整数
    local integer = math.floor(num)
    if integer < 10 then  -- 1位,保留2位小数
        local tempNum = math.floor(num * 100)/100
        newNum = string.format("%0.2f", tempNum) -- 0表示，小数不够时用0填充
    elseif integer < 100 and integer >= 10 then -- 2位，保留1位小数
        local tempNum = math.floor(num * 10)/10
        newNum = string.format("%0.1f", tempNum)
    else  -- 3位
        newNum = integer
    end

    return newNum
end

--- 1:20
function UnitConversion.ShowTime(time)
    if time < 60 and time > 0 then
        local data = string.format("%0.0f", time)
        return data
    elseif time < 3600 and 60 <= time then
        local minute = math.floor(time/60) 
        local second = time%60
        if second == 0 then 
            local data = string.format("%0.0f", minute)
            return data
        else
            local data = minute .. ':' .. string.format("%0.0f", second)
            return data
        end
    elseif time >= 3600 then
        local hour = math.floor(time/3600)
        local timeRest = time%3600

        if math.floor(timeRest/60) == 0 then 
            local second = timeRest%60
            if second == 0 then
                return string.format("%0.0f", hour)
            else 
                return hour .. ':' .. '00' .. ':' .. string.format("%0.0f", second)
            end

        elseif math.floor(timeRest/60) > 0 then
            local minute = math.floor(timeRest/60)
            if timeRest%60 == 0 then 
                return hour .. ':' .. string.format("%0.0f", minute)
            else
                local second = timeRest%60
                return hour .. ':' .. minute .. ':' .. string.format("%0.0f", second)
            end
        end
    else
        print("输入的数字为不合法的")
    end
end


--- 处理时间函数,这种转化为‘1h20m1s’这种
function UnitConversion.handleTime2(time)
    if time < 60 and time > 0 then
        local data = string.format("%0.0f", time) .. 's' -- 处理有小数的情况,如果前面写6位，则显示会空显示
        return data
    elseif time < 3600 and 60 <= time then
        local minute = math.floor(time/60) -- 向下取整
        local second = time%60
        if second == 0 then 
            local data = string.format("%0.0f", minute) .. 'm'
            return data
        else
            local data = minute .. 'm' .. string.format("%0.0f", second) .. 's'
            return data
        end
    elseif time >= 3600 then
        local hour = math.floor(time/3600)
        local timeRest = time%3600

        if math.floor(timeRest/60) == 0 then -- 小时，秒
            local second = timeRest%60
            if second == 0 then
                return string.format("%0.0f", hour) .. 'h'
            else 
                return hour .. 'h' .. string.format("%0.0f", second) .. 's'
            end

        elseif math.floor(timeRest/60) > 0 then -- 有分
            local minute = math.floor(timeRest/60)
            if timeRest%60 == 0 then 
                return hour .. 'h' .. string.format("%0.0f", minute) .. 'm'
            else
                local second = timeRest%60
                return hour .. 'h' .. minute .. 'm' .. string.format("%0.0f", second) .. 's'
            end
        end
    else
        print("输入的数字为不合法的")
    end
end

--- 处理时间函数,这种转化为‘1H20M5S’这种
function UnitConversion.handleTime3(time)
    if time < 60 and time > 0 then
        local data = string.format("%0.0f", time) .. 's' -- 处理有小数的情况,如果前面写6位，则显示会空显示
        return data
    elseif time < 3600 and 60 <= time then
        local minute = math.floor(time/60) -- 向下取整
        local second = time%60
        if second == 0 then 
            local data = string.format("%0.0f", minute) .. 'm'
            return data
        else
            local data = minute .. 'm' .. string.format("%0.0f", second) .. 's'
            return data
        end
    elseif time >= 3600 then
        local hour = math.floor(time/3600)
        local timeRest = time%3600

        if math.floor(timeRest/60) == 0 then -- 小时，秒
            local second = timeRest%60
            if second == 0 then
                return string.format("%0.0f", hour) .. 'h'
            else 
                return hour .. 'h' .. string.format("%0.0f", second) .. 's'
            end

        elseif math.floor(timeRest/60) > 0 then -- 有分
            local minute = math.floor(timeRest/60)
            if timeRest%60 == 0 then 
                return hour .. 'h' .. string.format("%0.0f", minute) .. 'm'
            else
                local second = timeRest%60
                return hour .. 'h' .. minute .. 'm' .. string.format("%0.0f", second) .. 's'
            end
        end
    else
        print("输入的数字为不合法的")
    end
end

--- 最大的单位有天，然后有小时，分钟，秒，并且显示的时候最多2位
function UnitConversion.handleTime4(time)
    if time < 60 and time >= 0 then
        local data = string.format("%0.0f", time) .. 's' -- 处理有小数的情况,如果前面写6位，则显示会空显示
        return data
    elseif time < 3600 and 60 <= time then
        local minute = math.floor(time/60) -- 向下取整
        local second = time%60
        if second == 0 then 
            local data = string.format("%0.0f", minute) .. 'm'
            return data
        else
            local data = minute .. 'm' .. string.format("%0.0f", second) .. 's'
            return data
        end
    elseif time >= 3600 and time < 24*3600 then
        local hour = math.floor(time/3600)
        local timeRest = time%3600

        if math.floor(timeRest/60) == 0 then -- 小时，秒
            local second = timeRest%60
            if second == 0 then
                return string.format("%0.0f", hour) .. 'h'
            else 
                return hour .. 'h' .. string.format("%0.0f", second) .. 's'
            end

        elseif math.floor(timeRest/60) > 0 then -- 有分
            local minute = math.floor(timeRest/60)
            if timeRest%60 == 0 then 
                return hour .. 'h' .. string.format("%0.0f", minute) .. 'm'
            else
                local second = timeRest%60
                --return hour .. 'H' .. minute .. 'M' .. string.format("%0.0f", second) .. 'S'
                return hour .. 'h' .. minute .. 'm'
            end
        end
    elseif time >= 24*3600 then
        local day = math.floor(time/(24*3600))
        local timeRest = time%(24*3600)
        if math.floor(timeRest/3600) == 0 then -- 天，（分，秒< 3600）
            if timeRest < 60 then -- 只有秒
                return day .. 'd' .. timeRest .. 's'
            else -- 有分，可能有秒
                local minute = math.floor(timeRest/60) -- 向下取整
                return day .. 'd' .. minute .. 'm'
            end
        elseif math.floor(timeRest/3600) > 0 then -- 天，（小时，分，秒）
            local hour = math.floor(timeRest/3600)
            return day .. 'd' .. hour .. 'h'
        end
    else
        print("输入的数字为不合法的")
    end
end


--- 时间计数法：00:59:06 只有小时，分，秒这种
function UnitConversion.timeCount(time)
    if time >= 0 and time < 60 then -- 秒
        if time < 10 then
            return "00:00:0" .. time
        else
            return "00:00:" .. time
        end
    elseif time >= 60 and  time < 3600 then
        local minute = math.floor(time/60) -- 向下取整
        local second = time%60
        if minute < 10 then
            minute = "0" .. minute
        end

        if second < 10 then
            second = "0" .. second
        end
        return "00:" .. minute .. ":" .. second

    elseif time == 3600 then   -- 现在只考虑1个小时的
        return "01:00:00"

    end


end



--- 时间计数法2：00:59:06 只有小时，分，秒这种(如果是天数，转化为小时就行了),最后是00:00:00结束
function UnitConversion.timeCount2(time)
    if time >= 0 and time < 60 then -- 秒
        if time < 10 then
            return "00:00:0" .. math.floor(time)
        else
            return "00:00:" .. math.floor(time)
        end
    elseif time >= 60 and  time < 3600 then
        local minute = math.floor(time/60) -- 向下取整
        local second = math.floor(time%60)
        if minute < 10 then
            minute = "0" .. minute
        end

        if second < 10 then
            second = "0" .. second
        end
        return "00:" .. minute .. ":" .. second

    elseif time >= 3600 then
        local hour = math.floor(time/3600) -- 向下取整
        if hour < 10 then
            hour = "0" .. hour
        end

        local mOrs = time%3600
        local minute = math.floor(mOrs/60) -- 向下取整
        local second = math.floor(time%60)
        if minute < 10 then
            minute = "0" .. minute
        end

        if second < 10 then
            second = "0" .. second
        end

        return hour .. ":" .. minute .. ":" .. second
    end


end


return UnitConversion
