--- 深拷贝对象
-- @param srcObj 对象
function deepCopy(srcObj)
	--- 已被拷贝的对象列表(解决循环引用的问题)
	local lookup={}

	local function _copy(object)
		if type(object) ~= "table" then
			return object
		elseif lookup[object] then
			return lookup[object]
		end

		local newTable={}
		lookup[object]=newTable

		for k,v in pairs(object) do
			newTable[_copy(k)]=_copy(v)
		end

		return setmetatable(newTable,getmetatable(object))
	end

	return _copy(srcObj)
end

--- 将src中的所有数据映射到des中
-- @param des 目标
-- @param template 模板
function mapping(des,template)
	template=deepCopy(template)
	for k,v in pairs(template) do
		des[k] = v
	end
	return des
end

--- 深度映射
function deepMapping(des,src)
	for k,v in pairs(src) do
		local desV=des[k]
		if desV and type(desV) == "table" and type(v) == "table" then
			deepMapping(desV,v)
		else
			des[k]=v
		end
	end
end

--- 从集合中随机选取指定个数的元素
-- @param list 原集合
-- @param count 选取的元素个数,原数目不足,则直接返回原数据的浅表副本
-- @return 随机选取的元素集合
function randomSelect(list,count)
	--复制集合,因为不能直接修改原表
	local copy={}
	local n=0
	for _,v in pairs(list) do
		n=n+1
		copy[n]=v
	end

	if n<count then
		return copy
	end

	local ret={}
	for i=1,count,1 do
		local r=random(1,n)
		ret[i]=table.remove(copy,r)
		n=n-1
	end

	return ret
end

--去除扩展名
function stripExtension(url)
	local idx = url:match(".+()%.%w+$")
	if(idx) then
		return url:sub(1, idx-1)
	else
		return url
	end
end

--获取扩展名
function getExtension(url)
	return url:match(".+%.(%w+)$")
end

function unrequire(url)
	package.loaded[url]=nil
end

function string.trim(text)
	return string.gsub(text,"^%s*(.-)%s*$", "%1")
end

--- 获取当前系统的时区偏移(秒)
function getLocalTimeZoneOffset()
	local test = os.time({year = 1970,month = 1,day = 2,hour = 0})
	--该值是本地真实时区
	local localTimeZoon = 86400 - test
	--判断是否在夏令时
	local tm = os.time()
	if math.floor(((tm + localTimeZoon)%86400)/3600) ~= tonumber(os.date("%H",tm)) then
		--夏令时
		localTimeZoon = localTimeZoon + 3600
	end

	return localTimeZoon
end

--- 获取服务器的时区偏移
function getServerTimeZoneOffset(timezoon,utc,hours)
	if math.floor(((utc + timezoon)%86400)/3600) ~= hours then
		--夏令时
		return timezoon + 3600
	end
	
	return timezoon
end
