local strFind = string.find
local strSub = string.sub
local tabInsert = table.insert
local strGsub = string.gsub
local strTrim = string.trim

--- 解析标签头
local function decodeHead(headText)
	local s, e = nil, 0
	s, e = strFind(headText, "%s", e + 1)
	if not s then
		return { __name__ = strTrim(headText) }
	end

	local h = { }
	h.__name__ = strSub(headText, 1, s - 1)

	-- 解析属性列表
	while true do
		s = e + 1
		_, e = strFind(headText, "=", s)
		if not e then
			break
		end

		local k = strSub(headText, s, e - 1)

		_, s = strFind(headText, "[^%s]", e + 1)
		if not s then
			return
		end

        if strSub(headText, s, s) == "'" then
            _, e = strFind(headText, "'", s+1)
        elseif strSub(headText, s, s) == "\"" then
             _, e = strFind(headText, "\"", s+1)
        else
		    _, e = strFind(headText, "%s", s)
        end
		local v = strSub(headText, s, e)
		h[strTrim(k)] = strGsub(v, "^%s*[\"|']?(.-)[\"|']?%s*$", "%1") -- 去除头尾空格和引号

		if not e then
			break
		end
	end

	return h
end

--- XML解析器
local XMLDecoder = { }

--- 解析HTML文本
-- @param text XML文本
function XMLDecoder:decode(text)
	local root = { }

	local elStack = Stack()
	elStack:push(root)

	--s:文本的开始位置,e:文本的结束位置(标签的开始位置),n:标签的结束位置
	local s, e, n = 1, 0, 1

	--当前标签类型
	local tag = 0
	while true do
		tag = 0 -- 标签开始

		-- 查找标签头
		_, e = strFind(text, "<", e + 1)

		-- 文本子对象
		local t
		if e then
			if strSub(text, e, e + 1) == "</" then
				tag = 1 -- 成对标签结束
			end

			-- 查找标签结束符
			_, n = strFind(text, ">", e + 1)
			if n then
				t = strSub(text, s, e - 1)
				
				if strSub(text, n - 1, n) == "/>" then
					tag = 2 -- 单标签结束
					n = n - 1
				end
			else
				-- 未找到标签结束符,则从 s 开始到整个字符串结束都被视为文本
				t = strSub(text, s)
			end
		else
			t = strSub(text, s)
		end
		
		
		--if t and strTrim(t) ~= "" then
        if t and t ~= "" then
			local topEl = elStack:look()
			if topEl then
				if not topEl.__child__ then
					topEl.__child__ = { }
				end

				tabInsert(topEl.__child__, t)
			end
		end

		-- 未找到后续的标签头,结束解析
		if not e then
			break
		end

		if tag == 1 then
			-- 当前标签为尾标签,从栈中取出顶标签
			local tailText = strSub(text, e + 2, n - 1)
			local topEl = elStack:look()
			if topEl then
				if topEl.__name__ == tailText then
					elStack:pop()
				end
			end
		else
			-- 当前标签为头标签,解析头标签属性,并压入栈
			local headText = strSub(text, e + 1, n - 1)
			local el = decodeHead(headText)

			local topEl = elStack:look()
			if topEl then
				if not topEl.__child__ then
					topEl.__child__ = { }
				end

				tabInsert(topEl.__child__, el)
			end
			if tag == 2 then
				n = n + 1 -- 因为单标签结束是两个字符("/>")
			else
				elStack:push(el)
			end
		end

		s = n + 1
	end

	return root
end

function XMLDecoder:escape(text)
	local strByte = string.byte
	return string.gsub(text, "[<|>|%%]", function(t)
		return "&#" .. strByte(t) .. ";"
	end )
end

function XMLDecoder:unescape(text)
	local strLen = string.len
	local strChar = string.char

	return string.gsub(text, "&#%d+;", function(t)
		local code = string.gsub(t, "%p", "")
		if strLen(t) - strLen(code) == 3 then
			return strChar(code)
		end
	end )
end

return XMLDecoder
