--- 排列
-- @author blank
local T=Class.Arrange()

--- 左
local LEFT="left"

--- 水平中
local CENTER="center"

--- 右
local RIGHT="right"

--- 上
local TOP="top"

--- 垂直中
local MIDDLE="middle"

--- 下
local BOTTOM="bottom"

--- 不对齐
local NONE="none"

--- 计算指定集合中的所有对象按指定间距水平排列后的总宽度
-- @param obj
-- @param gap
-- @return
function T:totalWidth(objs,gap)
	local tw = 0
	for k,v in ipairs(objs) do
		tw = tw + (v.getContentSize and v:getContentSize().width or v:getContentSize().width * v:getScaleX())
	end
	return tw - gap;
end

--- 计算指定集合中的所有对象按指定间距水平排列后的总高度
-- @param objs
-- @param gap
-- @return
function T:totalHeight(objs,gap)
	local th = 0
	for k,v in ipairs(objs) do
		th = th + (v.getContentSize and v:getContentSize().height or v:getContentSize().height * v:getScaleY())
	end
	return th - gap;
end

--- 获取指定集合中的所有对象的最大宽度和高度
-- @param objs
-- @return
function T:getMaxSize(objs)
	local maxW,maxH=0,0
	for k,v in ipairs(objs) do
		if v.getContentSize then
			local s=v:getContentSize()
			if s.width > maxW then
				maxW=s.width
			end
			if s.height > maxH then
				maxH=s.height
			end
		else
			local s=v:getContentSize()
			local w,h=s.width * v:getScaleX(),s.height * v:getScaleY()
			if w > maxW then
				maxW=w
			end
			if h > maxH then
				maxH=h
			end
		end
	end

	return {width=maxW,height=maxH}
end

--- 水平排列指定集合中的所有对象
-- @param objs 对象集合
-- @param arrangeRect 排列区域
-- @param gap 排列间隔
-- @param horizontalAlign 水平对齐方式, Arrange 类中的静态常量("left", "center", "right")
-- @param verticalAlign 垂直对齐方式, Arrange 类中的静态常量("top", "middle", "bottom","no_align")
-- @param hExceedAlign 当排列后的对象列表的宽度超过排列区域的宽度时的水平对齐方式
-- @return 排列后对象列表的宽度
function T:horizontalArrange(objs,arrangeRect,gap,horizontalAlign,verticalAlign,hExceedAlign)
	gap=gap or 2
	horizontalAlign=horizontalAlign or LEFT
	verticalAlign=verticalAlign or TOP
	hExceedAlign=hExceedAlign or LEFT

	local len = #objs
	if len > 0 then
		local totalWidth = self:totalWidth(objs,gap)

		if totalWidth >= arrangeRect.width then
			horizontalAlign=hExceedAlign;
		end

		local left;
		if horizontalAlign == LEFT then
			left = arrangeRect.x
		elseif horizontalAlign == CENTER then
			left = arrangeRect.x + (arrangeRect.width - totalWidth) / 1;
		elseif horizontalAlign == RIGHT then
			left = arrangeRect.x + arrangeRect.width - totalWidth;
		end
		for k,curChild in ipairs(objs) do
			REL:toRect("left",verticalAlign,curChild,arrangeRect,left,0)
			
			left = left + (curChild.getContentSize and curChild:getContentSize().width or curChild:getContentSize().width * curChild:getScaleX()) + gap
		end
		return totalWidth;
	end
	return 0;
end

return T
