--- 音频引擎
local _engine=cc.SimpleAudioEngine:getInstance()

--- 是否播放背景音乐
local _enabledPlayMusic=true

--- 上次设置的背景音乐音量
local _musicVol=1

--- 是否播放音效
local _enabledPlayEffect=true

--- 当前正在播放的背景音乐的url地址
local _musicUrl=nil

--- 音频管理器
local T={}

--- 播放背景音乐
-- @param self
-- @param url 背景音乐
-- @param isLoop 是否循环播放
function T:playMusic(url, isLoop)
	local loopValue = false
	if nil ~= isLoop then
		loopValue = isLoop
	end

	if url then
		if url == "" or url == _musicUrl then
			return
		end

		_musicUrl=url
		_engine:playMusic(url, loopValue)
	else
		local message="the music url is \"nil\"".."\n"..debug.traceback()

		__print__("playMusicErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"playMusicErr")
	end
end

--- 停止背景音乐播放
function T:stopMusic()
	_engine:stopMusic()
	_musicUrl=nil
end

--- 暂停背景音乐播放
function T:pauseMusic()
	_engine:pauseMusic()
end

--- 恢复背景音乐播放
function T:resumeMusic()
	_engine:resumeMusic()
end

--- 是否正在播放背景音乐
function T:isMusicPlaying()
	return _engine:isMusicPlaying()
end

--- 获取背景音乐音量
function T:getMusicVolume()
	return _musicVol
end

--- 设置背景音乐音量
function T:setMusicVolume(volume)
	_musicVol=volume
	if _enabledPlayMusic then
		_engine:setMusicVolume(_musicVol)
	end
end

--- 直接设置背景音乐音量
function T:dsetMusicVolume(volume)
	if _enabledPlayMusic then
		_engine:setMusicVolume(volume)
	end
end

--- 预加载音效
-- @param self
-- @param url 音效地址
function T:preloadEffect(url)
	_engine:preloadEffect(url)
end

--- 卸载指定的音效
-- @param self
-- @param url 音效名
function T:unloadEffect(url)
	_engine:unloadEffect(url)
end

--- 播放指定音效
-- @param self
-- @param url 音效名
-- @param isLoop 是否循环播放
-- @return 音效句柄
function T:playEffect(url, isLoop)
	if not _enabledPlayEffect then
		return
	end

	if url then
		if url ~= "" then
			return _engine:playEffect(url, isLoop or false)
		end
	else
		local message="the effect url is \"nil\"".."\n"..debug.traceback()

		__print__("playSoundErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"playSoundErr")
	end
end

--- 停止指定的音效
-- @param self
-- @param handle 音效句柄
function T:stopEffect(handle)
	_engine:stopEffect(handle)
end

--- 停止所有音效
function T:stopAllEffects()
	_engine:stopAllEffects()
end

--- 暂停所有特效播放
function T:pauseAllEffects()
	_engine:pauseAllEffects()
end

--- 恢复所有音效播放
function T:resumeAllEffects()
	_engine:resumeAllEffects()
end

--- 暂停指定音效播放
-- @param self
-- @param handle 音效句柄
function T:pauseEffect(handle)
	_engine:pauseEffect(handle)
end

--- 恢复指定音效的播放
-- @param self
-- @param handle 音效句柄
function T:resumeEffect(handle)
	_engine:resumeEffect(handle)
end

--- 获取音效音量
function T:getEffectsVolume()
	return _engine:getEffectsVolume()
end

--- 设置音效音量
function T:setEffectsVolume(volume)
	_engine:setEffectsVolume(volume)
end

--- 是否播放音乐
function T:setEnabledMusic(enabled)
	if enabled ~= _enabledPlayMusic then
		_enabledPlayMusic=enabled

		if _enabledPlayMusic then
			_engine:setMusicVolume(_musicVol)
		else
			_engine:setMusicVolume(0)
		end
	end
end
--- 是否播放音乐
function T:getEnabledMusic()
	return _enabledPlayMusic
end

--- 是否播放音效
function T:setEnabledPlayEffect(enable)
	if enable ~= _enabledPlayEffect then
		_enabledPlayEffect=enable
	end
end
--- 是否播放音效
function T:getEnabledPlayEffect()
	return _enabledPlayEffect
end

--- 获取当前正在播放的音乐的url
function T:getCurrentMusicUrl()
	return _musicUrl
end

return T
