--- 导演
Director=cc.Director:getInstance()
--- 计时器
Scheduler=Director:getScheduler()

--- 随机数据生成器
local rd=Random:new()

local Cocos2dxUtils={}

--- 获取指定区间内的一个随机数
-- @param min 区间下限
-- @param max 区间上限
-- @return 随机数
function random(min,max)
	return rd:randomBetweenInt(min,max+1)
end

--- 获取指定区间内的一个随机数
-- @param min 区间下限
-- @param max 区间上限
-- @return 随机数
function randomFloat(min,max)
	return rd:randomBetweenFloat(min,max)
end

--- 根据名称,获取子对象(会递归,即当前对象没此名称的子对象的话,会从其子对象中进行查找,依此类推)
-- @param self
-- @param widget 父对象
-- @param name 子对象名称
-- @return 获取到的子对象
function Cocos2dxUtils:getChildByName(widget,name)
	local f=widget.getChildByName

	local child
	if f then
		child=widget:getChildByName(name)
	end
	if child then return child end

	local f2=widget.getChildren

	local children
	if f2 then
		children=widget:getChildren()
	end
	for k,v in ipairs(children) do
		child=Cocos2dxUtils:getChildByName(v,name)
		if child then
			return child
		end
	end

	return nil
end

--- 延迟执行
-- @param self
-- @param callback 回调函数
-- @param delaySec 延迟的秒数
-- @return 任务ID
function Cocos2dxUtils:timeOut(callback,delaySec)
	if callback == nil then
		print("callback == nil",debug.traceback())
		return
	end

	local sId
	local function tick()
		Scheduler:unscheduleScriptEntry(sId)

		callback()
	end
	sId=Scheduler:scheduleScriptFunc(tick,delaySec,false)

	return sId
end

--- 间隔调用
-- @param self
-- @param callback 回调函数
-- @param delaySec 延迟的秒数
-- @return 任务ID
function Cocos2dxUtils:interval(callback,delaySec)
	if callback == nil then
		print("callback == nil",debug.traceback())
		return
	end

	return Scheduler:scheduleScriptFunc(callback,delaySec,false)
end

--- 清除任务(间隔调用/延迟执行)
-- @param self
-- @param schedulerId 任务ID
function Cocos2dxUtils:clearScheduler(schedulerId)
	if schedulerId == nil then
		print(debug.traceback())
	end
	Scheduler:unscheduleScriptEntry(schedulerId)
end

--- 创建动画对象
-- @param frefix 动画名前缀
-- @param beginIndex 开始索引
-- @param endIndex 结束索引
-- @param extension 扩展名
-- @param delay 帧延迟(帧间间隔)
-- @param clone 复制帧 如:{[16]=5},表示在尾号为16的帧后面复制5帧插入
function Cocos2dxUtils:createAnimation(frefix,beginIndex,endIndex,extension,delay,clone)
	local spriteCache=cc.SpriteFrameCache:getInstance()
	local frames={}

	local insert=table.insert
	for i=beginIndex,endIndex,1 do
		local name=string.format("%s%02d%s",frefix,i,extension)

		local frame=spriteCache:getSpriteFrame(name)
		if not frame then
			print("create frame fail:frame name is \""..name.."\"")
			return nil
		end

		insert(frames,frame)

		-- 复制帧(某些帧需要重复播放)
		if clone and clone[i] then
			for j=1,clone[i],1 do
				insert(frames,frame:clone())
			end
		end
	end

	return cc.Animation:createWithSpriteFrames(frames,delay)
end

--- 偏移
-- @param srcPoint 原点
-- @param offscet 偏移量
function Cocos2dxUtils.offset(srcPoint,offset)
	return cc.p(srcPoint.x+offset.x,srcPoint.y+offset.y)
end

function Cocos2dxUtils.loadShader(vsh,fsh)
	local glP=cc.GLProgram:new()
	glP:initWithFilenames(vsh, fsh)

	glP:bindAttribLocation(cc.ATTRIBUTE_NAME_POSITION,cc.VERTEX_ATTRIB_POSITION)
	glP:bindAttribLocation(cc.ATTRIBUTE_NAME_COLOR,cc.VERTEX_ATTRIB_COLOR)
	glP:bindAttribLocation(cc.ATTRIBUTE_NAME_TEX_COORD,cc.VERTEX_ATTRIB_TEX_COORDS)
	glP:link()
	glP:updateUniforms()

	return glP;
end

--- 字符串转换为 color4b 对象
-- @param text 字符串
function Cocos2dxUtils.str2c4b(text)
	local t=type(text)
	if t == "table" then
		return text
	elseif t == "number" then
		return {
			r=BitOperator:_and(text,0xff000000),
			g=BitOperator:_and(text,0x00ff0000),
			b=BitOperator:_and(text,0x0000ff00),
			a=BitOperator:_and(text,0x000000ff)
		}
	elseif t == "string" then
		local s=string.gsub(text,"0x","")
		if string.len(s) < 8 then
			s=s.."0000000"
		end

		local alpha=tonumber("0x" .. string.sub(s, 7, 8))

		return {
			r=tonumber("0x" .. string.sub(s, 1, 2)),
			g=tonumber("0x" .. string.sub(s, 3, 4)),
			b=tonumber("0x" .. string.sub(s, 5, 6)),
			a=alpha and 255 or alpha
		}
	end
end

--- 字符串转换为 size
-- @param text 字符串
function Cocos2dxUtils.str2Size(text)
	if type(text) == "table" then
		return text
	end

	local s,e=string.find(text,",")

	return {
		width=tonumber(string.sub(text,1,s-1)),
		height=tonumber(string.sub(text,e+1))
	}
end

--- 设置子对象的透明度级联
function Cocos2dxUtils:setCascadeOpacityEnabled(node,enabled)
	if node then
		local children=node:getChildren()
		if children then
			for _,v in pairs(children) do
				Cocos2dxUtils:setCascadeOpacityEnabled(v,enabled)
			end
		end

		node:setCascadeOpacityEnabled(enabled)
		if node.getInnerContainer then
			node:getInnerContainer():setCascadeOpacityEnabled(enabled)
		end
	end
end

--- 相对定位的函数， 对应c++函数 Lua版
function Cocos2dxUtils:toOther(horizontal, vertical, t, other, offsetX, offsetY)
	if not t then return end
	if not other then return end

	local otherSize = other:getContentSize()
	local otherW = otherSize.width * other:getScaleX()
	local otherH = otherSize.height * other:getScaleY()

	local otherP = cc.p(0, 0)
	otherP.x, otherP.y = other:getPosition()
	local otherAnchor = other:getAnchorPoint()

	local otherRealX=otherP.x-(otherW*otherAnchor.x)
	local otherRealY=otherP.y-(otherH*otherAnchor.y)

	local wP = cc.p(otherRealX, otherRealY)
	local otherParent = other:getParent()

	if otherParent then
		wP = otherParent:convertToWorldSpace(wP)
	end

	self:toRect(horizontal,vertical,t,cc.rect(wP.x,wP.y,otherW,otherH),offsetX,offsetY)
end

function Cocos2dxUtils:toRect(horizontal, vertical, target, rect, offsetX, offsetY)
	if not target then return end

	local x=rect.x
	local y=rect.y
	local w=rect.width
	local h=rect.height

	local size=target:getContentSize()
	local realSize=cc.size(size.width*target:getScaleX(),size.height*target:getScaleY())
	local anchor=target:getAnchorPoint()

	local p = cc.p(0, 0)
	p.x, p.y=target:getPosition()

	if horizontal == "left" then --相对于屏幕左边距
		p.x=x+offsetX+(realSize.width*anchor.x);
	elseif horizontal == "center" then --相对屏幕的水平中间点
		p.x=x+(w-realSize.width)/2+offsetX+(realSize.width*anchor.x)
	elseif horizontal == "right" then --相对于屏幕右边距
		p.x=x+w-offsetX-(realSize.width*(1-anchor.x));
	end

	if vertical == "top" then
		p.y=y+h-offsetY-(realSize.height*(1-anchor.y));
	elseif vertical == "middle" then
		p.y=y+(h-realSize.height)/2+offsetY+(realSize.height*anchor.y)
	elseif vertical == "bottom" then
		p.y=y+offsetY+(realSize.height*anchor.y)
	end

	local parent = target:getParent()

	if parent then
		target:setPosition(parent:convertToNodeSpace(p))
	else
		target:setPosition(p)
	end
end

--- 获得长度
function Cocos2dxUtils:getArrayLength(array)
	local n = 0

	for k, v in pairs(array) do
		n = n + 1
	end

	return n
end

--- 设置文本的字体
-- @param t table or text
function Cocos2dxUtils:setTextFont(t)
	-- t.fontType 这一个变量需手动设置 不然就会保持其原有大小

	local calFontSize = function(t)
		if not t then  return end
		if not t.fontType then return end     -- 如果没有设置这一项就退出
		if type(t.fontType) == "string" then
			if t.fontType == "default" then -- 默认大小
				t:setFontSize(22)
			end
		elseif type(t.fontType) == "number" then
			t:setFontSize(t.fontType)
		end
	end

	if type(t) == "table" then
		for i = 1, #t do
			t[i]:setFontName(defaultCssStyle.fontName)
			calFontSize(t[i])
		end
	else
		t:setFontName(defaultCssStyle.fontName)
		calFontSize(t)
	end

end

--- 获得提示特效
function Cocos2dxUtils:getEffect(name)
	local s = cc.Sprite:create()
	local ani = ResLoader:getAnimation(name)
	if not ani then
		return
	end

	local ac = cc.RepeatForever:create(cc.Animate:create(ani))
	s:runAction(ac)
	return s
end

--- 获取一个表的value的数目
-- @param tb 表
function Cocos2dxUtils:getValueNum(tb)
	local num = 0
	if not tb or type(tb) ~= "table" then
		return num
	end

	for k, v in pairs(tb) do
		num = num + 1
	end

	return num
end

--- 创建一个通用文本
function Cocos2dxUtils:createText()
	local text = XMLLabel()
	text:setAlign("center","middle")
	text:setContentSize(cc.size(300, 36))
	return text
end

--- 修改 ccui 中的loadingbar 为原生
--- 默认为 从左边开始
function Cocos2dxUtils:fixedLoadingBar(bar, resName)
	local anchor = bar:getAnchorPoint()
	local parent = bar:getParent()
	local px, py = bar:getPosition()

	-- 移除原来的bar
	bar:removeFromParent()
	bar = nil

	local uiProgress = cc.ProgressTimer:create(cc.Sprite:createWithSpriteFrame(ResLoader:getSpriteFrame(resName)))
	uiProgress:setBarChangeRate(cc.p(1,0))--进度条动画方向，（1,0）表示横方向，（0,1）表示纵方向
	uiProgress:setType(cc.PROGRESS_TIMER_TYPE_BAR)--进度条类型
	uiProgress:setMidpoint(cc.p(0,1))--进度条的起始点，（0,y）表示最左边，（1,,y）表示最右边，（x,1）表示最上面，（x,0）表示最下面。
	uiProgress:setPercentage(0)--初始值

	uiProgress:setAnchorPoint(anchor)
	uiProgress:setPosition(px, py)
	parent:addChild(uiProgress)
	return uiProgress
end


return Cocos2dxUtils
