--- 树形列表
-- @author：lqh
-- _leafNodelist	叶子节点集合
-- _rootNodelist	根节点集合
local MUIListTree = {}
local RootNode

function MUIListTree.extend(scrollList)
	for k,v in pairs(MUIListTree) do 	
		if k ~= "new" then
			scrollList[k] = v 
		end
	end
	return scrollList
end

--- 从树形列表获取一个根节点(需要外部扩展根节点的显示内容)
function MUIListTree:getRootNode()				
	local rootNode = RootNode.new(self,self._duration or 0.3)
	self:pushBackCustomItem(rootNode)
	return rootNode
end

--- 通过根点对象获取一个叶子节点
function MUIListTree:getLeafNode(basenode)
	local leafNode = ccui.Layout:create()
	leafNode:setClippingEnabled(true)
	self:insertCustomItem(leafNode,self:getIndex(basenode) + 1)
	return leafNode
end

--- 设置动画持续时间
function MUIListTree:setDuration(value)
	self._duration = value
end


--- 根节点layout(用于树形列表)
RootNode = class("RootNode",function() 
	local basenode = ccui.Layout:create()
	basenode:setTouchEnabled(true)
	return basenode
end)
function RootNode:ctor(list,duration)
	self.effMillis=500
	self._targetlist = list
	self:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			self:dispachEvent()
		end
	end)
end

--- 隐藏叶子节点
function RootNode:close()
	self._state = "closing"

	local l=self._targetlist
	local leafNode = self._leafNode

	if self._effAct then
		AbstractUI.clearScheduler(leafNode,self._effAct)
		self._effAct = nil
	end

	--缩放
	local scaleHandler=function(t)
		local w,h,keepMillis=leafNode._w,leafNode._h,self.effMillis

		local startMillis=TimeUtils:getCurrentMillis()-((h-leafNode:getContentSize().height)/h)*keepMillis
		self._effAct=AbstractUI.interval(leafNode,function()
			local lossTime=TimeUtils:getCurrentMillis()-startMillis
			if lossTime > keepMillis then
				lossTime=keepMillis

				AbstractUI.clearScheduler(leafNode,self._effAct)
				self._effAct = nil

				l:removeItem(l:getIndex(leafNode))
				self._leafNode = nil

				self._state = "closed"
			end

			local timeScale=1-lossTime/keepMillis

			leafNode:setContentSize({width=w,height=w*timeScale})
			l:requestRefreshView()
		end,0)
	end

	--隐藏
	self._effAct=cc.Sequence:create(
		cc.FadeOut:create(0.1),
		cc.CallFunc:create(scaleHandler)
	)
	leafNode:runAction(self._effAct)
end

--- 显示叶子节点
function RootNode:open()
	self._state = "opening"

	local l=self._targetlist
	local ic=l:getInnerContainer()

	local w,h,startMillis,keepMillis=0,0,0,self.effMillis

	local leafNode=self._leafNode
	--如果没有叶子节点,则创建叶子节点
	if not leafNode then
		leafNode = l:getLeafNode(self)
		if self._mlistener then
			self._mlistener(leafNode)
		end

		local s=leafNode:getContentSize()
		leafNode._w,leafNode._h=s.width,s.height
		w,h=leafNode._w,leafNode._h

		leafNode:setContentSize({width=w,height=0})
		leafNode:setOpacity(0)
		self._leafNode = leafNode

		startMillis=TimeUtils:getCurrentMillis()
	else
		w,h=leafNode._w,leafNode._h

		startMillis=TimeUtils:getCurrentMillis()-(leafNode:getContentSize().height/h)*keepMillis
	end

	if self._effAct then
		AbstractUI.clearScheduler(leafNode,self._effAct)
		self._effAct = nil
	end

	local tpt=self:convertToWorldSpace(cc.p(0,self:getContentSize().height))
	local ltpt=l:convertToWorldSpace(cc.p(0,l:getContentSize().height))
	local sy=ic:getPositionY()
	local yDiff=-(tpt.y-ltpt.y)

	local movedY=0
	self._effAct=AbstractUI.interval(leafNode,function()
		local lossTime=TimeUtils:getCurrentMillis()-startMillis
		if lossTime > keepMillis then
			lossTime=keepMillis

			AbstractUI.clearScheduler(leafNode,self._effAct)
			self._effAct = nil

			--显示
			self._effAct=cc.Sequence:create(
				cc.FadeIn:create(0.1),
				cc.CallFunc:create(function()
					self._state = "opened"
				end)
			)
			leafNode:runAction(self._effAct)
		end

		local timeScale=lossTime/keepMillis

		leafNode:setContentSize({width=w,height=math.floor(h*timeScale)})

		--注意,改变条目高时,内部容器的y坐标也会发生改变
		local curY=math.floor(yDiff*timeScale)
		ic:setPositionY(ic:getPositionY()+curY-movedY)
		movedY=curY

		l:requestRefreshView()
	end,0)
end

function RootNode:dispachEvent()
	local state=self._state
	if state == "opening" or state == "opened" then
		self:close()
	else
		self:open()
	end
end

function RootNode:addListener(listener)
	self._mlistener = listener
end

---使用示例
--function test:createMUIListTree()
--    local uiTemplate=ccs.GUIReader:getInstance():widgetFromJsonFile("res/uiJson/listEffectTest.json")
--    self:addChild(uiTemplate)

--    local list=Cocos2dxUtils:getChildByName(uiTemplate,"ListView_1")
--    local list2=MUIListTree.extend(list)

--    local node1=list2:getRootNode();
--    node1:setContentSize(cc.size(500,40))
--    node1:addListener(
--      function(leafContainer)
--        local subItem=ccs.GUIReader:getInstance():widgetFromJsonFile("res/uiJson/loginScene/Region_1.json")
--        leafContainer:addChild(subItem)
        
--        leafContainer:setContentSize(cc.size(subItem:getContentSize().width,subItem:getContentSize().height))
--    end)

--    local item=ccs.GUIReader:getInstance():widgetFromJsonFile("res/uiJson/loginScene/Region_1.json")
--    item:setTouchEnabled(true)
--    item.node=node1
--    item:addTouchEventListener(function(t,e)
--        if e==ccui.TouchEventType.ended then
--            t.node:dispachEvent()
--        end
--    end)
--    node1:addChild(item)
    
--    local node1=list2:getRootNode();
--    node1:setContentSize(cc.size(500,40))
--    node1:addListener(function(leafContainer)
--        local subItem=ccs.GUIReader:getInstance():widgetFromJsonFile("res/uiJson/loginScene/Region_1.json")
--        leafContainer:addChild(subItem)
        
--        leafContainer:setContentSize(cc.size(subItem:getContentSize().width,subItem:getContentSize().height))
--    end)
--    local item=ccs.GUIReader:getInstance():widgetFromJsonFile("res/uiJson/loginScene/Region_1.json")
--    node1:addChild(item)
--end

return MUIListTree