local _print=print
local function print(...)
	_print("\t<ResLoader>: ",table.concat({...},"\t"))
end

--- 资源加载器
local T=Class.ResLoader(LuaEventDispatcher2)

--- 事件-添加加载项
T.EVT_TYPE_ADD_ITEM="evt_type_add_item"

--- 事件-移除加载项
T.EVT_TYPE_REMOVE_ITEM="evt_type_remove_item"

--- 事件-开始加载
T.EVT_TYPE_LOAD_START="evt_type_load_start"

--- 事件-完成加载
T.EVT_TYPE_LOAD_COMPLETE="evt_type_load_complete"

--- 事件-单项开始加载
T.EVT_TYPE_ITEM_LOAD_START="evt_type_item_load_start"

--- 事件-单项加载完成
T.EVT_TYPE_ITEM_LOAD_COMPLETE="evt_type_item_load_complete"

--- 导演
T.Director=cc.Director:getInstance()

--- 纹理缓存器
T.TextureCache=T.Director:getTextureCache()

--- 帧缓存器
T.SpriteFrameCache=cc.SpriteFrameCache:getInstance()

--- 动画缓存器
T.AnimationCache=cc.AnimationCache:getInstance()

function T:def()
	LuaEventDispatcher2.def(self)

	--- 需要加载的资源队列
	self._resQueue=Queue()

	--- 当前正在加载的项
	-- _loadingItem

	--- 惰性异步加载器\
	-- _lazyAsyncLoader

	--- 加载/创建开始的时间
	self.startTime=0

	--- 是否使用异步加载
--	if cc.PLATFORM_OS_ANDROID == cc.Application:getInstance():getTargetPlatform() then
		self.isAsyncLoad=false
--	else
--		self.isAsyncLoad=true
--	end
end

function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 加载资源
-- @param item 资源项
function T:loadRes(item)
	self._resQueue:push(item)

	self:luaDispatchEvent(self.EVT_TYPE_ADD_ITEM,item)

	self:_tryStartLoad()
end

--- 加载资源
-- @param items 资源项列表
function T:loadReses(items)
	local resQueue=self._resQueue
	for _,v in ipairs(items) do
		resQueue:push(v)

		self:luaDispatchEvent(self.EVT_TYPE_ADD_ITEM,v)
	end
	self:_tryStartLoad()
end

--- 同步加载资源
-- @param items 资源列表
function T:syncLoad(items)
	if not items then
		return
	end

	for _,item in pairs(items) do
		--单项加载开始
		local startTime=TimeUtils:getCurrentMillis()

		if type(item) == "string" then
			local url=item
			local ext=getExtension(url)

			if ext=="mp3" or ext=="wav" then
				self:_loadAudio(url)
			else
				self:_loadBitmap_sync(url)
			end
		else
			local type=item.type
			if type == "animation" or type == "animations" then
				self:_createAnimations(item)
			elseif type == "audio" then
				self:_loadAudio(item.url)
			else
				self:_loadBitmap_sync(item.url,item.fmt)
			end
		end

		--单项加载完成
		print("cost millis->",TimeUtils:getCurrentMillis()-startTime)
	end
end

--- 初始化惰性异步加载器
function T:initLazyAsyncLoader()
	--异步单项加载完成
	local completeHandler=function(url)
		self:_itemCompleteHandler()
	end

	local l=CommonResLoader:new()
	l:luaAddEventListener("itemLoaded",completeHandler)
	self._lazyAsyncLoader=l
end

--- 加载
function T:_load()
	local resQueue=self._resQueue
	if resQueue:isEmpty() then
		self._loadingItem=nil

		return
	end

	local item=resQueue:look()
	self._loadingItem=item

	--单项加载开始
	self:luaDispatchEvent(self.EVT_TYPE_ITEM_LOAD_START,item)
	self.startTime=TimeUtils:getCurrentMillis()

	if type(item) == "string" then
		local url=item
		local ext=getExtension(url)

		if ext=="mp3" or ext=="wav" then
			self:_loadAudio(url)
		else
			if self.isAsyncLoad then
				self:_loadBitmap_async(url)

				return
			else
				self:_loadBitmap_sync(url)
			end
		end
	else
		local type=item.type
		if type == "animation" or type == "animations" then
			self:_createAnimations(item)
		elseif type == "audio" then
			self:_loadAudio(item.url)
		else
			if self.isAsyncLoad then
				self:_loadBitmap_async(item.url,item.fmt)

				return
			else
				self:_loadBitmap_sync(item.url,item.fmt)
			end
		end
	end

	self:_itemCompleteHandler()
end

--- 单项加载完成
function T:_itemCompleteHandler(item)
	local resQueue=self._resQueue

	--单项加载完成
	print("cost millis->",TimeUtils:getCurrentMillis()-self.startTime)
	self:luaDispatchEvent(self.EVT_TYPE_ITEM_LOAD_COMPLETE,item)

	resQueue:pop()
	if resQueue:isEmpty() then
		--所有加载完成
		self:luaDispatchEvent(self.EVT_TYPE_LOAD_COMPLETE)
		self._loadingItem=nil
	else
		Cocos2dxUtils:timeOut(function()
			self:_load()
		end,0)
	end
end

--- 尝试开始加载(如果未在加载中就开始加载)
function T:_tryStartLoad()
	if not self._loadingItem then
		--所有加载开始
		self:luaDispatchEvent(self.EVT_TYPE_LOAD_START,self._resQueue:getLength())

		self:_load()
		return true
	end
	return false
end

--- 加载音频文件
-- @param url url地址
function T:_loadAudio(url)
	print("[audio]:"..url)
	AudioManager:preloadEffect(url)
end

--- 加载图像文件(同步)
-- @param url url地址
-- @param fmt 图像格式
function T:_loadBitmap_sync(url,fmt)
	print("[bitmap sync]"..url)

	local ext=getExtension(url)

	if not fmt then
		if ext=="jpg" then
			fmt=cc.TEXTURE2_D_PIXEL_FORMAT_RG_B888
		else
			fmt=cc.TEXTURE2_D_PIXEL_FORMAT_AUTO
		end
	end

	cc.Texture2D:setDefaultAlphaPixelFormat(fmt)--设置默认格式
	if ext=="plist" then
		self.SpriteFrameCache:addSpriteFrames(url)
	else
		self.TextureCache:addImage(url)
	end
	cc.Texture2D:setDefaultAlphaPixelFormat(cc.TEXTURE2_D_PIXEL_FORMAT_AUTO)--恢复为自动格式
end

--- 加载图像文件(异步)
-- @param url url地址
-- @param fmt 图像格式
function T:_loadBitmap_async(url,fmt)
	print("[bitmap async]"..url)

	local ext=getExtension(url)

	if not fmt then
		if ext=="jpg" then
			fmt=cc.TEXTURE2_D_PIXEL_FORMAT_RG_B888
		else
			fmt=cc.TEXTURE2_D_PIXEL_FORMAT_AUTO
		end
	end

	if not self._lazyAsyncLoader then
		self:initLazyAsyncLoader()
	end
	self._lazyAsyncLoader:loadRes(url,fmt)
end

--- 创建动画
function T:_createAnimation(item)
	print("[animation]",item.name)

	local animation=Cocos2dxUtils:createAnimation(item.url,item.beginIndex,item.endIndex,item.extension,item.delay,item.clone)
	if animation then
		self.AnimationCache:addAnimation(animation,item.name)
	else
		print("create animation fail,name is \""..item.name.."\"")
	end
end

--- 创建动画
function T:_createAnimations(data)
	local type=data.type
	if type == "animation" then
		self:_createAnimation(data)
	elseif type == "animations" then
		for _,v in pairs(data.list) do
			self:_createAnimation(v)
		end
	end
end

function T:unloadRes(item)
	if type(item) == "string" then
		local url=item
		local ext=getExtension(url)
		print("<unload>"..url)

		if ext=="plist" then
			self.SpriteFrameCache:removeSpriteFramesFromFile(url)

			local bmpUrl
			repeat
				bmpUrl=string.gsub(url,"plist","pvr")
				if cc.FileUtils:getInstance():isFileExist(bmpUrl) then
					break
				end

				bmpUrl=string.gsub(url,"plist","pvr.ccz")
				if cc.FileUtils:getInstance():isFileExist(bmpUrl) then
					break
				end

				bmpUrl=string.gsub(url,"plist","pkm")
				if cc.FileUtils:getInstance():isFileExist(bmpUrl) then
					break
				end

				bmpUrl=string.gsub(url,"plist","dds")
				if cc.FileUtils:getInstance():isFileExist(bmpUrl) then
					break
				end

				bmpUrl=string.gsub(url,"plist","png")
				if cc.FileUtils:getInstance():isFileExist(bmpUrl) then
					break
				end

				bmpUrl=string.gsub(url,"plist","jpg")
				if cc.FileUtils:getInstance():isFileExist(bmpUrl) then
					break
				end
			until true
			self.TextureCache:removeTextureForKey(bmpUrl)
		elseif ext=="png" or ext=="jpg" then
			self.TextureCache:removeTextureForKey(url)
		elseif ext=="mp3" or ext=="wav" then
			AudioManager:unloadEffect(url)
		end
	else
		local releaseAnimation=function(name)
			local animation=self.AnimationCache:getAnimation(name)
			self.AnimationCache:removeAnimation(name)
		end

		if item.type == "animation" then
			releaseAnimation(item.name)
		elseif item.type == "animations" then
			for _,v in pairs(item.list) do
				releaseAnimation(v.name)
			end
		elseif item.url then
			self:unloadRes(item.url)
		end
	end
end

--- 卸载列表中的资源(列表会被倒序执行)
-- @param items 资源列表
function T:unloadReses(items)
	for i=#items,1,-1 do
		self:unloadRes(items[i])
	end
end


--- 从缓存中获取动画
-- @param self
-- @param name 动画名称
-- @return 动画
function T:getAnimation(name)
	if not name then return end
	local animation=self.AnimationCache:getAnimation(name)
	if not animation then
		local message="get Animation frame Error,the res name:"..name

		__print__("ResErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"ResErr")
	end
	return animation
end

--- 创建一个动画Sprite
-- @param self
-- @param name 动画名称
-- @return 动画
function T:createAnimationSprite(name)
	local uiSp=cc.Sprite:create()

	local animation=self:getAnimation(name)
	if animation then
		uiSp:runAction(
			cc.RepeatForever:create(cc.Animate:create(animation))
		)
	end

	return uiSp
end

--- 创建一个只运行一次的动画Sprite
-- @param self
-- @param name 动画名称
-- @return 动画
function T:createAnimationSpriteRunOnce(name)
	local uiSp=cc.Sprite:create()

	local animation=self:getAnimation(name)
	if animation then
		uiSp:runAction(cc.Sequence:create(
			cc.Animate:create(animation),
			cc.CallFunc:create(function(t)
				uiSp:removeFromParent()
			end)
		))
	end

	return uiSp
end

--- 创建一个随机开始帧的动画 Sprite
-- @param self
-- @param name 动画名称
-- @param isRandomDuration 是否随机播放间隔(1/4总持续时间内偏移)
-- @return 动画
function T:createAnimationSpriteRandom(name,isRandomDuration)
	local uiSp=cc.Sprite:create()

	local animation=self:getAnimation(name)
	if animation then
		local dur=animation:getDuration()

		--一个周期的播放持续时间,(1/4之内偏移)
		local animate=cc.Animate:create(animation)
		if isRandomDuration then
			local offset=randomFloat(0,dur/4)
			animate:setDuration(dur+offset-offset/2)
		end

		uiSp:runAction(cc.RepeatForever:create(animate))

		--播放的开始时间,在整个持续时间内随机
		CppCocos2dxUtils:setActionInterValElapsed(animate,randomFloat(0,dur));
	end

	return uiSp
end

--- 创建一个随机开始帧的动画 Sprite
-- @param self
-- @param name 动画名称
-- @param isRandStart 是否从随机位置开始播放
-- @param frameIntervalOffsetPercentage 帧间间隔随机的百分比,0表示不调整帧间隔
-- @param callback 播放完成的回调
-- @return 动画 Sprite
function T:createAnimationSpriteRand(name,isRandStart,frameIntervalOffsetPercentage,callback)
	return self:playAnimationWithRand(cc.Sprite:create(),name,isRandStart,frameIntervalOffsetPercentage,callback)
end

--- 播放随机动画
-- @param self
-- @param Sprite 对象
-- @param name 动画名称
-- @param isRandStart 是否从随机位置开始播放
-- @param frameIntervalOffsetPercentage 帧间间隔随机的百分比,0表示不调整帧间隔
-- @param callback 播放完成的回调
-- @return 动画 Sprite
function T:playAnimationWithRand(uiSprite,name,isRandStart,frameIntervalOffsetPercentage,callback)
	local animation=self:getAnimation(name)
	if animation then
		local dur=animation:getDuration()

		--一个周期的播放持续时间,(1/4之内偏移)
		local animate=cc.Animate:create(animation)
		if frameIntervalOffsetPercentage ~= 0 then
			local offset=randomFloat(0,dur/frameIntervalOffsetPercentage)
			animate:setDuration(dur+offset-offset/2)
		end

		if callback then
			uiSprite:runAction(cc.Sequence:create(animate,cc.CallFunc:create(function(t)
				callback(uiSprite,name,isRandStart,frameIntervalOffsetPercentage)
			end)))
		else
			uiSprite:runAction(cc.RepeatForever:create(animate))
		end

		if isRandStart then
			--播放的开始时间,在整个持续时间内随机
			CppCocos2dxUtils:setActionInterValElapsed(animate,randomFloat(0,dur));
		end
	end

	return uiSprite
end

--- 从缓存中获取指定名称的精灵帧
-- @param self
-- @param name 精灵帧名称
-- @return 精灵帧
function T:getSpriteFrame(name)
	local f=self.SpriteFrameCache:getSpriteFrame(name)
	if not f then
		local message="get Sprite frame Error,the res name:"..name

		__print__("ResErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"ResErr")
	end

	return f
end

--- 从缓存中获取指定名称的纹理
-- @param self
-- @param name 纹理名称
-- @return 纹理
function T:getTexture(url)
	local t=cc.TextureCache:getTextureForKey(url)
	if not t then
		local message="get Texture Error,the res name:"..url

		__print__("ResErr",message)
		LogRecorder:push(message,LOG_LEVEL_ERROR,"ResErr")
	end

	return t
end

--- 剩余未加载的资源项数
function T:getRemaining()
	return self._resQueue:getLength()
end

--- 根据特效类型得到特效，1：蓝色，2：紫色，默认蓝色
function T:getIconEffByType(type)
	if type == 1 then
		return "iconEff_blue"
	elseif type == 2 then
		return "iconEff_purple"
	else
		return "iconEff_blue"
	end

end

return T
