local _print=print
local function print(...)
	_print("<ResUpdate>: ",table.concat({...},"\t"))
end

local T=Class.ResUpdate(LuaEventDispatcher2)

--- 错误码-创建文件错误
T.ERR_CODE_CREATE_FILE=0
--- 错误码-网络错误
T.ERR_CODE_NETWORK=1
--- 错误码-没有新的版本号
T.ERR_CODE_NO_NEW_VERSION=2
--- 错误码-解压文件错误
T.ERR_CODE_UNCOMPRESS=3

--- 事件-开始更新lua代码和资源
T.EVT_TYPE_UPDATE_START="evt_type_update_start"

--- 事件-更新lua代码和资源过程
T.EVT_TYPE_UPDATE_PROGRESS="evt_type_update_progress"

--- 事件-更新lua代码和资源出错
T.EVT_TYPE_UPDATE_ERROR="evt_type_update_error"

--- 事件-更新单项lua代码和资源开始
T.EVT_TYPE_UPDATE_ITEM_START="evt_type_update_item_start"

--- 事件-更新单项lua代码和资源完成
T.EVT_TYPE_UPDATE_ITEM_COMPLETE="evt_type_update_item_complete"

--- 事件-更新lua代码和资源完成
T.EVT_TYPE_UPDATE_COMPLETE="evt_type_update_complete"

--- 定义
function T:def()
	LuaEventDispatcher2.def(self)

	--- 更新列表队列
	self.updateQueue=Queue()
end

--- 构造函数
function T:ctor()
	LuaEventDispatcher2.ctor(self)
end

--- 更新
function T:update()
	local uQueue=self.updateQueue

	local curUpdateItem=uQueue:look()
	if not curUpdateItem then
		return
	end

	--单项更新开始
	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_ITEM_START,curUpdateItem)

	local assetsMgr
	assetsMgr = self:_getAssetsMgr(curUpdateItem.url,CFG:getUpdateDir(),
		function()
			assetsMgr:release()
			assetsMgr = nil

			--存储更新后的版本号
			local updateCfg=CFG:getUpdateCfg()
			updateCfg.updatedVersion=curUpdateItem.sversion or "0.0.0"
			CFG:putUpdateCfg(updateCfg)

			--重新require新的文件
			xpcall(
				function()
					require("src.ReExcute")
					unrequire("src.ReExcute")
				end,
				function(errStr)
					local message=tostring(errStr).."\n"..debug.traceback()

					__print__("ResUpdateErr",message)
					LogRecorder:push(message,LOG_LEVEL_ERROR,"ResUpdateErr")
				end
			)

			--单项更新完成
			--移除当前更新项
			self:luaDispatchEvent(self.EVT_TYPE_UPDATE_ITEM_COMPLETE,uQueue:pop())

			if uQueue:isEmpty() then
				--全部更新完成
				self:luaDispatchEvent(self.EVT_TYPE_UPDATE_COMPLETE)
			end

			self:update()
		end,
		function(percent)
			if percent < 0 then
				percent=0
			elseif percent > 100 then
				percent=100
			end

			--更新过程
			self:luaDispatchEvent(self.EVT_TYPE_UPDATE_PROGRESS,percent)
		end,
		function(errCode)
			assetsMgr:release()
			assetsMgr = nil

			--更新出错
			self:luaDispatchEvent(self.EVT_TYPE_UPDATE_ERROR,errCode)
		end)
	assetsMgr:retain()
	assetsMgr:update()
end

--- 执行更新
-- @param updateList 更新列表(url列表)
function T:execUpdate(updateList)
	if not updateList or #updateList == 0 then
		return
	end

	local uQueue=self.updateQueue
	for k,v in ipairs(updateList) do
		uQueue:push(v)
	end

	self:luaDispatchEvent(self.EVT_TYPE_UPDATE_START)
	self:update()
end

--- 获取更新队列
function T:getUpdateQueue()
	return self.updateQueue
end

--- 获取当前正在更新的项
function T:getCurrentUpdateItem()
	return self.updateQueue:look()
end

--- 获取更新器
-- @param url 更新地址
-- @param savePath 保存更新文件的目录
-- @param onSuccess 更新完成的回调
-- @param onProgress 更新过程的回调
-- @param onError 更新错误的回调
function T:_getAssetsMgr(url,savePath,onSuccess,onProgress,onError)
	local assetsMgr = cc.AssetsManager:new(url,"placeholder",savePath)
	assetsMgr:setDelegate(onError, cc.ASSETSMANAGER_PROTOCOL_ERROR )
	assetsMgr:setDelegate(onProgress, cc.ASSETSMANAGER_PROTOCOL_PROGRESS)
	assetsMgr:setDelegate(onSuccess, cc.ASSETSMANAGER_PROTOCOL_SUCCESS )
	assetsMgr:setConnectionTimeout(3)

	return assetsMgr
end

--- 比较版本(如果 version1 版本号大于 version2 则返回正数,否则为负数,相等则返回为 0,返回的数值表示比较到第 n 级时不一致)
-- @param version1 版本号1
-- @param version2 版本号1
-- @return 如果 version1 版本号大于 version2 则返回正数,否则为负数,相等则返回为 0,返回的数值表示比较到第 n 级时不一致
function T:compareVersion(version1,version2)
	version1,version2=version1 or "0.0.0",version2 or "0.0.0"

	local ver1Arr,ver2Arr=StringUtils.split(version1,"."),StringUtils.split(version2,".")

	local l=math.max(#ver1Arr,#ver2Arr)
	for i=1,l,1 do
		local d=(tonumber(ver1Arr[i]) or 0) - (tonumber(ver2Arr[i]) or 0)

		if d > 0 then
			return i
		elseif d < 0 then
			return -i
		end
	end

	return 0
end

return T
