--- 过度场景
local excessiveScene=nil

--- 场景管理器
local SceneManager={}

--- 当前场景数据
SceneManager.sceneDataStack=Stack()

--- 切换场景
-- @param sceneData 场景数据
-- @param callback 切换完成的回调
-- @param args 参数列表
function SceneManager:switch(sceneData,callback,args)
	local function _switch()
		WindowManager.currentMgr=WindowManager()

		local scene=self:_createScene(sceneData.url,args)

		if Director:getRunningScene() then
			Director:replaceScene(scene)
		else
			Director:runWithScene(scene)
		end

		self.sceneDataStack:push(sceneData)

		if callback then
			callback(scene)
		end
	end

	--先切换到资源加载场景
	if sceneData.cfgUrl then
		if Director:getRunningScene() then
			if excessiveScene and excessiveScene.url then
				Director:replaceScene(require(excessiveScene.url).new())
			else
				Director:replaceScene(cc.Scene:create())
			end
		else
			if excessiveScene and excessiveScene.url then
				Director:runWithScene(require(excessiveScene.url).new())
			else
				Director:runWithScene(cc.Scene:create())
			end
		end

		self:unloadLastSceneRes()

		Cocos2dxUtils:timeOut(
			function()
				--加载新场景需要的资源
				local cfgUrl=sceneData.cfgUrl
				if cfgUrl then
					local cfg=require(cfgUrl)
					if cfg.preResList then
						local function loadCompleteHandler()
							ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,nil,loadCompleteHandler)

							_switch()
						end

						ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,nil,loadCompleteHandler)

						ResLoader:loadReses(cfg.preResList)
					end
				else
					_switch()
				end


			end, 0
		)
	else
		self:unloadLastSceneRes()

		_switch()
	end
end

function SceneManager:push(sceneData,callback,args)
	local function _push()
		local scene=self:_createScene(sceneData.url,args)
		Director:pushScene(scene)

		self.sceneDataStack:push(sceneData)
		if callback then
			callback(scene)
		end
	end

	--加载新场景需要的资源
	local cfgUrl=sceneData.cfgUrl
	if cfgUrl then
		local cfg=require(cfgUrl)
		if cfg.preResList then
			local function loadCompleteHandler()
				ResLoader:luaRemoveEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,nil,loadCompleteHandler)

				_push()
			end

			ResLoader:luaAddEventListener(ResLoader.EVT_TYPE_LOAD_COMPLETE,nil,loadCompleteHandler)

			ResLoader:loadReses(cfg.preResList)
		end
	else
		_push()
	end
end

function SceneManager:pop()
	Director:popScene()

	self:unloadLastSceneRes()
end

---释放上一场景的资源
function SceneManager:unloadLastSceneRes()
	local sceneData=self.sceneDataStack:pop()

	if sceneData then
		local cfgUrl=sceneData.cfgUrl
		if cfgUrl then
			local cfg=require(cfgUrl)
			if cfg.preResList then
				ResLoader:unloadReses(cfg.preResList)
			end
		end
	end
end

function SceneManager:_createScene(url,args)
	local sceneTemplate=require(url)

	local scene
	if args then
		scene=sceneTemplate.new(args)
	else
		scene=sceneTemplate.new()
	end

	return scene
end

return SceneManager
