--- 窗口管理器
-- @author blank
local T=Class.WindowManager(LuaEventDispatcher2)

--- 当前窗口管理器
T.currentMgr=nil

function T:def()
	LuaEventDispatcher2.def(self)

	self.currentMgr=self

	--- 当前场景数据
	self.windowStack=Stack()
end

--- 切换窗口
-- @param window 窗口实例
-- @param isShowEffect 需要显示特效
function T:_switch(window,isShowEffect)
	if isShowEffect == nil then
		isShowEffect=true
	end

	self:luaDispatchEvent(EVENT_START_OPEN,window)

	local lWindow=self.windowStack:pop()
	if lWindow then
		if BGMgr:isGuide() then
			BGMgr:fire(BGMgr.EVT_WINDOW_CLOSE_PREV,lWindow.__cname)
		end

		lWindow:removeFromParent()
	end

	Director:getRunningScene():addChild(window)
	self.windowStack:push(window)

	local function openCompleteHandler(window)
		window:luaRemoveEventListener(EVENT_OPEN_COMPLETE, nil, openCompleteHandler)

		self:luaDispatchEvent(EVENT_OPEN_COMPLETE,window)
	end

	if isShowEffect and not window:getIsOpenComplete() then
		window:luaAddEventListener(EVENT_OPEN_COMPLETE, nil, openCompleteHandler)
	else
		openCompleteHandler(window)
	end

	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_WINDOW_OPEN_AFTER,window.__cname)
	end
end


--- 压入窗口
-- @param window 窗口实例
-- @param isShowEffect 需要显示特效
function T:_push(window,isShowEffect)
	if isShowEffect == nil then
		isShowEffect=true
	end

	self:luaDispatchEvent(EVENT_START_OPEN,window)

	Director:getRunningScene():addChild(window)
	self.windowStack:push(window)

	local function openCompleteHandler(window)
		window:luaRemoveEventListener(EVENT_OPEN_COMPLETE, nil, openCompleteHandler)

		local lWindow=self.windowStack:look(1)
		if lWindow then
			lWindow:setVisible(false)
			--			lWindow:retain()
			--			lWindow:removeFromParent()

			lWindow:stopAllActions()
		end

		self:luaDispatchEvent(EVENT_OPEN_COMPLETE,window)
	end

	if isShowEffect and not window:getIsOpenComplete() then
		window:luaAddEventListener(EVENT_OPEN_COMPLETE, nil, openCompleteHandler)
	else
		openCompleteHandler(window)
	end
	
	if BGMgr:isGuide() then
		BGMgr:fire(BGMgr.EVT_WINDOW_OPEN_AFTER,window.__cname)
	end
end

--- 回退窗口
-- @param isShowEffect 需要显示特效
function T:_pop(isShowEffect)
	if isShowEffect == nil then
		isShowEffect=true
	end

	if not self.windowStack:isEmpty() then
		if isShowEffect then
			self:luaDispatchEvent(EVENT_START_CLOSE,self.windowStack:look())
		end

		local lFirstWindow=self.windowStack:pop()

		if BGMgr:isGuide() then
			BGMgr:fire(BGMgr.EVT_WINDOW_CLOSE_PREV,lFirstWindow.__cname)
		end

		if isShowEffect and lFirstWindow.playColseEffect then
			local function closeCompleteHandler(window)
				window:luaRemoveEventListener(EVENT_CLOSE_COMPLETE, nil, closeCompleteHandler)

				window:removeFromParent()

				self:luaDispatchEvent(EVENT_CLOSE_COMPLETE,window)

			end

			lFirstWindow:luaAddEventListener(EVENT_CLOSE_COMPLETE, nil, closeCompleteHandler)
			lFirstWindow:playColseEffect()
		else
			lFirstWindow:removeFromParent()
		end

		local lSecondWindow=self.windowStack:look()
		if lSecondWindow then
			lSecondWindow:setVisible(true)
			--			Director:getRunningScene():addChild(lSecondWindow)
			--			lSecondWindow:release()
		end
	end
end

--- 一直回退到指定的窗口
-- @param windowName 需要回退到的窗口名
function T:_popUtil(windowName)

end

--- 清理所有窗口
function T:_clear()
	while not self.windowStack:isEmpty() do
		local window=self.windowStack:pop()
		window:removeFromParent()
	end
end

function T:_release()
	self:_clear()
	self:releaseEventListeners()
end

--- 获取当前窗口
function T:_getCurrentWindow(i)
	return self.windowStack:look(i)
end

--- 获取当前场景的窗口数目
function T:_getLength()
	return self.windowStack:getLength()
end

function T:getCurrentSceneMgr()
	return self.currentMgr
end

function T:switch(window,isShowEffect)
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		mgr:_switch(window,isShowEffect)
	end
end

function T:push(window,isShowEffect)
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		mgr:_push(window,isShowEffect)
	end
end

function T:pop(isShowEffect)
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		mgr:_pop(isShowEffect)
	end
end

function T:popUtil()
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		mgr:_popUtil()
	end
end

function T:clear()
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		mgr:_clear()
	end
end

function T:release()
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		mgr:_release()
	end
end

function T:getCurrentWindow(i)
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		return mgr:_getCurrentWindow(i)
	end

	return nil
end

function T:getLength()
	local mgr=self:getCurrentSceneMgr()
	if mgr then
		return mgr:_getLength()
	end

	return 0
end

return T
