local TxtButton = Class.TxtButton(
	function() return ccui.Layout:create() end,
	LuaEventDispatcher2
)

function TxtButton:def()
	LuaEventDispatcher2.def(self)
	--- 文本显示框
	-- self.uiText=nil

	--- 是否已被选择
	self._isSelected = false
end

function TxtButton:ctor(text)
	LuaEventDispatcher2.ctor(self)

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
			elseif eventType == CocosEvent.EXIT then
			elseif eventType == CocosEvent.CLEAN_UP then
				self:releaseEventListeners()
			end
		end
	)

	self:init(text)
end

function TxtButton:init(text)
	self:setContentSize(cc.size(96, 48))

	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	self:setBackGroundColor(cc.c3b(0xcc, 0xcc, 0xcc))
	self:setBackGroundColorOpacity(180)

	if text then
		local uiText = XMLLabel()
		uiText:setHAlign("center")
		uiText:setVAlign("middle")
		uiText:setString(text)
		self:addChild(uiText)

		REL:relOther(uiText, self, 2, 2, 2, 2)

		self.uiText = uiText
	end

	self._addTouchEventListener = self.addTouchEventListener
	function self:addTouchEventListener(callback)
		self:luaAddEventListener("__touch_trigger", nil, callback)
	end

	self:setTouchEnabled(true)
	self:_addTouchEventListener(
		function(t, e)
			if e == ccui.TouchEventType.began then
				t:runAction(cc.Blink:create(0.2, 2))
			end

			t:luaDispatchEvent("__touch_trigger", t, e)
		end
	)
end

--- 设置是否已被选择
function TxtButton:setIsSelected(value)
	if value ~= self._isSelected then
		if self._isSelected then
			self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
			self:setBackGroundColor(cc.c3b(0xcc, 0xcc, 0xcc))
			self:setBackGroundColorOpacity(180)
		else
			self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
			self:setBackGroundColor(cc.c3b(0x88, 0x88, 0x88))
			self:setBackGroundColorOpacity(180)
		end
		self._isSelected = value
	end
end
--- 获取是否已被选择
function TxtButton:getIsSelected()
	return self._isSelected
end

--- 日志记录查看器
local T = Class.LogRecordLooker(
	function() return ccui.Layout:create() end,
	LuaEventDispatcher2
)

--- 默认显示条目数
T.DEFAULT_SHOW_ITEM_COUNT = 10

function T:def()
	LuaEventDispatcher2.def(self)

	--- 对象池栈
	self.POOL_STACK = Stack()

	--- 日志记录器
	-- self._recorder=nil

	--- "关闭"按钮
	-- self.uiCloseButton=nil

	--- "致命"按钮
	-- self.uiFatalButton=nil

	--- "错误"按钮
	-- self.uiErrorButton=nil

	--- "警告"按钮
	-- self.uiWarningButton=nil

	--- "调试"按钮
	-- self.uiDebugButton=nil

	--- "信息"按钮
	-- self.uiInfoButton=nil

	--- "内存"按钮
	-- self.uiMemButton=nil

	--- 文本项显示滚动容器
	-- self.uiItemsScrollContainer=nil

	--- 文本项显示滚动容器的内部容器的当前Y坐标(用于判断是向上滚动还是向下滚动)
	-- self.curtxtItemsInnerCY=nil

	--- 当前按钮
	-- self.curButton=nil

	--- 显示记录的开始索引
	self.recorderStartIndex = 1

	--- 显示记录的结束索引
	self.recorderEndIndex = 1

	--- 显示文本项的容器
	-- self.itemsContainer = nil

	--- 文本项列表
	self.itemsList = { }
end

function T:ctor()
	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:init()
end

--- 进入
function T:onEnter()
	self:luaDispatchEvent("open",self)
end

--- 退出
function T:onExit()
	local s=self.POOL_STACK
	while not s:isEmpty() do
		local item=s:pop()
		item:release()
	end

	self:rmRecorderEvtListeners()
	self:releaseEventListeners()
end

--- 内存清理
function T:cleanUp()

end

--- 初始化
function T:init()
	REL:relScreen(self, 0, 0, 0, 480)
	self:setTouchEnabled(true)
	self:setLocalZOrder(99998)

	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	self:setBackGroundColor(cc.c3b(0xff, 0xff, 0xff))
	self:setBackGroundColorOpacity(100)

	local right = 4

	-- "关闭"按钮
	--	local uiCloseButton = TxtButton("<span color='0xff0000'>close</span>")
	--	self:addChild(uiCloseButton)
	--	REL:toOther("right", "top", uiCloseButton, self, right, 4)
	--	self.uiCloseButton = uiCloseButton
	--	right = right + uiCloseButton:getContentSize().width + 4
	--	uiCloseButton:addTouchEventListener(
	--		function(t, e)
	--			if e == ccui.TouchEventType.ended then
	--				self:luaDispatchEvent("close",self)
	--				self:remvoeFromParent()
	--			end
	--		end
	--	)

	local function lvButtonTouchHandler(t, e)
		if e == ccui.TouchEventType.ended then
			self:setCurrentButton(t)

			if t.level == LOG_LEVEL_FATAL then
				cc.Director:getInstance():purgeCachedData()
			end

			if t.level == LOG_LEVEL_WARNING then
				--清除唯一KEY
				print("清除唯一KEY")
				DataStorage:putUniqueId("")
			end
		end
	end

	-- "内存"按钮
	local uiMemButton = TxtButton("<span color='0xffffff'>mem</span>")
	self:addChild(uiMemButton)
	REL:toOther("right", "top", uiMemButton, self, right, 4)
	self.uiMemButton = uiMemButton
	right = right + uiMemButton:getContentSize().width + 4
	uiMemButton:addTouchEventListener(function(t,e)
		if e == ccui.TouchEventType.ended then
			print(string.format("LUA VM MEMORY USED: %0.2f KB", collectgarbage("count")))
			print(cc.Director:getInstance():getTextureCache():getCachedTextureInfo())
		end
	end)

	-- "致命"按钮
	local uiFatalButton = TxtButton("<span color='0xff0000'>fatal</span>")
	uiFatalButton.level = LOG_LEVEL_FATAL
	self:addChild(uiFatalButton)
	REL:toOther("right", "top", uiFatalButton, self, right, 4)
	self.uiFatalButton = uiFatalButton
	right = right + uiFatalButton:getContentSize().width + 4
	uiFatalButton:addTouchEventListener(lvButtonTouchHandler)

	-- "错误"按钮
	local uiErrorButton = TxtButton("<span color='0xff0022'>error</span>")
	uiErrorButton.level = LOG_LEVEL_ERROR
	self:addChild(uiErrorButton)
	REL:toOther("right", "top", uiErrorButton, self, right, 4)
	self.uiErrorButton = uiErrorButton
	right = right + uiErrorButton:getContentSize().width + 4
	uiErrorButton:addTouchEventListener(lvButtonTouchHandler)

	-- "警告"按钮
	local uiWarningButton = TxtButton("<span color='0xffff00'>waining</span>")
	uiWarningButton.level = LOG_LEVEL_WARNING
	self:addChild(uiWarningButton)
	REL:toOther("right", "top", uiWarningButton, self, right, 4)
	self.uiWarningButton = uiWarningButton
	right = right + uiWarningButton:getContentSize().width + 4
	uiWarningButton:addTouchEventListener(lvButtonTouchHandler)

	-- "调试"按钮
	local uiDebugButton = TxtButton("<span color='0x0000ff'>debug</span>")
	uiDebugButton.level = LOG_LEVEL_DEBUG
	self:addChild(uiDebugButton)
	REL:toOther("right", "top", uiDebugButton, self, right, 4)
	self.uiDebugButton = uiDebugButton
	right = right + uiDebugButton:getContentSize().width + 4
	uiDebugButton:addTouchEventListener(lvButtonTouchHandler)

	-- "信息"按钮
	local uiInfoButton = TxtButton("<span color='0x0000ff'>info</span>")
	uiInfoButton.level = LOG_LEVEL_INFO
	self:addChild(uiInfoButton)
	REL:toOther("right", "top", uiInfoButton, self, right, 4)
	self.uiInfoButton = uiInfoButton
	right = right + uiInfoButton:getContentSize().width + 4
	uiInfoButton:addTouchEventListener(lvButtonTouchHandler)

	-- 文本显示区域
	local uiItemsScrollContainer = ccui.ScrollView:create()
	uiItemsScrollContainer:setTouchEnabled(true)
	uiItemsScrollContainer:setBounceEnabled(true)
	self:addChild(uiItemsScrollContainer)
	uiItemsScrollContainer:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	uiItemsScrollContainer:setBackGroundColor(cc.c3b(0x00, 0x00, 0x00))
	uiItemsScrollContainer:setBackGroundColorOpacity(100)
	REL:relOther(uiItemsScrollContainer, self, 4, 4, 52, 4)
	self.uiItemsScrollContainer = uiItemsScrollContainer
	self.curtxtItemsInnerCY = uiItemsScrollContainer:getInnerContainer():getPositionY()

	ccui.ScrollView.addEventListener(uiItemsScrollContainer,function(t,e)
		if e ~= ccui.ScrollviewEventType.scrolling then
			return
		end

		local ic = t:getInnerContainer()
		local y = ic:getPositionY()
		if y > self.curtxtItemsInnerCY then
			local wP = t:convertToWorldSpace(cc.p(0, t:getPositionY()))
			local b, t = wP.y, wP.y + t:getContentSize().height

			-- 向上移动
			self:toUpCalac(t, b)
			self.curtxtItemsInnerCY = ic:getPositionY()
		elseif y < self.curtxtItemsInnerCY then
			local wP = t:convertToWorldSpace(cc.p(0, t:getPositionY()))
			local b, t = wP.y, wP.y + t:getContentSize().height

			-- 向下移动
			self:toDnCalac(t, b)
			self.curtxtItemsInnerCY = ic:getPositionY()
		end
	end )

	local itemsContainer = cc.Node:create()
	uiItemsScrollContainer:addChild(itemsContainer)
	REL:toOther("center", "top", itemsContainer, uiItemsScrollContainer, 0, 0)
	self.itemsContainer = itemsContainer

	-- 默认显示"错误"信息
	self:setCurrentButton(uiErrorButton)
end

function T:toUpCalac(worldTLimit, worldBLimit, buffLimit)
	buffLimit=buffLimit or 300

	local items = self.itemsList

	local t = worldTLimit + buffLimit
	local b = worldBLimit - buffLimit

	-- 计算顶部元素是否应该被移除
	while true do
		local item = items[1]
		if not item then
			break
		end

		local wP = item:convertToWorldSpace(cc.p(0, 0))

		if wP.y <= t then
			break
		end

		self:_removeItem_t()
	end

	-- 计算底部是否应该添加元素
	while true do
		local item = items[#items]
		if not item then
			break
		end

		local wP = item:convertToWorldSpace(cc.p(0, item:getContentSize().height))

		if wP.y < b then
			break
		end

		-- 添加新元素
		if not self:_appendItem_b() then
			break
		end
	end
end

function T:toDnCalac(worldTLimit, worldBLimit, buffLimit)
	buffLimit=buffLimit or 300

	local items = self.itemsList

	local t = worldTLimit + buffLimit
	local b = worldBLimit - buffLimit

	-- 计算底部元素是否应该被移除
	while true do
		local item = items[#items]
		if not item then
			break
		end

		local wP = item:convertToWorldSpace(cc.p(0, item:getContentSize().height))

		if wP.y > b then
			break
		end

		self:_removeItem_b()
	end

	-- 计算顶部是否应该添加元素
	while true do
		local item = items[1]
		if not item then
			break
		end

		local wP = item:convertToWorldSpace(cc.p(0, 0))

		if wP.y > t then
			break
		end

		-- 添加新元素
		if not self:_appendItem_t() then
			break
		end
	end
end

function T:_removeItem_t()
	local s, e = self.recorderStartIndex, self.recorderEndIndex
	self.recorderStartIndex = s + 1

	local items = self.itemsList

	items[1]:removeFromParent()
	local l = table.remove(items, 1)

	self:calcHeight()
end

function T:_removeItem_b()
	local s, e = self.recorderStartIndex, self.recorderEndIndex
	self.recorderEndIndex = e - 1

	local items = self.itemsList
	items[#items]:removeFromParent()
	table.remove(items)

	self:calcHeight()
end

function T:_appendItem_t()
	local r = self._recorder
	if not r then
		return false
	end

	while true do
		local s = self.recorderStartIndex - 1
		if s < 1 then
			return false
		end

		if s <= r:getLength() then
			self.recorderStartIndex = s

			local item = r:look(s)
			if item.level >= self.curButton.level then
				self:appendItem_t(item)

				return true
			end
		end
	end

	return false
end

function T:_appendItem_b()
	local r = self._recorder
	if not r then
		return false
	end

	while true do
		local e = self.recorderEndIndex + 1
		if e > r:getLength() then
			return false
		end

		if e <= r:getLength() then
			self.recorderEndIndex = e

			local item = r:look(e)
			if item.level >= self.curButton.level then
				self:appendItem_b(item)

				return true
			end
		end
	end

	return false
end

--- 设置当前按钮
function T:setCurrentButton(button)
	if button == self.curButton or not button then
		return
	end
	if self.curButton then
		self.curButton:setIsSelected(false)
	end
	self.curButton = button
	button:setIsSelected(true)
	self:refreshCurLv()
end

function T:_createTxtItem(item)
	local uiTxtItem = ccui.Layout:create()
	uiTxtItem:retain()
	uiTxtItem:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then

			elseif eventType == CocosEvent.EXIT then
				uiTxtItem:retain()
				self.POOL_STACK:push(uiTxtItem)
			elseif eventType == CocosEvent.CLEAN_UP then

			end
		end
	)

	local uiText = cc.Label:createWithSystemFont("-", "Helvetica", 24)
	uiText:setWidth(self:getContentSize().width - 8)
	uiTxtItem:addChild(uiText)
	uiTxtItem.uiText = uiText

	function uiTxtItem:setString(txt)
		local uiText = uiTxtItem.uiText
		if uiText then
			uiText:setString(txt)

			local s = uiText:getContentSize()
			uiTxtItem:setContentSize(cc.size(s.width + 4, s.height + 4))
			REL:toOther("center", "middle", uiText, uiTxtItem, 0, 0)
		end
	end

	return uiTxtItem
end

--- 创建文本条目
function T:createTxtItem(item)
	--	local date = os.date("*t", item.timeMillisec)
	--	local dateStr = tostring(date.month) .. "-" .. tostring(date.day) .. " " .. tostring(date.hour) .. ":" .. tostring(date.min) .. ":" .. tostring(date.sec)
	--	local contentStr = item.message
	--	local txt = "<" .. item.id .. "> [" .. dateStr .. "] - [" .. item.category .. "]\n" .. item.message
	local txt=item.message

	local uiTxtItem = self.POOL_STACK:pop()
	if not uiTxtItem then
		uiTxtItem = self:_createTxtItem()
	end
	uiTxtItem:setString(txt)

	local lv = item.level
	if lv == LOG_LEVEL_DEBUG then
		uiTxtItem:setBackGroundColorType(ccui.LayoutBackGroundColorType.none)
	elseif lv == LOG_LEVEL_WARNING then
		uiTxtItem:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
		uiTxtItem:setBackGroundColorOpacity(100)
		uiTxtItem:setBackGroundColor(cc.c3b(0xff, 0xff, 0x00))
	elseif lv == LOG_LEVEL_ERROR then
		uiTxtItem:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
		uiTxtItem:setBackGroundColorOpacity(100)
		uiTxtItem:setBackGroundColor(cc.c3b(0xff, 0x00, 0x00))
	elseif lv == LOG_LEVEL_FATAL then
		uiTxtItem:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
		uiTxtItem:setBackGroundColorOpacity(100)
		uiTxtItem:setBackGroundColor(cc.c3b(0xff, 0x00, 0x00))
	else
		uiTxtItem:setBackGroundColorType(ccui.LayoutBackGroundColorType.none)
	end

	return uiTxtItem
end

--- 计算真实高度(应该只扩张,不收缩)
function T:calcHeight()
	local items = self.itemsList
	if #items <= 0 then
		return
	end

	local fItem = items[1]
	local lItem = items[#items]

	local t = fItem:getPositionY() + fItem:getContentSize().height
	local b = lItem:getPositionY()

	local c = self.uiItemsScrollContainer
	local iC = c:getInnerContainer()
	local h = math.abs(b - t)

	local itemC = self.itemsContainer

	local cS = c:getContentSize()
	if h >= cS.height then
		iC:setContentSize(cc.size(iC:getContentSize().width, h))

		local oldY = itemC:getPositionY()
		itemC:setPositionY(- b)
		iC:setPositionY(iC:getPositionY() -((- b) - oldY))
	else
		iC:setContentSize(cS)

		itemC:setPositionY(- b +(cS.height - h))
		iC:setPositionY(0)
	end
end

--- 追加条目-头部
function T:appendItem_t(item)
	if self.curButton.level <= item.level then
		local uiTxtItem = self:createTxtItem(item)

		local items = self.itemsList
		table.insert(items, 1, uiTxtItem)
		self.itemsContainer:addChild(uiTxtItem)
		uiTxtItem:release()

		local dnItem = items[2]
		if dnItem then
			REL:toOther("center", nil, uiTxtItem, self.uiItemsScrollContainer, 0, 0)
			uiTxtItem:setPositionY(dnItem:getPositionY() + dnItem:getContentSize().height + 4)
		else
			REL:toOther("center", "top", uiTxtItem, self.uiItemsScrollContainer, 0, 0)
		end

		self:calcHeight()
	end
end

--- 追加条目-尾部
function T:appendItem_b(item)
	if self.curButton.level <= item.level then
		local uiTxtItem = self:createTxtItem(item)

		local items = self.itemsList
		table.insert(items, uiTxtItem)
		self.itemsContainer:addChild(uiTxtItem)
		uiTxtItem:release()

		local c = self.uiItemsScrollContainer
		local ic = c:getInnerContainer()

		-- 添加新项前是否置于最底部
		local isToBottom =(math.abs(ic:getPositionY()) <= 10)

		local upItem = items[#items - 1]
		if upItem then
			REL:toOther("center", nil, uiTxtItem, c, 0, 0)
			uiTxtItem:setPositionY(upItem:getPositionY() - uiTxtItem:getContentSize().height - 4)
		else
			REL:toOther("center", "top", uiTxtItem, c, 0, 0)
		end

		self:calcHeight()

		-- 如果添加新项前置于最底部,则添加新项后仍置于最底部
		local h = ic:getContentSize().height
		if isToBottom then
			ic:setPositionY(0)
		end
	end
end

--- 刷新当前视图的显示条目
function T:refreshCurLv()
	local container = self.itemsContainer
	container:removeAllChildren()
	self.itemsList={}
	self:calcHeight()

	local r = self._recorder
	if not r then
		return
	end

	local e = r:getLength()
	self.recorderStartIndex, self.recorderEndIndex = e + 1, e

	local c, m = 0, self.DEFAULT_SHOW_ITEM_COUNT
	while true do
		if c > m then
			break
		end

		if not self:_appendItem_t() then
			break
		end
		c = c + 1
	end

	-- 滚动到底部
	local ic = self.uiItemsScrollContainer:getInnerContainer()
	local h = ic:getContentSize().height
	ic:setPositionY(0)
end

--- 记录器中被添加条目
function T:recorder_pushHandler(item)
	if item then
		self:_appendItem_b()
	end
end

--- 记录器中被移除条目
function T:recorder_popHandler(item)
	print("移除条目")
end

--- 添加记录器侦听
function T:addRcorderEvtListeners()
	self._recorder:luaAddEventListener("push", self, self.recorder_pushHandler)
	self._recorder:luaAddEventListener("pop", self, self.recorder_popHandler)
end

--- 移除记录器侦听
function T:rmRecorderEvtListeners()
	self._recorder:luaRemoveEventListener("push", self, self.recorder_pushHandler)
	self._recorder:luaRemoveEventListener("pop", self, self.recorder_popHandler)
end

--- 设置日志记录器
function T:setRecorder(value)
	if value ~= self._recorder then
		if self._recorder then
			self:rmRecorderEvtListeners()
		end

		self._recorder = value

		if self._recorder then
			self:refreshCurLv()
			self:addRcorderEvtListeners()
		end
	end
end
--- 获取日志记录器
function T:getRecorder()
	return self._recorder
end

return T
