--- 日志记录器浮动图标
local T=Class.LogRecorderFloatIcon(
	function() return ccui.Layout:create() end
)

--- 日志记录查看器
T.LogRecordLooker=require("src.frameworks.cocos.log.LogRecordLooker")

function T:def()
--- 日志记录查看器
-- self.logLooker=nil

--- 按下时的位置
-- self.touchBeginP=nil

--- 按下时的相对偏移
-- self.offset=nil
end

function T:ctor()
	self:retain()

	self:registerScriptHandler(
		function(eventType)
			if eventType == CocosEvent.ENTER then
				self:onEnter()
			elseif eventType == CocosEvent.EXIT then
				self:onExit()
			elseif eventType == CocosEvent.CLEAN_UP then
				self:cleanUp()
			end
		end
	)

	self:init()
end

--- 进入
function T:onEnter()
	self:release()
end

--- 退出
function T:onExit()
	self:retain()
end

--- 内存清理
function T:cleanUp()

end

--- 初始化
function T:init()
	self:setContentSize(cc.size(48,48))
	self:setLocalZOrder(99999)
	self:setTouchEnabled(false)

	self:toNormalStatus()

	self:initTouchListener()
end

--- 初始化 touch 事件侦听器
function T:initTouchListener()
	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	---开始拖动
	local onTouchBegan = function(t, e)
		local p=t:getLocation()

		local localP=self:convertToNodeSpace(p)
		local contendS=self:getContentSize()

		local rect=cc.rect(0,0,contendS.width,contendS.height)
		if cc.rectContainsPoint(rect, localP) then
			self.touchBeginP=p
			self.offset=localP

			self:toDownStatus()

			return true
		end

		return false
	end

	---拖动
	local onTouchMoved = function(t, e)
		local p=t:getLocation()

		local touchBeginP=self.touchBeginP
		if touchBeginP then
			if math.abs(p.x-touchBeginP.x) < 3 and math.abs(p.y-touchBeginP.y) < 3 then
				return false
			end
		end

		local offset=self.offset
		if offset then
			self.touchBeginP=nil
			self:setPosition(p.x-offset.x,p.y-offset.y)
		end

		return true
	end

	--- 拖动结束
	local onTouchEnded = function(t, e)
		local p=t:getLocation()

		self:toNormalStatus()
		self.offset=nil

		if self.touchBeginP then
			if self.logLooker then
				self:hideLooker()
			else
				self:showLooker()
			end
		end

		return true
	end

	local onTouchCanceled = function(t, e)
		return true
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(onTouchCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)
	local eventDispatcher = self:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

function T:toNormalStatus()
	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	self:setBackGroundColor(cc.c3b(0xff, 0x00, 0xff))
	self:setBackGroundColorOpacity(100)
end

function T:toDownStatus()
	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	self:setBackGroundColor(cc.c3b(0xff, 0xcc, 0xcc))
	self:setBackGroundColorOpacity(100)
end

function T:showLooker()
	local logLooker=T.LogRecordLooker()
	logLooker:setRecorder(LogRecorder)
	self.logLooker=logLooker

	logLooker:luaAddEventListener("close",nil,function()
		self.logLooker=nil
	end)

	Director:getRunningScene():addChild(logLooker)
	REL:toScreen("center", "top", logLooker, 0, 0)
end

function T:hideLooker()
	local logLooker=self.logLooker
	if logLooker and logLooker:getParent() then
		logLooker:removeFromParent()
	end

	self.logLooker=nil
end

local ins=T()

function T:show()
	if LogRecorder:getIsEnabled() then
		Director:getRunningScene():addChild(ins)
		REL:toScreen("center", "top", ins, 0, 0)
	end
end

function T:hide()
	ins:removeFromParent()
end

return T
