--- 带镂空效果的遮罩
-- @author blank
local T=Class.HollowMask(
	function()
		local obj=ccui.Layout:create()
		obj.__addTouchEventListener__ = obj.addTouchEventListener
		
		return obj
	end,
	AbstractUI,
	LuaEventDispatcher2
)

function T:def()
	AbstractUI.def(self)
	LuaEventDispatcher2.def(self)

	--- 镂空区域
	-- hollowAreas
end

--- 构造函数
-- @param hollowAreas 镂空区域列表
function T:ctor(hollowAreas)
	self.hollowAreas=hollowAreas

	LuaEventDispatcher2.ctor(self)
	AbstractUI.ctor(self)

	self:init()
end

function T:init()
	REL:relScreen(self,0,0,0,0)

	self:createClipper()
	self:initTouchListener()

	if self.hollowAreas then
		self:_updateHollowAreas()
	end
end

--- 创建镂空遮罩
function T:createClipper()
	local uiClipper=cc.ClippingNode:create()

	--镂空区域
	local uiHollow=cc.DrawNode:create()
	uiHollow:clear()
	self:addChild(uiHollow)
	self.uiHollow=uiHollow

	uiClipper:setStencil(uiHollow);
	uiClipper:setInverted(true)
	uiClipper:setAlphaThreshold(1);--默认值就是1

	local uiContainer=cc.Node:create()
	uiContainer:setPosition(0,0)
	uiClipper:addChild(uiContainer)

	local uiMask=ccui.Layout:create()
	uiMask:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	uiMask:setBackGroundColor(cc.c3b(0,0,0))
	uiMask:setBackGroundColorOpacity(80)
	uiContainer:addChild(uiMask)
	REL:relScreen(uiMask,0,0,0,0)
	self.uiMask=uiMask

	self:addChild(uiClipper)
	self.uiClipper=uiClipper
end

function T:onTouchBegan(isThrough,touchPt)
--	print("onTouchBegan",obj2str(isThrough))
end

function T:onTouchMoved(touchPt)
--	print("onTouchMoved",obj2str(touchPt))
end

function T:onTouchEnded(touchPt)
--	print("onTouchEnded",obj2str(touchPt))
end

function T:onTouchCanceled(touchPt)
--	print("onTouchCanceled",obj2str(touchPt))
end

--- 初始化 touch 事件侦听器
function T:initTouchListener()
	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	local onTouchBegan = function(t, e)
		local p=t:getLocation()

		local hollowAreas=self.hollowAreas
		if hollowAreas then
			local localP=self:convertToNodeSpace(p)
			for k,v in ipairs(hollowAreas) do
				if MathUtil.containsPointInPolygon(localP,v) then
					self:onTouchBegan(true,p)

					--允许touch事件穿透
					return false
				end
			end
		end

		self:onTouchBegan(false,p)
		return true
	end

	local onTouchMoved = function(t, e)
		self:onTouchMoved(t:getLocation())
		return true
	end

	local onTouchEnded = function(t, e)
		self:onTouchEnded(t:getLocation())
		return true
	end

	local onTouchCanceled = function(t, e)
		self:onTouchCanceled(t:getLocation())
		return true
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(onTouchCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)
	local eventDispatcher = self:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

--- 更新镂空区域列表
function T:_updateHollowAreas()
	local uiHollow=self.uiHollow
	uiHollow:clear()

	if self.hollowAreas then
		for k,v in ipairs(self.hollowAreas) do
			uiHollow:drawPolygon(v,#v,cc.c4f(1,1,1,0), 0, cc.c4f(1,0,0,1))
		end
	end
end

--- 设置镂空区域列表
-- @param value 镂空区域列表
function T:setHollowAreas(value)
	if self.hollowAreas ~= value then
		self.hollowAreas = value

		self:_updateHollowAreas()
	end
end
--- 获取镂空区域列表
function T:getHollowAreas()
	return self.hollowAreas
end

--- 设置遮罩颜色
function T:setMaskColor(c3bColor)
	self.uiMask:setBackGroundColor(c3bColor)
end
--- 获取遮罩颜色
function T:getMaskColor()
	return self.uiMask:getBackGroundColor()
end

--- 设置遮罩透明度
function T:setMaskOpacity(byteOpacity)
	self.uiMask:setBackGroundColorOpacity(byteOpacity)
end
--- 获取遮罩透明度
function T:getMaskOpacity()
	return self.uiMask:getBackGroundColorOpacity()
end

return T
