--- 可以显示XML格式的富文本
-- @author blank
local T=Class.XMLLabel(
	function()
		local o=ccui.Layout:create()
		o.__setSize__=o.setContentSize

		return o
	end,
	AbstractUI,
	LuaEventDispatcher2
)

--- 事件-文本内容的 size 被改变
T.EVENT_CHANGE_TEXT_SIZE="event_change_text_size"

--- 事件-触发了超链接(回调的参数有:target,触发对象;evtType,事件类型;evtArg,事件参数;textLabel,触发的文本;)
T.EVENT_TRIGGERED_LINK="event_triggered_link"

--- XML文本解析器
T.XMLDecoder=require("src.frameworks.base.XMLDecoder")

--- XML文本创建器
T.XMLTextCreator=require("src.frameworks.cocos.ui.XMLTextCreator")

--- 默认css样式表
T.cssStyleList=nil

--- 设置默认css样式表
function T:setCssStyleList(cssStyleList)
	self.cssStyleList=cssStyleList
end

function T:def()
	AbstractUI.def(self)
	LuaEventDispatcher2.def(self)

	--- 纯文本内容
	-- self._text=nil

	--- xml文本内容
	-- self._xmlText=nil

	--- 字体名
	-- self._fontName=nil

	--- 字号
	-- self._fontSize=nil

	--- 字体颜色
	-- self._color=nil

	--- css样式
	-- self._cssStyle=nil

	--- 自动大小的方式(
	--  none:不自动调整宽高
	--  width:自动调整宽度
	--  height:自动调整高度
	--  both:自动调整宽高
	--  )
	self._autoSizeLabel="none"

	--- 单行文本
	self._isSingleLine=false

	--- 水平对齐方式
	self._hAlign="left"

	--- 垂直对齐方式
	self._vAlign="top"

	--- 行间距
	self._lineSpacing=2

	--- 更新标记
	-- self.updateLabel=nil

	--- 渲染延迟SID
	-- self.updateDelayAct=nil

	--- 文本行列表
	-- self.uiTextLines=nil

	--- 文本渲染器
	-- self.uiTextBlock=nil
end

--- 构造函数
-- @param text 文本
-- @param fontSize 字号
-- @param fontName 字体名
-- @param hAlign 水平对齐方式
-- @param vAlign 垂直对齐方式
function T:ctor(text,fontSize,color,fontName,hAlign,vAlign)
	if text then
		self:setString(text)
	end
	self._fontSize=fontSize or self._fontSize
	self._color=color or self._color
	self._fontName=fontName or self._fontName
	self._hAlign=hAlign or self._hAlign
	self._vAlign=vAlign or self._vAlign

	AbstractUI.ctor(self)

	self:init()
end

--- 进入
function T:onEnter()
	AbstractUI.onEnter(self)

	if self.updateLabel then
		self:addUpdateLabel()
	end
end

--- 退出
function T:onExit()
	AbstractUI.onExit(self)

	if self.updateDelayAct then
		self:clearScheduler(self.updateDelayAct)
		self.updateDelayAct=nil
	end
end

--- 内存清理
function T:cleanUp()
	AbstractUI.cleanUp(self)

	self.updateLabel=nil
	self.uiTextLines=nil
end

--- 初始化
function T:init()
	local uiTextBlock=ccui.Layout:create()
	self:addChild(uiTextBlock)
	self.uiTextBlock=uiTextBlock

--	self:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
--	self:setBackGroundColor(cc.c3b(random(0,255),0,0))
--	self:setBackGroundColorOpacity(100)

	--			uiTextBlock:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	--			uiTextBlock:setBackGroundColor(cc.c3b(0,0,0))
	--			uiTextBlock:setBackGroundColorOpacity(100)
end

--- 对齐
function T:_align()
	local uiTextBlock=self.uiTextBlock
	local textSize=uiTextBlock:getContentSize()
	local tW,tH=textSize.width,textSize.height

	local size=self:getContentSize()
	local w,h=size.width,size.height

	--设置自动大小
	local autoSizeLabel=self._autoSizeLabel
	if autoSizeLabel == "both" or (w == 0 and h == 0) then
		self:__setSize__(textSize)
		w,h=tW,tH
	elseif autoSizeLabel == "width" or w == 0 then
		self:__setSize__(cc.size(tW,h))
		w=tW
	elseif autoSizeLabel == "height" or h == 0 then
		self:__setSize__(cc.size(w,tH))
		h=tH
	end

	--设置文本对齐
	local hAlign,vAlign=self._hAlign,self._vAlign
	local x,y=0,0
	if hAlign == "center" then
		x=(w-tW)/2
	elseif hAlign == "right" then
		x=w-tW
	else
		x=0
	end
	if vAlign == "middle" then
		y=(h-tH)/2
	elseif vAlign == "bottom" then
		y=0
	else
		y=h-tH
	end
	uiTextBlock:setPosition(x,y)
end

--- 绘制文本
function T:updateView()
	if not self.updateLabel then
		return
	end

	local uiTextBlock=self.uiTextBlock

	local widthLimit=self:getContentSize().width
	if self._autoSizeLabel == "both" or self._autoSizeLabel == "width" or self._isSingleLine then
		--自动宽
		widthLimit = 2048
	end

	if self.updateLabel == 1 then
		uiTextBlock:removeAllChildren()

		if self._xmlText and widthLimit > 0 then
			local textLines=self.XMLTextCreator:createTextLines(self._xmlText,widthLimit,self._cssStyle or {
				fontName=self._fontName,
				size=self._fontSize,
				color=self._color
			},self.cssStyleList,function(target,evtType,evtArg)
				self:luaDispatchEvent(self.EVENT_TRIGGERED_LINK,self,evtType,evtArg,target)
			end)
			self.uiTextLines=textLines
			if textLines then
				self.XMLTextCreator:add2TextLineBlock(uiTextBlock,textLines)

				self.XMLTextCreator:ArrangTextLines(self.uiTextBlock,self.uiTextLines,widthLimit,self._hAlign,self._lineSpacing)

				self:_align()
			else
				error("create text fail!")
			end

			self:luaDispatchEvent(self.EVENT_CHANGE_TEXT_SIZE,self)
		else
			self.uiTextLines=nil
		end
	else
		if self.uiTextLines then
			if self.updateLabel == 2 then
				self.XMLTextCreator:ArrangTextLines(self.uiTextBlock,self.uiTextLines,widthLimit,self._hAlign,self._lineSpacing)
			end

			self:_align()
			self:luaDispatchEvent(self.EVENT_CHANGE_TEXT_SIZE,self)
		end
	end

	self.updateLabel=nil
end

--- 添加更新标记
function T:addUpdateLabel()
	if self.updateDelayAct
		or
		not self:getParent()
		or
		not self:isRunning()
	then
		return
	end

	--- 下一帧绘制
	self.updateDelayAct=self:timeout(function()
		if self.updateView then
			self:updateView()
		end

		self.updateDelayAct=nil
	end,0)
end

--- 添加更新标记
function T:addDrawLabel()
	if self.updateLabel ~= 1 then
		self.updateLabel=1

		self:addUpdateLabel()
	end
end

--- 添加更新水平对齐方式标记
function T:addUpdateHAlign()
	if self.updateLabel and self.updateLabel <= 2 then
		return
	end

	self.updateLabel=2
	self:addUpdateLabel()
end

--- 添加更新垂直对齐方式标记
function T:addUpdateVAlign()
	if self.updateLabel and self.updateLabel <= 3 then
		return
	end

	self.updateLabel=3
	self:addUpdateLabel()
end

--- 设置XML文本
-- @param xml XML文本
function T:setXMLText(xml)
	if xml ~= self._xmlText then
		self._xmlText=xml

		self:addDrawLabel()
	end
end

--- 获取XML文本
function T:getXMLText()
	return self._xmlText
end

--- 设置文本
-- @param 文本
function T:setString(text)
	if text ~= self._text then
		self._text=text

		if text then
			self:setXMLText(self.XMLDecoder:decode(text))
		else
			self:setXMLText(nil)
		end
	end
end

--- 获取文本
function T:getString()
	return self._text
end

--- 设置字体名称
function T:setFontName(value)
	if value ~= self._fontName then
		self._fontName=value

		self:addDrawLabel()
	end
end
--- 获取字体名称
function T:getFontName()
	return self._fontName
end

--- 设置字号
function T:setFontSize(value)
	if value ~= self._fontSize then
		self._fontSize=value

		self:addDrawLabel()
	end
end
--- 获取字号
function T:getFontSize()
	return self._fontSize
end

--- 设置文本颜色
function T:setColor(value)
	if value ~= self._color then
		self._color=value

		self:addDrawLabel()
	end
end
--- 获取文本颜色
function T:getColor()
	return self._color
end

--- 设置css样式
function T:setCssStyle(value)
	if value ~= self._cssStyle then
		self._cssStyle=value

		self:addDrawLabel()
	end
end
--- 获取css样式
function T:getCssStyle()
	return self._cssStyle
end

--- 设置自动大小的方式
function T:setAutoSizeLabel(value)
	value = value or 0
	if value ~= self._autoSizeLabel then
		self._autoSizeLabel=value

		self:addUpdateHAlign()
	end
end
function T:getAutoSizeLabel()
	return self._autoSizeLabel
end

--- 设置水平对齐方式
function T:setHAlign(value)
	if value ~= self._hAlign then
		self._hAlign=value

		self:addUpdateHAlign()
	end
end
--- 获取水平对齐方式
function T:getHAlign()
	return self._hAlign
end

--- 设置垂直对齐方式
function T:setVAlign(value)
	if value ~= self._vAlign then
		self._vAlign=value

		self:addUpdateVAlign()
	end
end
--- 获取垂直对齐方式
function T:getVAlign()
	return self._vAlign
end

--- 设置对齐方式
function T:setAlign(hAlign,vAlign)
	self:setHAlign(hAlign)
	self:setVAlign(vAlign)
end
--- 获取对齐方式
-- @return 水平对齐方式,垂直对齐方式
function T:getAlign()
	return self._hAlign,self._vAlign
end

--- 设置行间距
function T:setLineSpacing(value)
	if value ~= self._lineSpacing then
		self._lineSpacing=value

		self:addUpdateHAlign()
	end
end
--- 获取行间距
function T:getLineSpacing()
	return self._lineSpacing
end

--- 是否为单行文本
function T:setIsSingleLine(value)
	if value ~= self._isSingleLine then
		self._isSingleLine=value

		self:addUpdateVAlign()
	end
end
--- 是否为单行文本
function T:getIsSingleLine()
	return self._isSingleLine
end

--- 设置大小
function T:setContentSize(value)
	local size=self:getContentSize()
	if value.width ~= size.width then
		self:__setSize__(value)
		self:addDrawLabel()
	elseif value.height ~= size.height then
		self:__setSize__(value)
		self:addUpdateVAlign()
	end
end

--- 设置宽
function T:setWidth(value)
	local size=self:getContentSize()
	if value ~= size.width then
		size.width=value
		self:__setSize__(size)

		self:addDrawLabel()
	end
end
--- 获取宽
function T:getWidth()
	return self:getContentSize().width
end

--- 设置高
function T:setHeight(value)
	local size=self:getContentSize()
	if value ~= size.height then
		size.height=value
		self:__setSize__(size)

		self:addUpdateVAlign()
	end
end
--- 获取高
function T:getHeight()
	return self:getContentSize().height
end

--- 获取文本的宽度
function T:getTextWidth()
	return self.uiTextBlock:getContentSize().width
end
--- 获取文本的高度
function T:getTextHeight()
	return self.uiTextBlock:getContentSize().height
end
--- 获取文本的尺寸
function T:getTextSize()
	return self.uiTextBlock:getContentSize()
end

return T
