--- 说明
-- 目前支持4种标签
-- 1.<span></span>(除下面其它标签的任意标签也可):文本
-- 	 属性:
-- 	 	  (1).size 				字号
-- 	  (2).color 			字体颜色, 形如: "0xaabbccdd" 等于 {r=0xaa,g=0xbb,b=0xcc,a=0xdd}
-- 	  (3).fontName, 		字体名称
-- 	  (4).outlineColor 		描边颜色,
-- 	  (5).outlineSize 		描边粗细, 形如: "2" 等于 2像素,(必须设置 outlineColor )
-- 	  (6).shadowColor 		阴影颜色, 如果指定了 outlineColor ,则此值无效
-- 	  (7).shadowOffset 		阴影偏移, 形如: "2,-2" 等于 cc.p(2,-2) (必须设置 shadowColor )
-- 	  (8).class				css样式名称(上面1至7的优先级高于此属性,如果未找到才会查找css样式)
-- 2.<img/>或<img></img>:图像
-- 	 属性:
-- 	 	  (1).src				图像的 url
-- 	 	  (2).srcType			图像的类型,分别为(ccui.TextureResType.plistType=1(默认),ccui.TextureResType.localType=0)
-- 	 	  (3).width				图像显示的宽,如果不指定,则为原始图像大小
-- 	 	  (4).height			图像显示的高,如果不指定,则为原始图像大小
-- 	 	  (5).color				图像的颜色偏移
-- 	 	  (6).name				图像的名称
-- 3.<mov/>或<mov></mov>:动画
-- 	 属性:
-- 	 	  (1).src				动画名(必须是预先构造好的动画对象)
-- 	 	  (2).width				动画显示的宽,如果不指定,默认为第一帧图像的宽
-- 	 	  (3).height			动画显示的高,如果不指定,默认为第一帧图像的高
-- 	 	  (4).loopDelay			循环播放延迟,形如:"1,5",表示随机1至5秒后重播;如果为"3",表示3秒后重播
-- 	 	  (5).repeatCount		重播次数,指定 loopDelay 后才有效.表示每次重播时重复播放的次数
-- 	 	  (6).name				动画的名称
-- 4.<br/>或<br></br>:换行
-- 5.<a></a>:事件
-- 属性:
-- 	 	  (1).event				事件参数(触发事件后传递给事件侦听器的参数)
-- 	 	  (2).同span标签

--- XML文本定义形式台下
-- 这是测试<span color='oxff0000' size=24>红色<span shadowColor='ox00ff00' shadowSize=2>描边</span>24号</span>

local XMLTextCreator = { }

--- 将字符串转换为宽字节数组
-- @param 字符串
local str2WCharArray = StringUtils.str2WCharArray

--- 字符串转换为 color4b 对象
-- @param text 字符串
local str2c4b = Cocos2dxUtils.str2c4b

--- 字符串转换为 size
-- @param text 字符串
local str2Size = Cocos2dxUtils.str2Size

local concat = table.concat
local strLen = string.len
local ceil = math.ceil
local floor = math.floor
local strFind = string.find
local strSub = string.sub
local strByte = string.byte
local abs=math.abs

--- 从宽字节字符数组中,从指定的位置开始获取到指定宽度的字符的结束位置
-- @param wChars 宽字节字符数组
-- @param widthLimit 宽度
-- @param s 字符数组中的开始位置
-- @param e 字符数组中的结束位置
-- @param size 字号
local function limitWChar(wChars, widthLimit, s, e, size)
	local w, l = 0, size / 2
	for i = s, e, 1 do
		w = w +(strLen(wChars[i]) == 1 and l or size)

		if w > widthLimit then
			return i - 1
		elseif w == widthLimit then
			return i
		end
	end

	return e
end

--- 判断字符是否为字母字符
local isLetter = function(char)
	local len=strLen(char)
	if len == 1 then
		local code = strByte(char)
		return code >= 65 and code <= 90 or code >= 97 and code <= 122
	end
	return false
end

--- 添加事件侦听器
-- @param host 侦听宿主
-- @param evtArg 事件参数
-- @param onLinkHandler 超链接处理函数
local addLinkListener=function(host, evtArg, onLinkHandler)
	local listener = cc.EventListenerTouchOneByOne:create()
	listener:setSwallowTouches(true)

	---开始拖动
	local onTouchBegan = function(t, e)
		local p=t:getLocation()

		local localP=host:convertToNodeSpace(p)
		local contendS=host:getContentSize()

		local rect=cc.rect(0,0,contendS.width,contendS.height)
		if cc.rectContainsPoint(rect, localP) then
			onLinkHandler(t,e,evtArg)

			return true
		end

		return false
	end

	---拖动
	local onTouchMoved = function(t, e)
		onLinkHandler(t,e,evtArg)
	end

	--- 拖动结束
	local onTouchEnded = function(t, e)
		onLinkHandler(t,e,evtArg)
	end

	local onTouchCanceled = function(t, e)
		onLinkHandler(t,e,evtArg)
	end

	listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
	listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
	listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
	listener:registerScriptHandler(onTouchCanceled, cc.Handler.EVENT_TOUCH_CANCELLED)
	host:getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, host)
end

--- 创建文本
-- @param text 文本
-- @param size 字号
-- @param fontName 字体名称
-- @param color 字体颜色
-- @param outLineColor 描边颜色
-- @param outLineSize 描边粗细
-- @param shadowColor 阴影颜色
-- @param shadowOffset 阴影偏移
-- @param evtArg 事件参数
-- @param onLinkHandler link事件处理器
-- @return 创建的文本显示对象
local createLabel=function(text, size, fontName, color, outLineColor, outLineSize, shadowColor, shadowOffset, evtArg, onLinkHandler)
	local l = cc.Label:createWithSystemFont(text, fontName, size)

	local isSetedColor = false

	if outLineColor then
		if color then
			l:setTextColor(color)
			isSetedColor = true
		end

		l:enableOutline(outLineColor, outLineSize)
	elseif shadowColor then
		l:enableShadow(shadowColor, shadowOffset)
	end

	-- 设置字体颜色
	if not isSetedColor and color then
		l:setColor(color)
	end
	
	-- 超链接参数
	if evtArg then
		addLinkListener(l,evtArg,onLinkHandler)
	end

	return l
end

--- 从CSS样式栈中获取字体描述(
-- 1.优先从当前元素上获取,其次从当前元素的 CSS 样式表中获取;
-- 2.如果当前元素无此属性,则从父元素中获取,经此类推,直到最顶层父节点
-- 3.如果元素中未找到此属性,则从 CSS 样式表中获取 default 样式中获取)
-- @param elementStack 元素栈
-- @param propName 属性名
-- @param cssList 公用css样式列表
local function getFontDesc(elementStack, propName, cssList)
	local max = elementStack:getLength() -1

	if cssList then
		for i = 0, max, 1 do
			local t = elementStack:look(i)
			local v = t[propName]
			if v then
				return v
			end

			local clsName = t["class"]
			if clsName then
				local cls = cssList[clsName]
				if cls then
					local v = cls[propName]
					if v then
						return v
					end
				end
			end
		end

		local cls = cssList["default"]
		if cls then
			return cls[propName]
		end
	else
		for i = 0, max, 1 do
			local t = elementStack:look(i)
			local v = t[propName]
			if v then
				return v
			end
		end
	end

	return nil
end

--- 创建文本
-- @param text 文本
-- @param elementStack 元素栈(最顶层即为当前元素)
-- @param widthLimit 行的最大宽度
-- @param textLines 文本行列表(数组),新生成的行将放到最后
-- @param cssList 公用css样式列表
-- @param onLinkHandler link事件处理器
local function createText(text, elementStack, widthLimit, textLines, cssList, onLinkHandler)
	local size = getFontDesc(elementStack, "size", cssList)
	size = size and tonumber(size) or 24
	local fontName = getFontDesc(elementStack, "fontName", cssList) or "helvetica"

	local color = getFontDesc(elementStack, "color", cssList)
	color = color and str2c4b(color) or nil

	local outlineColor, outlineSize
	local shadowColor, shadowOffset

	outlineColor = getFontDesc(elementStack, "outlineColor", cssList)
	if outlineColor then
		outlineColor = str2c4b(outlineColor)

		outlineSize = getFontDesc(elementStack, "outlineSize", cssList)
		outlineSize = outlineSize and tonumber(outlineSize) or 2
	else
		shadowColor = getFontDesc(elementStack, "shadowColor", cssList)
		if shadowColor then
			shadowColor = str2c4b(shadowColor)

			shadowOffset = getFontDesc(elementStack, "shadowOffset", cssList)
			if shadowOffset then
				shadowOffset = str2Size(shadowOffset)
			else
				shadowOffset = cc.size(2, -2)
			end
		end
	end

	-- 超链接参数
	local evtArg=getFontDesc(elementStack, "event", cssList)

	local halfSize = size / 2
	if halfSize > widthLimit then
		-- error("Line width is not sufficient to show 1 characters")
		local message = "Line width is not sufficient to show 1 characters" .. "\n" .. debug.traceback()

		__print__("XMLTextCreateErr", message)
		LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
	end

	local len = #textLines
	local line = textLines[len]

	local newline = function(wChars, s, e)
		line = { remainingWidth = widthLimit, maxHeight = 0, list = { } }
		len = len + 1
		textLines[len] = line

		return limitWChar(wChars, widthLimit, s, e, size)
	end

	-- 单字符数组
	local wChars = str2WCharArray(XMLDecoder:unescape(text))
	local s, e, state = 1, #wChars, 0
	while s <= e do
		local rw, sep, uiL = line.remainingWidth, s, nil

		-- 如果不足以显示下半个字符,则换行
		if rw < halfSize or state ~= 0 then
			sep, rw = newline(wChars, s, e), widthLimit
		else
			sep = limitWChar(wChars, rw, s, e, size)
			if sep < s then
				sep, rw = newline(wChars, s, e), widthLimit
			end
		end

		state = 0
		local w, h, offset = 0, 0, 0
		while true do
			while true do
				--	print(text,s,e,sep)

				if uiL then
					uiL:setString(concat(wChars, "", s, sep))
				else
					uiL = createLabel(concat(wChars, "", s, sep), size, fontName, color, outlineColor, outlineSize, shadowColor, shadowOffset,evtArg,onLinkHandler)
				end

				local cSize = uiL:getContentSize()
				w, h = cSize.width, cSize.height

				if state == 1 then
					break
				end

				local dw = rw - w
				if dw < 0 then--= - halfSize then
					-- 实际宽度已超出此行(必须处理)
					if offset == 1 then
						-- 上一次增加字符前是错误的(退回到上次增加前)

						offset = -1
						sep = sep + offset
				else
					if offset == 0 then
						offset = floor(dw / halfSize)
						-- 偏移超出宽度一半的宽度的字符数,且至少偏移1个字符
					else
						offset = -ceil(abs(offset) / 2)
					end

					sep = sep + offset
					if sep <= s then
						if offset == -1 then
							sep, rw = newline(wChars, s, e), widthLimit
						else
							offset = -1
						end
					end
				end
				elseif dw > halfSize then
					-- 至少还有半个字符的区域未被填充

					-- 如果已显示完所有文本,且宽度未超出
					if sep >= e then
						break
					end

					if offset == -1 then
						-- 上一次减去1个字符是正确的,此次生成的 Label 为最终 Label
						break
					else
						if offset == 0 then
							offset = floor(dw / halfSize)
							-- 偏移少出宽度一半的宽度的字符数,且至少偏移1个字符
						else
							offset = ceil(abs(offset) / 2)
						end

						sep = sep + offset
						if sep > e then
							offset = e - sep
							sep = e
						end
					end
				else
					break
				end
			end
			-- while

			-- 如果此行的最后一个字符为字母字符,则需要向前想找非字母字符,并从此处断行
			if state == 0 and sep < e and isLetter(wChars[sep]) and isLetter(wChars[sep + 1]) then
				for i = sep - 1, s, -1 do
					if not isLetter(wChars[i]) then
						sep = i

						state = 1
						break
					end
				end
				-- 如果未找到非字母字符
				if state ~= 1 then
					state = 2
					sep, rw = newline(wChars, s, e), widthLimit
				end
			else
				break
			end
		end
		-- while

		s = sep + 1
		uiL.__width__ = w
		line.remainingWidth = rw - w
		if h > line.maxHeight then
			line.maxHeight = h
		end

		local ls = line.list
		ls[#ls + 1] = uiL
	end
end

--- 追加图形(图像,动画)到文本行列表末尾
-- @param graphic 图形元素
-- @param widthLimit行的最大宽度
-- @param textLines 文本行列表(数组),新生成的行将放到最后
-- @param width 图形的宽
-- @param height 图形的高
-- @param evtArg 事件参数
-- @param onLinkHandler 超链接处理函数
local appendGraphic=function(graphic, widthLimit, textLines, width, height, evtArg, onLinkHandler)
	local line, rw

	line = textLines[#textLines]
	rw = line.remainingWidth

	-- 如果当前行显示不下此图形,则换行后必须显示此图形
	if width > rw then
		if width > widthLimit then
			local message = "the graphic's width > the line's width!" .. "\n" .. debug.traceback()

			__print__("XMLTextCreateErr", message)
			LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
			return
		end
		-- 换行
		textLines[#textLines + 1] = { remainingWidth = widthLimit, maxHeight = 0, list = { } }
	end

	rw = rw - width
	line.remainingWidth = rw
	if height > line.maxHeight then
		line.maxHeight = height
	end

	local ls = line.list
	ls[#ls + 1] = graphic

	if rw < 0 then
		-- 换行
		textLines[#textLines + 1] = { remainingWidth = widthLimit, maxHeight = 0, list = { } }
	end

	-- 超链接参数
	if evtArg then
		addLinkListener(graphic,evtArg,onLinkHandler)
	end
end

--- 创建图像
-- @param imgXML 图像元素
-- @param widthLimit行的最大宽度
-- @param textLines 文本行列表(数组),新生成的行将放到最后
-- @param cssList 公用css样式列表
-- @param elementStack 元素栈
-- @param onLinkHandler 超链接处理函数
local createImg=function(imgXML, widthLimit, textLines, cssList, elementStack, onLinkHandler)
	local src = imgXML.src

	if not src or src == "" then
		local message = "the uiImg label not has \"src\" attribute" .. "\n" .. debug.traceback()

		__print__("XMLTextCreateErr", message)
		LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
		return
	end

	local srcType = tonumber(imgXML.srcType or ccui.TextureResType.plistType)
	local uiImg = ccui.ImageView:create(src, srcType)

	if not uiImg then
		local message = "create image fail! src=\"" .. src .. "\"" .. "\n" .. debug.traceback()

		__print__("XMLTextCreateErr", message)
		LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
		return
	end

	-- 计算图片的宽高
	local w, h = tonumber(imgXML.width), tonumber(imgXML.height)
	local s = uiImg:getContentSize()
	if w then
		uiImg:setScaleX(w / s.width)
	else
		w = s.width
	end
	if h then
		uiImg:setScaleY(h / s.height)
	else
		h = s.height
	end

	-- 设置图片的显示颜色
	local color = imgXML.color
	if color then
		color = str2c4b(color)

		uiImg:setColor(color)
		uiImg:setOpacity(color.a)
	end

	-- 图像名称
	uiImg.__name__ = imgXML.name

	uiImg.__width__ = w
	appendGraphic(uiImg, widthLimit, textLines, w, h, imgXML.event or getFontDesc(elementStack, "event", cssList), onLinkHandler)
end

--- 创建动画
-- @param movXML 动画元素
-- @param widthLimit行的最大宽度
-- @param textLines 文本行列表(数组),新生成的行将放到最后
-- @param cssList 公用css样式列表
-- @param elementStack 元素栈
-- @param onLinkHandler 超链接处理函数
local function createMov(movXML, widthLimit, textLines, cssList, elementStack, onLinkHandler)
	local src = movXML.src
	if not src or src == "" then
		local message = "the mov label not has \"src\" attribute" .. "\n" .. debug.traceback()

		__print__("XMLTextCreateErr", message)
		LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
		return
	end

	local animation = ResLoader:getAnimation(src)
	if not animation then
		local message = "the move:\"" .. src .. "\" not exist" .. "\n" .. debug.traceback()

		__print__("XMLTextCreateErr", message)
		LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
		return
	end

	local uiMov = cc.Sprite:create()

	local animate = cc.Animate:create(animation);
	if movXML.loopDelay then
		local arr = StringUtils.split(movXML.loopDelay, ",")

		local l, t = tonumber(arr[1]) or 0, tonumber(arr[2]) or 0
		if t and l == t then
			t = nil
		end

		local rc = tonumber(movXML.repeatCount) or 1

		if t then
			local runAct
			runAct = function()
				uiMov:runAction(
					cc.Sequence:create(
						rc == 1 and animate or cc.Repeat:create(animate, rc),
						cc.DelayTime:create(randomFloat(l, t)),
						cc.CallFunc:create( function(t)
							runAct()
						end )
					)
				)
			end
			runAct()
		else
			uiMov:runAction(cc.RepeatForever:create(
				cc.Sequence:create(
					rc == 1 and animate or cc.Repeat:create(animate, rc),
					cc.DelayTime:create(l)
				)
			))
		end
	else
		uiMov:runAction(cc.RepeatForever:create(animate))
	end

	local w, h = tonumber(movXML.width), tonumber(movXML.height)
	-- 跟据第1帧的大小设置整个动画的大小
	local firstFrame = animation:getFrames()[1]
	if firstFrame then
		local s = firstFrame:getSpriteFrame():getOriginalSize()
		uiMov:setContentSize(s)
		if w then
			uiMov:setScaleX(w / s.width)
		else
			w = s.width
		end
		if h then
			uiMov:setScaleY(h / s.height)
		else
			h = s.height
		end
	else
		local message = "the animation \"" .. src .. "\" frames is empty" .. "\n" .. debug.traceback()

		__print__("XMLTextCreateErr", message)
		LogRecorder:push(message, LOG_LEVEL_ERROR, "XMLTextCreateErr")
		return
	end

	-- 动画名称
	uiMov.__name__ = movXML.name

	uiMov.__width__ = w
	appendGraphic(uiMov, widthLimit, textLines, w, h, movXML.event or getFontDesc(elementStack, "event", cssList),onLinkHandler)
end

local function _create(textXML, cssStack, widthLimit, textLines, cssList, onLinkHandler)
	local children = textXML.__child__
	if not children then
		return nil
	end

	cssStack:push(textXML)
	for _, v in pairs(children) do
		local t = type(v)
		if t == "string" then
			-- 解决 \n 和 \r 换行出错的问题
			local s, e = 1, 0
			while true do
				-- \r和\n都会作为断行符
				_, e = strFind(v, "[\r\n|\n\r|\n|\r]", e + 1)
				if e then
					createText(strSub(v, s, e - 1), cssStack, widthLimit, textLines, cssList, onLinkHandler)

					-- 换行
					textLines[#textLines + 1] = { remainingWidth = widthLimit, maxHeight = 0, list = { } }
				else
					createText(strSub(v, s), cssStack, widthLimit, textLines, cssList, onLinkHandler)
					break
				end
				s = e + 1
			end
		else
			-- 标签名
			local lName = v.__name__
			if lName == "br" then
				-- 换行
				textLines[#textLines + 1] = { remainingWidth = widthLimit, maxHeight = 0, list = { } }
			elseif lName == "img" then
				createImg(v, widthLimit, textLines, cssList, cssStack, onLinkHandler)
			elseif lName == "mov" then
				createMov(v, widthLimit, textLines, cssList, cssStack, onLinkHandler)
			else
				_create(v, cssStack, widthLimit, textLines, cssList, onLinkHandler)
			end
		end
	end
	cssStack:pop()
end

--- 创建文本行列表
-- @param textXML XML
-- @param widthLimit 文本块的最大宽度
-- @param defaultCssStyle 默认css样式
-- @param cssList 样式表
-- @param onLinkHandler 超链接处理回调函数
-- @return 文本行列表
function XMLTextCreator:createTextLines(textXML, widthLimit, defaultCssStyle, cssList, onLinkHandler)
	local textLines = { }
	textLines[#textLines + 1] = { remainingWidth = widthLimit, maxHeight = 0, list = { } }

	-- local lt=TimeUtils:getSyncMillis()

	local cssStack = Stack()
	if defaultCssStyle then
		cssStack:push(defaultCssStyle)
	end
	_create(textXML, cssStack, widthLimit, textLines, cssList, onLinkHandler)
	if defaultCssStyle then
		cssStack:pop()
	end
	cssStack = nil

	-- print("时间:",TimeUtils:getSyncMillis()-lt)

	return textLines
end

--- 添加文本行到文本块
-- @param uiTextBlock 文本行容器
-- @param textLines 文本行
function XMLTextCreator:add2TextLineBlock(uiTextBlock, textLines)
	for _, v in pairs(textLines) do
		for _, v1 in pairs(v.list) do
			uiTextBlock:addChild(v1)
		end
	end
end

--- 排列文本行
-- @param uiTextBlock 文本行容器
-- @param textLines 文本行
-- @param widthLimit 最大行宽
-- @param hAlign 水平对齐方式
-- @param lineSpacing 行间距
function XMLTextCreator:ArrangTextLines(uiTextBlock, textLines, widthLimit, hAlign, lineSpacing)
	lineSpacing = lineSpacing or 2

	-- 文本块的高
	local h = 0
	-- 最小剩余宽
	local minRemainingWidth = widthLimit
	for _, v in pairs(textLines) do
		if v.remainingWidth < minRemainingWidth then
			minRemainingWidth = v.remainingWidth
		end
		h = h +(v.maxHeight + lineSpacing)
	end
	h = h - lineSpacing
	minRemainingWidth = minRemainingWidth > 0 and minRemainingWidth or 0
	-- 文本块的宽
	local w = widthLimit - minRemainingWidth

	uiTextBlock:setContentSize( { width = w, height = h })

	-- 设置标签列表的位置
	local lbAnchorPoint = { x = 0, y = 0 }
	local left, bottom = nil, h
	for _, v in pairs(textLines) do
		if hAlign == "center" then
			left =(v.remainingWidth - minRemainingWidth) / 2
		elseif hAlign == "right" then
			left = v.remainingWidth - minRemainingWidth
		else
			left = 0
		end

		bottom = bottom - v.maxHeight
		for _, v1 in pairs(v.list) do
			v1:setAnchorPoint(lbAnchorPoint)
			v1:setPosition(left, bottom)

			left = left + v1.__width__
		end
		bottom = bottom - lineSpacing
	end
end

return XMLTextCreator
