local _print=print
local function print(...)
	_print("\t<AbstractGroundGrid>: ",table.concat({...},"\t"))
end

local floor=math.floor

--- 单元格
local AbstractGroundCell=require("src.frameworks.cocos.ui.ground.AbstractGroundCell")

--- 网格
local T = Class.AbstractGroundGrid(
	AbstractUI
)

function T:def()
	AbstractUI.def(self)

	--- 单元格列表
	self._cells = { }

	--- 上一次计算的可视区域坐标
	self._lsx,self._lsy,self._lex,self._ley=0,0,0,0
end

--- 构造函数
-- @param visibleRect 可见区域
-- @param cellSize 单元格尺寸
function T:ctor(visibleRect,cellSize)
	AbstractUI.ctor(self)

	--- 可见区域
	self._vRect = visibleRect
	self._cellSize=cellSize

	--- 更新
	self:updateGrid()
end

--- 创建单元格
-- @param x
-- @param y
-- @return cell
function T:_createCell(x,y)
	local cell=AbstractGroundCell()
	cell.x,cell.y=x,y
	cell.isShow=false

	return cell
end

--- 显示单元格
-- @param cell
-- @return cell
function T:_showCell(cell)
	cell.isShow=true

	return cell
end

--- 隐藏单元格
-- @param cell
-- @return cell
function T:_hideCell(cell)
	cell.isShow=false

	return cell
end

--- 更新
function T:updateGrid()
	local vRect = self._vRect
	local cw, ch = self._cellSize.width, self._cellSize.height

	local l, b = vRect.x, vRect.y
	local r, t = l + vRect.width, b + vRect.height

	--单元格以左下角对齐
	local sx, sy, ex, ey = floor(l / cw), floor(b / ch), floor(r / cw), floor(t / ch)

	-- 判断可视区内的单元格有没有改变
	if sx == self._lsx and sy == self._lsy and ex == self._lex and ey == self._ley then
		return
	end
	self._lsx,self._lsy,self._lex,self._ley=sx,sy,ex,ey
	--    print("可视区域:",sx,sy,ex,ey)

	local _cells = self._cells

	-- 将可视区外的单元格隐藏
	for x,col in pairs(_cells) do
		for y,cell in pairs(col) do
			if (x < sx or x > ex or y < sy or y > ey) and cell.isShow then
				--            	print("隐藏:",x,y)
				self:_hideCell(cell)
			end
		end
	end

	-- 将可视区内的单元格显示
	for x = sx, ex, 1 do
		local col = _cells[x]
		if not col then
			col = { }
			_cells[x] = col
		end

		for y = sy, ey, 1 do
			local cell = col[y]
			if not cell then
				cell = self:_createCell(x,y)
				col[y] = cell
			end

			if not cell.isShow then
				--				print("显示:",x,y)
				self:_showCell(cell)
			end
		end
	end
end

--坐标转化与判断>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--世界坐标原点为左上角,程序坐标原点为左下角
--世界坐标原点为(1,1),程序坐标原点为(0,0)

--- 网格坐标转化为地图坐标
-- @param grdX
-- @param grdY
-- @return mapX,mapY
function T:coordinateGrid2Map(grdX,grdY)
	return 0,0
end

--- 地图坐标转化为网格坐标
-- @param mapX
-- @param mapY
-- @return grdX,grdY
function T:coordinateMap2Grid(mapX,mapY)
	return 0,0
end

--- 设备坐标转化为网格坐标
-- @param dvcX
-- @param dvcY
-- @return grdX,grdY
function T:coordinateDevice2Grid(dvcX,dvcY)
	return 0,0
end

--- 网格坐标转化为设备坐标
-- @param grdX
-- @param grdY
-- @return dvcX,dvcY
function T:coordinateGrid2Device(grdX,grdY)
	return 0,0
end

--- 设备坐标转化为地图坐标
-- @param dvcX
-- @param dvcY
-- @return mapX,mapY
function T:coordinateDevice2Map(dvcX,dvcY)
	return self:coordinateGrid2Map(self:coordinateDevice2Grid(dvcX,dvcY))
end

--- 地图坐标转化为设备坐标
-- @param mapX
-- @param mapY
-- @return dvcX,dvcY
function T:coordinateMap2Device(mapX,mapY)
	return self:coordinateGrid2Device(self:coordinateMap2Grid(mapX,mapY))
end

--- 检查指定的网格坐标是否在地图区域内
-- @param grdX
-- @param grdY
function T:isInMapArea(grdX,grdY)
	return true
end
--坐标转化与判断<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

return T
