NET_CODE_SOCKET_CLOSE = 5

--- 通讯数据转换器
local _adapter=nil

--- 防误操作遮罩
local _uiMask=nil

--- 防误操作遮罩的层数
local _maskCount = 0

--- 防误操作显示计时器ID
local _maskTimeoutId = nil

--- 防误操作在此时间后变得可见
local _maskTimeoutSec = 0.5

--- 通讯超时时长
local TIMEOUT_SECOND=15

--- 通讯超时计时器列表
local TIMEOUT_TIMERS={}

--- 添加防误操作遮罩
local function addMask()
	if _uiMask then
		if _maskCount == 0 then
			if not _uiMask:getParent() then
				cc.Director:getInstance():getRunningScene():addChild(_uiMask)
			end

			if _maskTimeoutSec > 0 then
				_uiMask:setOpacity(0)

				if _maskTimeoutId then
					Cocos2dxUtils:clearScheduler(_maskTimeoutId)
				end
				_maskTimeoutId = Cocos2dxUtils:timeOut(
					function()
						_uiMask:setOpacity(255)

						_maskTimeoutId = nil
					end ,
					_maskTimeoutSec
				)
			else
				_uiMask:setOpacity(255)
			end
		end
		_maskCount = _maskCount + 1
	end
end

--- 移除防误操作遮罩
-- @param isForce 是否强制移除
local function rmMask(isForce)
	if _uiMask then
		_maskCount = isForce and 0 or (_maskCount - 1)

		if _maskTimeoutId then
			Cocos2dxUtils:clearScheduler(_maskTimeoutId)
			_maskTimeoutId = nil
		end

		if _maskCount == 0 then
			_uiMask:removeFromParent()
		end
	end
end

--- [静态]网络连接被断开
local function netEventHandler(type, code)
	if code == NET_CODE_SOCKET_CLOSE then
		_adapter=nil

		for _,v in pairs(TIMEOUT_TIMERS) do
			Cocos2dxUtils:clearScheduler(v)

			print("移除通讯超时")
		end
		TIMEOUT_TIMERS={}

		print("强制移除通讯时的防误操作遮罩了!")
		rmMask(true)
	end
end

--- 通讯命令的抽象类
local T = Class.AbstractC2SCommand()

function T:def()
--- 通道(频道,端口)
-- channel;

--- 当前请求的流水号
-- __id

--- 完成后的回调
-- _completeCallback;

--- 是否后台方法
-- self.isBackground = false
end

--- [静态]初始化数据转发器
function T:setStaticInitAdapter(adapter)
	-- 转换器已被释放掉,无需移除侦听器
	--    if _adapter then
	--        _adapter:luaRemoveEventListener("net_socket_event", netEventHandler)
	--    end

	_adapter = adapter

	if _adapter then
		_adapter:luaAddEventListener("net_socket_event", netEventHandler)
	end
end

--- [静态]防误操作遮罩
function T:setStaticUiMask(uiMask)
	if uiMask ~= _uiMask then
		if _uiMask then
			_uiMask:release()
		end
		_uiMask=uiMask
		_uiMask:retain()
	end
end

--- 构造函数
-- @param channel 通道
-- @param isBackground 是否为后台请求(即隐式请求,不显示防误操作遮罩)
function T:ctor(channel, isBackground)
	self.channel = channel
	self.isBackground = isBackground
end

--- 封装数据
function T:encode()

end

--- 解析数据
function T:decode(data)

end

--- 解析数据出错
function T:decodeErrHandler()
	local message="<解析数据器返回的数据出错>:channel=" .. self.channel

	__print__("NetErr",message)
	LogRecorder:push(message,LOG_LEVEL_ERROR,"NetErr")

	CommonBroadcast:luaDispatchEvent("net_decode_err",self)
end

--- 通讯完成,执行回调
-- @param ... 回调函数的参数
function T:complete(...)
	if self._completeCallback then
		self._completeCallback(...)

		self._completeCallback = nil
	end
end

--- 执行
-- @param 回调参数
function T:exec(callback)
	if not _adapter then
		return
	end

	self._completeCallback = callback

	-- 非后台请求,需要加防误操作遮罩
	if not self.isBackground then
		addMask()

		-- 通讯超时检测
		TIMEOUT_TIMERS[self]=Cocos2dxUtils:timeOut(
			function()
				TIMEOUT_TIMERS[self]=nil

				-- 移除防误操作遮罩
				rmMask()

				CommonBroadcast:luaDispatchEvent(EVT_TYPE_NET_TIME_OUT)
			end,
			TIMEOUT_SECOND
		)
	end

	-- 解析数据错误
	local function decodeErrHandler(type, id)
	print("解析数据错误:",id,self.__id)
		if id == self.__id then
			self:decodeErrHandler()
		end
	end
	-- 添加解析数据出错的侦听
	_adapter:luaAddEventListener("id_call_decode_err", decodeErrHandler)

	_adapter:send(
		self:encode(),
		self.channel,
		function(data, id)
			self.__id = id

			if TIMEOUT_TIMERS[self] then
				Cocos2dxUtils:clearScheduler(TIMEOUT_TIMERS[self])
				TIMEOUT_TIMERS[self]=nil
			end

			-- 移除防误操作遮罩
			if not self.isBackground then
				rmMask()
			end

			self:complete(self:decode(data))

			-- 移除解析数据出错的侦听
			_adapter:luaRemoveEventListener("id_call_decode_err", decodeErrHandler)
		end
	)
end

return T
