local luaj = LuaJavaBridge

--- 与lua/java交互的接口
local LuaInterface="org.cocos2dx.lua.huawei.LuaInterface_huaWei"

local AbstractSDKDelegate=require("src.frameworks.platformSDKs.AbstractSDKDelegate")

local T=Class.SDK_Android_huaWei_Delegate(AbstractSDKDelegate)

local _print=print
local function print(...)
	_print(string.format("\t<%s>:%s",T.__name,table.concat({...},"\t")))
end

--- SDK事件类型-快速登陆成功
T.SDK_EVT_TYPE_QUICKLY_LOGIN_SUCCESS=0

--- SDK事件类型-登陆成功
T.SDK_EVT_TYPE_LOGIN_SUCCESS=1

--- SDK事件类型-注册成功
T.SDK_EVT_TYPE_REGISTER_SUCCESS=2

--- SDK事件类型-登陆失败
T.SDK_EVT_TYPE_LOGIN_FAIL=3

--- SDK事件类型-登出
T.SDK_EVT_TYPE_LOGINOUT=4

--- SDK事件类型-支付成功
T.SDK_EVT_TYPE_PAY_SUCCESS=5

--- SDK事件类型-支付失败
T.SDK_EVT_TYPE_PAY_FAILURE=6

--- SDK事件类型-google支付成功
T.SDK_EVT_TYPE_GG_PAY_SUCCESS=7

--- SDK事件类型-分享成功
T.SDK_EVT_TYPE_SHARE_SUCCESS=8

--- SDK事件类型-分享失败
T.SDK_EVT_TYPE_SHARE_FAILURE=9

--- SDK事件类型-初始化完成
T.SDK_EVT_TYPE_INIT_COMPLETE=10

--事件类型>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--- 事件-初始化完成
T.EVT_TYPE_INIT_COMPLETE="evt_type_init_complete"
--事件类型<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

function T:def()
	AbstractSDKDelegate.def(self)

	--- 用户ID
	-- _uid

	--- 用户名
	-- _uName

	--- 登陆token
	-- _token

	--- 是否游客
	-- _isTourist
end

function T:ctor()
	AbstractSDKDelegate.ctor(self)

	self:init()
end

function T:init()
	print("初始化")

	local _execResultHandler=function(jsonstr)
		Cocos2dxUtils:timeOut(function()
			self:execResultHandler(jsonstr)
		end,0)
	end

	--设置回调
	luaj.callStaticMethod(
		LuaInterface,
		"set2luaCallback",
		{_execResultHandler},
		"(I)V"
	)

	CommonBroadcast:luaAddEventListener(EVT_TYPE_KEY_PRESSED,self,self.onKeyReseased)
	CommonBroadcast:luaAddEventListener(EVT_TYPE_KEY_RELEASED,self,self.onKeyReseased)
end

function T:execResultHandler(jsonstr)
	if not jsonstr then
		print("从java返回的事件数据为 nil")
		
		return
	end

	print(jsonstr)
	local result=json:decode(jsonstr)
	print(obj2str(result))

	local sdkEvtType=result.evtType
	print("sdkEvtType:",sdkEvtType)
	local data,other=result.data,result.other
	if sdkEvtType == self.SDK_EVT_TYPE_INIT_COMPLETE then

		print(string.format("初始化完成=>"))
		self:luaDispatchEvent(self.EVT_TYPE_INIT_COMPLETE)
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGIN_SUCCESS then
		--登陆成功
		self._isTourist=false
		self._uName=data.userName

		self._uid=data.userID
		self._token=data.accessToken

		print(string.format("登陆=>channelId:%s,uid:%s,uName:%s,token:%s",self._channelId,self._uid,self._uName,self._token))
		self:luaDispatchEvent(self.EVT_TYPE_LOGIN_SUCCESS)
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGIN_FAIL then
		print("登陆失败")
		self:luaDispatchEvent(self.EVT_TYPE_LOGIN_FAILURE)
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGINOUT then
		print("登出")
		self:luaDispatchEvent(self.EVT_TYPE_LOGINOUT)
	end
end

function T:onKeyPressed(keyCode,evt)

end

local uiCloseTip=nil

function T:onKeyReseased(keyCode,evt)
	if C_PFM == cc.PLATFORM_OS_ANDROID then
		if keyCode ~= cc.KeyCodeKey.KEY_BACK then
			return
		end

		if uiCloseTip then
			return
		end

		uiCloseTip=MUIAlter.show({
			info=Language:get("ug_quitTip"),
			flag=3,
			confirTtype=14,
			cancelType=11,
			callback=function(flag)
				uiCloseTip=nil

				if flag == FLAG_OK then
					luaj.callStaticMethod(LuaInterface,"doQuit",{},"()V")
				end
			end
		})
	end
end

function T:doLogin()
	luaj.callStaticMethod(LuaInterface,"login",{},"()V")
end

function T:doSwitchAccount()
	luaj.callStaticMethod(LuaInterface,"switchAccount",{},"()V")
end

--- 支付
-- @param itemName 商品名
-- @param unitPrice 商品单价
-- @param count 数量
-- @param gems 获取的钻石数
-- @param callBackInfo 支付完成的回调信息
-- @param callBackUrl 支付完成的回调url
function T:pay(itemName,unitPrice,count,gems,callBackInfo,callBackUrl,itemCode)

	local jsonObj={
		productName=itemName,
		price=unitPrice * count,
		productDesc=itemName,
		callbackInfo=callBackInfo,
		callbackUrl=callBackUrl,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"pay",{jsonstr},"(Ljava/lang/String;)V")
end

function T:traking_roleData(roleId,roleName,roleLv,zoneId,zoneName)
	local jsonObj={
		roleId=roleId,
		roleName=roleName,
		roleLv=roleLv,
		zoneId=zoneId,
		zoneName=zoneName,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"setRoleData",{jsonstr},"(Ljava/lang/String;)V")
end
T.trakingEnterGame=T.traking_roleData

function T:trakingUpdateUserInfo(roleId,roleName,roleLv,zoneId,zoneName)

end

--- 获取用户ID
function T:getUid()
	return self._uid
end

--- 获取用户名
function T:getUserName()
	return self._uName
end

--- 获取登陆token
function T:getAccessToken()
	return self._token
end

--- 获取是否是游客
function T:getIsTourist()
	return self._isTourist
end

--- 获取SDK代码
function T:getPlatformCode()
	return self._channelCode
end

return T()
