local luaj = LuaJavaBridge

--- 易接平台对照表
local SDK1PlatformContrast=require("src.frameworks.platformSDKs.sdk_1.SDK1PlatformContrast")
--- 与lua/java交互的接口
local LuaInterface="org.cocos2dx.lua.LuaInterface"

local AbstractSDKDelegate=require("src.frameworks.platformSDKs.AbstractSDKDelegate")

local T=Class.SDK_Android_1_Delegate(AbstractSDKDelegate)

local _print=print
local function print(...)
	_print(string.format("\t<%s>:%s",T.__name,table.concat({...},"\t")))
end

--- SDK事件类型-快速登陆成功
T.SDK_EVT_TYPE_QUICKLY_LOGIN_SUCCESS=0

--- SDK事件类型-登陆成功
T.SDK_EVT_TYPE_LOGIN_SUCCESS=1

--- SDK事件类型-注册成功
T.SDK_EVT_TYPE_REGISTER_SUCCESS=2

--- SDK事件类型-登陆失败
T.SDK_EVT_TYPE_LOGIN_FAIL=3

--- SDK事件类型-登出
T.SDK_EVT_TYPE_LOGINOUT=4

--- SDK事件类型-支付成功
T.SDK_EVT_TYPE_PAY_SUCCESS=5

--- SDK事件类型-支付失败
T.SDK_EVT_TYPE_PAY_FAILURE=6

--- SDK事件类型-google支付成功
T.SDK_EVT_TYPE_GG_PAY_SUCCESS=7

--- SDK事件类型-分享成功
T.SDK_EVT_TYPE_SHARE_SUCCESS=8

--- SDK事件类型-分享失败
T.SDK_EVT_TYPE_SHARE_FAILURE=9

--- SDK事件类型-初始化完成
T.SDK_EVT_TYPE_INIT_COMPLETE=10

--事件类型>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--- 事件-初始化完成
T.EVT_TYPE_INIT_COMPLETE="evt_type_init_complete"
--事件类型<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

function T:def()
	AbstractSDKDelegate.def(self)

	--- 是否初始化完成
	self._isInited=false

	--- APP id
	-- _appId

	--- 渠道ID
	-- _channelId

	--- 渠道代码(游戏服务器定义的)
	-- _channelCode

	--- 用户ID
	-- _uid

	--- 用户名
	-- _uName

	--- 登陆token
	-- _token

	--- 是否游客
	-- _isTourist
end

function T:ctor()
	AbstractSDKDelegate.ctor(self)

	self:init()
end

function T:init()
	print("初始化")

	local _execResultHandler=function(jsonstr)
		Cocos2dxUtils:timeOut(function()
			self:execResultHandler(jsonstr)
		end,0)
	end

	--设置回调
	luaj.callStaticMethod(
		LuaInterface,
		"set2luaCallback",
		{_execResultHandler},
		"(I)V"
	)

	CommonBroadcast:luaAddEventListener(EVT_TYPE_KEY_PRESSED,self,self.onKeyReseased)
	CommonBroadcast:luaAddEventListener(EVT_TYPE_KEY_RELEASED,self,self.onKeyReseased)
end

function T:execResultHandler(jsonstr)
	if not jsonstr then
		print("从java返回的事件数据为 nil")

		return
	end

	print(jsonstr)
	local result=json:decode(jsonstr)
	print(obj2str(result))

	local sdkEvtType=result.evtType
	print("sdkEvtType:",sdkEvtType)
	local data,other=result.data,result.other
	if sdkEvtType == self.SDK_EVT_TYPE_INIT_COMPLETE then
		self._isInited=true

		self._channelId=data.channelId
		self._channelCode=SDK1PlatformContrast.sCodeList[self._channelId]

		print(string.format("初始化完成=>channelId:%s",self._channelId))
		self:luaDispatchEvent(self.EVT_TYPE_INIT_COMPLETE)

		-- 百度需要调些接口调出公告页面
		if SDK_Delegate:getChannelId() == SDK1PlatformContrast.PLT_BAI_DU then
			local jsonObj={}

			luaj.callStaticMethod(LuaInterface,"setData",{"showInfo",json:encode(jsonObj)},"(Ljava/lang/String;Ljava/lang/String;)V")
		end
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGIN_SUCCESS then
		self._appId=data.appId

		--登陆成功
		self._isTourist=false

		local isBackLoadingScene=false

		--这是切换账号
		if (self._uid and data.userId ~= self._uid) or (self._uName and data.userName ~= self._uName) then
			local curScene=Director:getRunningScene()
			if curScene and curScene.__name ~= "LoadingScene" then
				isBackLoadingScene=true
			end
		end
		self._uName=data.userName

		self._uid=data.userId
		self._token=data.token

		print(string.format("登陆=>channelId:%s,uid:%s,uName:%s,token:%s",self._channelId,self._uid,self._uName,self._token))
		self:luaDispatchEvent(self.EVT_TYPE_LOGIN_SUCCESS)

		if isBackLoadingScene then
			require("src.app.controler.Back2LoginScenceCommand")()
		end
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGIN_FAIL then
		print("登陆失败")
		self:luaDispatchEvent(self.EVT_TYPE_LOGIN_FAILURE)
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGINOUT then
		print("登出")
		self:luaDispatchEvent(self.EVT_TYPE_LOGINOUT)
	end
end

function T:onKeyPressed(keyCode,evt)

end

local uiCloseTip=nil

function T:onKeyReseased(keyCode,evt)
	if C_PFM == cc.PLATFORM_OS_ANDROID then
		if keyCode ~= cc.KeyCodeKey.KEY_BACK then
			return
		end

		local channelId=self:getChannelId()
		if
			channelId == SDK1PlatformContrast.PLT_LE_WAN
			or
			channelId == SDK1PlatformContrast.PLT_360
			or
			channelId == SDK1PlatformContrast.PLT_AN_ZHI
			or
			channelId == SDK1PlatformContrast.PLT_DOWN_JOY
			or
			channelId == SDK1PlatformContrast.PLT_SOU_GOU
			or
			channelId == SDK1PlatformContrast.PLT_LENOVO
			or
			channelId == SDK1PlatformContrast.PLT_XIAO_MI
		then
			luaj.callStaticMethod(LuaInterface,"doQuit",{},"()V")

			return
		end

		if uiCloseTip then
			return
		end

		uiCloseTip=MUIAlter.show({
			info=Language:get("ug_quitTip"),
			flag=3,
			confirTtype=14,
			cancelType=11,
			callback=function(flag)
				uiCloseTip=nil

				if flag == FLAG_OK then
					luaj.callStaticMethod(LuaInterface,"doQuit",{},"()V")
				end
			end
		})
	end
end

function T:doLogin()
	luaj.callStaticMethod(LuaInterface,"login",{},"()V")
end

function T:doSwitchAccount()
	luaj.callStaticMethod(LuaInterface,"switchAccount",{},"()V")
end

--- 支付
-- @param itemName 商品名
-- @param unitPrice 商品单价
-- @param count 数量
-- @param gems 获取的钻石数
-- @param callBackInfo 支付完成的回调信息
-- @param callBackUrl 支付完成的回调url
function T:pay(itemName,unitPrice,count,gems,callBackInfo,callBackUrl,itemCode)
	local _unitPrice=unitPrice * 100

	local channelId=self:getChannelId()
	if
		channelId == SDK1PlatformContrast.PLT_SOU_GOU
	then
		local jsonObj={
			itemCode=itemCode,				--传入搜狗后台获取的计费点编码waresid
			unitPrice=_unitPrice,			--单位价格(人民币分)
			unitName=Language:get("uc29"),	--虚拟货币名称
			remain=gems/(unitPrice*count),	--自定义参数（人民币与游戏币的兑换比例）
			count=count,					--数量
			callBackInfo=callBackInfo,		--回调到服务器的信息
			callBackUrl=callBackUrl,		--回调服务器的地址
		}

		local jsonstr=json:encode(jsonObj)
		luaj.callStaticMethod(LuaInterface,"payExtend",{jsonstr},"(Ljava/lang/String;)V")

		return
	end

	local jsonObj={
		itemName=itemName,
		unitPrice=_unitPrice,
		count=count,
		callBackInfo=callBackInfo,
		callBackUrl=callBackUrl,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"pay",{jsonstr},"(Ljava/lang/String;)V")
end

function T:charge(itemName,unitPrice,count,callBackInfo,callBackUrl)
	local _unitPrice=unitPrice * 100

	local jsonObj={
		itemName=itemName,
		unitPrice=_unitPrice,
		count=count,
		callBackInfo=callBackInfo,
		callBackUrl=callBackUrl,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"charge",{jsonstr},"(Ljava/lang/String;)V")
end

function T:traking_roleData(roleId,roleName,roleLv,zoneId,zoneName)
	local jsonObj={
		roleId=roleId,
		roleName=roleName,
		roleLv=roleLv,
		zoneId=zoneId,
		zoneName=zoneName,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"setRoleData",{jsonstr},"(Ljava/lang/String;)V")
end

function T:traking_data(propertyName,dataTable)
	luaj.callStaticMethod(LuaInterface,"setData",{propertyName,json:encode(dataTable)},"(Ljava/lang/String;Ljava/lang/String;)V")
end

function T:trakingEnterGame(roleId,roleName,roleLv,zoneId,zoneName)
	if SDK_Delegate:getChannelId() == SDK1PlatformContrast.PLT_LE_WAN then
		local jsonObj={
			roleId=roleId,
			roleName=roleName,
			roleLevel=roleLv,
			zoneId=zoneId,
			zoneName=zoneName
		}

		luaj.callStaticMethod(LuaInterface,"setData",{"enterServer",json:encode(jsonObj)},"(Ljava/lang/String;Ljava/lang/String;)V")
	else
		local jsonObj={
			roleId=roleId,
			roleName=roleName,
			roleLv=roleLv,
			zoneId=zoneId,
			zoneName=zoneName,
		}

		luaj.callStaticMethod(LuaInterface,"setRoleData",{json:encode(jsonObj)},"(Ljava/lang/String;)V")
	end
end

function T:trakingUpdateUserInfo(roleId,roleName,roleLv,zoneId,zoneName)
	if SDK_Delegate:getChannelId() == SDK1PlatformContrast.PLT_LE_WAN then
		local jsonObj={
			roleId=roleId,
			roleName=roleName,
			roleLevel=roleLv,
			zoneId=zoneId,
			zoneName=zoneName
		}

		luaj.callStaticMethod(LuaInterface,"setData",{"gameinfo",json:encode(jsonObj)},"(Ljava/lang/String;Ljava/lang/String;)V")
	else
		local jsonObj={
			roleId=roleId,
			roleName=roleName,
			roleLv=roleLv,
			zoneId=zoneId,
			zoneName=zoneName,
		}

		luaj.callStaticMethod(LuaInterface,"setRoleData",{json:encode(jsonObj)},"(Ljava/lang/String;)V")
	end
end

--- 是否初始化完成
function T:getIsInited()
	return self._isInited
end

--- 获取appid
function T:getAppId()
	return self._appId
end

--- 获取渠道ID
function T:getChannelId()
	return self._channelId
end

--- 获取用户ID
function T:getUid()
	return self._uid
end

--- 获取用户名
function T:getUserName()
	return self._uName
end

--- 获取登陆token
function T:getAccessToken()
	return self._token
end

--- 获取是否是游客
function T:getIsTourist()
	return self._isTourist
end

--- 获取SDK代码
function T:getPlatformCode()
	return self._channelCode
end

return T()
