local luaj = LuaJavaBridge

--- 与lua/java交互的接口
local LuaInterface="org.cocos2dx.lua.vivo.LuaInterface_vivo"

local AbstractSDKDelegate=require("src.frameworks.platformSDKs.AbstractSDKDelegate")

local T=Class.SDK_Android_vivo_Delegate(AbstractSDKDelegate)

local _print=print
local function print(...)
	_print(string.format("\t<%s>:%s",T.__name,table.concat({...},"\t")))
end

--- SDK事件类型-快速登陆成功
T.SDK_EVT_TYPE_QUICKLY_LOGIN_SUCCESS=0

--- SDK事件类型-登陆成功
T.SDK_EVT_TYPE_LOGIN_SUCCESS=1

--- SDK事件类型-注册成功
T.SDK_EVT_TYPE_REGISTER_SUCCESS=2

--- SDK事件类型-登陆失败
T.SDK_EVT_TYPE_LOGIN_FAIL=3

--- SDK事件类型-登出
T.SDK_EVT_TYPE_LOGINOUT=4

--- SDK事件类型-支付成功
T.SDK_EVT_TYPE_PAY_SUCCESS=5

--- SDK事件类型-支付失败
T.SDK_EVT_TYPE_PAY_FAILURE=6

--- SDK事件类型-google支付成功
T.SDK_EVT_TYPE_GG_PAY_SUCCESS=7

--- SDK事件类型-分享成功
T.SDK_EVT_TYPE_SHARE_SUCCESS=8

--- SDK事件类型-分享失败
T.SDK_EVT_TYPE_SHARE_FAILURE=9

function T:def()
	AbstractSDKDelegate.def(self)

	--- 用户ID
	-- _uid

	--- 用户名
	-- _uName

	--- 登陆token
	-- _token

	--- 是否游客
	-- _isTourist
end

function T:ctor()
	AbstractSDKDelegate.ctor(self)

	self:init()
end

function T:init()
	print("初始化")

	local _execResultHandler=function(jsonstr)
		Cocos2dxUtils:timeOut(function()
			self:execResultHandler(jsonstr)
		end,0)
	end

	--设置回调
	luaj.callStaticMethod(
		LuaInterface,
		"set2luaCallback",
		{_execResultHandler},
		"(I)V"
	)

	CommonBroadcast:luaAddEventListener(EVT_TYPE_KEY_PRESSED,self,self.onKeyReseased)
	CommonBroadcast:luaAddEventListener(EVT_TYPE_KEY_RELEASED,self,self.onKeyReseased)
end

function T:execResultHandler(jsonstr)
	if not jsonstr then
		print("从java返回的事件数据为 nil")

		return
	end

	print(jsonstr)
	local result=json:decode(jsonstr)
	print(obj2str(result))

	local sdkEvtType=result.evtType
	print("sdkEvtType:",sdkEvtType)
	local data,other=result.data,result.other
	if sdkEvtType == self.SDK_EVT_TYPE_LOGIN_SUCCESS then
		--登陆成功
		self._isTourist=false
		self._uName=data.userName

		self._uid=data.openId
		self._token=data.authToken

		print(string.format("登陆=>uid:%s,uName:%s,token:%s",self._uid,self._uName,self._token))
		self:luaDispatchEvent(self.EVT_TYPE_LOGIN_SUCCESS)
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGIN_FAIL then
		print("登陆失败")
		self:luaDispatchEvent(self.EVT_TYPE_LOGIN_FAILURE)
	elseif sdkEvtType == self.SDK_EVT_TYPE_LOGINOUT then
		print("登出")
		self:luaDispatchEvent(self.EVT_TYPE_LOGINOUT)
	end
end

function T:onKeyPressed(keyCode,evt)

end

function T:onKeyReseased(keyCode,evt)
	if C_PFM == cc.PLATFORM_OS_ANDROID then
		if keyCode ~= cc.KeyCodeKey.KEY_BACK then
			return
		end

		uiCloseTip=MUIAlter.show({
			info=Language:get("ug_quitTip"),
			flag=3,
			confirTtype=14,
			cancelType=11,
			callback=function(flag)
				uiCloseTip=nil

				if flag == FLAG_OK then
					luaj.callStaticMethod(LuaInterface,"doQuit",{},"()V")
				end
			end
		})
	end
end

function T:doLogin()
	luaj.callStaticMethod(LuaInterface,"login",{},"()V")
end

function T:doSwitchAccount()
	luaj.callStaticMethod(LuaInterface,"switchAccount",{},"()V")
end

--- 支付
function T:pay(orderId,accessKey,itemName,unitPrice,count,level,callBackInfo,callBackUrl)
	local jsonObj={
		orderId=orderId,
		accessKey=accessKey,
		productName=itemName,
		productDes=itemName,
		price=unitPrice * count,
		roleId=UserInfo.id,
		roleName=UserInfo.name,
		level=UserInfo.level,
		callBackInfo=callBackInfo,
		callBackUrl=callBackUrl,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"pay",{jsonstr},"(Ljava/lang/String;)V")
end

function T:traking_roleData(roleId,roleName,roleLv,zoneId,zoneName)
	local jsonObj={
		roleId=roleId,
		roleName=roleName,
		roleLv=roleLv,
		zoneId=zoneId,
		zoneName=zoneName,
	}

	local jsonstr=json:encode(jsonObj)
	luaj.callStaticMethod(LuaInterface,"setRoleData",{jsonstr},"(Ljava/lang/String;)V")
end
T.trakingEnterGame=T.traking_roleData
T.trakingUpdateUserInfo=T.traking_roleData

--- 获取用户ID
function T:getUid()
	return self._uid
end

--- 获取用户名
function T:getUserName()
	return self._uName
end

--- 获取登陆token
function T:getAccessToken()
	return self._token
end

--- 获取是否是游客
function T:getIsTourist()
	return self._isTourist
end

--- 获取SDK代码
function T:getPlatformCode()
	return require("src.frameworks.platformSDKs.PlatformCodes").PLT_VIVO
end


return T()
