local Word = require("src/frameworks/wordFilter/Word.lua")

local strByte = string.byte
local strSub = string.sub
local strRep = string.rep

local function _filter(text, s, e, nextWords)
	local ch, w
	local i
	for i = s, e, 1 do
		ch = strByte(text, i)
		w = nextWords[ch]
		if w then
			if w.nextWords then
				nextWords = w.nextWords
			elseif w.replaceChs then
				return i + 1, w.replaceChs
			end
		elseif w and w.replaceChs then
			return i, w.replaceChs
		else
			return -1
		end
	end

	if w.replaceChs then
		return i, w.replaceChs
	end

	return -1
end

local T = { }

function T:filter(text, wordRoot)
	if not wordRoot then
		return text
	end

	local nextWords = wordRoot.nextWords
	local outText = ""

	local len = string.len(text)
	local s, e, r = 1, -1, nil
	for i = 1, len, 1 do
		if nextWords[strByte(text, i)] then
			e, r = _filter(text, i, len, nextWords)
			if e ~= -1 then
				outText = outText .. strSub(text, s, i - 1) .. r
				s, i = e, e
			end
		end
	end

	return outText .. strSub(text, s)
end

function T:contains(text,wordRoot)
	if not wordRoot then
		return false
	end

	local nextWords = wordRoot.nextWords

	local len = string.len(text)
	for i = 1, len, 1 do
		if nextWords[strByte(text, i)] then
			if _filter(text, i, len, nextWords) ~= -1 then
				return true
			end
		end
	end

	return false
end

--function T:test()
--	local filters = {
--		"bitch",
--		"minghui",
--		"minghuinews",
--		"naive",
--		"择民",
--		"在十月",
--		"张林",
--		"张万年"
--	}
--
--	local wordRoot = Word:resloves(filters)
--
--	local lt = TimeUtils:getSyncMillis()
--
--	local s
--	for i = 1, 100000, 1 do
--		s = T:filter("地张万年不bitch在十月为在张林择民naive", wordRoot)
--	end
--	print("时间:", TimeUtils:getSyncMillis() - lt)
--	print("筛选后", s)
--end

return T
